using System;
using System.Collections.Generic;
using System.Linq;
using System.Xml;
using System.Text.RegularExpressions;
using Au.Gov.Sbr.Xbrl.Document;
using DataContracts;
using Au.Gov.Sbr.Xbrl.Metadata;
using System.Runtime.CompilerServices;
using System.IO;
using System.Text;
using System.Xml.Schema;
using System.Reflection;
using System.Runtime.Serialization;


namespace Ato.EN.IntegrationServices.CodeGenerationRDTIS
{

    public partial class RDTIS2017Validator
    {
        /// <summary>
        /// The max parameter name length is restricted by the SBR1 and SBR2 schemas - the lowest common denominator of SBR 1 is used as the default
        /// </summary>
        private int _maxParameterNameLength;

        /// <summary>
        /// The max parameter value length is restricted by the SBR1 and SBR2 schemas
        /// </summary>
        private int _maxParameterValueLength;

        /// <summary>
        /// The SBR1 and ebms3 schemas do not allow parameter names or values to be the empty string
        /// </summary>
        private string _emptyParameterValue;

        /// <summary>
        /// Initializes a new instance of the <see cref="RDTIS2017Validator" /> class.
        /// </summary>
        /// <param name="maxParameterNameLength">Maximum length of the parameter name.</param>
        /// <param name="maxParameterValueLength">Maximum length of the parameter value.</param>
        /// <param name="emptyParameterValue">This value will be used in place of any parameter values that result in a null or empty value.</param>
        public RDTIS2017Validator(int maxParameterNameLength = 20, int maxParameterValueLength = 4096, string emptyParameterValue = "EMPTY")
        {
            _maxParameterNameLength = maxParameterNameLength;
            _maxParameterValueLength = maxParameterValueLength;
            _emptyParameterValue = emptyParameterValue;
        }

        #region Functions
        private static IEnumerable<string> Union(IEnumerable<string> list1, IEnumerable<string> list2)
        {
            IEnumerable<string> response;

            if (list1 == null && list2 == null)
            {
                response = null;
            }
            else if (list1 == null)
            {
                response = list2.Distinct();
            }
            else if (list2 == null)
            {
                response = list1.Distinct();
            }
            else
            {
                response = list1.Union(list2);
            }

            return response;
        }

        private static bool IsMatch(int? field, string expression, RegexOptions options = RegexOptions.None)
        {
            if (field == null)
                return false;
            else
                return Regex.IsMatch(Convert.ToString(field.Value), expression, options);
        }

        private static bool IsMatch(string field, string expression, RegexOptions options = RegexOptions.None)
        {
            if (field == null)
                return false;
            else
                return Regex.IsMatch(field, expression, options);
        }

        // This is just context and tuple counts where they are integer values - easier that changing the parsing logic to just return the value
        private static int Count(int count)
        {
            return count;
        }


        private static int Count<T>(IEnumerable<T> values)
        {
            return values == null ? 0 : values.Where(f => f != null).Count();
        }


        private static int Count<T>(ICollection<T> values)
        {
            return values == null ? 0 : values.Where(f => f != null).Count();
        }


        private static bool exists(bool value)
        {
            return value;
        }

        private static string GetValueOrEmpty(bool? val)
        {
            return (val.HasValue) ? val.ToString() : string.Empty;
        }

        private static string GetValueOrEmpty(DateTime? val)
        {
            return (val.HasValue) ? val.ToString() : string.Empty;
        }

         /// <summary>
         /// Get string value between [first] a and [last] b.
         /// </summary>
        public static string Between(string value, string a, string b)
        {
            int posA = value.IndexOf(a);
            int posB = value.LastIndexOf(b);
            if (posA == -1)
            {
                return "";
            }
            if (posB == -1)
            {
                return "";
            }
            int adjustedPosA = posA + a.Length;
            if (adjustedPosA >= posB)
            {
                return "";
            }
            return value.Substring(adjustedPosA, posB - adjustedPosA);
        }

         /// <summary>
         /// Get string value after [first] a.
         /// </summary>
        public static string Before(string value, string a)
        {
            int posA = value.IndexOf(a);
            if (posA == -1)
            {
                return "";
            }
            return value.Substring(0, posA);
        }

         /// <summary>
         /// Get string value after [last] a.
         /// </summary>
        public static string After(string value, string a)
        {
            int posA = value.LastIndexOf(a);
            if (posA == -1)
            {
                return "";
            }
            int adjustedPosA = posA + a.Length;
            if (adjustedPosA >= value.Length)
            {
                return "";
            }
            return value.Substring(adjustedPosA);
        }

        private static int Length(object field)
        {
            if (field == null)
            return 0;
            else
            return field.ToString().Trim().Length;
        }

        private static bool NotSameValues(IEnumerable<object> nodes)
        {
            if (nodes == null)
                return false;

            object[] nodesArray = nodes.Cast<object>().ToArray();
            return NotSameValues(nodesArray);
        }

        private static bool NotSameValues(params object[] nodes)
        {
            if (nodes == null)
                return false;

            return ((from x in nodes select x).Distinct().Count() == nodes.Count());
        }

        private static bool HasDuplicateValues(IEnumerable<object> nodes)
        {
            if (nodes == null)
                return false;

            object[] nodesArray = nodes.Cast<object>().ToArray();
            return HasDuplicateValues(nodesArray);
        }

        private static bool HasDuplicateValues(params object[] nodes)
        {
            if (nodes == null)
                return false;

            return !((from x in nodes select x).Distinct().Count() == nodes.Count());
        }

        private int DuplicateValueIndex(IEnumerable<object> values)
        {
            int response = 0;
            var hashset = new HashSet<object>();
            foreach (var value in values)
            {
                if (!hashset.Add(value))
                {
                    return response;
                }
                response++;
            }
            return response;
        }

        private int DuplicateValueIndex<T>(IEnumerable<T?> values) where T : struct
        {
            int response = 0;
            var hashset = new HashSet<T?>();
            foreach (var value in values)
            {
                if (!hashset.Add(value))
                {
                    return response;
                }
                response++;
            }
            return response;
        }

        private static bool IsDate(object value)
        {
            DateTime dateValue;
            return (value != null && DateTime.TryParse(value.ToString(), out dateValue));
        }

        public DateTime AsDate(string dateAsString)
        {
            DateTime response = DateTime.MinValue;
            DateTime date;

            if (DateTime.TryParse(dateAsString, out date))
            {
                response = date;
            }

            return response;
        }
				//The logic in After function expects "---" for day and "--" for month. 
				//Since hyphen was missing the date always returned null
        public DateTime? ConvertToDate(int day, int month, int year)
        {
					return ConvertToDate(day == 0 ? null : "---" + day.ToString(), month == 0 ? null : "--" + month.ToString(), year == 0 ? null : year.ToString());
        }

        public DateTime? ConvertToDate(string day, string month, string year)
        {
            DateTime? response;
            DateTime result;

            if (year == null || month == null || day == null)
            {
                return null;
            }
            string dateAsString = year + "-" + After(month, "--") + "-" + After(day, "---");
            if (DateTime.TryParse(dateAsString, out result))
            {
                response = result;
            }
            else
            {
                response = null;
            }
            return response;
        }


        public DateTime? ConvertToDate(string day, string month, int year)
        {
            DateTime? response;
            DateTime result;

            if (year == 0 || month == null || day == null)
            {
                return null;
            }
            string dateAsString = year.ToString() + "-" + After(month, "--") + "-" + After(day, "---");
            if (DateTime.TryParse(dateAsString, out result))
            {
                response = result;
            }
            else
            {
                response = null;
            }
            return response;
        }


        private static int Day(string dateAsString)
        {
            int response = 0;
            DateTime date;

            if (DateTime.TryParse(dateAsString, out date))
            {
                response = date.Day;
            }

            return response;
        }


        private static int Day(DateTime? date)
        {
            if (date == null)
                return 0;
            else
                return date.Value.Day;

        }


        private static string Month(string dateAsString)
        {
            string response = null;
            DateTime date;

            if (DateTime.TryParse(dateAsString, out date))
            {
                response = date.ToString("MMMM");
            }
            else
            {
                return "NotAMonth";
            }

            return response;
        }

        private static string Month(DateTime? date)
        {
            if (date == null)
                return "NotAMonth";
            else
                return date.Value.ToString("MMMM");
        }


        private static int MonthAsInt(string dateAsString)
        {
            int response = 0;
            DateTime date;

            if (DateTime.TryParse(dateAsString, out date))
            {
                response = date.Month;
            }

            return response;
        }

        private static int MonthAsInt(DateTime? date)
        {
            if (date == null)
            {
                return 0;
            }
            return date.Value.Month;
        }


        private static int Year(string dateAsString)
        {
            int response = 0;
            DateTime date;

            if (DateTime.TryParse(dateAsString, out date))
            {
                response = date.Year;
            }

            return response;
        }

        private static int Year(DateTime? date)
        {
            if (date == null)
                return 0;
            else
                return date.Value.Year;
        }


        private static int CurrentFinancialYear()
        {
            return DateToFinancialYear(DateTime.Now, 7);
        }

        private static int FinancialYear(string dateAsString)
        {
            return DateToFinancialYear(dateAsString, 7);
        }

        private static int FinancialYear(DateTime? date)
        {
            return DateToFinancialYear(date, 7);
        }

        private static int DateToFinancialYear(string dateAsString, int startingMonth)
        {
            int response = 0;
            DateTime date;
            if (DateTime.TryParse(dateAsString, out date))
            {
                response = DateToFinancialYear(date, startingMonth);
            }

            return response;
        }
        private static int DateToFinancialYear(DateTime? date, int startingMonth)
        {
            int response;
            if (date == null)
            {
                response = 0;
            }
            else
            {
                int year = date.Value.Year;
                int month = date.Value.Month;

                if (startingMonth > month)
                    response = year;
                else
                    response = year + 1;
            }
            return response;
        }


        private static int FBTYear(string dateAsString)
        {
            int response = 0;
            DateTime date;

            if (DateTime.TryParse(dateAsString, out date))
                response = FBTYear(date);
            return response;
        }

        private static int FBTYear(DateTime? date)
        {
            if (date == null)
            {
                return 0;
            }
            else
            {
                if (date.Value.Month > 3)
                    return date.Value.Year + 1;
                else
                    return date.Value.Year;
            }
        }


        private static bool IsNumeric(object value)
        {
            decimal numbervalue;
            return (value != null && decimal.TryParse(value.ToString(), out numbervalue));
        }

        private static bool NotMonetary(decimal? field, string sign, int digits, int decimals)
        {
            if (field == null)
            {
                return false;
            }
            else
            {
                string signExpression;
                string decimalExpression;
                int digitsToUse = digits - decimals;

                if (sign == "U")
                    signExpression = "^";
                else
                    signExpression = "^-?";

                if (decimals > 0)
                    decimalExpression = @"(\.\d{1," + decimals + "})?$";
                else
                    decimalExpression = @"$";

                return !(Regex.IsMatch(field.Value.ToString("0.#########################"), signExpression + @"\d{1," + digitsToUse + "}" + decimalExpression));
            }
        }

        private static bool NotNumeric(decimal? field, string sign, int digits, int decimals)
        {
            if (field == null)
            {
                return false;
            }
            else
            {
                string signExpression;
                string decimalExpression;
                int digitsToUse = digits - decimals;

                if (sign == "U")
                    signExpression = "^";
                else
                    signExpression = "^-?";

                if (decimals > 0)
                    decimalExpression = @"(\.\d{1," + decimals + "})?$";
                else
                    decimalExpression = @"$";

                return !(Regex.IsMatch(field.Value.ToString("0.#########################"), signExpression + @"\d{1," + digitsToUse + "}" + decimalExpression));
            }
        }

        private static bool NotNumeric(int? field, string sign, int digits, int decimals = 0)
        {
            if (field == null)
            {
                return false;
            }
            else
            {
                string signExpression;

                if (sign == "U")
                    signExpression = "^";
                else
                    signExpression = "^-?";

                return !(Regex.IsMatch(field.Value.ToString(), signExpression + @"\d{1," + digits + "}$"));
            }
        }

        private static bool NotNumeric(long? field, string sign, int digits, int decimals = 0)
        {
            if (field == null)
            {
                return false;
            }
            else
            {
                string signExpression;

                if (sign == "U")
                    signExpression = "^";
                else
                    signExpression = "^-?";

                return !(Regex.IsMatch(field.Value.ToString(), signExpression + @"\d{1," + digits + "}$"));
            }
        }


        private static bool OutsideRange(decimal field, decimal expression, int range)
        {
            bool response;

            response = (field < (expression - range)) || (field > (expression + range));

            return response;
        }


        private static bool FailsUSIAlgorithm(string usi, string abn)
        {
            bool response;
            if (usi == null || abn == null)
            {
                response = false;
            }
            else
            {
                usi = usi.Trim();
                abn = abn.Trim();
                if (usi.Length < 13 || abn.Length < 11)
                {
                    response = false;
                }
                else
                {
                    int numeric;
                    if (usi.Substring(0, 11) == abn && int.TryParse(usi.Substring(11, 2), out numeric))
                        response = false;
                    else if (Regex.IsMatch(usi, @"^[a-zA-Z]{3}\d{4}[a-zA-Z]{2}"))
                        response = false;
                    else
                        response = true;
                }
            }
            return response;
        }


        private static bool FailsTANAlgorithm(string tan)
        {
            bool response;
            decimal decimalTan;

            if (tan == null)
                return false;

            tan = tan.Trim();

            if (!decimal.TryParse(tan, out decimalTan))
                return true;

            if (tan.Length != 8)
                return true;

            decimal tanSum =
                7 * int.Parse(tan.Substring(0, 1)) +
                9 * int.Parse(tan.Substring(1, 1)) +
                8 * int.Parse(tan.Substring(2, 1)) +
                4 * int.Parse(tan.Substring(3, 1)) +
                6 * int.Parse(tan.Substring(4, 1)) +
                3 * int.Parse(tan.Substring(5, 1)) +
                5 * int.Parse(tan.Substring(6, 1)) +
                1 * int.Parse(tan.Substring(7, 1));

            if ((tanSum % 11) == 0)
                response = false;
            else
                response = true;

            return response;
        }


        private static bool FailsABNAlgorithm(string abn)
        {
            bool response;
            decimal decimalAbn;

            if (abn == null)
                return false;

            abn = abn.Trim();

            if (!decimal.TryParse(abn, out decimalAbn))
                return true;

            if (abn.Length != 11)
                return true;

            decimal abnSum =
                10 * (int.Parse(abn.Substring(0, 1)) - 1) +
                1 * int.Parse(abn.Substring(1, 1)) +
                3 * int.Parse(abn.Substring(2, 1)) +
                5 * int.Parse(abn.Substring(3, 1)) +
                7 * int.Parse(abn.Substring(4, 1)) +
                9 * int.Parse(abn.Substring(5, 1)) +
                11 * int.Parse(abn.Substring(6, 1)) +
                13 * int.Parse(abn.Substring(7, 1)) +
                15 * int.Parse(abn.Substring(8, 1)) +
                17 * int.Parse(abn.Substring(9, 1)) +
                19 * int.Parse(abn.Substring(10, 1));

            if ((abnSum % 89) == 0)
                response = false;
            else
                response = true;

            return response;
        }
        private static bool FailsACNAlgorithm(string acn)
        {
            bool response;
            decimal decimalAbn;
            if (acn == null)
                return false;

            acn = acn.Trim();

            if (!decimal.TryParse(acn, out decimalAbn))
                return true;

            if (acn.Length != 9)
                return true;

            decimal abnSum =
                8 * int.Parse(acn.Substring(0, 1)) +
                7 * int.Parse(acn.Substring(1, 1)) +
                6 * int.Parse(acn.Substring(2, 1)) +
                5 * int.Parse(acn.Substring(3, 1)) +
                4 * int.Parse(acn.Substring(4, 1)) +
                3 * int.Parse(acn.Substring(5, 1)) +
                2 * int.Parse(acn.Substring(6, 1)) +
                1 * int.Parse(acn.Substring(7, 1));

            decimal checkDigit = int.Parse(acn.Substring(8, 1));
            decimal acnRemainder = abnSum % 10;

            if (((10 - acnRemainder) % 10) == checkDigit)
                response = false;
            else
                response = true;

            return response;
        }


        private static bool FailsTFNAlgorithm(string tfn)
        {
            bool response;
            decimal decimalTfn;

            if (tfn == null)
                return false;

            tfn = tfn.Trim();
            tfn = Regex.Replace(tfn, "^0+", "");

            if (!decimal.TryParse(tfn, out decimalTfn))
                return true;

            if (tfn.Length < 8)
                return true;


            decimal tfn1To7Sum =
                1 * int.Parse(tfn.Substring(0, 1)) +
                4 * int.Parse(tfn.Substring(1, 1)) +
                3 * int.Parse(tfn.Substring(2, 1)) +
                7 * int.Parse(tfn.Substring(3, 1)) +
                5 * int.Parse(tfn.Substring(4, 1)) +
                8 * int.Parse(tfn.Substring(5, 1)) +
                6 * int.Parse(tfn.Substring(6, 1));

            decimal tfn8 = 9 * int.Parse(tfn.Substring(7, 1));

            if (tfn.Length == 8)
            {
                decimal tFNLg8WSum9 = 10 * int.Parse(tfn.Substring(7, 1));
                decimal tFNLg8WSum = tfn1To7Sum + tFNLg8WSum9;

                if ((tFNLg8WSum % 11) == 0)
                    response = false;
                else
                    response = true;
            }
            else if (tfn.Length == 9)
            {
                decimal tfn9 = 10 * int.Parse(tfn.Substring(8, 1));
                decimal tFNLg9WSum = tfn1To7Sum + tfn8 + tfn9;

                if ((tFNLg9WSum % 11) == 0)
                    response = false;
                else
                    response = true;
            }
            else
            {
                response = true;
            }

            return response;
        }


        private static decimal ConditionalValue(bool expression, decimal? trueVal, decimal? falseVal)
        {
             return expression ? trueVal.GetValueOrDefault() : falseVal.GetValueOrDefault();
        }

        private static decimal AsNumeric(string value)
        {
             decimal numberValue;
             decimal.TryParse(value, out numberValue);
             return numberValue;
        }

        private static bool RegexMatch(int? field, string expression, string flags="")
        {
            return IsMatch(field, expression, GetRegexOption(flags));
        }

        private static bool RegexMatch(string field, string expression, string flags="")
        {
            return IsMatch(field, expression, GetRegexOption(flags));
        }

        private static RegexOptions GetRegexOption(string flags)
        {
            RegexOptions options = RegexOptions.None;

            char[] characters = flags.ToCharArray();

            foreach (char character in characters)
            {
                switch (character)
                {
                    case 'i':                        
                        options = options | RegexOptions.IgnoreCase;                       
                        break;
                    case 'm':                        
                        options = options | RegexOptions.Multiline;                       
                        break;
                    case 's':                        
                            options = options | RegexOptions.Singleline;                       
                        break;
                    case 'n':                        
                            options = options | RegexOptions.ExplicitCapture;                       
                        break;
                    case 'x':                       
                            options = options | RegexOptions.IgnorePatternWhitespace;                       
                        break;
                }
            }

            return options;
        }

        /// <summary>
        /// Returns an occurrence index as [occurrenceIndex] of occurrenceIndex > 0, otherwise the empty string
        /// </summary>
        /// <param name="occurrenceIndex">Index of the occurrence.</param>
        /// <returns>Occurrence in XPath [#] format</returns>
        public string OccurrenceIndex(int occurrenceIndex)
        {
            return occurrenceIndex > 0 ? "[" + occurrenceIndex + "]" : "";
        }

        #endregion // Functions

        public RDTIS2013 ConsumedReport { get; private set; }

        private static Dictionary<string, ProcessMessageDocument> _processMessageDocuments = new Dictionary<string,ProcessMessageDocument>();

       // [MethodImpl(MethodImplOptions.AggressiveInlining)]
        public List<ProcessMessageDocument> ValidateReport(RDTIS2013 report)
        {

            List<ProcessMessageDocument> response = new List<ProcessMessageDocument>();
            ProcessMessageDocument processMessage;
            ProcessMessageParameter parameter;
            bool assertion;

            this.ConsumedReport = report;

            // ------------------------------------------------------------------------------
            // Validations are now done in code.
            // This version supports full validation of a report - for those rules that have successfully parsed and could be generated.
            // Generated Validations include:
            //   - Validation logic in C#.
            //   - Production of errors in EBMS, SBR and SWS format.
            // 
            // The generation at this stage does not support the following - and has to be completed manually (later versions of the generation will do this for you)
            //   - Business Rules that did not parse.  The ESR team will help support you where this happens as an effort is being made to rectify these
            //   - You will get TODO tasks for those that the parser could not cope with
            // ------------------------------------------------------------------------------
            // ------------------------------------------------------------------------------

            #region VR.ATO.GEN.000209

            /*  VR.ATO.GEN.000209
            Invalid context. The number of Reporting party contexts must equal 1
    
            Legacy Rule Format:
            IF COUNT(RP) <> 1
            RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            Count(^Context) <> 1
    
            Data Elements:
    
            ^Context = RP
            */
            assertion = (Count(report.RPCount) != 1);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.430296", Severity = ProcessMessageSeverity.Error,
                    Description = @"Invalid context. The number of Reporting party contexts must equal 1",
                    Location = "/xbrli:xbrl/xbrli:context" + OccurrenceIndex(report.RPLastOccurrenceIndex),
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.000209"} },
                };
                response.Add(processMessage);
            }
            #endregion // VR.ATO.GEN.000209
    
            #region VR.ATO.GEN.001021
    
            /*  VR.ATO.GEN.001021
            Entity Identifier Scheme is not set to http://www.ato.gov.au/tfn
    
            Legacy Rule Format:
            IF Identifier Scheme <> “http://www.ato.gov.au/tfn”
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            (^Context <> NULL) AND (^IdentifierScheme <> 'http://www.ato.gov.au/tfn')
    
            Data Elements:
    
            ^IdentifierScheme = RP
    
            ^Context = RP
            */
            assertion = (report.RPExists != false && report.RPIdentifierScheme != @"http://www.ato.gov.au/tfn");
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.001021", Severity = ProcessMessageSeverity.Error,
                    Description = @"Invalid identifier scheme.",
                    LongDescription = @"Invalid identifier scheme.  Indentifier scheme must be set to ""http://www.ato.gov.au/tfn""",
                    Location = "/xbrli:xbrl/xbrli:context" + OccurrenceIndex(report.RPOccurrenceIndex) + "/xbrli:entity/xbrli:identifier/@scheme",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.001021"} },
                };
                response.Add(processMessage);
            }
            #endregion // VR.ATO.GEN.001021
    
            #region VR.ATO.GEN.001021
    
            /*  VR.ATO.GEN.001021
            Entity Identifier Scheme is not set to http://www.ato.gov.au/tfn
    
            Legacy Rule Format:
            IF Identifier Scheme <> “http://www.ato.gov.au/tfn”
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            (^Context <> NULL) AND (^IdentifierScheme <> 'http://www.ato.gov.au/tfn')
    
            Data Elements:
    
            ^IdentifierScheme = RP:RP.CTAUS
    
            ^Context = RP:RP.CTAUS
            */
            assertion = (report.RPCTAUSExists != false && report.RPCTAUSIdentifierScheme != @"http://www.ato.gov.au/tfn");
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.001021", Severity = ProcessMessageSeverity.Error,
                    Description = @"Invalid identifier scheme.",
                    LongDescription = @"Invalid identifier scheme.  Indentifier scheme must be set to ""http://www.ato.gov.au/tfn""",
                    Location = "/xbrli:xbrl/xbrli:context" + OccurrenceIndex(report.RPCTAUSOccurrenceIndex) + "/xbrli:entity/xbrli:identifier/@scheme",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.001021"} },
                };
                response.Add(processMessage);
            }
            #endregion // VR.ATO.GEN.001021
    
            #region VR.ATO.GEN.001021
    
            /*  VR.ATO.GEN.001021
            Entity Identifier Scheme is not set to http://www.ato.gov.au/tfn
    
            Legacy Rule Format:
            IF Identifier Scheme <> “http://www.ato.gov.au/tfn”
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            (^Context <> NULL) AND (^IdentifierScheme <> 'http://www.ato.gov.au/tfn')
    
            Data Elements:
    
            ^IdentifierScheme = RP:RP.CTFOR
    
            ^Context = RP:RP.CTFOR
            */
            assertion = (report.RPCTFORExists != false && report.RPCTFORIdentifierScheme != @"http://www.ato.gov.au/tfn");
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.001021", Severity = ProcessMessageSeverity.Error,
                    Description = @"Invalid identifier scheme.",
                    LongDescription = @"Invalid identifier scheme.  Indentifier scheme must be set to ""http://www.ato.gov.au/tfn""",
                    Location = "/xbrli:xbrl/xbrli:context" + OccurrenceIndex(report.RPCTFOROccurrenceIndex) + "/xbrli:entity/xbrli:identifier/@scheme",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.001021"} },
                };
                response.Add(processMessage);
            }
            #endregion // VR.ATO.GEN.001021
    
            #region VR.ATO.GEN.001021
    
            /*  VR.ATO.GEN.001021
            Entity Identifier Scheme is not set to http://www.ato.gov.au/tfn
    
            Legacy Rule Format:
            IF Identifier Scheme <> “http://www.ato.gov.au/tfn”
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            (^Context <> NULL) AND (^IdentifierScheme <> 'http://www.ato.gov.au/tfn')
    
            Data Elements:
    
            ^IdentifierScheme = RP:RP.YPrevious
    
            ^Context = RP:RP.YPrevious
            */
            assertion = (report.RPYPreviousExists != false && report.RPYPreviousIdentifierScheme != @"http://www.ato.gov.au/tfn");
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.001021", Severity = ProcessMessageSeverity.Error,
                    Description = @"Invalid identifier scheme.",
                    LongDescription = @"Invalid identifier scheme.  Indentifier scheme must be set to ""http://www.ato.gov.au/tfn""",
                    Location = "/xbrli:xbrl/xbrli:context" + OccurrenceIndex(report.RPYPreviousOccurrenceIndex) + "/xbrli:entity/xbrli:identifier/@scheme",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.001021"} },
                };
                response.Add(processMessage);
            }
            #endregion // VR.ATO.GEN.001021
    
            #region VR.ATO.RDTIS.438000
    
            /*  VR.ATO.RDTIS.438000
            The Reporting Party, Concession Type Australian TFN does not match the Reporting Company TFN
    
            Legacy Rule Format:
            IF RP.CTAUS:entity.identifier.TFN <> RP:entity.identifier.TFN
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            ^TFN2 <> ^TFN
    
            Data Elements:
    
            ^TFN2 = RP:RP.CTAUS
    
            ^TFN = RP
            */
            assertion = ((report.RPCTAUSExists == true) && (report.RPCTAUSIdentifierTFN != report.RPIdentifierTFN));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.RDTIS.438000", Severity = ProcessMessageSeverity.Error,
                    Description = @"The supplied Reporting Company, Concession Type Australian TFN must be the same as the reporting Company TFN",
                    Location = "/xbrli:xbrl/xbrli:context" + OccurrenceIndex(report.RPCTAUSOccurrenceIndex) + "/xbrli:entity/xbrli:identifier",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.RDTIS.438000"} },
                };
                response.Add(processMessage);
            }
            #endregion // VR.ATO.RDTIS.438000
    
            #region VR.ATO.RDTIS.438001
    
            /*  VR.ATO.RDTIS.438001
            The Reporting Party, Concession Type Foreign TFN does not match the Reporting Company TFN
    
            Legacy Rule Format:
            IF RP.CTFOR:entity.identifier.TFN <> RP:entity.identifier.TFN
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            ^TFN2 <> ^TFN
    
            Data Elements:
    
            ^TFN2 = RP:RP.CTFOR
    
            ^TFN = RP
            */
            assertion = ((report.RPCTFORExists == true) && (report.RPCTFORIdentifierTFN != report.RPIdentifierTFN));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.RDTIS.438001", Severity = ProcessMessageSeverity.Error,
                    Description = @"The supplied Reporting Company, Concession Type Foreign TFN must be the same as the reporting Company TFN",
                    Location = "/xbrli:xbrl/xbrli:context" + OccurrenceIndex(report.RPCTFOROccurrenceIndex) + "/xbrli:entity/xbrli:identifier",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.RDTIS.438001"} },
                };
                response.Add(processMessage);
            }
            #endregion // VR.ATO.RDTIS.438001
    
            #region VR.ATO.RDTIS.438003
    
            /*  VR.ATO.RDTIS.438003
            A Group Member TFN is the same as the reporting Company TFN
    
            Legacy Rule Format:
            IF (entity.identifier.TFN WHERE CONTEXT(GM)) = RP:entity.identifier.TFN
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            AnyOccurrence(^TFN, ^TFN = ^OtherTFN)
    
            Data Elements:
    
            ^OtherTFN = RP
    
            ^TFN = GM
            */
            assertion = (report.GMContextCollection == null ? false : report.GMContextCollection.Any(TFNRepeat => TFNRepeat.IdentifierTFN == report.RPIdentifierTFN));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.RDTIS.438003", Severity = ProcessMessageSeverity.Error,
                    Description = @"The supplied Group Member TFN must not be the same as the reporting Company TFN",
                    Location = "/xbrli:xbrl/xbrli:context" + OccurrenceIndex(report.RPOccurrenceIndex) + "/xbrli:entity/xbrli:identifier",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.RDTIS.438003"} },
                };
                response.Add(processMessage);
            }
            #endregion // VR.ATO.RDTIS.438003
    
            #region VR.ATO.RDTIS.438004
    
            /*  VR.ATO.RDTIS.438004
            Invalid context. The number of Reporting party: Australian contexts exceeds 1
    
            Legacy Rule Format:
            IF COUNT(CONTEXT(RP.CTAUS)) > 1
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            Count(^Context) > 1
    
            Data Elements:
    
            ^Context = RP:RP.CTAUS
            */
            assertion = (Count(report.RPCTAUSCount) > 1);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.RDTIS.438004", Severity = ProcessMessageSeverity.Error,
                    Description = @"The number of Reporting party: Concession Type Australian contexts must not exceed 1",
                    Location = "/xbrli:xbrl/xbrli:context" + OccurrenceIndex(report.RPCTAUSLastOccurrenceIndex),
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.RDTIS.438004"} },
                };
                response.Add(processMessage);
            }
            #endregion // VR.ATO.RDTIS.438004
    
            #region VR.ATO.RDTIS.438005
    
            /*  VR.ATO.RDTIS.438005
            Invalid context. The number of Reporting party: Foreign contexts exceeds 1
    
            Legacy Rule Format:
            IF COUNT(CONTEXT(RP.CTFOR)) > 1
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            Count(^Context) > 1
    
            Data Elements:
    
            ^Context = RP:RP.CTFOR
            */
            assertion = (Count(report.RPCTFORCount) > 1);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.RDTIS.438005", Severity = ProcessMessageSeverity.Error,
                    Description = @"The number of Reporting party: Concession Type Foreign contexts must not exceed 1",
                    Location = "/xbrli:xbrl/xbrli:context" + OccurrenceIndex(report.RPCTFORLastOccurrenceIndex),
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.RDTIS.438005"} },
                };
                response.Add(processMessage);
            }
            #endregion // VR.ATO.RDTIS.438005
    
            #region VR.ATO.RDTIS.438006
    
            /*  VR.ATO.RDTIS.438006
            Invalid context. The number of Group Member contexts exceeds 19
    
            Legacy Rule Format:
            IF COUNT(CONTEXT(GM)) > 19
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            Count(^Context) > 19
    
            Data Elements:
    
            ^Context = GM
            */
            assertion = (Count(report.GMContextCollection) > 19);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.RDTIS.438006", Severity = ProcessMessageSeverity.Error,
                    Description = @"The number of Group Member contexts must not exceed 19",
                    Location = "/xbrli:xbrl/xbrli:context" + OccurrenceIndex(report.GMContextCollection == null ? 0 : (report.GMContextCollection.Count() == 0 ? 0 : report.GMContextCollection.Last().LastOccurrenceIndex)),
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.RDTIS.438006"} },
                };
                response.Add(processMessage);
            }
            #endregion // VR.ATO.RDTIS.438006
    
            #region VR.ATO.RDTIS.438009
    
            /*  VR.ATO.RDTIS.438009
            Mandatory field not supplied
    
            Legacy Rule Format:
            IF [RDTIS1] = NULL
               RETURN VALIDATION MESSAGE
            ENDIF
            
            [RDTIS1] = RDTIS:RP:pyin.02.07:Report.Amendment.Indicator

            Technical Business Rule Format:
            ^RDTIS1 = NULL
    
            Data Elements:
    
            RP:^RDTIS1 = pyin.02.07:Report.Amendment.Indicator
            */
            assertion = (report.RDTIS1 == null);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.001001", Severity = ProcessMessageSeverity.Error,
                    Description = @"Mandatory field not supplied",
                    Location = "/xbrli:xbrl/pyin.02.07:Report.Amendment.Indicator",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.RDTIS.438009"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "RDTIS1", Value = GetValueOrEmpty(report.RDTIS1) });
    
                response.Add(processMessage);
            }
            #endregion // VR.ATO.RDTIS.438009
    
            #region VR.ATO.RDTIS.438010
    
            /*  VR.ATO.RDTIS.438010
            Monetary field, must be unsigned, maximum of 11 chars, no decimals
    
            Legacy Rule Format:
            IF [RDTIS3] <> NULL AND [RDTIS3] <> MONETARY(U,11,0)
               RETURN VALIDATION MESSAGE
            ENDIF
            
            [RDTIS3] = RDTIS:RP:bafpr3.02.08:Expense.ResearchAndDevelopment.ResearchAndDevelopmentAccountingExpenditureAddBack.Amount

            Technical Business Rule Format:
            NotMonetary(^RDTIS3, 'U', 11, 0)
    
            Data Elements:
    
            RP:^RDTIS3 = bafpr3.02.08:Expense.ResearchAndDevelopment.ResearchAndDevelopmentAccountingExpenditureAddBack.Amount
            */
            assertion = NotMonetary(report.RDTIS3, @"U", 11, 0);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.400011", Severity = ProcessMessageSeverity.Error,
                    Description = @"Field incorrect format. Exceeds maximum digits or decimals and/or contains illegal characters",
                    LongDescription = @"Field is in incorrect format for a monetary field. Field either exceeds maximum digits or decimals and/or contains illegal characters and/or for a signed field does not have the sign as the first character",
                    Location = "/xbrli:xbrl/bafpr3.02.08:Expense.ResearchAndDevelopment.ResearchAndDevelopmentAccountingExpenditureAddBack.Amount",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.RDTIS.438010"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "RDTIS3", Value = report.RDTIS3.GetValueOrDefault().ToString() });
    
                response.Add(processMessage);
            }
            #endregion // VR.ATO.RDTIS.438010
    
            #region VR.ATO.RDTIS.438011
    
            /*  VR.ATO.RDTIS.438011
            Monetary field, must be unsigned, maximum of 11 chars, no decimals
    
            Legacy Rule Format:
            IF bafpr3.xx.xx:Expense.ResearchAndDevelopment.ContractedExpenditureResearchServiceProvider.Amount <> NULL AND bafpr3.xx.xx:Expense.ResearchAndDevelopment.ContractedExpenditureResearchServiceProvider.Amount <> MONETARY(U,11,0)
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            NotMonetary(^RDTIS4, 'U', 11, 0)
    
            Data Elements:
    
            RP.CTAUS:^RDTIS4 = bafpr3.02.09:Expense.ResearchAndDevelopment.ContractedExpenditureResearchServiceProvider.Amount
            */
            assertion = NotMonetary(report.RDTIS4, @"U", 11, 0);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.400011", Severity = ProcessMessageSeverity.Error,
                    Description = @"Field incorrect format. Exceeds maximum digits or decimals and/or contains illegal characters",
                    LongDescription = @"Field is in incorrect format for a monetary field. Field either exceeds maximum digits or decimals and/or contains illegal characters and/or for a signed field does not have the sign as the first character",
                    Location = "/xbrli:xbrl/bafpr3.02.09:Expense.ResearchAndDevelopment.ContractedExpenditureResearchServiceProvider.Amount[@contextRef='" + report.RPCTAUSId + "']",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.RDTIS.438011"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "RDTIS4", Value = report.RDTIS4.GetValueOrDefault().ToString() });
    
                response.Add(processMessage);
            }
            #endregion // VR.ATO.RDTIS.438011
    
            #region VR.ATO.RDTIS.438011
    
            /*  VR.ATO.RDTIS.438011
            Monetary field, must be unsigned, maximum of 11 chars, no decimals
    
            Legacy Rule Format:
            IF bafpr3.xx.xx:Expense.ResearchAndDevelopment.ContractedExpenditureResearchServiceProvider.Amount <> NULL AND bafpr3.xx.xx:Expense.ResearchAndDevelopment.ContractedExpenditureResearchServiceProvider.Amount <> MONETARY(U,11,0)
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            NotMonetary(^RDTIS5, 'U', 11, 0)
    
            Data Elements:
    
            RP.CTFOR:^RDTIS5 = bafpr3.02.09:Expense.ResearchAndDevelopment.ContractedExpenditureResearchServiceProvider.Amount
            */
            assertion = NotMonetary(report.RDTIS5, @"U", 11, 0);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.400011", Severity = ProcessMessageSeverity.Error,
                    Description = @"Field incorrect format. Exceeds maximum digits or decimals and/or contains illegal characters",
                    LongDescription = @"Field is in incorrect format for a monetary field. Field either exceeds maximum digits or decimals and/or contains illegal characters and/or for a signed field does not have the sign as the first character",
                    Location = "/xbrli:xbrl/bafpr3.02.09:Expense.ResearchAndDevelopment.ContractedExpenditureResearchServiceProvider.Amount[@contextRef='" + report.RPCTFORId + "']",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.RDTIS.438011"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "RDTIS5", Value = report.RDTIS5.GetValueOrDefault().ToString() });
    
                response.Add(processMessage);
            }
            #endregion // VR.ATO.RDTIS.438011
    
            #region VR.ATO.RDTIS.438012
    
            /*  VR.ATO.RDTIS.438012
            Monetary field, must be unsigned, maximum of 11 chars, no decimals
    
            Legacy Rule Format:
            IF bafpr3.xx.xx:Expense.ResearchAndDevelopment.ContractedExpenditureNonResearchServiceProvider.Amount <> NULL AND bafpr3.xx.xx:Expense.ResearchAndDevelopment.ContractedExpenditureNonResearchServiceProvider.Amount <> MONETARY(U,11,0)
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            NotMonetary(^RDTIS6, 'U', 11, 0)
    
            Data Elements:
    
            RP.CTAUS:^RDTIS6 = bafpr3.02.09:Expense.ResearchAndDevelopment.ContractedExpenditureNonResearchServiceProvider.Amount
            */
            assertion = NotMonetary(report.RDTIS6, @"U", 11, 0);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.400011", Severity = ProcessMessageSeverity.Error,
                    Description = @"Field incorrect format. Exceeds maximum digits or decimals and/or contains illegal characters",
                    LongDescription = @"Field is in incorrect format for a monetary field. Field either exceeds maximum digits or decimals and/or contains illegal characters and/or for a signed field does not have the sign as the first character",
                    Location = "/xbrli:xbrl/bafpr3.02.09:Expense.ResearchAndDevelopment.ContractedExpenditureNonResearchServiceProvider.Amount[@contextRef='" + report.RPCTAUSId + "']",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.RDTIS.438012"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "RDTIS6", Value = report.RDTIS6.GetValueOrDefault().ToString() });
    
                response.Add(processMessage);
            }
            #endregion // VR.ATO.RDTIS.438012
    
            #region VR.ATO.RDTIS.438012
    
            /*  VR.ATO.RDTIS.438012
            Monetary field, must be unsigned, maximum of 11 chars, no decimals
    
            Legacy Rule Format:
            IF bafpr3.xx.xx:Expense.ResearchAndDevelopment.ContractedExpenditureNonResearchServiceProvider.Amount <> NULL AND bafpr3.xx.xx:Expense.ResearchAndDevelopment.ContractedExpenditureNonResearchServiceProvider.Amount <> MONETARY(U,11,0)
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            NotMonetary(^RDTIS7, 'U', 11, 0)
    
            Data Elements:
    
            RP.CTFOR:^RDTIS7 = bafpr3.02.09:Expense.ResearchAndDevelopment.ContractedExpenditureNonResearchServiceProvider.Amount
            */
            assertion = NotMonetary(report.RDTIS7, @"U", 11, 0);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.400011", Severity = ProcessMessageSeverity.Error,
                    Description = @"Field incorrect format. Exceeds maximum digits or decimals and/or contains illegal characters",
                    LongDescription = @"Field is in incorrect format for a monetary field. Field either exceeds maximum digits or decimals and/or contains illegal characters and/or for a signed field does not have the sign as the first character",
                    Location = "/xbrli:xbrl/bafpr3.02.09:Expense.ResearchAndDevelopment.ContractedExpenditureNonResearchServiceProvider.Amount[@contextRef='" + report.RPCTFORId + "']",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.RDTIS.438012"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "RDTIS7", Value = report.RDTIS7.GetValueOrDefault().ToString() });
    
                response.Add(processMessage);
            }
            #endregion // VR.ATO.RDTIS.438012
    
            #region VR.ATO.RDTIS.438013
    
            /*  VR.ATO.RDTIS.438013
            Monetary field, must be unsigned, maximum of 11 chars, no decimals
    
            Legacy Rule Format:
            IF lrla.xx.xx:Remuneration.WagesAndSalaries.Salary.Amount <> NULL AND lrla.xx.xx:Remuneration.WagesAndSalaries.Salary.Amount <> MONETARY(U,11,0)
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            NotMonetary(^RDTIS8, 'U', 11, 0)
    
            Data Elements:
    
            RP.CTAUS:^RDTIS8 = lrla.02.12:Remuneration.WagesAndSalaries.Salary.Amount
            */
            assertion = NotMonetary(report.RDTIS8, @"U", 11, 0);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.400011", Severity = ProcessMessageSeverity.Error,
                    Description = @"Field incorrect format. Exceeds maximum digits or decimals and/or contains illegal characters",
                    LongDescription = @"Field is in incorrect format for a monetary field. Field either exceeds maximum digits or decimals and/or contains illegal characters and/or for a signed field does not have the sign as the first character",
                    Location = "/xbrli:xbrl/lrla.02.12:Remuneration.WagesAndSalaries.Salary.Amount[@contextRef='" + report.RPCTAUSId + "']",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.RDTIS.438013"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "RDTIS8", Value = report.RDTIS8.GetValueOrDefault().ToString() });
    
                response.Add(processMessage);
            }
            #endregion // VR.ATO.RDTIS.438013
    
            #region VR.ATO.RDTIS.438013
    
            /*  VR.ATO.RDTIS.438013
            Monetary field, must be unsigned, maximum of 11 chars, no decimals
    
            Legacy Rule Format:
            IF lrla.xx.xx:Remuneration.WagesAndSalaries.Salary.Amount <> NULL AND lrla.xx.xx:Remuneration.WagesAndSalaries.Salary.Amount <> MONETARY(U,11,0)
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            NotMonetary(^RDTIS9, 'U', 11, 0)
    
            Data Elements:
    
            RP.CTFOR:^RDTIS9 = lrla.02.12:Remuneration.WagesAndSalaries.Salary.Amount
            */
            assertion = NotMonetary(report.RDTIS9, @"U", 11, 0);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.400011", Severity = ProcessMessageSeverity.Error,
                    Description = @"Field incorrect format. Exceeds maximum digits or decimals and/or contains illegal characters",
                    LongDescription = @"Field is in incorrect format for a monetary field. Field either exceeds maximum digits or decimals and/or contains illegal characters and/or for a signed field does not have the sign as the first character",
                    Location = "/xbrli:xbrl/lrla.02.12:Remuneration.WagesAndSalaries.Salary.Amount[@contextRef='" + report.RPCTFORId + "']",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.RDTIS.438013"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "RDTIS9", Value = report.RDTIS9.GetValueOrDefault().ToString() });
    
                response.Add(processMessage);
            }
            #endregion // VR.ATO.RDTIS.438013
    
            #region VR.ATO.RDTIS.438014
    
            /*  VR.ATO.RDTIS.438014
            Monetary field, must be unsigned, maximum of 11 chars, no decimals
    
            Legacy Rule Format:
            IF bafpr3.xx.xx:Expense.ResearchAndDevelopment.OtherExpenditureRegisteredResearchAgency.Amount <> NULL AND bafpr3.xx.xx:Expense.ResearchAndDevelopment.OtherExpenditureRegisteredResearchAgency.Amount <> MONETARY(U,11,0)
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            NotMonetary(^RDTIS10, 'U', 11, 0)
    
            Data Elements:
    
            RP.CTAUS:^RDTIS10 = bafpr3.02.08:Expense.ResearchAndDevelopment.OtherExpenditureRegisteredResearchAgency.Amount
            */
            assertion = NotMonetary(report.RDTIS10, @"U", 11, 0);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.400011", Severity = ProcessMessageSeverity.Error,
                    Description = @"Field incorrect format. Exceeds maximum digits or decimals and/or contains illegal characters",
                    LongDescription = @"Field is in incorrect format for a monetary field. Field either exceeds maximum digits or decimals and/or contains illegal characters and/or for a signed field does not have the sign as the first character",
                    Location = "/xbrli:xbrl/bafpr3.02.08:Expense.ResearchAndDevelopment.OtherExpenditureRegisteredResearchAgency.Amount[@contextRef='" + report.RPCTAUSId + "']",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.RDTIS.438014"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "RDTIS10", Value = report.RDTIS10.GetValueOrDefault().ToString() });
    
                response.Add(processMessage);
            }
            #endregion // VR.ATO.RDTIS.438014
    
            #region VR.ATO.RDTIS.438014
    
            /*  VR.ATO.RDTIS.438014
            Monetary field, must be unsigned, maximum of 11 chars, no decimals
    
            Legacy Rule Format:
            IF bafpr3.xx.xx:Expense.ResearchAndDevelopment.OtherExpenditureRegisteredResearchAgency.Amount <> NULL AND bafpr3.xx.xx:Expense.ResearchAndDevelopment.OtherExpenditureRegisteredResearchAgency.Amount <> MONETARY(U,11,0)
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            NotMonetary(^RDTIS11, 'U', 11, 0)
    
            Data Elements:
    
            RP.CTFOR:^RDTIS11 = bafpr3.02.08:Expense.ResearchAndDevelopment.OtherExpenditureRegisteredResearchAgency.Amount
            */
            assertion = NotMonetary(report.RDTIS11, @"U", 11, 0);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.400011", Severity = ProcessMessageSeverity.Error,
                    Description = @"Field incorrect format. Exceeds maximum digits or decimals and/or contains illegal characters",
                    LongDescription = @"Field is in incorrect format for a monetary field. Field either exceeds maximum digits or decimals and/or contains illegal characters and/or for a signed field does not have the sign as the first character",
                    Location = "/xbrli:xbrl/bafpr3.02.08:Expense.ResearchAndDevelopment.OtherExpenditureRegisteredResearchAgency.Amount[@contextRef='" + report.RPCTFORId + "']",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.RDTIS.438014"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "RDTIS11", Value = report.RDTIS11.GetValueOrDefault().ToString() });
    
                response.Add(processMessage);
            }
            #endregion // VR.ATO.RDTIS.438014
    
            #region VR.ATO.RDTIS.438015
    
            /*  VR.ATO.RDTIS.438015
            Monetary field, must be unsigned, maximum of 11 chars, no decimals
    
            Legacy Rule Format:
            IF bafpr3.xx.xx:Expense.ResearchAndDevelopment.Feedstock.Amount <> NULL AND bafpr3.xx.xx:Expense.ResearchAndDevelopment.Feedstock.Amount <> MONETARY(U,11,0)
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            NotMonetary(^RDTIS12, 'U', 11, 0)
    
            Data Elements:
    
            RP.CTAUS:^RDTIS12 = bafpr3.02.08:Expense.ResearchAndDevelopment.Feedstock.Amount
            */
            assertion = NotMonetary(report.RDTIS12, @"U", 11, 0);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.400011", Severity = ProcessMessageSeverity.Error,
                    Description = @"Field incorrect format. Exceeds maximum digits or decimals and/or contains illegal characters",
                    LongDescription = @"Field is in incorrect format for a monetary field. Field either exceeds maximum digits or decimals and/or contains illegal characters and/or for a signed field does not have the sign as the first character",
                    Location = "/xbrli:xbrl/bafpr3.02.08:Expense.ResearchAndDevelopment.Feedstock.Amount[@contextRef='" + report.RPCTAUSId + "']",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.RDTIS.438015"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "RDTIS12", Value = report.RDTIS12.GetValueOrDefault().ToString() });
    
                response.Add(processMessage);
            }
            #endregion // VR.ATO.RDTIS.438015
    
            #region VR.ATO.RDTIS.438015
    
            /*  VR.ATO.RDTIS.438015
            Monetary field, must be unsigned, maximum of 11 chars, no decimals
    
            Legacy Rule Format:
            IF bafpr3.xx.xx:Expense.ResearchAndDevelopment.Feedstock.Amount <> NULL AND bafpr3.xx.xx:Expense.ResearchAndDevelopment.Feedstock.Amount <> MONETARY(U,11,0)
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            NotMonetary(^RDTIS13, 'U', 11, 0)
    
            Data Elements:
    
            RP.CTFOR:^RDTIS13 = bafpr3.02.08:Expense.ResearchAndDevelopment.Feedstock.Amount
            */
            assertion = NotMonetary(report.RDTIS13, @"U", 11, 0);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.400011", Severity = ProcessMessageSeverity.Error,
                    Description = @"Field incorrect format. Exceeds maximum digits or decimals and/or contains illegal characters",
                    LongDescription = @"Field is in incorrect format for a monetary field. Field either exceeds maximum digits or decimals and/or contains illegal characters and/or for a signed field does not have the sign as the first character",
                    Location = "/xbrli:xbrl/bafpr3.02.08:Expense.ResearchAndDevelopment.Feedstock.Amount[@contextRef='" + report.RPCTFORId + "']",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.RDTIS.438015"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "RDTIS13", Value = report.RDTIS13.GetValueOrDefault().ToString() });
    
                response.Add(processMessage);
            }
            #endregion // VR.ATO.RDTIS.438015
    
            #region VR.ATO.RDTIS.438016
    
            /*  VR.ATO.RDTIS.438016
            Monetary field, must be unsigned, maximum of 11 chars, no decimals
    
            Legacy Rule Format:
            IF bafpr3.xx.xx:Expense.ResearchAndDevelopment.AssociatesExpenditure.Amount <> NULL AND bafpr3.xx.xx:Expense.ResearchAndDevelopment.AssociatesExpenditure.Amount <> MONETARY(U,11,0)
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            NotMonetary(^RDTIS14, 'U', 11, 0)
    
            Data Elements:
    
            RP.CTAUS:^RDTIS14 = bafpr3.02.08:Expense.ResearchAndDevelopment.AssociatesExpenditure.Amount
            */
            assertion = NotMonetary(report.RDTIS14, @"U", 11, 0);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.400011", Severity = ProcessMessageSeverity.Error,
                    Description = @"Field incorrect format. Exceeds maximum digits or decimals and/or contains illegal characters",
                    LongDescription = @"Field is in incorrect format for a monetary field. Field either exceeds maximum digits or decimals and/or contains illegal characters and/or for a signed field does not have the sign as the first character",
                    Location = "/xbrli:xbrl/bafpr3.02.08:Expense.ResearchAndDevelopment.AssociatesExpenditure.Amount[@contextRef='" + report.RPCTAUSId + "']",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.RDTIS.438016"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "RDTIS14", Value = report.RDTIS14.GetValueOrDefault().ToString() });
    
                response.Add(processMessage);
            }
            #endregion // VR.ATO.RDTIS.438016
    
            #region VR.ATO.RDTIS.438016
    
            /*  VR.ATO.RDTIS.438016
            Monetary field, must be unsigned, maximum of 11 chars, no decimals
    
            Legacy Rule Format:
            IF bafpr3.xx.xx:Expense.ResearchAndDevelopment.AssociatesExpenditure.Amount <> NULL AND bafpr3.xx.xx:Expense.ResearchAndDevelopment.AssociatesExpenditure.Amount <> MONETARY(U,11,0)
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            NotMonetary(^RDTIS15, 'U', 11, 0)
    
            Data Elements:
    
            RP.CTFOR:^RDTIS15 = bafpr3.02.08:Expense.ResearchAndDevelopment.AssociatesExpenditure.Amount
            */
            assertion = NotMonetary(report.RDTIS15, @"U", 11, 0);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.400011", Severity = ProcessMessageSeverity.Error,
                    Description = @"Field incorrect format. Exceeds maximum digits or decimals and/or contains illegal characters",
                    LongDescription = @"Field is in incorrect format for a monetary field. Field either exceeds maximum digits or decimals and/or contains illegal characters and/or for a signed field does not have the sign as the first character",
                    Location = "/xbrli:xbrl/bafpr3.02.08:Expense.ResearchAndDevelopment.AssociatesExpenditure.Amount[@contextRef='" + report.RPCTFORId + "']",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.RDTIS.438016"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "RDTIS15", Value = report.RDTIS15.GetValueOrDefault().ToString() });
    
                response.Add(processMessage);
            }
            #endregion // VR.ATO.RDTIS.438016
    
            #region VR.ATO.RDTIS.438017
    
            /*  VR.ATO.RDTIS.438017
            Monetary field, must be unsigned, maximum of 11 chars, no decimals
    
            Legacy Rule Format:
            IF bafpr3.xx.xx:Expense.ResearchAndDevelopment.DeclineInValuePlantAndDepreciatingAssets.Amount <> NULL AND bafpr3.xx.xx:Expense.ResearchAndDevelopment.DeclineInValuePlantAndDepreciatingAssets.Amount <> MONETARY(U,11,0)
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            NotMonetary(^RDTIS16, 'U', 11, 0)
    
            Data Elements:
    
            RP.CTAUS:^RDTIS16 = bafpr3.02.08:Expense.ResearchAndDevelopment.DeclineInValuePlantAndDepreciatingAssets.Amount
            */
            assertion = NotMonetary(report.RDTIS16, @"U", 11, 0);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.400011", Severity = ProcessMessageSeverity.Error,
                    Description = @"Field incorrect format. Exceeds maximum digits or decimals and/or contains illegal characters",
                    LongDescription = @"Field is in incorrect format for a monetary field. Field either exceeds maximum digits or decimals and/or contains illegal characters and/or for a signed field does not have the sign as the first character",
                    Location = "/xbrli:xbrl/bafpr3.02.08:Expense.ResearchAndDevelopment.DeclineInValuePlantAndDepreciatingAssets.Amount[@contextRef='" + report.RPCTAUSId + "']",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.RDTIS.438017"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "RDTIS16", Value = report.RDTIS16.GetValueOrDefault().ToString() });
    
                response.Add(processMessage);
            }
            #endregion // VR.ATO.RDTIS.438017
    
            #region VR.ATO.RDTIS.438017
    
            /*  VR.ATO.RDTIS.438017
            Monetary field, must be unsigned, maximum of 11 chars, no decimals
    
            Legacy Rule Format:
            IF bafpr3.xx.xx:Expense.ResearchAndDevelopment.DeclineInValuePlantAndDepreciatingAssets.Amount <> NULL AND bafpr3.xx.xx:Expense.ResearchAndDevelopment.DeclineInValuePlantAndDepreciatingAssets.Amount <> MONETARY(U,11,0)
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            NotMonetary(^RDTIS17, 'U', 11, 0)
    
            Data Elements:
    
            RP.CTFOR:^RDTIS17 = bafpr3.02.08:Expense.ResearchAndDevelopment.DeclineInValuePlantAndDepreciatingAssets.Amount
            */
            assertion = NotMonetary(report.RDTIS17, @"U", 11, 0);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.400011", Severity = ProcessMessageSeverity.Error,
                    Description = @"Field incorrect format. Exceeds maximum digits or decimals and/or contains illegal characters",
                    LongDescription = @"Field is in incorrect format for a monetary field. Field either exceeds maximum digits or decimals and/or contains illegal characters and/or for a signed field does not have the sign as the first character",
                    Location = "/xbrli:xbrl/bafpr3.02.08:Expense.ResearchAndDevelopment.DeclineInValuePlantAndDepreciatingAssets.Amount[@contextRef='" + report.RPCTFORId + "']",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.RDTIS.438017"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "RDTIS17", Value = report.RDTIS17.GetValueOrDefault().ToString() });
    
                response.Add(processMessage);
            }
            #endregion // VR.ATO.RDTIS.438017
    
            #region VR.ATO.RDTIS.438018
    
            /*  VR.ATO.RDTIS.438018
            Monetary field, must be unsigned, maximum of 11 chars, no decimals
    
            Legacy Rule Format:
            IF bafpr3.xx.xx:Expense.ResearchAndDevelopment.BalancingLosses.Amount <> NULL AND bafpr3.xx.xx:Expense.ResearchAndDevelopment.BalancingLosses.Amount <> MONETARY(U,11,0)
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            NotMonetary(^RDTIS18, 'U', 11, 0)
    
            Data Elements:
    
            RP.CTAUS:^RDTIS18 = bafpr3.02.08:Expense.ResearchAndDevelopment.BalancingLosses.Amount
            */
            assertion = NotMonetary(report.RDTIS18, @"U", 11, 0);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.400011", Severity = ProcessMessageSeverity.Error,
                    Description = @"Field incorrect format. Exceeds maximum digits or decimals and/or contains illegal characters",
                    LongDescription = @"Field is in incorrect format for a monetary field. Field either exceeds maximum digits or decimals and/or contains illegal characters and/or for a signed field does not have the sign as the first character",
                    Location = "/xbrli:xbrl/bafpr3.02.08:Expense.ResearchAndDevelopment.BalancingLosses.Amount[@contextRef='" + report.RPCTAUSId + "']",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.RDTIS.438018"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "RDTIS18", Value = report.RDTIS18.GetValueOrDefault().ToString() });
    
                response.Add(processMessage);
            }
            #endregion // VR.ATO.RDTIS.438018
    
            #region VR.ATO.RDTIS.438018
    
            /*  VR.ATO.RDTIS.438018
            Monetary field, must be unsigned, maximum of 11 chars, no decimals
    
            Legacy Rule Format:
            IF bafpr3.xx.xx:Expense.ResearchAndDevelopment.BalancingLosses.Amount <> NULL AND bafpr3.xx.xx:Expense.ResearchAndDevelopment.BalancingLosses.Amount <> MONETARY(U,11,0)
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            NotMonetary(^RDTIS19, 'U', 11, 0)
    
            Data Elements:
    
            RP.CTFOR:^RDTIS19 = bafpr3.02.08:Expense.ResearchAndDevelopment.BalancingLosses.Amount
            */
            assertion = NotMonetary(report.RDTIS19, @"U", 11, 0);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.400011", Severity = ProcessMessageSeverity.Error,
                    Description = @"Field incorrect format. Exceeds maximum digits or decimals and/or contains illegal characters",
                    LongDescription = @"Field is in incorrect format for a monetary field. Field either exceeds maximum digits or decimals and/or contains illegal characters and/or for a signed field does not have the sign as the first character",
                    Location = "/xbrli:xbrl/bafpr3.02.08:Expense.ResearchAndDevelopment.BalancingLosses.Amount[@contextRef='" + report.RPCTFORId + "']",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.RDTIS.438018"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "RDTIS19", Value = report.RDTIS19.GetValueOrDefault().ToString() });
    
                response.Add(processMessage);
            }
            #endregion // VR.ATO.RDTIS.438018
    
            #region VR.ATO.RDTIS.438019
    
            /*  VR.ATO.RDTIS.438019
            Monetary field, must be unsigned, maximum of 11 chars, no decimals
    
            Legacy Rule Format:
            IF bafpr3.xx.xx:Expense.ResearchAndDevelopment.CooperativeResearchCentreContributions.Amount <> NULL AND bafpr3.xx.xx:Expense.ResearchAndDevelopment.CooperativeResearchCentreContributions.Amount <> MONETARY(U,11,0)
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            NotMonetary(^RDTIS20, 'U', 11, 0)
    
            Data Elements:
    
            RP.CTAUS:^RDTIS20 = bafpr3.02.08:Expense.ResearchAndDevelopment.CooperativeResearchCentreContributions.Amount
            */
            assertion = NotMonetary(report.RDTIS20, @"U", 11, 0);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.400011", Severity = ProcessMessageSeverity.Error,
                    Description = @"Field incorrect format. Exceeds maximum digits or decimals and/or contains illegal characters",
                    LongDescription = @"Field is in incorrect format for a monetary field. Field either exceeds maximum digits or decimals and/or contains illegal characters and/or for a signed field does not have the sign as the first character",
                    Location = "/xbrli:xbrl/bafpr3.02.08:Expense.ResearchAndDevelopment.CooperativeResearchCentreContributions.Amount[@contextRef='" + report.RPCTAUSId + "']",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.RDTIS.438019"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "RDTIS20", Value = report.RDTIS20.GetValueOrDefault().ToString() });
    
                response.Add(processMessage);
            }
            #endregion // VR.ATO.RDTIS.438019
    
            #region VR.ATO.RDTIS.438019
    
            /*  VR.ATO.RDTIS.438019
            Monetary field, must be unsigned, maximum of 11 chars, no decimals
    
            Legacy Rule Format:
            IF bafpr3.xx.xx:Expense.ResearchAndDevelopment.CooperativeResearchCentreContributions.Amount <> NULL AND bafpr3.xx.xx:Expense.ResearchAndDevelopment.CooperativeResearchCentreContributions.Amount <> MONETARY(U,11,0)
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            NotMonetary(^RDTIS21, 'U', 11, 0)
    
            Data Elements:
    
            RP.CTFOR:^RDTIS21 = bafpr3.02.08:Expense.ResearchAndDevelopment.CooperativeResearchCentreContributions.Amount
            */
            assertion = NotMonetary(report.RDTIS21, @"U", 11, 0);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.400011", Severity = ProcessMessageSeverity.Error,
                    Description = @"Field incorrect format. Exceeds maximum digits or decimals and/or contains illegal characters",
                    LongDescription = @"Field is in incorrect format for a monetary field. Field either exceeds maximum digits or decimals and/or contains illegal characters and/or for a signed field does not have the sign as the first character",
                    Location = "/xbrli:xbrl/bafpr3.02.08:Expense.ResearchAndDevelopment.CooperativeResearchCentreContributions.Amount[@contextRef='" + report.RPCTFORId + "']",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.RDTIS.438019"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "RDTIS21", Value = report.RDTIS21.GetValueOrDefault().ToString() });
    
                response.Add(processMessage);
            }
            #endregion // VR.ATO.RDTIS.438019
    
            #region VR.ATO.RDTIS.438020
    
            /*  VR.ATO.RDTIS.438020
            Monetary field, must be unsigned, maximum of 11 chars, no decimals
    
            Legacy Rule Format:
            IF bafpr3.xx.xx:Expense.ResearchAndDevelopment.AllocatedTotal.Amount <> NULL AND bafpr3.xx.xx:Expense.ResearchAndDevelopment.AllocatedTotal.Amount <> MONETARY(U,11,0)
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            NotMonetary(^RDTIS24, 'U', 11, 0)
    
            Data Elements:
    
            RP:^RDTIS24 = bafpr3.02.08:Expense.ResearchAndDevelopment.AllocatedTotal.Amount
            */
            assertion = NotMonetary(report.RDTIS24, @"U", 11, 0);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.400011", Severity = ProcessMessageSeverity.Error,
                    Description = @"Field incorrect format. Exceeds maximum digits or decimals and/or contains illegal characters",
                    LongDescription = @"Field is in incorrect format for a monetary field. Field either exceeds maximum digits or decimals and/or contains illegal characters and/or for a signed field does not have the sign as the first character",
                    Location = "/xbrli:xbrl/bafpr3.02.08:Expense.ResearchAndDevelopment.AllocatedTotal.Amount[@contextRef='" + report.RPId + "']",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.RDTIS.438020"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "RDTIS24", Value = report.RDTIS24.GetValueOrDefault().ToString() });
    
                response.Add(processMessage);
            }
            #endregion // VR.ATO.RDTIS.438020
    
            #region VR.ATO.RDTIS.438020
    
            /*  VR.ATO.RDTIS.438020
            Monetary field, must be unsigned, maximum of 11 chars, no decimals
    
            Legacy Rule Format:
            IF bafpr3.xx.xx:Expense.ResearchAndDevelopment.AllocatedTotal.Amount <> NULL AND bafpr3.xx.xx:Expense.ResearchAndDevelopment.AllocatedTotal.Amount <> MONETARY(U,11,0)
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            NotMonetary(^RDTIS22, 'U', 11, 0)
    
            Data Elements:
    
            RP.CTAUS:^RDTIS22 = bafpr3.02.08:Expense.ResearchAndDevelopment.AllocatedTotal.Amount
            */
            assertion = NotMonetary(report.RDTIS22, @"U", 11, 0);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.400011", Severity = ProcessMessageSeverity.Error,
                    Description = @"Field incorrect format. Exceeds maximum digits or decimals and/or contains illegal characters",
                    LongDescription = @"Field is in incorrect format for a monetary field. Field either exceeds maximum digits or decimals and/or contains illegal characters and/or for a signed field does not have the sign as the first character",
                    Location = "/xbrli:xbrl/bafpr3.02.08:Expense.ResearchAndDevelopment.AllocatedTotal.Amount[@contextRef='" + report.RPCTAUSId + "']",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.RDTIS.438020"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "RDTIS22", Value = report.RDTIS22.GetValueOrDefault().ToString() });
    
                response.Add(processMessage);
            }
            #endregion // VR.ATO.RDTIS.438020
    
            #region VR.ATO.RDTIS.438020
    
            /*  VR.ATO.RDTIS.438020
            Monetary field, must be unsigned, maximum of 11 chars, no decimals
    
            Legacy Rule Format:
            IF bafpr3.xx.xx:Expense.ResearchAndDevelopment.AllocatedTotal.Amount <> NULL AND bafpr3.xx.xx:Expense.ResearchAndDevelopment.AllocatedTotal.Amount <> MONETARY(U,11,0)
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            NotMonetary(^RDTIS23, 'U', 11, 0)
    
            Data Elements:
    
            RP.CTFOR:^RDTIS23 = bafpr3.02.08:Expense.ResearchAndDevelopment.AllocatedTotal.Amount
            */
            assertion = NotMonetary(report.RDTIS23, @"U", 11, 0);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.400011", Severity = ProcessMessageSeverity.Error,
                    Description = @"Field incorrect format. Exceeds maximum digits or decimals and/or contains illegal characters",
                    LongDescription = @"Field is in incorrect format for a monetary field. Field either exceeds maximum digits or decimals and/or contains illegal characters and/or for a signed field does not have the sign as the first character",
                    Location = "/xbrli:xbrl/bafpr3.02.08:Expense.ResearchAndDevelopment.AllocatedTotal.Amount[@contextRef='" + report.RPCTFORId + "']",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.RDTIS.438020"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "RDTIS23", Value = report.RDTIS23.GetValueOrDefault().ToString() });
    
                response.Add(processMessage);
            }
            #endregion // VR.ATO.RDTIS.438020
    
            #region VR.ATO.RDTIS.438021
    
            /*  VR.ATO.RDTIS.438021
            Australian owned R&D Total of allocated notional deductions must equal the sum of Australian owned R&D amounts
    
            Legacy Rule Format:
            IF [RDTIS22] <> NULL AND [RDTIS22] <> ([RDTIS4] + [RDTIS6] + [RDTIS8] + [RDTIS10] + [RDTIS12] + [RDTIS14] + [RDTIS16] + [RDTIS18] + [RDTIS20])
               RETURN VALIDATION MESSAGE
            ENDIF
            
            [RDTIS4] = RDTIS:RP.CTAUS:bafpr3.02.09:Expense.ResearchAndDevelopment.ContractedExpenditureResearchServiceProvider.Amount
            [RDTIS6] = RDTIS:RP.CTAUS:bafpr3.02.09:Expense.ResearchAndDevelopment.ContractedExpenditureNonResearchServiceProvider.Amount
            [RDTIS8] = RDTIS:RP.CTAUS:lrla.02.12:Remuneration.WagesAndSalaries.Salary.Amount
            [RDTIS10] = RDTIS:RP.CTAUS:bafpr3.02.08:Expense.ResearchAndDevelopment.OtherExpenditureRegisteredResearchAgency.Amount
            [RDTIS12] = RDTIS:RP.CTAUS:bafpr3.02.08:Expense.ResearchAndDevelopment.Feedstock.Amount
            [RDTIS14] = RDTIS:RP.CTAUS:bafpr3.02.08:Expense.ResearchAndDevelopment.AssociatesExpenditure.Amount
            [RDTIS16] = RDTIS:RP.CTAUS:bafpr3.02.08:Expense.ResearchAndDevelopment.DeclineInValuePlantAndDepreciatingAssets.Amount
            [RDTIS18] = RDTIS:RP.CTAUS:bafpr3.02.08:Expense.ResearchAndDevelopment.BalancingLosses.Amount
            [RDTIS20] = RDTIS:RP.CTAUS:bafpr3.02.08:Expense.ResearchAndDevelopment.CooperativeResearchCentreContributions.Amount
            [RDTIS22] = RDTIS:RP.CTAUS:bafpr3.02.08:Expense.ResearchAndDevelopment.AllocatedTotal.Amount

            Technical Business Rule Format:
            ^RDTIS22 <> NULL AND ^RDTIS22 <> (^RDTIS4 + ^RDTIS6 + ^RDTIS8 + ^RDTIS10 + ^RDTIS12 + ^RDTIS14 + ^RDTIS16 + ^RDTIS18 + ^RDTIS20)
    
            Data Elements:
    
            RP.CTAUS:^RDTIS22 = bafpr3.02.08:Expense.ResearchAndDevelopment.AllocatedTotal.Amount
    
            RP.CTAUS:^RDTIS4 = bafpr3.02.09:Expense.ResearchAndDevelopment.ContractedExpenditureResearchServiceProvider.Amount
    
            RP.CTAUS:^RDTIS6 = bafpr3.02.09:Expense.ResearchAndDevelopment.ContractedExpenditureNonResearchServiceProvider.Amount
    
            RP.CTAUS:^RDTIS8 = lrla.02.12:Remuneration.WagesAndSalaries.Salary.Amount
    
            RP.CTAUS:^RDTIS10 = bafpr3.02.08:Expense.ResearchAndDevelopment.OtherExpenditureRegisteredResearchAgency.Amount
    
            RP.CTAUS:^RDTIS12 = bafpr3.02.08:Expense.ResearchAndDevelopment.Feedstock.Amount
    
            RP.CTAUS:^RDTIS14 = bafpr3.02.08:Expense.ResearchAndDevelopment.AssociatesExpenditure.Amount
    
            RP.CTAUS:^RDTIS16 = bafpr3.02.08:Expense.ResearchAndDevelopment.DeclineInValuePlantAndDepreciatingAssets.Amount
    
            RP.CTAUS:^RDTIS18 = bafpr3.02.08:Expense.ResearchAndDevelopment.BalancingLosses.Amount
    
            RP.CTAUS:^RDTIS20 = bafpr3.02.08:Expense.ResearchAndDevelopment.CooperativeResearchCentreContributions.Amount
            */
            assertion = (report.RDTIS22 != null && report.RDTIS22.GetValueOrDefault() != report.RDTIS4.GetValueOrDefault() + report.RDTIS6.GetValueOrDefault() + report.RDTIS8.GetValueOrDefault() + report.RDTIS10.GetValueOrDefault() + report.RDTIS12.GetValueOrDefault() + report.RDTIS14.GetValueOrDefault() + report.RDTIS16.GetValueOrDefault() + report.RDTIS18.GetValueOrDefault() + report.RDTIS20.GetValueOrDefault());
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.RDTIS.438021", Severity = ProcessMessageSeverity.Error,
                    Description = @"Australian owned R&D Total of allocated notional deductions is incorrect",
                    LongDescription = @"Australian owned R&D Total of allocated notional deductions must equal the sum of Australian owned R&D amounts",
                    Location = "/xbrli:xbrl/bafpr3.02.08:Expense.ResearchAndDevelopment.AllocatedTotal.Amount[@contextRef='" + report.RPCTAUSId + "']",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.RDTIS.438021"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "RDTIS22", Value = report.RDTIS22.GetValueOrDefault().ToString() });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "RDTIS4", Value = report.RDTIS4.GetValueOrDefault().ToString() });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "RDTIS6", Value = report.RDTIS6.GetValueOrDefault().ToString() });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "RDTIS8", Value = report.RDTIS8.GetValueOrDefault().ToString() });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "RDTIS10", Value = report.RDTIS10.GetValueOrDefault().ToString() });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "RDTIS12", Value = report.RDTIS12.GetValueOrDefault().ToString() });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "RDTIS14", Value = report.RDTIS14.GetValueOrDefault().ToString() });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "RDTIS16", Value = report.RDTIS16.GetValueOrDefault().ToString() });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "RDTIS18", Value = report.RDTIS18.GetValueOrDefault().ToString() });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "RDTIS20", Value = report.RDTIS20.GetValueOrDefault().ToString() });
    
                response.Add(processMessage);
            }
            #endregion // VR.ATO.RDTIS.438021
    
            #region VR.ATO.RDTIS.438022
    
            /*  VR.ATO.RDTIS.438022
            Foreign owned R&D Total of allocated notional deductions must equal the sum of Foreign owned R&D amounts
    
            Legacy Rule Format:
            IF [RDTIS23] <> NULL AND [RDTIS23] <> ([RDTIS5] + [RDTIS7] + [RDTIS9] + [RDTIS11] + [RDTIS13] + [RDTIS15] + [RDTIS17] + [RDTIS19] + [RDTIS21])
               RETURN VALIDATION MESSAGE
            ENDIF
            
            [RDTIS5] = RDTIS:RP.CTFOR:bafpr3.02.09:Expense.ResearchAndDevelopment.ContractedExpenditureResearchServiceProvider.Amount
            [RDTIS7] = RDTIS:RP.CTFOR:bafpr3.02.09:Expense.ResearchAndDevelopment.ContractedExpenditureNonResearchServiceProvider.Amount
            [RDTIS9] = RDTIS:RP.CTFOR:lrla.02.12:Remuneration.WagesAndSalaries.Salary.Amount
            [RDTIS11] = RDTIS:RP.CTFOR:bafpr3.02.08:Expense.ResearchAndDevelopment.OtherExpenditureRegisteredResearchAgency.Amount
            [RDTIS13] = RDTIS:RP.CTFOR:bafpr3.02.08:Expense.ResearchAndDevelopment.Feedstock.Amount
            [RDTIS15] = RDTIS:RP.CTFOR:bafpr3.02.08:Expense.ResearchAndDevelopment.AssociatesExpenditure.Amount
            [RDTIS17] = RDTIS:RP.CTFOR:bafpr3.02.08:Expense.ResearchAndDevelopment.DeclineInValuePlantAndDepreciatingAssets.Amount
            [RDTIS19] = RDTIS:RP.CTFOR:bafpr3.02.08:Expense.ResearchAndDevelopment.BalancingLosses.Amount
            [RDTIS21] = RDTIS:RP.CTFOR:bafpr3.02.08:Expense.ResearchAndDevelopment.CooperativeResearchCentreContributions.Amount
            [RDTIS23] = RDTIS:RP.CTFOR:bafpr3.02.08:Expense.ResearchAndDevelopment.AllocatedTotal.Amount

            Technical Business Rule Format:
            ^RDTIS23 <> NULL AND ^RDTIS23 <> (^RDTIS5 + ^RDTIS7 + ^RDTIS9 + ^RDTIS11 + ^RDTIS13 + ^RDTIS15 + ^RDTIS17 + ^RDTIS19 + ^RDTIS21)
    
            Data Elements:
    
            RP.CTFOR:^RDTIS23 = bafpr3.02.08:Expense.ResearchAndDevelopment.AllocatedTotal.Amount
    
            RP.CTFOR:^RDTIS5 = bafpr3.02.09:Expense.ResearchAndDevelopment.ContractedExpenditureResearchServiceProvider.Amount
    
            RP.CTFOR:^RDTIS7 = bafpr3.02.09:Expense.ResearchAndDevelopment.ContractedExpenditureNonResearchServiceProvider.Amount
    
            RP.CTFOR:^RDTIS9 = lrla.02.12:Remuneration.WagesAndSalaries.Salary.Amount
    
            RP.CTFOR:^RDTIS11 = bafpr3.02.08:Expense.ResearchAndDevelopment.OtherExpenditureRegisteredResearchAgency.Amount
    
            RP.CTFOR:^RDTIS13 = bafpr3.02.08:Expense.ResearchAndDevelopment.Feedstock.Amount
    
            RP.CTFOR:^RDTIS15 = bafpr3.02.08:Expense.ResearchAndDevelopment.AssociatesExpenditure.Amount
    
            RP.CTFOR:^RDTIS17 = bafpr3.02.08:Expense.ResearchAndDevelopment.DeclineInValuePlantAndDepreciatingAssets.Amount
    
            RP.CTFOR:^RDTIS19 = bafpr3.02.08:Expense.ResearchAndDevelopment.BalancingLosses.Amount
    
            RP.CTFOR:^RDTIS21 = bafpr3.02.08:Expense.ResearchAndDevelopment.CooperativeResearchCentreContributions.Amount
            */
            assertion = (report.RDTIS23 != null && report.RDTIS23.GetValueOrDefault() != report.RDTIS5.GetValueOrDefault() + report.RDTIS7.GetValueOrDefault() + report.RDTIS9.GetValueOrDefault() + report.RDTIS11.GetValueOrDefault() + report.RDTIS13.GetValueOrDefault() + report.RDTIS15.GetValueOrDefault() + report.RDTIS17.GetValueOrDefault() + report.RDTIS19.GetValueOrDefault() + report.RDTIS21.GetValueOrDefault());
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.RDTIS.438022", Severity = ProcessMessageSeverity.Error,
                    Description = @"Foreign owned R&D Total of allocated notional deductions is incorrect",
                    LongDescription = @"Foreign owned R&D Total of allocated notional deductions must equal the sum of Foreign owned R&D amounts",
                    Location = "/xbrli:xbrl/bafpr3.02.08:Expense.ResearchAndDevelopment.AllocatedTotal.Amount[@contextRef='" + report.RPCTFORId + "']",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.RDTIS.438022"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "RDTIS23", Value = report.RDTIS23.GetValueOrDefault().ToString() });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "RDTIS5", Value = report.RDTIS5.GetValueOrDefault().ToString() });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "RDTIS7", Value = report.RDTIS7.GetValueOrDefault().ToString() });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "RDTIS9", Value = report.RDTIS9.GetValueOrDefault().ToString() });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "RDTIS11", Value = report.RDTIS11.GetValueOrDefault().ToString() });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "RDTIS13", Value = report.RDTIS13.GetValueOrDefault().ToString() });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "RDTIS15", Value = report.RDTIS15.GetValueOrDefault().ToString() });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "RDTIS17", Value = report.RDTIS17.GetValueOrDefault().ToString() });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "RDTIS19", Value = report.RDTIS19.GetValueOrDefault().ToString() });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "RDTIS21", Value = report.RDTIS21.GetValueOrDefault().ToString() });
    
                response.Add(processMessage);
            }
            #endregion // VR.ATO.RDTIS.438022
    
            #region VR.ATO.RDTIS.438024
    
            /*  VR.ATO.RDTIS.438024
            If lodging the Research and Development Tax Incentive Schedule the Total Notional R&D deductions must be greater than zero
    
            Legacy Rule Format:
            IF [RDTIS24] = NULL OR [RDTIS24] = 0
               RETURN VALIDATION MESSAGE
            ENDIF
            
            [RDTIS24] = RDTIS:RP:bafpr3.02.08:Expense.ResearchAndDevelopment.AllocatedTotal.Amount

            Technical Business Rule Format:
            ^RDTIS24 = NULL OR ^RDTIS24 = 0
    
            Data Elements:
    
            RP:^RDTIS24 = bafpr3.02.08:Expense.ResearchAndDevelopment.AllocatedTotal.Amount
            */
            assertion = (report.RDTIS24 == null || report.RDTIS24 == 0);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.RDTIS.438024", Severity = ProcessMessageSeverity.Error,
                    Description = @"Total Notional R&D deductions must be greater than zero",
                    LongDescription = @"If lodging the Research and Development Tax Incentive Schedule the Total Notional R&D deductions must be greater than zero",
                    Location = "/xbrli:xbrl/bafpr3.02.08:Expense.ResearchAndDevelopment.AllocatedTotal.Amount[@contextRef='" + report.RPId + "']",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.RDTIS.438024"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "RDTIS24", Value = report.RDTIS24.GetValueOrDefault().ToString() });
    
                response.Add(processMessage);
            }
            #endregion // VR.ATO.RDTIS.438024
    
            #region VR.ATO.RDTIS.438025
    
            /*  VR.ATO.RDTIS.438025
            Total Notional R&D deductions must equal the sum of Australian and Foreign owned R&D Total of allocated notional deductions
    
            Legacy Rule Format:
            IF ([RDTIS22] + [RDTIS23]) >= 20000 AND [RDTIS24] <> ([RDTIS22] + [RDTIS23])
               RETURN VALIDATION MESSAGE
            ENDIF
            
            [RDTIS22] = RDTIS:RP.CTAUS:bafpr3.02.08:Expense.ResearchAndDevelopment.AllocatedTotal.Amount
            [RDTIS23] = RDTIS:RP.CTFOR:bafpr3.02.08:Expense.ResearchAndDevelopment.AllocatedTotal.Amount
            [RDTIS24] = RDTIS:RP:bafpr3.02.08:Expense.ResearchAndDevelopment.AllocatedTotal.Amount

            Technical Business Rule Format:
            (^RDTIS22 + ^RDTIS23) >= 20000 AND ^RDTIS24 <> (^RDTIS22 + ^RDTIS23)
    
            Data Elements:
    
            RP:^RDTIS24 = bafpr3.02.08:Expense.ResearchAndDevelopment.AllocatedTotal.Amount
    
            RP.CTAUS:^RDTIS22 = bafpr3.02.08:Expense.ResearchAndDevelopment.AllocatedTotal.Amount
    
            RP.CTFOR:^RDTIS23 = bafpr3.02.08:Expense.ResearchAndDevelopment.AllocatedTotal.Amount
            */
            assertion = (report.RDTIS22.GetValueOrDefault() + report.RDTIS23.GetValueOrDefault() >= 20000 && report.RDTIS24.GetValueOrDefault() != report.RDTIS22.GetValueOrDefault() + report.RDTIS23.GetValueOrDefault());
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.RDTIS.438025", Severity = ProcessMessageSeverity.Error,
                    Description = @"Total Notional R&D deductions is incorrect",
                    LongDescription = @"If the sum of Australian and Foreign owned R&D Total of allocated notional deductions is equal to or greater than $20,000 then the Total Notional R&D deductions must equal the sum of Australian and Foreign owned R&D Total of allocated notional deductions",
                    Location = "/xbrli:xbrl/bafpr3.02.08:Expense.ResearchAndDevelopment.AllocatedTotal.Amount[@contextRef='" + report.RPId + "']",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.RDTIS.438025"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "RDTIS22", Value = report.RDTIS22.GetValueOrDefault().ToString() });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "RDTIS23", Value = report.RDTIS23.GetValueOrDefault().ToString() });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "RDTIS24", Value = report.RDTIS24.GetValueOrDefault().ToString() });
    
                response.Add(processMessage);
            }
            #endregion // VR.ATO.RDTIS.438025
    
            #region VR.ATO.RDTIS.438026
    
            /*  VR.ATO.RDTIS.438026
            Total Notional R&D deductions must equal the sum of Australian and Foreign owned R&D expenditure Research service provider amounts plus the sum of Australian and Foreign owned R&D expenditure Cooperative Research Centre contributions amounts
    
            Legacy Rule Format:
            IF ([RDTIS22] + [RDTIS23]) < 20000 AND [RDTIS24] <> ([RDTIS4] + [RDTIS5] + [RDTIS20] + [RDTIS21])
               RETURN VALIDATION MESSAGE
            ENDIF
            
            [RDTIS4] = RDTIS:RP.CTAUS:bafpr3.02.09:Expense.ResearchAndDevelopment.ContractedExpenditureResearchServiceProvider.Amount
            [RDTIS5] = RDTIS:RP.CTFOR:bafpr3.02.09:Expense.ResearchAndDevelopment.ContractedExpenditureResearchServiceProvider.Amount
            [RDTIS20] = RDTIS:RP.CTAUS:bafpr3.02.08:Expense.ResearchAndDevelopment.CooperativeResearchCentreContributions.Amount
            [RDTIS21] = RDTIS:RP.CTFOR:bafpr3.02.08:Expense.ResearchAndDevelopment.CooperativeResearchCentreContributions.Amount
            [RDTIS22] = RDTIS:RP.CTAUS:bafpr3.02.08:Expense.ResearchAndDevelopment.AllocatedTotal.Amount
            [RDTIS23] = RDTIS:RP.CTFOR:bafpr3.02.08:Expense.ResearchAndDevelopment.AllocatedTotal.Amount
            [RDTIS24] = RDTIS:RP:bafpr3.02.08:Expense.ResearchAndDevelopment.AllocatedTotal.Amount

            Technical Business Rule Format:
            (^RDTIS22 + ^RDTIS23) < 20000 AND ^RDTIS24 <> (^RDTIS4 + ^RDTIS5 + ^RDTIS20 + ^RDTIS21)
    
            Data Elements:
    
            RP:^RDTIS24 = bafpr3.02.08:Expense.ResearchAndDevelopment.AllocatedTotal.Amount
    
            RP.CTAUS:^RDTIS4 = bafpr3.02.09:Expense.ResearchAndDevelopment.ContractedExpenditureResearchServiceProvider.Amount
    
            RP.CTFOR:^RDTIS5 = bafpr3.02.09:Expense.ResearchAndDevelopment.ContractedExpenditureResearchServiceProvider.Amount
    
            RP.CTAUS:^RDTIS20 = bafpr3.02.08:Expense.ResearchAndDevelopment.CooperativeResearchCentreContributions.Amount
    
            RP.CTFOR:^RDTIS21 = bafpr3.02.08:Expense.ResearchAndDevelopment.CooperativeResearchCentreContributions.Amount
    
            RP.CTAUS:^RDTIS22 = bafpr3.02.08:Expense.ResearchAndDevelopment.AllocatedTotal.Amount
    
            RP.CTFOR:^RDTIS23 = bafpr3.02.08:Expense.ResearchAndDevelopment.AllocatedTotal.Amount
            */
            assertion = (report.RDTIS22.GetValueOrDefault() + report.RDTIS23.GetValueOrDefault() < 20000 && report.RDTIS24.GetValueOrDefault() != report.RDTIS4.GetValueOrDefault() + report.RDTIS5.GetValueOrDefault() + report.RDTIS20.GetValueOrDefault() + report.RDTIS21.GetValueOrDefault());
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.RDTIS.438026", Severity = ProcessMessageSeverity.Error,
                    Description = @"Total Notional R&D deductions is incorrect",
                    LongDescription = @"If the sum of Australian and Foreign owned R&D Total of allocated notional deductions is less than $20,000 then Total Notional R&D deductions must equal the sum of Australian and Foreign owned R&D expenditure Research service provider amounts plus the sum of Australian and Foreign owned R&D expenditure Cooperative Research Centre contributions amounts",
                    Location = "/xbrli:xbrl/bafpr3.02.08:Expense.ResearchAndDevelopment.AllocatedTotal.Amount[@contextRef='" + report.RPId + "']",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.RDTIS.438026"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "RDTIS22", Value = report.RDTIS22.GetValueOrDefault().ToString() });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "RDTIS23", Value = report.RDTIS23.GetValueOrDefault().ToString() });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "RDTIS24", Value = report.RDTIS24.GetValueOrDefault().ToString() });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "RDTIS4", Value = report.RDTIS4.GetValueOrDefault().ToString() });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "RDTIS5", Value = report.RDTIS5.GetValueOrDefault().ToString() });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "RDTIS20", Value = report.RDTIS20.GetValueOrDefault().ToString() });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "RDTIS21", Value = report.RDTIS21.GetValueOrDefault().ToString() });
    
                response.Add(processMessage);
            }
            #endregion // VR.ATO.RDTIS.438026
    
            #region VR.ATO.RDTIS.438027
    
            /*  VR.ATO.RDTIS.438027
            Monetary field, must be unsigned, maximum of 11 chars, no decimals
    
            Legacy Rule Format:
            IF [RDTIS25] <> NULL AND [RDTIS25] <> MONETARY(U,11,0)
               RETURN VALIDATION MESSAGE
            ENDIF
            
            [RDTIS25] = RDTIS:RP:bafpr3.02.08:Income.ResearchAndDevelopment.FeedstockRevenueTotal.Amount

            Technical Business Rule Format:
            NotMonetary(^RDTIS25, 'U', 11, 0)
    
            Data Elements:
    
            RP:^RDTIS25 = bafpr3.02.08:Income.ResearchAndDevelopment.FeedstockRevenueTotal.Amount
            */
            assertion = NotMonetary(report.RDTIS25, @"U", 11, 0);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.400011", Severity = ProcessMessageSeverity.Error,
                    Description = @"Field incorrect format. Exceeds maximum digits or decimals and/or contains illegal characters",
                    LongDescription = @"Field is in incorrect format for a monetary field. Field either exceeds maximum digits or decimals and/or contains illegal characters and/or for a signed field does not have the sign as the first character",
                    Location = "/xbrli:xbrl/bafpr3.02.08:Income.ResearchAndDevelopment.FeedstockRevenueTotal.Amount",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.RDTIS.438027"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "RDTIS25", Value = report.RDTIS25.GetValueOrDefault().ToString() });
    
                response.Add(processMessage);
            }
            #endregion // VR.ATO.RDTIS.438027
    
            #region VR.ATO.RDTIS.438028
    
            /*  VR.ATO.RDTIS.438028
            Monetary field, must be unsigned, maximum of 11 chars, no decimals
    
            Legacy Rule Format:
            IF [RDTIS26] <> NULL AND [RDTIS26] <> MONETARY(U,11,0)
               RETURN VALIDATION MESSAGE
            ENDIF
            
            [RDTIS26] = RDTIS:RP:bafpr3.02.08:Expense.ResearchAndDevelopment.FeedstockInput.Amount

            Technical Business Rule Format:
            NotMonetary(^RDTIS26, 'U', 11, 0)
    
            Data Elements:
    
            RP:^RDTIS26 = bafpr3.02.08:Expense.ResearchAndDevelopment.FeedstockInput.Amount
            */
            assertion = NotMonetary(report.RDTIS26, @"U", 11, 0);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.400011", Severity = ProcessMessageSeverity.Error,
                    Description = @"Field incorrect format. Exceeds maximum digits or decimals and/or contains illegal characters",
                    LongDescription = @"Field is in incorrect format for a monetary field. Field either exceeds maximum digits or decimals and/or contains illegal characters and/or for a signed field does not have the sign as the first character",
                    Location = "/xbrli:xbrl/bafpr3.02.08:Expense.ResearchAndDevelopment.FeedstockInput.Amount",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.RDTIS.438028"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "RDTIS26", Value = report.RDTIS26.GetValueOrDefault().ToString() });
    
                response.Add(processMessage);
            }
            #endregion // VR.ATO.RDTIS.438028
    
            #region VR.ATO.RDTIS.438029
    
            /*  VR.ATO.RDTIS.438029
            Monetary field, must be unsigned, maximum of 11 chars, no decimals
    
            Legacy Rule Format:
            IF [RDTIS27] <> NULL AND [RDTIS27] <> MONETARY(U,11,0)
               RETURN VALIDATION MESSAGE
            ENDIF
            
            [RDTIS27] = RDTIS:RP:bafpr3.02.08:Income.ResearchAndDevelopment.FeedstockNet.Amount

            Technical Business Rule Format:
            NotMonetary(^RDTIS27, 'U', 11, 0)
    
            Data Elements:
    
            RP:^RDTIS27 = bafpr3.02.08:Income.ResearchAndDevelopment.FeedstockNet.Amount
            */
            assertion = NotMonetary(report.RDTIS27, @"U", 11, 0);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.400011", Severity = ProcessMessageSeverity.Error,
                    Description = @"Field incorrect format. Exceeds maximum digits or decimals and/or contains illegal characters",
                    LongDescription = @"Field is in incorrect format for a monetary field. Field either exceeds maximum digits or decimals and/or contains illegal characters and/or for a signed field does not have the sign as the first character",
                    Location = "/xbrli:xbrl/bafpr3.02.08:Income.ResearchAndDevelopment.FeedstockNet.Amount",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.RDTIS.438029"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "RDTIS27", Value = report.RDTIS27.GetValueOrDefault().ToString() });
    
                response.Add(processMessage);
            }
            #endregion // VR.ATO.RDTIS.438029
    
            #region VR.ATO.RDTIS.438030
    
            /*  VR.ATO.RDTIS.438030
            Monetary field, must be unsigned, maximum of 11 chars, no decimals
    
            Legacy Rule Format:
            IF [RDTIS28] <> NULL AND [RDTIS28] <> MONETARY(U,11,0)
               RETURN VALIDATION MESSAGE
            ENDIF
            
            [RDTIS28] = RDTIS:RP:gfagg.02.03:ResearchAndDevelopment.GovernmentGrantsAndRecoupments.Amount

            Technical Business Rule Format:
            NotMonetary(^RDTIS28, 'U', 11, 0)
    
            Data Elements:
    
            RP:^RDTIS28 = gfagg.02.03:ResearchAndDevelopment.GovernmentGrantsAndRecoupments.Amount
            */
            assertion = NotMonetary(report.RDTIS28, @"U", 11, 0);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.400011", Severity = ProcessMessageSeverity.Error,
                    Description = @"Field incorrect format. Exceeds maximum digits or decimals and/or contains illegal characters",
                    LongDescription = @"Field is in incorrect format for a monetary field. Field either exceeds maximum digits or decimals and/or contains illegal characters and/or for a signed field does not have the sign as the first character",
                    Location = "/xbrli:xbrl/gfagg.02.03:ResearchAndDevelopment.GovernmentGrantsAndRecoupments.Amount",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.RDTIS.438030"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "RDTIS28", Value = report.RDTIS28.GetValueOrDefault().ToString() });
    
                response.Add(processMessage);
            }
            #endregion // VR.ATO.RDTIS.438030
    
            #region VR.ATO.RDTIS.438031
    
            /*  VR.ATO.RDTIS.438031
            Monetary field, must be unsigned, maximum of 11 chars, no decimals
    
            Legacy Rule Format:
            IF [RDTIS29] <> NULL AND [RDTIS29] <> MONETARY(U,11,0)
               RETURN VALIDATION MESSAGE
            ENDIF
            
            [RDTIS29] = RDTIS:RP:bafpr3.02.08:Expense.ResearchAndDevelopment.RecoupmentRelated.Amount

            Technical Business Rule Format:
            NotMonetary(^RDTIS29, 'U', 11, 0)
    
            Data Elements:
    
            RP:^RDTIS29 = bafpr3.02.08:Expense.ResearchAndDevelopment.RecoupmentRelated.Amount
            */
            assertion = NotMonetary(report.RDTIS29, @"U", 11, 0);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.400011", Severity = ProcessMessageSeverity.Error,
                    Description = @"Field incorrect format. Exceeds maximum digits or decimals and/or contains illegal characters",
                    LongDescription = @"Field is in incorrect format for a monetary field. Field either exceeds maximum digits or decimals and/or contains illegal characters and/or for a signed field does not have the sign as the first character",
                    Location = "/xbrli:xbrl/bafpr3.02.08:Expense.ResearchAndDevelopment.RecoupmentRelated.Amount",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.RDTIS.438031"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "RDTIS29", Value = report.RDTIS29.GetValueOrDefault().ToString() });
    
                response.Add(processMessage);
            }
            #endregion // VR.ATO.RDTIS.438031
    
            #region VR.ATO.RDTIS.438032
    
            /*  VR.ATO.RDTIS.438032
            Monetary field, must be unsigned, maximum of 11 chars, no decimals
    
            Legacy Rule Format:
            IF [RDTIS30] <> NULL AND [RDTIS30] <> MONETARY(U,11,0)
               RETURN VALIDATION MESSAGE
            ENDIF
            
            [RDTIS30] = RDTIS:RP:bafpr3.02.08:Expense.ResearchAndDevelopment.GrantRelated.Amount

            Technical Business Rule Format:
            NotMonetary(^RDTIS30, 'U', 11, 0)
    
            Data Elements:
    
            RP:^RDTIS30 = bafpr3.02.08:Expense.ResearchAndDevelopment.GrantRelated.Amount
            */
            assertion = NotMonetary(report.RDTIS30, @"U", 11, 0);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.400011", Severity = ProcessMessageSeverity.Error,
                    Description = @"Field incorrect format. Exceeds maximum digits or decimals and/or contains illegal characters",
                    LongDescription = @"Field is in incorrect format for a monetary field. Field either exceeds maximum digits or decimals and/or contains illegal characters and/or for a signed field does not have the sign as the first character",
                    Location = "/xbrli:xbrl/bafpr3.02.08:Expense.ResearchAndDevelopment.GrantRelated.Amount",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.RDTIS.438032"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "RDTIS30", Value = report.RDTIS30.GetValueOrDefault().ToString() });
    
                response.Add(processMessage);
            }
            #endregion // VR.ATO.RDTIS.438032
    
            #region VR.ATO.RDTIS.438033
    
            /*  VR.ATO.RDTIS.438033
            Monetary field, must be unsigned, maximum of 13 chars, may contain a decimal point and up to 2 decimals
    
            Legacy Rule Format:
            IF [RDTIS31] <> NULL AND [RDTIS31] <> MONETARY(U,13,2)
               RETURN VALIDATION MESSAGE
            ENDIF
            
            [RDTIS31] = RDTIS:RP:gfagc.02.04:TaxConcession.ResearchAndDevelopment.GovernmentGrantsRecoupmentsClawback.Amount

            Technical Business Rule Format:
            NotMonetary(^RDTIS31, 'U', 13, 2)
    
            Data Elements:
    
            RP:^RDTIS31 = gfagc.02.04:TaxConcession.ResearchAndDevelopment.GovernmentGrantsRecoupmentsClawback.Amount
            */
            assertion = NotMonetary(report.RDTIS31, @"U", 13, 2);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.400011", Severity = ProcessMessageSeverity.Error,
                    Description = @"Field incorrect format. Exceeds maximum digits or decimals and/or contains illegal characters",
                    LongDescription = @"Field is in incorrect format for a monetary field. Field either exceeds maximum digits or decimals and/or contains illegal characters and/or for a signed field does not have the sign as the first character",
                    Location = "/xbrli:xbrl/gfagc.02.04:TaxConcession.ResearchAndDevelopment.GovernmentGrantsRecoupmentsClawback.Amount",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.RDTIS.438033"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "RDTIS31", Value = report.RDTIS31.GetValueOrDefault().ToString() });
    
                response.Add(processMessage);
            }
            #endregion // VR.ATO.RDTIS.438033
    
            #region VR.ATO.RDTIS.438034
    
            /*  VR.ATO.RDTIS.438034
            Monetary field, must be unsigned, maximum of 11 chars, no decimals
    
            Legacy Rule Format:
            IF [RDTIS32] <> NULL AND [RDTIS32] <> MONETARY(U,11,0)
               RETURN VALIDATION MESSAGE
            ENDIF
            
            [RDTIS32] = RDTIS:RP:bafpr3.02.08:Expense.ResearchAndDevelopment.AssociatesUnpaid.Amount

            Technical Business Rule Format:
            NotMonetary(^RDTIS32, 'U', 11, 0)
    
            Data Elements:
    
            RP:^RDTIS32 = bafpr3.02.08:Expense.ResearchAndDevelopment.AssociatesUnpaid.Amount
            */
            assertion = NotMonetary(report.RDTIS32, @"U", 11, 0);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.400011", Severity = ProcessMessageSeverity.Error,
                    Description = @"Field incorrect format. Exceeds maximum digits or decimals and/or contains illegal characters",
                    LongDescription = @"Field is in incorrect format for a monetary field. Field either exceeds maximum digits or decimals and/or contains illegal characters and/or for a signed field does not have the sign as the first character",
                    Location = "/xbrli:xbrl/bafpr3.02.08:Expense.ResearchAndDevelopment.AssociatesUnpaid.Amount",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.RDTIS.438034"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "RDTIS32", Value = report.RDTIS32.GetValueOrDefault().ToString() });
    
                response.Add(processMessage);
            }
            #endregion // VR.ATO.RDTIS.438034
    
            #region VR.ATO.RDTIS.438035
    
            /*  VR.ATO.RDTIS.438035
            Monetary field, must be unsigned, maximum of 11 chars, no decimals
    
            Legacy Rule Format:
            IF [RDTIS33] <> NULL AND [RDTIS33] <> MONETARY(U,11,0)
               RETURN VALIDATION MESSAGE
            ENDIF
            
            [RDTIS33] = RDTIS:RP:bafpr3.02.08:Expense.ResearchAndDevelopment.AssociatesClaimed.Amount

            Technical Business Rule Format:
            NotMonetary(^RDTIS33, 'U', 11, 0)
    
            Data Elements:
    
            RP:^RDTIS33 = bafpr3.02.08:Expense.ResearchAndDevelopment.AssociatesClaimed.Amount
            */
            assertion = NotMonetary(report.RDTIS33, @"U", 11, 0);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.400011", Severity = ProcessMessageSeverity.Error,
                    Description = @"Field incorrect format. Exceeds maximum digits or decimals and/or contains illegal characters",
                    LongDescription = @"Field is in incorrect format for a monetary field. Field either exceeds maximum digits or decimals and/or contains illegal characters and/or for a signed field does not have the sign as the first character",
                    Location = "/xbrli:xbrl/bafpr3.02.08:Expense.ResearchAndDevelopment.AssociatesClaimed.Amount",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.RDTIS.438035"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "RDTIS33", Value = report.RDTIS33.GetValueOrDefault().ToString() });
    
                response.Add(processMessage);
            }
            #endregion // VR.ATO.RDTIS.438035
    
            #region VR.ATO.RDTIS.438036
    
            /*  VR.ATO.RDTIS.438036
            Monetary field, must be unsigned, maximum of 11 chars, no decimals
    
            Legacy Rule Format:
            IF [RDTIS34] <> NULL AND [RDTIS34] <> MONETARY(U,11,0)
               RETURN VALIDATION MESSAGE
            ENDIF
            
            [RDTIS34] = RDTIS:RP:bafpr3.02.08:Expense.ResearchAndDevelopment.AssociatesNotClaimed.Amount

            Technical Business Rule Format:
            NotMonetary(^RDTIS34, 'U', 11, 0)
    
            Data Elements:
    
            RP:^RDTIS34 = bafpr3.02.08:Expense.ResearchAndDevelopment.AssociatesNotClaimed.Amount
            */
            assertion = NotMonetary(report.RDTIS34, @"U", 11, 0);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.400011", Severity = ProcessMessageSeverity.Error,
                    Description = @"Field incorrect format. Exceeds maximum digits or decimals and/or contains illegal characters",
                    LongDescription = @"Field is in incorrect format for a monetary field. Field either exceeds maximum digits or decimals and/or contains illegal characters and/or for a signed field does not have the sign as the first character",
                    Location = "/xbrli:xbrl/bafpr3.02.08:Expense.ResearchAndDevelopment.AssociatesNotClaimed.Amount",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.RDTIS.438036"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "RDTIS34", Value = report.RDTIS34.GetValueOrDefault().ToString() });
    
                response.Add(processMessage);
            }
            #endregion // VR.ATO.RDTIS.438036
    
            #region VR.ATO.RDTIS.438037
    
            /*  VR.ATO.RDTIS.438037
            R&D expenditure paid to associates in the current year does not match the sum of Australian and Foreign owned R&D expenditure paid to associates in the current year
    
            Legacy Rule Format:
            IF [RDTIS34] <> NULL AND [RDTIS34] <> ([RDTIS14] + [RDTIS15])
               RETURN VALIDATION MESSAGE
            ENDIF
            
            [RDTIS14] = RDTIS:RP.CTAUS:bafpr3.02.08:Expense.ResearchAndDevelopment.AssociatesExpenditure.Amount
            [RDTIS15] = RDTIS:RP.CTFOR:bafpr3.02.08:Expense.ResearchAndDevelopment.AssociatesExpenditure.Amount
            [RDTIS34] = RDTIS:RP:bafpr3.02.08:Expense.ResearchAndDevelopment.AssociatesNotClaimed.Amount

            Technical Business Rule Format:
            ^RDTIS34 <> NULL AND ^RDTIS34 <> (^RDTIS14 + ^RDTIS15)
    
            Data Elements:
    
            RP:^RDTIS34 = bafpr3.02.08:Expense.ResearchAndDevelopment.AssociatesNotClaimed.Amount
    
            RP.CTAUS:^RDTIS14 = bafpr3.02.08:Expense.ResearchAndDevelopment.AssociatesExpenditure.Amount
    
            RP.CTFOR:^RDTIS15 = bafpr3.02.08:Expense.ResearchAndDevelopment.AssociatesExpenditure.Amount
            */
            assertion = (report.RDTIS34 != null && report.RDTIS34.GetValueOrDefault() != report.RDTIS14.GetValueOrDefault() + report.RDTIS15.GetValueOrDefault());
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.RDTIS.438037", Severity = ProcessMessageSeverity.Error,
                    Description = @"R&D expenditure paid to associates is incorrect",
                    LongDescription = @"R&D expenditure paid to associates in the current year must equal the sum of Australian and Foreign owned R&D expenditure paid to associates in the current year",
                    Location = "/xbrli:xbrl/bafpr3.02.08:Expense.ResearchAndDevelopment.AssociatesNotClaimed.Amount",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.RDTIS.438037"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "RDTIS34", Value = report.RDTIS34.GetValueOrDefault().ToString() });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "RDTIS14", Value = report.RDTIS14.GetValueOrDefault().ToString() });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "RDTIS15", Value = report.RDTIS15.GetValueOrDefault().ToString() });
    
                response.Add(processMessage);
            }
            #endregion // VR.ATO.RDTIS.438037
    
            #region VR.ATO.RDTIS.438038
    
            /*  VR.ATO.RDTIS.438038
            Monetary field, must be unsigned, maximum of 11 chars, no decimals
    
            Legacy Rule Format:
            IF [RDTIS35] <> NULL AND [RDTIS35] <> MONETARY(U,11,0)
               RETURN VALIDATION MESSAGE
            ENDIF
            
            [RDTIS35] = RDTIS:RP:bafpr3.02.08:Expense.ResearchAndDevelopment.AssociatesCarriedForward.Amount

            Technical Business Rule Format:
            NotMonetary(^RDTIS35, 'U', 11, 0)
    
            Data Elements:
    
            RP:^RDTIS35 = bafpr3.02.08:Expense.ResearchAndDevelopment.AssociatesCarriedForward.Amount
            */
            assertion = NotMonetary(report.RDTIS35, @"U", 11, 0);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.400011", Severity = ProcessMessageSeverity.Error,
                    Description = @"Field incorrect format. Exceeds maximum digits or decimals and/or contains illegal characters",
                    LongDescription = @"Field is in incorrect format for a monetary field. Field either exceeds maximum digits or decimals and/or contains illegal characters and/or for a signed field does not have the sign as the first character",
                    Location = "/xbrli:xbrl/bafpr3.02.08:Expense.ResearchAndDevelopment.AssociatesCarriedForward.Amount[@contextRef='" + report.RPId + "']",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.RDTIS.438038"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "RDTIS35", Value = report.RDTIS35.GetValueOrDefault().ToString() });
    
                response.Add(processMessage);
            }
            #endregion // VR.ATO.RDTIS.438038
    
            #region VR.ATO.RDTIS.438039
    
            /*  VR.ATO.RDTIS.438039
            "R&D expenditure incurred to associates to be carried forward" must equal "R&D expenditure to associates incurred in prior year, not paid, not claimed (carried forward)" plus "Current year R&D expenditure incurred to associates" minus "Current year R&D expenditure incurred to associates claimed under other provisions" minus "R&D expenditure paid to associates in the current year".
    
            Legacy Rule Format:
            IF [RDTIS35] <> NULL AND [RDTIS35] <> ([RDTIS51]+[RDTIS32] - [RDTIS33] - [RDTIS34])
               RETURN VALIDATION MESSAGE
            ENDIF
            
            [RDTIS32] = RDTIS:RP:bafpr3.02.08:Expense.ResearchAndDevelopment.AssociatesUnpaid.Amount
            [RDTIS33] = RDTIS:RP:bafpr3.02.08:Expense.ResearchAndDevelopment.AssociatesClaimed.Amount
            [RDTIS34] = RDTIS:RP:bafpr3.02.08:Expense.ResearchAndDevelopment.AssociatesNotClaimed.Amount
            [RDTIS35] = RDTIS:RP:bafpr3.02.08:Expense.ResearchAndDevelopment.AssociatesCarriedForward.Amount
            [RDTIS51] = RDTIS:RP.YPrevious:bafpr3.02.08:Expense.ResearchAndDevelopment.AssociatesCarriedForward.Amount

            Technical Business Rule Format:
            ^RDTIS35 <> NULL AND ^RDTIS35 <> (^RDTIS51+^RDTIS32 - ^RDTIS33 - ^RDTIS34)
    
            Data Elements:
    
            RP:^RDTIS35 = bafpr3.02.08:Expense.ResearchAndDevelopment.AssociatesCarriedForward.Amount
    
            RP:^RDTIS32 = bafpr3.02.08:Expense.ResearchAndDevelopment.AssociatesUnpaid.Amount
    
            RP:^RDTIS33 = bafpr3.02.08:Expense.ResearchAndDevelopment.AssociatesClaimed.Amount
    
            RP:^RDTIS34 = bafpr3.02.08:Expense.ResearchAndDevelopment.AssociatesNotClaimed.Amount
    
            RP.YPrevious:^RDTIS51 = bafpr3.02.08:Expense.ResearchAndDevelopment.AssociatesCarriedForward.Amount
            */
            assertion = (report.RDTIS35 != null && report.RDTIS35.GetValueOrDefault() != report.RDTIS51.GetValueOrDefault() + report.RDTIS32.GetValueOrDefault() - report.RDTIS33.GetValueOrDefault() - report.RDTIS34.GetValueOrDefault());
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.RDTIS.438061", Severity = ProcessMessageSeverity.Error,
                    Description = @"R&D expenditure incurred to associates to be carried forward is incorrect.",
                    LongDescription = @"""R&D expenditure incurred to associates to be carried forward"" must equal ""R&D expenditure to associates incurred in prior year, not paid, not claimed (carried forward)"" plus ""Current year R&D expenditure incurred to associates"" minus ""Current year R&D expenditure incurred to associates claimed under other provisions"" minus ""R&D expenditure paid to associates in the current year"".",
                    Location = "/xbrli:xbrl/bafpr3.02.08:Expense.ResearchAndDevelopment.AssociatesCarriedForward.Amount[@contextRef='" + report.RPId + "']",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.RDTIS.438039"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "RDTIS35", Value = report.RDTIS35.GetValueOrDefault().ToString() });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "RDTIS51", Value = report.RDTIS51.GetValueOrDefault().ToString() });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "RDTIS32", Value = report.RDTIS32.GetValueOrDefault().ToString() });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "RDTIS33", Value = report.RDTIS33.GetValueOrDefault().ToString() });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "RDTIS34", Value = report.RDTIS34.GetValueOrDefault().ToString() });
    
                response.Add(processMessage);
            }
            #endregion // VR.ATO.RDTIS.438039
    
            #region VR.ATO.RDTIS.438040
    
            /*  VR.ATO.RDTIS.438040
            Mandatory field not supplied
    
            Legacy Rule Format:
            IF [RDTIS36] = NULL
               RETURN VALIDATION MESSAGE
            ENDIF
            
            [RDTIS36] = RDTIS:RP:gfagc.02.04:TaxConcession.ExemptEntityOwnership.Indicator

            Technical Business Rule Format:
            ^RDTIS36 = NULL
    
            Data Elements:
    
            RP:^RDTIS36 = gfagc.02.04:TaxConcession.ExemptEntityOwnership.Indicator
            */
            assertion = (report.RDTIS36 == null);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.001001", Severity = ProcessMessageSeverity.Error,
                    Description = @"Mandatory field not supplied",
                    Location = "/xbrli:xbrl/gfagc.02.04:TaxConcession.ExemptEntityOwnership.Indicator",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.RDTIS.438040"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "RDTIS36", Value = GetValueOrEmpty(report.RDTIS36) });
    
                response.Add(processMessage);
            }
            #endregion // VR.ATO.RDTIS.438040
    
            #region VR.ATO.RDTIS.438042
    
            /*  VR.ATO.RDTIS.438042
            Monetary field, must be unsigned, maximum of 11 chars, no decimals
    
            Legacy Rule Format:
            IF gfagc.xx.xx:TaxConcession.ResearchAndDevelopment.TaxOffsetEntityTurnover.Amount <> NULL AND gfagc.xx.xx:TaxConcession.ResearchAndDevelopment.TaxOffsetEntityTurnover.Amount <> MONETARY(U,11,0)
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            NotMonetary(^RDTIS38, 'U', 11, 0)
    
            Data Elements:
    
            RP:^RDTIS38 = gfagc.02.04:TaxConcession.ResearchAndDevelopment.TaxOffsetEntityTurnover.Amount
            */
            assertion = NotMonetary(report.RDTIS38, @"U", 11, 0);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.400011", Severity = ProcessMessageSeverity.Error,
                    Description = @"Field incorrect format. Exceeds maximum digits or decimals and/or contains illegal characters",
                    LongDescription = @"Field is in incorrect format for a monetary field. Field either exceeds maximum digits or decimals and/or contains illegal characters and/or for a signed field does not have the sign as the first character",
                    Location = "/xbrli:xbrl/gfagc.02.04:TaxConcession.ResearchAndDevelopment.TaxOffsetEntityTurnover.Amount[@contextRef='" + report.RPId + "']",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.RDTIS.438042"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "RDTIS38", Value = report.RDTIS38.GetValueOrDefault().ToString() });
    
                response.Add(processMessage);
            }
            #endregion // VR.ATO.RDTIS.438042
    
            #region VR.ATO.RDTIS.438043
    
            /*  VR.ATO.RDTIS.438043
            One or more Group Member Turnover is present and the reporting Company turnover is not present
    
            Legacy Rule Format:
            IF (ANY OCCURRENCE OF ([RDTIS42]) <> NULL) AND [RDTIS38] = NULL
               RETURN VALIDATION MESSAGE
            ENDIF
            
            [RDTIS38] = RDTIS:RP:gfagc.02.04:TaxConcession.ResearchAndDevelopment.TaxOffsetEntityTurnover.Amount
            [RDTIS42] = RDTIS:GM:gfagc.02.04:TaxConcession.ResearchAndDevelopment.TaxOffsetEntityTurnover.Amount

            Technical Business Rule Format:
            (AnyOccurrence(^RDTIS42, ^RDTIS42 <> NULL)) AND ^RDTIS38 = NULL
    
            Data Elements:
    
            RP:^RDTIS38 = gfagc.02.04:TaxConcession.ResearchAndDevelopment.TaxOffsetEntityTurnover.Amount
    
            GM:^RDTIS42 = gfagc.02.04:TaxConcession.ResearchAndDevelopment.TaxOffsetEntityTurnover.Amount
            */
            assertion = ((report.GMContextCollection == null ? false : report.GMContextCollection.Any(RDTIS42Repeat => RDTIS42Repeat.RDTIS42 != null)) && report.RDTIS38 == null);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.RDTIS.438043", Severity = ProcessMessageSeverity.Error,
                    Description = @"With one or more Group Members present, the reporting Company turnover must be present",
                    Location = "/xbrli:xbrl/gfagc.02.04:TaxConcession.ResearchAndDevelopment.TaxOffsetEntityTurnover.Amount",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.RDTIS.438043"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "RDTIS42", Value = "RDTIS42" });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "RDTIS38", Value = report.RDTIS38.GetValueOrDefault().ToString() });
    
                response.Add(processMessage);
            }
            #endregion // VR.ATO.RDTIS.438043
    
            #region VR.ATO.RDTIS.438047
    
            /*  VR.ATO.RDTIS.438047
            Monetary field, must be unsigned, maximum of 11 chars, no decimals
    
            Legacy Rule Format:
            IF [RDTIS43] <> NULL AND [RDTIS43] <> MONETARY(U,11,0)
               RETURN VALIDATION MESSAGE
            ENDIF
            
            [RDTIS43] = RDTIS:RP:gfagc.02.05:TaxConcession.ResearchAndDevelopment.TaxOffsetTurnoverExclusion.Amount

            Technical Business Rule Format:
            NotMonetary(^RDTIS43, 'U', 11, 0)
    
            Data Elements:
    
            RP:^RDTIS43 = gfagc.02.05:TaxConcession.ResearchAndDevelopment.TaxOffsetTurnoverExclusion.Amount
            */
            assertion = NotMonetary(report.RDTIS43, @"U", 11, 0);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.400011", Severity = ProcessMessageSeverity.Error,
                    Description = @"Field incorrect format. Exceeds maximum digits or decimals and/or contains illegal characters",
                    LongDescription = @"Field is in incorrect format for a monetary field. Field either exceeds maximum digits or decimals and/or contains illegal characters and/or for a signed field does not have the sign as the first character",
                    Location = "/xbrli:xbrl/gfagc.02.05:TaxConcession.ResearchAndDevelopment.TaxOffsetTurnoverExclusion.Amount",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.RDTIS.438047"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "RDTIS43", Value = report.RDTIS43.GetValueOrDefault().ToString() });
    
                response.Add(processMessage);
            }
            #endregion // VR.ATO.RDTIS.438047
    
            #region VR.ATO.RDTIS.438048
    
            /*  VR.ATO.RDTIS.438048
            Monetary field, must be unsigned, maximum of 11 chars, no decimals
    
            Legacy Rule Format:
            IF [RDTIS44] <> NULL AND [RDTIS44] <> MONETARY(U,11,0)
               RETURN VALIDATION MESSAGE
            ENDIF
            
            [RDTIS44] = RDTIS:RP:gfagc.02.04:TaxConcession.ResearchAndDevelopment.TaxOffsetTurnoverTotal.Amount

            Technical Business Rule Format:
            NotMonetary(^RDTIS44, 'U', 11, 0)
    
            Data Elements:
    
            RP:^RDTIS44 = gfagc.02.04:TaxConcession.ResearchAndDevelopment.TaxOffsetTurnoverTotal.Amount
            */
            assertion = NotMonetary(report.RDTIS44, @"U", 11, 0);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.400011", Severity = ProcessMessageSeverity.Error,
                    Description = @"Field incorrect format. Exceeds maximum digits or decimals and/or contains illegal characters",
                    LongDescription = @"Field is in incorrect format for a monetary field. Field either exceeds maximum digits or decimals and/or contains illegal characters and/or for a signed field does not have the sign as the first character",
                    Location = "/xbrli:xbrl/gfagc.02.04:TaxConcession.ResearchAndDevelopment.TaxOffsetTurnoverTotal.Amount",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.RDTIS.438048"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "RDTIS44", Value = report.RDTIS44.GetValueOrDefault().ToString() });
    
                response.Add(processMessage);
            }
            #endregion // VR.ATO.RDTIS.438048
    
            #region VR.ATO.RDTIS.438049
    
            /*  VR.ATO.RDTIS.438049
            Aggregated Turnover of $20 million or greater' question is 'False' and the Aggregated Turnover is not present or present but is greater than or equal to $20 million
    
            Legacy Rule Format:
            IF [RDTIS37] = FALSE AND ([RDTIS44] = NULL OR [RDTIS44] >=20000000)
               RETURN VALIDATION MESSAGE
            ENDIF
            
            [RDTIS37] = RDTIS:RP:gfagc.02.04:TaxConcession.AggregatedTurnoverAtOrAboveThreshold.Indicator
            [RDTIS44] = RDTIS:RP:gfagc.02.04:TaxConcession.ResearchAndDevelopment.TaxOffsetTurnoverTotal.Amount

            Technical Business Rule Format:
            ^RDTIS37 = FALSE AND (^RDTIS44 = NULL OR ^RDTIS44 >=20000000)
    
            Data Elements:
    
            RP:^RDTIS44 = gfagc.02.04:TaxConcession.ResearchAndDevelopment.TaxOffsetTurnoverTotal.Amount
    
            RP:^RDTIS37 = gfagc.02.04:TaxConcession.AggregatedTurnoverAtOrAboveThreshold.Indicator
            */
            assertion = (report.RDTIS37 == false && (report.RDTIS44 == null || report.RDTIS44.GetValueOrDefault() >= 20000000));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.RDTIS.438049", Severity = ProcessMessageSeverity.Error,
                    Description = @"The indicated Aggregated Turnover is incorrect",
                    LongDescription = @"With an indicated Aggregated Turnover of less than $20 million, the Aggregated Turnover must be present and less than $20 million",
                    Location = "/xbrli:xbrl/gfagc.02.04:TaxConcession.ResearchAndDevelopment.TaxOffsetTurnoverTotal.Amount",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.RDTIS.438049"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "RDTIS37", Value = GetValueOrEmpty(report.RDTIS37) });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "RDTIS44", Value = report.RDTIS44.GetValueOrDefault().ToString() });
    
                response.Add(processMessage);
            }
            #endregion // VR.ATO.RDTIS.438049
    
            #region VR.ATO.RDTIS.438050
    
            /*  VR.ATO.RDTIS.438050
            Aggregated Turnover is not equal to the reporting Company turnover plus the sum of the Group Member turnovers minus amounts excluded from aggregated turnover
    
            Legacy Rule Format:
            IF [RDTIS44] <> NULL AND [RDTIS44] <> ([RDTIS38] + SUM([RDTIS42]) + [RDTIS52] - [RDTIS43])
               RETURN VALIDATION MESSAGE
            ENDIF
            
            [RDTIS38] = RDTIS:RP:gfagc.02.04:TaxConcession.ResearchAndDevelopment.TaxOffsetEntityTurnover.Amount
            [RDTIS42] = RDTIS:GM:gfagc.02.04:TaxConcession.ResearchAndDevelopment.TaxOffsetEntityTurnover.Amount
            [RDTIS43] = RDTIS:RP:gfagc.02.05:TaxConcession.ResearchAndDevelopment.TaxOffsetTurnoverExclusion.Amount
            [RDTIS44] = RDTIS:RP:gfagc.02.04:TaxConcession.ResearchAndDevelopment.TaxOffsetTurnoverTotal.Amount
            [RDTIS52] = RDTIS:RP:gfagc.02.07:TaxConcession.ResearchAndDevelopment.TaxOffsetAdditionalEntitiesTurnover.Amount

            Technical Business Rule Format:
            ^RDTIS44 <> NULL AND ^RDTIS44 <> (^RDTIS38 + Sum(^RDTIS42) + ^RDTIS52 - ^RDTIS43)
    
            Data Elements:
    
            RP:^RDTIS44 = gfagc.02.04:TaxConcession.ResearchAndDevelopment.TaxOffsetTurnoverTotal.Amount
    
            RP:^RDTIS38 = gfagc.02.04:TaxConcession.ResearchAndDevelopment.TaxOffsetEntityTurnover.Amount
    
            GM:^RDTIS42 = gfagc.02.04:TaxConcession.ResearchAndDevelopment.TaxOffsetEntityTurnover.Amount
    
            RP:^RDTIS43 = gfagc.02.05:TaxConcession.ResearchAndDevelopment.TaxOffsetTurnoverExclusion.Amount
    
            RP:^RDTIS52 = gfagc.02.07:TaxConcession.ResearchAndDevelopment.TaxOffsetAdditionalEntitiesTurnover.Amount
            */
            assertion = (report.RDTIS44 != null && report.RDTIS44.GetValueOrDefault() != report.RDTIS38.GetValueOrDefault() + (report.GMContextCollection == null ? 0 : report.GMContextCollection.Sum(f => f.RDTIS42.GetValueOrDefault())) + report.RDTIS52.GetValueOrDefault() - report.RDTIS43.GetValueOrDefault());
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.RDTIS.438050", Severity = ProcessMessageSeverity.Error,
                    Description = @"Aggregated Turnover is incorrect",
                    LongDescription = @"Aggregated Turnover must equal the reporting Company turnover plus the sum of Group Member turnovers minus amounts excluded from aggregated turnover",
                    Location = "/xbrli:xbrl/gfagc.02.04:TaxConcession.ResearchAndDevelopment.TaxOffsetTurnoverTotal.Amount",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.RDTIS.438050"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "RDTIS44", Value = report.RDTIS44.GetValueOrDefault().ToString() });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "RDTIS38", Value = report.RDTIS38.GetValueOrDefault().ToString() });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "RDTIS42", Value = "RDTIS42" });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "RDTIS52", Value = report.RDTIS52.GetValueOrDefault().ToString() });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "RDTIS43", Value = report.RDTIS43.GetValueOrDefault().ToString() });
    
                response.Add(processMessage);
            }
            #endregion // VR.ATO.RDTIS.438050
    
            #region VR.ATO.RDTIS.438051
    
            /*  VR.ATO.RDTIS.438051
            Monetary field, must be unsigned, maximum of 11 chars, no decimals
    
            Legacy Rule Format:
            IF [RDTIS45] <> NULL AND [RDTIS45] <> MONETARY(U,11,0)
               RETURN VALIDATION MESSAGE
            ENDIF
            
            [RDTIS45] = RDTIS:RP:gfagc.02.04:TaxConcession.ResearchAndDevelopment.TaxOffsetNotionalRefundable.Amount

            Technical Business Rule Format:
            NotMonetary(^RDTIS45, 'U', 11, 0)
    
            Data Elements:
    
            RP:^RDTIS45 = gfagc.02.04:TaxConcession.ResearchAndDevelopment.TaxOffsetNotionalRefundable.Amount
            */
            assertion = NotMonetary(report.RDTIS45, @"U", 11, 0);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.400011", Severity = ProcessMessageSeverity.Error,
                    Description = @"Field incorrect format. Exceeds maximum digits or decimals and/or contains illegal characters",
                    LongDescription = @"Field is in incorrect format for a monetary field. Field either exceeds maximum digits or decimals and/or contains illegal characters and/or for a signed field does not have the sign as the first character",
                    Location = "/xbrli:xbrl/gfagc.02.04:TaxConcession.ResearchAndDevelopment.TaxOffsetNotionalRefundable.Amount",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.RDTIS.438051"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "RDTIS45", Value = report.RDTIS45.GetValueOrDefault().ToString() });
    
                response.Add(processMessage);
            }
            #endregion // VR.ATO.RDTIS.438051
    
            #region VR.ATO.RDTIS.438052
    
            /*  VR.ATO.RDTIS.438052
            The Refundable R&D tax offset is being claimed: 'Exempt entity ownership of 50% or greater' question is 'False', 'Aggregated turnover of $20 million or greater' question is 'False' and the Refundable tax offset Total notional R&D deductions is not equal to the Total notional R&D deductions
    
            Legacy Rule Format:
            IF [RDTIS36] = FALSE AND [RDTIS37] = FALSE AND [RDTIS45] <> [RDTIS24]
               RETURN VALIDATION MESSAGE
            ENDIF
            
            [RDTIS24] = RDTIS:RP:bafpr3.02.08:Expense.ResearchAndDevelopment.AllocatedTotal.Amount
            [RDTIS36] = RDTIS:RP:gfagc.02.04:TaxConcession.ExemptEntityOwnership.Indicator
            [RDTIS37] = RDTIS:RP:gfagc.02.04:TaxConcession.AggregatedTurnoverAtOrAboveThreshold.Indicator
            [RDTIS45] = RDTIS:RP:gfagc.02.04:TaxConcession.ResearchAndDevelopment.TaxOffsetNotionalRefundable.Amount

            Technical Business Rule Format:
            ^RDTIS36 = FALSE AND ^RDTIS37 = FALSE AND ^RDTIS45 <> ^RDTIS24
    
            Data Elements:
    
            RP:^RDTIS45 = gfagc.02.04:TaxConcession.ResearchAndDevelopment.TaxOffsetNotionalRefundable.Amount
    
            RP:^RDTIS24 = bafpr3.02.08:Expense.ResearchAndDevelopment.AllocatedTotal.Amount
    
            RP:^RDTIS36 = gfagc.02.04:TaxConcession.ExemptEntityOwnership.Indicator
    
            RP:^RDTIS37 = gfagc.02.04:TaxConcession.AggregatedTurnoverAtOrAboveThreshold.Indicator
            */
            assertion = (report.RDTIS36 == false && report.RDTIS37 == false && report.RDTIS45.GetValueOrDefault() != report.RDTIS24.GetValueOrDefault());
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.RDTIS.438052", Severity = ProcessMessageSeverity.Error,
                    Description = @"Refundable tax offset Total notional R&D deductions must equal the Total notional R&D deductions",
                    LongDescription = @"The Refundable R&D tax offset is being claimed: 'Exempt entity ownership of 50% or greater' question is 'No', 'Aggregated turnover of $20 million or greater' question is 'No' and the Refundable tax offset Total notional R&D deductions is not equal to the Total notional R&D deductions",
                    Location = "/xbrli:xbrl/gfagc.02.04:TaxConcession.ResearchAndDevelopment.TaxOffsetNotionalRefundable.Amount",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.RDTIS.438052"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "RDTIS36", Value = GetValueOrEmpty(report.RDTIS36) });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "RDTIS37", Value = GetValueOrEmpty(report.RDTIS37) });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "RDTIS45", Value = report.RDTIS45.GetValueOrDefault().ToString() });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "RDTIS24", Value = report.RDTIS24.GetValueOrDefault().ToString() });
    
                response.Add(processMessage);
            }
            #endregion // VR.ATO.RDTIS.438052
    
            #region VR.ATO.RDTIS.438053
    
            /*  VR.ATO.RDTIS.438053
            Refundable tax offset Total notional R&D deductions and Non-refundable tax offset Total notional R&D deductions are both present
    
            Legacy Rule Format:
            IF [RDTIS45] <> NULL AND [RDTIS47] <> NULL
               RETURN VALIDATION MESSAGE
            ENDIF
            
            [RDTIS45] = RDTIS:RP:gfagc.02.04:TaxConcession.ResearchAndDevelopment.TaxOffsetNotionalRefundable.Amount
            [RDTIS47] = RDTIS:RP:gfagc.02.04:TaxConcession.ResearchAndDevelopment.TaxOffsetNotionalNonRefundable.Amount

            Technical Business Rule Format:
            ^RDTIS45 <> NULL AND ^RDTIS47 <> NULL
    
            Data Elements:
    
            RP:^RDTIS45 = gfagc.02.04:TaxConcession.ResearchAndDevelopment.TaxOffsetNotionalRefundable.Amount
    
            RP:^RDTIS47 = gfagc.02.04:TaxConcession.ResearchAndDevelopment.TaxOffsetNotionalNonRefundable.Amount
            */
            assertion = (report.RDTIS45 != null && report.RDTIS47 != null);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.RDTIS.438053", Severity = ProcessMessageSeverity.Error,
                    Description = @"Only a Refundable or Non-refundable tax offset is claimable.",
                    LongDescription = @"Only a Refundable or Non-refunable tax offset Total notional R&D deductions should be present",
                    Location = "/xbrli:xbrl/gfagc.02.04:TaxConcession.ResearchAndDevelopment.TaxOffsetNotionalRefundable.Amount",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.RDTIS.438053"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "RDTIS45", Value = report.RDTIS45.GetValueOrDefault().ToString() });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "RDTIS47", Value = report.RDTIS47.GetValueOrDefault().ToString() });
    
                response.Add(processMessage);
            }
            #endregion // VR.ATO.RDTIS.438053
    
            #region VR.ATO.RDTIS.438054
    
            /*  VR.ATO.RDTIS.438054
            Monetary field, must be unsigned, maximum of 13 chars, may contain a decimal point and up to 2 decimals
    
            Legacy Rule Format:
            IF [RDTIS46] <> NULL AND [RDTIS46] <> MONETARY(U,13,2)
               RETURN VALIDATION MESSAGE
            ENDIF
            
            [RDTIS46] = RDTIS:RP:gfagc.02.04:TaxConcession.ResearchAndDevelopment.TaxOffsetRefundable.Amount

            Technical Business Rule Format:
            NotMonetary(^RDTIS46, 'U', 13, 2)
    
            Data Elements:
    
            RP:^RDTIS46 = gfagc.02.04:TaxConcession.ResearchAndDevelopment.TaxOffsetRefundable.Amount
            */
            assertion = NotMonetary(report.RDTIS46, @"U", 13, 2);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.400011", Severity = ProcessMessageSeverity.Error,
                    Description = @"Field incorrect format. Exceeds maximum digits or decimals and/or contains illegal characters",
                    LongDescription = @"Field is in incorrect format for a monetary field. Field either exceeds maximum digits or decimals and/or contains illegal characters and/or for a signed field does not have the sign as the first character",
                    Location = "/xbrli:xbrl/gfagc.02.04:TaxConcession.ResearchAndDevelopment.TaxOffsetRefundable.Amount",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.RDTIS.438054"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "RDTIS46", Value = report.RDTIS46.GetValueOrDefault().ToString() });
    
                response.Add(processMessage);
            }
            #endregion // VR.ATO.RDTIS.438054
    
            #region VR.ATO.RDTIS.438056
    
            /*  VR.ATO.RDTIS.438056
            Monetary field, must be unsigned, maximum of 11 chars, no decimals
    
            Legacy Rule Format:
            IF [RDTIS47] <> NULL AND [RDTIS47] <> MONETARY(U,11,0)
               RETURN VALIDATION MESSAGE
            ENDIF
            
            [RDTIS47] = RDTIS:RP:gfagc.02.04:TaxConcession.ResearchAndDevelopment.TaxOffsetNotionalNonRefundable.Amount

            Technical Business Rule Format:
            NotMonetary(^RDTIS47, 'U', 11, 0)
    
            Data Elements:
    
            RP:^RDTIS47 = gfagc.02.04:TaxConcession.ResearchAndDevelopment.TaxOffsetNotionalNonRefundable.Amount
            */
            assertion = NotMonetary(report.RDTIS47, @"U", 11, 0);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.400011", Severity = ProcessMessageSeverity.Error,
                    Description = @"Field incorrect format. Exceeds maximum digits or decimals and/or contains illegal characters",
                    LongDescription = @"Field is in incorrect format for a monetary field. Field either exceeds maximum digits or decimals and/or contains illegal characters and/or for a signed field does not have the sign as the first character",
                    Location = "/xbrli:xbrl/gfagc.02.04:TaxConcession.ResearchAndDevelopment.TaxOffsetNotionalNonRefundable.Amount",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.RDTIS.438056"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "RDTIS47", Value = report.RDTIS47.GetValueOrDefault().ToString() });
    
                response.Add(processMessage);
            }
            #endregion // VR.ATO.RDTIS.438056
    
            #region VR.ATO.RDTIS.438057
    
            /*  VR.ATO.RDTIS.438057
            The Non-refundable R&D tax offset is being claimed: Group Totals Turnover is $20 million or greater or 'Exempt entity ownership of 50% or greater' question is 'True' or 'Aggregated turnover of $20 million or greater' question is 'True' and the Non-refundable tax offset Total notional R&D deductions is not equal to the Total notional R&D deductions
    
            Legacy Rule Format:
            IF ([RDTIS36] = TRUE OR [RDTIS37] = TRUE) AND [RDTIS47] <> [RDTIS24]
               RETURN VALIDATION MESSAGE
            ENDIF
            
            [RDTIS24] = RDTIS:RP:bafpr3.02.08:Expense.ResearchAndDevelopment.AllocatedTotal.Amount
            [RDTIS36] = RDTIS:RP:gfagc.02.04:TaxConcession.ExemptEntityOwnership.Indicator
            [RDTIS37] = RDTIS:RP:gfagc.02.04:TaxConcession.AggregatedTurnoverAtOrAboveThreshold.Indicator
            [RDTIS47] = RDTIS:RP:gfagc.02.04:TaxConcession.ResearchAndDevelopment.TaxOffsetNotionalNonRefundable.Amount

            Technical Business Rule Format:
            (^RDTIS36 = TRUE OR ^RDTIS37 = TRUE) AND ^RDTIS47 <> ^RDTIS24
    
            Data Elements:
    
            RP:^RDTIS47 = gfagc.02.04:TaxConcession.ResearchAndDevelopment.TaxOffsetNotionalNonRefundable.Amount
    
            RP:^RDTIS24 = bafpr3.02.08:Expense.ResearchAndDevelopment.AllocatedTotal.Amount
    
            RP:^RDTIS36 = gfagc.02.04:TaxConcession.ExemptEntityOwnership.Indicator
    
            RP:^RDTIS37 = gfagc.02.04:TaxConcession.AggregatedTurnoverAtOrAboveThreshold.Indicator
            */
            assertion = ((report.RDTIS36 == true || report.RDTIS37 == true) && report.RDTIS47.GetValueOrDefault() != report.RDTIS24.GetValueOrDefault());
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.RDTIS.438057", Severity = ProcessMessageSeverity.Error,
                    Description = @"Non-refundable tax offset Total notional R&D deductions must equal the Total notional R&D deductions",
                    LongDescription = @"The Non-refundable R&D tax offset is being claimed: Group Totals Turnover is $20 million or greater or 'Exempt entity ownership of 50% or greater' question is 'Yes' or 'Aggregated turnover of $20 million or greater' question is 'Yes' and the Non-refundable tax offset Total notional R&D deductions is not equal to the Total notional R&D deductions",
                    Location = "/xbrli:xbrl/gfagc.02.04:TaxConcession.ResearchAndDevelopment.TaxOffsetNotionalNonRefundable.Amount",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.RDTIS.438057"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "RDTIS36", Value = GetValueOrEmpty(report.RDTIS36) });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "RDTIS37", Value = GetValueOrEmpty(report.RDTIS37) });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "RDTIS47", Value = report.RDTIS47.GetValueOrDefault().ToString() });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "RDTIS24", Value = report.RDTIS24.GetValueOrDefault().ToString() });
    
                response.Add(processMessage);
            }
            #endregion // VR.ATO.RDTIS.438057
    
            #region VR.ATO.RDTIS.438058
    
            /*  VR.ATO.RDTIS.438058
            Monetary field, must be unsigned, maximum of 13 chars, may contain a decimal point and up to 2 decimals
    
            Legacy Rule Format:
            IF [RDTIS48] <> NULL AND [RDTIS48] <> MONETARY(U,13,2)
               RETURN VALIDATION MESSAGE
            ENDIF
            
            [RDTIS48] = RDTIS:RP:gfagc.02.04:TaxConcession.ResearchAndDevelopment.TaxOffsetNonRefundable.Amount

            Technical Business Rule Format:
            NotMonetary(^RDTIS48, 'U', 13, 2)
    
            Data Elements:
    
            RP:^RDTIS48 = gfagc.02.04:TaxConcession.ResearchAndDevelopment.TaxOffsetNonRefundable.Amount
            */
            assertion = NotMonetary(report.RDTIS48, @"U", 13, 2);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.400011", Severity = ProcessMessageSeverity.Error,
                    Description = @"Field incorrect format. Exceeds maximum digits or decimals and/or contains illegal characters",
                    LongDescription = @"Field is in incorrect format for a monetary field. Field either exceeds maximum digits or decimals and/or contains illegal characters and/or for a signed field does not have the sign as the first character",
                    Location = "/xbrli:xbrl/gfagc.02.04:TaxConcession.ResearchAndDevelopment.TaxOffsetNonRefundable.Amount",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.RDTIS.438058"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "RDTIS48", Value = report.RDTIS48.GetValueOrDefault().ToString() });
    
                response.Add(processMessage);
            }
            #endregion // VR.ATO.RDTIS.438058
    
            #region VR.ATO.RDTIS.438060
    
            /*  VR.ATO.RDTIS.438060
            Group Member context TFN is duplicated
    
            Legacy Rule Format:
            IF GM:entity.identifier.TFN = ANY OTHER OCCURRENCE(GM:entity.identifier.TFN)
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            HasDuplicateValues(^TFN)
    
            Data Elements:
    
            ^TFN = GM
            */
            assertion = HasDuplicateValues(report.GMContextCollection == null ? null : report.GMContextCollection.Select(f => f.IdentifierTFN));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.RDTIS.438060", Severity = ProcessMessageSeverity.Error,
                    Description = @"Group Member context TFN duplicated. Unique TFN required",
                    Location = "/xbrli:xbrl/xbrli:context[@id='" + report.GMContextCollection[DuplicateValueIndex(report.GMContextCollection.Select(f => f.IdentifierTFN))].Id + "']" + "/xbrli:entity/xbrli:identifier",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.RDTIS.438060"} },
                };
                response.Add(processMessage);
            }
            #endregion // VR.ATO.RDTIS.438060
    
            #region VR.ATO.RDTIS.438061
    
            /*  VR.ATO.RDTIS.438061
            Monetary field, must be unsigned, maximum of 11 chars, no decimals
    
            Legacy Rule Format:
            IF [RDTIS51] <> NULL AND [RDTIS51] <> MONETARY(U,11,0)
               RETURN VALIDATION MESSAGE
            ENDIF
            
            [RDTIS51] = RDTIS:RP.YPrevious:bafpr3.02.08:Expense.ResearchAndDevelopment.AssociatesCarriedForward.Amount

            Technical Business Rule Format:
            NotMonetary(^RDTIS51, 'U', 11, 0)
    
            Data Elements:
    
            RP.YPrevious:^RDTIS51 = bafpr3.02.08:Expense.ResearchAndDevelopment.AssociatesCarriedForward.Amount
            */
            assertion = NotMonetary(report.RDTIS51, @"U", 11, 0);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.400011", Severity = ProcessMessageSeverity.Error,
                    Description = @"Field incorrect format. Exceeds maximum digits or decimals and/or contains illegal characters",
                    LongDescription = @"Field is in incorrect format for a monetary field. Field either exceeds maximum digits or decimals and/or contains illegal characters and/or for a signed field does not have the sign as the first character",
                    Location = "/xbrli:xbrl/bafpr3.02.08:Expense.ResearchAndDevelopment.AssociatesCarriedForward.Amount[@contextRef='" + report.RPYPreviousId + "']",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.RDTIS.438061"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "RDTIS51", Value = report.RDTIS51.GetValueOrDefault().ToString() });
    
                response.Add(processMessage);
            }
            #endregion // VR.ATO.RDTIS.438061
    
            #region VR.ATO.RDTIS.438062
    
            /*  VR.ATO.RDTIS.438062
            The number of Reporting party: YPrevious contexts must not exceed 1
    
            Legacy Rule Format:
            IF COUNT(CONTEXT(RP.YPrevious)) > 1
            RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            Count(^Context) > 1
    
            Data Elements:
    
            ^Context = RP:RP.YPrevious
            */
            assertion = (Count(report.RPYPreviousCount) > 1);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.RDTIS.438062", Severity = ProcessMessageSeverity.Error,
                    Description = @"The number of Reporting party: YPrevious contexts must not exceed 1",
                    Location = "/xbrli:xbrl/xbrli:context" + OccurrenceIndex(report.RPYPreviousLastOccurrenceIndex),
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.RDTIS.438062"} },
                };
                response.Add(processMessage);
            }
            #endregion // VR.ATO.RDTIS.438062
    
            #region VR.ATO.RDTIS.438063
    
            /*  VR.ATO.RDTIS.438063
            The Reporting Party: YPrevious TFN does not match the Reporting Company TFN
    
            Legacy Rule Format:
            IF RP.YPrevious:entity.identifier.TFN <> RP:entity.identifier.TFN
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            ^TFN2 <> ^TFN
    
            Data Elements:
    
            ^TFN2 = RP:RP.YPrevious
    
            ^TFN = RP
            */
            assertion = ((report.RPYPreviousExists == true) && (report.RPYPreviousIdentifierTFN != report.RPIdentifierTFN));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.RDTIS.438063", Severity = ProcessMessageSeverity.Error,
                    Description = @"The Reporting Party: YPrevious TFN does not match the Reporting Company TFN",
                    Location = "/xbrli:xbrl/xbrli:context" + OccurrenceIndex(report.RPYPreviousOccurrenceIndex) + "/xbrli:entity/xbrli:identifier",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.RDTIS.438063"} },
                };
                response.Add(processMessage);
            }
            #endregion // VR.ATO.RDTIS.438063
    
            #region VR.ATO.RDTIS.438064
    
            /*  VR.ATO.RDTIS.438064
            Monetary field, must be unsigned, maximum of 11 chars, no decimals
    
            Legacy Rule Format:
            IF [RDTIS52] <> NULL AND [RDTIS52] <> MONETARY(U,11,0)
               RETURN VALIDATION MESSAGE
            ENDIF
            
            [RDTIS52] = RDTIS:RP:gfagc.02.07:TaxConcession.ResearchAndDevelopment.TaxOffsetAdditionalEntitiesTurnover.Amount

            Technical Business Rule Format:
            NotMonetary(^RDTIS52, 'U', 11, 0)
    
            Data Elements:
    
            RP:^RDTIS52 = gfagc.02.07:TaxConcession.ResearchAndDevelopment.TaxOffsetAdditionalEntitiesTurnover.Amount
            */
            assertion = NotMonetary(report.RDTIS52, @"U", 11, 0);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.400011", Severity = ProcessMessageSeverity.Error,
                    Description = @"Field incorrect format. Exceeds maximum digits or decimals and/or contains illegal characters",
                    LongDescription = @"Field is in incorrect format for a monetary field. Field either exceeds maximum digits or decimals and/or contains illegal characters and/or for a signed field does not have the sign as the first character",
                    Location = "/xbrli:xbrl/gfagc.02.07:TaxConcession.ResearchAndDevelopment.TaxOffsetAdditionalEntitiesTurnover.Amount",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.RDTIS.438064"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "RDTIS52", Value = report.RDTIS52.GetValueOrDefault().ToString() });
    
                response.Add(processMessage);
            }
            #endregion // VR.ATO.RDTIS.438064
    
            #region VR.ATO.RDTIS.438065
    
            /*  VR.ATO.RDTIS.438065
            Additional Group Members - Annual Turnover must not be completed if there are less then 19 specified group members
    
            Legacy Rule Format:
            IF [RDTIS52] <> NULL AND COUNT(CONTEXT(GM)) < 19 
              RETURN VALIDATION MESSAGE
            ENDIF
            
            [RDTIS52] = RDTIS:RP:gfagc.02.07:TaxConcession.ResearchAndDevelopment.TaxOffsetAdditionalEntitiesTurnover.Amount

            Technical Business Rule Format:
            ^RDTIS52 <> NULL AND Count(^Context) < 19
    
            Data Elements:
    
            RP:^RDTIS52 = gfagc.02.07:TaxConcession.ResearchAndDevelopment.TaxOffsetAdditionalEntitiesTurnover.Amount
    
            RP:^Context = GM
            */
            assertion = (report.RDTIS52 != null && Count(report.GMContextCollection) < 19);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.RDTIS.438065", Severity = ProcessMessageSeverity.Error,
                    Description = @"Additional Group Members - Annual Turnover must not be completed",
                    LongDescription = @"Additional Group Members - Annual Turnover must not be completed if there are less then 19 specified group members",
                    Location = "/xbrli:xbrl/gfagc.02.07:TaxConcession.ResearchAndDevelopment.TaxOffsetAdditionalEntitiesTurnover.Amount",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.RDTIS.438065"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "RDTIS52", Value = report.RDTIS52.GetValueOrDefault().ToString() });
    
                response.Add(processMessage);
            }
            #endregion // VR.ATO.RDTIS.438065
    
            #region VR.ATO.RDTIS.438066
    
            /*  VR.ATO.RDTIS.438066
            Aggregated turnover of $20 million or greater question is required if the answer to Expenditure incurred but not claimed (carried forward) question is answered with a ‘NO’
    
            Legacy Rule Format:
            IF [RDTIS36] = FALSE AND [RDTIS37] = NULL
               RETURN VALIDATION MESSAGE
            ENDIF
            
            [RDTIS36] = RDTIS:RP:gfagc.02.04:TaxConcession.ExemptEntityOwnership.Indicator
            [RDTIS37] = RDTIS:RP:gfagc.02.04:TaxConcession.AggregatedTurnoverAtOrAboveThreshold.Indicator

            Technical Business Rule Format:
            ^RDTIS36 = FALSE AND ^RDTIS37 = NULL
    
            Data Elements:
    
            RP:^RDTIS37 = gfagc.02.04:TaxConcession.AggregatedTurnoverAtOrAboveThreshold.Indicator
    
            RP:^RDTIS36 = gfagc.02.04:TaxConcession.ExemptEntityOwnership.Indicator
            */
            assertion = (report.RDTIS36 == false && report.RDTIS37 == null);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.RDTIS.438066", Severity = ProcessMessageSeverity.Error,
                    Description = @"Answer to 'Do you have an aggregated turnover of $20 million or greater?' is required",
                    LongDescription = @"If your company is not controlled by one or more income tax exempt entities, then you must indicate whether the company's aggreated turnover is $20 million or greater",
                    Location = "/xbrli:xbrl/gfagc.02.04:TaxConcession.AggregatedTurnoverAtOrAboveThreshold.Indicator",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.RDTIS.438066"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "RDTIS36", Value = GetValueOrEmpty(report.RDTIS36) });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "RDTIS37", Value = GetValueOrEmpty(report.RDTIS37) });
    
                response.Add(processMessage);
            }
            #endregion // VR.ATO.RDTIS.438066

            #region Repeating GMContext
            if (report.GMContextCollection != null)
            {
                int itemIndexContext1 = 0;
                foreach (RDTIS2013.GMContext gmContext in report.GMContextCollection)
                {
                    itemIndexContext1++;
            
                    #region VR.ATO.GEN.410038
            
                    /*  VR.ATO.GEN.410038
                    Non-individual name field cannot contain 'Pship', 'P'ship or 'P/Ship'
    
                    Legacy Rule Format:
                    IF (pyde.xx.xx:OrganisationNameDetails.OrganisationalName.Text STARTSWITH "T/A ") AND (pyde.xx.xx:OrganisationNameDetails.OrganisationalName.Text ENDSWITH SET(" Pship"," P'ship"," P/ship"))
                      RETURN VALIDATION MESSAGE
                    ENDIF
        
                    Technical Business Rule Format:
                    (StartsWith(^RDTIS41, 'T/A ')) AND (EndsWithSet(^RDTIS41, '" Pship"," P'ship"," P/ship"'))
            
                    Data Elements:
            
                    GM:^RDTIS41 = pyde.02.00:OrganisationNameDetails.OrganisationalName.Text
                    */
                    assertion = (IsMatch(gmContext.RDTIS41, @"^(T/A )",RegexOptions.IgnoreCase) && IsMatch(gmContext.RDTIS41, @"( Pship| P'ship| P/ship)$",RegexOptions.IgnoreCase));
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.GEN.410038", Severity = ProcessMessageSeverity.Error,
                            Description = @"Non-individual name field cannot start with 'T/A' and end with 'Pship', 'P'ship or 'P/Ship'",
                            Location = "/xbrli:xbrl/orgname2.02.00:OrganisationNameDetails/pyde.02.00:OrganisationNameDetails.OrganisationalName.Text",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.410038"} },
                        };
                        processMessage.Parameters.Add(new ProcessMessageParameter
                            { Name = "RDTIS41", Value = gmContext.RDTIS41 });
            
                        response.Add(processMessage);
                    }
                    #endregion // VR.ATO.GEN.410038
            
                    #region VR.ATO.GEN.410039
            
                    /*  VR.ATO.GEN.410039
                    Organisation name must contain at least one alpha or numeric character
    
                    Legacy Rule Format:
                    IF (pyde.xx.xx:OrganisationNameDetails.OrganisationalName.Text  <> NULLORBLANK) AND (pyde.xx.xx:OrganisationNameDetails.OrganisationalName.Text  DOES NOT CONTAIN SET("a-z", "A-Z", "0-9"))
                       RETURN VALIDATION MESSAGE
                    ENDIF
        
                    Technical Business Rule Format:
                    (^RDTIS41  <> BLANK) AND (NotContainsSet(^RDTIS41, '"a-z", "A-Z", "0-9"'))
            
                    Data Elements:
            
                    GM:^RDTIS41 = pyde.02.00:OrganisationNameDetails.OrganisationalName.Text
                    */
                    assertion = (string.IsNullOrWhiteSpace(gmContext.RDTIS41) != true && !(IsMatch(gmContext.RDTIS41, @"[a-z]|[A-Z]|[0-9]",RegexOptions.IgnoreCase)));
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.GEN.410039", Severity = ProcessMessageSeverity.Error,
                            Description = @"Organisation name must contain at least one alpha or numeric character",
                            Location = "/xbrli:xbrl/orgname2.02.00:OrganisationNameDetails/pyde.02.00:OrganisationNameDetails.OrganisationalName.Text",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.410039"} },
                        };
                        processMessage.Parameters.Add(new ProcessMessageParameter
                            { Name = "RDTIS41", Value = gmContext.RDTIS41 });
            
                        response.Add(processMessage);
                    }
                    #endregion // VR.ATO.GEN.410039
            
                    #region VR.ATO.GEN.410140
            
                    /*  VR.ATO.GEN.410140
                    Family name must contain at least one alphabetical character
    
                    Legacy Rule Format:
                    WHERE IN TUPLE(organisationname2.xx.xx:OrganisationNameDetails)
                    IF (pyde.xx.xx:OrganisationNameDetails.Currency.Code = NULLORBLANK)
                       RETURN VALIDATION MESSAGE
                    ENDIF
        
                    Technical Business Rule Format:
                    (^ORG <> NULL) AND ((^RDTIS40 = BLANK))
            
                    Data Elements:
            
                    GM:^RDTIS40 = pyde.02.00:OrganisationNameDetails.Currency.Code
            
                    GM:^ORG = orgname2.02.00:OrganisationNameDetails
                    */
                    assertion = (gmContext.OrganisationNameDetailsCollectionExists != false && string.IsNullOrWhiteSpace(gmContext.RDTIS40) == true);
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.GEN.410140", Severity = ProcessMessageSeverity.Error,
                            Description = @"Organisation Name Currency Code must be present",
                            Location = "/xbrli:xbrl/orgname2.02.00:OrganisationNameDetails/pyde.02.00:OrganisationNameDetails.Currency.Code",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.410140"} },
                        };
                        processMessage.Parameters.Add(new ProcessMessageParameter
                            { Name = "RDTIS40", Value = gmContext.RDTIS40 });
            
                        response.Add(processMessage);
                    }
                    #endregion // VR.ATO.GEN.410140
            
                    #region VR.ATO.GEN.410147
            
                    /*  VR.ATO.GEN.410147
                    Name Type Code must be present if Name of the Interposed Entity is present.
    
                    Legacy Rule Format:
                    WHERE IN TUPLE(organisationname2.xx.xx:OrganisationNameDetails)
                    IF (pyde.xx.xx:OrganisationNameDetails.OrganisationalNameType.Code = NULLORBLANK) 
                       RETURN VALIDATION MESSAGE
                    ENDIF
        
                    Technical Business Rule Format:
                    (^ORG <> NULL) AND ((^RDTIS39 = BLANK))
            
                    Data Elements:
            
                    GM:^RDTIS39 = pyde.02.00:OrganisationNameDetails.OrganisationalNameType.Code
            
                    GM:^ORG = orgname2.02.00:OrganisationNameDetails
                    */
                    assertion = (gmContext.OrganisationNameDetailsCollectionExists != false && string.IsNullOrWhiteSpace(gmContext.RDTIS39) == true);
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.GEN.410147", Severity = ProcessMessageSeverity.Error,
                            Description = @"Name Type Code must be present if Name of the Interposed Entity is present.",
                            Location = "/xbrli:xbrl/orgname2.02.00:OrganisationNameDetails/pyde.02.00:OrganisationNameDetails.OrganisationalNameType.Code",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.410147"} },
                        };
                        processMessage.Parameters.Add(new ProcessMessageParameter
                            { Name = "RDTIS39", Value = gmContext.RDTIS39 });
            
                        response.Add(processMessage);
                    }
                    #endregion // VR.ATO.GEN.410147
            
                    #region VR.ATO.GEN.410206
            
                    /*  VR.ATO.GEN.410206
                    Non Individual Name cannot contain "The trustee", "The Exec", "exec" or "the TTE"
    
                    Legacy Rule Format:
                    IF (pyde.xx.xx:OrganisationNameDetails.OrganisationalName.Text = FOUND("The trustee","The Exec","exec","The TTE"))
                       RETURN VALIDATION MESSAGE
                    ENDIF
        
                    Technical Business Rule Format:
                    (FoundSet(^RDTIS41, '"The trustee","The Exec","exec","The TTE"'))
            
                    Data Elements:
            
                    GM:^RDTIS41 = pyde.02.00:OrganisationNameDetails.OrganisationalName.Text
                    */
                    assertion = IsMatch(gmContext.RDTIS41, @"(^(The trustee|The Exec|exec|The TTE)$)|(^(.*)( The trustee | The Exec | exec | The TTE )(.*)$)|(^(The trustee |The Exec |exec |The TTE )(.*)$)|(^(.*)( The trustee| The Exec| exec| The TTE)$)",RegexOptions.IgnoreCase);
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.GEN.428042", Severity = ProcessMessageSeverity.Error,
                            Description = @"Non Individual Name cannot contain ""The trustee"", ""The Exec"", ""exec"" or ""the TTE""",
                            Location = "/xbrli:xbrl/orgname2.02.00:OrganisationNameDetails/pyde.02.00:OrganisationNameDetails.OrganisationalName.Text",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.410206"} },
                        };
                        processMessage.Parameters.Add(new ProcessMessageParameter
                            { Name = "RDTIS41", Value = gmContext.RDTIS41 });
            
                        response.Add(processMessage);
                    }
                    #endregion // VR.ATO.GEN.410206
            
                    #region VR.ATO.GEN.410215
            
                    /*  VR.ATO.GEN.410215
                    Organisation name must be present
    
                    Legacy Rule Format:
                    WHERE IN TUPLE(organisationname2.xx.xx:OrganisationNameDetails)
                    IF (pyde.xx.xx:OrganisationNameDetails.OrganisationalName.Text = NULLORBLANK)
                       RETURN VALIDATION MESSAGE
                    ENDIF
        
                    Technical Business Rule Format:
                    (^ORG <> NULL) AND ((^RDTIS41 = BLANK))
            
                    Data Elements:
            
                    GM:^RDTIS41 = pyde.02.00:OrganisationNameDetails.OrganisationalName.Text
            
                    GM:^ORG = orgname2.02.00:OrganisationNameDetails
                    */
                    assertion = (gmContext.OrganisationNameDetailsCollectionExists != false && string.IsNullOrWhiteSpace(gmContext.RDTIS41) == true);
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.GEN.410215", Severity = ProcessMessageSeverity.Error,
                            Description = @"Organisation name must be present",
                            Location = "/xbrli:xbrl/orgname2.02.00:OrganisationNameDetails/pyde.02.00:OrganisationNameDetails.OrganisationalName.Text",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.410215"} },
                        };
                        processMessage.Parameters.Add(new ProcessMessageParameter
                            { Name = "RDTIS41", Value = gmContext.RDTIS41 });
            
                        response.Add(processMessage);
                    }
                    #endregion // VR.ATO.GEN.410215
            
                    #region VR.ATO.GEN.428258
            
                    /*  VR.ATO.GEN.428258
                    Non Individual Name cannot contain space hyphen space combinations
    
                    Legacy Rule Format:
                    IF (pyde.xx.xx:OrganisationNameDetails.OrganisationalName.Text CONTAINS  " - " )
                       RETURN VALIDATION MESSAGE
                    ENDIF
        
                    Technical Business Rule Format:
                    Contains(^RDTIS41, ' - ')
            
                    Data Elements:
            
                    GM:^RDTIS41 = pyde.02.00:OrganisationNameDetails.OrganisationalName.Text
                    */
                    assertion = (gmContext.RDTIS41 == null ? false : gmContext.RDTIS41.ToUpperInvariant().Contains(@" - "));
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.GEN.000413", Severity = ProcessMessageSeverity.Error,
                            Description = @"Non Individual Name cannot contain space hyphen space combinations",
                            Location = "/xbrli:xbrl/orgname2.02.00:OrganisationNameDetails/pyde.02.00:OrganisationNameDetails.OrganisationalName.Text",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.428258"} },
                        };
                        processMessage.Parameters.Add(new ProcessMessageParameter
                            { Name = "RDTIS41", Value = gmContext.RDTIS41 });
            
                        response.Add(processMessage);
                    }
                    #endregion // VR.ATO.GEN.428258
            
                    #region VR.ATO.GEN.428259
            
                    /*  VR.ATO.GEN.428259
                    Non Individual Name cannot contain "P/L"
    
                    Legacy Rule Format:
                    IF (pyde.xx.xx:OrganisationNameDetails.OrganisationalName.Text CONTAINS "P/L")
                       RETURN VALIDATION MESSAGE
                    ENDIF
        
                    Technical Business Rule Format:
                    Contains(^RDTIS41, 'P/L')
            
                    Data Elements:
            
                    GM:^RDTIS41 = pyde.02.00:OrganisationNameDetails.OrganisationalName.Text
                    */
                    assertion = (gmContext.RDTIS41 == null ? false : gmContext.RDTIS41.ToUpperInvariant().Contains(@"P/L"));
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.GEN.000414", Severity = ProcessMessageSeverity.Error,
                            Description = @"Non Individual Name cannot contain ""P/L""",
                            Location = "/xbrli:xbrl/orgname2.02.00:OrganisationNameDetails/pyde.02.00:OrganisationNameDetails.OrganisationalName.Text",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.428259"} },
                        };
                        processMessage.Parameters.Add(new ProcessMessageParameter
                            { Name = "RDTIS41", Value = gmContext.RDTIS41 });
            
                        response.Add(processMessage);
                    }
                    #endregion // VR.ATO.GEN.428259
            
                    #region VR.ATO.GEN.428260
            
                    /*  VR.ATO.GEN.428260
                    Non Individual Name cannot contain repeated hyphen, apostrophe, or space characters
    
                    Legacy Rule Format:
                    IF (pyde.xx.xx:OrganisationNameDetails.OrganisationalName.Text CONTAINS SET("--","''","  ") )
                       RETURN VALIDATION MESSAGE
                    ENDIF
        
                    Technical Business Rule Format:
                    (ContainsSet(^RDTIS41, '"--","''","  "'))
            
                    Data Elements:
            
                    GM:^RDTIS41 = pyde.02.00:OrganisationNameDetails.OrganisationalName.Text
                    */
                    assertion = IsMatch(gmContext.RDTIS41, @"--|''|  ",RegexOptions.IgnoreCase);
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.GEN.000416", Severity = ProcessMessageSeverity.Error,
                            Description = @"Non Individual Name cannot contain repeated hyphen, apostrophe, or space characters",
                            Location = "/xbrli:xbrl/orgname2.02.00:OrganisationNameDetails/pyde.02.00:OrganisationNameDetails.OrganisationalName.Text",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.428260"} },
                        };
                        processMessage.Parameters.Add(new ProcessMessageParameter
                            { Name = "RDTIS41", Value = gmContext.RDTIS41 });
            
                        response.Add(processMessage);
                    }
                    #endregion // VR.ATO.GEN.428260
            
                    #region VR.ATO.GEN.428261
            
                    /*  VR.ATO.GEN.428261
                    Non Individual Name cannot end with "T/A", "T/A P'ship", "T/A Pship", "T/A P/Ship" or "T/A Partnership"
    
                    Legacy Rule Format:
                    IF (pyde.xx.xx:OrganisationNameDetails.OrganisationalName.Text ENDSWITH SET(" T/A"," T/A P'ship"," T/A Pship"," T/A P/Ship"," T/A Partnership"))
                      RETURN VALIDATION MESSAGE
                    ENDIF
        
                    Technical Business Rule Format:
                    (EndsWithSet(^RDTIS41, '" T/A"," T/A P'ship"," T/A Pship"," T/A P/Ship"," T/A Partnership"'))
            
                    Data Elements:
            
                    GM:^RDTIS41 = pyde.02.00:OrganisationNameDetails.OrganisationalName.Text
                    */
                    assertion = IsMatch(gmContext.RDTIS41, @"( T/A| T/A P'ship| T/A Pship| T/A P/Ship| T/A Partnership)$",RegexOptions.IgnoreCase);
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.GEN.000417", Severity = ProcessMessageSeverity.Error,
                            Description = @"Non Individual Name cannot end with ""T/A"", ""T/A P'ship"", ""T/A Pship"", ""T/A P/Ship"" or ""T/A Partnership""",
                            Location = "/xbrli:xbrl/orgname2.02.00:OrganisationNameDetails/pyde.02.00:OrganisationNameDetails.OrganisationalName.Text",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.428261"} },
                        };
                        processMessage.Parameters.Add(new ProcessMessageParameter
                            { Name = "RDTIS41", Value = gmContext.RDTIS41 });
            
                        response.Add(processMessage);
                    }
                    #endregion // VR.ATO.GEN.428261
            
                    #region VR.ATO.GEN.430850
            
                    /*  VR.ATO.GEN.430850
                    Orgnaisation name Currency code must be 'C' or 'P'
    
                    Legacy Rule Format:
                    IF pyde.xx.xx:OrganisationNameDetails.Currency.Code <> NULLORBLANKAND pyde.xx.xx:OrganisationNameDetails.Currency.Code <> SET ("C", "P")
                       RETURN VALIDATION MESSAGE
                    ENDIF
        
                    Technical Business Rule Format:
                    ^RDTIS40 <> BLANK AND NotInSet(^RDTIS40, '"C", "P"')
            
                    Data Elements:
            
                    GM:^RDTIS40 = pyde.02.00:OrganisationNameDetails.Currency.Code
                    */
                    assertion = (string.IsNullOrWhiteSpace(gmContext.RDTIS40) != true && !(IsMatch(gmContext.RDTIS40, @"^(C|P)$",RegexOptions.IgnoreCase)));
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.GEN.430850", Severity = ProcessMessageSeverity.Error,
                            Description = @"Orgnaisation name Currency code must be 'C' or 'P'",
                            LongDescription = @"Organisation Name Details Currency Code must be 'C' for current organisation name or 'P' for previous organisation name",
                            Location = "/xbrli:xbrl/orgname2.02.00:OrganisationNameDetails/pyde.02.00:OrganisationNameDetails.Currency.Code",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.430850"} },
                        };
                        processMessage.Parameters.Add(new ProcessMessageParameter
                            { Name = "RDTIS40", Value = gmContext.RDTIS40 });
            
                        response.Add(processMessage);
                    }
                    #endregion // VR.ATO.GEN.430850
            
                    #region VR.ATO.GEN.438006
            
                    /*  VR.ATO.GEN.438006
                    Organisation Name Type Code is not equal to 'MN' or Organisation Name Currency Code is not equal to 'C'
    
                    Legacy Rule Format:
                    WHERE IN TUPLE(orgname2.xx.xx:OrganisationNameDetails)
                    IF (pyde.xx.xx:OrganisationNameDetails.OrganisationalNameType.Code <> "MN") OR (pyin.xx.xx:OrganisationNameDetails.Currency.Code <> "C")
                       RETURN VALIDATION MESSAGE
                    ENDIF
        
                    Technical Business Rule Format:
                    (^ORG <> NULL) AND ((^RDTIS39 <> 'MN') OR (^RDTIS40 <> 'C'))
            
                    Data Elements:
            
                    GM:^RDTIS39 = pyde.02.00:OrganisationNameDetails.OrganisationalNameType.Code
            
                    GM:^RDTIS40 = pyde.02.00:OrganisationNameDetails.Currency.Code
            
                    GM:^ORG = orgname2.02.00:OrganisationNameDetails
                    */
                    assertion = (gmContext.OrganisationNameDetailsCollectionExists != false && (gmContext.RDTIS39 != @"MN" || gmContext.RDTIS40 != @"C"));
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.GEN.438006", Severity = ProcessMessageSeverity.Error,
                            Description = @"Organisation Name Type Code must be 'MN' and Organisation Name Currency Code must be 'C'.",
                            Location = "/xbrli:xbrl/orgname2.02.00:OrganisationNameDetails/pyde.02.00:OrganisationNameDetails.OrganisationalName.Text",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.438006"} },
                        };
                        processMessage.Parameters.Add(new ProcessMessageParameter
                            { Name = "RDTIS39", Value = gmContext.RDTIS39 });
            
                        processMessage.Parameters.Add(new ProcessMessageParameter
                            { Name = "RDTIS40", Value = gmContext.RDTIS40 });
            
                        response.Add(processMessage);
                    }
                    #endregion // VR.ATO.GEN.438006
            
                   
            
                    #region VR.ATO.RDTIS.438042
            
                    /*  VR.ATO.RDTIS.438042
                    Monetary field, must be unsigned, maximum of 11 chars, no decimals
    
                    Legacy Rule Format:
                    IF gfagc.xx.xx:TaxConcession.ResearchAndDevelopment.TaxOffsetEntityTurnover.Amount <> NULL AND gfagc.xx.xx:TaxConcession.ResearchAndDevelopment.TaxOffsetEntityTurnover.Amount <> MONETARY(U,11,0)
                       RETURN VALIDATION MESSAGE
                    ENDIF
        
                    Technical Business Rule Format:
                    NotMonetary(^RDTIS42, 'U', 11, 0)
            
                    Data Elements:
            
                    GM:^RDTIS42 = gfagc.02.04:TaxConcession.ResearchAndDevelopment.TaxOffsetEntityTurnover.Amount
                    */
                    assertion = NotMonetary(gmContext.RDTIS42, @"U", 11, 0);
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.GEN.400011", Severity = ProcessMessageSeverity.Error,
                            Description = @"Field incorrect format. Exceeds maximum digits or decimals and/or contains illegal characters",
                            LongDescription = @"Field is in incorrect format for a monetary field. Field either exceeds maximum digits or decimals and/or contains illegal characters and/or for a signed field does not have the sign as the first character",
                            Location = "/xbrli:xbrl/gfagc.02.04:TaxConcession.ResearchAndDevelopment.TaxOffsetEntityTurnover.Amount[@contextRef='" + gmContext.Id + "']",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.RDTIS.438042"} },
                        };
                        processMessage.Parameters.Add(new ProcessMessageParameter
                            { Name = "RDTIS42", Value = gmContext.RDTIS42.GetValueOrDefault().ToString() });
            
                        response.Add(processMessage);
                    }
                    #endregion // VR.ATO.RDTIS.438042
            
                    #region VR.ATO.RDTIS.438045
            
                    /*  VR.ATO.RDTIS.438045
                    A Group Member context is present with no Entity name present
    
                    Legacy Rule Format:
                    WHERE CONTEXT(GM)
                    IF [RDTIS41] = NULLORBLANK
                       RETURN VALIDATION MESSAGE
                    ENDIF
                    
                    [RDTIS41] = RDTIS:GM:pyde.02.00:OrganisationNameDetails.OrganisationalName.Text IN TUPLE(orgname2.02.00:OrganisationNameDetails)
        
                    Technical Business Rule Format:
                    ^RDTIS41 = BLANK
            
                    Data Elements:
            
                    GM:^RDTIS41 = pyde.02.00:OrganisationNameDetails.OrganisationalName.Text
                    */
                    assertion = (string.IsNullOrWhiteSpace(gmContext.RDTIS41) == true);
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.RDTIS.438045", Severity = ProcessMessageSeverity.Error,
                            Description = @"Each Group Member must have an Entity name",
                            Location = "/xbrli:xbrl/orgname2.02.00:OrganisationNameDetails/pyde.02.00:OrganisationNameDetails.OrganisationalName.Text",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.RDTIS.438045"} },
                        };
                        processMessage.Parameters.Add(new ProcessMessageParameter
                            { Name = "RDTIS41", Value = gmContext.RDTIS41 });
            
                        response.Add(processMessage);
                    }
                    #endregion // VR.ATO.RDTIS.438045
            
                    #region VR.ATO.RDTIS.438046
            
                    /*  VR.ATO.RDTIS.438046
                    A Group Member context is present with no Entity turnover present
    
                    Legacy Rule Format:
                    WHERE CONTEXT(GM)
                    IF [RDTIS42] = NULL
                       RETURN VALIDATION MESSAGE
                    ENDIF
                    
                    [RDTIS42] = RDTIS:GM:gfagc.02.04:TaxConcession.ResearchAndDevelopment.TaxOffsetEntityTurnover.Amount
        
                    Technical Business Rule Format:
                    ^RDTIS42 = NULL
            
                    Data Elements:
            
                    GM:^RDTIS42 = gfagc.02.04:TaxConcession.ResearchAndDevelopment.TaxOffsetEntityTurnover.Amount
                    */
                    assertion = (gmContext.RDTIS42 == null);
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.RDTIS.438046", Severity = ProcessMessageSeverity.Error,
                            Description = @"Each Group Member must have an Entity turnover",
                            Location = "/xbrli:xbrl/gfagc.02.04:TaxConcession.ResearchAndDevelopment.TaxOffsetEntityTurnover.Amount[@contextRef='" + gmContext.Id + "']",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.RDTIS.438046"} },
                        };
                        processMessage.Parameters.Add(new ProcessMessageParameter
                            { Name = "RDTIS42", Value = gmContext.RDTIS42.GetValueOrDefault().ToString() });
            
                        response.Add(processMessage);
                    }
                    #endregion // VR.ATO.RDTIS.438046
                    }
                }
        
                #endregion // Foreach loop

            foreach (ProcessMessageDocument currentProcessMessage in response)
            {
                if (currentProcessMessage.Parameters != null)
                {
                    foreach (ProcessMessageParameter currentParameter in currentProcessMessage.Parameters)
                    {
                        if (string.IsNullOrEmpty(currentParameter.Name))
                        {
                            currentParameter.Name = _emptyParameterValue;
                        }

                        if (currentParameter.Name.Length > _maxParameterNameLength)
                        {
                            currentParameter.Name = currentParameter.Name.Substring(0, _maxParameterNameLength - 1);
                        }

                        if (string.IsNullOrEmpty(currentParameter.Value))
                        {
                            currentParameter.Value = _emptyParameterValue;
                        }

                        if (currentParameter.Value.Length > _maxParameterValueLength)
                        {
                            currentParameter.Value = currentParameter.Value.Substring(0, _maxParameterValueLength - 1);
                        }
                    }
                }
            }

            return response;
        }


    }
} 
