using System;
using System.Collections.Generic;
using System.Linq;
using System.Xml;
using System.Text.RegularExpressions;
using Au.Gov.Sbr.Xbrl.Document;
using DataContracts;
using Au.Gov.Sbr.Xbrl.Metadata;
using System.Runtime.CompilerServices;
using System.IO;
using System.Text;
using System.Xml.Schema;
using System.Reflection;
using System.Runtime.Serialization; 

namespace Ato.EN.IntegrationServices.CodeGenerationPTR
{

    public partial class PTR2017Validator
    {
        /// <summary>
        /// The max parameter name length is restricted by the SBR1 and SBR2 schemas - the lowest common denominator of SBR 1 is used as the default
        /// </summary>
        private int _maxParameterNameLength;

        /// <summary>
        /// The max parameter value length is restricted by the SBR1 and SBR2 schemas
        /// </summary>
        private int _maxParameterValueLength;

        /// <summary>
        /// The SBR1 and ebms3 schemas do not allow parameter names or values to be the empty string
        /// </summary>
        private string _emptyParameterValue;

        /// <summary>
        /// Initializes a new instance of the <see cref="PTR2017Validator" /> class.
        /// </summary>
        /// <param name="maxParameterNameLength">Maximum length of the parameter name.</param>
        /// <param name="maxParameterValueLength">Maximum length of the parameter value.</param>
        /// <param name="emptyParameterValue">This value will be used in place of any parameter values that result in a null or empty value.</param>
        public PTR2017Validator(int maxParameterNameLength = 20, int maxParameterValueLength = 4096, string emptyParameterValue = "EMPTY")
        {
            _maxParameterNameLength = maxParameterNameLength;
            _maxParameterValueLength = maxParameterValueLength;
            _emptyParameterValue = emptyParameterValue;
        }

        #region Functions
        private static IEnumerable<string> Union(IEnumerable<string> list1, IEnumerable<string> list2)
        {
            IEnumerable<string> response;

            if (list1 == null && list2 == null)
            {
                response = null;
            }
            else if (list1 == null)
            {
                response = list2.Distinct();
            }
            else if (list2 == null)
            {
                response = list1.Distinct();
            }
            else
            {
                response = list1.Union(list2);
            }

            return response;
        }

        private static bool IsMatch(int? field, string expression, RegexOptions options = RegexOptions.None)
        {
            if (field == null)
                return false;
            else
                return Regex.IsMatch(Convert.ToString(field.Value), expression, options);
        }

        private static bool IsMatch(string field, string expression, RegexOptions options = RegexOptions.None)
        {
            if (field == null)
                return false;
            else
                return Regex.IsMatch(field, expression, options);
        }

        // This is just context and tuple counts where they are integer values - easier that changing the parsing logic to just return the value
        private static int Count(int count)
        {
            return count;
        }


        private static int Count<T>(IEnumerable<T> values)
        {
            return values == null ? 0 : values.Where(f => f != null).Count();
        }


        private static int Count<T>(ICollection<T> values)
        {
            return values == null ? 0 : values.Where(f => f != null).Count();
        }


        private static bool exists(bool value)
        {
            return value;
        }

        private static string GetValueOrEmpty(bool? val)
        {
            return (val.HasValue) ? val.ToString() : string.Empty;
        }

        private static string GetValueOrEmpty(DateTime? val)
        {
            return (val.HasValue) ? val.ToString() : string.Empty;
        }

        /// <summary>
        /// Get string value between [first] a and [last] b.
        /// </summary>
        public static string Between(string value, string a, string b)
        {
            int posA = value.IndexOf(a);
            int posB = value.LastIndexOf(b);
            if (posA == -1)
            {
                return "";
            }
            if (posB == -1)
            {
                return "";
            }
            int adjustedPosA = posA + a.Length;
            if (adjustedPosA >= posB)
            {
                return "";
            }
            return value.Substring(adjustedPosA, posB - adjustedPosA);
        }

        /// <summary>
        /// Get string value after [first] a.
        /// </summary>
        public static string Before(string value, string a)
        {
            int posA = value.IndexOf(a);
            if (posA == -1)
            {
                return "";
            }
            return value.Substring(0, posA);
        }

        /// <summary>
        /// Get string value after [last] a.
        /// </summary>
        public static string After(string value, string a)
        {
            int posA = value.LastIndexOf(a);
            if (posA == -1)
            {
                return "";
            }
            int adjustedPosA = posA + a.Length;
            if (adjustedPosA >= value.Length)
            {
                return "";
            }
            return value.Substring(adjustedPosA);
        }

        private static int Length(object field)
        {
            if (field == null)
                return 0;
            else
                return field.ToString().Trim().Length;
        }

        private static bool NotSameValues(IEnumerable<object> nodes)
        {
            if (nodes == null)
                return false;

            object[] nodesArray = nodes.Cast<object>().ToArray();
            return NotSameValues(nodesArray);
        }

        private static bool NotSameValues(params object[] nodes)
        {
            if (nodes == null)
                return false;

            return ((from x in nodes select x).Distinct().Count() == nodes.Count());
        }

        private static bool HasDuplicateValues(IEnumerable<object> nodes)
        {
            if (nodes == null)
                return false;

            object[] nodesArray = nodes.Cast<object>().ToArray();
            return HasDuplicateValues(nodesArray);
        }

        private static bool HasDuplicateValues(params object[] nodes)
        {
            if (nodes == null)
                return false;

            return !((from x in nodes select x).Distinct().Count() == nodes.Count());
        }

        private int DuplicateValueIndex(IEnumerable<object> values)
        {
            int response = 0;
            var hashset = new HashSet<object>();
            foreach (var value in values)
            {
                if (!hashset.Add(value))
                {
                    return response;
                }
                response++;
            }
            return response;
        }

        private int DuplicateValueIndex<T>(IEnumerable<T?> values) where T : struct
        {
            int response = 0;
            var hashset = new HashSet<T?>();
            foreach (var value in values)
            {
                if (!hashset.Add(value))
                {
                    return response;
                }
                response++;
            }
            return response;
        }

        private static bool IsDate(object value)
        {
            DateTime dateValue;
            return (value != null && DateTime.TryParse(value.ToString(), out dateValue));
        }

        public DateTime AsDate(string dateAsString)
        {
            DateTime response = DateTime.MinValue;
            DateTime date;

            if (DateTime.TryParse(dateAsString, out date))
            {
                response = date;
            }

            return response;
        }

        public DateTime? ConvertToDate(int day, int month, int year)
        {
            return ConvertToDate(day == 0 ? null : day.ToString(), month == 0 ? null : month.ToString(), year == 0 ? null : year.ToString());
        }

        public DateTime? ConvertToDate(string day, string month, string year)
        {
            DateTime? response;
            DateTime result;

            if (year == null || month == null || day == null)
            {
                return null;
            }
            string dateAsString = year + "-" + After(month, "--") + "-" + After(day, "---");
            if (DateTime.TryParse(dateAsString, out result))
            {
                response = result;
            }
            else
            {
                response = null;
            }
            return response;
        }


        public DateTime? ConvertToDate(string day, string month, int year)
        {
            DateTime? response;
            DateTime result;

            if (year == 0 || month == null || day == null)
            {
                return null;
            }
            string dateAsString = year.ToString() + "-" + After(month, "--") + "-" + After(day, "---");
            if (DateTime.TryParse(dateAsString, out result))
            {
                response = result;
            }
            else
            {
                response = null;
            }
            return response;
        }


        private static int Day(string dateAsString)
        {
            int response = 0;
            DateTime date;

            if (DateTime.TryParse(dateAsString, out date))
            {
                response = date.Day;
            }

            return response;
        }


        private static int Day(DateTime? date)
        {
            if (date == null)
                return 0;
            else
                return date.Value.Day;

        }


        private static string Month(string dateAsString)
        {
            string response = null;
            DateTime date;

            if (DateTime.TryParse(dateAsString, out date))
            {
                response = date.ToString("MMMM");
            }
            else
            {
                return "NotAMonth";
            }

            return response;
        }

        private static string Month(DateTime? date)
        {
            if (date == null)
                return "NotAMonth";
            else
                return date.Value.ToString("MMMM");
        }


        private static int MonthAsInt(string dateAsString)
        {
            int response = 0;
            DateTime date;

            if (DateTime.TryParse(dateAsString, out date))
            {
                response = date.Month;
            }

            return response;
        }

        private static int MonthAsInt(DateTime? date)
        {
            if (date == null)
            {
                return 0;
            }
            return date.Value.Month;
        }


        private static int Year(string dateAsString)
        {
            int response = 0;
            DateTime date;

            if (DateTime.TryParse(dateAsString, out date))
            {
                response = date.Year;
            }

            return response;
        }

        private static int Year(DateTime? date)
        {
            if (date == null)
                return 0;
            else
                return date.Value.Year;
        }


        private static int CurrentFinancialYear()
        {
            return DateToFinancialYear(DateTime.Now, 7);
        }

        private static int FinancialYear(string dateAsString)
        {
            return DateToFinancialYear(dateAsString, 7);
        }

        private static int FinancialYear(DateTime? date)
        {
            return DateToFinancialYear(date, 7);
        }

        private static int DateToFinancialYear(string dateAsString, int startingMonth)
        {
            int response = 0;
            DateTime date;
            if (DateTime.TryParse(dateAsString, out date))
            {
                response = DateToFinancialYear(date, startingMonth);
            }

            return response;
        }
        private static int DateToFinancialYear(DateTime? date, int startingMonth)
        {
            int response;
            if (date == null)
            {
                response = 0;
            }
            else
            {
                int year = date.Value.Year;
                int month = date.Value.Month;

                if (startingMonth > month)
                    response = year;
                else
                    response = year + 1;
            }
            return response;
        }


        private static int FBTYear(string dateAsString)
        {
            int response = 0;
            DateTime date;

            if (DateTime.TryParse(dateAsString, out date))
                response = FBTYear(date);
            return response;
        }

        private static int FBTYear(DateTime? date)
        {
            if (date == null)
            {
                return 0;
            }
            else
            {
                if (date.Value.Month > 3)
                    return date.Value.Year + 1;
                else
                    return date.Value.Year;
            }
        }


        private static bool IsNumeric(object value)
        {
            decimal numbervalue;
            return (value != null && decimal.TryParse(value.ToString(), out numbervalue));
        }

        private static bool NotMonetary(decimal? field, string sign, int digits, int decimals)
        {
            if (field == null)
            {
                return false;
            }
            else
            {
                string signExpression;
                string decimalExpression;
                int digitsToUse = digits - decimals;

                if (sign == "U")
                    signExpression = "^";
                else
                    signExpression = "^-?";

                if (decimals > 0)
                    decimalExpression = @"(\.\d{1," + decimals + "})?$";
                else
                    decimalExpression = @"$";

                return !(Regex.IsMatch(field.Value.ToString("0.#########################"), signExpression + @"\d{1," + digitsToUse + "}" + decimalExpression));
            }
        }

        private static bool NotNumeric(decimal? field, string sign, int digits, int decimals)
        {
            if (field == null)
            {
                return false;
            }
            else
            {
                string signExpression;
                string decimalExpression;
                int digitsToUse = digits - decimals;

                if (sign == "U")
                    signExpression = "^";
                else
                    signExpression = "^-?";

                if (decimals > 0)
                    decimalExpression = @"(\.\d{1," + decimals + "})?$";
                else
                    decimalExpression = @"$";

                return !(Regex.IsMatch(field.Value.ToString("0.#########################"), signExpression + @"\d{1," + digitsToUse + "}" + decimalExpression));
            }
        }

        private static bool NotNumeric(int? field, string sign, int digits, int decimals = 0)
        {
            if (field == null)
            {
                return false;
            }
            else
            {
                string signExpression;

                if (sign == "U")
                    signExpression = "^";
                else
                    signExpression = "^-?";

                return !(Regex.IsMatch(field.Value.ToString(), signExpression + @"\d{1," + digits + "}$"));
            }
        }

        private static bool NotNumeric(long? field, string sign, int digits, int decimals = 0)
        {
            if (field == null)
            {
                return false;
            }
            else
            {
                string signExpression;

                if (sign == "U")
                    signExpression = "^";
                else
                    signExpression = "^-?";

                return !(Regex.IsMatch(field.Value.ToString(), signExpression + @"\d{1," + digits + "}$"));
            }
        }


        private static bool OutsideRange(decimal field, decimal expression, int range)
        {
            bool response;

            response = (field < (expression - range)) || (field > (expression + range));

            return response;
        }


        private static bool FailsUSIAlgorithm(string usi, string abn)
        {
            bool response;
            if (usi == null || abn == null)
            {
                response = false;
            }
            else
            {
                usi = usi.Trim();
                abn = abn.Trim();
                if (usi.Length < 13 || abn.Length < 11)
                {
                    response = false;
                }
                else
                {
                    int numeric;
                    if (usi.Substring(0, 11) == abn && int.TryParse(usi.Substring(11, 2), out numeric))
                        response = false;
                    else if (Regex.IsMatch(usi, @"^[a-zA-Z]{3}\d{4}[a-zA-Z]{2}"))
                        response = false;
                    else
                        response = true;
                }
            }
            return response;
        }


        private static bool FailsTANAlgorithm(string tan)
        {
            bool response;
            decimal decimalTan;

            if (tan == null)
                return false;

            tan = tan.Trim();

            if (!decimal.TryParse(tan, out decimalTan))
                return true;

            if (tan.Length != 8)
                return true;

            decimal tanSum =
                7 * int.Parse(tan.Substring(0, 1)) +
                9 * int.Parse(tan.Substring(1, 1)) +
                8 * int.Parse(tan.Substring(2, 1)) +
                4 * int.Parse(tan.Substring(3, 1)) +
                6 * int.Parse(tan.Substring(4, 1)) +
                3 * int.Parse(tan.Substring(5, 1)) +
                5 * int.Parse(tan.Substring(6, 1)) +
                1 * int.Parse(tan.Substring(7, 1));

            if ((tanSum % 11) == 0)
                response = false;
            else
                response = true;

            return response;
        }


        private static bool FailsABNAlgorithm(string abn)
        {
            bool response;
            decimal decimalAbn;

            if (abn == null)
                return false;

            abn = abn.Trim();

            if (!decimal.TryParse(abn, out decimalAbn))
                return true;

            if (abn.Length != 11)
                return true;

            decimal abnSum =
                10 * (int.Parse(abn.Substring(0, 1)) - 1) +
                1 * int.Parse(abn.Substring(1, 1)) +
                3 * int.Parse(abn.Substring(2, 1)) +
                5 * int.Parse(abn.Substring(3, 1)) +
                7 * int.Parse(abn.Substring(4, 1)) +
                9 * int.Parse(abn.Substring(5, 1)) +
                11 * int.Parse(abn.Substring(6, 1)) +
                13 * int.Parse(abn.Substring(7, 1)) +
                15 * int.Parse(abn.Substring(8, 1)) +
                17 * int.Parse(abn.Substring(9, 1)) +
                19 * int.Parse(abn.Substring(10, 1));

            if ((abnSum % 89) == 0)
                response = false;
            else
                response = true;

            return response;
        }
        private static bool FailsACNAlgorithm(string acn)
        {
            bool response;
            decimal decimalAbn;
            if (acn == null)
                return false;

            acn = acn.Trim();

            if (!decimal.TryParse(acn, out decimalAbn))
                return true;

            if (acn.Length != 9)
                return true;

            decimal abnSum =
                8 * int.Parse(acn.Substring(0, 1)) +
                7 * int.Parse(acn.Substring(1, 1)) +
                6 * int.Parse(acn.Substring(2, 1)) +
                5 * int.Parse(acn.Substring(3, 1)) +
                4 * int.Parse(acn.Substring(4, 1)) +
                3 * int.Parse(acn.Substring(5, 1)) +
                2 * int.Parse(acn.Substring(6, 1)) +
                1 * int.Parse(acn.Substring(7, 1));

            decimal checkDigit = int.Parse(acn.Substring(8, 1));
            decimal acnRemainder = abnSum % 10;

            if (((10 - acnRemainder) % 10) == checkDigit)
                response = false;
            else
                response = true;

            return response;
        }


        private static bool FailsTFNAlgorithm(string tfn)
        {
            bool response;
            decimal decimalTfn;

            if (tfn == null)
                return false;

            tfn = tfn.Trim();
            tfn = Regex.Replace(tfn, "^0+", "");

            if (!decimal.TryParse(tfn, out decimalTfn))
                return true;

            if (tfn.Length < 8)
                return true;


            decimal tfn1To7Sum =
                1 * int.Parse(tfn.Substring(0, 1)) +
                4 * int.Parse(tfn.Substring(1, 1)) +
                3 * int.Parse(tfn.Substring(2, 1)) +
                7 * int.Parse(tfn.Substring(3, 1)) +
                5 * int.Parse(tfn.Substring(4, 1)) +
                8 * int.Parse(tfn.Substring(5, 1)) +
                6 * int.Parse(tfn.Substring(6, 1));

            decimal tfn8 = 9 * int.Parse(tfn.Substring(7, 1));

            if (tfn.Length == 8)
            {
                decimal tFNLg8WSum9 = 10 * int.Parse(tfn.Substring(7, 1));
                decimal tFNLg8WSum = tfn1To7Sum + tFNLg8WSum9;

                if ((tFNLg8WSum % 11) == 0)
                    response = false;
                else
                    response = true;
            }
            else if (tfn.Length == 9)
            {
                decimal tfn9 = 10 * int.Parse(tfn.Substring(8, 1));
                decimal tFNLg9WSum = tfn1To7Sum + tfn8 + tfn9;

                if ((tFNLg9WSum % 11) == 0)
                    response = false;
                else
                    response = true;
            }
            else
            {
                response = true;
            }

            return response;
        }


        private static decimal ConditionalValue(bool expression, decimal? trueVal, decimal? falseVal)
        {
            return expression ? trueVal.GetValueOrDefault() : falseVal.GetValueOrDefault();
        }

        private static decimal AsNumeric(string value)
        {
            decimal numberValue;
            decimal.TryParse(value, out numberValue);
            return numberValue;
        }

        private static bool RegexMatch(int? field, string expression, string flags = "")
        {
            return IsMatch(field, expression, GetRegexOption(flags));
        }

        private static bool RegexMatch(string field, string expression, string flags = "")
        {
            return IsMatch(field, expression, GetRegexOption(flags));
        }

        private static RegexOptions GetRegexOption(string flags)
        {
            RegexOptions options = RegexOptions.None;

            char[] characters = flags.ToCharArray();

            foreach (char character in characters)
            {
                switch (character)
                {
                    case 'i':
                        options = options | RegexOptions.IgnoreCase;
                        break;
                    case 'm':
                        options = options | RegexOptions.Multiline;
                        break;
                    case 's':
                        options = options | RegexOptions.Singleline;
                        break;
                    case 'n':
                        options = options | RegexOptions.ExplicitCapture;
                        break;
                    case 'x':
                        options = options | RegexOptions.IgnorePatternWhitespace;
                        break;
                }
            }

            return options;
        }

        /// <summary>
        /// Returns an occurrence index as [occurrenceIndex] of occurrenceIndex > 0, otherwise the empty string
        /// </summary>
        /// <param name="occurrenceIndex">Index of the occurrence.</param>
        /// <returns>Occurrence in XPath [#] format</returns>
        public string OccurrenceIndex(int occurrenceIndex)
        {
            return occurrenceIndex > 0 ? "[" + occurrenceIndex + "]" : "";
        }

        #endregion // Functions

        public PTR2017 ConsumedReport { get; private set; }

        private static Dictionary<string, ProcessMessageDocument> _processMessageDocuments = new Dictionary<string, ProcessMessageDocument>();

        [MethodImpl(MethodImplOptions.AggressiveInlining)]
        public List<ProcessMessageDocument> ValidateReport(PTR2017 report)
        {

            List<ProcessMessageDocument> response = new List<ProcessMessageDocument>();
            ProcessMessageDocument processMessage;
            ProcessMessageParameter parameter;
            bool assertion;

            this.ConsumedReport = report;

            // ------------------------------------------------------------------------------
            // Validations can now be done in code rather than just schematron.
            // This version supports full validation of a report - for those rules that have successfully parsed and could be generated.
            // Generated Validations include:
            //   - Validation logic in C# rather than being Schematron based.
            //   - Production of errors in EBMS, SBR and SWS format.
            // 
            // The generation at this stage does not support the following - and has to be completed manually (later versions of the generation will do this for you)
            //   - Business Rules that did not parse.  The ESR team will help support you where this happens as an effort is being made to rectify these
            //   - You will get TODO tasks for those that the parser could not cope with
            // ------------------------------------------------------------------------------


            #region VR.ATO.GEN.000201

            /*  VR.ATO.GEN.000201
            Period start date must be prior to Period end date
    
            Legacy Rule Format:
            IF period.startDate >= period.endDate WHERE CONTEXT(ALL)
            RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            ^StartDate >= ^EndDate
    
            Data Elements:
    
            ^StartDate = INT
    
            ^EndDate = INT
            */
            assertion = ((report.INTExists == true) && (report.INTStartDate >= report.INTEndDate));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.200009",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Period start date must be prior to Period end date",
                    Location = "/xbrli:xbrl/xbrli:context" + OccurrenceIndex(report.INTOccurrenceIndex) + "/xbrli:period/xbrli:startDate",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.000201" } },
                };
                response.Add(processMessage);
            }
            #endregion // VR.ATO.GEN.000201

            #region VR.ATO.GEN.000201

            /*  VR.ATO.GEN.000201
            Period start date must be prior to Period end date
    
            Legacy Rule Format:
            IF period.startDate >= period.endDate WHERE CONTEXT(ALL)
            RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            ^StartDate >= ^EndDate
    
            Data Elements:
    
            ^StartDate = RP
    
            ^EndDate = RP
            */
            assertion = ((report.RPExists == true) && (report.RPStartDate >= report.RPEndDate));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.200009",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Period start date must be prior to Period end date",
                    Location = "/xbrli:xbrl/xbrli:context" + OccurrenceIndex(report.RPOccurrenceIndex) + "/xbrli:period/xbrli:startDate",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.000201" } },
                };
                response.Add(processMessage);
            }
            #endregion // VR.ATO.GEN.000201

            #region VR.ATO.GEN.000201

            /*  VR.ATO.GEN.000201
            Period start date must be prior to Period end date
    
            Legacy Rule Format:
            IF period.startDate >= period.endDate WHERE CONTEXT(ALL)
            RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            ^StartDate >= ^EndDate
    
            Data Elements:
    
            ^StartDate = RP:RP.GeneralPool
    
            ^EndDate = RP:RP.GeneralPool
            */
            assertion = ((report.RPGeneralPoolExists == true) && (report.RPGeneralPoolStartDate >= report.RPGeneralPoolEndDate));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.200009",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Period start date must be prior to Period end date",
                    Location = "/xbrli:xbrl/xbrli:context" + OccurrenceIndex(report.RPGeneralPoolOccurrenceIndex) + "/xbrli:period/xbrli:startDate",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.000201" } },
                };
                response.Add(processMessage);
            }
            #endregion // VR.ATO.GEN.000201

            #region VR.ATO.GEN.000201

            /*  VR.ATO.GEN.000201
            Period start date must be prior to Period end date
    
            Legacy Rule Format:
            IF period.startDate >= period.endDate WHERE CONTEXT(ALL)
            RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            ^StartDate >= ^EndDate
    
            Data Elements:
    
            ^StartDate = RP:RP.JAUS
    
            ^EndDate = RP:RP.JAUS
            */
            assertion = ((report.RPJAUSExists == true) && (report.RPJAUSStartDate >= report.RPJAUSEndDate));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.200009",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Period start date must be prior to Period end date",
                    Location = "/xbrli:xbrl/xbrli:context" + OccurrenceIndex(report.RPJAUSOccurrenceIndex) + "/xbrli:period/xbrli:startDate",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.000201" } },
                };
                response.Add(processMessage);
            }
            #endregion // VR.ATO.GEN.000201

            #region VR.ATO.GEN.000201

            /*  VR.ATO.GEN.000201
            Period start date must be prior to Period end date
    
            Legacy Rule Format:
            IF period.startDate >= period.endDate WHERE CONTEXT(ALL)
            RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            ^StartDate >= ^EndDate
    
            Data Elements:
    
            ^StartDate = RP.JAUS:RP.JAUS.NonPrim
    
            ^EndDate = RP.JAUS:RP.JAUS.NonPrim
            */
            assertion = ((report.RPJAUSNonPrimExists == true) && (report.RPJAUSNonPrimStartDate >= report.RPJAUSNonPrimEndDate));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.200009",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Period start date must be prior to Period end date",
                    Location = "/xbrli:xbrl/xbrli:context" + OccurrenceIndex(report.RPJAUSNonPrimOccurrenceIndex) + "/xbrli:period/xbrli:startDate",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.000201" } },
                };
                response.Add(processMessage);
            }
            #endregion // VR.ATO.GEN.000201

            #region VR.ATO.GEN.000201

            /*  VR.ATO.GEN.000201
            Period start date must be prior to Period end date
    
            Legacy Rule Format:
            IF period.startDate >= period.endDate WHERE CONTEXT(ALL)
            RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            ^StartDate >= ^EndDate
    
            Data Elements:
    
            ^StartDate = RP.JAUS:RP.JAUS.Prim
    
            ^EndDate = RP.JAUS:RP.JAUS.Prim
            */
            assertion = ((report.RPJAUSPrimExists == true) && (report.RPJAUSPrimStartDate >= report.RPJAUSPrimEndDate));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.200009",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Period start date must be prior to Period end date",
                    Location = "/xbrli:xbrl/xbrli:context" + OccurrenceIndex(report.RPJAUSPrimOccurrenceIndex) + "/xbrli:period/xbrli:startDate",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.000201" } },
                };
                response.Add(processMessage);
            }
            #endregion // VR.ATO.GEN.000201

            #region VR.ATO.GEN.000201

            /*  VR.ATO.GEN.000201
            Period start date must be prior to Period end date
    
            Legacy Rule Format:
            IF period.startDate >= period.endDate WHERE CONTEXT(ALL)
            RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            ^StartDate >= ^EndDate
    
            Data Elements:
    
            ^StartDate = RP:RP.JFOREIGN
    
            ^EndDate = RP:RP.JFOREIGN
            */
            assertion = ((report.RPJFOREIGNExists == true) && (report.RPJFOREIGNStartDate >= report.RPJFOREIGNEndDate));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.200009",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Period start date must be prior to Period end date",
                    Location = "/xbrli:xbrl/xbrli:context" + OccurrenceIndex(report.RPJFOREIGNOccurrenceIndex) + "/xbrli:period/xbrli:startDate",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.000201" } },
                };
                response.Add(processMessage);
            }
            #endregion // VR.ATO.GEN.000201

            #region VR.ATO.GEN.000201

            /*  VR.ATO.GEN.000201
            Period start date must be prior to Period end date
    
            Legacy Rule Format:
            IF period.startDate >= period.endDate WHERE CONTEXT(ALL)
            RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            ^StartDate >= ^EndDate
    
            Data Elements:
    
            ^StartDate = RP.JFOREIGN:RP.JFOREIGN.CFCLC
    
            ^EndDate = RP.JFOREIGN:RP.JFOREIGN.CFCLC
            */
            assertion = ((report.RPJFOREIGNCFCLCExists == true) && (report.RPJFOREIGNCFCLCStartDate >= report.RPJFOREIGNCFCLCEndDate));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.200009",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Period start date must be prior to Period end date",
                    Location = "/xbrli:xbrl/xbrli:context" + OccurrenceIndex(report.RPJFOREIGNCFCLCOccurrenceIndex) + "/xbrli:period/xbrli:startDate",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.000201" } },
                };
                response.Add(processMessage);
            }
            #endregion // VR.ATO.GEN.000201

            #region VR.ATO.GEN.000201

            /*  VR.ATO.GEN.000201
            Period start date must be prior to Period end date
    
            Legacy Rule Format:
            IF period.startDate >= period.endDate WHERE CONTEXT(ALL)
            RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            ^StartDate >= ^EndDate
    
            Data Elements:
    
            ^StartDate = RP.JFOREIGN:RP.JFOREIGN.CFCUC
    
            ^EndDate = RP.JFOREIGN:RP.JFOREIGN.CFCUC
            */
            assertion = ((report.RPJFOREIGNCFCUCExists == true) && (report.RPJFOREIGNCFCUCStartDate >= report.RPJFOREIGNCFCUCEndDate));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.200009",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Period start date must be prior to Period end date",
                    Location = "/xbrli:xbrl/xbrli:context" + OccurrenceIndex(report.RPJFOREIGNCFCUCOccurrenceIndex) + "/xbrli:period/xbrli:startDate",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.000201" } },
                };
                response.Add(processMessage);
            }
            #endregion // VR.ATO.GEN.000201

            #region VR.ATO.GEN.000201

            /*  VR.ATO.GEN.000201
            Period start date must be prior to Period end date
    
            Legacy Rule Format:
            IF period.startDate >= period.endDate WHERE CONTEXT(ALL)
            RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            ^StartDate >= ^EndDate
    
            Data Elements:
    
            ^StartDate = RP:RP.LowCost
    
            ^EndDate = RP:RP.LowCost
            */
            assertion = ((report.RPLowCostExists == true) && (report.RPLowCostStartDate >= report.RPLowCostEndDate));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.200009",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Period start date must be prior to Period end date",
                    Location = "/xbrli:xbrl/xbrli:context" + OccurrenceIndex(report.RPLowCostOccurrenceIndex) + "/xbrli:period/xbrli:startDate",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.000201" } },
                };
                response.Add(processMessage);
            }
            #endregion // VR.ATO.GEN.000201

            #region VR.ATO.GEN.000201

            /*  VR.ATO.GEN.000201
            Period start date must be prior to Period end date
    
            Legacy Rule Format:
            IF period.startDate >= period.endDate WHERE CONTEXT(ALL)
            RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            ^StartDate >= ^EndDate
    
            Data Elements:
    
            ^StartDate = RP:RP.NonPrim
    
            ^EndDate = RP:RP.NonPrim
            */
            assertion = ((report.RPNonPrimExists == true) && (report.RPNonPrimStartDate >= report.RPNonPrimEndDate));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.200009",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Period start date must be prior to Period end date",
                    Location = "/xbrli:xbrl/xbrli:context" + OccurrenceIndex(report.RPNonPrimOccurrenceIndex) + "/xbrli:period/xbrli:startDate",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.000201" } },
                };
                response.Add(processMessage);
            }
            #endregion // VR.ATO.GEN.000201

            #region VR.ATO.GEN.000201

            /*  VR.ATO.GEN.000201
            Period start date must be prior to Period end date
    
            Legacy Rule Format:
            IF period.startDate >= period.endDate WHERE CONTEXT(ALL)
            RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            ^StartDate >= ^EndDate
    
            Data Elements:
    
            ^StartDate = RP:RP.Prim
    
            ^EndDate = RP:RP.Prim
            */
            assertion = ((report.RPPrimExists == true) && (report.RPPrimStartDate >= report.RPPrimEndDate));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.200009",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Period start date must be prior to Period end date",
                    Location = "/xbrli:xbrl/xbrli:context" + OccurrenceIndex(report.RPPrimOccurrenceIndex) + "/xbrli:period/xbrli:startDate",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.000201" } },
                };
                response.Add(processMessage);
            }
            #endregion // VR.ATO.GEN.000201

            #region VR.ATO.GEN.000201

            /*  VR.ATO.GEN.000201
            Period start date must be prior to Period end date
    
            Legacy Rule Format:
            IF period.startDate >= period.endDate WHERE CONTEXT(ALL)
            RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            ^StartDate >= ^EndDate
    
            Data Elements:
    
            ^StartDate = RP:RP.TOFA
    
            ^EndDate = RP:RP.TOFA
            */
            assertion = ((report.RPTOFAExists == true) && (report.RPTOFAStartDate >= report.RPTOFAEndDate));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.200009",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Period start date must be prior to Period end date",
                    Location = "/xbrli:xbrl/xbrli:context" + OccurrenceIndex(report.RPTOFAOccurrenceIndex) + "/xbrli:period/xbrli:startDate",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.000201" } },
                };
                response.Add(processMessage);
            }
            #endregion // VR.ATO.GEN.000201

            #region VR.ATO.GEN.000209

            /*  VR.ATO.GEN.000209
            Invalid context. The number of Reporting party contexts must equal 1
    
            Legacy Rule Format:
            IF COUNT(RP) <> 1
            RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            Count(^Context) <> 1
    
            Data Elements:
    
            ^Context = RP
            */
            assertion = (Count(report.RPCount) != 1);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.430296",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Invalid context. The number of Reporting party contexts must equal 1",
                    Location = "/xbrli:xbrl/xbrli:context" + OccurrenceIndex(report.RPLastOccurrenceIndex),
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.000209" } },
                };
                response.Add(processMessage);
            }
            #endregion // VR.ATO.GEN.000209

            #region VR.ATO.GEN.000227

            /*  VR.ATO.GEN.000227
            Invalid context. The number of Intermediary contexts must not exceed 1
    
            Legacy Rule Format:
            IF COUNT(INT) > 1
            RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            Count(^Context) > 1
    
            Data Elements:
    
            ^Context = INT
            */
            assertion = (Count(report.INTCount) > 1);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.436280",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Invalid context. The number of Intermediary contexts must not exceed 1",
                    Location = "/xbrli:xbrl/xbrli:context" + OccurrenceIndex(report.INTLastOccurrenceIndex),
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.000227" } },
                };
                response.Add(processMessage);
            }
            #endregion // VR.ATO.GEN.000227

            #region VR.ATO.GEN.000406

            /*  VR.ATO.GEN.000406
            Address line 1 contains incorrect 'care of' reference.  The only acceptable 'care of' reference is C/-
    
            Legacy Rule Format:
            IF (pyde.xx.xx:AddressDetails.Line1.Text <> NULLORBLANK) AND (pyde.xx.xx:AddressDetails.Line1.Text STARTSWITH SET("C/O ","C/ ","Care Of ","CO ")) OR (pyde.xx.xx:AddressDetails.Line1.Text CONTAINS SET(" C/O "," C/ "," Care Of "))
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            (^PTR117 <> BLANK) AND (StartsWithSet(^PTR117,'"C/O ","C/ ","Care Of ","CO "')) OR (ContainsSet(^PTR117, '" C/O "," C/ "," Care Of "'))
    
            Data Elements:
    
            RP:^PTR117 = pyde.02.00:AddressDetails.Line1.Text WHERE [AddressDetails.Usage.Code='BUS'].[AddressDetails.Currency.Code='C']
            */
            assertion = (string.IsNullOrWhiteSpace(report.PTR117) != true && IsMatch(report.PTR117, @"^(C/O |C/ |Care Of |CO )", RegexOptions.IgnoreCase) || IsMatch(report.PTR117, @" C/O | C/ | Care Of ", RegexOptions.IgnoreCase));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.000406",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Address line 1 contains incorrect 'care of' reference.  The only acceptable 'care of' reference is C/-",
                    Location = "/xbrli:xbrl/address2.02.02:AddressDetails" + OccurrenceIndex(report.AddressDetailsCollection_BUS_C_RPOccurrenceIndex) + "/pyde.02.00:AddressDetails.Line1.Text",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.000406" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "PTR117", Value = report.PTR117 });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.GEN.000406

            #region VR.ATO.GEN.000406

            /*  VR.ATO.GEN.000406
            Address line 1 contains incorrect 'care of' reference.  The only acceptable 'care of' reference is C/-
    
            Legacy Rule Format:
            IF (pyde.xx.xx:AddressDetails.Line1.Text <> NULLORBLANK) AND (pyde.xx.xx:AddressDetails.Line1.Text STARTSWITH SET("C/O ","C/ ","Care Of ","CO ")) OR (pyde.xx.xx:AddressDetails.Line1.Text CONTAINS SET(" C/O "," C/ "," Care Of "))
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            (^PTR9 <> BLANK) AND (StartsWithSet(^PTR9,'"C/O ","C/ ","Care Of ","CO "')) OR (ContainsSet(^PTR9, '" C/O "," C/ "," Care Of "'))
    
            Data Elements:
    
            RP:^PTR9 = pyde.02.00:AddressDetails.Line1.Text WHERE [AddressDetails.Usage.Code='POS'].[AddressDetails.Currency.Code='C']
            */
            assertion = (string.IsNullOrWhiteSpace(report.PTR9) != true && IsMatch(report.PTR9, @"^(C/O |C/ |Care Of |CO )", RegexOptions.IgnoreCase) || IsMatch(report.PTR9, @" C/O | C/ | Care Of ", RegexOptions.IgnoreCase));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.000406",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Address line 1 contains incorrect 'care of' reference.  The only acceptable 'care of' reference is C/-",
                    Location = "/xbrli:xbrl/address2.02.02:AddressDetails" + OccurrenceIndex(report.AddressDetailsCollection_POS_C_RPOccurrenceIndex) + "/pyde.02.00:AddressDetails.Line1.Text",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.000406" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "PTR9", Value = report.PTR9 });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.GEN.000406

            #region VR.ATO.GEN.000406

            /*  VR.ATO.GEN.000406
            Address line 1 contains incorrect 'care of' reference.  The only acceptable 'care of' reference is C/-
    
            Legacy Rule Format:
            IF (pyde.xx.xx:AddressDetails.Line1.Text <> NULLORBLANK) AND (pyde.xx.xx:AddressDetails.Line1.Text STARTSWITH SET("C/O ","C/ ","Care Of ","CO ")) OR (pyde.xx.xx:AddressDetails.Line1.Text CONTAINS SET(" C/O "," C/ "," Care Of "))
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            (^PTR14 <> BLANK) AND (StartsWithSet(^PTR14,'"C/O ","C/ ","Care Of ","CO "')) OR (ContainsSet(^PTR14, '" C/O "," C/ "," Care Of "'))
    
            Data Elements:
    
            RP:^PTR14 = pyde.02.00:AddressDetails.Line1.Text WHERE [AddressDetails.Usage.Code='POS'].[AddressDetails.Currency.Code='P']
            */
            assertion = (string.IsNullOrWhiteSpace(report.PTR14) != true && IsMatch(report.PTR14, @"^(C/O |C/ |Care Of |CO )", RegexOptions.IgnoreCase) || IsMatch(report.PTR14, @" C/O | C/ | Care Of ", RegexOptions.IgnoreCase));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.000406",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Address line 1 contains incorrect 'care of' reference.  The only acceptable 'care of' reference is C/-",
                    Location = "/xbrli:xbrl/address2.02.02:AddressDetails" + OccurrenceIndex(report.AddressDetailsCollection_POS_P_RPOccurrenceIndex) + "/pyde.02.00:AddressDetails.Line1.Text",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.000406" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "PTR14", Value = report.PTR14 });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.GEN.000406

            #region VR.ATO.GEN.000408

            /*  VR.ATO.GEN.000408
            Care of' is not valid for address line 2
    
            Legacy Rule Format:
            IF (pyde.xx.xx:AddressDetails.Line2.Text <> NULLORBLANK) AND (pyde.xx.xx:AddressDetails.Line1.Text STARTSWITH SET("C/O ","C/ ","Care Of ","CO ")) OR (pyde.xx.xx:AddressDetails.Line1.Text CONTAINS SET(" C/O "," C/ "," Care Of "))
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            (^PTR213 <> BLANK) AND (StartsWithSet(^PTR117,'"C/O ","C/ ","Care Of ","CO "')) OR (ContainsSet(^PTR117, '" C/O "," C/ "," Care Of "'))
    
            Data Elements:
    
            RP:^PTR213 = pyde.02.00:AddressDetails.Line2.Text WHERE [AddressDetails.Usage.Code='BUS'].[AddressDetails.Currency.Code='C']
    
            RP:^PTR117 = pyde.02.00:AddressDetails.Line1.Text WHERE [AddressDetails.Usage.Code='BUS'].[AddressDetails.Currency.Code='C']
            */
            assertion = (string.IsNullOrWhiteSpace(report.PTR213) != true && IsMatch(report.PTR117, @"^(C/O |C/ |Care Of |CO )", RegexOptions.IgnoreCase) || IsMatch(report.PTR117, @" C/O | C/ | Care Of ", RegexOptions.IgnoreCase));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.000408",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Care of' is not valid for address line 2",
                    Location = "/xbrli:xbrl/address2.02.02:AddressDetails" + OccurrenceIndex(report.AddressDetailsCollection_BUS_C_RPOccurrenceIndex) + "/pyde.02.00:AddressDetails.Line2.Text",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.000408" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "PTR213", Value = report.PTR213 });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "PTR117", Value = report.PTR117 });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.GEN.000408

            #region VR.ATO.GEN.000408

            /*  VR.ATO.GEN.000408
            Care of' is not valid for address line 2
    
            Legacy Rule Format:
            IF (pyde.xx.xx:AddressDetails.Line2.Text <> NULLORBLANK) AND (pyde.xx.xx:AddressDetails.Line1.Text STARTSWITH SET("C/O ","C/ ","Care Of ","CO ")) OR (pyde.xx.xx:AddressDetails.Line1.Text CONTAINS SET(" C/O "," C/ "," Care Of "))
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            (^PTR211 <> BLANK) AND (StartsWithSet(^PTR9,'"C/O ","C/ ","Care Of ","CO "')) OR (ContainsSet(^PTR9, '" C/O "," C/ "," Care Of "'))
    
            Data Elements:
    
            RP:^PTR211 = pyde.02.00:AddressDetails.Line2.Text WHERE [AddressDetails.Usage.Code='POS'].[AddressDetails.Currency.Code='C']
    
            RP:^PTR9 = pyde.02.00:AddressDetails.Line1.Text WHERE [AddressDetails.Usage.Code='POS'].[AddressDetails.Currency.Code='C']
            */
            assertion = (string.IsNullOrWhiteSpace(report.PTR211) != true && IsMatch(report.PTR9, @"^(C/O |C/ |Care Of |CO )", RegexOptions.IgnoreCase) || IsMatch(report.PTR9, @" C/O | C/ | Care Of ", RegexOptions.IgnoreCase));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.000408",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Care of' is not valid for address line 2",
                    Location = "/xbrli:xbrl/address2.02.02:AddressDetails" + OccurrenceIndex(report.AddressDetailsCollection_POS_C_RPOccurrenceIndex) + "/pyde.02.00:AddressDetails.Line2.Text",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.000408" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "PTR211", Value = report.PTR211 });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "PTR9", Value = report.PTR9 });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.GEN.000408

            #region VR.ATO.GEN.000408

            /*  VR.ATO.GEN.000408
            Care of' is not valid for address line 2
    
            Legacy Rule Format:
            IF (pyde.xx.xx:AddressDetails.Line2.Text <> NULLORBLANK) AND (pyde.xx.xx:AddressDetails.Line1.Text STARTSWITH SET("C/O ","C/ ","Care Of ","CO ")) OR (pyde.xx.xx:AddressDetails.Line1.Text CONTAINS SET(" C/O "," C/ "," Care Of "))
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            (^PTR212 <> BLANK) AND (StartsWithSet(^PTR14,'"C/O ","C/ ","Care Of ","CO "')) OR (ContainsSet(^PTR14, '" C/O "," C/ "," Care Of "'))
    
            Data Elements:
    
            RP:^PTR212 = pyde.02.00:AddressDetails.Line2.Text WHERE [AddressDetails.Usage.Code='POS'].[AddressDetails.Currency.Code='P']
    
            RP:^PTR14 = pyde.02.00:AddressDetails.Line1.Text WHERE [AddressDetails.Usage.Code='POS'].[AddressDetails.Currency.Code='P']
            */
            assertion = (string.IsNullOrWhiteSpace(report.PTR212) != true && IsMatch(report.PTR14, @"^(C/O |C/ |Care Of |CO )", RegexOptions.IgnoreCase) || IsMatch(report.PTR14, @" C/O | C/ | Care Of ", RegexOptions.IgnoreCase));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.000408",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Care of' is not valid for address line 2",
                    Location = "/xbrli:xbrl/address2.02.02:AddressDetails" + OccurrenceIndex(report.AddressDetailsCollection_POS_P_RPOccurrenceIndex) + "/pyde.02.00:AddressDetails.Line2.Text",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.000408" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "PTR212", Value = report.PTR212 });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "PTR14", Value = report.PTR14 });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.GEN.000408

            #region VR.ATO.GEN.000458

            /*  VR.ATO.GEN.000458
            Suffix is invalid
    
            Legacy Rule Format:
            IF (pyde.xx.xx:PersonNameDetails.NameSuffix.Text <> NULLORBLANK) AND (pyde.xx.xx:PersonNameDetails.NameSuffix.Text <> SET(DOMAIN(SUFFIX CODES)))
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            (^PTR21 <> BLANK) AND (NotInDomain(^PTR21, 'SUFFIX CODES'))
    
            Data Elements:
    
            RP:^PTR21 = pyde.02.00:PersonNameDetails.NameSuffix.Text WHERE [PersonNameDetails.Usage.Code='MailRecipient'].[PersonNameDetails.PersonNameType.Code='LGL']
            */
            assertion = (string.IsNullOrWhiteSpace(report.PTR21) != true && !(IsMatch(report.PTR21, @"^(ESQ|II|III|IV|JNR|JP|MHA|MHR|MLA|MLC|MP|QC|SNR)$", RegexOptions.IgnoreCase)));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.000458",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Suffix is invalid",
                    Location = "/xbrli:xbrl/ptr.0006.lodge.req.02.00:MailRecipient/prsnstrcnm3.02.01:PersonNameDetails" + OccurrenceIndex(report.MailRecipient_PersonNameDetailsCollection_MailRecipient_LGL_RPOccurrenceIndex) + "/pyde.02.00:PersonNameDetails.NameSuffix.Text",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.000458" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "PTR21", Value = report.PTR21 });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.GEN.000458

            #region VR.ATO.GEN.000459

            /*  VR.ATO.GEN.000459
            Title is invalid
    
            Legacy Rule Format:
            IF (pyde.xx.xx:PersonNameDetails.Title.Text <> NULLORBLANK) AND (pyde.xx.xx:PersonNameDetails.Title.Text <> SET(DOMAIN(TITLE CODES)))
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            (^PTR19 <> BLANK) AND (NotInDomain(^PTR19, 'TITLE CODES'))
    
            Data Elements:
    
            RP:^PTR19 = pyde.02.00:PersonNameDetails.Title.Text WHERE [PersonNameDetails.Usage.Code='MailRecipient'].[PersonNameDetails.PersonNameType.Code='LGL']
            */
            assertion = (string.IsNullOrWhiteSpace(report.PTR19) != true && !(IsMatch(report.PTR19, @"^(2LT|AB|ABBOT|AC|ACM|ADM|AIR CDRE|ALDERMAN|AM|ARCHBISHOP|ARCHDEACON|ASSOC PROF|AVM|BARON|BARONESS|BISHOP|BR|BRIG|CANON|CAPT|CARDINAL|CDRE|CDT|CHAP|CMDR|CMM|COL|CONST|COUNT|COUNTESS|CPL|CPO|DAME|DEACON|DEACONESS|DEAN|DEPUTY SUPT|DR|DUCHESS|DUKE|EARL|EF|FLGOFF|FLT LT|FR|FSGT|GEN|GNR|GP CAPT|HON JUDGE|HON JUST|HON|HRH|INSP|JUDGE|JUST|LAC|LACW|LADY|LBDR|LCPL|LORD|LS|LT CMDR|LT COL|LT GEN|LT|MADAM|MAJ GEN|MAJ|MASTER|MATRON|MAYOR|MAYORESS|MIDN|MISS|MON|MOST REV|MR|MRS|MS|PASTOR|PATRIARCH|PLT OFF|PO|PRIOR|PROF|PTE|RABBI|RADM|RECTOR|REV|RF|RT HON|RT REV BISHOP|RT REV MON|RT REV|SBLT|SEN|SGT|SIR|SMN|SNR CONST|SQN LDR|SR|SSGT|SUPR|SWAMI|TF|VADM|VERY REV|VICAR|VISCOUNT|WG CDR|WO|WO1|WO2)$", RegexOptions.IgnoreCase)));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.000459",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Title is invalid",
                    Location = "/xbrli:xbrl/ptr.0006.lodge.req.02.00:MailRecipient/prsnstrcnm3.02.01:PersonNameDetails" + OccurrenceIndex(report.MailRecipient_PersonNameDetailsCollection_MailRecipient_LGL_RPOccurrenceIndex) + "/pyde.02.00:PersonNameDetails.Title.Text",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.000459" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "PTR19", Value = report.PTR19 });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.GEN.000459

            #region VR.ATO.GEN.001023

            /*  VR.ATO.GEN.001023
            TFN in entity identifier of context does not match the TFN provided for the Reporting Party
    
            Legacy Rule Format:
            IF (entity.identifier.TFN WHERE CONTEXT(ALL)) <> RP:entity.identifier.TFN
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            ^Context <> NULL AND ^OtherTFN <> ^TFN
    
            Data Elements:
    
            ^Context = RP:RP.Closing(Instant)
    
            ^OtherTFN = RP:RP.Closing(Instant)
    
            ^TFN = RP
            */
            assertion = (report.RPClosingInstantExists != false && report.RPClosingInstantIdentifierTFN != report.RPIdentifierTFN);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.001023",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Unexpected TFN mismatch with Reporting Party",
                    LongDescription = @"TFN in entity identifier of context does not match the TFN provided for the Reporting Party",
                    Location = "/xbrli:xbrl/xbrli:context" + OccurrenceIndex(report.RPClosingInstantOccurrenceIndex) + "/xbrli:entity/xbrli:identifier",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.001023" } },
                };
                response.Add(processMessage);
            }
            #endregion // VR.ATO.GEN.001023

            #region VR.ATO.GEN.001023

            /*  VR.ATO.GEN.001023
            TFN in entity identifier of context does not match the TFN provided for the Reporting Party
    
            Legacy Rule Format:
            IF (entity.identifier.TFN WHERE CONTEXT(ALL)) <> RP:entity.identifier.TFN
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            ^Context <> NULL AND ^OtherTFN <> ^TFN
    
            Data Elements:
    
            ^Context = RP:RP.GeneralPool
    
            ^OtherTFN = RP:RP.GeneralPool
    
            ^TFN = RP
            */
            assertion = (report.RPGeneralPoolExists != false && report.RPGeneralPoolIdentifierTFN != report.RPIdentifierTFN);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.001023",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Unexpected TFN mismatch with Reporting Party",
                    LongDescription = @"TFN in entity identifier of context does not match the TFN provided for the Reporting Party",
                    Location = "/xbrli:xbrl/xbrli:context" + OccurrenceIndex(report.RPGeneralPoolOccurrenceIndex) + "/xbrli:entity/xbrli:identifier",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.001023" } },
                };
                response.Add(processMessage);
            }
            #endregion // VR.ATO.GEN.001023

            #region VR.ATO.GEN.001023

            /*  VR.ATO.GEN.001023
            TFN in entity identifier of context does not match the TFN provided for the Reporting Party
    
            Legacy Rule Format:
            IF (entity.identifier.TFN WHERE CONTEXT(ALL)) <> RP:entity.identifier.TFN
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            ^Context <> NULL AND ^OtherTFN <> ^TFN
    
            Data Elements:
    
            ^Context = RP:RP.JAUS
    
            ^OtherTFN = RP:RP.JAUS
    
            ^TFN = RP
            */
            assertion = (report.RPJAUSExists != false && report.RPJAUSIdentifierTFN != report.RPIdentifierTFN);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.001023",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Unexpected TFN mismatch with Reporting Party",
                    LongDescription = @"TFN in entity identifier of context does not match the TFN provided for the Reporting Party",
                    Location = "/xbrli:xbrl/xbrli:context" + OccurrenceIndex(report.RPJAUSOccurrenceIndex) + "/xbrli:entity/xbrli:identifier",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.001023" } },
                };
                response.Add(processMessage);
            }
            #endregion // VR.ATO.GEN.001023

            #region VR.ATO.GEN.001023

            /*  VR.ATO.GEN.001023
            TFN in entity identifier of context does not match the TFN provided for the Reporting Party
    
            Legacy Rule Format:
            IF (entity.identifier.TFN WHERE CONTEXT(ALL)) <> RP:entity.identifier.TFN
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            ^Context <> NULL AND ^OtherTFN <> ^TFN
    
            Data Elements:
    
            ^Context = RP.JAUS:RP.JAUS.NonPrim
    
            ^OtherTFN = RP.JAUS:RP.JAUS.NonPrim
    
            ^TFN = RP
            */
            assertion = (report.RPJAUSNonPrimExists != false && report.RPJAUSNonPrimIdentifierTFN != report.RPIdentifierTFN);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.001023",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Unexpected TFN mismatch with Reporting Party",
                    LongDescription = @"TFN in entity identifier of context does not match the TFN provided for the Reporting Party",
                    Location = "/xbrli:xbrl/xbrli:context" + OccurrenceIndex(report.RPJAUSNonPrimOccurrenceIndex) + "/xbrli:entity/xbrli:identifier",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.001023" } },
                };
                response.Add(processMessage);
            }
            #endregion // VR.ATO.GEN.001023

            #region VR.ATO.GEN.001023

            /*  VR.ATO.GEN.001023
            TFN in entity identifier of context does not match the TFN provided for the Reporting Party
    
            Legacy Rule Format:
            IF (entity.identifier.TFN WHERE CONTEXT(ALL)) <> RP:entity.identifier.TFN
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            ^Context <> NULL AND ^OtherTFN <> ^TFN
    
            Data Elements:
    
            ^Context = RP.JAUS:RP.JAUS.Prim
    
            ^OtherTFN = RP.JAUS:RP.JAUS.Prim
    
            ^TFN = RP
            */
            assertion = (report.RPJAUSPrimExists != false && report.RPJAUSPrimIdentifierTFN != report.RPIdentifierTFN);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.001023",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Unexpected TFN mismatch with Reporting Party",
                    LongDescription = @"TFN in entity identifier of context does not match the TFN provided for the Reporting Party",
                    Location = "/xbrli:xbrl/xbrli:context" + OccurrenceIndex(report.RPJAUSPrimOccurrenceIndex) + "/xbrli:entity/xbrli:identifier",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.001023" } },
                };
                response.Add(processMessage);
            }
            #endregion // VR.ATO.GEN.001023

            #region VR.ATO.GEN.001023

            /*  VR.ATO.GEN.001023
            TFN in entity identifier of context does not match the TFN provided for the Reporting Party
    
            Legacy Rule Format:
            IF (entity.identifier.TFN WHERE CONTEXT(ALL)) <> RP:entity.identifier.TFN
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            ^Context <> NULL AND ^OtherTFN <> ^TFN
    
            Data Elements:
    
            ^Context = RP:RP.JFOREIGN
    
            ^OtherTFN = RP:RP.JFOREIGN
    
            ^TFN = RP
            */
            assertion = (report.RPJFOREIGNExists != false && report.RPJFOREIGNIdentifierTFN != report.RPIdentifierTFN);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.001023",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Unexpected TFN mismatch with Reporting Party",
                    LongDescription = @"TFN in entity identifier of context does not match the TFN provided for the Reporting Party",
                    Location = "/xbrli:xbrl/xbrli:context" + OccurrenceIndex(report.RPJFOREIGNOccurrenceIndex) + "/xbrli:entity/xbrli:identifier",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.001023" } },
                };
                response.Add(processMessage);
            }
            #endregion // VR.ATO.GEN.001023

            #region VR.ATO.GEN.001023

            /*  VR.ATO.GEN.001023
            TFN in entity identifier of context does not match the TFN provided for the Reporting Party
    
            Legacy Rule Format:
            IF (entity.identifier.TFN WHERE CONTEXT(ALL)) <> RP:entity.identifier.TFN
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            ^Context <> NULL AND ^OtherTFN <> ^TFN
    
            Data Elements:
    
            ^Context = RP.JFOREIGN:RP.JFOREIGN.CFCLC
    
            ^OtherTFN = RP.JFOREIGN:RP.JFOREIGN.CFCLC
    
            ^TFN = RP
            */
            assertion = (report.RPJFOREIGNCFCLCExists != false && report.RPJFOREIGNCFCLCIdentifierTFN != report.RPIdentifierTFN);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.001023",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Unexpected TFN mismatch with Reporting Party",
                    LongDescription = @"TFN in entity identifier of context does not match the TFN provided for the Reporting Party",
                    Location = "/xbrli:xbrl/xbrli:context" + OccurrenceIndex(report.RPJFOREIGNCFCLCOccurrenceIndex) + "/xbrli:entity/xbrli:identifier",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.001023" } },
                };
                response.Add(processMessage);
            }
            #endregion // VR.ATO.GEN.001023

            #region VR.ATO.GEN.001023

            /*  VR.ATO.GEN.001023
            TFN in entity identifier of context does not match the TFN provided for the Reporting Party
    
            Legacy Rule Format:
            IF (entity.identifier.TFN WHERE CONTEXT(ALL)) <> RP:entity.identifier.TFN
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            ^Context <> NULL AND ^OtherTFN <> ^TFN
    
            Data Elements:
    
            ^Context = RP.JFOREIGN:RP.JFOREIGN.CFCUC
    
            ^OtherTFN = RP.JFOREIGN:RP.JFOREIGN.CFCUC
    
            ^TFN = RP
            */
            assertion = (report.RPJFOREIGNCFCUCExists != false && report.RPJFOREIGNCFCUCIdentifierTFN != report.RPIdentifierTFN);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.001023",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Unexpected TFN mismatch with Reporting Party",
                    LongDescription = @"TFN in entity identifier of context does not match the TFN provided for the Reporting Party",
                    Location = "/xbrli:xbrl/xbrli:context" + OccurrenceIndex(report.RPJFOREIGNCFCUCOccurrenceIndex) + "/xbrli:entity/xbrli:identifier",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.001023" } },
                };
                response.Add(processMessage);
            }
            #endregion // VR.ATO.GEN.001023

            #region VR.ATO.GEN.001023

            /*  VR.ATO.GEN.001023
            TFN in entity identifier of context does not match the TFN provided for the Reporting Party
    
            Legacy Rule Format:
            IF (entity.identifier.TFN WHERE CONTEXT(ALL)) <> RP:entity.identifier.TFN
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            ^Context <> NULL AND ^OtherTFN <> ^TFN
    
            Data Elements:
    
            ^Context = RP:RP.LowCost
    
            ^OtherTFN = RP:RP.LowCost
    
            ^TFN = RP
            */
            assertion = (report.RPLowCostExists != false && report.RPLowCostIdentifierTFN != report.RPIdentifierTFN);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.001023",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Unexpected TFN mismatch with Reporting Party",
                    LongDescription = @"TFN in entity identifier of context does not match the TFN provided for the Reporting Party",
                    Location = "/xbrli:xbrl/xbrli:context" + OccurrenceIndex(report.RPLowCostOccurrenceIndex) + "/xbrli:entity/xbrli:identifier",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.001023" } },
                };
                response.Add(processMessage);
            }
            #endregion // VR.ATO.GEN.001023

            #region VR.ATO.GEN.001023

            /*  VR.ATO.GEN.001023
            TFN in entity identifier of context does not match the TFN provided for the Reporting Party
    
            Legacy Rule Format:
            IF (entity.identifier.TFN WHERE CONTEXT(ALL)) <> RP:entity.identifier.TFN
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            ^Context <> NULL AND ^OtherTFN <> ^TFN
    
            Data Elements:
    
            ^Context = RP:RP.NonPrim
    
            ^OtherTFN = RP:RP.NonPrim
    
            ^TFN = RP
            */
            assertion = (report.RPNonPrimExists != false && report.RPNonPrimIdentifierTFN != report.RPIdentifierTFN);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.001023",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Unexpected TFN mismatch with Reporting Party",
                    LongDescription = @"TFN in entity identifier of context does not match the TFN provided for the Reporting Party",
                    Location = "/xbrli:xbrl/xbrli:context" + OccurrenceIndex(report.RPNonPrimOccurrenceIndex) + "/xbrli:entity/xbrli:identifier",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.001023" } },
                };
                response.Add(processMessage);
            }
            #endregion // VR.ATO.GEN.001023

            #region VR.ATO.GEN.001023

            /*  VR.ATO.GEN.001023
            TFN in entity identifier of context does not match the TFN provided for the Reporting Party
    
            Legacy Rule Format:
            IF (entity.identifier.TFN WHERE CONTEXT(ALL)) <> RP:entity.identifier.TFN
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            ^Context <> NULL AND ^OtherTFN <> ^TFN
    
            Data Elements:
    
            ^Context = RP:RP.Opening(Instant)
    
            ^OtherTFN = RP:RP.Opening(Instant)
    
            ^TFN = RP
            */
            assertion = (report.RPOpeningInstantExists != false && report.RPOpeningInstantIdentifierTFN != report.RPIdentifierTFN);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.001023",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Unexpected TFN mismatch with Reporting Party",
                    LongDescription = @"TFN in entity identifier of context does not match the TFN provided for the Reporting Party",
                    Location = "/xbrli:xbrl/xbrli:context" + OccurrenceIndex(report.RPOpeningInstantOccurrenceIndex) + "/xbrli:entity/xbrli:identifier",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.001023" } },
                };
                response.Add(processMessage);
            }
            #endregion // VR.ATO.GEN.001023

            #region VR.ATO.GEN.001023

            /*  VR.ATO.GEN.001023
            TFN in entity identifier of context does not match the TFN provided for the Reporting Party
    
            Legacy Rule Format:
            IF (entity.identifier.TFN WHERE CONTEXT(ALL)) <> RP:entity.identifier.TFN
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            ^Context <> NULL AND ^OtherTFN <> ^TFN
    
            Data Elements:
    
            ^Context = RP:RP.Prim
    
            ^OtherTFN = RP:RP.Prim
    
            ^TFN = RP
            */
            assertion = (report.RPPrimExists != false && report.RPPrimIdentifierTFN != report.RPIdentifierTFN);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.001023",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Unexpected TFN mismatch with Reporting Party",
                    LongDescription = @"TFN in entity identifier of context does not match the TFN provided for the Reporting Party",
                    Location = "/xbrli:xbrl/xbrli:context" + OccurrenceIndex(report.RPPrimOccurrenceIndex) + "/xbrli:entity/xbrli:identifier",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.001023" } },
                };
                response.Add(processMessage);
            }
            #endregion // VR.ATO.GEN.001023

            #region VR.ATO.GEN.001023

            /*  VR.ATO.GEN.001023
            TFN in entity identifier of context does not match the TFN provided for the Reporting Party
    
            Legacy Rule Format:
            IF (entity.identifier.TFN WHERE CONTEXT(ALL)) <> RP:entity.identifier.TFN
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            ^Context <> NULL AND ^OtherTFN <> ^TFN
    
            Data Elements:
    
            ^Context = RP:RP.TOFA
    
            ^OtherTFN = RP:RP.TOFA
    
            ^TFN = RP
            */
            assertion = (report.RPTOFAExists != false && report.RPTOFAIdentifierTFN != report.RPIdentifierTFN);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.001023",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Unexpected TFN mismatch with Reporting Party",
                    LongDescription = @"TFN in entity identifier of context does not match the TFN provided for the Reporting Party",
                    Location = "/xbrli:xbrl/xbrli:context" + OccurrenceIndex(report.RPTOFAOccurrenceIndex) + "/xbrli:entity/xbrli:identifier",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.001023" } },
                };
                response.Add(processMessage);
            }
            #endregion // VR.ATO.GEN.001023

            #region VR.ATO.GEN.010028

            /*  VR.ATO.GEN.010028
            If the Amendment Indicator is 'yes' (true), then the Amendment Sequence Number, Amendment Type and Amendment Reason must be present and not blank
    
            Legacy Rule Format:
            IF pyin.xx.xx:Report.Amendment.Indicator = TRUE AND (pyin.xx.xx:Report.AmendmentSequence.Number = NULL OR pyin.xx.xx:Report.AmendmentType.Code = NULLORBLANK OR pyin.xx.xx:Report.AmendmentReason.Text = NULLORBLANK)
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            ^PTR316 = TRUE AND (^PTR319 = NULL OR ^PTR317 = BLANK OR ^PTR318 = BLANK)
    
            Data Elements:
    
            RP:^PTR316 = pyin.02.28:Report.Amendment.Indicator
    
            RP:^PTR319 = pyin.02.08:Report.AmendmentSequence.Number
    
            RP:^PTR317 = pyin.02.08:Report.AmendmentType.Code
    
            RP:^PTR318 = pyin.02.08:Report.AmendmentReason.Text
            */
            assertion = (report.PTR316 == true && (report.PTR319 == null || string.IsNullOrWhiteSpace(report.PTR317) == true || string.IsNullOrWhiteSpace(report.PTR318) == true));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.010028",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"If the return is an amendment, sequence number, type and reason must be present",
                    LongDescription = @"If the Amendment Indicator is 'yes' (true), then the Amendment Sequence Number, Amendment Type and Amendment Reason must be present and not blank.",
                    Location = "/xbrli:xbrl/pyin.02.28:Report.Amendment.Indicator",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.010028" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "PTR316", Value = GetValueOrEmpty(report.PTR316) });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "PTR319", Value = report.PTR319.GetValueOrDefault().ToString() });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "PTR317", Value = report.PTR317 });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "PTR318", Value = report.PTR318 });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.GEN.010028

            #region VR.ATO.GEN.010029

            /*  VR.ATO.GEN.010029
            If an Amendment Sequence Number, Amendment Type or Amendment Reason is present, then the Amendment Indicator must be 'yes' (true)
    
            Legacy Rule Format:
            IF pyin.xx.xx:Report.Amendment.Indicator <> TRUE AND (pyin.xx.xx:Report.AmendmentSequence.Number <> NULL OR pyin.xx.xx:Report.AmendmentType.Code <> NULLORBLANK OR pyin.xx.xx:Report.AmendmentReason.Text <> NULLORBLANK)
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            ^PTR316 <> TRUE AND (^PTR319 <> NULL OR ^PTR317 <> BLANK OR ^PTR318 <> BLANK)
    
            Data Elements:
    
            RP:^PTR316 = pyin.02.28:Report.Amendment.Indicator
    
            RP:^PTR319 = pyin.02.08:Report.AmendmentSequence.Number
    
            RP:^PTR317 = pyin.02.08:Report.AmendmentType.Code
    
            RP:^PTR318 = pyin.02.08:Report.AmendmentReason.Text
            */
            assertion = (report.PTR316 != true && (report.PTR319 != null || string.IsNullOrWhiteSpace(report.PTR317) != true || string.IsNullOrWhiteSpace(report.PTR318) != true));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.010029",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"If the return is an amendment, Amendment Indicator must be 'yes' (true)",
                    LongDescription = @"If an Amendment Sequence Number, Amendment Type or Amendment Reason is present, then the Amendment Indicator must be 'yes' (true)",
                    Location = "/xbrli:xbrl/pyin.02.28:Report.Amendment.Indicator",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.010029" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "PTR316", Value = GetValueOrEmpty(report.PTR316) });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "PTR319", Value = report.PTR319.GetValueOrDefault().ToString() });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "PTR317", Value = report.PTR317 });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "PTR318", Value = report.PTR318 });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.GEN.010029

            #region VR.ATO.GEN.010030

            /*  VR.ATO.GEN.010030
            Amendment sequence number must be a number between 1 and 9
    
            Legacy Rule Format:
            IF pyin.xx.xx:Report.AmendmentSequence.Number <> NULL AND pyin.xx.xx:Report.AmendmentSequence.Number <> SET(1-9)
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            ^PTR319 <> NULL AND (^PTR319 < 1 OR ^PTR319 > 9)
    
            Data Elements:
    
            RP:^PTR319 = pyin.02.08:Report.AmendmentSequence.Number
            */
            assertion = (report.PTR319 != null && (report.PTR319.GetValueOrDefault() < 1 || report.PTR319.GetValueOrDefault() > 9));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.010030",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Amendment sequence number must be a number between 1 and 9",
                    Location = "/xbrli:xbrl/pyin.02.08:Report.AmendmentSequence.Number",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.010030" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "PTR319", Value = report.PTR319.GetValueOrDefault().ToString() });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.GEN.010030

            #region VR.ATO.GEN.300003

            /*  VR.ATO.GEN.300003
            For an Australian address, state or territory must be one of: ACT, NSW, NT, QLD, SA, VIC, WA, TAS, AAT
    
            Legacy Rule Format:
            IF (pyde.xx.xx:AddressDetails.StateOrTerritory.Code <> NULLORBLANK) AND (pyde.xx.xx:AddressDetails.StateOrTerritory.Code <> SET ("ACT","NSW","NT","QLD","SA","VIC","WA","TAS","AAT"))
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            (^PTR119 <> BLANK) AND (NotInSet(^PTR119, '"ACT","NSW","NT","QLD","SA","VIC","WA","TAS","AAT"'))
    
            Data Elements:
    
            RP:^PTR119 = pyde.02.00:AddressDetails.StateOrTerritory.Code WHERE [AddressDetails.Usage.Code='BUS'].[AddressDetails.Currency.Code='C']
            */
            assertion = (string.IsNullOrWhiteSpace(report.PTR119) != true && !(IsMatch(report.PTR119, @"^(ACT|NSW|NT|QLD|SA|VIC|WA|TAS|AAT)$", RegexOptions.IgnoreCase)));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.300003",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"For an Australian address, state or territory must be one of: ACT, NSW, NT, QLD, SA, VIC, WA, TAS, AAT",
                    Location = "/xbrli:xbrl/address2.02.02:AddressDetails" + OccurrenceIndex(report.AddressDetailsCollection_BUS_C_RPOccurrenceIndex) + "/pyde.02.00:AddressDetails.StateOrTerritory.Code",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.300003" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "PTR119", Value = report.PTR119 });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.GEN.300003

            #region VR.ATO.GEN.300003

            /*  VR.ATO.GEN.300003
            For an Australian address, state or territory must be one of: ACT, NSW, NT, QLD, SA, VIC, WA, TAS, AAT
    
            Legacy Rule Format:
            IF (pyde.xx.xx:AddressDetails.StateOrTerritory.Code <> NULLORBLANK) AND (pyde.xx.xx:AddressDetails.StateOrTerritory.Code <> SET ("ACT","NSW","NT","QLD","SA","VIC","WA","TAS","AAT"))
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            (^PTR11 <> BLANK) AND (NotInSet(^PTR11, '"ACT","NSW","NT","QLD","SA","VIC","WA","TAS","AAT"'))
    
            Data Elements:
    
            RP:^PTR11 = pyde.02.00:AddressDetails.StateOrTerritory.Code WHERE [AddressDetails.Usage.Code='POS'].[AddressDetails.Currency.Code='C']
            */
            assertion = (string.IsNullOrWhiteSpace(report.PTR11) != true && !(IsMatch(report.PTR11, @"^(ACT|NSW|NT|QLD|SA|VIC|WA|TAS|AAT)$", RegexOptions.IgnoreCase)));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.300003",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"For an Australian address, state or territory must be one of: ACT, NSW, NT, QLD, SA, VIC, WA, TAS, AAT",
                    Location = "/xbrli:xbrl/address2.02.02:AddressDetails" + OccurrenceIndex(report.AddressDetailsCollection_POS_C_RPOccurrenceIndex) + "/pyde.02.00:AddressDetails.StateOrTerritory.Code",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.300003" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "PTR11", Value = report.PTR11 });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.GEN.300003

            #region VR.ATO.GEN.300003

            /*  VR.ATO.GEN.300003
            For an Australian address, state or territory must be one of: ACT, NSW, NT, QLD, SA, VIC, WA, TAS, AAT
    
            Legacy Rule Format:
            IF (pyde.xx.xx:AddressDetails.StateOrTerritory.Code <> NULLORBLANK) AND (pyde.xx.xx:AddressDetails.StateOrTerritory.Code <> SET ("ACT","NSW","NT","QLD","SA","VIC","WA","TAS","AAT"))
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            (^PTR16 <> BLANK) AND (NotInSet(^PTR16, '"ACT","NSW","NT","QLD","SA","VIC","WA","TAS","AAT"'))
    
            Data Elements:
    
            RP:^PTR16 = pyde.02.00:AddressDetails.StateOrTerritory.Code WHERE [AddressDetails.Usage.Code='POS'].[AddressDetails.Currency.Code='P']
            */
            assertion = (string.IsNullOrWhiteSpace(report.PTR16) != true && !(IsMatch(report.PTR16, @"^(ACT|NSW|NT|QLD|SA|VIC|WA|TAS|AAT)$", RegexOptions.IgnoreCase)));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.300003",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"For an Australian address, state or territory must be one of: ACT, NSW, NT, QLD, SA, VIC, WA, TAS, AAT",
                    Location = "/xbrli:xbrl/address2.02.02:AddressDetails" + OccurrenceIndex(report.AddressDetailsCollection_POS_P_RPOccurrenceIndex) + "/pyde.02.00:AddressDetails.StateOrTerritory.Code",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.300003" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "PTR16", Value = report.PTR16 });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.GEN.300003

            #region VR.ATO.GEN.408010

            /*  VR.ATO.GEN.408010
            Name field cannot contain 'space hyphen space'.
    
            Legacy Rule Format:
            WHERE IN TUPLE(personunstructuredname1.xx.xx:PersonUnstructuredName)
            IF (pyde.xx.xx:PersonUnstructuredName.FullName.Text CONTAINS " - ")
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            (Contains(^PTR242, ' - '))
    
            Data Elements:
    
            INT:^PTR242 = pyde.02.00:PersonUnstructuredName.FullName.Text WHERE [PersonUnstructuredName.Usage.Code='Contact']
            */
            assertion = (report.PTR242 == null ? false : report.PTR242.ToUpperInvariant().Contains(@" - "));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.408010",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Name field cannot contain 'space hyphen space'.",
                    LongDescription = @"Name field cannot contain 'space hyphen space'.",
                    Location = "/xbrli:xbrl/prsnunstrcnm1.02.01:PersonUnstructuredName" + OccurrenceIndex(report.PersonUnstructuredNameCollection_Contact_INTOccurrenceIndex) + "/pyde.02.00:PersonUnstructuredName.FullName.Text",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.408010" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "PTR242", Value = report.PTR242 });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.GEN.408010

            #region VR.ATO.GEN.408010

            /*  VR.ATO.GEN.408010
            Name field cannot contain 'space hyphen space'.
    
            Legacy Rule Format:
            WHERE IN TUPLE(personunstructuredname1.xx.xx:PersonUnstructuredName)
            IF (pyde.xx.xx:PersonUnstructuredName.FullName.Text CONTAINS " - ")
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            (Contains(^PTR314, ' - '))
    
            Data Elements:
    
            INT:^PTR314 = pyde.02.00:PersonUnstructuredName.FullName.Text WHERE [PersonUnstructuredName.Usage.Code='DeclarationSignatory']
            */
            assertion = (report.PTR314 == null ? false : report.PTR314.ToUpperInvariant().Contains(@" - "));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.408010",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Name field cannot contain 'space hyphen space'.",
                    LongDescription = @"Name field cannot contain 'space hyphen space'.",
                    Location = "/xbrli:xbrl/declaration2.02.01:Declaration" + OccurrenceIndex(report.DeclarationCollection_TrueAndCorrect_INTOccurrenceIndex) + "/prsnunstrcnm1.02.01:PersonUnstructuredName" + OccurrenceIndex(report.Declaration_PersonUnstructuredNameCollection_DeclarationSignatory_INTOccurrenceIndex) + "/pyde.02.00:PersonUnstructuredName.FullName.Text",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.408010" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "PTR314", Value = report.PTR314 });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.GEN.408010

            #region VR.ATO.GEN.408010

            /*  VR.ATO.GEN.408010
            Name field cannot contain 'space hyphen space'.
    
            Legacy Rule Format:
            WHERE IN TUPLE(personunstructuredname1.xx.xx:PersonUnstructuredName)
            IF (pyde.xx.xx:PersonUnstructuredName.FullName.Text CONTAINS " - ")
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            (Contains(^PTR264, ' - '))
    
            Data Elements:
    
            RP:^PTR264 = pyde.02.00:PersonUnstructuredName.FullName.Text WHERE [PersonUnstructuredName.Usage.Code='DeclarationSignatory']
            */
            assertion = (report.PTR264 == null ? false : report.PTR264.ToUpperInvariant().Contains(@" - "));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.408010",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Name field cannot contain 'space hyphen space'.",
                    LongDescription = @"Name field cannot contain 'space hyphen space'.",
                    Location = "/xbrli:xbrl/declaration2.02.01:Declaration" + OccurrenceIndex(report.DeclarationCollection_TrueAndCorrect_RPOccurrenceIndex) + "/prsnunstrcnm1.02.01:PersonUnstructuredName" + OccurrenceIndex(report.Declaration_PersonUnstructuredNameCollection_DeclarationSignatory_RPOccurrenceIndex) + "/pyde.02.00:PersonUnstructuredName.FullName.Text",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.408010" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "PTR264", Value = report.PTR264 });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.GEN.408010

            #region VR.ATO.GEN.410002

            /*  VR.ATO.GEN.410002
            Address Line 2 must be present if Address Line 1 contains 'C/-'
    
            Legacy Rule Format:
            WHERE IN TUPLE(addressdetails2.xx.xx:AddressDetails)
            IF ((pyde.xx.xx:AddressDetails.Line1.Text CONTAINS "C/-") AND (pyde.xx.xx:AddressDetails.Line2.Text = NULLORBLANK))  
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            ((Contains(^PTR117, 'C/-')) AND (^PTR213 = BLANK))
    
            Data Elements:
    
            RP:^PTR117 = pyde.02.00:AddressDetails.Line1.Text WHERE [AddressDetails.Usage.Code='BUS'].[AddressDetails.Currency.Code='C']
    
            RP:^PTR213 = pyde.02.00:AddressDetails.Line2.Text WHERE [AddressDetails.Usage.Code='BUS'].[AddressDetails.Currency.Code='C']
            */
            assertion = ((report.PTR117 == null ? false : report.PTR117.ToUpperInvariant().Contains(@"C/-")) && string.IsNullOrWhiteSpace(report.PTR213) == true);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.410002",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Address Line 2 must be present if Address Line 1 contains 'C/-'",
                    Location = "/xbrli:xbrl/address2.02.02:AddressDetails" + OccurrenceIndex(report.AddressDetailsCollection_BUS_C_RPOccurrenceIndex) + "/pyde.02.00:AddressDetails.Line1.Text",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.410002" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "PTR117", Value = report.PTR117 });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "PTR213", Value = report.PTR213 });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.GEN.410002

            #region VR.ATO.GEN.410002

            /*  VR.ATO.GEN.410002
            Address Line 2 must be present if Address Line 1 contains 'C/-'
    
            Legacy Rule Format:
            WHERE IN TUPLE(addressdetails2.xx.xx:AddressDetails)
            IF ((pyde.xx.xx:AddressDetails.Line1.Text CONTAINS "C/-") AND (pyde.xx.xx:AddressDetails.Line2.Text = NULLORBLANK))  
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            ((Contains(^PTR9, 'C/-')) AND (^PTR211 = BLANK))
    
            Data Elements:
    
            RP:^PTR9 = pyde.02.00:AddressDetails.Line1.Text WHERE [AddressDetails.Usage.Code='POS'].[AddressDetails.Currency.Code='C']
    
            RP:^PTR211 = pyde.02.00:AddressDetails.Line2.Text WHERE [AddressDetails.Usage.Code='POS'].[AddressDetails.Currency.Code='C']
            */
            assertion = ((report.PTR9 == null ? false : report.PTR9.ToUpperInvariant().Contains(@"C/-")) && string.IsNullOrWhiteSpace(report.PTR211) == true);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.410002",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Address Line 2 must be present if Address Line 1 contains 'C/-'",
                    Location = "/xbrli:xbrl/address2.02.02:AddressDetails" + OccurrenceIndex(report.AddressDetailsCollection_POS_C_RPOccurrenceIndex) + "/pyde.02.00:AddressDetails.Line1.Text",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.410002" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "PTR9", Value = report.PTR9 });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "PTR211", Value = report.PTR211 });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.GEN.410002

            #region VR.ATO.GEN.410002

            /*  VR.ATO.GEN.410002
            Address Line 2 must be present if Address Line 1 contains 'C/-'
    
            Legacy Rule Format:
            WHERE IN TUPLE(addressdetails2.xx.xx:AddressDetails)
            IF ((pyde.xx.xx:AddressDetails.Line1.Text CONTAINS "C/-") AND (pyde.xx.xx:AddressDetails.Line2.Text = NULLORBLANK))  
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            ((Contains(^PTR14, 'C/-')) AND (^PTR212 = BLANK))
    
            Data Elements:
    
            RP:^PTR14 = pyde.02.00:AddressDetails.Line1.Text WHERE [AddressDetails.Usage.Code='POS'].[AddressDetails.Currency.Code='P']
    
            RP:^PTR212 = pyde.02.00:AddressDetails.Line2.Text WHERE [AddressDetails.Usage.Code='POS'].[AddressDetails.Currency.Code='P']
            */
            assertion = ((report.PTR14 == null ? false : report.PTR14.ToUpperInvariant().Contains(@"C/-")) && string.IsNullOrWhiteSpace(report.PTR212) == true);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.410002",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Address Line 2 must be present if Address Line 1 contains 'C/-'",
                    Location = "/xbrli:xbrl/address2.02.02:AddressDetails" + OccurrenceIndex(report.AddressDetailsCollection_POS_P_RPOccurrenceIndex) + "/pyde.02.00:AddressDetails.Line1.Text",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.410002" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "PTR14", Value = report.PTR14 });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "PTR212", Value = report.PTR212 });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.GEN.410002

            #region VR.ATO.GEN.410008

            /*  VR.ATO.GEN.410008
            Address Line 3 must be left blank
    
            Legacy Rule Format:
            IF (pyde.xx.xx:AddressDetails.Line3.Text <> NULLORBLANK) 
              RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            ^PTR220 <> BLANK
    
            Data Elements:
    
            RP:^PTR220 = pyde.02.00:AddressDetails.Line3.Text WHERE [AddressDetails.Usage.Code='BUS'].[AddressDetails.Currency.Code='C']
            */
            assertion = (string.IsNullOrWhiteSpace(report.PTR220) != true);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.410008",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Address Line 3 must be left blank",
                    LongDescription = @"Lines 3 and 4 of the address are not used on this form",
                    Location = "/xbrli:xbrl/address2.02.02:AddressDetails" + OccurrenceIndex(report.AddressDetailsCollection_BUS_C_RPOccurrenceIndex) + "/pyde.02.00:AddressDetails.Line3.Text",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.410008" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "PTR220", Value = report.PTR220 });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.GEN.410008

            #region VR.ATO.GEN.410008

            /*  VR.ATO.GEN.410008
            Address Line 3 must be left blank
    
            Legacy Rule Format:
            IF (pyde.xx.xx:AddressDetails.Line3.Text <> NULLORBLANK) 
              RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            ^PTR214 <> BLANK
    
            Data Elements:
    
            RP:^PTR214 = pyde.02.00:AddressDetails.Line3.Text WHERE [AddressDetails.Usage.Code='POS'].[AddressDetails.Currency.Code='C']
            */
            assertion = (string.IsNullOrWhiteSpace(report.PTR214) != true);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.410008",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Address Line 3 must be left blank",
                    LongDescription = @"Lines 3 and 4 of the address are not used on this form",
                    Location = "/xbrli:xbrl/address2.02.02:AddressDetails" + OccurrenceIndex(report.AddressDetailsCollection_POS_C_RPOccurrenceIndex) + "/pyde.02.00:AddressDetails.Line3.Text",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.410008" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "PTR214", Value = report.PTR214 });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.GEN.410008

            #region VR.ATO.GEN.410008

            /*  VR.ATO.GEN.410008
            Address Line 3 must be left blank
    
            Legacy Rule Format:
            IF (pyde.xx.xx:AddressDetails.Line3.Text <> NULLORBLANK) 
              RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            ^PTR217 <> BLANK
    
            Data Elements:
    
            RP:^PTR217 = pyde.02.00:AddressDetails.Line3.Text WHERE [AddressDetails.Usage.Code='POS'].[AddressDetails.Currency.Code='P']
            */
            assertion = (string.IsNullOrWhiteSpace(report.PTR217) != true);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.410008",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Address Line 3 must be left blank",
                    LongDescription = @"Lines 3 and 4 of the address are not used on this form",
                    Location = "/xbrli:xbrl/address2.02.02:AddressDetails" + OccurrenceIndex(report.AddressDetailsCollection_POS_P_RPOccurrenceIndex) + "/pyde.02.00:AddressDetails.Line3.Text",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.410008" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "PTR217", Value = report.PTR217 });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.GEN.410008

            #region VR.ATO.GEN.410013

            /*  VR.ATO.GEN.410013
            Address Line 4 must be left blank
    
            Legacy Rule Format:
            IF (pyde.xx.xx:AddressDetails.Line4.Text <> NULLORBLANK)
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            ^PTR221 <> BLANK
    
            Data Elements:
    
            RP:^PTR221 = pyde.02.00:AddressDetails.Line4.Text WHERE [AddressDetails.Usage.Code='BUS'].[AddressDetails.Currency.Code='C']
            */
            assertion = (string.IsNullOrWhiteSpace(report.PTR221) != true);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.410013",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Address Line 4 must be left blank",
                    LongDescription = @"Lines 3 and 4 of the address are not used on this form",
                    Location = "/xbrli:xbrl/address2.02.02:AddressDetails" + OccurrenceIndex(report.AddressDetailsCollection_BUS_C_RPOccurrenceIndex) + "/pyde.02.00:AddressDetails.Line4.Text",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.410013" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "PTR221", Value = report.PTR221 });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.GEN.410013

            #region VR.ATO.GEN.410013

            /*  VR.ATO.GEN.410013
            Address Line 4 must be left blank
    
            Legacy Rule Format:
            IF (pyde.xx.xx:AddressDetails.Line4.Text <> NULLORBLANK)
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            ^PTR215 <> BLANK
    
            Data Elements:
    
            RP:^PTR215 = pyde.02.00:AddressDetails.Line4.Text WHERE [AddressDetails.Usage.Code='POS'].[AddressDetails.Currency.Code='C']
            */
            assertion = (string.IsNullOrWhiteSpace(report.PTR215) != true);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.410013",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Address Line 4 must be left blank",
                    LongDescription = @"Lines 3 and 4 of the address are not used on this form",
                    Location = "/xbrli:xbrl/address2.02.02:AddressDetails" + OccurrenceIndex(report.AddressDetailsCollection_POS_C_RPOccurrenceIndex) + "/pyde.02.00:AddressDetails.Line4.Text",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.410013" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "PTR215", Value = report.PTR215 });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.GEN.410013

            #region VR.ATO.GEN.410013

            /*  VR.ATO.GEN.410013
            Address Line 4 must be left blank
    
            Legacy Rule Format:
            IF (pyde.xx.xx:AddressDetails.Line4.Text <> NULLORBLANK)
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            ^PTR218 <> BLANK
    
            Data Elements:
    
            RP:^PTR218 = pyde.02.00:AddressDetails.Line4.Text WHERE [AddressDetails.Usage.Code='POS'].[AddressDetails.Currency.Code='P']
            */
            assertion = (string.IsNullOrWhiteSpace(report.PTR218) != true);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.410013",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Address Line 4 must be left blank",
                    LongDescription = @"Lines 3 and 4 of the address are not used on this form",
                    Location = "/xbrli:xbrl/address2.02.02:AddressDetails" + OccurrenceIndex(report.AddressDetailsCollection_POS_P_RPOccurrenceIndex) + "/pyde.02.00:AddressDetails.Line4.Text",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.410013" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "PTR218", Value = report.PTR218 });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.GEN.410013

            #region VR.ATO.GEN.410038

            /*  VR.ATO.GEN.410038
            Non-individual name field cannot contain 'Pship', 'P'ship or 'P/Ship'
    
            Legacy Rule Format:
            IF (pyde.xx.xx:OrganisationNameDetails.OrganisationalName.Text STARTSWITH "T/A ") AND (pyde.xx.xx:OrganisationNameDetails.OrganisationalName.Text ENDSWITH SET(" Pship"," P'ship"," P/ship"))
              RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            (StartsWith(^PTR116, 'T/A ')) AND (EndsWithSet(^PTR116, '" Pship"," P'ship"," P/ship"'))
    
            Data Elements:
    
            RP:^PTR116 = pyde.02.00:OrganisationNameDetails.OrganisationalName.Text WHERE [OrganisationNameDetails.OrganisationalNameType.Code='MTR'].[OrganisationNameDetails.Currency.Code='C']
            */
            assertion = (IsMatch(report.PTR116, @"^(T/A )", RegexOptions.IgnoreCase) && IsMatch(report.PTR116, @"( Pship| P'ship| P/ship)$", RegexOptions.IgnoreCase));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.410038",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Non-individual name field cannot start with 'T/A' and end with 'Pship', 'P'ship or 'P/Ship'",
                    Location = "/xbrli:xbrl/orgname2.02.00:OrganisationNameDetails" + OccurrenceIndex(report.OrganisationNameDetailsCollection_MTR_C_RPOccurrenceIndex) + "/pyde.02.00:OrganisationNameDetails.OrganisationalName.Text",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.410038" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "PTR116", Value = report.PTR116 });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.GEN.410038

            #region VR.ATO.GEN.410038

            /*  VR.ATO.GEN.410038
            Non-individual name field cannot contain 'Pship', 'P'ship or 'P/Ship'
    
            Legacy Rule Format:
            IF (pyde.xx.xx:OrganisationNameDetails.OrganisationalName.Text STARTSWITH "T/A ") AND (pyde.xx.xx:OrganisationNameDetails.OrganisationalName.Text ENDSWITH SET(" Pship"," P'ship"," P/ship"))
              RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            (StartsWith(^PTR24, 'T/A ')) AND (EndsWithSet(^PTR24, '" Pship"," P'ship"," P/ship"'))
    
            Data Elements:
    
            RP:^PTR24 = pyde.02.00:OrganisationNameDetails.OrganisationalName.Text WHERE [OrganisationNameDetails.OrganisationalNameType.Code='MN'].[OrganisationNameDetails.Currency.Code='C']
            */
            assertion = (IsMatch(report.PTR24, @"^(T/A )", RegexOptions.IgnoreCase) && IsMatch(report.PTR24, @"( Pship| P'ship| P/ship)$", RegexOptions.IgnoreCase));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.410038",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Non-individual name field cannot start with 'T/A' and end with 'Pship', 'P'ship or 'P/Ship'",
                    Location = "/xbrli:xbrl/ptr.0006.lodge.req.02.00:MailRecipient/orgname2.02.00:OrganisationNameDetails" + OccurrenceIndex(report.MailRecipient_OrganisationNameDetailsCollection_MN_C_RPOccurrenceIndex) + "/pyde.02.00:OrganisationNameDetails.OrganisationalName.Text",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.410038" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "PTR24", Value = report.PTR24 });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.GEN.410038

            #region VR.ATO.GEN.410038

            /*  VR.ATO.GEN.410038
            Non-individual name field cannot contain 'Pship', 'P'ship or 'P/Ship'
    
            Legacy Rule Format:
            IF (pyde.xx.xx:OrganisationNameDetails.OrganisationalName.Text STARTSWITH "T/A ") AND (pyde.xx.xx:OrganisationNameDetails.OrganisationalName.Text ENDSWITH SET(" Pship"," P'ship"," P/ship"))
              RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            (StartsWith(^PTR6, 'T/A ')) AND (EndsWithSet(^PTR6, '" Pship"," P'ship"," P/ship"'))
    
            Data Elements:
    
            RP:^PTR6 = pyde.02.00:OrganisationNameDetails.OrganisationalName.Text WHERE [OrganisationNameDetails.OrganisationalNameType.Code='MN'].[OrganisationNameDetails.Currency.Code='C']
            */
            assertion = (IsMatch(report.PTR6, @"^(T/A )", RegexOptions.IgnoreCase) && IsMatch(report.PTR6, @"( Pship| P'ship| P/ship)$", RegexOptions.IgnoreCase));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.410038",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Non-individual name field cannot start with 'T/A' and end with 'Pship', 'P'ship or 'P/Ship'",
                    Location = "/xbrli:xbrl/orgname2.02.00:OrganisationNameDetails" + OccurrenceIndex(report.OrganisationNameDetailsCollection_MN_C_RPOccurrenceIndex) + "/pyde.02.00:OrganisationNameDetails.OrganisationalName.Text",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.410038" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "PTR6", Value = report.PTR6 });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.GEN.410038

            #region VR.ATO.GEN.410038

            /*  VR.ATO.GEN.410038
            Non-individual name field cannot contain 'Pship', 'P'ship or 'P/Ship'
    
            Legacy Rule Format:
            IF (pyde.xx.xx:OrganisationNameDetails.OrganisationalName.Text STARTSWITH "T/A ") AND (pyde.xx.xx:OrganisationNameDetails.OrganisationalName.Text ENDSWITH SET(" Pship"," P'ship"," P/ship"))
              RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            (StartsWith(^PTR8, 'T/A ')) AND (EndsWithSet(^PTR8, '" Pship"," P'ship"," P/ship"'))
    
            Data Elements:
    
            RP:^PTR8 = pyde.02.00:OrganisationNameDetails.OrganisationalName.Text WHERE [OrganisationNameDetails.Currency.Code='P'].[OrganisationNameDetails.OrganisationalNameType.Code='MN']
            */
            assertion = (IsMatch(report.PTR8, @"^(T/A )", RegexOptions.IgnoreCase) && IsMatch(report.PTR8, @"( Pship| P'ship| P/ship)$", RegexOptions.IgnoreCase));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.410038",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Non-individual name field cannot start with 'T/A' and end with 'Pship', 'P'ship or 'P/Ship'",
                    Location = "/xbrli:xbrl/orgname2.02.00:OrganisationNameDetails" + OccurrenceIndex(report.OrganisationNameDetailsCollection_P_MN_RPOccurrenceIndex) + "/pyde.02.00:OrganisationNameDetails.OrganisationalName.Text",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.410038" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "PTR8", Value = report.PTR8 });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.GEN.410038

            #region VR.ATO.GEN.410039

            /*  VR.ATO.GEN.410039
            Organisation name must contain at least one alpha or numeric character
    
            Legacy Rule Format:
            IF (pyde.xx.xx:OrganisationNameDetails.OrganisationalName.Text  <> NULLORBLANK) AND (pyde.xx.xx:OrganisationNameDetails.OrganisationalName.Text  DOES NOT CONTAIN SET("a-z", "A-Z", "0-9"))
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            (^PTR116  <> BLANK) AND (NotContainsSet(^PTR116, '"a-z", "A-Z", "0-9"'))
    
            Data Elements:
    
            RP:^PTR116 = pyde.02.00:OrganisationNameDetails.OrganisationalName.Text WHERE [OrganisationNameDetails.OrganisationalNameType.Code='MTR'].[OrganisationNameDetails.Currency.Code='C']
            */
            assertion = (string.IsNullOrWhiteSpace(report.PTR116) != true && !(IsMatch(report.PTR116, @"[a-z]|[A-Z]|[0-9]", RegexOptions.IgnoreCase)));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.410039",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Organisation name must contain at least one alpha or numeric character",
                    Location = "/xbrli:xbrl/orgname2.02.00:OrganisationNameDetails" + OccurrenceIndex(report.OrganisationNameDetailsCollection_MTR_C_RPOccurrenceIndex) + "/pyde.02.00:OrganisationNameDetails.OrganisationalName.Text",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.410039" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "PTR116", Value = report.PTR116 });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.GEN.410039

            #region VR.ATO.GEN.410039

            /*  VR.ATO.GEN.410039
            Organisation name must contain at least one alpha or numeric character
    
            Legacy Rule Format:
            IF (pyde.xx.xx:OrganisationNameDetails.OrganisationalName.Text  <> NULLORBLANK) AND (pyde.xx.xx:OrganisationNameDetails.OrganisationalName.Text  DOES NOT CONTAIN SET("a-z", "A-Z", "0-9"))
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            (^PTR24  <> BLANK) AND (NotContainsSet(^PTR24, '"a-z", "A-Z", "0-9"'))
    
            Data Elements:
    
            RP:^PTR24 = pyde.02.00:OrganisationNameDetails.OrganisationalName.Text WHERE [OrganisationNameDetails.OrganisationalNameType.Code='MN'].[OrganisationNameDetails.Currency.Code='C']
            */
            assertion = (string.IsNullOrWhiteSpace(report.PTR24) != true && !(IsMatch(report.PTR24, @"[a-z]|[A-Z]|[0-9]", RegexOptions.IgnoreCase)));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.410039",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Organisation name must contain at least one alpha or numeric character",
                    Location = "/xbrli:xbrl/ptr.0006.lodge.req.02.00:MailRecipient/orgname2.02.00:OrganisationNameDetails" + OccurrenceIndex(report.MailRecipient_OrganisationNameDetailsCollection_MN_C_RPOccurrenceIndex) + "/pyde.02.00:OrganisationNameDetails.OrganisationalName.Text",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.410039" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "PTR24", Value = report.PTR24 });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.GEN.410039

            #region VR.ATO.GEN.410039

            /*  VR.ATO.GEN.410039
            Organisation name must contain at least one alpha or numeric character
    
            Legacy Rule Format:
            IF (pyde.xx.xx:OrganisationNameDetails.OrganisationalName.Text  <> NULLORBLANK) AND (pyde.xx.xx:OrganisationNameDetails.OrganisationalName.Text  DOES NOT CONTAIN SET("a-z", "A-Z", "0-9"))
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            (^PTR6  <> BLANK) AND (NotContainsSet(^PTR6, '"a-z", "A-Z", "0-9"'))
    
            Data Elements:
    
            RP:^PTR6 = pyde.02.00:OrganisationNameDetails.OrganisationalName.Text WHERE [OrganisationNameDetails.OrganisationalNameType.Code='MN'].[OrganisationNameDetails.Currency.Code='C']
            */
            assertion = (string.IsNullOrWhiteSpace(report.PTR6) != true && !(IsMatch(report.PTR6, @"[a-z]|[A-Z]|[0-9]", RegexOptions.IgnoreCase)));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.410039",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Organisation name must contain at least one alpha or numeric character",
                    Location = "/xbrli:xbrl/orgname2.02.00:OrganisationNameDetails" + OccurrenceIndex(report.OrganisationNameDetailsCollection_MN_C_RPOccurrenceIndex) + "/pyde.02.00:OrganisationNameDetails.OrganisationalName.Text",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.410039" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "PTR6", Value = report.PTR6 });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.GEN.410039

            #region VR.ATO.GEN.410039

            /*  VR.ATO.GEN.410039
            Organisation name must contain at least one alpha or numeric character
    
            Legacy Rule Format:
            IF (pyde.xx.xx:OrganisationNameDetails.OrganisationalName.Text  <> NULLORBLANK) AND (pyde.xx.xx:OrganisationNameDetails.OrganisationalName.Text  DOES NOT CONTAIN SET("a-z", "A-Z", "0-9"))
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            (^PTR8  <> BLANK) AND (NotContainsSet(^PTR8, '"a-z", "A-Z", "0-9"'))
    
            Data Elements:
    
            RP:^PTR8 = pyde.02.00:OrganisationNameDetails.OrganisationalName.Text WHERE [OrganisationNameDetails.Currency.Code='P'].[OrganisationNameDetails.OrganisationalNameType.Code='MN']
            */
            assertion = (string.IsNullOrWhiteSpace(report.PTR8) != true && !(IsMatch(report.PTR8, @"[a-z]|[A-Z]|[0-9]", RegexOptions.IgnoreCase)));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.410039",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Organisation name must contain at least one alpha or numeric character",
                    Location = "/xbrli:xbrl/orgname2.02.00:OrganisationNameDetails" + OccurrenceIndex(report.OrganisationNameDetailsCollection_P_MN_RPOccurrenceIndex) + "/pyde.02.00:OrganisationNameDetails.OrganisationalName.Text",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.410039" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "PTR8", Value = report.PTR8 });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.GEN.410039

            #region VR.ATO.GEN.410040

            /*  VR.ATO.GEN.410040
            Family name must contain at least one alphabetical character
    
            Legacy Rule Format:
            IF (pyde.xx.xx:PersonNameDetails.FamilyName.Text <> NULLORBLANK) AND (pyde.xx.xx:PersonNameDetails.FamilyName.Text DOES NOT CONTAIN SET("A-Z","a-z"))
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            (^PTR20 <> BLANK) AND (NotContainsSet(^PTR20, '"A-Z","a-z"'))
    
            Data Elements:
    
            RP:^PTR20 = pyde.02.00:PersonNameDetails.FamilyName.Text WHERE [PersonNameDetails.Usage.Code='MailRecipient'].[PersonNameDetails.PersonNameType.Code='LGL']
            */
            assertion = (string.IsNullOrWhiteSpace(report.PTR20) != true && !(IsMatch(report.PTR20, @"[A-Z]|[a-z]", RegexOptions.IgnoreCase)));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.410040",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Family name must contain at least one alphabetical character",
                    Location = "/xbrli:xbrl/ptr.0006.lodge.req.02.00:MailRecipient/prsnstrcnm3.02.01:PersonNameDetails" + OccurrenceIndex(report.MailRecipient_PersonNameDetailsCollection_MailRecipient_LGL_RPOccurrenceIndex) + "/pyde.02.00:PersonNameDetails.FamilyName.Text",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.410040" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "PTR20", Value = report.PTR20 });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.GEN.410040

            #region VR.ATO.GEN.410063

            /*  VR.ATO.GEN.410063
            Given Name must contain at least one alphabetical character
    
            Legacy Rule Format:
            IF (pyde.xx.xx:PersonNameDetails.GivenName.Text <> NULLORBLANK) AND (pyde.xx.xx:PersonNameDetails.GivenName.Text DOES NOT CONTAIN SET("A-Z","a-z"))
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            (^PTR22 <> BLANK) AND (NotContainsSet(^PTR22, '"A-Z","a-z"'))
    
            Data Elements:
    
            RP:^PTR22 = pyde.02.00:PersonNameDetails.GivenName.Text WHERE [PersonNameDetails.Usage.Code='MailRecipient'].[PersonNameDetails.PersonNameType.Code='LGL']
            */
            assertion = (string.IsNullOrWhiteSpace(report.PTR22) != true && !(IsMatch(report.PTR22, @"[A-Z]|[a-z]", RegexOptions.IgnoreCase)));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.410063",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Given Name must contain at least one alphabetical character",
                    Location = "/xbrli:xbrl/ptr.0006.lodge.req.02.00:MailRecipient/prsnstrcnm3.02.01:PersonNameDetails" + OccurrenceIndex(report.MailRecipient_PersonNameDetailsCollection_MailRecipient_LGL_RPOccurrenceIndex) + "/pyde.02.00:PersonNameDetails.GivenName.Text",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.410063" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "PTR22", Value = report.PTR22 });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.GEN.410063

            #region VR.ATO.GEN.410131

            /*  VR.ATO.GEN.410131
            Other Given Name must contain at least one alphabetical character
    
            Legacy Rule Format:
            IF (pyde.xx.xx:PersonNameDetails.OtherGivenName.Text <> NULLORBLANK) AND (pyde.xx.xx:PersonNameDetails.OtherGivenName.Text DOES NOT CONTAIN SET("A-Z","a-z"))
                 RETURN VALIDATION MESSAGE 
            ENDIF

            Technical Business Rule Format:
            (^PTR23 <> BLANK) AND (NotContainsSet(^PTR23, '"A-Z","a-z"'))
    
            Data Elements:
    
            RP:^PTR23 = pyde.02.00:PersonNameDetails.OtherGivenName.Text WHERE [PersonNameDetails.Usage.Code='MailRecipient'].[PersonNameDetails.PersonNameType.Code='LGL']
            */
            assertion = (string.IsNullOrWhiteSpace(report.PTR23) != true && !(IsMatch(report.PTR23, @"[A-Z]|[a-z]", RegexOptions.IgnoreCase)));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.410131",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Other Given Name must contain at least one alphabetical character",
                    Location = "/xbrli:xbrl/ptr.0006.lodge.req.02.00:MailRecipient/prsnstrcnm3.02.01:PersonNameDetails" + OccurrenceIndex(report.MailRecipient_PersonNameDetailsCollection_MailRecipient_LGL_RPOccurrenceIndex) + "/pyde.02.00:PersonNameDetails.OtherGivenName.Text",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.410131" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "PTR23", Value = report.PTR23 });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.GEN.410131

            #region VR.ATO.GEN.410167

            /*  VR.ATO.GEN.410167
            Overseas address indicator must be present
    
            Legacy Rule Format:
            WHERE IN TUPLE(addressdetails2.xx.xx:AddressDetails)
            IF (pyde.xx.xx:AddressDetails.OverseasAddress.Indicator = NULLORBLANK)
              RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            (^AddressDetails <> NULL) AND ((^PTR222 = BLANK))
    
            Data Elements:
    
            RP:^PTR222 = pyde.02.00:AddressDetails.OverseasAddress.Indicator WHERE [AddressDetails.Usage.Code='BUS'].[AddressDetails.Currency.Code='C']
    
            ^AddressDetails = address2.02.02:AddressDetails
            */
            assertion = (report.AddressDetailsCollection_BUS_C_RPExists != false && report.PTR222 == null);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.410167",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Overseas address indicator must be present",
                    Location = "/xbrli:xbrl/address2.02.02:AddressDetails" + OccurrenceIndex(report.AddressDetailsCollection_BUS_C_RPOccurrenceIndex) + "/pyde.02.00:AddressDetails.OverseasAddress.Indicator",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.410167" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "PTR222", Value = GetValueOrEmpty(report.PTR222) });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.GEN.410167

            #region VR.ATO.GEN.410167

            /*  VR.ATO.GEN.410167
            Overseas address indicator must be present
    
            Legacy Rule Format:
            WHERE IN TUPLE(addressdetails2.xx.xx:AddressDetails)
            IF (pyde.xx.xx:AddressDetails.OverseasAddress.Indicator = NULLORBLANK)
              RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            (^AddressDetails <> NULL) AND ((^PTR216 = BLANK))
    
            Data Elements:
    
            RP:^PTR216 = pyde.02.00:AddressDetails.OverseasAddress.Indicator WHERE [AddressDetails.Usage.Code='POS'].[AddressDetails.Currency.Code='C']
    
            ^AddressDetails = address2.02.02:AddressDetails
            */
            assertion = (report.AddressDetailsCollection_POS_C_RPExists != false && report.PTR216 == null);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.410167",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Overseas address indicator must be present",
                    Location = "/xbrli:xbrl/address2.02.02:AddressDetails" + OccurrenceIndex(report.AddressDetailsCollection_POS_C_RPOccurrenceIndex) + "/pyde.02.00:AddressDetails.OverseasAddress.Indicator",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.410167" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "PTR216", Value = GetValueOrEmpty(report.PTR216) });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.GEN.410167

            #region VR.ATO.GEN.410167

            /*  VR.ATO.GEN.410167
            Overseas address indicator must be present
    
            Legacy Rule Format:
            WHERE IN TUPLE(addressdetails2.xx.xx:AddressDetails)
            IF (pyde.xx.xx:AddressDetails.OverseasAddress.Indicator = NULLORBLANK)
              RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            (^AddressDetails <> NULL) AND ((^PTR219 = BLANK))
    
            Data Elements:
    
            RP:^PTR219 = pyde.02.00:AddressDetails.OverseasAddress.Indicator WHERE [AddressDetails.Usage.Code='POS'].[AddressDetails.Currency.Code='P']
    
            ^AddressDetails = address2.02.02:AddressDetails
            */
            assertion = (report.AddressDetailsCollection_POS_P_RPExists != false && report.PTR219 == null);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.410167",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Overseas address indicator must be present",
                    Location = "/xbrli:xbrl/address2.02.02:AddressDetails" + OccurrenceIndex(report.AddressDetailsCollection_POS_P_RPOccurrenceIndex) + "/pyde.02.00:AddressDetails.OverseasAddress.Indicator",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.410167" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "PTR219", Value = GetValueOrEmpty(report.PTR219) });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.GEN.410167

            #region VR.ATO.GEN.410183

            /*  VR.ATO.GEN.410183
            Person Name Currency Code must be present
    
            Legacy Rule Format:
            WHERE IN TUPLE(personstructuredname3.xx.xx:PersonNameDetails)
            IF (pyde.xx.xx:PersonNameDetails.Currency.Code = NULLORBLANK)
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            (^PNAM <> NULL) AND ((^PTR233 = BLANK))
    
            Data Elements:
    
            RP:^PTR233 = pyde.02.00:PersonNameDetails.Currency.Code WHERE [PersonNameDetails.Usage.Code='MailRecipient'].[PersonNameDetails.PersonNameType.Code='LGL']
    
            ^PNAM = prsnstrcnm3.02.01:PersonNameDetails
            */
            assertion = (report.MailRecipient_PersonNameDetailsCollection_MailRecipient_LGL_RPExists != false && string.IsNullOrWhiteSpace(report.PTR233) == true);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.410183",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Person Name Currency Code must be present",
                    Location = "/xbrli:xbrl/ptr.0006.lodge.req.02.00:MailRecipient/prsnstrcnm3.02.01:PersonNameDetails" + OccurrenceIndex(report.MailRecipient_PersonNameDetailsCollection_MailRecipient_LGL_RPOccurrenceIndex) + "/pyde.02.00:PersonNameDetails.Currency.Code",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.410183" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "PTR233", Value = report.PTR233 });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.GEN.410183

            #region VR.ATO.GEN.410191

            /*  VR.ATO.GEN.410191
            Country code must be present for overseas address
    
            Legacy Rule Format:
            WHERE IN TUPLE(addressdetails2.xx.xx:AddressDetails)
            IF (pyde.xx.xx:AddressDetails.OverseasAddress.Indicator = TRUE) AND (pyde.xx.xx:AddressDetails.Country.Code = NULLORBLANK)
              RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            (^PTR222 = TRUE) AND (^PTR304 = BLANK)
    
            Data Elements:
    
            RP:^PTR304 = pyde.02.08:AddressDetails.Country.Code WHERE [AddressDetails.Usage.Code='BUS'].[AddressDetails.Currency.Code='C']
    
            RP:^PTR222 = pyde.02.00:AddressDetails.OverseasAddress.Indicator WHERE [AddressDetails.Usage.Code='BUS'].[AddressDetails.Currency.Code='C']
            */
            assertion = (report.PTR222 == true && string.IsNullOrWhiteSpace(report.PTR304) == true);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.410191",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Country code must be present for overseas address",
                    LongDescription = @"When the overseas address indicator is 'TRUE', the corresponding Country Code must be supplied",
                    Location = "/xbrli:xbrl/address2.02.02:AddressDetails" + OccurrenceIndex(report.AddressDetailsCollection_BUS_C_RPOccurrenceIndex) + "/pyde.02.08:AddressDetails.Country.Code",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.410191" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "PTR222", Value = GetValueOrEmpty(report.PTR222) });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "PTR304", Value = report.PTR304 });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.GEN.410191

            #region VR.ATO.GEN.410191

            /*  VR.ATO.GEN.410191
            Country code must be present for overseas address
    
            Legacy Rule Format:
            WHERE IN TUPLE(addressdetails2.xx.xx:AddressDetails)
            IF (pyde.xx.xx:AddressDetails.OverseasAddress.Indicator = TRUE) AND (pyde.xx.xx:AddressDetails.Country.Code = NULLORBLANK)
              RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            (^PTR216 = TRUE) AND (^PTR302 = BLANK)
    
            Data Elements:
    
            RP:^PTR302 = pyde.02.08:AddressDetails.Country.Code WHERE [AddressDetails.Usage.Code='POS'].[AddressDetails.Currency.Code='C']
    
            RP:^PTR216 = pyde.02.00:AddressDetails.OverseasAddress.Indicator WHERE [AddressDetails.Usage.Code='POS'].[AddressDetails.Currency.Code='C']
            */
            assertion = (report.PTR216 == true && string.IsNullOrWhiteSpace(report.PTR302) == true);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.410191",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Country code must be present for overseas address",
                    LongDescription = @"When the overseas address indicator is 'TRUE', the corresponding Country Code must be supplied",
                    Location = "/xbrli:xbrl/address2.02.02:AddressDetails" + OccurrenceIndex(report.AddressDetailsCollection_POS_C_RPOccurrenceIndex) + "/pyde.02.08:AddressDetails.Country.Code",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.410191" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "PTR216", Value = GetValueOrEmpty(report.PTR216) });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "PTR302", Value = report.PTR302 });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.GEN.410191

            #region VR.ATO.GEN.410191

            /*  VR.ATO.GEN.410191
            Country code must be present for overseas address
    
            Legacy Rule Format:
            WHERE IN TUPLE(addressdetails2.xx.xx:AddressDetails)
            IF (pyde.xx.xx:AddressDetails.OverseasAddress.Indicator = TRUE) AND (pyde.xx.xx:AddressDetails.Country.Code = NULLORBLANK)
              RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            (^PTR219 = TRUE) AND (^PTR303 = BLANK)
    
            Data Elements:
    
            RP:^PTR303 = pyde.02.08:AddressDetails.Country.Code WHERE [AddressDetails.Usage.Code='POS'].[AddressDetails.Currency.Code='P']
    
            RP:^PTR219 = pyde.02.00:AddressDetails.OverseasAddress.Indicator WHERE [AddressDetails.Usage.Code='POS'].[AddressDetails.Currency.Code='P']
            */
            assertion = (report.PTR219 == true && string.IsNullOrWhiteSpace(report.PTR303) == true);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.410191",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Country code must be present for overseas address",
                    LongDescription = @"When the overseas address indicator is 'TRUE', the corresponding Country Code must be supplied",
                    Location = "/xbrli:xbrl/address2.02.02:AddressDetails" + OccurrenceIndex(report.AddressDetailsCollection_POS_P_RPOccurrenceIndex) + "/pyde.02.08:AddressDetails.Country.Code",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.410191" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "PTR219", Value = GetValueOrEmpty(report.PTR219) });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "PTR303", Value = report.PTR303 });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.GEN.410191

            #region VR.ATO.GEN.410192

            /*  VR.ATO.GEN.410192
            Country code must be a valid code
    
            Legacy Rule Format:
            WHERE IN TUPLE(addressdetails2.xx.xx:AddressDetails)
            IF (pyde.xx.xx:AddressDetails.Country.Code <> NULLORBLANK) AND (pyde.xx.xx:AddressDetails.Country.Code <> SET(DOMAIN(COUNTRY CODES)))
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            (^PTR304 <> BLANK) AND (NotInDomain(^PTR304, 'COUNTRY CODES'))
    
            Data Elements:
    
            RP:^PTR304 = pyde.02.08:AddressDetails.Country.Code WHERE [AddressDetails.Usage.Code='BUS'].[AddressDetails.Currency.Code='C']
            */
            assertion = (string.IsNullOrWhiteSpace(report.PTR304) != true && !(IsMatch(report.PTR304, @"^(ad|ae|af|ag|ai|al|am|an|ao|aq|ar|as|at|au|aw|ax|az|ba|bb|bd|be|bf|bg|bh|bi|bj|bl|bm|bn|bo|bq|br|bs|bt|bv|bw|by|bz|ca|cc|cd|cf|cg|ch|ci|ck|cl|cm|cn|co|cr|cu|cv|cw|cx|cy|cz|de|dj|dk|dm|do|dz|ec|ee|eg|eh|er|es|et|fi|fj|fk|fm|fo|fr|ga|gb|gd|ge|gf|gg|gh|gi|gl|gm|gn|gp|gq|gr|gs|gt|gu|gw|gy|hk|hm|hn|hr|ht|hu|id|ie|il|im|in|io|iq|ir|is|it|je|jm|jo|jp|ke|kg|kh|ki|km|kn|kp|kr|kw|ky|kz|la|lb|lc|li|lk|lr|ls|lt|lu|lv|ly|ma|mc|md|me|mf|mg|mh|mk|ml|mm|mn|mo|mp|mq|mr|ms|mt|mu|mv|mw|mx|my|mz|na|nc|ne|nf|ng|ni|nl|no|np|nr|nu|nz|om|pa|pe|pf|pg|ph|pk|pl|pm|pn|pr|ps|pt|pw|py|qa|re|ro|rs|ru|rw|sa|sb|sc|sd|se|sg|sh|si|sj|sk|sl|sm|sn|so|sr|ss|st|sv|sx|sy|sz|tc|td|tf|tg|th|tj|tk|tl|tm|tn|to|tr|tt|tv|tw|tz|ua|ug|um|us|uy|uz|va|vc|ve|vg|vi|vn|vu|wf|ws|ye|yt|za|zm|zw)$", RegexOptions.IgnoreCase)));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.410192",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Country code must be a valid code",
                    LongDescription = @"Country Code must be one of the two character lower-case codes from Australian Standard 4590, Section 5, Sub-section 5.15",
                    Location = "/xbrli:xbrl/address2.02.02:AddressDetails" + OccurrenceIndex(report.AddressDetailsCollection_BUS_C_RPOccurrenceIndex) + "/pyde.02.08:AddressDetails.Country.Code",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.410192" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "PTR304", Value = report.PTR304 });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.GEN.410192

            #region VR.ATO.GEN.410192

            /*  VR.ATO.GEN.410192
            Country code must be a valid code
    
            Legacy Rule Format:
            WHERE IN TUPLE(addressdetails2.xx.xx:AddressDetails)
            IF (pyde.xx.xx:AddressDetails.Country.Code <> NULLORBLANK) AND (pyde.xx.xx:AddressDetails.Country.Code <> SET(DOMAIN(COUNTRY CODES)))
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            (^PTR302 <> BLANK) AND (NotInDomain(^PTR302, 'COUNTRY CODES'))
    
            Data Elements:
    
            RP:^PTR302 = pyde.02.08:AddressDetails.Country.Code WHERE [AddressDetails.Usage.Code='POS'].[AddressDetails.Currency.Code='C']
            */
            assertion = (string.IsNullOrWhiteSpace(report.PTR302) != true && !(IsMatch(report.PTR302, @"^(ad|ae|af|ag|ai|al|am|an|ao|aq|ar|as|at|au|aw|ax|az|ba|bb|bd|be|bf|bg|bh|bi|bj|bl|bm|bn|bo|bq|br|bs|bt|bv|bw|by|bz|ca|cc|cd|cf|cg|ch|ci|ck|cl|cm|cn|co|cr|cu|cv|cw|cx|cy|cz|de|dj|dk|dm|do|dz|ec|ee|eg|eh|er|es|et|fi|fj|fk|fm|fo|fr|ga|gb|gd|ge|gf|gg|gh|gi|gl|gm|gn|gp|gq|gr|gs|gt|gu|gw|gy|hk|hm|hn|hr|ht|hu|id|ie|il|im|in|io|iq|ir|is|it|je|jm|jo|jp|ke|kg|kh|ki|km|kn|kp|kr|kw|ky|kz|la|lb|lc|li|lk|lr|ls|lt|lu|lv|ly|ma|mc|md|me|mf|mg|mh|mk|ml|mm|mn|mo|mp|mq|mr|ms|mt|mu|mv|mw|mx|my|mz|na|nc|ne|nf|ng|ni|nl|no|np|nr|nu|nz|om|pa|pe|pf|pg|ph|pk|pl|pm|pn|pr|ps|pt|pw|py|qa|re|ro|rs|ru|rw|sa|sb|sc|sd|se|sg|sh|si|sj|sk|sl|sm|sn|so|sr|ss|st|sv|sx|sy|sz|tc|td|tf|tg|th|tj|tk|tl|tm|tn|to|tr|tt|tv|tw|tz|ua|ug|um|us|uy|uz|va|vc|ve|vg|vi|vn|vu|wf|ws|ye|yt|za|zm|zw)$", RegexOptions.IgnoreCase)));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.410192",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Country code must be a valid code",
                    LongDescription = @"Country Code must be one of the two character lower-case codes from Australian Standard 4590, Section 5, Sub-section 5.15",
                    Location = "/xbrli:xbrl/address2.02.02:AddressDetails" + OccurrenceIndex(report.AddressDetailsCollection_POS_C_RPOccurrenceIndex) + "/pyde.02.08:AddressDetails.Country.Code",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.410192" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "PTR302", Value = report.PTR302 });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.GEN.410192

            #region VR.ATO.GEN.410192

            /*  VR.ATO.GEN.410192
            Country code must be a valid code
    
            Legacy Rule Format:
            WHERE IN TUPLE(addressdetails2.xx.xx:AddressDetails)
            IF (pyde.xx.xx:AddressDetails.Country.Code <> NULLORBLANK) AND (pyde.xx.xx:AddressDetails.Country.Code <> SET(DOMAIN(COUNTRY CODES)))
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            (^PTR303 <> BLANK) AND (NotInDomain(^PTR303, 'COUNTRY CODES'))
    
            Data Elements:
    
            RP:^PTR303 = pyde.02.08:AddressDetails.Country.Code WHERE [AddressDetails.Usage.Code='POS'].[AddressDetails.Currency.Code='P']
            */
            assertion = (string.IsNullOrWhiteSpace(report.PTR303) != true && !(IsMatch(report.PTR303, @"^(ad|ae|af|ag|ai|al|am|an|ao|aq|ar|as|at|au|aw|ax|az|ba|bb|bd|be|bf|bg|bh|bi|bj|bl|bm|bn|bo|bq|br|bs|bt|bv|bw|by|bz|ca|cc|cd|cf|cg|ch|ci|ck|cl|cm|cn|co|cr|cu|cv|cw|cx|cy|cz|de|dj|dk|dm|do|dz|ec|ee|eg|eh|er|es|et|fi|fj|fk|fm|fo|fr|ga|gb|gd|ge|gf|gg|gh|gi|gl|gm|gn|gp|gq|gr|gs|gt|gu|gw|gy|hk|hm|hn|hr|ht|hu|id|ie|il|im|in|io|iq|ir|is|it|je|jm|jo|jp|ke|kg|kh|ki|km|kn|kp|kr|kw|ky|kz|la|lb|lc|li|lk|lr|ls|lt|lu|lv|ly|ma|mc|md|me|mf|mg|mh|mk|ml|mm|mn|mo|mp|mq|mr|ms|mt|mu|mv|mw|mx|my|mz|na|nc|ne|nf|ng|ni|nl|no|np|nr|nu|nz|om|pa|pe|pf|pg|ph|pk|pl|pm|pn|pr|ps|pt|pw|py|qa|re|ro|rs|ru|rw|sa|sb|sc|sd|se|sg|sh|si|sj|sk|sl|sm|sn|so|sr|ss|st|sv|sx|sy|sz|tc|td|tf|tg|th|tj|tk|tl|tm|tn|to|tr|tt|tv|tw|tz|ua|ug|um|us|uy|uz|va|vc|ve|vg|vi|vn|vu|wf|ws|ye|yt|za|zm|zw)$", RegexOptions.IgnoreCase)));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.410192",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Country code must be a valid code",
                    LongDescription = @"Country Code must be one of the two character lower-case codes from Australian Standard 4590, Section 5, Sub-section 5.15",
                    Location = "/xbrli:xbrl/address2.02.02:AddressDetails" + OccurrenceIndex(report.AddressDetailsCollection_POS_P_RPOccurrenceIndex) + "/pyde.02.08:AddressDetails.Country.Code",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.410192" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "PTR303", Value = report.PTR303 });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.GEN.410192

            #region VR.ATO.GEN.410194

            /*  VR.ATO.GEN.410194
            Address Line 1 must not exceed 38 characters
    
            Legacy Rule Format:
            IF LENGTH(pyde.xx.xx:AddressDetails.Line1.Text) > 38 
                RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            Length(^PTR117) > 38
    
            Data Elements:
    
            RP:^PTR117 = pyde.02.00:AddressDetails.Line1.Text WHERE [AddressDetails.Usage.Code='BUS'].[AddressDetails.Currency.Code='C']
            */
            assertion = (Length(report.PTR117) > 38);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.410194",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Address Line 1 must not exceed 38 characters",
                    LongDescription = @"Although SBR sets a limit of 50 characters for the Line 1 of an address, the maximum allowable for this form is 38",
                    Location = "/xbrli:xbrl/address2.02.02:AddressDetails" + OccurrenceIndex(report.AddressDetailsCollection_BUS_C_RPOccurrenceIndex) + "/pyde.02.00:AddressDetails.Line1.Text",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.410194" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "PTR117", Value = report.PTR117 });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.GEN.410194

            #region VR.ATO.GEN.410194

            /*  VR.ATO.GEN.410194
            Address Line 1 must not exceed 38 characters
    
            Legacy Rule Format:
            IF LENGTH(pyde.xx.xx:AddressDetails.Line1.Text) > 38 
                RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            Length(^PTR9) > 38
    
            Data Elements:
    
            RP:^PTR9 = pyde.02.00:AddressDetails.Line1.Text WHERE [AddressDetails.Usage.Code='POS'].[AddressDetails.Currency.Code='C']
            */
            assertion = (Length(report.PTR9) > 38);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.410194",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Address Line 1 must not exceed 38 characters",
                    LongDescription = @"Although SBR sets a limit of 50 characters for the Line 1 of an address, the maximum allowable for this form is 38",
                    Location = "/xbrli:xbrl/address2.02.02:AddressDetails" + OccurrenceIndex(report.AddressDetailsCollection_POS_C_RPOccurrenceIndex) + "/pyde.02.00:AddressDetails.Line1.Text",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.410194" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "PTR9", Value = report.PTR9 });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.GEN.410194

            #region VR.ATO.GEN.410194

            /*  VR.ATO.GEN.410194
            Address Line 1 must not exceed 38 characters
    
            Legacy Rule Format:
            IF LENGTH(pyde.xx.xx:AddressDetails.Line1.Text) > 38 
                RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            Length(^PTR14) > 38
    
            Data Elements:
    
            RP:^PTR14 = pyde.02.00:AddressDetails.Line1.Text WHERE [AddressDetails.Usage.Code='POS'].[AddressDetails.Currency.Code='P']
            */
            assertion = (Length(report.PTR14) > 38);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.410194",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Address Line 1 must not exceed 38 characters",
                    LongDescription = @"Although SBR sets a limit of 50 characters for the Line 1 of an address, the maximum allowable for this form is 38",
                    Location = "/xbrli:xbrl/address2.02.02:AddressDetails" + OccurrenceIndex(report.AddressDetailsCollection_POS_P_RPOccurrenceIndex) + "/pyde.02.00:AddressDetails.Line1.Text",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.410194" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "PTR14", Value = report.PTR14 });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.GEN.410194

            #region VR.ATO.GEN.410195

            /*  VR.ATO.GEN.410195
            Address Line 2 must not exceed 38 characters
    
            Legacy Rule Format:
            IF LENGTH(pyde.xx.xx:AddressDetails.Line2.Text) > 38
                 RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            Length(^PTR213) > 38
    
            Data Elements:
    
            RP:^PTR213 = pyde.02.00:AddressDetails.Line2.Text WHERE [AddressDetails.Usage.Code='BUS'].[AddressDetails.Currency.Code='C']
            */
            assertion = (Length(report.PTR213) > 38);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.410195",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Address Line 2 must not exceed 38 characters",
                    LongDescription = @"Although SBR sets a limit of 50 characters for the Line 2 of an address, the maximum allowable for this form is 38",
                    Location = "/xbrli:xbrl/address2.02.02:AddressDetails" + OccurrenceIndex(report.AddressDetailsCollection_BUS_C_RPOccurrenceIndex) + "/pyde.02.00:AddressDetails.Line2.Text",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.410195" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "PTR213", Value = report.PTR213 });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.GEN.410195

            #region VR.ATO.GEN.410195

            /*  VR.ATO.GEN.410195
            Address Line 2 must not exceed 38 characters
    
            Legacy Rule Format:
            IF LENGTH(pyde.xx.xx:AddressDetails.Line2.Text) > 38
                 RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            Length(^PTR211) > 38
    
            Data Elements:
    
            RP:^PTR211 = pyde.02.00:AddressDetails.Line2.Text WHERE [AddressDetails.Usage.Code='POS'].[AddressDetails.Currency.Code='C']
            */
            assertion = (Length(report.PTR211) > 38);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.410195",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Address Line 2 must not exceed 38 characters",
                    LongDescription = @"Although SBR sets a limit of 50 characters for the Line 2 of an address, the maximum allowable for this form is 38",
                    Location = "/xbrli:xbrl/address2.02.02:AddressDetails" + OccurrenceIndex(report.AddressDetailsCollection_POS_C_RPOccurrenceIndex) + "/pyde.02.00:AddressDetails.Line2.Text",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.410195" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "PTR211", Value = report.PTR211 });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.GEN.410195

            #region VR.ATO.GEN.410195

            /*  VR.ATO.GEN.410195
            Address Line 2 must not exceed 38 characters
    
            Legacy Rule Format:
            IF LENGTH(pyde.xx.xx:AddressDetails.Line2.Text) > 38
                 RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            Length(^PTR212) > 38
    
            Data Elements:
    
            RP:^PTR212 = pyde.02.00:AddressDetails.Line2.Text WHERE [AddressDetails.Usage.Code='POS'].[AddressDetails.Currency.Code='P']
            */
            assertion = (Length(report.PTR212) > 38);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.410195",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Address Line 2 must not exceed 38 characters",
                    LongDescription = @"Although SBR sets a limit of 50 characters for the Line 2 of an address, the maximum allowable for this form is 38",
                    Location = "/xbrli:xbrl/address2.02.02:AddressDetails" + OccurrenceIndex(report.AddressDetailsCollection_POS_P_RPOccurrenceIndex) + "/pyde.02.00:AddressDetails.Line2.Text",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.410195" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "PTR212", Value = report.PTR212 });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.GEN.410195

            #region VR.ATO.GEN.410200

            /*  VR.ATO.GEN.410200
            Family name cannot contain "Exec for", "Rep for" or "Trustee for"
    
            Legacy Rule Format:
            IF (pyde.xx.xx:PersonNameDetails.FamilyName.Text = FOUND("Exec for","Rep for","Trustee for"))
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            (FoundSet(^PTR20, '"Exec for","Rep for","Trustee for"'))
    
            Data Elements:
    
            RP:^PTR20 = pyde.02.00:PersonNameDetails.FamilyName.Text WHERE [PersonNameDetails.Usage.Code='MailRecipient'].[PersonNameDetails.PersonNameType.Code='LGL']
            */
            assertion = IsMatch(report.PTR20, @"(^(Exec for|Rep for|Trustee for)$)|(^(.*)( Exec for | Rep for | Trustee for )(.*)$)|(^(Exec for |Rep for |Trustee for )(.*)$)|(^(.*)( Exec for| Rep for| Trustee for)$)", RegexOptions.IgnoreCase);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.000424",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Family name cannot contain ""Exec for"", ""Rep for"" or ""Trustee for""",
                    Location = "/xbrli:xbrl/ptr.0006.lodge.req.02.00:MailRecipient/prsnstrcnm3.02.01:PersonNameDetails" + OccurrenceIndex(report.MailRecipient_PersonNameDetailsCollection_MailRecipient_LGL_RPOccurrenceIndex) + "/pyde.02.00:PersonNameDetails.FamilyName.Text",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.410200" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "PTR20", Value = report.PTR20 });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.GEN.410200

            #region VR.ATO.GEN.410201

            /*  VR.ATO.GEN.410201
            Family name cannot contain "MR", "MRS", "MISS" or "MS"
    
            Legacy Rule Format:
            IF (pyde.xx.xx:PersonNameDetails.FamilyName.Text = FOUND("MR","MRS","MISS","MS"))
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            (FoundSet(^PTR20, '"MR","MRS","MISS","MS"'))
    
            Data Elements:
    
            RP:^PTR20 = pyde.02.00:PersonNameDetails.FamilyName.Text WHERE [PersonNameDetails.Usage.Code='MailRecipient'].[PersonNameDetails.PersonNameType.Code='LGL']
            */
            assertion = IsMatch(report.PTR20, @"(^(MR|MRS|MISS|MS)$)|(^(.*)( MR | MRS | MISS | MS )(.*)$)|(^(MR |MRS |MISS |MS )(.*)$)|(^(.*)( MR| MRS| MISS| MS)$)", RegexOptions.IgnoreCase);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.000426",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Family name cannot contain ""MR"", ""MRS"", ""MISS"" or ""MS""",
                    Location = "/xbrli:xbrl/ptr.0006.lodge.req.02.00:MailRecipient/prsnstrcnm3.02.01:PersonNameDetails" + OccurrenceIndex(report.MailRecipient_PersonNameDetailsCollection_MailRecipient_LGL_RPOccurrenceIndex) + "/pyde.02.00:PersonNameDetails.FamilyName.Text",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.410201" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "PTR20", Value = report.PTR20 });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.GEN.410201

            #region VR.ATO.GEN.410202

            /*  VR.ATO.GEN.410202
            First name cannot contain suffix ESQ, II, III, IV, JNR, JP, MHA, MHR, MLA, MLC, MP, QC or SNR
    
            Legacy Rule Format:
            IF (pyde.xx.xx:PersonNameDetails.GivenName.Text = FOUND("ESQ","II","III","IV","JNR","JP","MHA","MHR","MLA","MLC","MP","QC","SNR"))
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            (FoundSet(^PTR22, '"ESQ","II","III","IV","JNR","JP","MHA","MHR","MLA","MLC","MP","QC","SNR"'))
    
            Data Elements:
    
            RP:^PTR22 = pyde.02.00:PersonNameDetails.GivenName.Text WHERE [PersonNameDetails.Usage.Code='MailRecipient'].[PersonNameDetails.PersonNameType.Code='LGL']
            */
            assertion = IsMatch(report.PTR22, @"(^(ESQ|II|III|IV|JNR|JP|MHA|MHR|MLA|MLC|MP|QC|SNR)$)|(^(.*)( ESQ | II | III | IV | JNR | JP | MHA | MHR | MLA | MLC | MP | QC | SNR )(.*)$)|(^(ESQ |II |III |IV |JNR |JP |MHA |MHR |MLA |MLC |MP |QC |SNR )(.*)$)|(^(.*)( ESQ| II| III| IV| JNR| JP| MHA| MHR| MLA| MLC| MP| QC| SNR)$)", RegexOptions.IgnoreCase);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.000436",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"First name cannot contain suffix ESQ, II, III, IV, JNR, JP, MHA, MHR, MLA, MLC, MP, QC or SNR",
                    Location = "/xbrli:xbrl/ptr.0006.lodge.req.02.00:MailRecipient/prsnstrcnm3.02.01:PersonNameDetails" + OccurrenceIndex(report.MailRecipient_PersonNameDetailsCollection_MailRecipient_LGL_RPOccurrenceIndex) + "/pyde.02.00:PersonNameDetails.GivenName.Text",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.410202" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "PTR22", Value = report.PTR22 });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.GEN.410202

            #region VR.ATO.GEN.410203

            /*  VR.ATO.GEN.410203
            First name cannot contain "Exec for", "Rep for" or "Trustee for"
    
            Legacy Rule Format:
            IF (pyde.xx.xx:PersonNameDetails.GivenName.Text = FOUND("Exec for","Rep for","Trustee for"))
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            (FoundSet(^PTR22, '"Exec for","Rep for","Trustee for"'))
    
            Data Elements:
    
            RP:^PTR22 = pyde.02.00:PersonNameDetails.GivenName.Text WHERE [PersonNameDetails.Usage.Code='MailRecipient'].[PersonNameDetails.PersonNameType.Code='LGL']
            */
            assertion = IsMatch(report.PTR22, @"(^(Exec for|Rep for|Trustee for)$)|(^(.*)( Exec for | Rep for | Trustee for )(.*)$)|(^(Exec for |Rep for |Trustee for )(.*)$)|(^(.*)( Exec for| Rep for| Trustee for)$)", RegexOptions.IgnoreCase);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.000437",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"First name cannot contain ""Exec for"", ""Rep for"" or ""Trustee for""",
                    Location = "/xbrli:xbrl/ptr.0006.lodge.req.02.00:MailRecipient/prsnstrcnm3.02.01:PersonNameDetails" + OccurrenceIndex(report.MailRecipient_PersonNameDetailsCollection_MailRecipient_LGL_RPOccurrenceIndex) + "/pyde.02.00:PersonNameDetails.GivenName.Text",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.410203" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "PTR22", Value = report.PTR22 });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.GEN.410203

            #region VR.ATO.GEN.410204

            /*  VR.ATO.GEN.410204
            First name cannot contain "MR", "MRS", "MISS" or "MS"
    
            Legacy Rule Format:
            IF (pyde.xx.xx:PersonNameDetails.GivenName.Text = FOUND("MR","MRS","MISS","MS"))  
               RETURN VALIDATION MESSAGE 
            ENDIF

            Technical Business Rule Format:
            (FoundSet(^PTR22, '"MR","MRS","MISS","MS"'))
    
            Data Elements:
    
            RP:^PTR22 = pyde.02.00:PersonNameDetails.GivenName.Text WHERE [PersonNameDetails.Usage.Code='MailRecipient'].[PersonNameDetails.PersonNameType.Code='LGL']
            */
            assertion = IsMatch(report.PTR22, @"(^(MR|MRS|MISS|MS)$)|(^(.*)( MR | MRS | MISS | MS )(.*)$)|(^(MR |MRS |MISS |MS )(.*)$)|(^(.*)( MR| MRS| MISS| MS)$)", RegexOptions.IgnoreCase);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.000438",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"First name cannot contain ""MR"", ""MRS"", ""MISS"" or ""MS""",
                    Location = "/xbrli:xbrl/ptr.0006.lodge.req.02.00:MailRecipient/prsnstrcnm3.02.01:PersonNameDetails" + OccurrenceIndex(report.MailRecipient_PersonNameDetailsCollection_MailRecipient_LGL_RPOccurrenceIndex) + "/pyde.02.00:PersonNameDetails.GivenName.Text",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.410204" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "PTR22", Value = report.PTR22 });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.GEN.410204

            #region VR.ATO.GEN.410205

            /*  VR.ATO.GEN.410205
            Address Line 1 must not contain 'AS ABOVE'
    
            Legacy Rule Format:
            IF (pyde.xx.xx:AddressDetails.Line1.Text = FOUND("AS ABOVE"))
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            (FoundSet(^PTR117, '"AS ABOVE"'))
    
            Data Elements:
    
            RP:^PTR117 = pyde.02.00:AddressDetails.Line1.Text WHERE [AddressDetails.Usage.Code='BUS'].[AddressDetails.Currency.Code='C']
            */
            assertion = IsMatch(report.PTR117, @"(^(AS ABOVE)$)|(^(.*)( AS ABOVE )(.*)$)|(^(AS ABOVE )(.*)$)|(^(.*)( AS ABOVE)$)", RegexOptions.IgnoreCase);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.410001",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Address Line 1 must not contain 'AS ABOVE'",
                    Location = "/xbrli:xbrl/address2.02.02:AddressDetails" + OccurrenceIndex(report.AddressDetailsCollection_BUS_C_RPOccurrenceIndex) + "/pyde.02.00:AddressDetails.Line1.Text",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.410205" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "PTR117", Value = report.PTR117 });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.GEN.410205

            #region VR.ATO.GEN.410205

            /*  VR.ATO.GEN.410205
            Address Line 1 must not contain 'AS ABOVE'
    
            Legacy Rule Format:
            IF (pyde.xx.xx:AddressDetails.Line1.Text = FOUND("AS ABOVE"))
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            (FoundSet(^PTR9, '"AS ABOVE"'))
    
            Data Elements:
    
            RP:^PTR9 = pyde.02.00:AddressDetails.Line1.Text WHERE [AddressDetails.Usage.Code='POS'].[AddressDetails.Currency.Code='C']
            */
            assertion = IsMatch(report.PTR9, @"(^(AS ABOVE)$)|(^(.*)( AS ABOVE )(.*)$)|(^(AS ABOVE )(.*)$)|(^(.*)( AS ABOVE)$)", RegexOptions.IgnoreCase);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.410001",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Address Line 1 must not contain 'AS ABOVE'",
                    Location = "/xbrli:xbrl/address2.02.02:AddressDetails" + OccurrenceIndex(report.AddressDetailsCollection_POS_C_RPOccurrenceIndex) + "/pyde.02.00:AddressDetails.Line1.Text",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.410205" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "PTR9", Value = report.PTR9 });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.GEN.410205

            #region VR.ATO.GEN.410205

            /*  VR.ATO.GEN.410205
            Address Line 1 must not contain 'AS ABOVE'
    
            Legacy Rule Format:
            IF (pyde.xx.xx:AddressDetails.Line1.Text = FOUND("AS ABOVE"))
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            (FoundSet(^PTR14, '"AS ABOVE"'))
    
            Data Elements:
    
            RP:^PTR14 = pyde.02.00:AddressDetails.Line1.Text WHERE [AddressDetails.Usage.Code='POS'].[AddressDetails.Currency.Code='P']
            */
            assertion = IsMatch(report.PTR14, @"(^(AS ABOVE)$)|(^(.*)( AS ABOVE )(.*)$)|(^(AS ABOVE )(.*)$)|(^(.*)( AS ABOVE)$)", RegexOptions.IgnoreCase);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.410001",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Address Line 1 must not contain 'AS ABOVE'",
                    Location = "/xbrli:xbrl/address2.02.02:AddressDetails" + OccurrenceIndex(report.AddressDetailsCollection_POS_P_RPOccurrenceIndex) + "/pyde.02.00:AddressDetails.Line1.Text",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.410205" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "PTR14", Value = report.PTR14 });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.GEN.410205

            #region VR.ATO.GEN.410206

            /*  VR.ATO.GEN.410206
            Non Individual Name cannot contain "The trustee", "The Exec", "exec" or "the TTE"
    
            Legacy Rule Format:
            IF (pyde.xx.xx:OrganisationNameDetails.OrganisationalName.Text = FOUND("The trustee","The Exec","exec","The TTE"))
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            (FoundSet(^PTR116, '"The trustee","The Exec","exec","The TTE"'))
    
            Data Elements:
    
            RP:^PTR116 = pyde.02.00:OrganisationNameDetails.OrganisationalName.Text WHERE [OrganisationNameDetails.OrganisationalNameType.Code='MTR'].[OrganisationNameDetails.Currency.Code='C']
            */
            assertion = IsMatch(report.PTR116, @"(^(The trustee|The Exec|exec|The TTE)$)|(^(.*)( The trustee | The Exec | exec | The TTE )(.*)$)|(^(The trustee |The Exec |exec |The TTE )(.*)$)|(^(.*)( The trustee| The Exec| exec| The TTE)$)", RegexOptions.IgnoreCase);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.428042",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Non Individual Name cannot contain ""The trustee"", ""The Exec"", ""exec"" or ""the TTE""",
                    Location = "/xbrli:xbrl/orgname2.02.00:OrganisationNameDetails" + OccurrenceIndex(report.OrganisationNameDetailsCollection_MTR_C_RPOccurrenceIndex) + "/pyde.02.00:OrganisationNameDetails.OrganisationalName.Text",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.410206" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "PTR116", Value = report.PTR116 });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.GEN.410206

            #region VR.ATO.GEN.410206

            /*  VR.ATO.GEN.410206
            Non Individual Name cannot contain "The trustee", "The Exec", "exec" or "the TTE"
    
            Legacy Rule Format:
            IF (pyde.xx.xx:OrganisationNameDetails.OrganisationalName.Text = FOUND("The trustee","The Exec","exec","The TTE"))
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            (FoundSet(^PTR24, '"The trustee","The Exec","exec","The TTE"'))
    
            Data Elements:
    
            RP:^PTR24 = pyde.02.00:OrganisationNameDetails.OrganisationalName.Text WHERE [OrganisationNameDetails.OrganisationalNameType.Code='MN'].[OrganisationNameDetails.Currency.Code='C']
            */
            assertion = IsMatch(report.PTR24, @"(^(The trustee|The Exec|exec|The TTE)$)|(^(.*)( The trustee | The Exec | exec | The TTE )(.*)$)|(^(The trustee |The Exec |exec |The TTE )(.*)$)|(^(.*)( The trustee| The Exec| exec| The TTE)$)", RegexOptions.IgnoreCase);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.428042",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Non Individual Name cannot contain ""The trustee"", ""The Exec"", ""exec"" or ""the TTE""",
                    Location = "/xbrli:xbrl/ptr.0006.lodge.req.02.00:MailRecipient/orgname2.02.00:OrganisationNameDetails" + OccurrenceIndex(report.MailRecipient_OrganisationNameDetailsCollection_MN_C_RPOccurrenceIndex) + "/pyde.02.00:OrganisationNameDetails.OrganisationalName.Text",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.410206" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "PTR24", Value = report.PTR24 });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.GEN.410206

            #region VR.ATO.GEN.410206

            /*  VR.ATO.GEN.410206
            Non Individual Name cannot contain "The trustee", "The Exec", "exec" or "the TTE"
    
            Legacy Rule Format:
            IF (pyde.xx.xx:OrganisationNameDetails.OrganisationalName.Text = FOUND("The trustee","The Exec","exec","The TTE"))
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            (FoundSet(^PTR6, '"The trustee","The Exec","exec","The TTE"'))
    
            Data Elements:
    
            RP:^PTR6 = pyde.02.00:OrganisationNameDetails.OrganisationalName.Text WHERE [OrganisationNameDetails.OrganisationalNameType.Code='MN'].[OrganisationNameDetails.Currency.Code='C']
            */
            assertion = IsMatch(report.PTR6, @"(^(The trustee|The Exec|exec|The TTE)$)|(^(.*)( The trustee | The Exec | exec | The TTE )(.*)$)|(^(The trustee |The Exec |exec |The TTE )(.*)$)|(^(.*)( The trustee| The Exec| exec| The TTE)$)", RegexOptions.IgnoreCase);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.428042",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Non Individual Name cannot contain ""The trustee"", ""The Exec"", ""exec"" or ""the TTE""",
                    Location = "/xbrli:xbrl/orgname2.02.00:OrganisationNameDetails" + OccurrenceIndex(report.OrganisationNameDetailsCollection_MN_C_RPOccurrenceIndex) + "/pyde.02.00:OrganisationNameDetails.OrganisationalName.Text",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.410206" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "PTR6", Value = report.PTR6 });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.GEN.410206

            #region VR.ATO.GEN.410206

            /*  VR.ATO.GEN.410206
            Non Individual Name cannot contain "The trustee", "The Exec", "exec" or "the TTE"
    
            Legacy Rule Format:
            IF (pyde.xx.xx:OrganisationNameDetails.OrganisationalName.Text = FOUND("The trustee","The Exec","exec","The TTE"))
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            (FoundSet(^PTR8, '"The trustee","The Exec","exec","The TTE"'))
    
            Data Elements:
    
            RP:^PTR8 = pyde.02.00:OrganisationNameDetails.OrganisationalName.Text WHERE [OrganisationNameDetails.Currency.Code='P'].[OrganisationNameDetails.OrganisationalNameType.Code='MN']
            */
            assertion = IsMatch(report.PTR8, @"(^(The trustee|The Exec|exec|The TTE)$)|(^(.*)( The trustee | The Exec | exec | The TTE )(.*)$)|(^(The trustee |The Exec |exec |The TTE )(.*)$)|(^(.*)( The trustee| The Exec| exec| The TTE)$)", RegexOptions.IgnoreCase);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.428042",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Non Individual Name cannot contain ""The trustee"", ""The Exec"", ""exec"" or ""the TTE""",
                    Location = "/xbrli:xbrl/orgname2.02.00:OrganisationNameDetails" + OccurrenceIndex(report.OrganisationNameDetailsCollection_P_MN_RPOccurrenceIndex) + "/pyde.02.00:OrganisationNameDetails.OrganisationalName.Text",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.410206" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "PTR8", Value = report.PTR8 });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.GEN.410206

            #region VR.ATO.GEN.410207

            /*  VR.ATO.GEN.410207
            Other given name cannot contain suffix ESQ, II, III, IV, JNR, JP, MHA, MHR, MLA, MLC, MP, QC or SNR
    
            Legacy Rule Format:
            IF (pyde.xx.xx:PersonNameDetails.OtherGivenName.Text = FOUND("ESQ","II","III","IV","JNR","JP","MHA","MHR","MLA","MLC","MP","QC","SNR"))
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            (FoundSet(^PTR23, '"ESQ","II","III","IV","JNR","JP","MHA","MHR","MLA","MLC","MP","QC","SNR"'))
    
            Data Elements:
    
            RP:^PTR23 = pyde.02.00:PersonNameDetails.OtherGivenName.Text WHERE [PersonNameDetails.Usage.Code='MailRecipient'].[PersonNameDetails.PersonNameType.Code='LGL']
            */
            assertion = IsMatch(report.PTR23, @"(^(ESQ|II|III|IV|JNR|JP|MHA|MHR|MLA|MLC|MP|QC|SNR)$)|(^(.*)( ESQ | II | III | IV | JNR | JP | MHA | MHR | MLA | MLC | MP | QC | SNR )(.*)$)|(^(ESQ |II |III |IV |JNR |JP |MHA |MHR |MLA |MLC |MP |QC |SNR )(.*)$)|(^(.*)( ESQ| II| III| IV| JNR| JP| MHA| MHR| MLA| MLC| MP| QC| SNR)$)", RegexOptions.IgnoreCase);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.000448",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Other given name cannot contain suffix ESQ, II, III, IV, JNR, JP, MHA, MHR, MLA, MLC, MP, QC or SNR",
                    Location = "/xbrli:xbrl/ptr.0006.lodge.req.02.00:MailRecipient/prsnstrcnm3.02.01:PersonNameDetails" + OccurrenceIndex(report.MailRecipient_PersonNameDetailsCollection_MailRecipient_LGL_RPOccurrenceIndex) + "/pyde.02.00:PersonNameDetails.OtherGivenName.Text",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.410207" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "PTR23", Value = report.PTR23 });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.GEN.410207

            #region VR.ATO.GEN.410208

            /*  VR.ATO.GEN.410208
            Other given name cannot contain "Exec for", "Rep for" or "Trustee for"
    
            Legacy Rule Format:
            IF (pyde.xx.xx:PersonNameDetails.OtherGivenName.Text = FOUND("Exec for","Rep for","Trustee for"))
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            (FoundSet(^PTR23, '"Exec for","Rep for","Trustee for"'))
    
            Data Elements:
    
            RP:^PTR23 = pyde.02.00:PersonNameDetails.OtherGivenName.Text WHERE [PersonNameDetails.Usage.Code='MailRecipient'].[PersonNameDetails.PersonNameType.Code='LGL']
            */
            assertion = IsMatch(report.PTR23, @"(^(Exec for|Rep for|Trustee for)$)|(^(.*)( Exec for | Rep for | Trustee for )(.*)$)|(^(Exec for |Rep for |Trustee for )(.*)$)|(^(.*)( Exec for| Rep for| Trustee for)$)", RegexOptions.IgnoreCase);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.000449",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Other given name cannot contain ""Exec for"", ""Rep for"" or ""Trustee for""",
                    Location = "/xbrli:xbrl/ptr.0006.lodge.req.02.00:MailRecipient/prsnstrcnm3.02.01:PersonNameDetails" + OccurrenceIndex(report.MailRecipient_PersonNameDetailsCollection_MailRecipient_LGL_RPOccurrenceIndex) + "/pyde.02.00:PersonNameDetails.OtherGivenName.Text",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.410208" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "PTR23", Value = report.PTR23 });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.GEN.410208

            #region VR.ATO.GEN.410209

            /*  VR.ATO.GEN.410209
            Other given name cannot contain "MR", "MRS", "MISS" or "MS"
    
            Legacy Rule Format:
            IF (pyde.xx.xx:PersonNameDetails.OtherGivenName.Text = FOUND("MR","MRS","MISS","MS"))
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            (FoundSet(^PTR23, '"MR","MRS","MISS","MS"'))
    
            Data Elements:
    
            RP:^PTR23 = pyde.02.00:PersonNameDetails.OtherGivenName.Text WHERE [PersonNameDetails.Usage.Code='MailRecipient'].[PersonNameDetails.PersonNameType.Code='LGL']
            */
            assertion = IsMatch(report.PTR23, @"(^(MR|MRS|MISS|MS)$)|(^(.*)( MR | MRS | MISS | MS )(.*)$)|(^(MR |MRS |MISS |MS )(.*)$)|(^(.*)( MR| MRS| MISS| MS)$)", RegexOptions.IgnoreCase);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.000450",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Other given name cannot contain ""MR"", ""MRS"", ""MISS"" or ""MS""",
                    Location = "/xbrli:xbrl/ptr.0006.lodge.req.02.00:MailRecipient/prsnstrcnm3.02.01:PersonNameDetails" + OccurrenceIndex(report.MailRecipient_PersonNameDetailsCollection_MailRecipient_LGL_RPOccurrenceIndex) + "/pyde.02.00:PersonNameDetails.OtherGivenName.Text",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.410209" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "PTR23", Value = report.PTR23 });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.GEN.410209

            #region VR.ATO.GEN.410211

            /*  VR.ATO.GEN.410211
            Overseas address indicator must not be 'TRUE' if Country Code is 'au'
    
            Legacy Rule Format:
            WHERE IN TUPLE(addressdetails2.xx.xx:AddressDetails)
            IF (pyde.xx.xx:AddressDetails.Country.Code = "au") AND  (pyde.xx.xx:AddressDetails.OverseasAddress.Indicator = TRUE)   
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            (^PTR304 = 'au') AND  (^PTR222 = TRUE)
    
            Data Elements:
    
            RP:^PTR304 = pyde.02.08:AddressDetails.Country.Code WHERE [AddressDetails.Usage.Code='BUS'].[AddressDetails.Currency.Code='C']
    
            RP:^PTR222 = pyde.02.00:AddressDetails.OverseasAddress.Indicator WHERE [AddressDetails.Usage.Code='BUS'].[AddressDetails.Currency.Code='C']
            */
            assertion = (report.PTR304 == @"au" && report.PTR222 == true);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.410211",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Overseas address indicator must not be 'TRUE' if Country Code is 'au'",
                    LongDescription = @"The overseas address indicator must be 'FALSE' if the Country Code is 'au' - representing Australia. If the address is an overseas address please supply the correct Country Code",
                    Location = "/xbrli:xbrl/address2.02.02:AddressDetails" + OccurrenceIndex(report.AddressDetailsCollection_BUS_C_RPOccurrenceIndex) + "/pyde.02.08:AddressDetails.Country.Code",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.410211" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "PTR304", Value = report.PTR304 });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "PTR222", Value = GetValueOrEmpty(report.PTR222) });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.GEN.410211

            #region VR.ATO.GEN.410211

            /*  VR.ATO.GEN.410211
            Overseas address indicator must not be 'TRUE' if Country Code is 'au'
    
            Legacy Rule Format:
            WHERE IN TUPLE(addressdetails2.xx.xx:AddressDetails)
            IF (pyde.xx.xx:AddressDetails.Country.Code = "au") AND  (pyde.xx.xx:AddressDetails.OverseasAddress.Indicator = TRUE)   
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            (^PTR302 = 'au') AND  (^PTR216 = TRUE)
    
            Data Elements:
    
            RP:^PTR302 = pyde.02.08:AddressDetails.Country.Code WHERE [AddressDetails.Usage.Code='POS'].[AddressDetails.Currency.Code='C']
    
            RP:^PTR216 = pyde.02.00:AddressDetails.OverseasAddress.Indicator WHERE [AddressDetails.Usage.Code='POS'].[AddressDetails.Currency.Code='C']
            */
            assertion = (report.PTR302 == @"au" && report.PTR216 == true);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.410211",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Overseas address indicator must not be 'TRUE' if Country Code is 'au'",
                    LongDescription = @"The overseas address indicator must be 'FALSE' if the Country Code is 'au' - representing Australia. If the address is an overseas address please supply the correct Country Code",
                    Location = "/xbrli:xbrl/address2.02.02:AddressDetails" + OccurrenceIndex(report.AddressDetailsCollection_POS_C_RPOccurrenceIndex) + "/pyde.02.08:AddressDetails.Country.Code",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.410211" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "PTR302", Value = report.PTR302 });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "PTR216", Value = GetValueOrEmpty(report.PTR216) });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.GEN.410211

            #region VR.ATO.GEN.410211

            /*  VR.ATO.GEN.410211
            Overseas address indicator must not be 'TRUE' if Country Code is 'au'
    
            Legacy Rule Format:
            WHERE IN TUPLE(addressdetails2.xx.xx:AddressDetails)
            IF (pyde.xx.xx:AddressDetails.Country.Code = "au") AND  (pyde.xx.xx:AddressDetails.OverseasAddress.Indicator = TRUE)   
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            (^PTR303 = 'au') AND  (^PTR219 = TRUE)
    
            Data Elements:
    
            RP:^PTR303 = pyde.02.08:AddressDetails.Country.Code WHERE [AddressDetails.Usage.Code='POS'].[AddressDetails.Currency.Code='P']
    
            RP:^PTR219 = pyde.02.00:AddressDetails.OverseasAddress.Indicator WHERE [AddressDetails.Usage.Code='POS'].[AddressDetails.Currency.Code='P']
            */
            assertion = (report.PTR303 == @"au" && report.PTR219 == true);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.410211",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Overseas address indicator must not be 'TRUE' if Country Code is 'au'",
                    LongDescription = @"The overseas address indicator must be 'FALSE' if the Country Code is 'au' - representing Australia. If the address is an overseas address please supply the correct Country Code",
                    Location = "/xbrli:xbrl/address2.02.02:AddressDetails" + OccurrenceIndex(report.AddressDetailsCollection_POS_P_RPOccurrenceIndex) + "/pyde.02.08:AddressDetails.Country.Code",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.410211" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "PTR303", Value = report.PTR303 });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "PTR219", Value = GetValueOrEmpty(report.PTR219) });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.GEN.410211

            #region VR.ATO.GEN.410212

            /*  VR.ATO.GEN.410212
            If an overseas Country Code is used, the Overseas Address indicator must be 'TRUE'
    
            Legacy Rule Format:
            WHERE IN TUPLE(addressdetails2.xx.xx:AddressDetails)IF (pyde.xx.xx:AddressDetails.Country.Code <> NULLORBLANK) AND (pyde.xx.xx:AddressDetails.Country.Code <> "au") AND  (pyde.xx.xx:AddressDetails.OverseasAddress.Indicator = FALSE)
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            (^PTR304 <> BLANK) AND (^PTR304 <> 'au') AND  (^PTR222 = FALSE)
    
            Data Elements:
    
            RP:^PTR304 = pyde.02.08:AddressDetails.Country.Code WHERE [AddressDetails.Usage.Code='BUS'].[AddressDetails.Currency.Code='C']
    
            RP:^PTR222 = pyde.02.00:AddressDetails.OverseasAddress.Indicator WHERE [AddressDetails.Usage.Code='BUS'].[AddressDetails.Currency.Code='C']
            */
            assertion = (string.IsNullOrWhiteSpace(report.PTR304) != true && report.PTR304 != @"au" && report.PTR222 == false);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.410212",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"If an overseas Country Code is used, the Overseas Address indicator must be 'TRUE'",
                    LongDescription = @"If the address is an Australian address, the overseas address indicator must be set to 'FALSE' and you do not need to supply a Country Code. If the address is an overseas address, the overseas address indicator must be set to 'TRUE' and you must supply a Country Code other than 'au'",
                    Location = "/xbrli:xbrl/address2.02.02:AddressDetails" + OccurrenceIndex(report.AddressDetailsCollection_BUS_C_RPOccurrenceIndex) + "/pyde.02.08:AddressDetails.Country.Code",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.410212" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "PTR304", Value = report.PTR304 });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "PTR222", Value = GetValueOrEmpty(report.PTR222) });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.GEN.410212

            #region VR.ATO.GEN.410212

            /*  VR.ATO.GEN.410212
            If an overseas Country Code is used, the Overseas Address indicator must be 'TRUE'
    
            Legacy Rule Format:
            WHERE IN TUPLE(addressdetails2.xx.xx:AddressDetails)IF (pyde.xx.xx:AddressDetails.Country.Code <> NULLORBLANK) AND (pyde.xx.xx:AddressDetails.Country.Code <> "au") AND  (pyde.xx.xx:AddressDetails.OverseasAddress.Indicator = FALSE)
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            (^PTR302 <> BLANK) AND (^PTR302 <> 'au') AND  (^PTR216 = FALSE)
    
            Data Elements:
    
            RP:^PTR302 = pyde.02.08:AddressDetails.Country.Code WHERE [AddressDetails.Usage.Code='POS'].[AddressDetails.Currency.Code='C']
    
            RP:^PTR216 = pyde.02.00:AddressDetails.OverseasAddress.Indicator WHERE [AddressDetails.Usage.Code='POS'].[AddressDetails.Currency.Code='C']
            */
            assertion = (string.IsNullOrWhiteSpace(report.PTR302) != true && report.PTR302 != @"au" && report.PTR216 == false);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.410212",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"If an overseas Country Code is used, the Overseas Address indicator must be 'TRUE'",
                    LongDescription = @"If the address is an Australian address, the overseas address indicator must be set to 'FALSE' and you do not need to supply a Country Code. If the address is an overseas address, the overseas address indicator must be set to 'TRUE' and you must supply a Country Code other than 'au'",
                    Location = "/xbrli:xbrl/address2.02.02:AddressDetails" + OccurrenceIndex(report.AddressDetailsCollection_POS_C_RPOccurrenceIndex) + "/pyde.02.08:AddressDetails.Country.Code",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.410212" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "PTR302", Value = report.PTR302 });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "PTR216", Value = GetValueOrEmpty(report.PTR216) });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.GEN.410212

            #region VR.ATO.GEN.410212

            /*  VR.ATO.GEN.410212
            If an overseas Country Code is used, the Overseas Address indicator must be 'TRUE'
    
            Legacy Rule Format:
            WHERE IN TUPLE(addressdetails2.xx.xx:AddressDetails)IF (pyde.xx.xx:AddressDetails.Country.Code <> NULLORBLANK) AND (pyde.xx.xx:AddressDetails.Country.Code <> "au") AND  (pyde.xx.xx:AddressDetails.OverseasAddress.Indicator = FALSE)
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            (^PTR303 <> BLANK) AND (^PTR303 <> 'au') AND  (^PTR219 = FALSE)
    
            Data Elements:
    
            RP:^PTR303 = pyde.02.08:AddressDetails.Country.Code WHERE [AddressDetails.Usage.Code='POS'].[AddressDetails.Currency.Code='P']
    
            RP:^PTR219 = pyde.02.00:AddressDetails.OverseasAddress.Indicator WHERE [AddressDetails.Usage.Code='POS'].[AddressDetails.Currency.Code='P']
            */
            assertion = (string.IsNullOrWhiteSpace(report.PTR303) != true && report.PTR303 != @"au" && report.PTR219 == false);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.410212",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"If an overseas Country Code is used, the Overseas Address indicator must be 'TRUE'",
                    LongDescription = @"If the address is an Australian address, the overseas address indicator must be set to 'FALSE' and you do not need to supply a Country Code. If the address is an overseas address, the overseas address indicator must be set to 'TRUE' and you must supply a Country Code other than 'au'",
                    Location = "/xbrli:xbrl/address2.02.02:AddressDetails" + OccurrenceIndex(report.AddressDetailsCollection_POS_P_RPOccurrenceIndex) + "/pyde.02.08:AddressDetails.Country.Code",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.410212" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "PTR303", Value = report.PTR303 });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "PTR219", Value = GetValueOrEmpty(report.PTR219) });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.GEN.410212

            #region VR.ATO.GEN.410213

            /*  VR.ATO.GEN.410213
            Country Name must be left blank
    
            Legacy Rule Format:
            IF (pyde.xx.xx:AddressDetails.CountryName.Text <> NULLORBLANK)
                RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            ^PTR263 <> BLANK
    
            Data Elements:
    
            RP:^PTR263 = pyde.02.08:AddressDetails.CountryName.Text WHERE [AddressDetails.Usage.Code='BUS'].[AddressDetails.Currency.Code='C']
            */
            assertion = (string.IsNullOrWhiteSpace(report.PTR263) != true);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.410213",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Country Name must be left blank",
                    LongDescription = @"For SBR submissions to the ATO, the Country Name must be left blank. To supply the country for an overseas address please supply a Country Code",
                    Location = "/xbrli:xbrl/address2.02.02:AddressDetails" + OccurrenceIndex(report.AddressDetailsCollection_BUS_C_RPOccurrenceIndex) + "/pyde.02.08:AddressDetails.CountryName.Text",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.410213" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "PTR263", Value = report.PTR263 });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.GEN.410213

            #region VR.ATO.GEN.410213

            /*  VR.ATO.GEN.410213
            Country Name must be left blank
    
            Legacy Rule Format:
            IF (pyde.xx.xx:AddressDetails.CountryName.Text <> NULLORBLANK)
                RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            ^PTR13 <> BLANK
    
            Data Elements:
    
            RP:^PTR13 = pyde.02.08:AddressDetails.CountryName.Text WHERE [AddressDetails.Usage.Code='POS'].[AddressDetails.Currency.Code='C']
            */
            assertion = (string.IsNullOrWhiteSpace(report.PTR13) != true);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.410213",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Country Name must be left blank",
                    LongDescription = @"For SBR submissions to the ATO, the Country Name must be left blank. To supply the country for an overseas address please supply a Country Code",
                    Location = "/xbrli:xbrl/address2.02.02:AddressDetails" + OccurrenceIndex(report.AddressDetailsCollection_POS_C_RPOccurrenceIndex) + "/pyde.02.08:AddressDetails.CountryName.Text",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.410213" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "PTR13", Value = report.PTR13 });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.GEN.410213

            #region VR.ATO.GEN.410213

            /*  VR.ATO.GEN.410213
            Country Name must be left blank
    
            Legacy Rule Format:
            IF (pyde.xx.xx:AddressDetails.CountryName.Text <> NULLORBLANK)
                RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            ^PTR18 <> BLANK
    
            Data Elements:
    
            RP:^PTR18 = pyde.02.08:AddressDetails.CountryName.Text WHERE [AddressDetails.Usage.Code='POS'].[AddressDetails.Currency.Code='P']
            */
            assertion = (string.IsNullOrWhiteSpace(report.PTR18) != true);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.410213",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Country Name must be left blank",
                    LongDescription = @"For SBR submissions to the ATO, the Country Name must be left blank. To supply the country for an overseas address please supply a Country Code",
                    Location = "/xbrli:xbrl/address2.02.02:AddressDetails" + OccurrenceIndex(report.AddressDetailsCollection_POS_P_RPOccurrenceIndex) + "/pyde.02.08:AddressDetails.CountryName.Text",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.410213" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "PTR18", Value = report.PTR18 });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.GEN.410213

            #region VR.ATO.GEN.410214

            /*  VR.ATO.GEN.410214
            Address Line 1 must be present
    
            Legacy Rule Format:
            WHERE IN TUPLE(addressdetails2.xx.xx:AddressDetails)
            IF (pyde.xx.xx:AddressDetails.Line1.Text = NULLORBLANK)
              RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            (^AddressDetails <> NULL) AND ((^PTR117 = BLANK))
    
            Data Elements:
    
            RP:^PTR117 = pyde.02.00:AddressDetails.Line1.Text WHERE [AddressDetails.Usage.Code='BUS'].[AddressDetails.Currency.Code='C']
    
            ^AddressDetails = address2.02.02:AddressDetails
            */
            assertion = (report.AddressDetailsCollection_BUS_C_RPExists != false && string.IsNullOrWhiteSpace(report.PTR117) == true);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.410214",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Address Line 1 must be present",
                    Location = "/xbrli:xbrl/address2.02.02:AddressDetails" + OccurrenceIndex(report.AddressDetailsCollection_BUS_C_RPOccurrenceIndex) + "/pyde.02.00:AddressDetails.Line1.Text",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.410214" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "PTR117", Value = report.PTR117 });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.GEN.410214

            #region VR.ATO.GEN.410214

            /*  VR.ATO.GEN.410214
            Address Line 1 must be present
    
            Legacy Rule Format:
            WHERE IN TUPLE(addressdetails2.xx.xx:AddressDetails)
            IF (pyde.xx.xx:AddressDetails.Line1.Text = NULLORBLANK)
              RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            (^AddressDetails <> NULL) AND ((^PTR9 = BLANK))
    
            Data Elements:
    
            RP:^PTR9 = pyde.02.00:AddressDetails.Line1.Text WHERE [AddressDetails.Usage.Code='POS'].[AddressDetails.Currency.Code='C']
    
            ^AddressDetails = address2.02.02:AddressDetails
            */
            assertion = (report.AddressDetailsCollection_POS_C_RPExists != false && string.IsNullOrWhiteSpace(report.PTR9) == true);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.410214",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Address Line 1 must be present",
                    Location = "/xbrli:xbrl/address2.02.02:AddressDetails" + OccurrenceIndex(report.AddressDetailsCollection_POS_C_RPOccurrenceIndex) + "/pyde.02.00:AddressDetails.Line1.Text",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.410214" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "PTR9", Value = report.PTR9 });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.GEN.410214

            #region VR.ATO.GEN.410214

            /*  VR.ATO.GEN.410214
            Address Line 1 must be present
    
            Legacy Rule Format:
            WHERE IN TUPLE(addressdetails2.xx.xx:AddressDetails)
            IF (pyde.xx.xx:AddressDetails.Line1.Text = NULLORBLANK)
              RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            (^AddressDetails <> NULL) AND ((^PTR14 = BLANK))
    
            Data Elements:
    
            RP:^PTR14 = pyde.02.00:AddressDetails.Line1.Text WHERE [AddressDetails.Usage.Code='POS'].[AddressDetails.Currency.Code='P']
    
            ^AddressDetails = address2.02.02:AddressDetails
            */
            assertion = (report.AddressDetailsCollection_POS_P_RPExists != false && string.IsNullOrWhiteSpace(report.PTR14) == true);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.410214",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Address Line 1 must be present",
                    Location = "/xbrli:xbrl/address2.02.02:AddressDetails" + OccurrenceIndex(report.AddressDetailsCollection_POS_P_RPOccurrenceIndex) + "/pyde.02.00:AddressDetails.Line1.Text",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.410214" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "PTR14", Value = report.PTR14 });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.GEN.410214

            #region VR.ATO.GEN.410215

            /*  VR.ATO.GEN.410215
            Organisation name must be present
    
            Legacy Rule Format:
            WHERE IN TUPLE(organisationname2.xx.xx:OrganisationNameDetails)
            IF (pyde.xx.xx:OrganisationNameDetails.OrganisationalName.Text = NULLORBLANK)
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            (^ORG <> NULL) AND ((^PTR116 = BLANK))
    
            Data Elements:
    
            RP:^PTR116 = pyde.02.00:OrganisationNameDetails.OrganisationalName.Text WHERE [OrganisationNameDetails.OrganisationalNameType.Code='MTR'].[OrganisationNameDetails.Currency.Code='C']
    
            ^ORG = orgname2.02.00:OrganisationNameDetails
            */
            assertion = (report.OrganisationNameDetailsCollection_MTR_C_RPExists != false && string.IsNullOrWhiteSpace(report.PTR116) == true);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.410215",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Organisation name must be present",
                    Location = "/xbrli:xbrl/orgname2.02.00:OrganisationNameDetails" + OccurrenceIndex(report.OrganisationNameDetailsCollection_MTR_C_RPOccurrenceIndex) + "/pyde.02.00:OrganisationNameDetails.OrganisationalName.Text",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.410215" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "PTR116", Value = report.PTR116 });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.GEN.410215

            #region VR.ATO.GEN.410215

            /*  VR.ATO.GEN.410215
            Organisation name must be present
    
            Legacy Rule Format:
            WHERE IN TUPLE(organisationname2.xx.xx:OrganisationNameDetails)
            IF (pyde.xx.xx:OrganisationNameDetails.OrganisationalName.Text = NULLORBLANK)
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            (^ORG <> NULL) AND ((^PTR24 = BLANK))
    
            Data Elements:
    
            RP:^PTR24 = pyde.02.00:OrganisationNameDetails.OrganisationalName.Text WHERE [OrganisationNameDetails.OrganisationalNameType.Code='MN'].[OrganisationNameDetails.Currency.Code='C']
    
            ^ORG = orgname2.02.00:OrganisationNameDetails
            */
            assertion = (report.MailRecipient_OrganisationNameDetailsCollection_MN_C_RPExists != false && string.IsNullOrWhiteSpace(report.PTR24) == true);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.410215",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Organisation name must be present",
                    Location = "/xbrli:xbrl/ptr.0006.lodge.req.02.00:MailRecipient/orgname2.02.00:OrganisationNameDetails" + OccurrenceIndex(report.MailRecipient_OrganisationNameDetailsCollection_MN_C_RPOccurrenceIndex) + "/pyde.02.00:OrganisationNameDetails.OrganisationalName.Text",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.410215" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "PTR24", Value = report.PTR24 });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.GEN.410215

            #region VR.ATO.GEN.410215

            /*  VR.ATO.GEN.410215
            Organisation name must be present
    
            Legacy Rule Format:
            WHERE IN TUPLE(organisationname2.xx.xx:OrganisationNameDetails)
            IF (pyde.xx.xx:OrganisationNameDetails.OrganisationalName.Text = NULLORBLANK)
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            (^ORG <> NULL) AND ((^PTR6 = BLANK))
    
            Data Elements:
    
            RP:^PTR6 = pyde.02.00:OrganisationNameDetails.OrganisationalName.Text WHERE [OrganisationNameDetails.OrganisationalNameType.Code='MN'].[OrganisationNameDetails.Currency.Code='C']
    
            ^ORG = orgname2.02.00:OrganisationNameDetails
            */
            assertion = (report.OrganisationNameDetailsCollection_MN_C_RPExists != false && string.IsNullOrWhiteSpace(report.PTR6) == true);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.410215",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Organisation name must be present",
                    Location = "/xbrli:xbrl/orgname2.02.00:OrganisationNameDetails" + OccurrenceIndex(report.OrganisationNameDetailsCollection_MN_C_RPOccurrenceIndex) + "/pyde.02.00:OrganisationNameDetails.OrganisationalName.Text",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.410215" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "PTR6", Value = report.PTR6 });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.GEN.410215

            #region VR.ATO.GEN.410215

            /*  VR.ATO.GEN.410215
            Organisation name must be present
    
            Legacy Rule Format:
            WHERE IN TUPLE(organisationname2.xx.xx:OrganisationNameDetails)
            IF (pyde.xx.xx:OrganisationNameDetails.OrganisationalName.Text = NULLORBLANK)
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            (^ORG <> NULL) AND ((^PTR8 = BLANK))
    
            Data Elements:
    
            RP:^PTR8 = pyde.02.00:OrganisationNameDetails.OrganisationalName.Text WHERE [OrganisationNameDetails.Currency.Code='P'].[OrganisationNameDetails.OrganisationalNameType.Code='MN']
    
            ^ORG = orgname2.02.00:OrganisationNameDetails
            */
            assertion = (report.OrganisationNameDetailsCollection_P_MN_RPExists != false && string.IsNullOrWhiteSpace(report.PTR8) == true);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.410215",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Organisation name must be present",
                    Location = "/xbrli:xbrl/orgname2.02.00:OrganisationNameDetails" + OccurrenceIndex(report.OrganisationNameDetailsCollection_P_MN_RPOccurrenceIndex) + "/pyde.02.00:OrganisationNameDetails.OrganisationalName.Text",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.410215" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "PTR8", Value = report.PTR8 });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.GEN.410215

            #region VR.ATO.GEN.410217

            /*  VR.ATO.GEN.410217
            Family Name must be present
    
            Legacy Rule Format:
            WHERE IN TUPLE(personstructuredname3.xx.xx:PersonNameDetails)
            IF (pyde.xx.xx:PersonNameDetails.FamilyName.Text = NULLORBLANK)
                RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            (^PNAM <> NULL) AND ((^PTR20 = BLANK))
    
            Data Elements:
    
            RP:^PTR20 = pyde.02.00:PersonNameDetails.FamilyName.Text WHERE [PersonNameDetails.Usage.Code='MailRecipient'].[PersonNameDetails.PersonNameType.Code='LGL']
    
            ^PNAM = prsnstrcnm3.02.01:PersonNameDetails
            */
            assertion = (report.MailRecipient_PersonNameDetailsCollection_MailRecipient_LGL_RPExists != false && string.IsNullOrWhiteSpace(report.PTR20) == true);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.410217",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Family Name must be present",
                    Location = "/xbrli:xbrl/ptr.0006.lodge.req.02.00:MailRecipient/prsnstrcnm3.02.01:PersonNameDetails" + OccurrenceIndex(report.MailRecipient_PersonNameDetailsCollection_MailRecipient_LGL_RPOccurrenceIndex) + "/pyde.02.00:PersonNameDetails.FamilyName.Text",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.410217" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "PTR20", Value = report.PTR20 });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.GEN.410217

            #region VR.ATO.GEN.410218

            /*  VR.ATO.GEN.410218
            FirstName must be entered if OtherGivenNames is entered
    
            Legacy Rule Format:
            WHERE IN TUPLE(personstructuredname3.xx.xx:PersonNameDetails) 
            IF (pyde.xx.xx:PersonNameDetails.OtherGivenName.Text <> NULLORBLANK) AND (pyde.xx.xx:PersonNameDetails.GivenName.Text = NULLORBLANK) 
                RETURN VALIDATION MESSAGE 
             ENDIF

            Technical Business Rule Format:
            (^PNAM <> NULL) AND ((^PTR23 <> BLANK) AND (^PTR22 = BLANK))
    
            Data Elements:
    
            RP:^PTR22 = pyde.02.00:PersonNameDetails.GivenName.Text WHERE [PersonNameDetails.Usage.Code='MailRecipient'].[PersonNameDetails.PersonNameType.Code='LGL']
    
            RP:^PTR23 = pyde.02.00:PersonNameDetails.OtherGivenName.Text WHERE [PersonNameDetails.Usage.Code='MailRecipient'].[PersonNameDetails.PersonNameType.Code='LGL']
    
            ^PNAM = prsnstrcnm3.02.01:PersonNameDetails
            */
            assertion = (report.MailRecipient_PersonNameDetailsCollection_MailRecipient_LGL_RPExists != false && (string.IsNullOrWhiteSpace(report.PTR23) != true && string.IsNullOrWhiteSpace(report.PTR22) == true));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.410218",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"FirstName must be entered if OtherGivenNames is entered",
                    Location = "/xbrli:xbrl/ptr.0006.lodge.req.02.00:MailRecipient/prsnstrcnm3.02.01:PersonNameDetails" + OccurrenceIndex(report.MailRecipient_PersonNameDetailsCollection_MailRecipient_LGL_RPOccurrenceIndex) + "/pyde.02.00:PersonNameDetails.GivenName.Text",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.410218" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "PTR23", Value = report.PTR23 });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "PTR22", Value = report.PTR22 });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.GEN.410218

            #region VR.ATO.GEN.428016

            /*  VR.ATO.GEN.428016
            Tax File Number has failed the algorithm check.
    
            Legacy Rule Format:
            IF (RP:entity.identifier.TFN <> NULLORBLANK) AND (TFNALGORITHM (RP:entity.identifier.TFN) = FALSE)
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            (^TFN <> BLANK) AND (FailsTFNAlgorithm(^TFN))
    
            Data Elements:
    
            ^TFN = RP
            */
            assertion = (string.IsNullOrWhiteSpace(report.RPIdentifierTFN) != true && FailsTFNAlgorithm(report.RPIdentifierTFN));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.428016",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Tax File Number has failed the algorithm check",
                    Location = "/xbrli:xbrl/xbrli:context" + OccurrenceIndex(report.RPOccurrenceIndex) + "/xbrli:entity/xbrli:identifier",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.428016" } },
                };
                response.Add(processMessage);
            }
            #endregion // VR.ATO.GEN.428016

            #region VR.ATO.GEN.428025

            /*  VR.ATO.GEN.428025
            Previous postal address is only required when an address change has occurred since the last return
    
            Legacy Rule Format:
            IF (RP:pyde.xx.xx:AddressDetails.Line1.Text WHERE((TUPLE ELEMENT pyde.xx.xx:AddressDetails.Usage.Code = "POS") AND (TUPLE ELEMENT pyde.xx.xx:AddressDetails.Currency.Code = "C"))) = (RP:pyde.xx.xx:AddressDetails.Line1.Text WHERE((TUPLE ELEMENT pyde.xx.xx:AddressDetails.Usage.Code = "POS") AND (TUPLE ELEMENT pyde.xx.xx:AddressDetails.Currency.Code = "P")))
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            ^PTR14 = ^PTR9
    
            Data Elements:
    
            RP:^PTR14 = pyde.02.00:AddressDetails.Line1.Text WHERE [AddressDetails.Usage.Code='POS'].[AddressDetails.Currency.Code='P']
    
            RP:^PTR9 = pyde.02.00:AddressDetails.Line1.Text WHERE [AddressDetails.Usage.Code='POS'].[AddressDetails.Currency.Code='C']
            */
            assertion = (report.PTR14 == report.PTR9);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.428025",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Previous postal address is only required when an address change has occurred since the last return",
                    Location = "/xbrli:xbrl/address2.02.02:AddressDetails" + OccurrenceIndex(report.AddressDetailsCollection_POS_P_RPOccurrenceIndex) + "/pyde.02.00:AddressDetails.Line1.Text",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.428025" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "PTR14", Value = report.PTR14 });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "PTR9", Value = report.PTR9 });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.GEN.428025

            #region VR.ATO.GEN.428230

            /*  VR.ATO.GEN.428230
            Suburb/Town/City cannot contain state code
    
            Legacy Rule Format:
            IF (pyde.xx.xx:AddressDetails.LocalityName.Text = FOUND("QLD","NSW","VIC","SA","WA","NT","ACT","TAS"))
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            (FoundSet(^PTR118, '"QLD","NSW","VIC","SA","WA","NT","ACT","TAS"'))
    
            Data Elements:
    
            RP:^PTR118 = pyde.02.00:AddressDetails.LocalityName.Text WHERE [AddressDetails.Usage.Code='BUS'].[AddressDetails.Currency.Code='C']
            */
            assertion = IsMatch(report.PTR118, @"(^(QLD|NSW|VIC|SA|WA|NT|ACT|TAS)$)|(^(.*)( QLD | NSW | VIC | SA | WA | NT | ACT | TAS )(.*)$)|(^(QLD |NSW |VIC |SA |WA |NT |ACT |TAS )(.*)$)|(^(.*)( QLD| NSW| VIC| SA| WA| NT| ACT| TAS)$)", RegexOptions.IgnoreCase);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.000411",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Suburb/Town/City cannot contain state code",
                    Location = "/xbrli:xbrl/address2.02.02:AddressDetails" + OccurrenceIndex(report.AddressDetailsCollection_BUS_C_RPOccurrenceIndex) + "/pyde.02.00:AddressDetails.LocalityName.Text",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.428230" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "PTR118", Value = report.PTR118 });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.GEN.428230

            #region VR.ATO.GEN.428230

            /*  VR.ATO.GEN.428230
            Suburb/Town/City cannot contain state code
    
            Legacy Rule Format:
            IF (pyde.xx.xx:AddressDetails.LocalityName.Text = FOUND("QLD","NSW","VIC","SA","WA","NT","ACT","TAS"))
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            (FoundSet(^PTR10, '"QLD","NSW","VIC","SA","WA","NT","ACT","TAS"'))
    
            Data Elements:
    
            RP:^PTR10 = pyde.02.00:AddressDetails.LocalityName.Text WHERE [AddressDetails.Usage.Code='POS'].[AddressDetails.Currency.Code='C']
            */
            assertion = IsMatch(report.PTR10, @"(^(QLD|NSW|VIC|SA|WA|NT|ACT|TAS)$)|(^(.*)( QLD | NSW | VIC | SA | WA | NT | ACT | TAS )(.*)$)|(^(QLD |NSW |VIC |SA |WA |NT |ACT |TAS )(.*)$)|(^(.*)( QLD| NSW| VIC| SA| WA| NT| ACT| TAS)$)", RegexOptions.IgnoreCase);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.000411",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Suburb/Town/City cannot contain state code",
                    Location = "/xbrli:xbrl/address2.02.02:AddressDetails" + OccurrenceIndex(report.AddressDetailsCollection_POS_C_RPOccurrenceIndex) + "/pyde.02.00:AddressDetails.LocalityName.Text",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.428230" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "PTR10", Value = report.PTR10 });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.GEN.428230

            #region VR.ATO.GEN.428230

            /*  VR.ATO.GEN.428230
            Suburb/Town/City cannot contain state code
    
            Legacy Rule Format:
            IF (pyde.xx.xx:AddressDetails.LocalityName.Text = FOUND("QLD","NSW","VIC","SA","WA","NT","ACT","TAS"))
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            (FoundSet(^PTR15, '"QLD","NSW","VIC","SA","WA","NT","ACT","TAS"'))
    
            Data Elements:
    
            RP:^PTR15 = pyde.02.00:AddressDetails.LocalityName.Text WHERE [AddressDetails.Usage.Code='POS'].[AddressDetails.Currency.Code='P']
            */
            assertion = IsMatch(report.PTR15, @"(^(QLD|NSW|VIC|SA|WA|NT|ACT|TAS)$)|(^(.*)( QLD | NSW | VIC | SA | WA | NT | ACT | TAS )(.*)$)|(^(QLD |NSW |VIC |SA |WA |NT |ACT |TAS )(.*)$)|(^(.*)( QLD| NSW| VIC| SA| WA| NT| ACT| TAS)$)", RegexOptions.IgnoreCase);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.000411",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Suburb/Town/City cannot contain state code",
                    Location = "/xbrli:xbrl/address2.02.02:AddressDetails" + OccurrenceIndex(report.AddressDetailsCollection_POS_P_RPOccurrenceIndex) + "/pyde.02.00:AddressDetails.LocalityName.Text",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.428230" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "PTR15", Value = report.PTR15 });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.GEN.428230

            #region VR.ATO.GEN.428231

            /*  VR.ATO.GEN.428231
            Family name cannot contain suffix ESQ, II, III, IV, JNR, JP, MHA, MHR, MLA, MLC, MP, QC or SNR
    
            Legacy Rule Format:
            IF (pyde.xx.xx:PersonNameDetails.FamilyName.Text = FOUND("ESQ","II","III","IV","JNR","JP","MHA","MHR","MLA","MLC","MP","QC","SNR"))
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            (FoundSet(^PTR20, '"ESQ","II","III","IV","JNR","JP","MHA","MHR","MLA","MLC","MP","QC","SNR"'))
    
            Data Elements:
    
            RP:^PTR20 = pyde.02.00:PersonNameDetails.FamilyName.Text WHERE [PersonNameDetails.Usage.Code='MailRecipient'].[PersonNameDetails.PersonNameType.Code='LGL']
            */
            assertion = IsMatch(report.PTR20, @"(^(ESQ|II|III|IV|JNR|JP|MHA|MHR|MLA|MLC|MP|QC|SNR)$)|(^(.*)( ESQ | II | III | IV | JNR | JP | MHA | MHR | MLA | MLC | MP | QC | SNR )(.*)$)|(^(ESQ |II |III |IV |JNR |JP |MHA |MHR |MLA |MLC |MP |QC |SNR )(.*)$)|(^(.*)( ESQ| II| III| IV| JNR| JP| MHA| MHR| MLA| MLC| MP| QC| SNR)$)", RegexOptions.IgnoreCase);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.000422",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Family name cannot contain suffix ESQ, II, III, IV, JNR, JP, MHA, MHR, MLA, MLC, MP, QC or SNR",
                    Location = "/xbrli:xbrl/ptr.0006.lodge.req.02.00:MailRecipient/prsnstrcnm3.02.01:PersonNameDetails" + OccurrenceIndex(report.MailRecipient_PersonNameDetailsCollection_MailRecipient_LGL_RPOccurrenceIndex) + "/pyde.02.00:PersonNameDetails.FamilyName.Text",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.428231" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "PTR20", Value = report.PTR20 });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.GEN.428231

            #region VR.ATO.GEN.428247

            /*  VR.ATO.GEN.428247
            The Tax Agent number has failed the Tax Agent number algorithm check
    
            Legacy Rule Format:
            IF ((pyid.xx.xx:Identifiers.TaxAgentNumber.Identifier) <> NULLORBLANK) AND (TANALGORITHM(pyid.xx.xx:Identifiers.TaxAgentNumber.Identifier) = FALSE)
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            (^PTR2 <> BLANK) AND (FailsTANAlgorithm(^PTR2))
    
            Data Elements:
    
            INT:^PTR2 = pyid.02.00:Identifiers.TaxAgentNumber.Identifier
            */
            assertion = (string.IsNullOrWhiteSpace(report.PTR2) != true && FailsTANAlgorithm(report.PTR2));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.410009",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"The Tax Agent number has failed the Tax Agent number algorithm check",
                    Location = "/xbrli:xbrl/pyid.02.00:Identifiers.TaxAgentNumber.Identifier",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.428247" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "PTR2", Value = report.PTR2 });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.GEN.428247

            #region VR.ATO.GEN.428253

            /*  VR.ATO.GEN.428253
            Invalid ABN
    
            Legacy Rule Format:
            IF (INT:entity.identifier.ABN <> NULLORBLANK) AND (ABNALGORITHM (INT:entity.identifier.ABN) = FALSE)
              RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            (^ABN <> BLANK) AND (FailsABNAlgorithm(^ABN))
    
            Data Elements:
    
            ^ABN = INT
            */
            assertion = (string.IsNullOrWhiteSpace(report.INTIdentifierABN) != true && FailsABNAlgorithm(report.INTIdentifierABN));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.200011",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Invalid ABN",
                    Location = "/xbrli:xbrl/xbrli:context" + OccurrenceIndex(report.INTOccurrenceIndex) + "/xbrli:entity/xbrli:identifier",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.428253" } },
                };
                response.Add(processMessage);
            }
            #endregion // VR.ATO.GEN.428253

            #region VR.ATO.GEN.428254

            /*  VR.ATO.GEN.428254
            Address line 1 cannot contain "UNKNOWN"
    
            Legacy Rule Format:
            IF (pyde.xx.xx:AddressDetails.Line1.Text CONTAINS "UNKNOWN")
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            Contains(^PTR117, 'UNKNOWN')
    
            Data Elements:
    
            RP:^PTR117 = pyde.02.00:AddressDetails.Line1.Text WHERE [AddressDetails.Usage.Code='BUS'].[AddressDetails.Currency.Code='C']
            */
            assertion = (report.PTR117 == null ? false : report.PTR117.ToUpperInvariant().Contains(@"UNKNOWN"));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.000405",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Address line 1 cannot contain ""UNKNOWN""",
                    Location = "/xbrli:xbrl/address2.02.02:AddressDetails" + OccurrenceIndex(report.AddressDetailsCollection_BUS_C_RPOccurrenceIndex) + "/pyde.02.00:AddressDetails.Line1.Text",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.428254" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "PTR117", Value = report.PTR117 });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.GEN.428254

            #region VR.ATO.GEN.428254

            /*  VR.ATO.GEN.428254
            Address line 1 cannot contain "UNKNOWN"
    
            Legacy Rule Format:
            IF (pyde.xx.xx:AddressDetails.Line1.Text CONTAINS "UNKNOWN")
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            Contains(^PTR9, 'UNKNOWN')
    
            Data Elements:
    
            RP:^PTR9 = pyde.02.00:AddressDetails.Line1.Text WHERE [AddressDetails.Usage.Code='POS'].[AddressDetails.Currency.Code='C']
            */
            assertion = (report.PTR9 == null ? false : report.PTR9.ToUpperInvariant().Contains(@"UNKNOWN"));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.000405",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Address line 1 cannot contain ""UNKNOWN""",
                    Location = "/xbrli:xbrl/address2.02.02:AddressDetails" + OccurrenceIndex(report.AddressDetailsCollection_POS_C_RPOccurrenceIndex) + "/pyde.02.00:AddressDetails.Line1.Text",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.428254" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "PTR9", Value = report.PTR9 });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.GEN.428254

            #region VR.ATO.GEN.428254

            /*  VR.ATO.GEN.428254
            Address line 1 cannot contain "UNKNOWN"
    
            Legacy Rule Format:
            IF (pyde.xx.xx:AddressDetails.Line1.Text CONTAINS "UNKNOWN")
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            Contains(^PTR14, 'UNKNOWN')
    
            Data Elements:
    
            RP:^PTR14 = pyde.02.00:AddressDetails.Line1.Text WHERE [AddressDetails.Usage.Code='POS'].[AddressDetails.Currency.Code='P']
            */
            assertion = (report.PTR14 == null ? false : report.PTR14.ToUpperInvariant().Contains(@"UNKNOWN"));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.000405",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Address line 1 cannot contain ""UNKNOWN""",
                    Location = "/xbrli:xbrl/address2.02.02:AddressDetails" + OccurrenceIndex(report.AddressDetailsCollection_POS_P_RPOccurrenceIndex) + "/pyde.02.00:AddressDetails.Line1.Text",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.428254" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "PTR14", Value = report.PTR14 });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.GEN.428254

            #region VR.ATO.GEN.428255

            /*  VR.ATO.GEN.428255
            Address line 2 cannot contain "UNKNOWN"
    
            Legacy Rule Format:
            IF (pyde.xx.xx:AddressDetails.Line2.Text CONTAINS "UNKNOWN")
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            Contains(^PTR213, 'UNKNOWN')
    
            Data Elements:
    
            RP:^PTR213 = pyde.02.00:AddressDetails.Line2.Text WHERE [AddressDetails.Usage.Code='BUS'].[AddressDetails.Currency.Code='C']
            */
            assertion = (report.PTR213 == null ? false : report.PTR213.ToUpperInvariant().Contains(@"UNKNOWN"));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.000407",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Address line 2 cannot contain ""UNKNOWN""",
                    Location = "/xbrli:xbrl/address2.02.02:AddressDetails" + OccurrenceIndex(report.AddressDetailsCollection_BUS_C_RPOccurrenceIndex) + "/pyde.02.00:AddressDetails.Line2.Text",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.428255" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "PTR213", Value = report.PTR213 });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.GEN.428255

            #region VR.ATO.GEN.428255

            /*  VR.ATO.GEN.428255
            Address line 2 cannot contain "UNKNOWN"
    
            Legacy Rule Format:
            IF (pyde.xx.xx:AddressDetails.Line2.Text CONTAINS "UNKNOWN")
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            Contains(^PTR211, 'UNKNOWN')
    
            Data Elements:
    
            RP:^PTR211 = pyde.02.00:AddressDetails.Line2.Text WHERE [AddressDetails.Usage.Code='POS'].[AddressDetails.Currency.Code='C']
            */
            assertion = (report.PTR211 == null ? false : report.PTR211.ToUpperInvariant().Contains(@"UNKNOWN"));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.000407",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Address line 2 cannot contain ""UNKNOWN""",
                    Location = "/xbrli:xbrl/address2.02.02:AddressDetails" + OccurrenceIndex(report.AddressDetailsCollection_POS_C_RPOccurrenceIndex) + "/pyde.02.00:AddressDetails.Line2.Text",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.428255" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "PTR211", Value = report.PTR211 });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.GEN.428255

            #region VR.ATO.GEN.428255

            /*  VR.ATO.GEN.428255
            Address line 2 cannot contain "UNKNOWN"
    
            Legacy Rule Format:
            IF (pyde.xx.xx:AddressDetails.Line2.Text CONTAINS "UNKNOWN")
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            Contains(^PTR212, 'UNKNOWN')
    
            Data Elements:
    
            RP:^PTR212 = pyde.02.00:AddressDetails.Line2.Text WHERE [AddressDetails.Usage.Code='POS'].[AddressDetails.Currency.Code='P']
            */
            assertion = (report.PTR212 == null ? false : report.PTR212.ToUpperInvariant().Contains(@"UNKNOWN"));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.000407",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Address line 2 cannot contain ""UNKNOWN""",
                    Location = "/xbrli:xbrl/address2.02.02:AddressDetails" + OccurrenceIndex(report.AddressDetailsCollection_POS_P_RPOccurrenceIndex) + "/pyde.02.00:AddressDetails.Line2.Text",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.428255" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "PTR212", Value = report.PTR212 });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.GEN.428255

            #region VR.ATO.GEN.428256

            /*  VR.ATO.GEN.428256
            Suburb/Town/City cannot contain "UNKNOWN"
    
            Legacy Rule Format:
            IF (pyde.xx.xx:AddressDetails.LocalityName.Text CONTAINS "UNKNOWN")
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            Contains(^PTR118, 'UNKNOWN')
    
            Data Elements:
    
            RP:^PTR118 = pyde.02.00:AddressDetails.LocalityName.Text WHERE [AddressDetails.Usage.Code='BUS'].[AddressDetails.Currency.Code='C']
            */
            assertion = (report.PTR118 == null ? false : report.PTR118.ToUpperInvariant().Contains(@"UNKNOWN"));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.000409",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Suburb/Town/City cannot contain ""UNKNOWN""",
                    Location = "/xbrli:xbrl/address2.02.02:AddressDetails" + OccurrenceIndex(report.AddressDetailsCollection_BUS_C_RPOccurrenceIndex) + "/pyde.02.00:AddressDetails.LocalityName.Text",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.428256" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "PTR118", Value = report.PTR118 });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.GEN.428256

            #region VR.ATO.GEN.428256

            /*  VR.ATO.GEN.428256
            Suburb/Town/City cannot contain "UNKNOWN"
    
            Legacy Rule Format:
            IF (pyde.xx.xx:AddressDetails.LocalityName.Text CONTAINS "UNKNOWN")
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            Contains(^PTR10, 'UNKNOWN')
    
            Data Elements:
    
            RP:^PTR10 = pyde.02.00:AddressDetails.LocalityName.Text WHERE [AddressDetails.Usage.Code='POS'].[AddressDetails.Currency.Code='C']
            */
            assertion = (report.PTR10 == null ? false : report.PTR10.ToUpperInvariant().Contains(@"UNKNOWN"));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.000409",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Suburb/Town/City cannot contain ""UNKNOWN""",
                    Location = "/xbrli:xbrl/address2.02.02:AddressDetails" + OccurrenceIndex(report.AddressDetailsCollection_POS_C_RPOccurrenceIndex) + "/pyde.02.00:AddressDetails.LocalityName.Text",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.428256" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "PTR10", Value = report.PTR10 });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.GEN.428256

            #region VR.ATO.GEN.428256

            /*  VR.ATO.GEN.428256
            Suburb/Town/City cannot contain "UNKNOWN"
    
            Legacy Rule Format:
            IF (pyde.xx.xx:AddressDetails.LocalityName.Text CONTAINS "UNKNOWN")
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            Contains(^PTR15, 'UNKNOWN')
    
            Data Elements:
    
            RP:^PTR15 = pyde.02.00:AddressDetails.LocalityName.Text WHERE [AddressDetails.Usage.Code='POS'].[AddressDetails.Currency.Code='P']
            */
            assertion = (report.PTR15 == null ? false : report.PTR15.ToUpperInvariant().Contains(@"UNKNOWN"));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.000409",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Suburb/Town/City cannot contain ""UNKNOWN""",
                    Location = "/xbrli:xbrl/address2.02.02:AddressDetails" + OccurrenceIndex(report.AddressDetailsCollection_POS_P_RPOccurrenceIndex) + "/pyde.02.00:AddressDetails.LocalityName.Text",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.428256" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "PTR15", Value = report.PTR15 });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.GEN.428256

            #region VR.ATO.GEN.428258

            /*  VR.ATO.GEN.428258
            Non Individual Name cannot contain space hyphen space combinations
    
            Legacy Rule Format:
            IF (pyde.xx.xx:OrganisationNameDetails.OrganisationalName.Text CONTAINS  " - " )
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            Contains(^PTR116, ' - ')
    
            Data Elements:
    
            RP:^PTR116 = pyde.02.00:OrganisationNameDetails.OrganisationalName.Text WHERE [OrganisationNameDetails.OrganisationalNameType.Code='MTR'].[OrganisationNameDetails.Currency.Code='C']
            */
            assertion = (report.PTR116 == null ? false : report.PTR116.ToUpperInvariant().Contains(@" - "));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.000413",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Non Individual Name cannot contain space hyphen space combinations",
                    Location = "/xbrli:xbrl/orgname2.02.00:OrganisationNameDetails" + OccurrenceIndex(report.OrganisationNameDetailsCollection_MTR_C_RPOccurrenceIndex) + "/pyde.02.00:OrganisationNameDetails.OrganisationalName.Text",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.428258" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "PTR116", Value = report.PTR116 });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.GEN.428258

            #region VR.ATO.GEN.428258

            /*  VR.ATO.GEN.428258
            Non Individual Name cannot contain space hyphen space combinations
    
            Legacy Rule Format:
            IF (pyde.xx.xx:OrganisationNameDetails.OrganisationalName.Text CONTAINS  " - " )
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            Contains(^PTR24, ' - ')
    
            Data Elements:
    
            RP:^PTR24 = pyde.02.00:OrganisationNameDetails.OrganisationalName.Text WHERE [OrganisationNameDetails.OrganisationalNameType.Code='MN'].[OrganisationNameDetails.Currency.Code='C']
            */
            assertion = (report.PTR24 == null ? false : report.PTR24.ToUpperInvariant().Contains(@" - "));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.000413",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Non Individual Name cannot contain space hyphen space combinations",
                    Location = "/xbrli:xbrl/ptr.0006.lodge.req.02.00:MailRecipient/orgname2.02.00:OrganisationNameDetails" + OccurrenceIndex(report.MailRecipient_OrganisationNameDetailsCollection_MN_C_RPOccurrenceIndex) + "/pyde.02.00:OrganisationNameDetails.OrganisationalName.Text",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.428258" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "PTR24", Value = report.PTR24 });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.GEN.428258

            #region VR.ATO.GEN.428258

            /*  VR.ATO.GEN.428258
            Non Individual Name cannot contain space hyphen space combinations
    
            Legacy Rule Format:
            IF (pyde.xx.xx:OrganisationNameDetails.OrganisationalName.Text CONTAINS  " - " )
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            Contains(^PTR6, ' - ')
    
            Data Elements:
    
            RP:^PTR6 = pyde.02.00:OrganisationNameDetails.OrganisationalName.Text WHERE [OrganisationNameDetails.OrganisationalNameType.Code='MN'].[OrganisationNameDetails.Currency.Code='C']
            */
            assertion = (report.PTR6 == null ? false : report.PTR6.ToUpperInvariant().Contains(@" - "));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.000413",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Non Individual Name cannot contain space hyphen space combinations",
                    Location = "/xbrli:xbrl/orgname2.02.00:OrganisationNameDetails" + OccurrenceIndex(report.OrganisationNameDetailsCollection_MN_C_RPOccurrenceIndex) + "/pyde.02.00:OrganisationNameDetails.OrganisationalName.Text",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.428258" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "PTR6", Value = report.PTR6 });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.GEN.428258

            #region VR.ATO.GEN.428258

            /*  VR.ATO.GEN.428258
            Non Individual Name cannot contain space hyphen space combinations
    
            Legacy Rule Format:
            IF (pyde.xx.xx:OrganisationNameDetails.OrganisationalName.Text CONTAINS  " - " )
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            Contains(^PTR8, ' - ')
    
            Data Elements:
    
            RP:^PTR8 = pyde.02.00:OrganisationNameDetails.OrganisationalName.Text WHERE [OrganisationNameDetails.Currency.Code='P'].[OrganisationNameDetails.OrganisationalNameType.Code='MN']
            */
            assertion = (report.PTR8 == null ? false : report.PTR8.ToUpperInvariant().Contains(@" - "));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.000413",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Non Individual Name cannot contain space hyphen space combinations",
                    Location = "/xbrli:xbrl/orgname2.02.00:OrganisationNameDetails" + OccurrenceIndex(report.OrganisationNameDetailsCollection_P_MN_RPOccurrenceIndex) + "/pyde.02.00:OrganisationNameDetails.OrganisationalName.Text",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.428258" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "PTR8", Value = report.PTR8 });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.GEN.428258

            #region VR.ATO.GEN.428259

            /*  VR.ATO.GEN.428259
            Non Individual Name cannot contain "P/L"
    
            Legacy Rule Format:
            IF (pyde.xx.xx:OrganisationNameDetails.OrganisationalName.Text CONTAINS "P/L")
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            Contains(^PTR116, 'P/L')
    
            Data Elements:
    
            RP:^PTR116 = pyde.02.00:OrganisationNameDetails.OrganisationalName.Text WHERE [OrganisationNameDetails.OrganisationalNameType.Code='MTR'].[OrganisationNameDetails.Currency.Code='C']
            */
            assertion = (report.PTR116 == null ? false : report.PTR116.ToUpperInvariant().Contains(@"P/L"));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.000414",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Non Individual Name cannot contain ""P/L""",
                    Location = "/xbrli:xbrl/orgname2.02.00:OrganisationNameDetails" + OccurrenceIndex(report.OrganisationNameDetailsCollection_MTR_C_RPOccurrenceIndex) + "/pyde.02.00:OrganisationNameDetails.OrganisationalName.Text",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.428259" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "PTR116", Value = report.PTR116 });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.GEN.428259

            #region VR.ATO.GEN.428259

            /*  VR.ATO.GEN.428259
            Non Individual Name cannot contain "P/L"
    
            Legacy Rule Format:
            IF (pyde.xx.xx:OrganisationNameDetails.OrganisationalName.Text CONTAINS "P/L")
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            Contains(^PTR24, 'P/L')
    
            Data Elements:
    
            RP:^PTR24 = pyde.02.00:OrganisationNameDetails.OrganisationalName.Text WHERE [OrganisationNameDetails.OrganisationalNameType.Code='MN'].[OrganisationNameDetails.Currency.Code='C']
            */
            assertion = (report.PTR24 == null ? false : report.PTR24.ToUpperInvariant().Contains(@"P/L"));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.000414",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Non Individual Name cannot contain ""P/L""",
                    Location = "/xbrli:xbrl/ptr.0006.lodge.req.02.00:MailRecipient/orgname2.02.00:OrganisationNameDetails" + OccurrenceIndex(report.MailRecipient_OrganisationNameDetailsCollection_MN_C_RPOccurrenceIndex) + "/pyde.02.00:OrganisationNameDetails.OrganisationalName.Text",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.428259" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "PTR24", Value = report.PTR24 });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.GEN.428259

            #region VR.ATO.GEN.428259

            /*  VR.ATO.GEN.428259
            Non Individual Name cannot contain "P/L"
    
            Legacy Rule Format:
            IF (pyde.xx.xx:OrganisationNameDetails.OrganisationalName.Text CONTAINS "P/L")
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            Contains(^PTR6, 'P/L')
    
            Data Elements:
    
            RP:^PTR6 = pyde.02.00:OrganisationNameDetails.OrganisationalName.Text WHERE [OrganisationNameDetails.OrganisationalNameType.Code='MN'].[OrganisationNameDetails.Currency.Code='C']
            */
            assertion = (report.PTR6 == null ? false : report.PTR6.ToUpperInvariant().Contains(@"P/L"));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.000414",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Non Individual Name cannot contain ""P/L""",
                    Location = "/xbrli:xbrl/orgname2.02.00:OrganisationNameDetails" + OccurrenceIndex(report.OrganisationNameDetailsCollection_MN_C_RPOccurrenceIndex) + "/pyde.02.00:OrganisationNameDetails.OrganisationalName.Text",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.428259" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "PTR6", Value = report.PTR6 });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.GEN.428259

            #region VR.ATO.GEN.428259

            /*  VR.ATO.GEN.428259
            Non Individual Name cannot contain "P/L"
    
            Legacy Rule Format:
            IF (pyde.xx.xx:OrganisationNameDetails.OrganisationalName.Text CONTAINS "P/L")
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            Contains(^PTR8, 'P/L')
    
            Data Elements:
    
            RP:^PTR8 = pyde.02.00:OrganisationNameDetails.OrganisationalName.Text WHERE [OrganisationNameDetails.Currency.Code='P'].[OrganisationNameDetails.OrganisationalNameType.Code='MN']
            */
            assertion = (report.PTR8 == null ? false : report.PTR8.ToUpperInvariant().Contains(@"P/L"));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.000414",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Non Individual Name cannot contain ""P/L""",
                    Location = "/xbrli:xbrl/orgname2.02.00:OrganisationNameDetails" + OccurrenceIndex(report.OrganisationNameDetailsCollection_P_MN_RPOccurrenceIndex) + "/pyde.02.00:OrganisationNameDetails.OrganisationalName.Text",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.428259" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "PTR8", Value = report.PTR8 });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.GEN.428259

            #region VR.ATO.GEN.428260

            /*  VR.ATO.GEN.428260
            Non Individual Name cannot contain repeated hyphen, apostrophe, or space characters
    
            Legacy Rule Format:
            IF (pyde.xx.xx:OrganisationNameDetails.OrganisationalName.Text CONTAINS SET("--","''","  ") )
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            (ContainsSet(^PTR116, '"--","''","  "'))
    
            Data Elements:
    
            RP:^PTR116 = pyde.02.00:OrganisationNameDetails.OrganisationalName.Text WHERE [OrganisationNameDetails.OrganisationalNameType.Code='MTR'].[OrganisationNameDetails.Currency.Code='C']
            */
            assertion = IsMatch(report.PTR116, @"--|''|  ", RegexOptions.IgnoreCase);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.000416",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Non Individual Name cannot contain repeated hyphen, apostrophe, or space characters",
                    Location = "/xbrli:xbrl/orgname2.02.00:OrganisationNameDetails" + OccurrenceIndex(report.OrganisationNameDetailsCollection_MTR_C_RPOccurrenceIndex) + "/pyde.02.00:OrganisationNameDetails.OrganisationalName.Text",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.428260" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "PTR116", Value = report.PTR116 });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.GEN.428260

            #region VR.ATO.GEN.428260

            /*  VR.ATO.GEN.428260
            Non Individual Name cannot contain repeated hyphen, apostrophe, or space characters
    
            Legacy Rule Format:
            IF (pyde.xx.xx:OrganisationNameDetails.OrganisationalName.Text CONTAINS SET("--","''","  ") )
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            (ContainsSet(^PTR24, '"--","''","  "'))
    
            Data Elements:
    
            RP:^PTR24 = pyde.02.00:OrganisationNameDetails.OrganisationalName.Text WHERE [OrganisationNameDetails.OrganisationalNameType.Code='MN'].[OrganisationNameDetails.Currency.Code='C']
            */
            assertion = IsMatch(report.PTR24, @"--|''|  ", RegexOptions.IgnoreCase);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.000416",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Non Individual Name cannot contain repeated hyphen, apostrophe, or space characters",
                    Location = "/xbrli:xbrl/ptr.0006.lodge.req.02.00:MailRecipient/orgname2.02.00:OrganisationNameDetails" + OccurrenceIndex(report.MailRecipient_OrganisationNameDetailsCollection_MN_C_RPOccurrenceIndex) + "/pyde.02.00:OrganisationNameDetails.OrganisationalName.Text",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.428260" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "PTR24", Value = report.PTR24 });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.GEN.428260

            #region VR.ATO.GEN.428260

            /*  VR.ATO.GEN.428260
            Non Individual Name cannot contain repeated hyphen, apostrophe, or space characters
    
            Legacy Rule Format:
            IF (pyde.xx.xx:OrganisationNameDetails.OrganisationalName.Text CONTAINS SET("--","''","  ") )
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            (ContainsSet(^PTR6, '"--","''","  "'))
    
            Data Elements:
    
            RP:^PTR6 = pyde.02.00:OrganisationNameDetails.OrganisationalName.Text WHERE [OrganisationNameDetails.OrganisationalNameType.Code='MN'].[OrganisationNameDetails.Currency.Code='C']
            */
            assertion = IsMatch(report.PTR6, @"--|''|  ", RegexOptions.IgnoreCase);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.000416",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Non Individual Name cannot contain repeated hyphen, apostrophe, or space characters",
                    Location = "/xbrli:xbrl/orgname2.02.00:OrganisationNameDetails" + OccurrenceIndex(report.OrganisationNameDetailsCollection_MN_C_RPOccurrenceIndex) + "/pyde.02.00:OrganisationNameDetails.OrganisationalName.Text",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.428260" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "PTR6", Value = report.PTR6 });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.GEN.428260

            #region VR.ATO.GEN.428260

            /*  VR.ATO.GEN.428260
            Non Individual Name cannot contain repeated hyphen, apostrophe, or space characters
    
            Legacy Rule Format:
            IF (pyde.xx.xx:OrganisationNameDetails.OrganisationalName.Text CONTAINS SET("--","''","  ") )
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            (ContainsSet(^PTR8, '"--","''","  "'))
    
            Data Elements:
    
            RP:^PTR8 = pyde.02.00:OrganisationNameDetails.OrganisationalName.Text WHERE [OrganisationNameDetails.Currency.Code='P'].[OrganisationNameDetails.OrganisationalNameType.Code='MN']
            */
            assertion = IsMatch(report.PTR8, @"--|''|  ", RegexOptions.IgnoreCase);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.000416",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Non Individual Name cannot contain repeated hyphen, apostrophe, or space characters",
                    Location = "/xbrli:xbrl/orgname2.02.00:OrganisationNameDetails" + OccurrenceIndex(report.OrganisationNameDetailsCollection_P_MN_RPOccurrenceIndex) + "/pyde.02.00:OrganisationNameDetails.OrganisationalName.Text",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.428260" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "PTR8", Value = report.PTR8 });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.GEN.428260

            #region VR.ATO.GEN.428261

            /*  VR.ATO.GEN.428261
            Non Individual Name cannot end with "T/A", "T/A P'ship", "T/A Pship", "T/A P/Ship" or "T/A Partnership"
    
            Legacy Rule Format:
            IF (pyde.xx.xx:OrganisationNameDetails.OrganisationalName.Text ENDSWITH SET(" T/A"," T/A P'ship"," T/A Pship"," T/A P/Ship"," T/A Partnership"))
              RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            (EndsWithSet(^PTR116, '" T/A"," T/A P'ship"," T/A Pship"," T/A P/Ship"," T/A Partnership"'))
    
            Data Elements:
    
            RP:^PTR116 = pyde.02.00:OrganisationNameDetails.OrganisationalName.Text WHERE [OrganisationNameDetails.OrganisationalNameType.Code='MTR'].[OrganisationNameDetails.Currency.Code='C']
            */
            assertion = IsMatch(report.PTR116, @"( T/A| T/A P'ship| T/A Pship| T/A P/Ship| T/A Partnership)$", RegexOptions.IgnoreCase);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.000417",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Non Individual Name cannot end with ""T/A"", ""T/A P'ship"", ""T/A Pship"", ""T/A P/Ship"" or ""T/A Partnership""",
                    Location = "/xbrli:xbrl/orgname2.02.00:OrganisationNameDetails" + OccurrenceIndex(report.OrganisationNameDetailsCollection_MTR_C_RPOccurrenceIndex) + "/pyde.02.00:OrganisationNameDetails.OrganisationalName.Text",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.428261" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "PTR116", Value = report.PTR116 });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.GEN.428261

            #region VR.ATO.GEN.428261

            /*  VR.ATO.GEN.428261
            Non Individual Name cannot end with "T/A", "T/A P'ship", "T/A Pship", "T/A P/Ship" or "T/A Partnership"
    
            Legacy Rule Format:
            IF (pyde.xx.xx:OrganisationNameDetails.OrganisationalName.Text ENDSWITH SET(" T/A"," T/A P'ship"," T/A Pship"," T/A P/Ship"," T/A Partnership"))
              RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            (EndsWithSet(^PTR24, '" T/A"," T/A P'ship"," T/A Pship"," T/A P/Ship"," T/A Partnership"'))
    
            Data Elements:
    
            RP:^PTR24 = pyde.02.00:OrganisationNameDetails.OrganisationalName.Text WHERE [OrganisationNameDetails.OrganisationalNameType.Code='MN'].[OrganisationNameDetails.Currency.Code='C']
            */
            assertion = IsMatch(report.PTR24, @"( T/A| T/A P'ship| T/A Pship| T/A P/Ship| T/A Partnership)$", RegexOptions.IgnoreCase);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.000417",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Non Individual Name cannot end with ""T/A"", ""T/A P'ship"", ""T/A Pship"", ""T/A P/Ship"" or ""T/A Partnership""",
                    Location = "/xbrli:xbrl/ptr.0006.lodge.req.02.00:MailRecipient/orgname2.02.00:OrganisationNameDetails" + OccurrenceIndex(report.MailRecipient_OrganisationNameDetailsCollection_MN_C_RPOccurrenceIndex) + "/pyde.02.00:OrganisationNameDetails.OrganisationalName.Text",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.428261" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "PTR24", Value = report.PTR24 });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.GEN.428261

            #region VR.ATO.GEN.428261

            /*  VR.ATO.GEN.428261
            Non Individual Name cannot end with "T/A", "T/A P'ship", "T/A Pship", "T/A P/Ship" or "T/A Partnership"
    
            Legacy Rule Format:
            IF (pyde.xx.xx:OrganisationNameDetails.OrganisationalName.Text ENDSWITH SET(" T/A"," T/A P'ship"," T/A Pship"," T/A P/Ship"," T/A Partnership"))
              RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            (EndsWithSet(^PTR6, '" T/A"," T/A P'ship"," T/A Pship"," T/A P/Ship"," T/A Partnership"'))
    
            Data Elements:
    
            RP:^PTR6 = pyde.02.00:OrganisationNameDetails.OrganisationalName.Text WHERE [OrganisationNameDetails.OrganisationalNameType.Code='MN'].[OrganisationNameDetails.Currency.Code='C']
            */
            assertion = IsMatch(report.PTR6, @"( T/A| T/A P'ship| T/A Pship| T/A P/Ship| T/A Partnership)$", RegexOptions.IgnoreCase);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.000417",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Non Individual Name cannot end with ""T/A"", ""T/A P'ship"", ""T/A Pship"", ""T/A P/Ship"" or ""T/A Partnership""",
                    Location = "/xbrli:xbrl/orgname2.02.00:OrganisationNameDetails" + OccurrenceIndex(report.OrganisationNameDetailsCollection_MN_C_RPOccurrenceIndex) + "/pyde.02.00:OrganisationNameDetails.OrganisationalName.Text",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.428261" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "PTR6", Value = report.PTR6 });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.GEN.428261

            #region VR.ATO.GEN.428261

            /*  VR.ATO.GEN.428261
            Non Individual Name cannot end with "T/A", "T/A P'ship", "T/A Pship", "T/A P/Ship" or "T/A Partnership"
    
            Legacy Rule Format:
            IF (pyde.xx.xx:OrganisationNameDetails.OrganisationalName.Text ENDSWITH SET(" T/A"," T/A P'ship"," T/A Pship"," T/A P/Ship"," T/A Partnership"))
              RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            (EndsWithSet(^PTR8, '" T/A"," T/A P'ship"," T/A Pship"," T/A P/Ship"," T/A Partnership"'))
    
            Data Elements:
    
            RP:^PTR8 = pyde.02.00:OrganisationNameDetails.OrganisationalName.Text WHERE [OrganisationNameDetails.Currency.Code='P'].[OrganisationNameDetails.OrganisationalNameType.Code='MN']
            */
            assertion = IsMatch(report.PTR8, @"( T/A| T/A P'ship| T/A Pship| T/A P/Ship| T/A Partnership)$", RegexOptions.IgnoreCase);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.000417",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Non Individual Name cannot end with ""T/A"", ""T/A P'ship"", ""T/A Pship"", ""T/A P/Ship"" or ""T/A Partnership""",
                    Location = "/xbrli:xbrl/orgname2.02.00:OrganisationNameDetails" + OccurrenceIndex(report.OrganisationNameDetailsCollection_P_MN_RPOccurrenceIndex) + "/pyde.02.00:OrganisationNameDetails.OrganisationalName.Text",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.428261" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "PTR8", Value = report.PTR8 });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.GEN.428261

            #region VR.ATO.GEN.428262

            /*  VR.ATO.GEN.428262
            Family Name cannot contain "space hyphen space"
    
            Legacy Rule Format:
            IF (pyde.xx.xx:PersonNameDetails.FamilyName.Text CONTAINS " - ")
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            Contains(^PTR20, ' - ')
    
            Data Elements:
    
            RP:^PTR20 = pyde.02.00:PersonNameDetails.FamilyName.Text WHERE [PersonNameDetails.Usage.Code='MailRecipient'].[PersonNameDetails.PersonNameType.Code='LGL']
            */
            assertion = (report.PTR20 == null ? false : report.PTR20.ToUpperInvariant().Contains(@" - "));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.000423",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Family Name cannot contain ""space hyphen space""",
                    Location = "/xbrli:xbrl/ptr.0006.lodge.req.02.00:MailRecipient/prsnstrcnm3.02.01:PersonNameDetails" + OccurrenceIndex(report.MailRecipient_PersonNameDetailsCollection_MailRecipient_LGL_RPOccurrenceIndex) + "/pyde.02.00:PersonNameDetails.FamilyName.Text",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.428262" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "PTR20", Value = report.PTR20 });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.GEN.428262

            #region VR.ATO.GEN.428263

            /*  VR.ATO.GEN.428263
            Family name cannot contain repeated hyphen, apostrophe, or space characters
    
            Legacy Rule Format:
            IF (pyde.xx.xx:PersonNameDetails.FamilyName.Text CONTAINS SET("--","''","  ") )
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            (ContainsSet(^PTR20, '"--","''","  "'))
    
            Data Elements:
    
            RP:^PTR20 = pyde.02.00:PersonNameDetails.FamilyName.Text WHERE [PersonNameDetails.Usage.Code='MailRecipient'].[PersonNameDetails.PersonNameType.Code='LGL']
            */
            assertion = IsMatch(report.PTR20, @"--|''|  ", RegexOptions.IgnoreCase);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.000427",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Family name cannot contain repeated hyphen, apostrophe, or space characters",
                    Location = "/xbrli:xbrl/ptr.0006.lodge.req.02.00:MailRecipient/prsnstrcnm3.02.01:PersonNameDetails" + OccurrenceIndex(report.MailRecipient_PersonNameDetailsCollection_MailRecipient_LGL_RPOccurrenceIndex) + "/pyde.02.00:PersonNameDetails.FamilyName.Text",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.428263" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "PTR20", Value = report.PTR20 });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.GEN.428263

            #region VR.ATO.GEN.428264

            /*  VR.ATO.GEN.428264
            First name cannot contain space hyphen space
    
            Legacy Rule Format:
            IF (pyde.xx.xx:PersonNameDetails.GivenName.Text CONTAINS  " - ")
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            Contains(^PTR22, ' - ')
    
            Data Elements:
    
            RP:^PTR22 = pyde.02.00:PersonNameDetails.GivenName.Text WHERE [PersonNameDetails.Usage.Code='MailRecipient'].[PersonNameDetails.PersonNameType.Code='LGL']
            */
            assertion = (report.PTR22 == null ? false : report.PTR22.ToUpperInvariant().Contains(@" - "));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.000434",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"First name cannot contain space hyphen space",
                    Location = "/xbrli:xbrl/ptr.0006.lodge.req.02.00:MailRecipient/prsnstrcnm3.02.01:PersonNameDetails" + OccurrenceIndex(report.MailRecipient_PersonNameDetailsCollection_MailRecipient_LGL_RPOccurrenceIndex) + "/pyde.02.00:PersonNameDetails.GivenName.Text",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.428264" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "PTR22", Value = report.PTR22 });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.GEN.428264

            #region VR.ATO.GEN.428265

            /*  VR.ATO.GEN.428265
            First name cannot contain repeated hyphen, apostrophe, or space characters
    
            Legacy Rule Format:
            IF (pyde.xx.xx:PersonNameDetails.GivenName.Text CONTAINS SET("--","''","  ") )
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            (ContainsSet(^PTR22, '"--","''","  "'))
    
            Data Elements:
    
            RP:^PTR22 = pyde.02.00:PersonNameDetails.GivenName.Text WHERE [PersonNameDetails.Usage.Code='MailRecipient'].[PersonNameDetails.PersonNameType.Code='LGL']
            */
            assertion = IsMatch(report.PTR22, @"--|''|  ", RegexOptions.IgnoreCase);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.000439",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"First name cannot contain repeated hyphen, apostrophe, or space characters",
                    Location = "/xbrli:xbrl/ptr.0006.lodge.req.02.00:MailRecipient/prsnstrcnm3.02.01:PersonNameDetails" + OccurrenceIndex(report.MailRecipient_PersonNameDetailsCollection_MailRecipient_LGL_RPOccurrenceIndex) + "/pyde.02.00:PersonNameDetails.GivenName.Text",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.428265" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "PTR22", Value = report.PTR22 });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.GEN.428265

            #region VR.ATO.GEN.428266

            /*  VR.ATO.GEN.428266
            Other given name cannot contain space hyphen space
    
            Legacy Rule Format:
            IF (pyde.xx.xx:PersonNameDetails.OtherGivenName.Text CONTAINS  " - ")
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            Contains(^PTR23, ' - ')
    
            Data Elements:
    
            RP:^PTR23 = pyde.02.00:PersonNameDetails.OtherGivenName.Text WHERE [PersonNameDetails.Usage.Code='MailRecipient'].[PersonNameDetails.PersonNameType.Code='LGL']
            */
            assertion = (report.PTR23 == null ? false : report.PTR23.ToUpperInvariant().Contains(@" - "));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.000446",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Other given name cannot contain space hyphen space",
                    Location = "/xbrli:xbrl/ptr.0006.lodge.req.02.00:MailRecipient/prsnstrcnm3.02.01:PersonNameDetails" + OccurrenceIndex(report.MailRecipient_PersonNameDetailsCollection_MailRecipient_LGL_RPOccurrenceIndex) + "/pyde.02.00:PersonNameDetails.OtherGivenName.Text",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.428266" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "PTR23", Value = report.PTR23 });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.GEN.428266

            #region VR.ATO.GEN.428267

            /*  VR.ATO.GEN.428267
            Other given name cannot contain repeated hyphen, apostrophe, or space characters
    
            Legacy Rule Format:
            IF (pyde.xx.xx:PersonNameDetails.OtherGivenName.Text CONTAINS SET("--","''","  ") )
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            (ContainsSet(^PTR23, '"--","''","  "'))
    
            Data Elements:
    
            RP:^PTR23 = pyde.02.00:PersonNameDetails.OtherGivenName.Text WHERE [PersonNameDetails.Usage.Code='MailRecipient'].[PersonNameDetails.PersonNameType.Code='LGL']
            */
            assertion = IsMatch(report.PTR23, @"--|''|  ", RegexOptions.IgnoreCase);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.000451",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Other given name cannot contain repeated hyphen, apostrophe, or space characters",
                    Location = "/xbrli:xbrl/ptr.0006.lodge.req.02.00:MailRecipient/prsnstrcnm3.02.01:PersonNameDetails" + OccurrenceIndex(report.MailRecipient_PersonNameDetailsCollection_MailRecipient_LGL_RPOccurrenceIndex) + "/pyde.02.00:PersonNameDetails.OtherGivenName.Text",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.428267" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "PTR23", Value = report.PTR23 });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.GEN.428267

            #region VR.ATO.GEN.430206

            /*  VR.ATO.GEN.430206
            Hours taken to prepare and complete form must be 1 to 999
    
            Legacy Rule Format:
            IF (pyin.xx.xx:Report.CompletionHours.Number <> NULL) AND (pyin.xx.xx:Report.CompletionHours.Number <> SET (1-999))
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            (^PTR169 <> NULL) AND (^PTR169 < 1 OR ^PTR169 > 999)
    
            Data Elements:
    
            RP:^PTR169 = pyin.02.23:Report.CompletionHours.Number
            */
            assertion = (report.PTR169 != null && (report.PTR169.GetValueOrDefault() < 1 || report.PTR169.GetValueOrDefault() > 999));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.430206",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Hours taken to prepare and complete form must be 1 to 999",
                    Location = "/xbrli:xbrl/pyin.02.23:Report.CompletionHours.Number",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.430206" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "PTR169", Value = report.PTR169.GetValueOrDefault().ToString() });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.GEN.430206

            #region VR.ATO.GEN.430245

            /*  VR.ATO.GEN.430245
            Suburb/Town/City must be present in an address
    
            Legacy Rule Format:
            WHERE IN TUPLE(addressdetails2.xx.xx:AddressDetails)
            IF (pyde.xx.xx:AddressDetails.LocalityName.Text = NULLORBLANK)
              RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            (^AddressDetails <> NULL) AND ((^PTR118 = BLANK))
    
            Data Elements:
    
            RP:^PTR118 = pyde.02.00:AddressDetails.LocalityName.Text WHERE [AddressDetails.Usage.Code='BUS'].[AddressDetails.Currency.Code='C']
    
            ^AddressDetails = address2.02.02:AddressDetails
            */
            assertion = (report.AddressDetailsCollection_BUS_C_RPExists != false && string.IsNullOrWhiteSpace(report.PTR118) == true);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.430245",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Suburb/Town/City must be present in an address",
                    Location = "/xbrli:xbrl/address2.02.02:AddressDetails" + OccurrenceIndex(report.AddressDetailsCollection_BUS_C_RPOccurrenceIndex) + "/pyde.02.00:AddressDetails.LocalityName.Text",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.430245" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "PTR118", Value = report.PTR118 });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.GEN.430245

            #region VR.ATO.GEN.430245

            /*  VR.ATO.GEN.430245
            Suburb/Town/City must be present in an address
    
            Legacy Rule Format:
            WHERE IN TUPLE(addressdetails2.xx.xx:AddressDetails)
            IF (pyde.xx.xx:AddressDetails.LocalityName.Text = NULLORBLANK)
              RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            (^AddressDetails <> NULL) AND ((^PTR10 = BLANK))
    
            Data Elements:
    
            RP:^PTR10 = pyde.02.00:AddressDetails.LocalityName.Text WHERE [AddressDetails.Usage.Code='POS'].[AddressDetails.Currency.Code='C']
    
            ^AddressDetails = address2.02.02:AddressDetails
            */
            assertion = (report.AddressDetailsCollection_POS_C_RPExists != false && string.IsNullOrWhiteSpace(report.PTR10) == true);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.430245",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Suburb/Town/City must be present in an address",
                    Location = "/xbrli:xbrl/address2.02.02:AddressDetails" + OccurrenceIndex(report.AddressDetailsCollection_POS_C_RPOccurrenceIndex) + "/pyde.02.00:AddressDetails.LocalityName.Text",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.430245" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "PTR10", Value = report.PTR10 });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.GEN.430245

            #region VR.ATO.GEN.430245

            /*  VR.ATO.GEN.430245
            Suburb/Town/City must be present in an address
    
            Legacy Rule Format:
            WHERE IN TUPLE(addressdetails2.xx.xx:AddressDetails)
            IF (pyde.xx.xx:AddressDetails.LocalityName.Text = NULLORBLANK)
              RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            (^AddressDetails <> NULL) AND ((^PTR15 = BLANK))
    
            Data Elements:
    
            RP:^PTR15 = pyde.02.00:AddressDetails.LocalityName.Text WHERE [AddressDetails.Usage.Code='POS'].[AddressDetails.Currency.Code='P']
    
            ^AddressDetails = address2.02.02:AddressDetails
            */
            assertion = (report.AddressDetailsCollection_POS_P_RPExists != false && string.IsNullOrWhiteSpace(report.PTR15) == true);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.430245",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Suburb/Town/City must be present in an address",
                    Location = "/xbrli:xbrl/address2.02.02:AddressDetails" + OccurrenceIndex(report.AddressDetailsCollection_POS_P_RPOccurrenceIndex) + "/pyde.02.00:AddressDetails.LocalityName.Text",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.430245" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "PTR15", Value = report.PTR15 });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.GEN.430245

            #region VR.ATO.GEN.430246

            /*  VR.ATO.GEN.430246
            Suburb/Town/City cannot contain numeric characters
    
            Legacy Rule Format:
            WHERE IN TUPLE(addressdetails2.xx.xx:AddressDetails)
            IF (pyde.xx.xx:AddressDetails.OverseasAddress.Indicator = FALSE) AND (pyde.xx.xx:AddressDetails.LocalityName.Text CONTAINS NUMERICS SET(0-9))
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            (^PTR222 = FALSE) AND (ContainsSet(^PTR118, '0-9'))
    
            Data Elements:
    
            RP:^PTR118 = pyde.02.00:AddressDetails.LocalityName.Text WHERE [AddressDetails.Usage.Code='BUS'].[AddressDetails.Currency.Code='C']
    
            RP:^PTR222 = pyde.02.00:AddressDetails.OverseasAddress.Indicator WHERE [AddressDetails.Usage.Code='BUS'].[AddressDetails.Currency.Code='C']
            */
            assertion = (report.PTR222 == false && IsMatch(report.PTR118, @"[0-9]", RegexOptions.IgnoreCase));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.000410",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Suburb/Town/City cannot contain numeric characters",
                    Location = "/xbrli:xbrl/address2.02.02:AddressDetails" + OccurrenceIndex(report.AddressDetailsCollection_BUS_C_RPOccurrenceIndex) + "/pyde.02.00:AddressDetails.LocalityName.Text",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.430246" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "PTR222", Value = GetValueOrEmpty(report.PTR222) });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "PTR118", Value = report.PTR118 });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.GEN.430246

            #region VR.ATO.GEN.430246

            /*  VR.ATO.GEN.430246
            Suburb/Town/City cannot contain numeric characters
    
            Legacy Rule Format:
            WHERE IN TUPLE(addressdetails2.xx.xx:AddressDetails)
            IF (pyde.xx.xx:AddressDetails.OverseasAddress.Indicator = FALSE) AND (pyde.xx.xx:AddressDetails.LocalityName.Text CONTAINS NUMERICS SET(0-9))
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            (^PTR216 = FALSE) AND (ContainsSet(^PTR10, '0-9'))
    
            Data Elements:
    
            RP:^PTR10 = pyde.02.00:AddressDetails.LocalityName.Text WHERE [AddressDetails.Usage.Code='POS'].[AddressDetails.Currency.Code='C']
    
            RP:^PTR216 = pyde.02.00:AddressDetails.OverseasAddress.Indicator WHERE [AddressDetails.Usage.Code='POS'].[AddressDetails.Currency.Code='C']
            */
            assertion = (report.PTR216 == false && IsMatch(report.PTR10, @"[0-9]", RegexOptions.IgnoreCase));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.000410",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Suburb/Town/City cannot contain numeric characters",
                    Location = "/xbrli:xbrl/address2.02.02:AddressDetails" + OccurrenceIndex(report.AddressDetailsCollection_POS_C_RPOccurrenceIndex) + "/pyde.02.00:AddressDetails.LocalityName.Text",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.430246" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "PTR216", Value = GetValueOrEmpty(report.PTR216) });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "PTR10", Value = report.PTR10 });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.GEN.430246

            #region VR.ATO.GEN.430246

            /*  VR.ATO.GEN.430246
            Suburb/Town/City cannot contain numeric characters
    
            Legacy Rule Format:
            WHERE IN TUPLE(addressdetails2.xx.xx:AddressDetails)
            IF (pyde.xx.xx:AddressDetails.OverseasAddress.Indicator = FALSE) AND (pyde.xx.xx:AddressDetails.LocalityName.Text CONTAINS NUMERICS SET(0-9))
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            (^PTR219 = FALSE) AND (ContainsSet(^PTR15, '0-9'))
    
            Data Elements:
    
            RP:^PTR15 = pyde.02.00:AddressDetails.LocalityName.Text WHERE [AddressDetails.Usage.Code='POS'].[AddressDetails.Currency.Code='P']
    
            RP:^PTR219 = pyde.02.00:AddressDetails.OverseasAddress.Indicator WHERE [AddressDetails.Usage.Code='POS'].[AddressDetails.Currency.Code='P']
            */
            assertion = (report.PTR219 == false && IsMatch(report.PTR15, @"[0-9]", RegexOptions.IgnoreCase));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.000410",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Suburb/Town/City cannot contain numeric characters",
                    Location = "/xbrli:xbrl/address2.02.02:AddressDetails" + OccurrenceIndex(report.AddressDetailsCollection_POS_P_RPOccurrenceIndex) + "/pyde.02.00:AddressDetails.LocalityName.Text",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.430246" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "PTR219", Value = GetValueOrEmpty(report.PTR219) });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "PTR15", Value = report.PTR15 });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.GEN.430246

            #region VR.ATO.GEN.430252

            /*  VR.ATO.GEN.430252
            Unstructured name must not be more than 200 characters
    
            Legacy Rule Format:
            IF LENGTH(pyde.xx.xx:PersonUnstructuredName.FullName.Text) > 200
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            Length(^PTR242) > 200
    
            Data Elements:
    
            INT:^PTR242 = pyde.02.00:PersonUnstructuredName.FullName.Text WHERE [PersonUnstructuredName.Usage.Code='Contact']
            */
            assertion = (Length(report.PTR242) > 200);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.430252",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Unstructured name must not be more than 200 characters",
                    Location = "/xbrli:xbrl/prsnunstrcnm1.02.01:PersonUnstructuredName" + OccurrenceIndex(report.PersonUnstructuredNameCollection_Contact_INTOccurrenceIndex) + "/pyde.02.00:PersonUnstructuredName.FullName.Text",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.430252" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "PTR242", Value = report.PTR242 });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.GEN.430252

            #region VR.ATO.GEN.430252

            /*  VR.ATO.GEN.430252
            Unstructured name must not be more than 200 characters
    
            Legacy Rule Format:
            IF LENGTH(pyde.xx.xx:PersonUnstructuredName.FullName.Text) > 200
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            Length(^PTR314) > 200
    
            Data Elements:
    
            INT:^PTR314 = pyde.02.00:PersonUnstructuredName.FullName.Text WHERE [PersonUnstructuredName.Usage.Code='DeclarationSignatory']
            */
            assertion = (Length(report.PTR314) > 200);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.430252",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Unstructured name must not be more than 200 characters",
                    Location = "/xbrli:xbrl/declaration2.02.01:Declaration" + OccurrenceIndex(report.DeclarationCollection_TrueAndCorrect_INTOccurrenceIndex) + "/prsnunstrcnm1.02.01:PersonUnstructuredName" + OccurrenceIndex(report.Declaration_PersonUnstructuredNameCollection_DeclarationSignatory_INTOccurrenceIndex) + "/pyde.02.00:PersonUnstructuredName.FullName.Text",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.430252" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "PTR314", Value = report.PTR314 });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.GEN.430252

            #region VR.ATO.GEN.430252

            /*  VR.ATO.GEN.430252
            Unstructured name must not be more than 200 characters
    
            Legacy Rule Format:
            IF LENGTH(pyde.xx.xx:PersonUnstructuredName.FullName.Text) > 200
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            Length(^PTR264) > 200
    
            Data Elements:
    
            RP:^PTR264 = pyde.02.00:PersonUnstructuredName.FullName.Text WHERE [PersonUnstructuredName.Usage.Code='DeclarationSignatory']
            */
            assertion = (Length(report.PTR264) > 200);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.430252",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Unstructured name must not be more than 200 characters",
                    Location = "/xbrli:xbrl/declaration2.02.01:Declaration" + OccurrenceIndex(report.DeclarationCollection_TrueAndCorrect_RPOccurrenceIndex) + "/prsnunstrcnm1.02.01:PersonUnstructuredName" + OccurrenceIndex(report.Declaration_PersonUnstructuredNameCollection_DeclarationSignatory_RPOccurrenceIndex) + "/pyde.02.00:PersonUnstructuredName.FullName.Text",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.430252" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "PTR264", Value = report.PTR264 });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.GEN.430252

            #region VR.ATO.GEN.430258

            /*  VR.ATO.GEN.430258
            Invalid context. The number of Reporting party: Primary Production contexts must not exceed 1
    
            Legacy Rule Format:
            IF COUNT(CONTEXT(RP.Prim)) > 1 
            RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            Count(^Context) > 1
    
            Data Elements:
    
            ^Context = RP:RP.Prim
            */
            assertion = (Count(report.RPPrimCount) > 1);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.430258",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Invalid context. The number of Reporting party: Primary Production contexts must not exceed 1",
                    Location = "/xbrli:xbrl/xbrli:context" + OccurrenceIndex(report.RPPrimLastOccurrenceIndex),
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.430258" } },
                };
                response.Add(processMessage);
            }
            #endregion // VR.ATO.GEN.430258

            #region VR.ATO.GEN.430259

            /*  VR.ATO.GEN.430259
            Invalid context. The number of Reporting party: Non-Primary Production contexts must not exceed 1
    
            Legacy Rule Format:
            IF COUNT(CONTEXT(RP.NonPrim)) > 1 
            RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            Count(^Context) > 1
    
            Data Elements:
    
            ^Context = RP:RP.NonPrim
            */
            assertion = (Count(report.RPNonPrimCount) > 1);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.430259",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Invalid context. The number of Reporting party: Non-Primary Production contexts must not exceed 1",
                    Location = "/xbrli:xbrl/xbrli:context" + OccurrenceIndex(report.RPNonPrimLastOccurrenceIndex),
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.430259" } },
                };
                response.Add(processMessage);
            }
            #endregion // VR.ATO.GEN.430259

            #region VR.ATO.GEN.430266

            /*  VR.ATO.GEN.430266
            Invalid context. The number of Reporting party: Australian contexts must not exceed 1
    
            Legacy Rule Format:
            IF COUNT(CONTEXT(RP.JAus)) > 1 
            RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            Count(^Context) > 1
    
            Data Elements:
    
            ^Context = RP:RP.JAUS
            */
            assertion = (Count(report.RPJAUSCount) > 1);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.430266",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Invalid context. The number of Reporting party: Australian contexts must not exceed 1",
                    Location = "/xbrli:xbrl/xbrli:context" + OccurrenceIndex(report.RPJAUSLastOccurrenceIndex),
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.430266" } },
                };
                response.Add(processMessage);
            }
            #endregion // VR.ATO.GEN.430266

            #region VR.ATO.GEN.430267

            /*  VR.ATO.GEN.430267
            Invalid context. The number of Reporting party: Foreign contexts must not exceed 1
    
            Legacy Rule Format:
            IF COUNT(CONTEXT(RP.JForeign)) > 1 
            RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            Count(^Context) > 1
    
            Data Elements:
    
            ^Context = RP:RP.JFOREIGN
            */
            assertion = (Count(report.RPJFOREIGNCount) > 1);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.430267",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Invalid context. The number of Reporting party: Foreign contexts must not exceed 1",
                    Location = "/xbrli:xbrl/xbrli:context" + OccurrenceIndex(report.RPJFOREIGNLastOccurrenceIndex),
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.430267" } },
                };
                response.Add(processMessage);
            }
            #endregion // VR.ATO.GEN.430267

            #region VR.ATO.GEN.430269

            /*  VR.ATO.GEN.430269
            Invalid context. The number of Reporting party: Primary: Australian contexts must not exceed 1
    
            Legacy Rule Format:
            IF COUNT(CONTEXT(RP.Prim.JAus)) > 1 
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            Count(^Context) > 1
    
            Data Elements:
    
            ^Context = RP.JAUS:RP.JAUS.Prim
            */
            assertion = (Count(report.RPJAUSPrimCount) > 1);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.430269",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Invalid context. The number of Reporting party: Primary: Australian contexts must not exceed 1",
                    Location = "/xbrli:xbrl/xbrli:context" + OccurrenceIndex(report.RPJAUSPrimLastOccurrenceIndex),
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.430269" } },
                };
                response.Add(processMessage);
            }
            #endregion // VR.ATO.GEN.430269

            #region VR.ATO.GEN.430270

            /*  VR.ATO.GEN.430270
            Invalid context. The number of Reporting party: Non-Primary: Australian contexts must not exceed 1
    
            Legacy Rule Format:
            IF COUNT(CONTEXT(RP.NonPrim.JAus)) > 1 
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            Count(^Context) > 1
    
            Data Elements:
    
            ^Context = RP.JAUS:RP.JAUS.NonPrim
            */
            assertion = (Count(report.RPJAUSNonPrimCount) > 1);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.430270",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Invalid context. The number of Reporting party: Non-Primary: Australian contexts must not exceed 1",
                    Location = "/xbrli:xbrl/xbrli:context" + OccurrenceIndex(report.RPJAUSNonPrimLastOccurrenceIndex),
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.430270" } },
                };
                response.Add(processMessage);
            }
            #endregion // VR.ATO.GEN.430270

            #region VR.ATO.GEN.430273

            /*  VR.ATO.GEN.430273
            Invalid context. The number of Reporting party: ListedCountry: Foreign contexts must not exceed 1
    
            Legacy Rule Format:
            IF COUNT(CONTEXT(RP.CFCLC.JForeign)) > 1 
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            Count(^Context) > 1
    
            Data Elements:
    
            ^Context = RP.JFOREIGN:RP.JFOREIGN.CFCLC
            */
            assertion = (Count(report.RPJFOREIGNCFCLCCount) > 1);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.430273",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Invalid context. The number of Reporting party: ListedCountry: Foreign contexts must not exceed 1",
                    Location = "/xbrli:xbrl/xbrli:context" + OccurrenceIndex(report.RPJFOREIGNCFCLCLastOccurrenceIndex),
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.430273" } },
                };
                response.Add(processMessage);
            }
            #endregion // VR.ATO.GEN.430273

            #region VR.ATO.GEN.430275

            /*  VR.ATO.GEN.430275
            Invalid context. The number of Reporting party: UnlistedCountry: Foreign contexts must not exceed 1
    
            Legacy Rule Format:
            IF COUNT(CONTEXT(RP.CFCUC.JForeign)) > 1 
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            Count(^Context) > 1
    
            Data Elements:
    
            ^Context = RP.JFOREIGN:RP.JFOREIGN.CFCUC
            */
            assertion = (Count(report.RPJFOREIGNCFCUCCount) > 1);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.430275",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Invalid context. The number of Reporting party: UnlistedCountry: Foreign contexts must not exceed 1",
                    Location = "/xbrli:xbrl/xbrli:context" + OccurrenceIndex(report.RPJFOREIGNCFCUCLastOccurrenceIndex),
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.430275" } },
                };
                response.Add(processMessage);
            }
            #endregion // VR.ATO.GEN.430275

            #region VR.ATO.GEN.430279

            /*  VR.ATO.GEN.430279
            Invalid context. The number of Reporting Party Instant: Opening contexts must not exceed 1
    
            Legacy Rule Format:
            IF COUNT(CONTEXT(RP.Opening(Instant))) > 1 
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            Count(^Context) > 1
    
            Data Elements:
    
            ^Context = RP:RP.Opening(Instant)
            */
            assertion = (Count(report.RPOpeningInstantCount) > 1);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.430279",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Invalid context. The number of Reporting Party Instant: Opening contexts must not exceed 1",
                    Location = "/xbrli:xbrl/xbrli:context" + OccurrenceIndex(report.RPOpeningInstantLastOccurrenceIndex),
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.430279" } },
                };
                response.Add(processMessage);
            }
            #endregion // VR.ATO.GEN.430279

            #region VR.ATO.GEN.430280

            /*  VR.ATO.GEN.430280
            Invalid context. The number of Reporting Party Instant: Opening contexts must not exceed 1
    
            Legacy Rule Format:
            IF COUNT(CONTEXT(RP.Closing(Instant))) > 1 
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            Count(^Context) > 1
    
            Data Elements:
    
            ^Context = RP:RP.Closing(Instant)
            */
            assertion = (Count(report.RPClosingInstantCount) > 1);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.430280",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Invalid context. The number of Reporting Party Instant: Closing contexts must not exceed 1",
                    Location = "/xbrli:xbrl/xbrli:context" + OccurrenceIndex(report.RPClosingInstantLastOccurrenceIndex),
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.430280" } },
                };
                response.Add(processMessage);
            }
            #endregion // VR.ATO.GEN.430280

            #region VR.ATO.GEN.430297

            /*  VR.ATO.GEN.430297
            Invalid context. The number of Intermediary contexts must not exceed 1
    
            Legacy Rule Format:
            IF LENGTH(bafot.xx.xx:RegulatoryDisclosures.GeneralInformationAboutFinancialStatements.Text) > 3000
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            Length(^PTR315) > 3000
    
            Data Elements:
    
            RP:^PTR315 = bafot.02.00:RegulatoryDisclosures.GeneralInformationAboutFinancialStatements.Text
            */
            assertion = (Length(report.PTR315) > 3000);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.430297",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"The Other attachments schedule must not exceed 3000 characters",
                    LongDescription = @"The 'Other attachments' schedule is represented by the 'Regulatory Disclosures. General Information About Financial Statements.Text' element",
                    Location = "/xbrli:xbrl/bafot.02.00:RegulatoryDisclosures.GeneralInformationAboutFinancialStatements.Text",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.430297" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "PTR315", Value = report.PTR315 });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.GEN.430297

            #region VR.ATO.GEN.430302

            /*  VR.ATO.GEN.430302
            Invalid context. The number of Reporting party: TOFA contexts must not exceed 1
    
            Legacy Rule Format:
            IF COUNT(CONTEXT(RP.TOFA)) > 1 
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            Count(^Context) > 1
    
            Data Elements:
    
            ^Context = RP:RP.TOFA
            */
            assertion = (Count(report.RPTOFACount) > 1);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.430302",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Invalid context. The number of Reporting party: TOFA contexts must not exceed 1",
                    Location = "/xbrli:xbrl/xbrli:context" + OccurrenceIndex(report.RPTOFALastOccurrenceIndex),
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.430302" } },
                };
                response.Add(processMessage);
            }
            #endregion // VR.ATO.GEN.430302

            #region VR.ATO.GEN.430312

            /*  VR.ATO.GEN.430312
            If an Intermediary context exists, then a Tax Agent (Intermediary) contact telephone must be present
    
            Legacy Rule Format:
            IF CONTEXT(INT) <> NULL AND INT:pyde.xx.xx:ElectronicContact.Telephone.Minimal.Number = NULL
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            ^Context <> NULL AND ^PTR173 = NULL
    
            Data Elements:
    
            INT:^PTR173 = pyde.02.00:ElectronicContact.Telephone.Minimal.Number WHERE [ElectronicContact.Telephone.Usage.Code='03']
    
            INT:^Context = INT
            */
            assertion = (report.INTExists != false && report.PTR173 == null);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.430312",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Tax Agent (Intermediary) contact telephone must be present",
                    Location = "/xbrli:xbrl/phone1.02.00:ElectronicContactTelephone" + OccurrenceIndex(report.ElectronicContactTelephoneCollection_03_INTOccurrenceIndex) + "/pyde.02.00:ElectronicContact.Telephone.Minimal.Number",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.430312" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "PTR173", Value = report.PTR173 });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.GEN.430312

            #region VR.ATO.GEN.430318

            /*  VR.ATO.GEN.430318
            ABN in the context definition must pass the ABN algorithm check
    
            Legacy Rule Format:
            IF (entity.identifier.ABN <> NULLORBLANK) AND (ABNALGORITHM(entity.identifier.ABN) = FALSE)
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            (^ABN <> BLANK) AND (FailsABNAlgorithm(^ABN))
    
            Data Elements:
    
            ^ABN = INT
            */
            assertion = (string.IsNullOrWhiteSpace(report.INTIdentifierABN) != true && FailsABNAlgorithm(report.INTIdentifierABN));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.430318",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"ABN in context must be a valid ABN",
                    Location = "/xbrli:xbrl/xbrli:context" + OccurrenceIndex(report.INTOccurrenceIndex) + "/xbrli:entity/xbrli:identifier",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.430318" } },
                };
                response.Add(processMessage);
            }
            #endregion // VR.ATO.GEN.430318

            #region VR.ATO.GEN.432417

            /*  VR.ATO.GEN.432417
            ABN for reporting party must be provided when there is no intermediary
    
            Legacy Rule Format:
            IF CONTEXT(INT) = NULL AND RP:pyid.xx.xx:Identifiers.AustralianBusinessNumber.Identifier = NULLORBLANK
                    RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            ^Context = NULL AND ^PTR7 = BLANK
    
            Data Elements:
    
            RP:^PTR7 = pyid.02.00:Identifiers.AustralianBusinessNumber.Identifier
    
            RP:^Context = INT
            */
            assertion = (report.INTExists == false && string.IsNullOrWhiteSpace(report.PTR7) == true);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.432417",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"ABN for reporting party must be provided when there is no intermediary",
                    Location = "/xbrli:xbrl/pyid.02.00:Identifiers.AustralianBusinessNumber.Identifier",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.432417" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "PTR7", Value = report.PTR7 });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.GEN.432417

            #region VR.ATO.GEN.434147

            /*  VR.ATO.GEN.434147
            State and post code must be present for an Australian address
    
            Legacy Rule Format:
            WHERE IN TUPLE(addressdetails2.xx.xx:AddressDetails)
            IF pyde.xx.xx:AddressDetails.OverseasAddressIndicator = FALSE AND (pyde.xx.xx:AddressDetails.StateOrTerritory.Code = NULLORBLANK OR pyde.xx.xx:AddressDetails.Postcode.Text = NULLORBLANK)
               RETURN VALIDATION MESSAGE
            END IF

            Technical Business Rule Format:
            (^AddressDetails <> NULL) AND (^PTR222 = FALSE AND (^PTR119 = BLANK OR ^PTR120 = BLANK))
    
            Data Elements:
    
            RP:^PTR222 = pyde.02.00:AddressDetails.OverseasAddress.Indicator WHERE [AddressDetails.Usage.Code='BUS'].[AddressDetails.Currency.Code='C']
    
            RP:^PTR119 = pyde.02.00:AddressDetails.StateOrTerritory.Code WHERE [AddressDetails.Usage.Code='BUS'].[AddressDetails.Currency.Code='C']
    
            RP:^PTR120 = pyde.02.00:AddressDetails.Postcode.Text WHERE [AddressDetails.Usage.Code='BUS'].[AddressDetails.Currency.Code='C']
    
            ^AddressDetails = address2.02.02:AddressDetails
            */
            assertion = (report.AddressDetailsCollection_BUS_C_RPExists != false && (report.PTR222 == false && (string.IsNullOrWhiteSpace(report.PTR119) == true || string.IsNullOrWhiteSpace(report.PTR120) == true)));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.434147",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"State and post code must be present for an Australian address",
                    LongDescription = @"If the overseas indicator is no (false), then an Australian state code and post code must be present",
                    Location = "/xbrli:xbrl/address2.02.02:AddressDetails" + OccurrenceIndex(report.AddressDetailsCollection_BUS_C_RPOccurrenceIndex) + "/pyde.02.00:AddressDetails.OverseasAddress.Indicator",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.434147" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "PTR222", Value = GetValueOrEmpty(report.PTR222) });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "PTR119", Value = report.PTR119 });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "PTR120", Value = report.PTR120 });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.GEN.434147

            #region VR.ATO.GEN.434147

            /*  VR.ATO.GEN.434147
            State and post code must be present for an Australian address
    
            Legacy Rule Format:
            WHERE IN TUPLE(addressdetails2.xx.xx:AddressDetails)
            IF pyde.xx.xx:AddressDetails.OverseasAddressIndicator = FALSE AND (pyde.xx.xx:AddressDetails.StateOrTerritory.Code = NULLORBLANK OR pyde.xx.xx:AddressDetails.Postcode.Text = NULLORBLANK)
               RETURN VALIDATION MESSAGE
            END IF

            Technical Business Rule Format:
            (^AddressDetails <> NULL) AND (^PTR216 = FALSE AND (^PTR11 = BLANK OR ^PTR12 = BLANK))
    
            Data Elements:
    
            RP:^PTR216 = pyde.02.00:AddressDetails.OverseasAddress.Indicator WHERE [AddressDetails.Usage.Code='POS'].[AddressDetails.Currency.Code='C']
    
            RP:^PTR11 = pyde.02.00:AddressDetails.StateOrTerritory.Code WHERE [AddressDetails.Usage.Code='POS'].[AddressDetails.Currency.Code='C']
    
            RP:^PTR12 = pyde.02.00:AddressDetails.Postcode.Text WHERE [AddressDetails.Usage.Code='POS'].[AddressDetails.Currency.Code='C']
    
            ^AddressDetails = address2.02.02:AddressDetails
            */
            assertion = (report.AddressDetailsCollection_POS_C_RPExists != false && (report.PTR216 == false && (string.IsNullOrWhiteSpace(report.PTR11) == true || string.IsNullOrWhiteSpace(report.PTR12) == true)));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.434147",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"State and post code must be present for an Australian address",
                    LongDescription = @"If the overseas indicator is no (false), then an Australian state code and post code must be present",
                    Location = "/xbrli:xbrl/address2.02.02:AddressDetails" + OccurrenceIndex(report.AddressDetailsCollection_POS_C_RPOccurrenceIndex) + "/pyde.02.00:AddressDetails.OverseasAddress.Indicator",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.434147" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "PTR216", Value = GetValueOrEmpty(report.PTR216) });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "PTR11", Value = report.PTR11 });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "PTR12", Value = report.PTR12 });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.GEN.434147

            #region VR.ATO.GEN.434147

            /*  VR.ATO.GEN.434147
            State and post code must be present for an Australian address
    
            Legacy Rule Format:
            WHERE IN TUPLE(addressdetails2.xx.xx:AddressDetails)
            IF pyde.xx.xx:AddressDetails.OverseasAddressIndicator = FALSE AND (pyde.xx.xx:AddressDetails.StateOrTerritory.Code = NULLORBLANK OR pyde.xx.xx:AddressDetails.Postcode.Text = NULLORBLANK)
               RETURN VALIDATION MESSAGE
            END IF

            Technical Business Rule Format:
            (^AddressDetails <> NULL) AND (^PTR219 = FALSE AND (^PTR16 = BLANK OR ^PTR17 = BLANK))
    
            Data Elements:
    
            RP:^PTR219 = pyde.02.00:AddressDetails.OverseasAddress.Indicator WHERE [AddressDetails.Usage.Code='POS'].[AddressDetails.Currency.Code='P']
    
            RP:^PTR16 = pyde.02.00:AddressDetails.StateOrTerritory.Code WHERE [AddressDetails.Usage.Code='POS'].[AddressDetails.Currency.Code='P']
    
            RP:^PTR17 = pyde.02.00:AddressDetails.Postcode.Text WHERE [AddressDetails.Usage.Code='POS'].[AddressDetails.Currency.Code='P']
    
            ^AddressDetails = address2.02.02:AddressDetails
            */
            assertion = (report.AddressDetailsCollection_POS_P_RPExists != false && (report.PTR219 == false && (string.IsNullOrWhiteSpace(report.PTR16) == true || string.IsNullOrWhiteSpace(report.PTR17) == true)));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.434147",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"State and post code must be present for an Australian address",
                    LongDescription = @"If the overseas indicator is no (false), then an Australian state code and post code must be present",
                    Location = "/xbrli:xbrl/address2.02.02:AddressDetails" + OccurrenceIndex(report.AddressDetailsCollection_POS_P_RPOccurrenceIndex) + "/pyde.02.00:AddressDetails.OverseasAddress.Indicator",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.434147" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "PTR219", Value = GetValueOrEmpty(report.PTR219) });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "PTR16", Value = report.PTR16 });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "PTR17", Value = report.PTR17 });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.GEN.434147

            #region VR.ATO.GEN.434223

            /*  VR.ATO.GEN.434223
            ABN must pass the ABN algorithm
    
            Legacy Rule Format:
            IF (pyid.xx.xx:Identifiers.AustralianBusinessNumber.Identifer <> NULLORBLANK) and (ABNALGORITHM(pyid.xx.xx:Identifiers.AustralianBusinessNumber.Identifier) = FALSE)
            RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            (^PTR25 <> BLANK) and (FailsABNAlgorithm(^PTR25))
    
            Data Elements:
    
            RP:^PTR25 = pyid.02.00:Identifiers.AustralianBusinessNumber.Identifier
            */
            assertion = (string.IsNullOrWhiteSpace(report.PTR25) != true && FailsABNAlgorithm(report.PTR25));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.000477",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"ABN is invalid",
                    Location = "/xbrli:xbrl/ptr.0006.lodge.req.02.00:MailRecipient/pyid.02.00:Identifiers.AustralianBusinessNumber.Identifier",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.434223" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "PTR25", Value = report.PTR25 });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.GEN.434223

            #region VR.ATO.GEN.434223

            /*  VR.ATO.GEN.434223
            ABN must pass the ABN algorithm
    
            Legacy Rule Format:
            IF (pyid.xx.xx:Identifiers.AustralianBusinessNumber.Identifer <> NULLORBLANK) and (ABNALGORITHM(pyid.xx.xx:Identifiers.AustralianBusinessNumber.Identifier) = FALSE)
            RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            (^PTR7 <> BLANK) and (FailsABNAlgorithm(^PTR7))
    
            Data Elements:
    
            RP:^PTR7 = pyid.02.00:Identifiers.AustralianBusinessNumber.Identifier
            */
            assertion = (string.IsNullOrWhiteSpace(report.PTR7) != true && FailsABNAlgorithm(report.PTR7));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.000477",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"ABN is invalid",
                    Location = "/xbrli:xbrl/pyid.02.00:Identifiers.AustralianBusinessNumber.Identifier",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.434223" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "PTR7", Value = report.PTR7 });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.GEN.434223

            #region VR.ATO.GEN.438010

            /*  VR.ATO.GEN.438010
            If the income includes personal services income (PSI) then total amount of income and deductions, results test satisfied and business determination indicators must all be completed
    
            Legacy Rule Format:
            IF (RP:bafpr1.xx.xx:Income.PersonalServicesIncomeIncluded.Indicator = TRUE) AND ((RP:bafpr3.xx.xx:Income.PersonalServicesIncome.Total.Amount = NULL) OR (RP:bafpr3.xx.xx:Expense.PersonalServicesIncome.Deduction.Amount = NULL) OR (RP:rvctc2.xx.xx:IncomeTax.PersonalServicesIncome.IndividualResultsTestSatisfied.Indicator = NULL) OR (RP:rvctc2.xx.xx:IncomeTax.PersonalServicesIncome.IndividualPersonalServiceBusinessDeterminationHeld.Indicator = NULL))
              RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            (^PTR110 = TRUE) AND ((^PTR265 = NULL) OR (^PTR266 = NULL) OR (^PTR268 = NULL) OR (^PTR271 = NULL))
    
            Data Elements:
    
            RP:^PTR110 = bafpr1.02.13:Income.PersonalServicesIncomeIncluded.Indicator
    
            RP:^PTR265 = bafpr3.02.11:Income.PersonalServicesIncome.Total.Amount
    
            RP:^PTR266 = bafpr3.02.11:Expense.PersonalServicesIncome.Deduction.Amount
    
            RP:^PTR268 = rvctc2.02.15:IncomeTax.PersonalServicesIncome.IndividualResultsTestSatisfied.Indicator
    
            RP:^PTR271 = rvctc2.02.15:IncomeTax.PersonalServicesIncome.IndividualPersonalServiceBusinessDeterminationHeld.Indicator
            */
            assertion = (report.PTR110 == true && (report.PTR265 == null || report.PTR266 == null || report.PTR268 == null || report.PTR271 == null));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.438010",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Personal services income section completed incorrectly",
                    LongDescription = @"If the income includes personal services income (PSI) then total amount of income and deductions, results test satisfied and business determination indicators must all be completed",
                    Location = "/xbrli:xbrl/bafpr1.02.13:Income.PersonalServicesIncomeIncluded.Indicator",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.438010" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "PTR110", Value = GetValueOrEmpty(report.PTR110) });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "PTR265", Value = report.PTR265.GetValueOrDefault().ToString() });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "PTR266", Value = report.PTR266.GetValueOrDefault().ToString() });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "PTR268", Value = GetValueOrEmpty(report.PTR268) });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "PTR271", Value = GetValueOrEmpty(report.PTR271) });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.GEN.438010

            #region VR.ATO.GEN.438011

            /*  VR.ATO.GEN.438011
            If the income does not include personal services income (PSI) then total amount of income and deductions, results test satisfied , business determination,  unrelated clients test, employment test and business premises test indicators must not be completed
    
            Legacy Rule Format:
            IF (RP:bafpr1.xx.xx:Income.PersonalServicesIncomeIncluded.Indicator = FALSE) AND ((RP:bafpr3.xx.xx:Income.PersonalServicesIncome.Total.Amount <> NULL) OR (RP:bafpr3.xx.xx:Expense.PersonalServicesIncome.Deduction.Amount <> NULL) OR (RP:rvctc2.xx.xx:IncomeTax.PersonalServicesIncome.IndividualResultsTestSatisfied.Indicator <> NULL) OR (RP:rvctc2.xx.xx:IncomeTax.PersonalServicesIncome.IndividualPersonalServiceBusinessDeterminationHeld.Indicator <> NULL) OR (RP:rvctc2.xx.xx:IncomeTax.PersonalServicesIncome.UnrelatedClientsTestSatisfied.Indicator <> NULL) OR (RP:rvctc2.xx.xx:IncomeTax.PersonalServicesIncome.EmploymentTestSatisfied.Indicator <> NULL) OR (RP:rvctc2.xx.xx:IncomeTax.PersonalServicesIncome.BusinessPremisesTestSatisfied.Indicator <> NULL))
              RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            (^PTR110 = FALSE) AND ((^PTR265 <> NULL) OR (^PTR266 <> NULL) OR (^PTR268 <> NULL) OR (^PTR271 <> NULL) OR (^PTR276 <> NULL) OR (^PTR272 <> NULL) OR (^PTR275 <> NULL))
    
            Data Elements:
    
            RP:^PTR110 = bafpr1.02.13:Income.PersonalServicesIncomeIncluded.Indicator
    
            RP:^PTR265 = bafpr3.02.11:Income.PersonalServicesIncome.Total.Amount
    
            RP:^PTR266 = bafpr3.02.11:Expense.PersonalServicesIncome.Deduction.Amount
    
            RP:^PTR268 = rvctc2.02.15:IncomeTax.PersonalServicesIncome.IndividualResultsTestSatisfied.Indicator
    
            RP:^PTR271 = rvctc2.02.15:IncomeTax.PersonalServicesIncome.IndividualPersonalServiceBusinessDeterminationHeld.Indicator
    
            RP:^PTR276 = rvctc2.02.15:IncomeTax.PersonalServicesIncome.UnrelatedClientsTestSatisfied.Indicator
    
            RP:^PTR272 = rvctc2.02.15:IncomeTax.PersonalServicesIncome.EmploymentTestSatisfied.Indicator
    
            RP:^PTR275 = rvctc2.02.15:IncomeTax.PersonalServicesIncome.BusinessPremisesTestSatisfied.Indicator
            */
            assertion = (report.PTR110 == false && (report.PTR265 != null || report.PTR266 != null || report.PTR268 != null || report.PTR271 != null || report.PTR276 != null || report.PTR272 != null || report.PTR275 != null));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.438011",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Personal services income section completed incorrectly",
                    LongDescription = @"If the income does not include personal services income (PSI) then total amount of income and deductions,results test satisfied,business determination,unrelated clients test,employment test and business premises test indicators must not be completed",
                    Location = "/xbrli:xbrl/bafpr1.02.13:Income.PersonalServicesIncomeIncluded.Indicator",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.438011" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "PTR110", Value = GetValueOrEmpty(report.PTR110) });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "PTR265", Value = report.PTR265.GetValueOrDefault().ToString() });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "PTR266", Value = report.PTR266.GetValueOrDefault().ToString() });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "PTR268", Value = GetValueOrEmpty(report.PTR268) });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "PTR271", Value = GetValueOrEmpty(report.PTR271) });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "PTR276", Value = GetValueOrEmpty(report.PTR276) });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "PTR272", Value = GetValueOrEmpty(report.PTR272) });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "PTR275", Value = GetValueOrEmpty(report.PTR275) });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.GEN.438011

            #region VR.ATO.GEN.438012

            /*  VR.ATO.GEN.438012
            Monetary field must be unsigned, maximum of 11 numeric characters, no decimals
    
            Legacy Rule Format:
            IF (bafpr3.xx.xx:Income.PersonalServicesIncome.Total.Amount <> NULL) AND (bafpr3.xx.xx:Income.PersonalServicesIncome.Total.Amount <> MONETARY(U,11,0))
              RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            NotMonetary(^PTR265, 'U', 11, 0)
    
            Data Elements:
    
            RP:^PTR265 = bafpr3.02.11:Income.PersonalServicesIncome.Total.Amount
            */
            assertion = NotMonetary(report.PTR265, @"U", 11, 0);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.400011",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Field incorrect format. Exceeds maximum digits or decimals and/or contains illegal characters",
                    LongDescription = @"Field is in incorrect format for a monetary field. Field either exceeds maximum digits or decimals and/or contains illegal characters and/or for a signed field does not have the sign as the first character",
                    Location = "/xbrli:xbrl/bafpr3.02.11:Income.PersonalServicesIncome.Total.Amount",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.438012" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "PTR265", Value = report.PTR265.GetValueOrDefault().ToString() });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.GEN.438012

            #region VR.ATO.GEN.438013

            /*  VR.ATO.GEN.438013
            Monetary field must be unsigned, maximum of 11 numeric characters, no decimals
    
            Legacy Rule Format:
            IF (bafpr3.xx.xx:Expense.PersonalServicesIncome.Deduction.Amount <> NULL) AND (bafpr3.xx.xx:Expense.PersonalServicesIncome.Deduction.Amount <> MONETARY(U,11,0))
              RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            NotMonetary(^PTR266, 'U', 11, 0)
    
            Data Elements:
    
            RP:^PTR266 = bafpr3.02.11:Expense.PersonalServicesIncome.Deduction.Amount
            */
            assertion = NotMonetary(report.PTR266, @"U", 11, 0);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.400011",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Field incorrect format. Exceeds maximum digits or decimals and/or contains illegal characters",
                    LongDescription = @"Field is in incorrect format for a monetary field. Field either exceeds maximum digits or decimals and/or contains illegal characters and/or for a signed field does not have the sign as the first character",
                    Location = "/xbrli:xbrl/bafpr3.02.11:Expense.PersonalServicesIncome.Deduction.Amount",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.438013" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "PTR266", Value = report.PTR266.GetValueOrDefault().ToString() });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.GEN.438013

            #region VR.ATO.GEN.438014

            /*  VR.ATO.GEN.438014
            Monetary field must be unsigned, maximum of 11 numeric characters, no decimals
    
            Legacy Rule Format:
            IF (bafpo7.xx.xx:Assets.DepreciatingAssets.AdjustableValuesTotal.Amount <> NULL) AND (bafpo7.xx.xx:Assets.DepreciatingAssets.AdjustableValuesTotal.Amount <> MONETARY(U,11,0))
              RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            NotMonetary(^PTR279, 'U', 11, 0)
    
            Data Elements:
    
            RP:^PTR279 = bafpo7.02.02:Assets.DepreciatingAssets.AdjustableValuesTotal.Amount
            */
            assertion = NotMonetary(report.PTR279, @"U", 11, 0);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.400011",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Field incorrect format. Exceeds maximum digits or decimals and/or contains illegal characters",
                    LongDescription = @"Field is in incorrect format for a monetary field. Field either exceeds maximum digits or decimals and/or contains illegal characters and/or for a signed field does not have the sign as the first character",
                    Location = "/xbrli:xbrl/bafpo7.02.02:Assets.DepreciatingAssets.AdjustableValuesTotal.Amount",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.438014" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "PTR279", Value = report.PTR279.GetValueOrDefault().ToString() });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.GEN.438014

            #region VR.ATO.GEN.438015

            /*  VR.ATO.GEN.438015
            Monetary field must be unsigned, maximum of 11 numeric characters, no decimals
    
            Legacy Rule Format:
            IF (bafpr3.xx.xx:Income.DepreciatingAssets.IntangibleBalancingAdjustment.Amount <> NULL) AND (bafpr3.xx.xx:Income.DepreciatingAssets.IntangibleBalancingAdjustment.Amount <> MONETARY(U,11,0))
              RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            NotMonetary(^PTR282, 'U', 11, 0)
    
            Data Elements:
    
            RP:^PTR282 = bafpr3.02.11:Income.DepreciatingAssets.IntangibleBalancingAdjustment.Amount
            */
            assertion = NotMonetary(report.PTR282, @"U", 11, 0);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.400011",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Field incorrect format. Exceeds maximum digits or decimals and/or contains illegal characters",
                    LongDescription = @"Field is in incorrect format for a monetary field. Field either exceeds maximum digits or decimals and/or contains illegal characters and/or for a signed field does not have the sign as the first character",
                    Location = "/xbrli:xbrl/bafpr3.02.11:Income.DepreciatingAssets.IntangibleBalancingAdjustment.Amount",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.438015" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "PTR282", Value = report.PTR282.GetValueOrDefault().ToString() });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.GEN.438015

            #region VR.ATO.GEN.438016

            /*  VR.ATO.GEN.438016
            Monetary field must be unsigned, maximum of 11 numeric characters, no decimals
    
            Legacy Rule Format:
            IF (bafpr3.xx.xx:Expense.DepreciatingAssets.IntangibleBalancingAdjustment.Amount <> NULL) AND (bafpr3.xx.xx:Expense.DepreciatingAssets.IntangibleBalancingAdjustment.Amount <> MONETARY(U,11,0))
              RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            NotMonetary(^PTR281, 'U', 11, 0)
    
            Data Elements:
    
            RP:^PTR281 = bafpr3.02.11:Expense.DepreciatingAssets.IntangibleBalancingAdjustment.Amount
            */
            assertion = NotMonetary(report.PTR281, @"U", 11, 0);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.400011",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Field incorrect format. Exceeds maximum digits or decimals and/or contains illegal characters",
                    LongDescription = @"Field is in incorrect format for a monetary field. Field either exceeds maximum digits or decimals and/or contains illegal characters and/or for a signed field does not have the sign as the first character",
                    Location = "/xbrli:xbrl/bafpr3.02.11:Expense.DepreciatingAssets.IntangibleBalancingAdjustment.Amount",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.438016" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "PTR281", Value = report.PTR281.GetValueOrDefault().ToString() });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.GEN.438016

            #region VR.ATO.GEN.438029

            /*  VR.ATO.GEN.438029
            Entity identifier scheme for ReportingParty must be"www.ato.gov.au/tfn"
    
            Legacy Rule Format:
            IF (RprtPyType.xx.xx:ReportPartyTypeDimension = “RprtPyType.xx.xx:ReportingParty”) AND (entity.identifier.scheme <> "http://www.ato.gov.au/tfn")
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            (^Context <> NULL) AND (^IdentifierScheme <> 'http://www.ato.gov.au/tfn')
    
            Data Elements:
    
            ^IdentifierScheme = RP
    
            ^Context = RP
            */
            assertion = (report.RPExists != false && report.RPIdentifierScheme != @"http://www.ato.gov.au/tfn");
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.438029",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Entity identifier scheme for ReportingParty must be""http://www.ato.gov.au/tfn""",
                    Location = "/xbrli:xbrl/xbrli:context" + OccurrenceIndex(report.RPOccurrenceIndex) + "/xbrli:entity/xbrli:identifier/@scheme",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.438029" } },
                };
                response.Add(processMessage);
            }
            #endregion // VR.ATO.GEN.438029

            #region VR.ATO.GEN.438029

            /*  VR.ATO.GEN.438029
            Entity identifier scheme for ReportingParty must be"www.ato.gov.au/tfn"
    
            Legacy Rule Format:
            IF (RprtPyType.xx.xx:ReportPartyTypeDimension = “RprtPyType.xx.xx:ReportingParty”) AND (entity.identifier.scheme <> "http://www.ato.gov.au/tfn")
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            (^Context <> NULL) AND (^IdentifierScheme <> 'http://www.ato.gov.au/tfn')
    
            Data Elements:
    
            ^IdentifierScheme = RP:RP.Closing(Instant)
    
            ^Context = RP:RP.Closing(Instant)
            */
            assertion = (report.RPClosingInstantExists != false && report.RPClosingInstantIdentifierScheme != @"http://www.ato.gov.au/tfn");
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.438029",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Entity identifier scheme for ReportingParty must be""http://www.ato.gov.au/tfn""",
                    Location = "/xbrli:xbrl/xbrli:context" + OccurrenceIndex(report.RPClosingInstantOccurrenceIndex) + "/xbrli:entity/xbrli:identifier/@scheme",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.438029" } },
                };
                response.Add(processMessage);
            }
            #endregion // VR.ATO.GEN.438029

            #region VR.ATO.GEN.438029

            /*  VR.ATO.GEN.438029
            Entity identifier scheme for ReportingParty must be"www.ato.gov.au/tfn"
    
            Legacy Rule Format:
            IF (RprtPyType.xx.xx:ReportPartyTypeDimension = “RprtPyType.xx.xx:ReportingParty”) AND (entity.identifier.scheme <> "http://www.ato.gov.au/tfn")
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            (^Context <> NULL) AND (^IdentifierScheme <> 'http://www.ato.gov.au/tfn')
    
            Data Elements:
    
            ^IdentifierScheme = RP:RP.GeneralPool
    
            ^Context = RP:RP.GeneralPool
            */
            assertion = (report.RPGeneralPoolExists != false && report.RPGeneralPoolIdentifierScheme != @"http://www.ato.gov.au/tfn");
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.438029",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Entity identifier scheme for ReportingParty must be""http://www.ato.gov.au/tfn""",
                    Location = "/xbrli:xbrl/xbrli:context" + OccurrenceIndex(report.RPGeneralPoolOccurrenceIndex) + "/xbrli:entity/xbrli:identifier/@scheme",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.438029" } },
                };
                response.Add(processMessage);
            }
            #endregion // VR.ATO.GEN.438029

            #region VR.ATO.GEN.438029

            /*  VR.ATO.GEN.438029
            Entity identifier scheme for ReportingParty must be"www.ato.gov.au/tfn"
    
            Legacy Rule Format:
            IF (RprtPyType.xx.xx:ReportPartyTypeDimension = “RprtPyType.xx.xx:ReportingParty”) AND (entity.identifier.scheme <> "http://www.ato.gov.au/tfn")
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            (^Context <> NULL) AND (^IdentifierScheme <> 'http://www.ato.gov.au/tfn')
    
            Data Elements:
    
            ^IdentifierScheme = RP:RP.JAUS
    
            ^Context = RP:RP.JAUS
            */
            assertion = (report.RPJAUSExists != false && report.RPJAUSIdentifierScheme != @"http://www.ato.gov.au/tfn");
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.438029",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Entity identifier scheme for ReportingParty must be""http://www.ato.gov.au/tfn""",
                    Location = "/xbrli:xbrl/xbrli:context" + OccurrenceIndex(report.RPJAUSOccurrenceIndex) + "/xbrli:entity/xbrli:identifier/@scheme",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.438029" } },
                };
                response.Add(processMessage);
            }
            #endregion // VR.ATO.GEN.438029

            #region VR.ATO.GEN.438029

            /*  VR.ATO.GEN.438029
            Entity identifier scheme for ReportingParty must be"www.ato.gov.au/tfn"
    
            Legacy Rule Format:
            IF (RprtPyType.xx.xx:ReportPartyTypeDimension = “RprtPyType.xx.xx:ReportingParty”) AND (entity.identifier.scheme <> "http://www.ato.gov.au/tfn")
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            (^Context <> NULL) AND (^IdentifierScheme <> 'http://www.ato.gov.au/tfn')
    
            Data Elements:
    
            ^IdentifierScheme = RP.JAUS:RP.JAUS.NonPrim
    
            ^Context = RP.JAUS:RP.JAUS.NonPrim
            */
            assertion = (report.RPJAUSNonPrimExists != false && report.RPJAUSNonPrimIdentifierScheme != @"http://www.ato.gov.au/tfn");
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.438029",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Entity identifier scheme for ReportingParty must be""http://www.ato.gov.au/tfn""",
                    Location = "/xbrli:xbrl/xbrli:context" + OccurrenceIndex(report.RPJAUSNonPrimOccurrenceIndex) + "/xbrli:entity/xbrli:identifier/@scheme",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.438029" } },
                };
                response.Add(processMessage);
            }
            #endregion // VR.ATO.GEN.438029

            #region VR.ATO.GEN.438029

            /*  VR.ATO.GEN.438029
            Entity identifier scheme for ReportingParty must be"www.ato.gov.au/tfn"
    
            Legacy Rule Format:
            IF (RprtPyType.xx.xx:ReportPartyTypeDimension = “RprtPyType.xx.xx:ReportingParty”) AND (entity.identifier.scheme <> "http://www.ato.gov.au/tfn")
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            (^Context <> NULL) AND (^IdentifierScheme <> 'http://www.ato.gov.au/tfn')
    
            Data Elements:
    
            ^IdentifierScheme = RP.JAUS:RP.JAUS.Prim
    
            ^Context = RP.JAUS:RP.JAUS.Prim
            */
            assertion = (report.RPJAUSPrimExists != false && report.RPJAUSPrimIdentifierScheme != @"http://www.ato.gov.au/tfn");
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.438029",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Entity identifier scheme for ReportingParty must be""http://www.ato.gov.au/tfn""",
                    Location = "/xbrli:xbrl/xbrli:context" + OccurrenceIndex(report.RPJAUSPrimOccurrenceIndex) + "/xbrli:entity/xbrli:identifier/@scheme",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.438029" } },
                };
                response.Add(processMessage);
            }
            #endregion // VR.ATO.GEN.438029

            #region VR.ATO.GEN.438029

            /*  VR.ATO.GEN.438029
            Entity identifier scheme for ReportingParty must be"www.ato.gov.au/tfn"
    
            Legacy Rule Format:
            IF (RprtPyType.xx.xx:ReportPartyTypeDimension = “RprtPyType.xx.xx:ReportingParty”) AND (entity.identifier.scheme <> "http://www.ato.gov.au/tfn")
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            (^Context <> NULL) AND (^IdentifierScheme <> 'http://www.ato.gov.au/tfn')
    
            Data Elements:
    
            ^IdentifierScheme = RP:RP.JFOREIGN
    
            ^Context = RP:RP.JFOREIGN
            */
            assertion = (report.RPJFOREIGNExists != false && report.RPJFOREIGNIdentifierScheme != @"http://www.ato.gov.au/tfn");
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.438029",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Entity identifier scheme for ReportingParty must be""http://www.ato.gov.au/tfn""",
                    Location = "/xbrli:xbrl/xbrli:context" + OccurrenceIndex(report.RPJFOREIGNOccurrenceIndex) + "/xbrli:entity/xbrli:identifier/@scheme",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.438029" } },
                };
                response.Add(processMessage);
            }
            #endregion // VR.ATO.GEN.438029

            #region VR.ATO.GEN.438029

            /*  VR.ATO.GEN.438029
            Entity identifier scheme for ReportingParty must be"www.ato.gov.au/tfn"
    
            Legacy Rule Format:
            IF (RprtPyType.xx.xx:ReportPartyTypeDimension = “RprtPyType.xx.xx:ReportingParty”) AND (entity.identifier.scheme <> "http://www.ato.gov.au/tfn")
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            (^Context <> NULL) AND (^IdentifierScheme <> 'http://www.ato.gov.au/tfn')
    
            Data Elements:
    
            ^IdentifierScheme = RP.JFOREIGN:RP.JFOREIGN.CFCLC
    
            ^Context = RP.JFOREIGN:RP.JFOREIGN.CFCLC
            */
            assertion = (report.RPJFOREIGNCFCLCExists != false && report.RPJFOREIGNCFCLCIdentifierScheme != @"http://www.ato.gov.au/tfn");
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.438029",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Entity identifier scheme for ReportingParty must be""http://www.ato.gov.au/tfn""",
                    Location = "/xbrli:xbrl/xbrli:context" + OccurrenceIndex(report.RPJFOREIGNCFCLCOccurrenceIndex) + "/xbrli:entity/xbrli:identifier/@scheme",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.438029" } },
                };
                response.Add(processMessage);
            }
            #endregion // VR.ATO.GEN.438029

            #region VR.ATO.GEN.438029

            /*  VR.ATO.GEN.438029
            Entity identifier scheme for ReportingParty must be"www.ato.gov.au/tfn"
    
            Legacy Rule Format:
            IF (RprtPyType.xx.xx:ReportPartyTypeDimension = “RprtPyType.xx.xx:ReportingParty”) AND (entity.identifier.scheme <> "http://www.ato.gov.au/tfn")
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            (^Context <> NULL) AND (^IdentifierScheme <> 'http://www.ato.gov.au/tfn')
    
            Data Elements:
    
            ^IdentifierScheme = RP.JFOREIGN:RP.JFOREIGN.CFCUC
    
            ^Context = RP.JFOREIGN:RP.JFOREIGN.CFCUC
            */
            assertion = (report.RPJFOREIGNCFCUCExists != false && report.RPJFOREIGNCFCUCIdentifierScheme != @"http://www.ato.gov.au/tfn");
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.438029",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Entity identifier scheme for ReportingParty must be""http://www.ato.gov.au/tfn""",
                    Location = "/xbrli:xbrl/xbrli:context" + OccurrenceIndex(report.RPJFOREIGNCFCUCOccurrenceIndex) + "/xbrli:entity/xbrli:identifier/@scheme",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.438029" } },
                };
                response.Add(processMessage);
            }
            #endregion // VR.ATO.GEN.438029

            #region VR.ATO.GEN.438029

            /*  VR.ATO.GEN.438029
            Entity identifier scheme for ReportingParty must be"www.ato.gov.au/tfn"
    
            Legacy Rule Format:
            IF (RprtPyType.xx.xx:ReportPartyTypeDimension = “RprtPyType.xx.xx:ReportingParty”) AND (entity.identifier.scheme <> "http://www.ato.gov.au/tfn")
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            (^Context <> NULL) AND (^IdentifierScheme <> 'http://www.ato.gov.au/tfn')
    
            Data Elements:
    
            ^IdentifierScheme = RP:RP.LowCost
    
            ^Context = RP:RP.LowCost
            */
            assertion = (report.RPLowCostExists != false && report.RPLowCostIdentifierScheme != @"http://www.ato.gov.au/tfn");
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.438029",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Entity identifier scheme for ReportingParty must be""http://www.ato.gov.au/tfn""",
                    Location = "/xbrli:xbrl/xbrli:context" + OccurrenceIndex(report.RPLowCostOccurrenceIndex) + "/xbrli:entity/xbrli:identifier/@scheme",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.438029" } },
                };
                response.Add(processMessage);
            }
            #endregion // VR.ATO.GEN.438029

            #region VR.ATO.GEN.438029

            /*  VR.ATO.GEN.438029
            Entity identifier scheme for ReportingParty must be"www.ato.gov.au/tfn"
    
            Legacy Rule Format:
            IF (RprtPyType.xx.xx:ReportPartyTypeDimension = “RprtPyType.xx.xx:ReportingParty”) AND (entity.identifier.scheme <> "http://www.ato.gov.au/tfn")
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            (^Context <> NULL) AND (^IdentifierScheme <> 'http://www.ato.gov.au/tfn')
    
            Data Elements:
    
            ^IdentifierScheme = RP:RP.NonPrim
    
            ^Context = RP:RP.NonPrim
            */
            assertion = (report.RPNonPrimExists != false && report.RPNonPrimIdentifierScheme != @"http://www.ato.gov.au/tfn");
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.438029",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Entity identifier scheme for ReportingParty must be""http://www.ato.gov.au/tfn""",
                    Location = "/xbrli:xbrl/xbrli:context" + OccurrenceIndex(report.RPNonPrimOccurrenceIndex) + "/xbrli:entity/xbrli:identifier/@scheme",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.438029" } },
                };
                response.Add(processMessage);
            }
            #endregion // VR.ATO.GEN.438029

            #region VR.ATO.GEN.438029

            /*  VR.ATO.GEN.438029
            Entity identifier scheme for ReportingParty must be"www.ato.gov.au/tfn"
    
            Legacy Rule Format:
            IF (RprtPyType.xx.xx:ReportPartyTypeDimension = “RprtPyType.xx.xx:ReportingParty”) AND (entity.identifier.scheme <> "http://www.ato.gov.au/tfn")
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            (^Context <> NULL) AND (^IdentifierScheme <> 'http://www.ato.gov.au/tfn')
    
            Data Elements:
    
            ^IdentifierScheme = RP:RP.Opening(Instant)
    
            ^Context = RP:RP.Opening(Instant)
            */
            assertion = (report.RPOpeningInstantExists != false && report.RPOpeningInstantIdentifierScheme != @"http://www.ato.gov.au/tfn");
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.438029",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Entity identifier scheme for ReportingParty must be""http://www.ato.gov.au/tfn""",
                    Location = "/xbrli:xbrl/xbrli:context" + OccurrenceIndex(report.RPOpeningInstantOccurrenceIndex) + "/xbrli:entity/xbrli:identifier/@scheme",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.438029" } },
                };
                response.Add(processMessage);
            }
            #endregion // VR.ATO.GEN.438029

            #region VR.ATO.GEN.438029

            /*  VR.ATO.GEN.438029
            Entity identifier scheme for ReportingParty must be"www.ato.gov.au/tfn"
    
            Legacy Rule Format:
            IF (RprtPyType.xx.xx:ReportPartyTypeDimension = “RprtPyType.xx.xx:ReportingParty”) AND (entity.identifier.scheme <> "http://www.ato.gov.au/tfn")
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            (^Context <> NULL) AND (^IdentifierScheme <> 'http://www.ato.gov.au/tfn')
    
            Data Elements:
    
            ^IdentifierScheme = RP:RP.Prim
    
            ^Context = RP:RP.Prim
            */
            assertion = (report.RPPrimExists != false && report.RPPrimIdentifierScheme != @"http://www.ato.gov.au/tfn");
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.438029",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Entity identifier scheme for ReportingParty must be""http://www.ato.gov.au/tfn""",
                    Location = "/xbrli:xbrl/xbrli:context" + OccurrenceIndex(report.RPPrimOccurrenceIndex) + "/xbrli:entity/xbrli:identifier/@scheme",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.438029" } },
                };
                response.Add(processMessage);
            }
            #endregion // VR.ATO.GEN.438029

            #region VR.ATO.GEN.438029

            /*  VR.ATO.GEN.438029
            Entity identifier scheme for ReportingParty must be"www.ato.gov.au/tfn"
    
            Legacy Rule Format:
            IF (RprtPyType.xx.xx:ReportPartyTypeDimension = “RprtPyType.xx.xx:ReportingParty”) AND (entity.identifier.scheme <> "http://www.ato.gov.au/tfn")
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            (^Context <> NULL) AND (^IdentifierScheme <> 'http://www.ato.gov.au/tfn')
    
            Data Elements:
    
            ^IdentifierScheme = RP:RP.TOFA
    
            ^Context = RP:RP.TOFA
            */
            assertion = (report.RPTOFAExists != false && report.RPTOFAIdentifierScheme != @"http://www.ato.gov.au/tfn");
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.438029",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Entity identifier scheme for ReportingParty must be""http://www.ato.gov.au/tfn""",
                    Location = "/xbrli:xbrl/xbrli:context" + OccurrenceIndex(report.RPTOFAOccurrenceIndex) + "/xbrli:entity/xbrli:identifier/@scheme",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.438029" } },
                };
                response.Add(processMessage);
            }
            #endregion // VR.ATO.GEN.438029

            #region VR.ATO.GEN.438030

            /*  VR.ATO.GEN.438030
            Entity identifier scheme for Intermediary must be"www.ato.gov.au/abn"
    
            Legacy Rule Format:
            IF (RprtPyType.xx.xx:ReportPartyTypeDimension = “RprtPyType.xx.xx:Intermediary”) AND (entity.identifier.scheme <> “http://www.ato.gov.au/abn”)
                RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            ^Context <> NULL AND ^IdentifierScheme <> 'http://www.ato.gov.au/abn'
    
            Data Elements:
    
            ^IdentifierScheme = INT
    
            ^Context = INT
            */
            assertion = (report.INTExists != false && report.INTIdentifierScheme != @"http://www.ato.gov.au/abn");
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.438030",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Entity identifier scheme for Intermediary must be""http://www.ato.gov.au/abn""",
                    Location = "/xbrli:xbrl/xbrli:context" + OccurrenceIndex(report.INTOccurrenceIndex) + "/xbrli:entity/xbrli:identifier/@scheme",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.438030" } },
                };
                response.Add(processMessage);
            }
            #endregion // VR.ATO.GEN.438030

            #region VR.ATO.GEN.438035

            /*  VR.ATO.GEN.438035
            Unstructured name must not  contain invalid characters ~ ^ < > + , * 
            Invalid characters ~^<>+  have been removed from the techincal business rule as they will be picked by the XBRL validator.
    
            Legacy Rule Format:
            IF (pyde.xx.xx:PersonUnstructuredName.FullName.Text <> NULLORBLANK AND pyde.xx.xx:PersonUnstructuredName.FullName.Text CONTAINS SET(",","*"))
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            ContainsSet(^PTR242, '",","*"')
    
            Data Elements:
    
            INT:^PTR242 = pyde.02.00:PersonUnstructuredName.FullName.Text WHERE [PersonUnstructuredName.Usage.Code='Contact']
            */
            assertion = IsMatch(report.PTR242, @",|\*", RegexOptions.IgnoreCase);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.438035",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Unstructured name must not contain invalid characters ~ ^ < > + , *",
                    Location = "/xbrli:xbrl/prsnunstrcnm1.02.01:PersonUnstructuredName" + OccurrenceIndex(report.PersonUnstructuredNameCollection_Contact_INTOccurrenceIndex) + "/pyde.02.00:PersonUnstructuredName.FullName.Text",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.438035" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "PTR242", Value = report.PTR242 });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.GEN.438035

            #region VR.ATO.GEN.438035

            /*  VR.ATO.GEN.438035
            Unstructured name must not  contain invalid characters ~ ^ < > + , * 
            Invalid characters ~^<>+  have been removed from the techincal business rule as they will be picked by the XBRL validator.
    
            Legacy Rule Format:
            IF (pyde.xx.xx:PersonUnstructuredName.FullName.Text <> NULLORBLANK AND pyde.xx.xx:PersonUnstructuredName.FullName.Text CONTAINS SET(",","*"))
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            ContainsSet(^PTR314, '",","*"')
    
            Data Elements:
    
            INT:^PTR314 = pyde.02.00:PersonUnstructuredName.FullName.Text WHERE [PersonUnstructuredName.Usage.Code='DeclarationSignatory']
            */
            assertion = IsMatch(report.PTR314, @",|\*", RegexOptions.IgnoreCase);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.438035",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Unstructured name must not contain invalid characters ~ ^ < > + , *",
                    Location = "/xbrli:xbrl/declaration2.02.01:Declaration" + OccurrenceIndex(report.DeclarationCollection_TrueAndCorrect_INTOccurrenceIndex) + "/prsnunstrcnm1.02.01:PersonUnstructuredName" + OccurrenceIndex(report.Declaration_PersonUnstructuredNameCollection_DeclarationSignatory_INTOccurrenceIndex) + "/pyde.02.00:PersonUnstructuredName.FullName.Text",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.438035" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "PTR314", Value = report.PTR314 });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.GEN.438035

            #region VR.ATO.GEN.438035

            /*  VR.ATO.GEN.438035
            Unstructured name must not  contain invalid characters ~ ^ < > + , * 
            Invalid characters ~^<>+  have been removed from the techincal business rule as they will be picked by the XBRL validator.
    
            Legacy Rule Format:
            IF (pyde.xx.xx:PersonUnstructuredName.FullName.Text <> NULLORBLANK AND pyde.xx.xx:PersonUnstructuredName.FullName.Text CONTAINS SET(",","*"))
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            ContainsSet(^PTR264, '",","*"')
    
            Data Elements:
    
            RP:^PTR264 = pyde.02.00:PersonUnstructuredName.FullName.Text WHERE [PersonUnstructuredName.Usage.Code='DeclarationSignatory']
            */
            assertion = IsMatch(report.PTR264, @",|\*", RegexOptions.IgnoreCase);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.438035",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Unstructured name must not contain invalid characters ~ ^ < > + , *",
                    Location = "/xbrli:xbrl/declaration2.02.01:Declaration" + OccurrenceIndex(report.DeclarationCollection_TrueAndCorrect_RPOccurrenceIndex) + "/prsnunstrcnm1.02.01:PersonUnstructuredName" + OccurrenceIndex(report.Declaration_PersonUnstructuredNameCollection_DeclarationSignatory_RPOccurrenceIndex) + "/pyde.02.00:PersonUnstructuredName.FullName.Text",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.438035" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "PTR264", Value = report.PTR264 });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.GEN.438035

            #region VR.ATO.GEN.438051

            /*  VR.ATO.GEN.438051
            Date of opening balance cannot be later than the start date of Reporting period
    
            Legacy Rule Format:
            IF RP.Opening(Instant):period.instantDate <> RP:period.startDate
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            ^InstantDate1 <> ^StartDate1
    
            Data Elements:
    
            ^StartDate1 = RP
    
            ^InstantDate1 = RP:RP.Opening(Instant)
            */
            assertion = ((report.RPOpeningInstantExists == true) && (report.RPOpeningInstantInstantDate != report.RPStartDate));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.438051",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Date of opening balance must match with the start date of Reporting period",
                    Location = "/xbrli:xbrl/xbrli:context" + OccurrenceIndex(report.RPOccurrenceIndex) + "/xbrli:period/xbrli:startDate",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.438051" } },
                };
                response.Add(processMessage);
            }
            #endregion // VR.ATO.GEN.438051

            #region VR.ATO.GEN.438052

            /*  VR.ATO.GEN.438052
            Date of closing balance cannot be later than the end date of Reporting period
    
            Legacy Rule Format:
            IF RP.Closing(Instant):period.instantDate <> RP:period.endDate
              RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            ^InstantDate1 <> ^EndDate
    
            Data Elements:
    
            ^EndDate = RP
    
            ^InstantDate1 = RP:RP.Closing(Instant)
            */
            assertion = ((report.RPClosingInstantExists == true) && (report.RPClosingInstantInstantDate != report.RPEndDate));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.438052",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Date of closing balance must match with the end date of Reporting period",
                    Location = "/xbrli:xbrl/xbrli:context" + OccurrenceIndex(report.RPOccurrenceIndex) + "/xbrli:period/xbrli:endDate",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.438052" } },
                };
                response.Add(processMessage);
            }
            #endregion // VR.ATO.GEN.438052

            #region VR.ATO.GEN.438053

            /*  VR.ATO.GEN.438053
            Period start date within context must match start date within reporting party context
    
            Legacy Rule Format:
            IF period.startDate  <> RP:period.startDate
            RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            (^Context <> NULL) AND (^StartDate  <> ^StartDate1)
    
            Data Elements:
    
            ^StartDate = INT
    
            ^StartDate1 = RP
    
            ^Context = INT
            */
            assertion = (report.INTExists != false && report.INTStartDate != report.RPStartDate);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.438053",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Period start date must match Reporting Party period start date",
                    Location = "/xbrli:xbrl/xbrli:context" + OccurrenceIndex(report.INTOccurrenceIndex) + "/xbrli:period/xbrli:startDate",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.438053" } },
                };
                response.Add(processMessage);
            }
            #endregion // VR.ATO.GEN.438053

            #region VR.ATO.GEN.438053

            /*  VR.ATO.GEN.438053
            Period start date within context must match start date within reporting party context
    
            Legacy Rule Format:
            IF period.startDate  <> RP:period.startDate
            RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            (^Context <> NULL) AND (^StartDate  <> ^StartDate1)
    
            Data Elements:
    
            ^StartDate = RP
    
            ^StartDate1 = RP
    
            ^Context = RP
            */
            assertion = (report.RPExists != false && report.RPStartDate != report.RPStartDate);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.438053",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Period start date must match Reporting Party period start date",
                    Location = "/xbrli:xbrl/xbrli:context" + OccurrenceIndex(report.RPOccurrenceIndex) + "/xbrli:period/xbrli:startDate",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.438053" } },
                };
                response.Add(processMessage);
            }
            #endregion // VR.ATO.GEN.438053

            #region VR.ATO.GEN.438053

            /*  VR.ATO.GEN.438053
            Period start date within context must match start date within reporting party context
    
            Legacy Rule Format:
            IF period.startDate  <> RP:period.startDate
            RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            (^Context <> NULL) AND (^StartDate  <> ^StartDate1)
    
            Data Elements:
    
            ^StartDate = RP:RP.GeneralPool
    
            ^StartDate1 = RP
    
            ^Context = RP:RP.GeneralPool
            */
            assertion = (report.RPGeneralPoolExists != false && report.RPGeneralPoolStartDate != report.RPStartDate);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.438053",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Period start date must match Reporting Party period start date",
                    Location = "/xbrli:xbrl/xbrli:context" + OccurrenceIndex(report.RPGeneralPoolOccurrenceIndex) + "/xbrli:period/xbrli:startDate",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.438053" } },
                };
                response.Add(processMessage);
            }
            #endregion // VR.ATO.GEN.438053

            #region VR.ATO.GEN.438053

            /*  VR.ATO.GEN.438053
            Period start date within context must match start date within reporting party context
    
            Legacy Rule Format:
            IF period.startDate  <> RP:period.startDate
            RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            (^Context <> NULL) AND (^StartDate  <> ^StartDate1)
    
            Data Elements:
    
            ^StartDate = RP:RP.JAUS
    
            ^StartDate1 = RP
    
            ^Context = RP:RP.JAUS
            */
            assertion = (report.RPJAUSExists != false && report.RPJAUSStartDate != report.RPStartDate);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.438053",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Period start date must match Reporting Party period start date",
                    Location = "/xbrli:xbrl/xbrli:context" + OccurrenceIndex(report.RPJAUSOccurrenceIndex) + "/xbrli:period/xbrli:startDate",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.438053" } },
                };
                response.Add(processMessage);
            }
            #endregion // VR.ATO.GEN.438053

            #region VR.ATO.GEN.438053

            /*  VR.ATO.GEN.438053
            Period start date within context must match start date within reporting party context
    
            Legacy Rule Format:
            IF period.startDate  <> RP:period.startDate
            RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            (^Context <> NULL) AND (^StartDate  <> ^StartDate1)
    
            Data Elements:
    
            ^StartDate = RP.JAUS:RP.JAUS.NonPrim
    
            ^StartDate1 = RP
    
            ^Context = RP.JAUS:RP.JAUS.NonPrim
            */
            assertion = (report.RPJAUSNonPrimExists != false && report.RPJAUSNonPrimStartDate != report.RPStartDate);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.438053",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Period start date must match Reporting Party period start date",
                    Location = "/xbrli:xbrl/xbrli:context" + OccurrenceIndex(report.RPJAUSNonPrimOccurrenceIndex) + "/xbrli:period/xbrli:startDate",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.438053" } },
                };
                response.Add(processMessage);
            }
            #endregion // VR.ATO.GEN.438053

            #region VR.ATO.GEN.438053

            /*  VR.ATO.GEN.438053
            Period start date within context must match start date within reporting party context
    
            Legacy Rule Format:
            IF period.startDate  <> RP:period.startDate
            RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            (^Context <> NULL) AND (^StartDate  <> ^StartDate1)
    
            Data Elements:
    
            ^StartDate = RP.JAUS:RP.JAUS.Prim
    
            ^StartDate1 = RP
    
            ^Context = RP.JAUS:RP.JAUS.Prim
            */
            assertion = (report.RPJAUSPrimExists != false && report.RPJAUSPrimStartDate != report.RPStartDate);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.438053",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Period start date must match Reporting Party period start date",
                    Location = "/xbrli:xbrl/xbrli:context" + OccurrenceIndex(report.RPJAUSPrimOccurrenceIndex) + "/xbrli:period/xbrli:startDate",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.438053" } },
                };
                response.Add(processMessage);
            }
            #endregion // VR.ATO.GEN.438053

            #region VR.ATO.GEN.438053

            /*  VR.ATO.GEN.438053
            Period start date within context must match start date within reporting party context
    
            Legacy Rule Format:
            IF period.startDate  <> RP:period.startDate
            RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            (^Context <> NULL) AND (^StartDate  <> ^StartDate1)
    
            Data Elements:
    
            ^StartDate = RP:RP.JFOREIGN
    
            ^StartDate1 = RP
    
            ^Context = RP:RP.JFOREIGN
            */
            assertion = (report.RPJFOREIGNExists != false && report.RPJFOREIGNStartDate != report.RPStartDate);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.438053",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Period start date must match Reporting Party period start date",
                    Location = "/xbrli:xbrl/xbrli:context" + OccurrenceIndex(report.RPJFOREIGNOccurrenceIndex) + "/xbrli:period/xbrli:startDate",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.438053" } },
                };
                response.Add(processMessage);
            }
            #endregion // VR.ATO.GEN.438053

            #region VR.ATO.GEN.438053

            /*  VR.ATO.GEN.438053
            Period start date within context must match start date within reporting party context
    
            Legacy Rule Format:
            IF period.startDate  <> RP:period.startDate
            RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            (^Context <> NULL) AND (^StartDate  <> ^StartDate1)
    
            Data Elements:
    
            ^StartDate = RP.JFOREIGN:RP.JFOREIGN.CFCLC
    
            ^StartDate1 = RP
    
            ^Context = RP.JFOREIGN:RP.JFOREIGN.CFCLC
            */
            assertion = (report.RPJFOREIGNCFCLCExists != false && report.RPJFOREIGNCFCLCStartDate != report.RPStartDate);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.438053",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Period start date must match Reporting Party period start date",
                    Location = "/xbrli:xbrl/xbrli:context" + OccurrenceIndex(report.RPJFOREIGNCFCLCOccurrenceIndex) + "/xbrli:period/xbrli:startDate",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.438053" } },
                };
                response.Add(processMessage);
            }
            #endregion // VR.ATO.GEN.438053

            #region VR.ATO.GEN.438053

            /*  VR.ATO.GEN.438053
            Period start date within context must match start date within reporting party context
    
            Legacy Rule Format:
            IF period.startDate  <> RP:period.startDate
            RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            (^Context <> NULL) AND (^StartDate  <> ^StartDate1)
    
            Data Elements:
    
            ^StartDate = RP.JFOREIGN:RP.JFOREIGN.CFCUC
    
            ^StartDate1 = RP
    
            ^Context = RP.JFOREIGN:RP.JFOREIGN.CFCUC
            */
            assertion = (report.RPJFOREIGNCFCUCExists != false && report.RPJFOREIGNCFCUCStartDate != report.RPStartDate);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.438053",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Period start date must match Reporting Party period start date",
                    Location = "/xbrli:xbrl/xbrli:context" + OccurrenceIndex(report.RPJFOREIGNCFCUCOccurrenceIndex) + "/xbrli:period/xbrli:startDate",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.438053" } },
                };
                response.Add(processMessage);
            }
            #endregion // VR.ATO.GEN.438053

            #region VR.ATO.GEN.438053

            /*  VR.ATO.GEN.438053
            Period start date within context must match start date within reporting party context
    
            Legacy Rule Format:
            IF period.startDate  <> RP:period.startDate
            RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            (^Context <> NULL) AND (^StartDate  <> ^StartDate1)
    
            Data Elements:
    
            ^StartDate = RP:RP.LowCost
    
            ^StartDate1 = RP
    
            ^Context = RP:RP.LowCost
            */
            assertion = (report.RPLowCostExists != false && report.RPLowCostStartDate != report.RPStartDate);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.438053",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Period start date must match Reporting Party period start date",
                    Location = "/xbrli:xbrl/xbrli:context" + OccurrenceIndex(report.RPLowCostOccurrenceIndex) + "/xbrli:period/xbrli:startDate",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.438053" } },
                };
                response.Add(processMessage);
            }
            #endregion // VR.ATO.GEN.438053

            #region VR.ATO.GEN.438053

            /*  VR.ATO.GEN.438053
            Period start date within context must match start date within reporting party context
    
            Legacy Rule Format:
            IF period.startDate  <> RP:period.startDate
            RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            (^Context <> NULL) AND (^StartDate  <> ^StartDate1)
    
            Data Elements:
    
            ^StartDate = RP:RP.NonPrim
    
            ^StartDate1 = RP
    
            ^Context = RP:RP.NonPrim
            */
            assertion = (report.RPNonPrimExists != false && report.RPNonPrimStartDate != report.RPStartDate);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.438053",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Period start date must match Reporting Party period start date",
                    Location = "/xbrli:xbrl/xbrli:context" + OccurrenceIndex(report.RPNonPrimOccurrenceIndex) + "/xbrli:period/xbrli:startDate",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.438053" } },
                };
                response.Add(processMessage);
            }
            #endregion // VR.ATO.GEN.438053

            #region VR.ATO.GEN.438053

            /*  VR.ATO.GEN.438053
            Period start date within context must match start date within reporting party context
    
            Legacy Rule Format:
            IF period.startDate  <> RP:period.startDate
            RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            (^Context <> NULL) AND (^StartDate  <> ^StartDate1)
    
            Data Elements:
    
            ^StartDate = RP:RP.Prim
    
            ^StartDate1 = RP
    
            ^Context = RP:RP.Prim
            */
            assertion = (report.RPPrimExists != false && report.RPPrimStartDate != report.RPStartDate);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.438053",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Period start date must match Reporting Party period start date",
                    Location = "/xbrli:xbrl/xbrli:context" + OccurrenceIndex(report.RPPrimOccurrenceIndex) + "/xbrli:period/xbrli:startDate",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.438053" } },
                };
                response.Add(processMessage);
            }
            #endregion // VR.ATO.GEN.438053

            #region VR.ATO.GEN.438053

            /*  VR.ATO.GEN.438053
            Period start date within context must match start date within reporting party context
    
            Legacy Rule Format:
            IF period.startDate  <> RP:period.startDate
            RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            (^Context <> NULL) AND (^StartDate  <> ^StartDate1)
    
            Data Elements:
    
            ^StartDate = RP:RP.TOFA
    
            ^StartDate1 = RP
    
            ^Context = RP:RP.TOFA
            */
            assertion = (report.RPTOFAExists != false && report.RPTOFAStartDate != report.RPStartDate);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.438053",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Period start date must match Reporting Party period start date",
                    Location = "/xbrli:xbrl/xbrli:context" + OccurrenceIndex(report.RPTOFAOccurrenceIndex) + "/xbrli:period/xbrli:startDate",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.438053" } },
                };
                response.Add(processMessage);
            }
            #endregion // VR.ATO.GEN.438053

            #region VR.ATO.GEN.438054

            /*  VR.ATO.GEN.438054
            Period end date within context must match end date within reporting party context
    
            Legacy Rule Format:
            IF period.endDate <> RP:period.endDate
            RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            (^Context <> NULL) AND (^EndDate1 <> ^EndDate)
    
            Data Elements:
    
            ^Context = INT
    
            ^EndDate1 = INT
    
            ^EndDate = RP
            */
            assertion = (report.INTExists != false && report.INTEndDate != report.RPEndDate);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.438054",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Period end date must match Reporting Party period end date",
                    Location = "/xbrli:xbrl/xbrli:context" + OccurrenceIndex(report.INTOccurrenceIndex) + "/xbrli:period/xbrli:endDate",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.438054" } },
                };
                response.Add(processMessage);
            }
            #endregion // VR.ATO.GEN.438054

            #region VR.ATO.GEN.438054

            /*  VR.ATO.GEN.438054
            Period end date within context must match end date within reporting party context
    
            Legacy Rule Format:
            IF period.endDate <> RP:period.endDate
            RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            (^Context <> NULL) AND (^EndDate1 <> ^EndDate)
    
            Data Elements:
    
            ^Context = RP
    
            ^EndDate1 = RP
    
            ^EndDate = RP
            */
            assertion = (report.RPExists != false && report.RPEndDate != report.RPEndDate);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.438054",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Period end date must match Reporting Party period end date",
                    Location = "/xbrli:xbrl/xbrli:context" + OccurrenceIndex(report.RPOccurrenceIndex) + "/xbrli:period/xbrli:endDate",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.438054" } },
                };
                response.Add(processMessage);
            }
            #endregion // VR.ATO.GEN.438054

            #region VR.ATO.GEN.438054

            /*  VR.ATO.GEN.438054
            Period end date within context must match end date within reporting party context
    
            Legacy Rule Format:
            IF period.endDate <> RP:period.endDate
            RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            (^Context <> NULL) AND (^EndDate1 <> ^EndDate)
    
            Data Elements:
    
            ^Context = RP:RP.GeneralPool
    
            ^EndDate1 = RP:RP.GeneralPool
    
            ^EndDate = RP
            */
            assertion = (report.RPGeneralPoolExists != false && report.RPGeneralPoolEndDate != report.RPEndDate);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.438054",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Period end date must match Reporting Party period end date",
                    Location = "/xbrli:xbrl/xbrli:context" + OccurrenceIndex(report.RPGeneralPoolOccurrenceIndex) + "/xbrli:period/xbrli:endDate",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.438054" } },
                };
                response.Add(processMessage);
            }
            #endregion // VR.ATO.GEN.438054

            #region VR.ATO.GEN.438054

            /*  VR.ATO.GEN.438054
            Period end date within context must match end date within reporting party context
    
            Legacy Rule Format:
            IF period.endDate <> RP:period.endDate
            RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            (^Context <> NULL) AND (^EndDate1 <> ^EndDate)
    
            Data Elements:
    
            ^Context = RP:RP.JAUS
    
            ^EndDate1 = RP:RP.JAUS
    
            ^EndDate = RP
            */
            assertion = (report.RPJAUSExists != false && report.RPJAUSEndDate != report.RPEndDate);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.438054",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Period end date must match Reporting Party period end date",
                    Location = "/xbrli:xbrl/xbrli:context" + OccurrenceIndex(report.RPJAUSOccurrenceIndex) + "/xbrli:period/xbrli:endDate",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.438054" } },
                };
                response.Add(processMessage);
            }
            #endregion // VR.ATO.GEN.438054

            #region VR.ATO.GEN.438054

            /*  VR.ATO.GEN.438054
            Period end date within context must match end date within reporting party context
    
            Legacy Rule Format:
            IF period.endDate <> RP:period.endDate
            RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            (^Context <> NULL) AND (^EndDate1 <> ^EndDate)
    
            Data Elements:
    
            ^Context = RP.JAUS:RP.JAUS.NonPrim
    
            ^EndDate1 = RP.JAUS:RP.JAUS.NonPrim
    
            ^EndDate = RP
            */
            assertion = (report.RPJAUSNonPrimExists != false && report.RPJAUSNonPrimEndDate != report.RPEndDate);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.438054",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Period end date must match Reporting Party period end date",
                    Location = "/xbrli:xbrl/xbrli:context" + OccurrenceIndex(report.RPJAUSNonPrimOccurrenceIndex) + "/xbrli:period/xbrli:endDate",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.438054" } },
                };
                response.Add(processMessage);
            }
            #endregion // VR.ATO.GEN.438054

            #region VR.ATO.GEN.438054

            /*  VR.ATO.GEN.438054
            Period end date within context must match end date within reporting party context
    
            Legacy Rule Format:
            IF period.endDate <> RP:period.endDate
            RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            (^Context <> NULL) AND (^EndDate1 <> ^EndDate)
    
            Data Elements:
    
            ^Context = RP.JAUS:RP.JAUS.Prim
    
            ^EndDate1 = RP.JAUS:RP.JAUS.Prim
    
            ^EndDate = RP
            */
            assertion = (report.RPJAUSPrimExists != false && report.RPJAUSPrimEndDate != report.RPEndDate);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.438054",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Period end date must match Reporting Party period end date",
                    Location = "/xbrli:xbrl/xbrli:context" + OccurrenceIndex(report.RPJAUSPrimOccurrenceIndex) + "/xbrli:period/xbrli:endDate",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.438054" } },
                };
                response.Add(processMessage);
            }
            #endregion // VR.ATO.GEN.438054

            #region VR.ATO.GEN.438054

            /*  VR.ATO.GEN.438054
            Period end date within context must match end date within reporting party context
    
            Legacy Rule Format:
            IF period.endDate <> RP:period.endDate
            RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            (^Context <> NULL) AND (^EndDate1 <> ^EndDate)
    
            Data Elements:
    
            ^Context = RP:RP.JFOREIGN
    
            ^EndDate1 = RP:RP.JFOREIGN
    
            ^EndDate = RP
            */
            assertion = (report.RPJFOREIGNExists != false && report.RPJFOREIGNEndDate != report.RPEndDate);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.438054",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Period end date must match Reporting Party period end date",
                    Location = "/xbrli:xbrl/xbrli:context" + OccurrenceIndex(report.RPJFOREIGNOccurrenceIndex) + "/xbrli:period/xbrli:endDate",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.438054" } },
                };
                response.Add(processMessage);
            }
            #endregion // VR.ATO.GEN.438054

            #region VR.ATO.GEN.438054

            /*  VR.ATO.GEN.438054
            Period end date within context must match end date within reporting party context
    
            Legacy Rule Format:
            IF period.endDate <> RP:period.endDate
            RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            (^Context <> NULL) AND (^EndDate1 <> ^EndDate)
    
            Data Elements:
    
            ^Context = RP.JFOREIGN:RP.JFOREIGN.CFCLC
    
            ^EndDate1 = RP.JFOREIGN:RP.JFOREIGN.CFCLC
    
            ^EndDate = RP
            */
            assertion = (report.RPJFOREIGNCFCLCExists != false && report.RPJFOREIGNCFCLCEndDate != report.RPEndDate);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.438054",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Period end date must match Reporting Party period end date",
                    Location = "/xbrli:xbrl/xbrli:context" + OccurrenceIndex(report.RPJFOREIGNCFCLCOccurrenceIndex) + "/xbrli:period/xbrli:endDate",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.438054" } },
                };
                response.Add(processMessage);
            }
            #endregion // VR.ATO.GEN.438054

            #region VR.ATO.GEN.438054

            /*  VR.ATO.GEN.438054
            Period end date within context must match end date within reporting party context
    
            Legacy Rule Format:
            IF period.endDate <> RP:period.endDate
            RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            (^Context <> NULL) AND (^EndDate1 <> ^EndDate)
    
            Data Elements:
    
            ^Context = RP.JFOREIGN:RP.JFOREIGN.CFCUC
    
            ^EndDate1 = RP.JFOREIGN:RP.JFOREIGN.CFCUC
    
            ^EndDate = RP
            */
            assertion = (report.RPJFOREIGNCFCUCExists != false && report.RPJFOREIGNCFCUCEndDate != report.RPEndDate);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.438054",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Period end date must match Reporting Party period end date",
                    Location = "/xbrli:xbrl/xbrli:context" + OccurrenceIndex(report.RPJFOREIGNCFCUCOccurrenceIndex) + "/xbrli:period/xbrli:endDate",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.438054" } },
                };
                response.Add(processMessage);
            }
            #endregion // VR.ATO.GEN.438054

            #region VR.ATO.GEN.438054

            /*  VR.ATO.GEN.438054
            Period end date within context must match end date within reporting party context
    
            Legacy Rule Format:
            IF period.endDate <> RP:period.endDate
            RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            (^Context <> NULL) AND (^EndDate1 <> ^EndDate)
    
            Data Elements:
    
            ^Context = RP:RP.LowCost
    
            ^EndDate1 = RP:RP.LowCost
    
            ^EndDate = RP
            */
            assertion = (report.RPLowCostExists != false && report.RPLowCostEndDate != report.RPEndDate);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.438054",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Period end date must match Reporting Party period end date",
                    Location = "/xbrli:xbrl/xbrli:context" + OccurrenceIndex(report.RPLowCostOccurrenceIndex) + "/xbrli:period/xbrli:endDate",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.438054" } },
                };
                response.Add(processMessage);
            }
            #endregion // VR.ATO.GEN.438054

            #region VR.ATO.GEN.438054

            /*  VR.ATO.GEN.438054
            Period end date within context must match end date within reporting party context
    
            Legacy Rule Format:
            IF period.endDate <> RP:period.endDate
            RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            (^Context <> NULL) AND (^EndDate1 <> ^EndDate)
    
            Data Elements:
    
            ^Context = RP:RP.NonPrim
    
            ^EndDate1 = RP:RP.NonPrim
    
            ^EndDate = RP
            */
            assertion = (report.RPNonPrimExists != false && report.RPNonPrimEndDate != report.RPEndDate);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.438054",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Period end date must match Reporting Party period end date",
                    Location = "/xbrli:xbrl/xbrli:context" + OccurrenceIndex(report.RPNonPrimOccurrenceIndex) + "/xbrli:period/xbrli:endDate",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.438054" } },
                };
                response.Add(processMessage);
            }
            #endregion // VR.ATO.GEN.438054

            #region VR.ATO.GEN.438054

            /*  VR.ATO.GEN.438054
            Period end date within context must match end date within reporting party context
    
            Legacy Rule Format:
            IF period.endDate <> RP:period.endDate
            RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            (^Context <> NULL) AND (^EndDate1 <> ^EndDate)
    
            Data Elements:
    
            ^Context = RP:RP.Prim
    
            ^EndDate1 = RP:RP.Prim
    
            ^EndDate = RP
            */
            assertion = (report.RPPrimExists != false && report.RPPrimEndDate != report.RPEndDate);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.438054",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Period end date must match Reporting Party period end date",
                    Location = "/xbrli:xbrl/xbrli:context" + OccurrenceIndex(report.RPPrimOccurrenceIndex) + "/xbrli:period/xbrli:endDate",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.438054" } },
                };
                response.Add(processMessage);
            }
            #endregion // VR.ATO.GEN.438054

            #region VR.ATO.GEN.438054

            /*  VR.ATO.GEN.438054
            Period end date within context must match end date within reporting party context
    
            Legacy Rule Format:
            IF period.endDate <> RP:period.endDate
            RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            (^Context <> NULL) AND (^EndDate1 <> ^EndDate)
    
            Data Elements:
    
            ^Context = RP:RP.TOFA
    
            ^EndDate1 = RP:RP.TOFA
    
            ^EndDate = RP
            */
            assertion = (report.RPTOFAExists != false && report.RPTOFAEndDate != report.RPEndDate);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.438054",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Period end date must match Reporting Party period end date",
                    Location = "/xbrli:xbrl/xbrli:context" + OccurrenceIndex(report.RPTOFAOccurrenceIndex) + "/xbrli:period/xbrli:endDate",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.438054" } },
                };
                response.Add(processMessage);
            }
            #endregion // VR.ATO.GEN.438054

            #region VR.ATO.GEN.438063

            /*  VR.ATO.GEN.438063
            If an Intermediary is present, Intermediary declaration statement accepted indicator must be supplied and must be “True”.
    
            Legacy Rule Format:
            IF INT<>NULL AND INT:pyin.xx.xx:Declaration.StatementAccepted.Indicator <> TRUE
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            ^INT <> NULL AND ^PTR295 <> TRUE
    
            Data Elements:
    
            INT:^PTR295 = pyin.02.00:Declaration.StatementAccepted.Indicator WHERE [Declaration.StatementType.Code='TrueAndCorrect']
    
            INT:^INT = INT
            */
            assertion = (report.INTExists != false && report.PTR295 != true);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.438063",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Declaration statement accepted indicator is required.",
                    LongDescription = @"If an Intermediary is present, Intermediary declaration statement accepted indicator must be supplied and must be ""True"".",
                    Location = "/xbrli:xbrl/declaration2.02.01:Declaration" + OccurrenceIndex(report.DeclarationCollection_TrueAndCorrect_INTOccurrenceIndex) + "/pyin.02.00:Declaration.StatementAccepted.Indicator",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.438063" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "PTR295", Value = GetValueOrEmpty(report.PTR295) });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.GEN.438063

            #region VR.ATO.GEN.438064

            /*  VR.ATO.GEN.438064
            If an Intermediary is not present, Reporting Party declaration statement accepted indicator must be supplied and must be “True”.
    
            Legacy Rule Format:
            IF INT=NULL AND RP:pyin.xx.xx:Declaration.StatementAccepted.Indicator <> TRUE
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            ^INT = NULL AND ^PTR290 <> TRUE
    
            Data Elements:
    
            RP:^PTR290 = pyin.02.00:Declaration.StatementAccepted.Indicator WHERE [Declaration.StatementType.Code='TrueAndCorrect']
    
            RP:^INT = INT
            */
            assertion = (report.INTExists == false && report.PTR290 != true);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.438064",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Declaration statement accepted indicator is required.",
                    LongDescription = @"If an Intermediary is not present, Reporting Party declaration statement accepted indicator must be supplied and must be ""True"".",
                    Location = "/xbrli:xbrl/declaration2.02.01:Declaration" + OccurrenceIndex(report.DeclarationCollection_TrueAndCorrect_INTOccurrenceIndex) + "/pyin.02.00:Declaration.StatementAccepted.Indicator",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.438064" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "PTR290", Value = GetValueOrEmpty(report.PTR290) });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.GEN.438064

            #region VR.ATO.GEN.438065

            /*  VR.ATO.GEN.438065
            If an Intermediary is present, Intermediary declaration signatory identifier must be supplied.
    
            Legacy Rule Format:
            IF INT<>NULL AND INT:pyin.xx.xx:Declaration.SignatoryIdentifier.Text = NULLORBLANK
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            ^INT <> NULL AND ^PTR297 = BLANK
    
            Data Elements:
    
            INT:^PTR297 = pyin.02.00:Declaration.SignatoryIdentifier.Text WHERE [Declaration.StatementType.Code='TrueAndCorrect']
    
            INT:^INT = INT
            */
            assertion = (report.INTExists != false && string.IsNullOrWhiteSpace(report.PTR297) == true);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.438065",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Declaration signatory identifier is required.",
                    LongDescription = @"If an Intermediary is present, Intermediary declaration signatory identifier must be supplied.",
                    Location = "/xbrli:xbrl/declaration2.02.01:Declaration" + OccurrenceIndex(report.DeclarationCollection_TrueAndCorrect_INTOccurrenceIndex) + "/pyin.02.00:Declaration.SignatoryIdentifier.Text",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.438065" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "PTR297", Value = report.PTR297 });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.GEN.438065

            #region VR.ATO.GEN.438066

            /*  VR.ATO.GEN.438066
            If an Intermediary is not present, Reporting Party declaration signatory identifier must be supplied.
    
            Legacy Rule Format:
            IF INT=NULL AND RP:pyin.xx.xx:Declaration.SignatoryIdentifier.Text = NULLORBLANK
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            ^INT = NULL AND ^PTR292 = BLANK
    
            Data Elements:
    
            RP:^PTR292 = pyin.02.00:Declaration.SignatoryIdentifier.Text WHERE [Declaration.StatementType.Code='TrueAndCorrect']
    
            RP:^INT = INT
            */
            assertion = (report.INTExists == false && string.IsNullOrWhiteSpace(report.PTR292) == true);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.438066",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Declaration signatory identifier is required.",
                    LongDescription = @"If an Intermediary is not present, Reporting Party declaration signatory identifier must be supplied.",
                    Location = "/xbrli:xbrl/declaration2.02.01:Declaration" + OccurrenceIndex(report.DeclarationCollection_TrueAndCorrect_INTOccurrenceIndex) + "/pyin.02.00:Declaration.SignatoryIdentifier.Text",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.438066" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "PTR292", Value = report.PTR292 });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.GEN.438066

            #region VR.ATO.GEN.438067

            /*  VR.ATO.GEN.438067
            If an Intermediary is present, Intermediary declaration signature date must be supplied.
    
            Legacy Rule Format:
            IF INT<>NULL AND INT:pyin.xx.xx:Declaration.Signature.Date = NULL
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            ^INT <> NULL AND ^PTR294 = NULL
    
            Data Elements:
    
            INT:^PTR294 = pyin.02.00:Declaration.Signature.Date WHERE [Declaration.StatementType.Code='TrueAndCorrect']
    
            INT:^INT = INT
            */
            assertion = (report.INTExists != false && report.PTR294 == null);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.438067",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Declaration signature date is required.",
                    LongDescription = @"If an Intermediary is present, Intermediary declaration signature date must be supplied.",
                    Location = "/xbrli:xbrl/declaration2.02.01:Declaration" + OccurrenceIndex(report.DeclarationCollection_TrueAndCorrect_INTOccurrenceIndex) + "/pyin.02.00:Declaration.Signature.Date",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.438067" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "PTR294", Value = GetValueOrEmpty(report.PTR294) });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.GEN.438067

            #region VR.ATO.GEN.438068

            /*  VR.ATO.GEN.438068
            If an Intermediary is not present, Reporting Party declaration signature date must be supplied.
    
            Legacy Rule Format:
            IF INT=NULL AND RP:pyin.xx.xx:Declaration.Signature.Date = NULL
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            ^INT = NULL AND ^PTR289 = NULL
    
            Data Elements:
    
            RP:^PTR289 = pyin.02.00:Declaration.Signature.Date WHERE [Declaration.StatementType.Code='TrueAndCorrect']
    
            RP:^INT = INT
            */
            assertion = (report.INTExists == false && report.PTR289 == null);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.438068",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Declaration signature date is required.",
                    LongDescription = @"If an Intermediary is not present, Reporting Party declaration signature date must be supplied.",
                    Location = "/xbrli:xbrl/declaration2.02.01:Declaration" + OccurrenceIndex(report.DeclarationCollection_TrueAndCorrect_INTOccurrenceIndex) + "/pyin.02.00:Declaration.Signature.Date",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.438068" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "PTR289", Value = GetValueOrEmpty(report.PTR289) });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.GEN.438068

            #region VR.ATO.PTR.430001

            /*  VR.ATO.PTR.430001
            Depreciation expenses must not exceed total Small business entity depreciating assets
    
            Legacy Rule Format:
            IF ([PTR54] < ([PTR141] + [PTR142]))
                RETURN VALIDATION MESSAGE
            ENDIF
            
            [PTR54] = PTR:RP.JAUS:bafpr1.02.04:Expense.DepreciationandAmortisation.Amount
            [PTR141] = PTR:RP.LowCost:bafpr3.02.11:Expense.DepreciatingAssets.SmallBusinessEntity.Amount
            [PTR142] = PTR:RP.GeneralPool:bafpr3.02.11:Expense.DepreciatingAssets.SmallBusinessEntity.Amount

            Technical Business Rule Format:
            (^PTR54 < (^PTR141 + ^PTR142))
    
            Data Elements:
    
            RP.JAUS:^PTR54 = bafpr1.02.04:Expense.DepreciationandAmortisation.Amount
    
            RP.LowCost:^PTR141 = bafpr3.02.11:Expense.DepreciatingAssets.SmallBusinessEntity.Amount
    
            RP.GeneralPool:^PTR142 = bafpr3.02.11:Expense.DepreciatingAssets.SmallBusinessEntity.Amount
            */
            assertion = (report.PTR54.GetValueOrDefault() < report.PTR141.GetValueOrDefault() + report.PTR142.GetValueOrDefault());
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.430001",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Small business entity depreciating assets must not exceed Depreciation expenses",
                    LongDescription = @"The sum of deductions for 'Small business entity depreciating assets' must not exceed the 'Depreciation expenses' amount",
                    Location = "/xbrli:xbrl/bafpr1.02.04:Expense.DepreciationandAmortisation.Amount",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.PTR.430001" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "PTR54", Value = report.PTR54.GetValueOrDefault().ToString() });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "PTR141", Value = report.PTR141.GetValueOrDefault().ToString() });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "PTR142", Value = report.PTR142.GetValueOrDefault().ToString() });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.PTR.430001

            #region VR.ATO.PTR.430009

            /*  VR.ATO.PTR.430009
            Previous postal address must not be the same as current postal address
    
            Legacy Rule Format:
            IF ([PTR14] = [PTR9] AND [PTR15] = [PTR10] AND [PTR16] = [PTR11] AND [PTR17] = [PTR12]) 
                RETURN VALIDATION MESSAGE
            ENDIF
            
            [PTR9] = PTR:RP:pyde.02.00:AddressDetails.Line1.Text WHERE (pyde.02.01:AddressDetails.Usage.Code = "POS" AND pyde.02.00:AddressDetails.Currency.Code = "C") IN TUPLE(address2.02.02:AddressDetails)
            [PTR10] = PTR:RP:pyde.02.00:AddressDetails.LocalityName.Text WHERE (pyde.02.01:AddressDetails.Usage.Code = "POS" AND pyde.02.00:AddressDetails.Currency.Code = "C") IN TUPLE(address2.02.02:AddressDetails)
            [PTR11] = PTR:RP:pyde.02.00:AddressDetails.StateOrTerritory.Code WHERE (pyde.02.01:AddressDetails.Usage.Code = "POS" AND pyde.02.00:AddressDetails.Currency.Code = "C") IN TUPLE(address2.02.02:AddressDetails)
            [PTR12] = PTR:RP:pyde.02.00:AddressDetails.Postcode.Text WHERE (pyde.02.01:AddressDetails.Usage.Code = "POS" AND pyde.02.00:AddressDetails.Currency.Code = "C") IN TUPLE(address2.02.02:AddressDetails)
            [PTR14] = PTR:RP:pyde.02.00:AddressDetails.Line1.Text WHERE (pyde.02.01:AddressDetails.Usage.Code = "POS" AND pyde.02.00:AddressDetails.Currency.Code = "P") IN TUPLE(address2.02.02:AddressDetails)
            [PTR15] = PTR:RP:pyde.02.00:AddressDetails.LocalityName.Text WHERE (pyde.02.01:AddressDetails.Usage.Code = "POS" AND pyde.02.00:AddressDetails.Currency.Code = "P") IN TUPLE(address2.02.02:AddressDetails)
            [PTR16] = PTR:RP:pyde.02.00:AddressDetails.StateOrTerritory.Code WHERE (pyde.02.01:AddressDetails.Usage.Code = "POS" AND pyde.02.00:AddressDetails.Currency.Code = "P") IN TUPLE(address2.02.02:AddressDetails)
            [PTR17] = PTR:RP:pyde.02.00:AddressDetails.Postcode.Text WHERE (pyde.02.01:AddressDetails.Usage.Code = "POS" AND pyde.02.00:AddressDetails.Currency.Code = "P") IN TUPLE(address2.02.02:AddressDetails)

            Technical Business Rule Format:
            ^PTR14 = ^PTR9 AND ^PTR15 = ^PTR10 AND ^PTR16 = ^PTR11 AND ^PTR17 = ^PTR12
    
            Data Elements:
    
            RP:^PTR14 = pyde.02.00:AddressDetails.Line1.Text WHERE [AddressDetails.Usage.Code='POS'].[AddressDetails.Currency.Code='P']
    
            RP:^PTR9 = pyde.02.00:AddressDetails.Line1.Text WHERE [AddressDetails.Usage.Code='POS'].[AddressDetails.Currency.Code='C']
    
            RP:^PTR10 = pyde.02.00:AddressDetails.LocalityName.Text WHERE [AddressDetails.Usage.Code='POS'].[AddressDetails.Currency.Code='C']
    
            RP:^PTR11 = pyde.02.00:AddressDetails.StateOrTerritory.Code WHERE [AddressDetails.Usage.Code='POS'].[AddressDetails.Currency.Code='C']
    
            RP:^PTR12 = pyde.02.00:AddressDetails.Postcode.Text WHERE [AddressDetails.Usage.Code='POS'].[AddressDetails.Currency.Code='C']
    
            RP:^PTR15 = pyde.02.00:AddressDetails.LocalityName.Text WHERE [AddressDetails.Usage.Code='POS'].[AddressDetails.Currency.Code='P']
    
            RP:^PTR16 = pyde.02.00:AddressDetails.StateOrTerritory.Code WHERE [AddressDetails.Usage.Code='POS'].[AddressDetails.Currency.Code='P']
    
            RP:^PTR17 = pyde.02.00:AddressDetails.Postcode.Text WHERE [AddressDetails.Usage.Code='POS'].[AddressDetails.Currency.Code='P']
            */
            assertion = (report.PTR14 == report.PTR9 && report.PTR15 == report.PTR10 && report.PTR16 == report.PTR11 && report.PTR17 == report.PTR12);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.430009",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Previous postal address must not be the same as current postal address",
                    LongDescription = @"Previous postal address is only required when an address change has occurred since the last return.  Delete previous postal address or input the old address if a change of address has occurred.",
                    Location = "/xbrli:xbrl/address2.02.02:AddressDetails" + OccurrenceIndex(report.AddressDetailsCollection_POS_P_RPOccurrenceIndex) + "/pyde.02.00:AddressDetails.Line1.Text",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.PTR.430009" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "PTR14", Value = report.PTR14 });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "PTR9", Value = report.PTR9 });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "PTR15", Value = report.PTR15 });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "PTR10", Value = report.PTR10 });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "PTR16", Value = report.PTR16 });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "PTR11", Value = report.PTR11 });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "PTR17", Value = report.PTR17 });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "PTR12", Value = report.PTR12 });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.PTR.430009

            #region VR.ATO.PTR.430010

            /*  VR.ATO.PTR.430010
            Name of company or trust and name of individual must not both be present
    
            Legacy Rule Format:
            IF [PTR24] <> NULLORBLANK AND [PTR20] <> NULLORBLANK 
                RETURN VALIDATION MESSAGE
            ENDIF
            
            [PTR20] = PTR:RP:pyde.02.00:PersonNameDetails.FamilyName.Text WHERE (pyde.02.05:PersonNameDetails.Usage.Code = "MailRecipient" AND pyde.02.00:PersonNameDetails.PersonNameType.Code = "LGL") IN TUPLE(prsnstrcnm3.02.01:PersonNameDetails) IN TUPLE(ptr.0006.lodge.req.02.00:MailRecipient)
            [PTR24] = PTR:RP:pyde.02.00:OrganisationNameDetails.OrganisationalName.Text WHERE (pyde.02.00:OrganisationNameDetails.OrganisationalNameType.Code = "MN" AND pyde.02.00:OrganisationNameDetails.Currency.Code = "C") IN TUPLE(orgname2.02.00:OrganisationNameDetails) IN TUPLE(ptr.0006.lodge.req.02.00:MailRecipient)

            Technical Business Rule Format:
            ^PTR24 <> BLANK AND ^PTR20 <> BLANK
    
            Data Elements:
    
            RP:^PTR20 = pyde.02.00:PersonNameDetails.FamilyName.Text WHERE [PersonNameDetails.Usage.Code='MailRecipient'].[PersonNameDetails.PersonNameType.Code='LGL']
    
            RP:^PTR24 = pyde.02.00:OrganisationNameDetails.OrganisationalName.Text WHERE [OrganisationNameDetails.OrganisationalNameType.Code='MN'].[OrganisationNameDetails.Currency.Code='C']
            */
            assertion = (string.IsNullOrWhiteSpace(report.PTR24) != true && string.IsNullOrWhiteSpace(report.PTR20) != true);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.PTR.430010",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Name of company or trust and name of individual must not both be present",
                    Location = "/xbrli:xbrl/ptr.0006.lodge.req.02.00:MailRecipient/prsnstrcnm3.02.01:PersonNameDetails" + OccurrenceIndex(report.MailRecipient_PersonNameDetailsCollection_MailRecipient_LGL_RPOccurrenceIndex) + "/pyde.02.00:PersonNameDetails.FamilyName.Text",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.PTR.430010" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "PTR24", Value = report.PTR24 });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "PTR20", Value = report.PTR20 });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.PTR.430010

            #region VR.ATO.PTR.430012

            /*  VR.ATO.PTR.430012
            Name of the partner to whom notices should be sent must be present
    
            Legacy Rule Format:
            IF [PTR20] = NULLORBLANK AND [PTR24] = NULLORBLANK
                RETURN VALIDATION MESSAGE
            ENDIF
            
            [PTR20] = PTR:RP:pyde.02.00:PersonNameDetails.FamilyName.Text WHERE (pyde.02.05:PersonNameDetails.Usage.Code = "MailRecipient" AND pyde.02.00:PersonNameDetails.PersonNameType.Code = "LGL") IN TUPLE(prsnstrcnm3.02.01:PersonNameDetails) IN TUPLE(ptr.0006.lodge.req.02.00:MailRecipient)
            [PTR24] = PTR:RP:pyde.02.00:OrganisationNameDetails.OrganisationalName.Text WHERE (pyde.02.00:OrganisationNameDetails.OrganisationalNameType.Code = "MN" AND pyde.02.00:OrganisationNameDetails.Currency.Code = "C") IN TUPLE(orgname2.02.00:OrganisationNameDetails) IN TUPLE(ptr.0006.lodge.req.02.00:MailRecipient)

            Technical Business Rule Format:
            ^PTR20 = BLANK AND ^PTR24 = BLANK
    
            Data Elements:
    
            RP:^PTR24 = pyde.02.00:OrganisationNameDetails.OrganisationalName.Text WHERE [OrganisationNameDetails.OrganisationalNameType.Code='MN'].[OrganisationNameDetails.Currency.Code='C']
    
            RP:^PTR20 = pyde.02.00:PersonNameDetails.FamilyName.Text WHERE [PersonNameDetails.Usage.Code='MailRecipient'].[PersonNameDetails.PersonNameType.Code='LGL']
            */
            assertion = (string.IsNullOrWhiteSpace(report.PTR20) == true && string.IsNullOrWhiteSpace(report.PTR24) == true);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.PTR.430012",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Name of the partner to whom notices should be sent must be present",
                    LongDescription = @"If the partner is an individual, provide the name of the individual. If the partner is a company or trust, provide the name of the company or trust",
                    Location = "/xbrli:xbrl/ptr.0006.lodge.req.02.00:MailRecipient/orgname2.02.00:OrganisationNameDetails" + OccurrenceIndex(report.MailRecipient_OrganisationNameDetailsCollection_MN_C_RPOccurrenceIndex) + "/pyde.02.00:OrganisationNameDetails.OrganisationalName.Text",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.PTR.430012" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "PTR20", Value = report.PTR20 });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "PTR24", Value = report.PTR24 });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.PTR.430012

            #region VR.ATO.PTR.430013

            /*  VR.ATO.PTR.430013
            Name of the company or trust must be present if the ABN is present
    
            Legacy Rule Format:
            IF [PTR25] <> NULLORBLANK AND [PTR24] = NULLORBLANK 
                RETURN VALIDATION MESSAGE
            ENDIF
            
            [PTR24] = PTR:RP:pyde.02.00:OrganisationNameDetails.OrganisationalName.Text WHERE (pyde.02.00:OrganisationNameDetails.OrganisationalNameType.Code = "MN" AND pyde.02.00:OrganisationNameDetails.Currency.Code = "C") IN TUPLE(orgname2.02.00:OrganisationNameDetails) IN TUPLE(ptr.0006.lodge.req.02.00:MailRecipient)
            [PTR25] = PTR:RP:pyid.02.00:Identifiers.AustralianBusinessNumber.Identifier IN TUPLE(ptr.0006.lodge.req.02.00:MailRecipient)

            Technical Business Rule Format:
            ^PTR25 <> BLANK AND ^PTR24 = BLANK
    
            Data Elements:
    
            RP:^PTR24 = pyde.02.00:OrganisationNameDetails.OrganisationalName.Text WHERE [OrganisationNameDetails.OrganisationalNameType.Code='MN'].[OrganisationNameDetails.Currency.Code='C']
    
            RP:^PTR25 = pyid.02.00:Identifiers.AustralianBusinessNumber.Identifier
            */
            assertion = (string.IsNullOrWhiteSpace(report.PTR25) != true && string.IsNullOrWhiteSpace(report.PTR24) == true);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.PTR.430013",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Name of the company or trust must be present if the ABN is present",
                    Location = "/xbrli:xbrl/ptr.0006.lodge.req.02.00:MailRecipient/orgname2.02.00:OrganisationNameDetails" + OccurrenceIndex(report.MailRecipient_OrganisationNameDetailsCollection_MN_C_RPOccurrenceIndex) + "/pyde.02.00:OrganisationNameDetails.OrganisationalName.Text",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.PTR.430013" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "PTR25", Value = report.PTR25 });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "PTR24", Value = report.PTR24 });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.PTR.430013

            #region VR.ATO.PTR.430015

            /*  VR.ATO.PTR.430015
            Description of main business activity and total business income must be present
    
            Legacy Rule Format:
            IF [PTR31] <> NULL AND (([PTR30] = NULLORBLANK) OR ([PTR35] = NULL AND [PTR36] = NULL AND [PTR38] = NULL AND [PTR39] = NULL AND [PTR41] = NULL AND [PTR43] = NULL AND [PTR44] = NULL)) 
                RETURN VALIDATION MESSAGE
            ENDIF
            
            [PTR30] = PTR:RP:pyde.02.05:OrganisationDetails.MainIncomeActivity.Description
            [PTR31] = PTR:RP:pyde.02.30:OrganisationDetails.OrganisationIndustry2006Extended.Code
            [PTR35] = PTR:RP.JAUS.Prim:lrla.02.00:Remuneration.ABNNotQuotedPaymentGross.Amount
            [PTR36] = PTR:RP.JAUS.NonPrim:lrla.02.00:Remuneration.ABNNotQuotedPaymentGross.Amount
            [PTR38] = PTR:RP.JAUS.NonPrim:lrla.02.00:Remuneration.PaymentToForeignResidentGross.Amount
            [PTR39] = PTR:RP.JAUS.Prim:gfagg.02.00:GovernmentFunding.GovernmentIndustryPaymentsAssessable.Amount
            [PTR41] = PTR:RP.JAUS.NonPrim:gfagg.02.00:GovernmentFunding.GovernmentIndustryPaymentsAssessable.Amount
            [PTR43] = PTR:RP.JAUS.Prim:bafpr1.02.04:Income.Other.Amount
            [PTR44] = PTR:RP.JAUS.NonPrim:bafpr1.02.04:Income.Other.Amount

            Technical Business Rule Format:
            ^PTR31 <> NULL AND ((^PTR30 = BLANK) OR (^PTR35 = NULL AND ^PTR36 = NULL AND ^PTR38 = NULL AND ^PTR39 = NULL AND ^PTR41 = NULL AND ^PTR43 = NULL AND ^PTR44 = NULL))
    
            Data Elements:
    
            RP:^PTR30 = pyde.02.05:OrganisationDetails.MainIncomeActivity.Description
    
            RP:^PTR31 = pyde.02.30:OrganisationDetails.OrganisationIndustry2006Extended.Code
    
            RP.JAUS.Prim:^PTR35 = lrla.02.00:Remuneration.ABNNotQuotedPaymentGross.Amount
    
            RP.JAUS.NonPrim:^PTR36 = lrla.02.00:Remuneration.ABNNotQuotedPaymentGross.Amount
    
            RP.JAUS.NonPrim:^PTR38 = lrla.02.00:Remuneration.PaymentToForeignResidentGross.Amount
    
            RP.JAUS.Prim:^PTR39 = gfagg.02.00:GovernmentFunding.GovernmentIndustryPaymentsAssessable.Amount
    
            RP.JAUS.NonPrim:^PTR41 = gfagg.02.00:GovernmentFunding.GovernmentIndustryPaymentsAssessable.Amount
    
            RP.JAUS.Prim:^PTR43 = bafpr1.02.04:Income.Other.Amount
    
            RP.JAUS.NonPrim:^PTR44 = bafpr1.02.04:Income.Other.Amount
            */
            assertion = (report.PTR31 != null && (string.IsNullOrWhiteSpace(report.PTR30) == true || report.PTR35 == null && report.PTR36 == null && report.PTR38 == null && report.PTR39 == null && report.PTR41 == null && report.PTR43 == null && report.PTR44 == null));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.430015",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Description of main business activity and total business income must be present",
                    LongDescription = @"If an Industry code is present, then the 'Description of main business activity' and business income (Primary Production and/or Non-Primary Production) must be present",
                    Location = "/xbrli:xbrl/pyde.02.05:OrganisationDetails.MainIncomeActivity.Description",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.PTR.430015" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "PTR31", Value = report.PTR31 });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "PTR30", Value = report.PTR30 });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "PTR35", Value = report.PTR35.GetValueOrDefault().ToString() });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "PTR36", Value = report.PTR36.GetValueOrDefault().ToString() });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "PTR38", Value = report.PTR38.GetValueOrDefault().ToString() });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "PTR39", Value = report.PTR39.GetValueOrDefault().ToString() });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "PTR41", Value = report.PTR41.GetValueOrDefault().ToString() });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "PTR43", Value = report.PTR43.GetValueOrDefault().ToString() });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "PTR44", Value = report.PTR44.GetValueOrDefault().ToString() });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.PTR.430015

            #region VR.ATO.PTR.430016

            /*  VR.ATO.PTR.430016
            Description of main business and industry code must be present
    
            Legacy Rule Format:
            IF [PTR32] <> NULL AND ([PTR31] = NULL OR [PTR30] = NULLORBLANK) 
                RETURN VALIDATION MESSAGE
            ENDIF
            
            [PTR30] = PTR:RP:pyde.02.05:OrganisationDetails.MainIncomeActivity.Description
            [PTR31] = PTR:RP:pyde.02.30:OrganisationDetails.OrganisationIndustry2006Extended.Code
            [PTR32] = PTR:RP:pyde.02.11:OrganisationDetails.ActivityEvent.Code

            Technical Business Rule Format:
            ^PTR32 <> NULL AND (^PTR31 = NULL OR ^PTR30 = BLANK)
    
            Data Elements:
    
            RP:^PTR30 = pyde.02.05:OrganisationDetails.MainIncomeActivity.Description
    
            RP:^PTR31 = pyde.02.30:OrganisationDetails.OrganisationIndustry2006Extended.Code
    
            RP:^PTR32 = pyde.02.11:OrganisationDetails.ActivityEvent.Code
            */
            assertion = (report.PTR32 != null && (report.PTR31 == null || string.IsNullOrWhiteSpace(report.PTR30) == true));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.430016",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Description of main business and industry code must be present",
                    LongDescription = @"If Status of Business code is present, the Description of main business activity and Industry code are also required",
                    Location = "/xbrli:xbrl/pyde.02.05:OrganisationDetails.MainIncomeActivity.Description",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.PTR.430016" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "PTR32", Value = report.PTR32 });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "PTR31", Value = report.PTR31 });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "PTR30", Value = report.PTR30 });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.PTR.430016

            #region VR.ATO.PTR.430017

            /*  VR.ATO.PTR.430017
            PP business income must be present
    
            Legacy Rule Format:
            IF [PTR61] <> NULL AND ([PTR35] = NULL  AND [PTR39] = NULL AND [PTR43] = NULL)
               RETURN VALIDATION MESSAGE
            ENDIF
            
            [PTR35] = PTR:RP.JAUS.Prim:lrla.02.00:Remuneration.ABNNotQuotedPaymentGross.Amount
            [PTR39] = PTR:RP.JAUS.Prim:gfagg.02.00:GovernmentFunding.GovernmentIndustryPaymentsAssessable.Amount
            [PTR43] = PTR:RP.JAUS.Prim:bafpr1.02.04:Income.Other.Amount
            [PTR61] = PTR:RP.JAUS.Prim:bafpr1.02.23:Income.Net.Amount

            Technical Business Rule Format:
            ^PTR61 <> NULL AND (^PTR35 = NULL  AND ^PTR39 = NULL AND ^PTR43 = NULL)
    
            Data Elements:
    
            RP.JAUS.Prim:^PTR35 = lrla.02.00:Remuneration.ABNNotQuotedPaymentGross.Amount
    
            RP.JAUS.Prim:^PTR39 = gfagg.02.00:GovernmentFunding.GovernmentIndustryPaymentsAssessable.Amount
    
            RP.JAUS.Prim:^PTR43 = bafpr1.02.04:Income.Other.Amount
    
            RP.JAUS.Prim:^PTR61 = bafpr1.02.23:Income.Net.Amount
            */
            assertion = (report.PTR61 != null && (report.PTR35 == null && report.PTR39 == null && report.PTR43 == null));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.430017",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"PP business income must be present",
                    LongDescription = @"Net primary production income/loss amount is present without total business income from primary production (Gross payments subject to Foreign Resident Withholding, Gross payments where ABN not quoted, Assessable government industry payments, Other business income).",
                    Location = "/xbrli:xbrl/lrla.02.00:Remuneration.ABNNotQuotedPaymentGross.Amount[@contextRef='" + report.RPJAUSPrimId + "']",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.PTR.430017" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "PTR61", Value = report.PTR61.GetValueOrDefault().ToString() });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "PTR35", Value = report.PTR35.GetValueOrDefault().ToString() });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "PTR39", Value = report.PTR39.GetValueOrDefault().ToString() });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "PTR43", Value = report.PTR43.GetValueOrDefault().ToString() });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.PTR.430017

            #region VR.ATO.PTR.430018

            /*  VR.ATO.PTR.430018
            Total business income amount required
    
            Legacy Rule Format:
            IF ([PTR116] <> NULLORBLANK OR [PTR117] <> NULLORBLANK) AND ([PTR35] = NULL AND [PTR36] = NULL AND [PTR38] = NULL AND [PTR39] = NULL AND [PTR41] = NULL AND [PTR43] = NULL AND [PTR44] = NULL) 
                RETURN VALIDATION MESSAGE
            ENDIF
            
            [PTR35] = PTR:RP.JAUS.Prim:lrla.02.00:Remuneration.ABNNotQuotedPaymentGross.Amount
            [PTR36] = PTR:RP.JAUS.NonPrim:lrla.02.00:Remuneration.ABNNotQuotedPaymentGross.Amount
            [PTR38] = PTR:RP.JAUS.NonPrim:lrla.02.00:Remuneration.PaymentToForeignResidentGross.Amount
            [PTR39] = PTR:RP.JAUS.Prim:gfagg.02.00:GovernmentFunding.GovernmentIndustryPaymentsAssessable.Amount
            [PTR41] = PTR:RP.JAUS.NonPrim:gfagg.02.00:GovernmentFunding.GovernmentIndustryPaymentsAssessable.Amount
            [PTR43] = PTR:RP.JAUS.Prim:bafpr1.02.04:Income.Other.Amount
            [PTR44] = PTR:RP.JAUS.NonPrim:bafpr1.02.04:Income.Other.Amount
            [PTR116] = PTR:RP:pyde.02.00:OrganisationNameDetails.OrganisationalName.Text WHERE (pyde.02.00:OrganisationNameDetails.OrganisationalNameType.Code = "MTR" AND pyde.02.00:OrganisationNameDetails.Currency.Code = "C") IN TUPLE(orgname2.02.00:OrganisationNameDetails)
            [PTR117] = PTR:RP:pyde.02.00:AddressDetails.Line1.Text WHERE (pyde.02.01:AddressDetails.Usage.Code = "BUS" AND pyde.02.00:AddressDetails.Currency.Code = "C") IN TUPLE(address2.02.02:AddressDetails)

            Technical Business Rule Format:
            (^PTR116 <> BLANK OR ^PTR117 <> BLANK) AND (^PTR35 = NULL AND ^PTR36 = NULL AND ^PTR38 = NULL AND ^PTR39 = NULL AND ^PTR41 = NULL AND ^PTR43 = NULL AND ^PTR44 = NULL)
    
            Data Elements:
    
            RP.JAUS.Prim:^PTR35 = lrla.02.00:Remuneration.ABNNotQuotedPaymentGross.Amount
    
            RP.JAUS.NonPrim:^PTR36 = lrla.02.00:Remuneration.ABNNotQuotedPaymentGross.Amount
    
            RP.JAUS.NonPrim:^PTR38 = lrla.02.00:Remuneration.PaymentToForeignResidentGross.Amount
    
            RP.JAUS.Prim:^PTR39 = gfagg.02.00:GovernmentFunding.GovernmentIndustryPaymentsAssessable.Amount
    
            RP.JAUS.NonPrim:^PTR41 = gfagg.02.00:GovernmentFunding.GovernmentIndustryPaymentsAssessable.Amount
    
            RP.JAUS.Prim:^PTR43 = bafpr1.02.04:Income.Other.Amount
    
            RP.JAUS.NonPrim:^PTR44 = bafpr1.02.04:Income.Other.Amount
    
            RP:^PTR116 = pyde.02.00:OrganisationNameDetails.OrganisationalName.Text WHERE [OrganisationNameDetails.OrganisationalNameType.Code='MTR'].[OrganisationNameDetails.Currency.Code='C']
    
            RP:^PTR117 = pyde.02.00:AddressDetails.Line1.Text WHERE [AddressDetails.Usage.Code='BUS'].[AddressDetails.Currency.Code='C']
            */
            assertion = ((string.IsNullOrWhiteSpace(report.PTR116) != true || string.IsNullOrWhiteSpace(report.PTR117) != true) && (report.PTR35 == null && report.PTR36 == null && report.PTR38 == null && report.PTR39 == null && report.PTR41 == null && report.PTR43 == null && report.PTR44 == null));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.430018",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Total business income amount required",
                    LongDescription = @"Business name of main business or street address of main business is present and total business income (primary production or non-primary production) is missing at business income and expenses.",
                    Location = "/xbrli:xbrl/lrla.02.00:Remuneration.ABNNotQuotedPaymentGross.Amount[@contextRef='" + report.RPJAUSPrimId + "']",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.PTR.430018" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "PTR116", Value = report.PTR116 });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "PTR117", Value = report.PTR117 });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "PTR35", Value = report.PTR35.GetValueOrDefault().ToString() });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "PTR36", Value = report.PTR36.GetValueOrDefault().ToString() });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "PTR38", Value = report.PTR38.GetValueOrDefault().ToString() });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "PTR39", Value = report.PTR39.GetValueOrDefault().ToString() });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "PTR41", Value = report.PTR41.GetValueOrDefault().ToString() });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "PTR43", Value = report.PTR43.GetValueOrDefault().ToString() });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "PTR44", Value = report.PTR44.GetValueOrDefault().ToString() });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.PTR.430018

            #region VR.ATO.PTR.430021

            /*  VR.ATO.PTR.430021
            Monetary field must be unsigned, maximum of 11 numeric characters, no decimals
    
            Legacy Rule Format:
            IF [PTR35] <> NULL AND [PTR35] <> MONETARY(U,11,0)
               RETURN VALIDATION MESSAGE
            ENDIF
            
            [PTR35] = PTR:RP.JAUS.Prim:lrla.02.00:Remuneration.ABNNotQuotedPaymentGross.Amount

            Technical Business Rule Format:
            NotMonetary(^PTR35, 'U', 11, 0)
    
            Data Elements:
    
            RP.JAUS.Prim:^PTR35 = lrla.02.00:Remuneration.ABNNotQuotedPaymentGross.Amount
            */
            assertion = NotMonetary(report.PTR35, @"U", 11, 0);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.400011",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Field incorrect format. Exceeds maximum digits or decimals and/or contains illegal characters",
                    LongDescription = @"Field is in incorrect format for a monetary field. Field either exceeds maximum digits or decimals and/or contains illegal characters and/or for a signed field does not have the sign as the first character",
                    Location = "/xbrli:xbrl/lrla.02.00:Remuneration.ABNNotQuotedPaymentGross.Amount[@contextRef='" + report.RPJAUSPrimId + "']",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.PTR.430021" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "PTR35", Value = report.PTR35.GetValueOrDefault().ToString() });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.PTR.430021

            #region VR.ATO.PTR.430022

            /*  VR.ATO.PTR.430022
            Non-PP business income must be present
    
            Legacy Rule Format:
            IF [PTR62] <> NULL AND ([PTR36] = NULL AND [PTR38] = NULL AND [PTR41] = NULL AND [PTR44] = NULL) 
                RETURN VALIDATION MESSAGE
            ENDIF
            
            [PTR36] = PTR:RP.JAUS.NonPrim:lrla.02.00:Remuneration.ABNNotQuotedPaymentGross.Amount
            [PTR38] = PTR:RP.JAUS.NonPrim:lrla.02.00:Remuneration.PaymentToForeignResidentGross.Amount
            [PTR41] = PTR:RP.JAUS.NonPrim:gfagg.02.00:GovernmentFunding.GovernmentIndustryPaymentsAssessable.Amount
            [PTR44] = PTR:RP.JAUS.NonPrim:bafpr1.02.04:Income.Other.Amount
            [PTR62] = PTR:RP.JAUS.NonPrim:bafpr1.02.23:Income.Net.Amount

            Technical Business Rule Format:
            ^PTR62 <> NULL AND (^PTR36 = NULL AND ^PTR38 = NULL AND ^PTR41 = NULL AND ^PTR44 = NULL)
    
            Data Elements:
    
            RP.JAUS.NonPrim:^PTR36 = lrla.02.00:Remuneration.ABNNotQuotedPaymentGross.Amount
    
            RP.JAUS.NonPrim:^PTR38 = lrla.02.00:Remuneration.PaymentToForeignResidentGross.Amount
    
            RP.JAUS.NonPrim:^PTR41 = gfagg.02.00:GovernmentFunding.GovernmentIndustryPaymentsAssessable.Amount
    
            RP.JAUS.NonPrim:^PTR44 = bafpr1.02.04:Income.Other.Amount
    
            RP.JAUS.NonPrim:^PTR62 = bafpr1.02.23:Income.Net.Amount
            */
            assertion = (report.PTR62 != null && (report.PTR36 == null && report.PTR38 == null && report.PTR41 == null && report.PTR44 == null));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.430022",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Non-PP business income must be present",
                    LongDescription = @"Net non-primary production income/loss amount is present without total business income non-primary production (Gross payments where ABN not quoted, Gross payments subject to foreign resident withholding, Assessable government industry payments, Other business income).",
                    Location = "/xbrli:xbrl/lrla.02.00:Remuneration.ABNNotQuotedPaymentGross.Amount[@contextRef='" + report.RPJAUSNonPrimId + "']",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.PTR.430022" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "PTR62", Value = report.PTR62.GetValueOrDefault().ToString() });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "PTR36", Value = report.PTR36.GetValueOrDefault().ToString() });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "PTR38", Value = report.PTR38.GetValueOrDefault().ToString() });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "PTR41", Value = report.PTR41.GetValueOrDefault().ToString() });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "PTR44", Value = report.PTR44.GetValueOrDefault().ToString() });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.PTR.430022

            #region VR.ATO.PTR.430023

            /*  VR.ATO.PTR.430023
            Net non-PP income must be present
    
            Legacy Rule Format:
            IF [PTR62] = NULL AND ([PTR36] <> NULL OR [PTR38] <> NULL OR [PTR41] <> NULL OR [PTR44] <> NULL) 
                RETURN VALIDATION MESSAGE
            ENDIF
            
            [PTR36] = PTR:RP.JAUS.NonPrim:lrla.02.00:Remuneration.ABNNotQuotedPaymentGross.Amount
            [PTR38] = PTR:RP.JAUS.NonPrim:lrla.02.00:Remuneration.PaymentToForeignResidentGross.Amount
            [PTR41] = PTR:RP.JAUS.NonPrim:gfagg.02.00:GovernmentFunding.GovernmentIndustryPaymentsAssessable.Amount
            [PTR44] = PTR:RP.JAUS.NonPrim:bafpr1.02.04:Income.Other.Amount
            [PTR62] = PTR:RP.JAUS.NonPrim:bafpr1.02.23:Income.Net.Amount

            Technical Business Rule Format:
            ^PTR62 = NULL AND (^PTR36 <> NULL OR ^PTR38 <> NULL OR ^PTR41 <> NULL OR ^PTR44 <> NULL)
    
            Data Elements:
    
            RP.JAUS.NonPrim:^PTR62 = bafpr1.02.23:Income.Net.Amount
    
            RP.JAUS.NonPrim:^PTR36 = lrla.02.00:Remuneration.ABNNotQuotedPaymentGross.Amount
    
            RP.JAUS.NonPrim:^PTR38 = lrla.02.00:Remuneration.PaymentToForeignResidentGross.Amount
    
            RP.JAUS.NonPrim:^PTR41 = gfagg.02.00:GovernmentFunding.GovernmentIndustryPaymentsAssessable.Amount
    
            RP.JAUS.NonPrim:^PTR44 = bafpr1.02.04:Income.Other.Amount
            */
            assertion = (report.PTR62 == null && (report.PTR36 != null || report.PTR38 != null || report.PTR41 != null || report.PTR44 != null));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.430023",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Net non-PP income must be present",
                    LongDescription = @"Total business income from non-primary production (Gross payments where ABN not quoted, Gross payments subject to foreign resident withholding, Assessable government industry payments, Other business income) is present without net non-primary production income/loss amount.",
                    Location = "/xbrli:xbrl/bafpr1.02.23:Income.Net.Amount[@contextRef='" + report.RPJAUSNonPrimId + "']",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.PTR.430023" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "PTR62", Value = report.PTR62.GetValueOrDefault().ToString() });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "PTR36", Value = report.PTR36.GetValueOrDefault().ToString() });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "PTR38", Value = report.PTR38.GetValueOrDefault().ToString() });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "PTR41", Value = report.PTR41.GetValueOrDefault().ToString() });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "PTR44", Value = report.PTR44.GetValueOrDefault().ToString() });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.PTR.430023

            #region VR.ATO.PTR.430024

            /*  VR.ATO.PTR.430024
            Monetary field must be unsigned, maximum of 11 numeric characters, no decimals
    
            Legacy Rule Format:
            IF [PTR36] <> NULL AND [PTR36] <> MONETARY(U,11,0)
               RETURN VALIDATION MESSAGE
            ENDIF
            
            [PTR36] = PTR:RP.JAUS.NonPrim:lrla.02.00:Remuneration.ABNNotQuotedPaymentGross.Amount

            Technical Business Rule Format:
            NotMonetary(^PTR36, 'U', 11, 0)
    
            Data Elements:
    
            RP.JAUS.NonPrim:^PTR36 = lrla.02.00:Remuneration.ABNNotQuotedPaymentGross.Amount
            */
            assertion = NotMonetary(report.PTR36, @"U", 11, 0);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.400011",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Field incorrect format. Exceeds maximum digits or decimals and/or contains illegal characters",
                    LongDescription = @"Field is in incorrect format for a monetary field. Field either exceeds maximum digits or decimals and/or contains illegal characters and/or for a signed field does not have the sign as the first character",
                    Location = "/xbrli:xbrl/lrla.02.00:Remuneration.ABNNotQuotedPaymentGross.Amount[@contextRef='" + report.RPJAUSNonPrimId + "']",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.PTR.430024" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "PTR36", Value = report.PTR36.GetValueOrDefault().ToString() });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.PTR.430024

            #region VR.ATO.PTR.430027

            /*  VR.ATO.PTR.430027
            Credit for tax withheld - foreign resident withholding (exlcluding capital gains) must not exceed 50% of gross payments
    
            Legacy Rule Format:
            IF [PTR350] > 0 AND [PTR350] > ([PTR38] *0.50) + 1
                RETURN VALIDATION MESSAGE
            ENDIF
            
            [PTR38] = PTR:RP.JAUS.NonPrim:lrla.02.00:Remuneration.PaymentToForeignResidentGross.Amount
            [PTR350] = PTR:RP.JAUS:rvctc2.02.13:IncomeTax.PayAsYouGoWithholding.CreditForAmountsWithheldFromForeignResidents.Amount

            Technical Business Rule Format:
            ^PTR350 > 0 AND ^PTR350 > (^PTR38 *0.50) + 1
    
            Data Elements:
    
            RP.JAUS:^PTR350 = rvctc2.02.13:IncomeTax.PayAsYouGoWithholding.CreditForAmountsWithheldFromForeignResidents.Amount
    
            RP.JAUS.NonPrim:^PTR38 = lrla.02.00:Remuneration.PaymentToForeignResidentGross.Amount
            */
            assertion = (report.PTR350.GetValueOrDefault() > 0 && report.PTR350.GetValueOrDefault() > report.PTR38.GetValueOrDefault() * 0.50M + 1);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.PTR.430027",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Credit for tax withheld - foreign resident withholding (excluding capital gains) must not exceed 50% of gross payments",
                    LongDescription = @"Credit for tax withheld - foreign resident withholding (excluding capital gains) must not exceed 50% of total of Primary production and Non-primary production Gross payments subject to foreign resident withholding",
                    Location = "/xbrli:xbrl/rvctc2.02.13:IncomeTax.PayAsYouGoWithholding.CreditForAmountsWithheldFromForeignResidents.Amount[@contextRef='" + report.RPJAUSId + "']",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.PTR.430027" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "PTR350", Value = report.PTR350.GetValueOrDefault().ToString() });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "PTR38", Value = report.PTR38.GetValueOrDefault().ToString() });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.PTR.430027

            #region VR.ATO.PTR.430028

            /*  VR.ATO.PTR.430028
            Monetary field must be unsigned, maximum of 11 numeric characters, no decimals
    
            Legacy Rule Format:
            IF [PTR38] <> NULL AND [PTR38] <> MONETARY(U,11,0)
               RETURN VALIDATION MESSAGE
            ENDIF
            
            [PTR38] = PTR:RP.JAUS.NonPrim:lrla.02.00:Remuneration.PaymentToForeignResidentGross.Amount

            Technical Business Rule Format:
            NotMonetary(^PTR38, 'U', 11, 0)
    
            Data Elements:
    
            RP.JAUS.NonPrim:^PTR38 = lrla.02.00:Remuneration.PaymentToForeignResidentGross.Amount
            */
            assertion = NotMonetary(report.PTR38, @"U", 11, 0);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.400011",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Field incorrect format. Exceeds maximum digits or decimals and/or contains illegal characters",
                    LongDescription = @"Field is in incorrect format for a monetary field. Field either exceeds maximum digits or decimals and/or contains illegal characters and/or for a signed field does not have the sign as the first character",
                    Location = "/xbrli:xbrl/lrla.02.00:Remuneration.PaymentToForeignResidentGross.Amount",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.PTR.430028" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "PTR38", Value = report.PTR38.GetValueOrDefault().ToString() });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.PTR.430028

            #region VR.ATO.PTR.430029

            /*  VR.ATO.PTR.430029
            Monetary field must be unsigned, maximum of 11 numeric characters, no decimals
    
            Legacy Rule Format:
            IF [PTR39] <> NULL AND [PTR39] <> MONETARY(U,11,0)
               RETURN VALIDATION MESSAGE
            ENDIF
            
            [PTR39] = PTR:RP.JAUS.Prim:gfagg.02.00:GovernmentFunding.GovernmentIndustryPaymentsAssessable.Amount

            Technical Business Rule Format:
            NotMonetary(^PTR39, 'U', 11, 0)
    
            Data Elements:
    
            RP.JAUS.Prim:^PTR39 = gfagg.02.00:GovernmentFunding.GovernmentIndustryPaymentsAssessable.Amount
            */
            assertion = NotMonetary(report.PTR39, @"U", 11, 0);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.400011",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Field incorrect format. Exceeds maximum digits or decimals and/or contains illegal characters",
                    LongDescription = @"Field is in incorrect format for a monetary field. Field either exceeds maximum digits or decimals and/or contains illegal characters and/or for a signed field does not have the sign as the first character",
                    Location = "/xbrli:xbrl/gfagg.02.00:GovernmentFunding.GovernmentIndustryPaymentsAssessable.Amount[@contextRef='" + report.RPJAUSPrimId + "']",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.PTR.430029" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "PTR39", Value = report.PTR39.GetValueOrDefault().ToString() });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.PTR.430029

            #region VR.ATO.PTR.430030

            /*  VR.ATO.PTR.430030
            PP Assessable government industry payments amount must be present
    
            Legacy Rule Format:
            IF [PTR258] = TRUE AND ([PTR39] = 0 OR [PTR39] = NULL) 
                RETURN VALIDATION MESSAGE
            ENDIF
            
            [PTR39] = PTR:RP.JAUS.Prim:gfagg.02.00:GovernmentFunding.GovernmentIndustryPaymentsAssessable.Amount
            [PTR258] = PTR:RP.JAUS.Prim:gfagg.02.02:GovernmentFunding.GovernmentIndustryPaymentsAssessable.Indicator

            Technical Business Rule Format:
            ^PTR258 = TRUE AND (^PTR39 = 0 OR ^PTR39 = NULL)
    
            Data Elements:
    
            RP.JAUS.Prim:^PTR39 = gfagg.02.00:GovernmentFunding.GovernmentIndustryPaymentsAssessable.Amount
    
            RP.JAUS.Prim:^PTR258 = gfagg.02.02:GovernmentFunding.GovernmentIndustryPaymentsAssessable.Indicator
            */
            assertion = (report.PTR258 == true && (report.PTR39 == 0 || report.PTR39 == null));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.430030",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"PP Assessable government industry payments amount must be present",
                    LongDescription = @"If Primary production - Assessable government industry payments action code is present, Primary production - Assessable government industry payments amount must be present and not zero",
                    Location = "/xbrli:xbrl/gfagg.02.00:GovernmentFunding.GovernmentIndustryPaymentsAssessable.Amount[@contextRef='" + report.RPJAUSPrimId + "']",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.PTR.430030" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "PTR258", Value = GetValueOrEmpty(report.PTR258) });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "PTR39", Value = report.PTR39.GetValueOrDefault().ToString() });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.PTR.430030

            #region VR.ATO.PTR.430031

            /*  VR.ATO.PTR.430031
            Monetary field must be unsigned, maximum of 11 numeric characters, no decimals
    
            Legacy Rule Format:
            IF [PTR41] <> NULL AND [PTR41] <> MONETARY(U,11,0)
               RETURN VALIDATION MESSAGE
            ENDIF
            
            [PTR41] = PTR:RP.JAUS.NonPrim:gfagg.02.00:GovernmentFunding.GovernmentIndustryPaymentsAssessable.Amount

            Technical Business Rule Format:
            NotMonetary(^PTR41, 'U', 11, 0)
    
            Data Elements:
    
            RP.JAUS.NonPrim:^PTR41 = gfagg.02.00:GovernmentFunding.GovernmentIndustryPaymentsAssessable.Amount
            */
            assertion = NotMonetary(report.PTR41, @"U", 11, 0);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.400011",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Field incorrect format. Exceeds maximum digits or decimals and/or contains illegal characters",
                    LongDescription = @"Field is in incorrect format for a monetary field. Field either exceeds maximum digits or decimals and/or contains illegal characters and/or for a signed field does not have the sign as the first character",
                    Location = "/xbrli:xbrl/gfagg.02.00:GovernmentFunding.GovernmentIndustryPaymentsAssessable.Amount[@contextRef='" + report.RPJAUSNonPrimId + "']",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.PTR.430031" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "PTR41", Value = report.PTR41.GetValueOrDefault().ToString() });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.PTR.430031

            #region VR.ATO.PTR.430032

            /*  VR.ATO.PTR.430032
            Non-PP Assessable government industry payments amount must be present
    
            Legacy Rule Format:
            IF [PTR259] = TRUE AND ([PTR41] = 0 OR [PTR41] = NULL) 
                RETURN VALIDATION MESSAGE
            ENDIF
            
            [PTR41] = PTR:RP.JAUS.NonPrim:gfagg.02.00:GovernmentFunding.GovernmentIndustryPaymentsAssessable.Amount
            [PTR259] = PTR:RP.JAUS.NonPrim:gfagg.02.02:GovernmentFunding.GovernmentIndustryPaymentsAssessable.Indicator

            Technical Business Rule Format:
            ^PTR259 = TRUE AND (^PTR41 = 0 OR ^PTR41 = NULL)
    
            Data Elements:
    
            RP.JAUS.NonPrim:^PTR41 = gfagg.02.00:GovernmentFunding.GovernmentIndustryPaymentsAssessable.Amount
    
            RP.JAUS.NonPrim:^PTR259 = gfagg.02.02:GovernmentFunding.GovernmentIndustryPaymentsAssessable.Indicator
            */
            assertion = (report.PTR259 == true && (report.PTR41 == 0 || report.PTR41 == null));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.430032",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Non-PP Assessable government industry payments amount must be present",
                    LongDescription = @"If Non-primary production - Assessable government industry payments action code is present, Non-primary production - Assessable government industry payments must be present and not zero",
                    Location = "/xbrli:xbrl/gfagg.02.00:GovernmentFunding.GovernmentIndustryPaymentsAssessable.Amount[@contextRef='" + report.RPJAUSNonPrimId + "']",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.PTR.430032" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "PTR259", Value = GetValueOrEmpty(report.PTR259) });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "PTR41", Value = report.PTR41.GetValueOrDefault().ToString() });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.PTR.430032

            #region VR.ATO.PTR.430033

            /*  VR.ATO.PTR.430033
            Monetary field may be signed, maximum of 11 chars, no decimals
    
            Legacy Rule Format:
            IF [PTR43] <> NULL AND [PTR43] <> MONETARY(S,11,0)
               RETURN VALIDATION MESSAGE
            ENDIF
            
            [PTR43] = PTR:RP.JAUS.Prim:bafpr1.02.04:Income.Other.Amount

            Technical Business Rule Format:
            NotMonetary(^PTR43, 'S', 11, 0)
    
            Data Elements:
    
            RP.JAUS.Prim:^PTR43 = bafpr1.02.04:Income.Other.Amount
            */
            assertion = NotMonetary(report.PTR43, @"S", 11, 0);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.400011",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Field incorrect format. Exceeds maximum digits or decimals and/or contains illegal characters",
                    LongDescription = @"Field is in incorrect format for a monetary field. Field either exceeds maximum digits or decimals and/or contains illegal characters and/or for a signed field does not have the sign as the first character",
                    Location = "/xbrli:xbrl/bafpr1.02.04:Income.Other.Amount[@contextRef='" + report.RPJAUSPrimId + "']",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.PTR.430033" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "PTR43", Value = report.PTR43.GetValueOrDefault().ToString() });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.PTR.430033

            #region VR.ATO.PTR.430034

            /*  VR.ATO.PTR.430034
            Monetary field may be signed, maximum of 11 chars, no decimals
    
            Legacy Rule Format:
            IF [PTR44] <> NULL AND [PTR44] <> MONETARY(S,11,0)
               RETURN VALIDATION MESSAGE
            ENDIF
            
            [PTR44] = PTR:RP.JAUS.NonPrim:bafpr1.02.04:Income.Other.Amount

            Technical Business Rule Format:
            NotMonetary(^PTR44, 'S', 11, 0)
    
            Data Elements:
    
            RP.JAUS.NonPrim:^PTR44 = bafpr1.02.04:Income.Other.Amount
            */
            assertion = NotMonetary(report.PTR44, @"S", 11, 0);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.400011",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Field incorrect format. Exceeds maximum digits or decimals and/or contains illegal characters",
                    LongDescription = @"Field is in incorrect format for a monetary field. Field either exceeds maximum digits or decimals and/or contains illegal characters and/or for a signed field does not have the sign as the first character",
                    Location = "/xbrli:xbrl/bafpr1.02.04:Income.Other.Amount[@contextRef='" + report.RPJAUSNonPrimId + "']",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.PTR.430034" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "PTR44", Value = report.PTR44.GetValueOrDefault().ToString() });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.PTR.430034

            #region VR.ATO.PTR.430035

            /*  VR.ATO.PTR.430035
            Monetary field must be unsigned, maximum of 11 numeric characters, no decimals
    
            Legacy Rule Format:
            IF [PTR45] <> NULL AND [PTR45] <> MONETARY(U,11,0)
               RETURN VALIDATION MESSAGE
            ENDIF
            
            [PTR45] = PTR:RP.JAUS:bafpr1.02.02:Expense.ForeignResidentWithholding.Amount

            Technical Business Rule Format:
            NotMonetary(^PTR45, 'U', 11, 0)
    
            Data Elements:
    
            RP.JAUS:^PTR45 = bafpr1.02.02:Expense.ForeignResidentWithholding.Amount
            */
            assertion = NotMonetary(report.PTR45, @"U", 11, 0);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.400011",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Field incorrect format. Exceeds maximum digits or decimals and/or contains illegal characters",
                    LongDescription = @"Field is in incorrect format for a monetary field. Field either exceeds maximum digits or decimals and/or contains illegal characters and/or for a signed field does not have the sign as the first character",
                    Location = "/xbrli:xbrl/bafpr1.02.02:Expense.ForeignResidentWithholding.Amount",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.PTR.430035" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "PTR45", Value = report.PTR45.GetValueOrDefault().ToString() });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.PTR.430035

            #region VR.ATO.PTR.430036

            /*  VR.ATO.PTR.430036
            Monetary field must be unsigned, maximum of 11 numeric characters, no decimals
    
            Legacy Rule Format:
            IF [PTR47] <> NULL AND [PTR47] <> MONETARY(U,11,0)
               RETURN VALIDATION MESSAGE
            ENDIF
            
            [PTR47] = PTR:RP.JAUS:emsup.02.02:SuperannuationContribution.EmployerContributions.Amount

            Technical Business Rule Format:
            NotMonetary(^PTR47, 'U', 11, 0)
    
            Data Elements:
    
            RP.JAUS:^PTR47 = emsup.02.02:SuperannuationContribution.EmployerContributions.Amount
            */
            assertion = NotMonetary(report.PTR47, @"U", 11, 0);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.400011",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Field incorrect format. Exceeds maximum digits or decimals and/or contains illegal characters",
                    LongDescription = @"Field is in incorrect format for a monetary field. Field either exceeds maximum digits or decimals and/or contains illegal characters and/or for a signed field does not have the sign as the first character",
                    Location = "/xbrli:xbrl/emsup.02.02:SuperannuationContribution.EmployerContributions.Amount",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.PTR.430036" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "PTR47", Value = report.PTR47.GetValueOrDefault().ToString() });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.PTR.430036

            #region VR.ATO.PTR.430038

            /*  VR.ATO.PTR.430038
            Monetary field must be unsigned, maximum of 11 numeric characters, no decimals
    
            Legacy Rule Format:
            IF [PTR50] <> NULL AND [PTR50] <> MONETARY(U,11,0)
               RETURN VALIDATION MESSAGE
            ENDIF
            
            [PTR50] = PTR:RP.JAUS:bafpr2.02.04:Expense.Operating.LeaseTotal.Amount

            Technical Business Rule Format:
            NotMonetary(^PTR50, 'U', 11, 0)
    
            Data Elements:
    
            RP.JAUS:^PTR50 = bafpr2.02.04:Expense.Operating.LeaseTotal.Amount
            */
            assertion = NotMonetary(report.PTR50, @"U", 11, 0);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.400011",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Field incorrect format. Exceeds maximum digits or decimals and/or contains illegal characters",
                    LongDescription = @"Field is in incorrect format for a monetary field. Field either exceeds maximum digits or decimals and/or contains illegal characters and/or for a signed field does not have the sign as the first character",
                    Location = "/xbrli:xbrl/bafpr2.02.04:Expense.Operating.LeaseTotal.Amount",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.PTR.430038" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "PTR50", Value = report.PTR50.GetValueOrDefault().ToString() });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.PTR.430038

            #region VR.ATO.PTR.430040

            /*  VR.ATO.PTR.430040
            Overseas royalty expenses must not exceed Total royalty expenses
    
            Legacy Rule Format:
            IF [PTR138] > 0 AND [PTR138] > [PTR347] 
                RETURN VALIDATION MESSAGE
            ENDIF
            
            [PTR138] = PTR:RP.JFOREIGN:bafpr1.02.27:Expense.Royalties.Amount
            [PTR347] = PTR:RP.JAUS:bafpr1.02.27:Expense.Royalties.Amount

            Technical Business Rule Format:
            ^PTR138 > 0 AND ^PTR138 > ^PTR347
    
            Data Elements:
    
            RP.JFOREIGN:^PTR138 = bafpr1.02.27:Expense.Royalties.Amount
    
            RP.JAUS:^PTR347 = bafpr1.02.27:Expense.Royalties.Amount
            */
            assertion = (report.PTR138.GetValueOrDefault() > 0 && report.PTR138.GetValueOrDefault() > report.PTR347.GetValueOrDefault());
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.430040",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Overseas royalty expenses must not exceed Total royalty expenses",
                    LongDescription = @"Overseas royalty expenses - under Business and professional items must not be greater than Total royalty expenses - under Business income and expenses",
                    Location = "/xbrli:xbrl/bafpr1.02.27:Expense.Royalties.Amount[@contextRef='" + report.RPJFOREIGNId + "']",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.PTR.430040" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "PTR138", Value = report.PTR138.GetValueOrDefault().ToString() });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "PTR347", Value = report.PTR347.GetValueOrDefault().ToString() });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.PTR.430040

            #region VR.ATO.PTR.430044

            /*  VR.ATO.PTR.430044
            Motor vehicle expenses amount must be present if action code is present
    
            Legacy Rule Format:
            IF [PTR56] <> NULL AND ([PTR348] = 0 OR [PTR348] = NULL)
                RETURN VALIDATION MESSAGE
            ENDIF
            
            [PTR56] = PTR:RP.JAUS:bafpr2.02.04:Expense.Operating.MotorVehicleTypeOrClaimMethod.Code
            [PTR348] = PTR:RP.JAUS:bafpr2.02.16:Expense.Operating.MotorVehicle.Amount

            Technical Business Rule Format:
            ^PTR56 <> NULL AND (^PTR348 = 0 OR ^PTR348 = NULL)
    
            Data Elements:
    
            RP.JAUS:^PTR348 = bafpr2.02.16:Expense.Operating.MotorVehicle.Amount
    
            RP.JAUS:^PTR56 = bafpr2.02.04:Expense.Operating.MotorVehicleTypeOrClaimMethod.Code
            */
            assertion = (report.PTR56 != null && (report.PTR348 == 0 || report.PTR348 == null));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.PTR.430044",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Motor vehicle expenses amount must be present if action code is present",
                    LongDescription = @"If Motor vehicle expenses action code is present, there must be a motor vehicle expense amount",
                    Location = "/xbrli:xbrl/bafpr2.02.16:Expense.Operating.MotorVehicle.Amount",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.PTR.430044" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "PTR56", Value = report.PTR56 });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "PTR348", Value = report.PTR348.GetValueOrDefault().ToString() });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.PTR.430044

            #region VR.ATO.PTR.430046

            /*  VR.ATO.PTR.430046
            Motor vehicle expenses action code must be present
    
            Legacy Rule Format:
            IF [PTR348] > 0 AND [PTR56] = NULL
                RETURN VALIDATION MESSAGE
            ENDIF
            
            [PTR56] = PTR:RP.JAUS:bafpr2.02.04:Expense.Operating.MotorVehicleTypeOrClaimMethod.Code
            [PTR348] = PTR:RP.JAUS:bafpr2.02.16:Expense.Operating.MotorVehicle.Amount

            Technical Business Rule Format:
            ^PTR348 > 0 AND ^PTR56 = NULL
    
            Data Elements:
    
            RP.JAUS:^PTR56 = bafpr2.02.04:Expense.Operating.MotorVehicleTypeOrClaimMethod.Code
    
            RP.JAUS:^PTR348 = bafpr2.02.16:Expense.Operating.MotorVehicle.Amount
            */
            assertion = (report.PTR348.GetValueOrDefault() > 0 && report.PTR56 == null);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.PTR.430046",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Motor vehicle expenses action code must be present",
                    LongDescription = @"If a motor vehicle expense amount is present, a motor vehicle expense action code must be present. Codes are: S - Cents per Kilometre, T - 12% of cost of car, O - One-third actual expenses, B - Logbook, or N - Expenses of other vehicles",
                    Location = "/xbrli:xbrl/bafpr2.02.04:Expense.Operating.MotorVehicleTypeOrClaimMethod.Code",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.PTR.430046" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "PTR348", Value = report.PTR348.GetValueOrDefault().ToString() });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "PTR56", Value = report.PTR56 });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.PTR.430046

            #region VR.ATO.PTR.430047

            /*  VR.ATO.PTR.430047
            Monetary field may be signed, maximum of 11 chars, no decimals
    
            Legacy Rule Format:
            IF [PTR59] <> NULL AND [PTR59] <> MONETARY(S,11,0)
               RETURN VALIDATION MESSAGE
            ENDIF
            
            [PTR59] = PTR:RP.JAUS:bafpr1.02.04:Income.ReconciliationAdjustmentTotal.Amount

            Technical Business Rule Format:
            NotMonetary(^PTR59, 'S', 11, 0)
    
            Data Elements:
    
            RP.JAUS:^PTR59 = bafpr1.02.04:Income.ReconciliationAdjustmentTotal.Amount
            */
            assertion = NotMonetary(report.PTR59, @"S", 11, 0);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.400011",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Field incorrect format. Exceeds maximum digits or decimals and/or contains illegal characters",
                    LongDescription = @"Field is in incorrect format for a monetary field. Field either exceeds maximum digits or decimals and/or contains illegal characters and/or for a signed field does not have the sign as the first character",
                    Location = "/xbrli:xbrl/bafpr1.02.04:Income.ReconciliationAdjustmentTotal.Amount",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.PTR.430047" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "PTR59", Value = report.PTR59.GetValueOrDefault().ToString() });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.PTR.430047

            #region VR.ATO.PTR.430048

            /*  VR.ATO.PTR.430048
            Monetary field may be signed, maximum of 11 chars, no decimals
    
            Legacy Rule Format:
            IF [PTR328] <> NULL AND [PTR328] <> MONETARY(S,11,0)
               RETURN VALIDATION MESSAGE
            ENDIF
            
            [PTR328] = PTR:RP.JAUS:bafpr1.02.10:Expense.ReconciliationAdjustmentTotal.Amount

            Technical Business Rule Format:
            NotMonetary(^PTR328, 'S', 11, 0)
    
            Data Elements:
    
            RP.JAUS:^PTR328 = bafpr1.02.10:Expense.ReconciliationAdjustmentTotal.Amount
            */
            assertion = NotMonetary(report.PTR328, @"S", 11, 0);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.400011",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Field incorrect format. Exceeds maximum digits or decimals and/or contains illegal characters",
                    LongDescription = @"Field is in incorrect format for a monetary field. Field either exceeds maximum digits or decimals and/or contains illegal characters and/or for a signed field does not have the sign as the first character",
                    Location = "/xbrli:xbrl/bafpr1.02.10:Expense.ReconciliationAdjustmentTotal.Amount",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.PTR.430048" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "PTR328", Value = report.PTR328.GetValueOrDefault().ToString() });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.PTR.430048

            #region VR.ATO.PTR.430049

            /*  VR.ATO.PTR.430049
            PP Net income or loss from business must be present
    
            Legacy Rule Format:
            IF ([PTR35] <> NULL OR [PTR39] <> NULL OR [PTR43] <> NULL) AND [PTR61] = NULL 
                RETURN VALIDATION MESSAGE
            ENDIF
            
            [PTR35] = PTR:RP.JAUS.Prim:lrla.02.00:Remuneration.ABNNotQuotedPaymentGross.Amount
            [PTR39] = PTR:RP.JAUS.Prim:gfagg.02.00:GovernmentFunding.GovernmentIndustryPaymentsAssessable.Amount
            [PTR43] = PTR:RP.JAUS.Prim:bafpr1.02.04:Income.Other.Amount
            [PTR61] = PTR:RP.JAUS.Prim:bafpr1.02.23:Income.Net.Amount

            Technical Business Rule Format:
            (^PTR35 <> NULL OR ^PTR39 <> NULL OR ^PTR43 <> NULL) AND ^PTR61 = NULL
    
            Data Elements:
    
            RP.JAUS.Prim:^PTR61 = bafpr1.02.23:Income.Net.Amount
    
            RP.JAUS.Prim:^PTR35 = lrla.02.00:Remuneration.ABNNotQuotedPaymentGross.Amount
    
            RP.JAUS.Prim:^PTR39 = gfagg.02.00:GovernmentFunding.GovernmentIndustryPaymentsAssessable.Amount
    
            RP.JAUS.Prim:^PTR43 = bafpr1.02.04:Income.Other.Amount
            */
            assertion = ((report.PTR35 != null || report.PTR39 != null || report.PTR43 != null) && report.PTR61 == null);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.430049",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"PP Net income or loss from business must be present",
                    LongDescription = @"If Total business income from primary production (Gross payments, Assessable government industry payments, other business income) is present, Primary production Net primary production income or loss from business amount must be present",
                    Location = "/xbrli:xbrl/bafpr1.02.23:Income.Net.Amount[@contextRef='" + report.RPJAUSPrimId + "']",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.PTR.430049" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "PTR35", Value = report.PTR35.GetValueOrDefault().ToString() });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "PTR39", Value = report.PTR39.GetValueOrDefault().ToString() });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "PTR43", Value = report.PTR43.GetValueOrDefault().ToString() });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "PTR61", Value = report.PTR61.GetValueOrDefault().ToString() });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.PTR.430049

            #region VR.ATO.PTR.430050

            /*  VR.ATO.PTR.430050
            Total Net income or loss from business must equal total Income minus total Expenses
    
            Legacy Rule Format:
            IF ([PTR35] + [PTR36] + [PTR38] + [PTR39] + [PTR41] + [PTR43] + [PTR44]) - ([PTR45] + [PTR46] + [PTR47] + [PTR343] + [PTR344] + [PTR50] + [PTR345] + [PTR346] + [PTR347] + [PTR54] + [PTR348] + [PTR349] + [PTR58]) + [PTR59] + [PTR328] <> ([PTR61] + [PTR62]) 
                RETURN VALIDATION MESSAGE
            ENDIF
            
            [PTR35] = PTR:RP.JAUS.Prim:lrla.02.00:Remuneration.ABNNotQuotedPaymentGross.Amount
            [PTR36] = PTR:RP.JAUS.NonPrim:lrla.02.00:Remuneration.ABNNotQuotedPaymentGross.Amount
            [PTR38] = PTR:RP.JAUS.NonPrim:lrla.02.00:Remuneration.PaymentToForeignResidentGross.Amount
            [PTR39] = PTR:RP.JAUS.Prim:gfagg.02.00:GovernmentFunding.GovernmentIndustryPaymentsAssessable.Amount
            [PTR41] = PTR:RP.JAUS.NonPrim:gfagg.02.00:GovernmentFunding.GovernmentIndustryPaymentsAssessable.Amount
            [PTR43] = PTR:RP.JAUS.Prim:bafpr1.02.04:Income.Other.Amount
            [PTR44] = PTR:RP.JAUS.NonPrim:bafpr1.02.04:Income.Other.Amount
            [PTR45] = PTR:RP.JAUS:bafpr1.02.02:Expense.ForeignResidentWithholding.Amount
            [PTR46] = PTR:RP.JAUS:lrla.02.16:Remuneration.WagesAndSalaries.PaymentsContractorAndConsultant.Amount
            [PTR47] = PTR:RP.JAUS:emsup.02.02:SuperannuationContribution.EmployerContributions.Amount
            [PTR50] = PTR:RP.JAUS:bafpr2.02.04:Expense.Operating.LeaseTotal.Amount
            [PTR54] = PTR:RP.JAUS:bafpr1.02.04:Expense.DepreciationandAmortisation.Amount
            [PTR58] = PTR:RP.JAUS:bafpr1.02.04:Expense.DeductibleOtherTotal.Amount
            [PTR59] = PTR:RP.JAUS:bafpr1.02.04:Income.ReconciliationAdjustmentTotal.Amount
            [PTR61] = PTR:RP.JAUS.Prim:bafpr1.02.23:Income.Net.Amount
            [PTR62] = PTR:RP.JAUS.NonPrim:bafpr1.02.23:Income.Net.Amount
            [PTR328] = PTR:RP.JAUS:bafpr1.02.10:Expense.ReconciliationAdjustmentTotal.Amount
            [PTR343] = PTR:RP.JAUS:bafpr2.02.16:Expense.Operating.CostOfSales.Amount
            [PTR344] = PTR:RP.JAUS:bafpr2.02.16:Expense.Operating.BadDebts.Amount
            [PTR345] = PTR:RP.JAUS:bafpr2.02.16:Expense.Operating.Rent.Amount
            [PTR346] = PTR:RP.JAUS:bafpr2.02.17:Expense.Operating.Interest.Amount
            [PTR347] = PTR:RP.JAUS:bafpr1.02.27:Expense.Royalties.Amount
            [PTR348] = PTR:RP.JAUS:bafpr2.02.16:Expense.Operating.MotorVehicle.Amount
            [PTR349] = PTR:RP.JAUS:bafpr2.02.16:Expense.Operating.RepairsAndMaintenance.Amount

            Technical Business Rule Format:
            (^PTR35 + ^PTR36 + ^PTR38 + ^PTR39 + ^PTR41 + ^PTR43 + ^PTR44) - (^PTR45 + ^PTR46 + ^PTR47 + ^PTR343 + ^PTR344 + ^PTR50 + ^PTR345 + ^PTR346 + ^PTR347 + ^PTR54 + ^PTR348 + ^PTR349 + ^PTR58) + ^PTR59 + ^PTR328 <> (^PTR61 + ^PTR62)
    
            Data Elements:
    
            RP.JAUS.Prim:^PTR61 = bafpr1.02.23:Income.Net.Amount
    
            RP.JAUS.Prim:^PTR35 = lrla.02.00:Remuneration.ABNNotQuotedPaymentGross.Amount
    
            RP.JAUS.NonPrim:^PTR36 = lrla.02.00:Remuneration.ABNNotQuotedPaymentGross.Amount
    
            RP.JAUS.NonPrim:^PTR38 = lrla.02.00:Remuneration.PaymentToForeignResidentGross.Amount
    
            RP.JAUS.Prim:^PTR39 = gfagg.02.00:GovernmentFunding.GovernmentIndustryPaymentsAssessable.Amount
    
            RP.JAUS.NonPrim:^PTR41 = gfagg.02.00:GovernmentFunding.GovernmentIndustryPaymentsAssessable.Amount
    
            RP.JAUS.Prim:^PTR43 = bafpr1.02.04:Income.Other.Amount
    
            RP.JAUS.NonPrim:^PTR44 = bafpr1.02.04:Income.Other.Amount
    
            RP.JAUS:^PTR45 = bafpr1.02.02:Expense.ForeignResidentWithholding.Amount
    
            RP.JAUS:^PTR46 = lrla.02.16:Remuneration.WagesAndSalaries.PaymentsContractorAndConsultant.Amount
    
            RP.JAUS:^PTR47 = emsup.02.02:SuperannuationContribution.EmployerContributions.Amount
    
            RP.JAUS:^PTR50 = bafpr2.02.04:Expense.Operating.LeaseTotal.Amount
    
            RP.JAUS:^PTR54 = bafpr1.02.04:Expense.DepreciationandAmortisation.Amount
    
            RP.JAUS:^PTR58 = bafpr1.02.04:Expense.DeductibleOtherTotal.Amount
    
            RP.JAUS:^PTR59 = bafpr1.02.04:Income.ReconciliationAdjustmentTotal.Amount
    
            RP.JAUS.NonPrim:^PTR62 = bafpr1.02.23:Income.Net.Amount
    
            RP.JAUS:^PTR328 = bafpr1.02.10:Expense.ReconciliationAdjustmentTotal.Amount
    
            RP.JAUS:^PTR343 = bafpr2.02.16:Expense.Operating.CostOfSales.Amount
    
            RP.JAUS:^PTR344 = bafpr2.02.16:Expense.Operating.BadDebts.Amount
    
            RP.JAUS:^PTR345 = bafpr2.02.16:Expense.Operating.Rent.Amount
    
            RP.JAUS:^PTR346 = bafpr2.02.17:Expense.Operating.Interest.Amount
    
            RP.JAUS:^PTR347 = bafpr1.02.27:Expense.Royalties.Amount
    
            RP.JAUS:^PTR348 = bafpr2.02.16:Expense.Operating.MotorVehicle.Amount
    
            RP.JAUS:^PTR349 = bafpr2.02.16:Expense.Operating.RepairsAndMaintenance.Amount
            */
            assertion = (report.PTR35.GetValueOrDefault() + report.PTR36.GetValueOrDefault() + report.PTR38.GetValueOrDefault() + report.PTR39.GetValueOrDefault() + report.PTR41.GetValueOrDefault() + report.PTR43.GetValueOrDefault() + report.PTR44.GetValueOrDefault() - (report.PTR45.GetValueOrDefault() + report.PTR46.GetValueOrDefault() + report.PTR47.GetValueOrDefault() + report.PTR343.GetValueOrDefault() + report.PTR344.GetValueOrDefault() + report.PTR50.GetValueOrDefault() + report.PTR345.GetValueOrDefault() + report.PTR346.GetValueOrDefault() + report.PTR347.GetValueOrDefault() + report.PTR54.GetValueOrDefault() + report.PTR348.GetValueOrDefault() + report.PTR349.GetValueOrDefault() + report.PTR58.GetValueOrDefault()) + report.PTR59.GetValueOrDefault() + report.PTR328.GetValueOrDefault() != report.PTR61.GetValueOrDefault() + report.PTR62.GetValueOrDefault());
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.430050",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Total Net income or loss must equal total Income, plus Reconciliation items, minus total Expenses",
                    LongDescription = @"The total of Primary production and Non-primary production Net business income must equal the total of all income amounts, plus Reconciliation items, minus the sum of all expenses amounts",
                    Location = "/xbrli:xbrl/bafpr1.02.23:Income.Net.Amount[@contextRef='" + report.RPJAUSPrimId + "']",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.PTR.430050" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "PTR35", Value = report.PTR35.GetValueOrDefault().ToString() });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "PTR36", Value = report.PTR36.GetValueOrDefault().ToString() });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "PTR38", Value = report.PTR38.GetValueOrDefault().ToString() });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "PTR39", Value = report.PTR39.GetValueOrDefault().ToString() });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "PTR41", Value = report.PTR41.GetValueOrDefault().ToString() });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "PTR43", Value = report.PTR43.GetValueOrDefault().ToString() });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "PTR44", Value = report.PTR44.GetValueOrDefault().ToString() });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "PTR45", Value = report.PTR45.GetValueOrDefault().ToString() });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "PTR46", Value = report.PTR46.GetValueOrDefault().ToString() });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "PTR47", Value = report.PTR47.GetValueOrDefault().ToString() });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "PTR343", Value = report.PTR343.GetValueOrDefault().ToString() });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "PTR344", Value = report.PTR344.GetValueOrDefault().ToString() });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "PTR50", Value = report.PTR50.GetValueOrDefault().ToString() });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "PTR345", Value = report.PTR345.GetValueOrDefault().ToString() });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "PTR346", Value = report.PTR346.GetValueOrDefault().ToString() });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "PTR347", Value = report.PTR347.GetValueOrDefault().ToString() });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "PTR54", Value = report.PTR54.GetValueOrDefault().ToString() });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "PTR348", Value = report.PTR348.GetValueOrDefault().ToString() });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "PTR349", Value = report.PTR349.GetValueOrDefault().ToString() });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "PTR58", Value = report.PTR58.GetValueOrDefault().ToString() });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "PTR59", Value = report.PTR59.GetValueOrDefault().ToString() });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "PTR328", Value = report.PTR328.GetValueOrDefault().ToString() });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "PTR61", Value = report.PTR61.GetValueOrDefault().ToString() });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "PTR62", Value = report.PTR62.GetValueOrDefault().ToString() });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.PTR.430050

            #region VR.ATO.PTR.430051

            /*  VR.ATO.PTR.430051
            Net income or loss from business must be present
    
            Legacy Rule Format:
            IF ([PTR63] <> NULL OR [PTR350] <> NULL) AND ([PTR61] = NULL AND [PTR62] = NULL)
                RETURN VALIDATION MESSAGE
            ENDIF
            
            [PTR61] = PTR:RP.JAUS.Prim:bafpr1.02.23:Income.Net.Amount
            [PTR62] = PTR:RP.JAUS.NonPrim:bafpr1.02.23:Income.Net.Amount
            [PTR63] = PTR:RP.JAUS:rvctc2.02.00:IncomeTax.PayAsYouGoWithholding.CreditForTaxWithheldWhereABNNotQuoted.Amount
            [PTR350] = PTR:RP.JAUS:rvctc2.02.13:IncomeTax.PayAsYouGoWithholding.CreditForAmountsWithheldFromForeignResidents.Amount

            Technical Business Rule Format:
            (^PTR63 <> NULL OR ^PTR350 <> NULL) AND (^PTR61 = NULL AND ^PTR62 = NULL)
    
            Data Elements:
    
            RP.JAUS.Prim:^PTR61 = bafpr1.02.23:Income.Net.Amount
    
            RP.JAUS.NonPrim:^PTR62 = bafpr1.02.23:Income.Net.Amount
    
            RP.JAUS:^PTR63 = rvctc2.02.00:IncomeTax.PayAsYouGoWithholding.CreditForTaxWithheldWhereABNNotQuoted.Amount
    
            RP.JAUS:^PTR350 = rvctc2.02.13:IncomeTax.PayAsYouGoWithholding.CreditForAmountsWithheldFromForeignResidents.Amount
            */
            assertion = ((report.PTR63 != null || report.PTR350 != null) && (report.PTR61 == null && report.PTR62 == null));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.430051",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Net income or loss from business must be present",
                    LongDescription = @"If Tax withheld where ABN not quoted is present or Credit for tax withheld - Foreign resident withholding is present, then Net income or loss from business must be present",
                    Location = "/xbrli:xbrl/bafpr1.02.23:Income.Net.Amount[@contextRef='" + report.RPJAUSPrimId + "']",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.PTR.430051" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "PTR63", Value = report.PTR63.GetValueOrDefault().ToString() });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "PTR350", Value = report.PTR350.GetValueOrDefault().ToString() });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "PTR61", Value = report.PTR61.GetValueOrDefault().ToString() });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "PTR62", Value = report.PTR62.GetValueOrDefault().ToString() });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.PTR.430051

            #region VR.ATO.PTR.430052

            /*  VR.ATO.PTR.430052
            PP Net income/loss from business or distribution from partnerships or trusts must be present
    
            Legacy Rule Format:
            IF (ANY OCCURRENCE OF([PTR158]) > 0) AND ([PTR61] = NULL AND [PTR65] = NULL AND [PTR66] = NULL) 
                RETURN VALIDATION MESSAGE
            ENDIF
            
            [PTR61] = PTR:RP.JAUS.Prim:bafpr1.02.23:Income.Net.Amount
            [PTR65] = PTR:RP.JAUS.Prim:bafpr1.02.02:Income.PartnershipDistributionGross.Amount
            [PTR66] = PTR:RP.JAUS.Prim:bafpr1.02.14:Income.TrustShareNet.Amount
            [PTR158] = PTR:RP.Prim:bafpr1.02.04:Income.BeneficiaryShare.Amount IN TUPLE(ptr.0006.lodge.req.02.00:StatementOfDistribution)

            Technical Business Rule Format:
            (AnyOccurrence(^PTR158, ^PTR158 > 0)) AND (^PTR61 = NULL AND ^PTR65 = NULL AND ^PTR66 = NULL)
    
            Data Elements:
    
            RP.JAUS.Prim:^PTR61 = bafpr1.02.23:Income.Net.Amount
    
            RP.JAUS.Prim:^PTR65 = bafpr1.02.02:Income.PartnershipDistributionGross.Amount
    
            RP.JAUS.Prim:^PTR66 = bafpr1.02.14:Income.TrustShareNet.Amount
    
            RP.Prim:^PTR158 = bafpr1.02.04:Income.BeneficiaryShare.Amount
            */
            assertion = ((report.StatementOfDistributionCollection == null ? false : report.StatementOfDistributionCollection.Any(PTR158Repeat => PTR158Repeat.PTR158.GetValueOrDefault() > 0)) && (report.PTR61 == null && report.PTR65 == null && report.PTR66 == null));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.PTR.430052",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"PP Net income or loss from business or a distribution from partnerships or trusts must be present",
                    LongDescription = @"If an amount, greater than zero, is present for a Partner in the distribution statement at Share of income  - Primary production, there must also be an amount greater than zero at Primary production Net income or loss from business, or Primary production - Distribution from partnerships, or Primary production - Distribution from trusts",
                    Location = "/xbrli:xbrl/bafpr1.02.23:Income.Net.Amount",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.PTR.430052" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "PTR158", Value = "PTR158" });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "PTR61", Value = report.PTR61.GetValueOrDefault().ToString() });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "PTR65", Value = report.PTR65.GetValueOrDefault().ToString() });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "PTR66", Value = report.PTR66.GetValueOrDefault().ToString() });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.PTR.430052

            #region VR.ATO.PTR.430054

            /*  VR.ATO.PTR.430054
            Monetary field may be signed, maximum of 11 chars, no decimals
    
            Legacy Rule Format:
            IF [PTR61] <> NULL AND [PTR61] <> MONETARY(S,11,0)
               RETURN VALIDATION MESSAGE
            ENDIF
            
            [PTR61] = PTR:RP.JAUS.Prim:bafpr1.02.23:Income.Net.Amount

            Technical Business Rule Format:
            NotMonetary(^PTR61, 'S', 11, 0)
    
            Data Elements:
    
            RP.JAUS.Prim:^PTR61 = bafpr1.02.23:Income.Net.Amount
            */
            assertion = NotMonetary(report.PTR61, @"S", 11, 0);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.400011",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Field incorrect format. Exceeds maximum digits or decimals and/or contains illegal characters",
                    LongDescription = @"Field is in incorrect format for a monetary field. Field either exceeds maximum digits or decimals and/or contains illegal characters and/or for a signed field does not have the sign as the first character",
                    Location = "/xbrli:xbrl/bafpr1.02.23:Income.Net.Amount[@contextRef='" + report.RPJAUSPrimId + "']",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.PTR.430054" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "PTR61", Value = report.PTR61.GetValueOrDefault().ToString() });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.PTR.430054

            #region VR.ATO.PTR.430055

            /*  VR.ATO.PTR.430055
            Monetary field may be signed, maximum of 11 chars, no decimals
    
            Legacy Rule Format:
            IF [PTR62] <> NULL AND [PTR62] <> MONETARY(S,11,0)
               RETURN VALIDATION MESSAGE
            ENDIF
            
            [PTR62] = PTR:RP.JAUS.NonPrim:bafpr1.02.23:Income.Net.Amount

            Technical Business Rule Format:
            NotMonetary(^PTR62, 'S', 11, 0)
    
            Data Elements:
    
            RP.JAUS.NonPrim:^PTR62 = bafpr1.02.23:Income.Net.Amount
            */
            assertion = NotMonetary(report.PTR62, @"S", 11, 0);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.400011",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Field incorrect format. Exceeds maximum digits or decimals and/or contains illegal characters",
                    LongDescription = @"Field is in incorrect format for a monetary field. Field either exceeds maximum digits or decimals and/or contains illegal characters and/or for a signed field does not have the sign as the first character",
                    Location = "/xbrli:xbrl/bafpr1.02.23:Income.Net.Amount[@contextRef='" + report.RPJAUSNonPrimId + "']",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.PTR.430055" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "PTR62", Value = report.PTR62.GetValueOrDefault().ToString() });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.PTR.430055

            #region VR.ATO.PTR.430056

            /*  VR.ATO.PTR.430056
            Tax withheld where ABN not quoted cannot be greater than 50% of Gross payments (PP and Non-PP) where ABN not quoted.
    
            Legacy Rule Format:
            IF [PTR63] > 0 AND [PTR63] > (([PTR35] + [PTR36]) *0.50) + 1 
                RETURN VALIDATION MESSAGE
            ENDIF
            
            [PTR35] = PTR:RP.JAUS.Prim:lrla.02.00:Remuneration.ABNNotQuotedPaymentGross.Amount
            [PTR36] = PTR:RP.JAUS.NonPrim:lrla.02.00:Remuneration.ABNNotQuotedPaymentGross.Amount
            [PTR63] = PTR:RP.JAUS:rvctc2.02.00:IncomeTax.PayAsYouGoWithholding.CreditForTaxWithheldWhereABNNotQuoted.Amount

            Technical Business Rule Format:
            ^PTR63 > 0 AND ^PTR63 > ((^PTR35 + ^PTR36) *0.50) + 1
    
            Data Elements:
    
            RP.JAUS:^PTR63 = rvctc2.02.00:IncomeTax.PayAsYouGoWithholding.CreditForTaxWithheldWhereABNNotQuoted.Amount
    
            RP.JAUS.Prim:^PTR35 = lrla.02.00:Remuneration.ABNNotQuotedPaymentGross.Amount
    
            RP.JAUS.NonPrim:^PTR36 = lrla.02.00:Remuneration.ABNNotQuotedPaymentGross.Amount
            */
            assertion = (report.PTR63.GetValueOrDefault() > 0 && report.PTR63.GetValueOrDefault() > (report.PTR35.GetValueOrDefault() + report.PTR36.GetValueOrDefault()) * 0.50M + 1);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.430056",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Tax withheld where ABN not quoted cannot be greater than 50% of Gross payments (PP and Non-PP) where ABN not quoted.",
                    LongDescription = @"Tax withheld where ABN not quoted cannot be greater than 50% of Gross payments (Primary production and Non-primary production) where ABN not quoted.",
                    Location = "/xbrli:xbrl/rvctc2.02.00:IncomeTax.PayAsYouGoWithholding.CreditForTaxWithheldWhereABNNotQuoted.Amount[@contextRef='" + report.RPJAUSId + "']",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.PTR.430056" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "PTR63", Value = report.PTR63.GetValueOrDefault().ToString() });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "PTR35", Value = report.PTR35.GetValueOrDefault().ToString() });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "PTR36", Value = report.PTR36.GetValueOrDefault().ToString() });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.PTR.430056

            #region VR.ATO.PTR.430057

            /*  VR.ATO.PTR.430057
            PP or Non-PP Gross payments where ABN not quoted must be present
    
            Legacy Rule Format:
            IF [PTR63] <> NULL AND [PTR35] = NULL AND [PTR36] = NULL 
                RETURN VALIDATION MESSAGE
            ENDIF
            
            [PTR35] = PTR:RP.JAUS.Prim:lrla.02.00:Remuneration.ABNNotQuotedPaymentGross.Amount
            [PTR36] = PTR:RP.JAUS.NonPrim:lrla.02.00:Remuneration.ABNNotQuotedPaymentGross.Amount
            [PTR63] = PTR:RP.JAUS:rvctc2.02.00:IncomeTax.PayAsYouGoWithholding.CreditForTaxWithheldWhereABNNotQuoted.Amount

            Technical Business Rule Format:
            ^PTR63 <> NULL AND ^PTR35 = NULL AND ^PTR36 = NULL
    
            Data Elements:
    
            RP.JAUS:^PTR63 = rvctc2.02.00:IncomeTax.PayAsYouGoWithholding.CreditForTaxWithheldWhereABNNotQuoted.Amount
    
            RP.JAUS.Prim:^PTR35 = lrla.02.00:Remuneration.ABNNotQuotedPaymentGross.Amount
    
            RP.JAUS.NonPrim:^PTR36 = lrla.02.00:Remuneration.ABNNotQuotedPaymentGross.Amount
            */
            assertion = (report.PTR63 != null && report.PTR35 == null && report.PTR36 == null);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.430057",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"PP or Non-PP Gross payments where ABN not quoted must be present",
                    LongDescription = @"If Tax withheld where ABN not quoted is present there must be an amount at Gross payments where ABN not quoted for either Primary production or Non-primary production",
                    Location = "/xbrli:xbrl/rvctc2.02.00:IncomeTax.PayAsYouGoWithholding.CreditForTaxWithheldWhereABNNotQuoted.Amount[@contextRef='" + report.RPJAUSId + "']",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.PTR.430057" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "PTR63", Value = report.PTR63.GetValueOrDefault().ToString() });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "PTR35", Value = report.PTR35.GetValueOrDefault().ToString() });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "PTR36", Value = report.PTR36.GetValueOrDefault().ToString() });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.PTR.430057

            #region VR.ATO.PTR.430058

            /*  VR.ATO.PTR.430058
            Tax withheld where ABN not quoted must be present
    
            Legacy Rule Format:
            IF ([PTR35] > 0 OR [PTR36] > 0) AND [PTR63] = NULL 
                RETURN VALIDATION MESSAGE
            ENDIF
            
            [PTR35] = PTR:RP.JAUS.Prim:lrla.02.00:Remuneration.ABNNotQuotedPaymentGross.Amount
            [PTR36] = PTR:RP.JAUS.NonPrim:lrla.02.00:Remuneration.ABNNotQuotedPaymentGross.Amount
            [PTR63] = PTR:RP.JAUS:rvctc2.02.00:IncomeTax.PayAsYouGoWithholding.CreditForTaxWithheldWhereABNNotQuoted.Amount

            Technical Business Rule Format:
            (^PTR35 > 0 OR ^PTR36 > 0) AND ^PTR63 = NULL
    
            Data Elements:
    
            RP.JAUS:^PTR63 = rvctc2.02.00:IncomeTax.PayAsYouGoWithholding.CreditForTaxWithheldWhereABNNotQuoted.Amount
    
            RP.JAUS.Prim:^PTR35 = lrla.02.00:Remuneration.ABNNotQuotedPaymentGross.Amount
    
            RP.JAUS.NonPrim:^PTR36 = lrla.02.00:Remuneration.ABNNotQuotedPaymentGross.Amount
            */
            assertion = ((report.PTR35.GetValueOrDefault() > 0 || report.PTR36.GetValueOrDefault() > 0) && report.PTR63 == null);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.430058",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Tax withheld where ABN not quoted must be present",
                    LongDescription = @"If either Primary production or Non-primary production Gross payments where ABN not quoted are greater than zero, then an amount at Tax withheld where ABN not quoted must be present",
                    Location = "/xbrli:xbrl/rvctc2.02.00:IncomeTax.PayAsYouGoWithholding.CreditForTaxWithheldWhereABNNotQuoted.Amount[@contextRef='" + report.RPJAUSId + "']",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.PTR.430058" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "PTR35", Value = report.PTR35.GetValueOrDefault().ToString() });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "PTR36", Value = report.PTR36.GetValueOrDefault().ToString() });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "PTR63", Value = report.PTR63.GetValueOrDefault().ToString() });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.PTR.430058

            #region VR.ATO.PTR.430059

            /*  VR.ATO.PTR.430059
            Credit for tax withheld - foreign resident withholding (excluding capital gains) must be present
    
            Legacy Rule Format:
            IF ([PTR38] > 0) AND [PTR350] = NULL
                RETURN VALIDATION MESSAGE
            ENDIF
            
            [PTR38] = PTR:RP.JAUS.NonPrim:lrla.02.00:Remuneration.PaymentToForeignResidentGross.Amount
            [PTR350] = PTR:RP.JAUS:rvctc2.02.13:IncomeTax.PayAsYouGoWithholding.CreditForAmountsWithheldFromForeignResidents.Amount

            Technical Business Rule Format:
            (^PTR38 > 0) AND ^PTR350 = NULL
    
            Data Elements:
    
            RP.JAUS:^PTR350 = rvctc2.02.13:IncomeTax.PayAsYouGoWithholding.CreditForAmountsWithheldFromForeignResidents.Amount
    
            RP.JAUS.NonPrim:^PTR38 = lrla.02.00:Remuneration.PaymentToForeignResidentGross.Amount
            */
            assertion = (report.PTR38.GetValueOrDefault() > 0 && report.PTR350 == null);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.PTR.430059",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Credit for tax withheld - foreign resident withholding (excluding capital gains) must be present",
                    LongDescription = @"If Primary production or Non-primary production Gross payments subject to foreign resident withholding is greater than zero, then there must be an amount at Credit for tax withheld - foreign resident withholding (excluding capital gains)",
                    Location = "/xbrli:xbrl/rvctc2.02.13:IncomeTax.PayAsYouGoWithholding.CreditForAmountsWithheldFromForeignResidents.Amount[@contextRef='" + report.RPJAUSId + "']",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.PTR.430059" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "PTR38", Value = report.PTR38.GetValueOrDefault().ToString() });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "PTR350", Value = report.PTR350.GetValueOrDefault().ToString() });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.PTR.430059

            #region VR.ATO.PTR.430061

            /*  VR.ATO.PTR.430061
            PP - Income distribution from partnerships and/or trusts must be present
    
            Legacy Rule Format:
            IF [PTR68] > 0 AND ([PTR65] = NULL AND [PTR66] = NULL) 
                RETURN VALIDATION MESSAGE
            ENDIF
            
            [PTR65] = PTR:RP.JAUS.Prim:bafpr1.02.02:Income.PartnershipDistributionGross.Amount
            [PTR66] = PTR:RP.JAUS.Prim:bafpr1.02.14:Income.TrustShareNet.Amount
            [PTR68] = PTR:RP.JAUS.Prim:bafpr1.02.04:Expense.DistributionDeductionsOther.Amount

            Technical Business Rule Format:
            ^PTR68 > 0 AND (^PTR65 = NULL AND ^PTR66 = NULL)
    
            Data Elements:
    
            RP.JAUS.Prim:^PTR65 = bafpr1.02.02:Income.PartnershipDistributionGross.Amount
    
            RP.JAUS.Prim:^PTR66 = bafpr1.02.14:Income.TrustShareNet.Amount
    
            RP.JAUS.Prim:^PTR68 = bafpr1.02.04:Expense.DistributionDeductionsOther.Amount
            */
            assertion = (report.PTR68.GetValueOrDefault() > 0 && (report.PTR65 == null && report.PTR66 == null));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.430061",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"PP - Income distribution from partnerships and/or trusts must be present",
                    LongDescription = @"If there is an amount present at 'Primary production - Deductions relating to distribution', then there must also be an amount at 'Primary production - Distribution from partnerships' or 'Primary production - Distribution from trusts'",
                    Location = "/xbrli:xbrl/bafpr1.02.02:Income.PartnershipDistributionGross.Amount",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.PTR.430061" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "PTR68", Value = report.PTR68.GetValueOrDefault().ToString() });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "PTR65", Value = report.PTR65.GetValueOrDefault().ToString() });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "PTR66", Value = report.PTR66.GetValueOrDefault().ToString() });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.PTR.430061

            #region VR.ATO.PTR.430063

            /*  VR.ATO.PTR.430063
            Distribution from partnerships or trusts must be present
    
            Legacy Rule Format:
            IF ([PTR72] > 0 OR [PTR323] > 0 OR [PTR74] > 0 OR [PTR75] > 0 OR [PTR76] > 0) AND ([PTR65] = NULL AND [PTR66] = NULL AND [PTR69] = NULL AND [PTR70] = NULL AND [PTR322] = NULL) 
                RETURN VALIDATION MESSAGE
            ENDIF
            
            [PTR65] = PTR:RP.JAUS.Prim:bafpr1.02.02:Income.PartnershipDistributionGross.Amount
            [PTR66] = PTR:RP.JAUS.Prim:bafpr1.02.14:Income.TrustShareNet.Amount
            [PTR69] = PTR:RP.JAUS.NonPrim:bafpr1.02.04:Income.PartnershipDistributionLessForeignIncome.Amount
            [PTR70] = PTR:RP.JAUS.NonPrim:bafpr1.02.13:Income.TrustShareNetExcludeNetCapitalGainsAndForeignIncomeAndDistributionFranked.Amount
            [PTR72] = PTR:RP.JAUS.NonPrim:bafpr1.02.04:Expense.DistributionDeductionsOther.Amount
            [PTR74] = PTR:RP.JAUS:rvctc2.02.00:IncomeTax.FrankingCredits.FrankingCreditShareReceivedIndirectly.Amount
            [PTR75] = PTR:RP.JAUS:rvctc2.02.00:IncomeTax.PayAsYouGoWithholding.CreditForAmountsWithheldFromInterestAndDividendsWhereTFNNotQuotedShareReceivedIndirectly.Amount
            [PTR76] = PTR:RP.JAUS:rvctc4.02.00:InternationalDealings.CreditForTaxWithheldFromForeignResidentWithholdingShareReceivedIndirectly.Amount
            [PTR322] = PTR:RP.JAUS.NonPrim:bafpr1.02.04:Income.TrustDistributionFranked.Amount
            [PTR323] = PTR:RP.JAUS.NonPrim:rvctc2.02.05:IncomeTax.Deduction.TrustDistributionFranked.Amount

            Technical Business Rule Format:
            (^PTR72 > 0 OR ^PTR323 > 0 OR ^PTR74 > 0 OR ^PTR75 > 0 OR ^PTR76 > 0) AND (^PTR65 = NULL AND ^PTR66 = NULL AND ^PTR69 = NULL AND ^PTR70 = NULL AND ^PTR322 = NULL)
    
            Data Elements:
    
            RP.JAUS.Prim:^PTR65 = bafpr1.02.02:Income.PartnershipDistributionGross.Amount
    
            RP.JAUS.Prim:^PTR66 = bafpr1.02.14:Income.TrustShareNet.Amount
    
            RP.JAUS.NonPrim:^PTR69 = bafpr1.02.04:Income.PartnershipDistributionLessForeignIncome.Amount
    
            RP.JAUS.NonPrim:^PTR70 = bafpr1.02.13:Income.TrustShareNetExcludeNetCapitalGainsAndForeignIncomeAndDistributionFranked.Amount
    
            RP.JAUS.NonPrim:^PTR72 = bafpr1.02.04:Expense.DistributionDeductionsOther.Amount
    
            RP.JAUS:^PTR74 = rvctc2.02.00:IncomeTax.FrankingCredits.FrankingCreditShareReceivedIndirectly.Amount
    
            RP.JAUS:^PTR75 = rvctc2.02.00:IncomeTax.PayAsYouGoWithholding.CreditForAmountsWithheldFromInterestAndDividendsWhereTFNNotQuotedShareReceivedIndirectly.Amount
    
            RP.JAUS:^PTR76 = rvctc4.02.00:InternationalDealings.CreditForTaxWithheldFromForeignResidentWithholdingShareReceivedIndirectly.Amount
    
            RP.JAUS.NonPrim:^PTR322 = bafpr1.02.04:Income.TrustDistributionFranked.Amount
    
            RP.JAUS.NonPrim:^PTR323 = rvctc2.02.05:IncomeTax.Deduction.TrustDistributionFranked.Amount
            */
            assertion = ((report.PTR72.GetValueOrDefault() > 0 || report.PTR323.GetValueOrDefault() > 0 || report.PTR74.GetValueOrDefault() > 0 || report.PTR75.GetValueOrDefault() > 0 || report.PTR76.GetValueOrDefault() > 0) && (report.PTR65 == null && report.PTR66 == null && report.PTR69 == null && report.PTR70 == null && report.PTR322 == null));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.430063",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Distribution from partnerships or trusts must be present",
                    LongDescription = @"If an amount is present at 'Deductions relating to distribution', 'Share of credit for tax withheld where ABN not quoted', 'Share of franking credit from franked dividends', 'Share of credit for TFN amounts withheld from interest dividends and unit trust distributions', or 'Share of credit for tax withheld from foreign resident withholding'; then there must be an amount present at 'Primary production - Distribution from partnerships', 'Primary production - Distribution from trusts', 'Non-primary production - Distribution from partnerships less foreign income', or 'Distribution from trusts less net capital gain and foreign income'",
                    Location = "/xbrli:xbrl/bafpr1.02.02:Income.PartnershipDistributionGross.Amount",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.PTR.430063" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "PTR72", Value = report.PTR72.GetValueOrDefault().ToString() });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "PTR323", Value = report.PTR323.GetValueOrDefault().ToString() });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "PTR74", Value = report.PTR74.GetValueOrDefault().ToString() });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "PTR75", Value = report.PTR75.GetValueOrDefault().ToString() });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "PTR76", Value = report.PTR76.GetValueOrDefault().ToString() });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "PTR65", Value = report.PTR65.GetValueOrDefault().ToString() });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "PTR66", Value = report.PTR66.GetValueOrDefault().ToString() });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "PTR69", Value = report.PTR69.GetValueOrDefault().ToString() });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "PTR70", Value = report.PTR70.GetValueOrDefault().ToString() });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "PTR322", Value = report.PTR322.GetValueOrDefault().ToString() });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.PTR.430063

            #region VR.ATO.PTR.430064

            /*  VR.ATO.PTR.430064
            Monetary field may be signed, maximum of 11 chars, no decimals
    
            Legacy Rule Format:
            IF [PTR65] <> NULL AND [PTR65] <> MONETARY(S,11,0)
               RETURN VALIDATION MESSAGE
            ENDIF
            
            [PTR65] = PTR:RP.JAUS.Prim:bafpr1.02.02:Income.PartnershipDistributionGross.Amount

            Technical Business Rule Format:
            NotMonetary(^PTR65, 'S', 11, 0)
    
            Data Elements:
    
            RP.JAUS.Prim:^PTR65 = bafpr1.02.02:Income.PartnershipDistributionGross.Amount
            */
            assertion = NotMonetary(report.PTR65, @"S", 11, 0);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.400011",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Field incorrect format. Exceeds maximum digits or decimals and/or contains illegal characters",
                    LongDescription = @"Field is in incorrect format for a monetary field. Field either exceeds maximum digits or decimals and/or contains illegal characters and/or for a signed field does not have the sign as the first character",
                    Location = "/xbrli:xbrl/bafpr1.02.02:Income.PartnershipDistributionGross.Amount",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.PTR.430064" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "PTR65", Value = report.PTR65.GetValueOrDefault().ToString() });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.PTR.430064

            #region VR.ATO.PTR.430067

            /*  VR.ATO.PTR.430067
            Monetary field must be unsigned, maximum of 11 numeric characters, no decimals
    
            Legacy Rule Format:
            IF [PTR68] <> NULL AND [PTR68] <> MONETARY(U,11,0)
               RETURN VALIDATION MESSAGE
            ENDIF
            
            [PTR68] = PTR:RP.JAUS.Prim:bafpr1.02.04:Expense.DistributionDeductionsOther.Amount

            Technical Business Rule Format:
            NotMonetary(^PTR68, 'U', 11, 0)
    
            Data Elements:
    
            RP.JAUS.Prim:^PTR68 = bafpr1.02.04:Expense.DistributionDeductionsOther.Amount
            */
            assertion = NotMonetary(report.PTR68, @"U", 11, 0);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.400011",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Field incorrect format. Exceeds maximum digits or decimals and/or contains illegal characters",
                    LongDescription = @"Field is in incorrect format for a monetary field. Field either exceeds maximum digits or decimals and/or contains illegal characters and/or for a signed field does not have the sign as the first character",
                    Location = "/xbrli:xbrl/bafpr1.02.04:Expense.DistributionDeductionsOther.Amount[@contextRef='" + report.RPJAUSPrimId + "']",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.PTR.430067" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "PTR68", Value = report.PTR68.GetValueOrDefault().ToString() });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.PTR.430067

            #region VR.ATO.PTR.430068

            /*  VR.ATO.PTR.430068
            Non-PP distribution amount must be present
    
            Legacy Rule Format:
            IF ([PTR74] <> NULL AND [PTR75] <> NULL) AND ([PTR69] = NULL AND [PTR70] = NULL AND [PTR72] = NULL AND [PTR322] = NULL AND [PTR323] = NULL) 
                RETURN VALIDATION MESSAGE
            ENDIF
            
            [PTR69] = PTR:RP.JAUS.NonPrim:bafpr1.02.04:Income.PartnershipDistributionLessForeignIncome.Amount
            [PTR70] = PTR:RP.JAUS.NonPrim:bafpr1.02.13:Income.TrustShareNetExcludeNetCapitalGainsAndForeignIncomeAndDistributionFranked.Amount
            [PTR72] = PTR:RP.JAUS.NonPrim:bafpr1.02.04:Expense.DistributionDeductionsOther.Amount
            [PTR74] = PTR:RP.JAUS:rvctc2.02.00:IncomeTax.FrankingCredits.FrankingCreditShareReceivedIndirectly.Amount
            [PTR75] = PTR:RP.JAUS:rvctc2.02.00:IncomeTax.PayAsYouGoWithholding.CreditForAmountsWithheldFromInterestAndDividendsWhereTFNNotQuotedShareReceivedIndirectly.Amount
            [PTR322] = PTR:RP.JAUS.NonPrim:bafpr1.02.04:Income.TrustDistributionFranked.Amount
            [PTR323] = PTR:RP.JAUS.NonPrim:rvctc2.02.05:IncomeTax.Deduction.TrustDistributionFranked.Amount

            Technical Business Rule Format:
            (^PTR74 <> NULL AND ^PTR75 <> NULL) AND (^PTR69 = NULL AND ^PTR70 = NULL AND ^PTR72 = NULL AND ^PTR322 = NULL AND ^PTR323 = NULL)
    
            Data Elements:
    
            RP.JAUS.NonPrim:^PTR69 = bafpr1.02.04:Income.PartnershipDistributionLessForeignIncome.Amount
    
            RP.JAUS.NonPrim:^PTR70 = bafpr1.02.13:Income.TrustShareNetExcludeNetCapitalGainsAndForeignIncomeAndDistributionFranked.Amount
    
            RP.JAUS.NonPrim:^PTR72 = bafpr1.02.04:Expense.DistributionDeductionsOther.Amount
    
            RP.JAUS:^PTR74 = rvctc2.02.00:IncomeTax.FrankingCredits.FrankingCreditShareReceivedIndirectly.Amount
    
            RP.JAUS:^PTR75 = rvctc2.02.00:IncomeTax.PayAsYouGoWithholding.CreditForAmountsWithheldFromInterestAndDividendsWhereTFNNotQuotedShareReceivedIndirectly.Amount
    
            RP.JAUS.NonPrim:^PTR322 = bafpr1.02.04:Income.TrustDistributionFranked.Amount
    
            RP.JAUS.NonPrim:^PTR323 = rvctc2.02.05:IncomeTax.Deduction.TrustDistributionFranked.Amount
            */
            assertion = (report.PTR74 != null && report.PTR75 != null && (report.PTR69 == null && report.PTR70 == null && report.PTR72 == null && report.PTR322 == null && report.PTR323 == null));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.430068",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Non-PP distribution amount must be present",
                    LongDescription = @"If there is an amount at Share of franking credits from franked dividends, and Share of credit for TFN amounts withheld from interest, dividends and unit trust distributions; then there must be an amount at Non-Primary Production Distribution from partnerships less foreign income; Distribution from trusts less net capital gain and foreign income; or Deductions relating to distribution",
                    Location = "/xbrli:xbrl/bafpr1.02.04:Income.PartnershipDistributionLessForeignIncome.Amount",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.PTR.430068" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "PTR74", Value = report.PTR74.GetValueOrDefault().ToString() });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "PTR75", Value = report.PTR75.GetValueOrDefault().ToString() });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "PTR69", Value = report.PTR69.GetValueOrDefault().ToString() });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "PTR70", Value = report.PTR70.GetValueOrDefault().ToString() });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "PTR72", Value = report.PTR72.GetValueOrDefault().ToString() });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "PTR322", Value = report.PTR322.GetValueOrDefault().ToString() });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "PTR323", Value = report.PTR323.GetValueOrDefault().ToString() });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.PTR.430068

            #region VR.ATO.PTR.430069

            /*  VR.ATO.PTR.430069
            Monetary field may be signed, maximum of 11 chars, no decimals
    
            Legacy Rule Format:
            IF [PTR69] <> NULL AND [PTR69] <> MONETARY(S,11,0)
               RETURN VALIDATION MESSAGE
            ENDIF
            
            [PTR69] = PTR:RP.JAUS.NonPrim:bafpr1.02.04:Income.PartnershipDistributionLessForeignIncome.Amount

            Technical Business Rule Format:
            NotMonetary(^PTR69, 'S', 11, 0)
    
            Data Elements:
    
            RP.JAUS.NonPrim:^PTR69 = bafpr1.02.04:Income.PartnershipDistributionLessForeignIncome.Amount
            */
            assertion = NotMonetary(report.PTR69, @"S", 11, 0);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.400011",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Field incorrect format. Exceeds maximum digits or decimals and/or contains illegal characters",
                    LongDescription = @"Field is in incorrect format for a monetary field. Field either exceeds maximum digits or decimals and/or contains illegal characters and/or for a signed field does not have the sign as the first character",
                    Location = "/xbrli:xbrl/bafpr1.02.04:Income.PartnershipDistributionLessForeignIncome.Amount",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.PTR.430069" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "PTR69", Value = report.PTR69.GetValueOrDefault().ToString() });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.PTR.430069

            #region VR.ATO.PTR.430074

            /*  VR.ATO.PTR.430074
            Monetary field must be unsigned, maximum of 11 numeric characters, no decimals
    
            Legacy Rule Format:
            IF [PTR72] <> NULL AND [PTR72] <> MONETARY(U,11,0)
               RETURN VALIDATION MESSAGE
            ENDIF
            
            [PTR72] = PTR:RP.JAUS.NonPrim:bafpr1.02.04:Expense.DistributionDeductionsOther.Amount

            Technical Business Rule Format:
            NotMonetary(^PTR72, 'U', 11, 0)
    
            Data Elements:
    
            RP.JAUS.NonPrim:^PTR72 = bafpr1.02.04:Expense.DistributionDeductionsOther.Amount
            */
            assertion = NotMonetary(report.PTR72, @"U", 11, 0);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.400011",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Field incorrect format. Exceeds maximum digits or decimals and/or contains illegal characters",
                    LongDescription = @"Field is in incorrect format for a monetary field. Field either exceeds maximum digits or decimals and/or contains illegal characters and/or for a signed field does not have the sign as the first character",
                    Location = "/xbrli:xbrl/bafpr1.02.04:Expense.DistributionDeductionsOther.Amount[@contextRef='" + report.RPJAUSNonPrimId + "']",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.PTR.430074" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "PTR72", Value = report.PTR72.GetValueOrDefault().ToString() });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.PTR.430074

            #region VR.ATO.PTR.430075

            /*  VR.ATO.PTR.430075
            Monetary field must be unsigned, maximum of 11 numeric characters, no decimals
    
            Legacy Rule Format:
            IF [PTR73] <> NULL AND [PTR73] <> MONETARY(U,11,0)
               RETURN VALIDATION MESSAGE
            ENDIF
            
            [PTR73] = PTR:RP.JAUS:rvctc2.02.00:IncomeTax.PayAsYouGoWithholding.CreditForTaxWithheldWhereABNNotQuotedShareReceivedIndirectly.Amount

            Technical Business Rule Format:
            NotMonetary(^PTR73, 'U', 11, 0)
    
            Data Elements:
    
            RP.JAUS:^PTR73 = rvctc2.02.00:IncomeTax.PayAsYouGoWithholding.CreditForTaxWithheldWhereABNNotQuotedShareReceivedIndirectly.Amount
            */
            assertion = NotMonetary(report.PTR73, @"U", 11, 0);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.400011",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Field incorrect format. Exceeds maximum digits or decimals and/or contains illegal characters",
                    LongDescription = @"Field is in incorrect format for a monetary field. Field either exceeds maximum digits or decimals and/or contains illegal characters and/or for a signed field does not have the sign as the first character",
                    Location = "/xbrli:xbrl/rvctc2.02.00:IncomeTax.PayAsYouGoWithholding.CreditForTaxWithheldWhereABNNotQuotedShareReceivedIndirectly.Amount",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.PTR.430075" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "PTR73", Value = report.PTR73.GetValueOrDefault().ToString() });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.PTR.430075

            #region VR.ATO.PTR.430076

            /*  VR.ATO.PTR.430076
            Monetary field - unsigned, maximum of 13 characters, may include a decimal point and up to 2 decimals
    
            Legacy Rule Format:
            IF [PTR74] <> NULL AND [PTR74] <> MONETARY(U,13,2)
               RETURN VALIDATION MESSAGE
            ENDIF
            
            [PTR74] = PTR:RP.JAUS:rvctc2.02.00:IncomeTax.FrankingCredits.FrankingCreditShareReceivedIndirectly.Amount

            Technical Business Rule Format:
            NotMonetary(^PTR74, 'U', 13, 2)
    
            Data Elements:
    
            RP.JAUS:^PTR74 = rvctc2.02.00:IncomeTax.FrankingCredits.FrankingCreditShareReceivedIndirectly.Amount
            */
            assertion = NotMonetary(report.PTR74, @"U", 13, 2);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.400011",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Field incorrect format. Exceeds maximum digits or decimals and/or contains illegal characters",
                    LongDescription = @"Field is in incorrect format for a monetary field. Field either exceeds maximum digits or decimals and/or contains illegal characters and/or for a signed field does not have the sign as the first character",
                    Location = "/xbrli:xbrl/rvctc2.02.00:IncomeTax.FrankingCredits.FrankingCreditShareReceivedIndirectly.Amount",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.PTR.430076" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "PTR74", Value = report.PTR74.GetValueOrDefault().ToString() });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.PTR.430076

            #region VR.ATO.PTR.430077

            /*  VR.ATO.PTR.430077
            Monetary field - unsigned, maximum of 13 characters, may include a decimal point and up to 2 decimals
    
            Legacy Rule Format:
            IF [PTR75] <> NULL AND [PTR75] <> MONETARY(U,13,2)
               RETURN VALIDATION MESSAGE
            ENDIF
            
            [PTR75] = PTR:RP.JAUS:rvctc2.02.00:IncomeTax.PayAsYouGoWithholding.CreditForAmountsWithheldFromInterestAndDividendsWhereTFNNotQuotedShareReceivedIndirectly.Amount

            Technical Business Rule Format:
            NotMonetary(^PTR75, 'U', 13, 2)
    
            Data Elements:
    
            RP.JAUS:^PTR75 = rvctc2.02.00:IncomeTax.PayAsYouGoWithholding.CreditForAmountsWithheldFromInterestAndDividendsWhereTFNNotQuotedShareReceivedIndirectly.Amount
            */
            assertion = NotMonetary(report.PTR75, @"U", 13, 2);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.400011",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Field incorrect format. Exceeds maximum digits or decimals and/or contains illegal characters",
                    LongDescription = @"Field is in incorrect format for a monetary field. Field either exceeds maximum digits or decimals and/or contains illegal characters and/or for a signed field does not have the sign as the first character",
                    Location = "/xbrli:xbrl/rvctc2.02.00:IncomeTax.PayAsYouGoWithholding.CreditForAmountsWithheldFromInterestAndDividendsWhereTFNNotQuotedShareReceivedIndirectly.Amount",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.PTR.430077" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "PTR75", Value = report.PTR75.GetValueOrDefault().ToString() });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.PTR.430077

            #region VR.ATO.PTR.430078

            /*  VR.ATO.PTR.430078
            Monetary field must be unsigned, maximum of 11 numeric characters, no decimals
    
            Legacy Rule Format:
            IF [PTR76] <> NULL AND [PTR76] <> MONETARY(U,11,0)
               RETURN VALIDATION MESSAGE
            ENDIF
            
            [PTR76] = PTR:RP.JAUS:rvctc4.02.00:InternationalDealings.CreditForTaxWithheldFromForeignResidentWithholdingShareReceivedIndirectly.Amount

            Technical Business Rule Format:
            NotMonetary(^PTR76, 'U', 11, 0)
    
            Data Elements:
    
            RP.JAUS:^PTR76 = rvctc4.02.00:InternationalDealings.CreditForTaxWithheldFromForeignResidentWithholdingShareReceivedIndirectly.Amount
            */
            assertion = NotMonetary(report.PTR76, @"U", 11, 0);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.400011",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Field incorrect format. Exceeds maximum digits or decimals and/or contains illegal characters",
                    LongDescription = @"Field is in incorrect format for a monetary field. Field either exceeds maximum digits or decimals and/or contains illegal characters and/or for a signed field does not have the sign as the first character",
                    Location = "/xbrli:xbrl/rvctc4.02.00:InternationalDealings.CreditForTaxWithheldFromForeignResidentWithholdingShareReceivedIndirectly.Amount",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.PTR.430078" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "PTR76", Value = report.PTR76.GetValueOrDefault().ToString() });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.PTR.430078

            #region VR.ATO.PTR.430079

            /*  VR.ATO.PTR.430079
            Gross rent must be present
    
            Legacy Rule Format:
            IF ([PTR78] > 0 OR [PTR79] > 0 OR [PTR80] > 0) AND [PTR77] = NULL 
                RETURN VALIDATION MESSAGE
            ENDIF
            
            [PTR77] = PTR:RP.JAUS:bafpr2.02.00:Income.Operating.RentalIncomeGross.Amount
            [PTR78] = PTR:RP.JAUS:bafpr1.02.05:Expense.Interest.Amount
            [PTR79] = PTR:RP.JAUS:bafpr1.02.02:Expense.CapitalWorksDeduction.Amount
            [PTR80] = PTR:RP.JAUS:rvctc2.02.00:IncomeTax.Deduction.RentalIncomeDeductionsOtherThanInterestAndCapitalWorks.Amount

            Technical Business Rule Format:
            (^PTR78 > 0 OR ^PTR79 > 0 OR ^PTR80 > 0) AND ^PTR77 = NULL
    
            Data Elements:
    
            RP.JAUS:^PTR77 = bafpr2.02.00:Income.Operating.RentalIncomeGross.Amount
    
            RP.JAUS:^PTR78 = bafpr1.02.05:Expense.Interest.Amount
    
            RP.JAUS:^PTR79 = bafpr1.02.02:Expense.CapitalWorksDeduction.Amount
    
            RP.JAUS:^PTR80 = rvctc2.02.00:IncomeTax.Deduction.RentalIncomeDeductionsOtherThanInterestAndCapitalWorks.Amount
            */
            assertion = ((report.PTR78.GetValueOrDefault() > 0 || report.PTR79.GetValueOrDefault() > 0 || report.PTR80.GetValueOrDefault() > 0) && report.PTR77 == null);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.430079",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Gross rent must be present",
                    LongDescription = @"If Interest deductions, Capital works deductions, or Other rental deductions are greater than zero, Gross rent must be present",
                    Location = "/xbrli:xbrl/bafpr2.02.00:Income.Operating.RentalIncomeGross.Amount",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.PTR.430079" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "PTR78", Value = report.PTR78.GetValueOrDefault().ToString() });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "PTR79", Value = report.PTR79.GetValueOrDefault().ToString() });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "PTR80", Value = report.PTR80.GetValueOrDefault().ToString() });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "PTR77", Value = report.PTR77.GetValueOrDefault().ToString() });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.PTR.430079

            #region VR.ATO.PTR.430080

            /*  VR.ATO.PTR.430080
            Rent deductions must be present
    
            Legacy Rule Format:
            IF [PTR77] > 0 AND ([PTR78] = NULL AND [PTR79] = NULL AND [PTR80] = NULL) 
                RETURN VALIDATION MESSAGE
            ENDIF
            
            [PTR77] = PTR:RP.JAUS:bafpr2.02.00:Income.Operating.RentalIncomeGross.Amount
            [PTR78] = PTR:RP.JAUS:bafpr1.02.05:Expense.Interest.Amount
            [PTR79] = PTR:RP.JAUS:bafpr1.02.02:Expense.CapitalWorksDeduction.Amount
            [PTR80] = PTR:RP.JAUS:rvctc2.02.00:IncomeTax.Deduction.RentalIncomeDeductionsOtherThanInterestAndCapitalWorks.Amount

            Technical Business Rule Format:
            ^PTR77 > 0 AND (^PTR78 = NULL AND ^PTR79 = NULL AND ^PTR80 = NULL)
    
            Data Elements:
    
            RP.JAUS:^PTR78 = bafpr1.02.05:Expense.Interest.Amount
    
            RP.JAUS:^PTR77 = bafpr2.02.00:Income.Operating.RentalIncomeGross.Amount
    
            RP.JAUS:^PTR79 = bafpr1.02.02:Expense.CapitalWorksDeduction.Amount
    
            RP.JAUS:^PTR80 = rvctc2.02.00:IncomeTax.Deduction.RentalIncomeDeductionsOtherThanInterestAndCapitalWorks.Amount
            */
            assertion = (report.PTR77.GetValueOrDefault() > 0 && (report.PTR78 == null && report.PTR79 == null && report.PTR80 == null));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.430080",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Rent deductions must be present",
                    LongDescription = @"If Gross rent is greater than zero, then there must be an amount at Interest deductions, Capital works deductions or Other rental deductions. If there are no rental deductions, then zero must be present in at least one of these fields",
                    Location = "/xbrli:xbrl/bafpr1.02.05:Expense.Interest.Amount[@contextRef='" + report.RPJAUSId + "']",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.PTR.430080" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "PTR77", Value = report.PTR77.GetValueOrDefault().ToString() });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "PTR78", Value = report.PTR78.GetValueOrDefault().ToString() });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "PTR79", Value = report.PTR79.GetValueOrDefault().ToString() });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "PTR80", Value = report.PTR80.GetValueOrDefault().ToString() });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.PTR.430080

            #region VR.ATO.PTR.430085

            /*  VR.ATO.PTR.430085
            Monetary field must be unsigned, maximum of 11 numeric characters, no decimals
    
            Legacy Rule Format:
            IF [PTR77] <> NULL AND [PTR77] <> MONETARY(U,11,0)
               RETURN VALIDATION MESSAGE
            ENDIF
            
            [PTR77] = PTR:RP.JAUS:bafpr2.02.00:Income.Operating.RentalIncomeGross.Amount

            Technical Business Rule Format:
            NotMonetary(^PTR77, 'U', 11, 0)
    
            Data Elements:
    
            RP.JAUS:^PTR77 = bafpr2.02.00:Income.Operating.RentalIncomeGross.Amount
            */
            assertion = NotMonetary(report.PTR77, @"U", 11, 0);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.400011",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Field incorrect format. Exceeds maximum digits or decimals and/or contains illegal characters",
                    LongDescription = @"Field is in incorrect format for a monetary field. Field either exceeds maximum digits or decimals and/or contains illegal characters and/or for a signed field does not have the sign as the first character",
                    Location = "/xbrli:xbrl/bafpr2.02.00:Income.Operating.RentalIncomeGross.Amount",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.PTR.430085" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "PTR77", Value = report.PTR77.GetValueOrDefault().ToString() });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.PTR.430085

            #region VR.ATO.PTR.430087

            /*  VR.ATO.PTR.430087
            Monetary field must be unsigned, maximum of 11 numeric characters, no decimals
    
            Legacy Rule Format:
            IF [PTR78] <> NULL AND [PTR78] <> MONETARY(U,11,0)
               RETURN VALIDATION MESSAGE
            ENDIF
            
            [PTR78] = PTR:RP.JAUS:bafpr1.02.05:Expense.Interest.Amount

            Technical Business Rule Format:
            NotMonetary(^PTR78, 'U', 11, 0)
    
            Data Elements:
    
            RP.JAUS:^PTR78 = bafpr1.02.05:Expense.Interest.Amount
            */
            assertion = NotMonetary(report.PTR78, @"U", 11, 0);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.400011",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Field incorrect format. Exceeds maximum digits or decimals and/or contains illegal characters",
                    LongDescription = @"Field is in incorrect format for a monetary field. Field either exceeds maximum digits or decimals and/or contains illegal characters and/or for a signed field does not have the sign as the first character",
                    Location = "/xbrli:xbrl/bafpr1.02.05:Expense.Interest.Amount[@contextRef='" + report.RPJAUSId + "']",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.PTR.430087" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "PTR78", Value = report.PTR78.GetValueOrDefault().ToString() });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.PTR.430087

            #region VR.ATO.PTR.430089

            /*  VR.ATO.PTR.430089
            Monetary field must be unsigned, maximum of 11 numeric characters, no decimals
    
            Legacy Rule Format:
            IF [PTR79] <> NULL AND [PTR79] <> MONETARY(U,11,0)
               RETURN VALIDATION MESSAGE
            ENDIF
            
            [PTR79] = PTR:RP.JAUS:bafpr1.02.02:Expense.CapitalWorksDeduction.Amount

            Technical Business Rule Format:
            NotMonetary(^PTR79, 'U', 11, 0)
    
            Data Elements:
    
            RP.JAUS:^PTR79 = bafpr1.02.02:Expense.CapitalWorksDeduction.Amount
            */
            assertion = NotMonetary(report.PTR79, @"U", 11, 0);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.400011",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Field incorrect format. Exceeds maximum digits or decimals and/or contains illegal characters",
                    LongDescription = @"Field is in incorrect format for a monetary field. Field either exceeds maximum digits or decimals and/or contains illegal characters and/or for a signed field does not have the sign as the first character",
                    Location = "/xbrli:xbrl/bafpr1.02.02:Expense.CapitalWorksDeduction.Amount",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.PTR.430089" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "PTR79", Value = report.PTR79.GetValueOrDefault().ToString() });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.PTR.430089

            #region VR.ATO.PTR.430091

            /*  VR.ATO.PTR.430091
            Monetary field must be unsigned, maximum of 11 numeric characters, no decimals
    
            Legacy Rule Format:
            IF [PTR80] <> NULL AND [PTR80] <> MONETARY(U,11,0)
               RETURN VALIDATION MESSAGE
            ENDIF
            
            [PTR80] = PTR:RP.JAUS:rvctc2.02.00:IncomeTax.Deduction.RentalIncomeDeductionsOtherThanInterestAndCapitalWorks.Amount

            Technical Business Rule Format:
            NotMonetary(^PTR80, 'U', 11, 0)
    
            Data Elements:
    
            RP.JAUS:^PTR80 = rvctc2.02.00:IncomeTax.Deduction.RentalIncomeDeductionsOtherThanInterestAndCapitalWorks.Amount
            */
            assertion = NotMonetary(report.PTR80, @"U", 11, 0);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.400011",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Field incorrect format. Exceeds maximum digits or decimals and/or contains illegal characters",
                    LongDescription = @"Field is in incorrect format for a monetary field. Field either exceeds maximum digits or decimals and/or contains illegal characters and/or for a signed field does not have the sign as the first character",
                    Location = "/xbrli:xbrl/rvctc2.02.00:IncomeTax.Deduction.RentalIncomeDeductionsOtherThanInterestAndCapitalWorks.Amount",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.PTR.430091" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "PTR80", Value = report.PTR80.GetValueOrDefault().ToString() });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.PTR.430091

            #region VR.ATO.PTR.430092

            /*  VR.ATO.PTR.430092
            Gross interest amount greater than zero must be present
    
            Legacy Rule Format:
            IF [PTR83] > 0 AND ([PTR82] = 0 OR [PTR82] = NULL)
                RETURN VALIDATION MESSAGE
            ENDIF
            
            [PTR82] = PTR:RP.JAUS:bafpr2.02.02:Income.Interest.Gross.Amount
            [PTR83] = PTR:RP.JAUS:rvctc1.02.16:IncomeTax.TFNAmountsWithheldFromGrossInterest.Amount

            Technical Business Rule Format:
            ^PTR83 > 0 AND (^PTR82 = 0 OR ^PTR82 = NULL)
    
            Data Elements:
    
            RP.JAUS:^PTR82 = bafpr2.02.02:Income.Interest.Gross.Amount
    
            RP.JAUS:^PTR83 = rvctc1.02.16:IncomeTax.TFNAmountsWithheldFromGrossInterest.Amount
            */
            assertion = (report.PTR83.GetValueOrDefault() > 0 && (report.PTR82 == 0 || report.PTR82 == null));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.430092",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Gross interest amount greater than zero must be present",
                    LongDescription = @"If an amount greater than zero is present at TFN amounts withheld from gross interest, then an amount greater than zero must also be present at Gross interest",
                    Location = "/xbrli:xbrl/bafpr2.02.02:Income.Interest.Gross.Amount",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.PTR.430092" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "PTR83", Value = report.PTR83.GetValueOrDefault().ToString() });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "PTR82", Value = report.PTR82.GetValueOrDefault().ToString() });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.PTR.430092

            #region VR.ATO.PTR.430093

            /*  VR.ATO.PTR.430093
            TFN amounts withheld from dividends must not exceed 50% of Gross interest
    
            Legacy Rule Format:
            IF [PTR83] > 0 AND ([PTR82] * 0.5) <= [PTR83] 
                RETURN VALIDATION MESSAGE
            ENDIF
            
            [PTR82] = PTR:RP.JAUS:bafpr2.02.02:Income.Interest.Gross.Amount
            [PTR83] = PTR:RP.JAUS:rvctc1.02.16:IncomeTax.TFNAmountsWithheldFromGrossInterest.Amount

            Technical Business Rule Format:
            ^PTR83 > 0 AND (^PTR82 * 0.5) <= ^PTR83
    
            Data Elements:
    
            RP.JAUS:^PTR83 = rvctc1.02.16:IncomeTax.TFNAmountsWithheldFromGrossInterest.Amount
    
            RP.JAUS:^PTR82 = bafpr2.02.02:Income.Interest.Gross.Amount
            */
            assertion = (report.PTR83.GetValueOrDefault() > 0 && report.PTR82.GetValueOrDefault() * 0.5M <= report.PTR83.GetValueOrDefault());
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.430093",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"TFN amounts withheld from dividends must not exceed 50% of Gross interest",
                    Location = "/xbrli:xbrl/rvctc1.02.16:IncomeTax.TFNAmountsWithheldFromGrossInterest.Amount",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.PTR.430093" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "PTR83", Value = report.PTR83.GetValueOrDefault().ToString() });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "PTR82", Value = report.PTR82.GetValueOrDefault().ToString() });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.PTR.430093

            #region VR.ATO.PTR.430094

            /*  VR.ATO.PTR.430094
            Monetary field - unsigned, maximum of 13 characters, may include a decimal point and up to 2 decimals
    
            Legacy Rule Format:
            IF [PTR83] <> NULL AND [PTR83] <> MONETARY(U,13,2)
               RETURN VALIDATION MESSAGE
            ENDIF
            
            [PTR83] = PTR:RP.JAUS:rvctc1.02.16:IncomeTax.TFNAmountsWithheldFromGrossInterest.Amount

            Technical Business Rule Format:
            NotMonetary(^PTR83, 'U', 13, 2)
    
            Data Elements:
    
            RP.JAUS:^PTR83 = rvctc1.02.16:IncomeTax.TFNAmountsWithheldFromGrossInterest.Amount
            */
            assertion = NotMonetary(report.PTR83, @"U", 13, 2);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.400011",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Field incorrect format. Exceeds maximum digits or decimals and/or contains illegal characters",
                    LongDescription = @"Field is in incorrect format for a monetary field. Field either exceeds maximum digits or decimals and/or contains illegal characters and/or for a signed field does not have the sign as the first character",
                    Location = "/xbrli:xbrl/rvctc1.02.16:IncomeTax.TFNAmountsWithheldFromGrossInterest.Amount",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.PTR.430094" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "PTR83", Value = report.PTR83.GetValueOrDefault().ToString() });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.PTR.430094

            #region VR.ATO.PTR.430095

            /*  VR.ATO.PTR.430095
            Dividends amounts must be present
    
            Legacy Rule Format:
            IF [PTR87] > 0 AND ([PTR84] = 0 OR [PTR84] = NULL) AND ([PTR85] = 0 OR [PTR85] = NULL) 
                RETURN VALIDATION MESSAGE
            ENDIF
            
            [PTR84] = PTR:RP.JAUS:bafpr1.02.05:Income.DividendsUnfranked.Amount
            [PTR85] = PTR:RP.JAUS:bafpr1.02.00:Income.DividendsFranked.Amount
            [PTR87] = PTR:RP.JAUS:rvctc1.02.00:IncomeTax.DividendsTFNAmountsWithheld.Amount

            Technical Business Rule Format:
            ^PTR87 > 0 AND (^PTR84 = 0 OR ^PTR84 = NULL) AND (^PTR85 = 0 OR ^PTR85 = NULL)
    
            Data Elements:
    
            RP.JAUS:^PTR84 = bafpr1.02.05:Income.DividendsUnfranked.Amount
    
            RP.JAUS:^PTR85 = bafpr1.02.00:Income.DividendsFranked.Amount
    
            RP.JAUS:^PTR87 = rvctc1.02.00:IncomeTax.DividendsTFNAmountsWithheld.Amount
            */
            assertion = (report.PTR87.GetValueOrDefault() > 0 && (report.PTR84 == 0 || report.PTR84 == null) && (report.PTR85 == 0 || report.PTR85 == null));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.430095",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Dividends amounts must be present",
                    LongDescription = @"If an amount greater than zero is present at TFN amounts withheld from dividends, then there must also be an amount greater than zero at Unfranked amount or Franked amount",
                    Location = "/xbrli:xbrl/bafpr1.02.05:Income.DividendsUnfranked.Amount",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.PTR.430095" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "PTR87", Value = report.PTR87.GetValueOrDefault().ToString() });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "PTR84", Value = report.PTR84.GetValueOrDefault().ToString() });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "PTR85", Value = report.PTR85.GetValueOrDefault().ToString() });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.PTR.430095

            #region VR.ATO.PTR.430096

            /*  VR.ATO.PTR.430096
            TFN amount withheld must not exceed unfranked dividends plus 50% of franked dividends
    
            Legacy Rule Format:
            IF ([PTR87] > 0 AND [PTR84] > 0 AND [PTR85] > 0) AND ([PTR84] + (([PTR85]*0.5) + 1)) < [PTR87] 
                RETURN VALIDATION MESSAGE
            ENDIF
            
            [PTR84] = PTR:RP.JAUS:bafpr1.02.05:Income.DividendsUnfranked.Amount
            [PTR85] = PTR:RP.JAUS:bafpr1.02.00:Income.DividendsFranked.Amount
            [PTR87] = PTR:RP.JAUS:rvctc1.02.00:IncomeTax.DividendsTFNAmountsWithheld.Amount

            Technical Business Rule Format:
            (^PTR87 > 0 AND ^PTR84 > 0 AND ^PTR85 > 0) AND (^PTR84 + ((^PTR85*0.5) + 1)) < ^PTR87
    
            Data Elements:
    
            RP.JAUS:^PTR84 = bafpr1.02.05:Income.DividendsUnfranked.Amount
    
            RP.JAUS:^PTR85 = bafpr1.02.00:Income.DividendsFranked.Amount
    
            RP.JAUS:^PTR87 = rvctc1.02.00:IncomeTax.DividendsTFNAmountsWithheld.Amount
            */
            assertion = (report.PTR87.GetValueOrDefault() > 0 && report.PTR84.GetValueOrDefault() > 0 && report.PTR85.GetValueOrDefault() > 0 && report.PTR84.GetValueOrDefault() + (report.PTR85.GetValueOrDefault() * 0.5M + 1) < report.PTR87.GetValueOrDefault());
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.430096",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"TFN amount withheld must not exceed unfranked dividends plus 50% of franked dividends",
                    LongDescription = @"The amount at TFN amounts withheld from dividends must not be greater than Unfranked amount plus 50% of Franked amount",
                    Location = "/xbrli:xbrl/bafpr1.02.05:Income.DividendsUnfranked.Amount",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.PTR.430096" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "PTR87", Value = report.PTR87.GetValueOrDefault().ToString() });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "PTR84", Value = report.PTR84.GetValueOrDefault().ToString() });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "PTR85", Value = report.PTR85.GetValueOrDefault().ToString() });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.PTR.430096

            #region VR.ATO.PTR.430097

            /*  VR.ATO.PTR.430097
            Monetary field must be unsigned, maximum of 11 numeric characters, no decimals
    
            Legacy Rule Format:
            IF [PTR84] <> NULL AND [PTR84] <> MONETARY(U,11,0)
               RETURN VALIDATION MESSAGE
            ENDIF
            
            [PTR84] = PTR:RP.JAUS:bafpr1.02.05:Income.DividendsUnfranked.Amount

            Technical Business Rule Format:
            NotMonetary(^PTR84, 'U', 11, 0)
    
            Data Elements:
    
            RP.JAUS:^PTR84 = bafpr1.02.05:Income.DividendsUnfranked.Amount
            */
            assertion = NotMonetary(report.PTR84, @"U", 11, 0);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.400011",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Field incorrect format. Exceeds maximum digits or decimals and/or contains illegal characters",
                    LongDescription = @"Field is in incorrect format for a monetary field. Field either exceeds maximum digits or decimals and/or contains illegal characters and/or for a signed field does not have the sign as the first character",
                    Location = "/xbrli:xbrl/bafpr1.02.05:Income.DividendsUnfranked.Amount",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.PTR.430097" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "PTR84", Value = report.PTR84.GetValueOrDefault().ToString() });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.PTR.430097

            #region VR.ATO.PTR.430098

            /*  VR.ATO.PTR.430098
            Franking credit cannot be greater than 48% of dividends-franked amount.
    
            Legacy Rule Format:
            IF [PTR86] > 0 AND (([PTR85] * 0.48) < ([PTR86] - 1)) 
                RETURN VALIDATION MESSAGE
            ENDIF
            
            [PTR85] = PTR:RP.JAUS:bafpr1.02.00:Income.DividendsFranked.Amount
            [PTR86] = PTR:RP.JAUS:rvctc1.02.03:IncomeTax.FrankingCredits.Amount

            Technical Business Rule Format:
            ^PTR86 > 0 AND ((^PTR85 * 0.48) < (^PTR86 - 1))
    
            Data Elements:
    
            RP.JAUS:^PTR86 = rvctc1.02.03:IncomeTax.FrankingCredits.Amount
    
            RP.JAUS:^PTR85 = bafpr1.02.00:Income.DividendsFranked.Amount
            */
            assertion = (report.PTR86.GetValueOrDefault() > 0 && report.PTR85.GetValueOrDefault() * 0.48M < report.PTR86.GetValueOrDefault() - 1);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.430098",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Franking credit cannot be greater than 48% of dividends-franked amount.",
                    LongDescription = @"Franking credit must not exceed 48% of Dividends - Franked amount",
                    Location = "/xbrli:xbrl/rvctc1.02.03:IncomeTax.FrankingCredits.Amount",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.PTR.430098" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "PTR86", Value = report.PTR86.GetValueOrDefault().ToString() });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "PTR85", Value = report.PTR85.GetValueOrDefault().ToString() });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.PTR.430098

            #region VR.ATO.PTR.430099

            /*  VR.ATO.PTR.430099
            Monetary field must be unsigned, maximum of 11 numeric characters, no decimals
    
            Legacy Rule Format:
            IF [PTR85] <> NULL AND [PTR85] <> MONETARY(U,11,0)
               RETURN VALIDATION MESSAGE
            ENDIF
            
            [PTR85] = PTR:RP.JAUS:bafpr1.02.00:Income.DividendsFranked.Amount

            Technical Business Rule Format:
            NotMonetary(^PTR85, 'U', 11, 0)
    
            Data Elements:
    
            RP.JAUS:^PTR85 = bafpr1.02.00:Income.DividendsFranked.Amount
            */
            assertion = NotMonetary(report.PTR85, @"U", 11, 0);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.400011",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Field incorrect format. Exceeds maximum digits or decimals and/or contains illegal characters",
                    LongDescription = @"Field is in incorrect format for a monetary field. Field either exceeds maximum digits or decimals and/or contains illegal characters and/or for a signed field does not have the sign as the first character",
                    Location = "/xbrli:xbrl/bafpr1.02.00:Income.DividendsFranked.Amount",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.PTR.430099" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "PTR85", Value = report.PTR85.GetValueOrDefault().ToString() });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.PTR.430099

            #region VR.ATO.PTR.430100

            /*  VR.ATO.PTR.430100
            Franking credit amount must be present
    
            Legacy Rule Format:
            IF [PTR85] > 0 AND [PTR86] = NULL 
                RETURN VALIDATION MESSAGE
            ENDIF
            
            [PTR85] = PTR:RP.JAUS:bafpr1.02.00:Income.DividendsFranked.Amount
            [PTR86] = PTR:RP.JAUS:rvctc1.02.03:IncomeTax.FrankingCredits.Amount

            Technical Business Rule Format:
            ^PTR85 > 0 AND ^PTR86 = NULL
    
            Data Elements:
    
            RP.JAUS:^PTR86 = rvctc1.02.03:IncomeTax.FrankingCredits.Amount
    
            RP.JAUS:^PTR85 = bafpr1.02.00:Income.DividendsFranked.Amount
            */
            assertion = (report.PTR85.GetValueOrDefault() > 0 && report.PTR86 == null);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.430100",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Franking credit amount must be present",
                    LongDescription = @"If Dividends - Franked amount is present, then Franking credit must be present",
                    Location = "/xbrli:xbrl/rvctc1.02.03:IncomeTax.FrankingCredits.Amount",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.PTR.430100" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "PTR85", Value = report.PTR85.GetValueOrDefault().ToString() });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "PTR86", Value = report.PTR86.GetValueOrDefault().ToString() });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.PTR.430100

            #region VR.ATO.PTR.430101

            /*  VR.ATO.PTR.430101
            Monetary field must be unsigned, maximum of 11 numeric characters, no decimals
    
            Legacy Rule Format:
            IF [PTR86] <> NULL AND [PTR86] <> MONETARY(U,11,0)
               RETURN VALIDATION MESSAGE
            ENDIF
            
            [PTR86] = PTR:RP.JAUS:rvctc1.02.03:IncomeTax.FrankingCredits.Amount

            Technical Business Rule Format:
            NotMonetary(^PTR86, 'U', 11, 0)
    
            Data Elements:
    
            RP.JAUS:^PTR86 = rvctc1.02.03:IncomeTax.FrankingCredits.Amount
            */
            assertion = NotMonetary(report.PTR86, @"U", 11, 0);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.400011",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Field incorrect format. Exceeds maximum digits or decimals and/or contains illegal characters",
                    LongDescription = @"Field is in incorrect format for a monetary field. Field either exceeds maximum digits or decimals and/or contains illegal characters and/or for a signed field does not have the sign as the first character",
                    Location = "/xbrli:xbrl/rvctc1.02.03:IncomeTax.FrankingCredits.Amount",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.PTR.430101" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "PTR86", Value = report.PTR86.GetValueOrDefault().ToString() });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.PTR.430101

            #region VR.ATO.PTR.430102

            /*  VR.ATO.PTR.430102
            TFN amounts withheld from dividends must not exceed 50% of the Unfranked amount
    
            Legacy Rule Format:
            IF [PTR87] > 0 AND (([PTR84]*0.5) + 1) < [PTR87] 
                RETURN VALIDATION MESSAGE
            ENDIF
            
            [PTR84] = PTR:RP.JAUS:bafpr1.02.05:Income.DividendsUnfranked.Amount
            [PTR87] = PTR:RP.JAUS:rvctc1.02.00:IncomeTax.DividendsTFNAmountsWithheld.Amount

            Technical Business Rule Format:
            ^PTR87 > 0 AND ((^PTR84*0.5) + 1) < ^PTR87
    
            Data Elements:
    
            RP.JAUS:^PTR87 = rvctc1.02.00:IncomeTax.DividendsTFNAmountsWithheld.Amount
    
            RP.JAUS:^PTR84 = bafpr1.02.05:Income.DividendsUnfranked.Amount
            */
            assertion = (report.PTR87.GetValueOrDefault() > 0 && report.PTR84.GetValueOrDefault() * 0.5M + 1 < report.PTR87.GetValueOrDefault());
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.430102",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"TFN amounts withheld from dividends must not exceed 50% of the Unfranked amount",
                    Location = "/xbrli:xbrl/rvctc1.02.00:IncomeTax.DividendsTFNAmountsWithheld.Amount",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.PTR.430102" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "PTR87", Value = report.PTR87.GetValueOrDefault().ToString() });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "PTR84", Value = report.PTR84.GetValueOrDefault().ToString() });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.PTR.430102

            #region VR.ATO.PTR.430103

            /*  VR.ATO.PTR.430103
            Monetary field - unsigned, maximum of 13 characters, may include a decimal point and up to 2 decimals
    
            Legacy Rule Format:
            IF [PTR87] <> NULL AND [PTR87] <> MONETARY(U,13,2)
               RETURN VALIDATION MESSAGE
            ENDIF
            
            [PTR87] = PTR:RP.JAUS:rvctc1.02.00:IncomeTax.DividendsTFNAmountsWithheld.Amount

            Technical Business Rule Format:
            NotMonetary(^PTR87, 'U', 13, 2)
    
            Data Elements:
    
            RP.JAUS:^PTR87 = rvctc1.02.00:IncomeTax.DividendsTFNAmountsWithheld.Amount
            */
            assertion = NotMonetary(report.PTR87, @"U", 13, 2);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.400011",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Field incorrect format. Exceeds maximum digits or decimals and/or contains illegal characters",
                    LongDescription = @"Field is in incorrect format for a monetary field. Field either exceeds maximum digits or decimals and/or contains illegal characters and/or for a signed field does not have the sign as the first character",
                    Location = "/xbrli:xbrl/rvctc1.02.00:IncomeTax.DividendsTFNAmountsWithheld.Amount",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.PTR.430103" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "PTR87", Value = report.PTR87.GetValueOrDefault().ToString() });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.PTR.430103

            #region VR.ATO.PTR.430106

            /*  VR.ATO.PTR.430106
            Gross interest or dividend income must be present
    
            Legacy Rule Format:
            IF [PTR90] > 0 AND ([PTR82] = NULL AND [PTR84] = NULL AND [PTR85] = NULL) 
                RETURN VALIDATION MESSAGE
            ENDIF
            
            [PTR82] = PTR:RP.JAUS:bafpr2.02.02:Income.Interest.Gross.Amount
            [PTR84] = PTR:RP.JAUS:bafpr1.02.05:Income.DividendsUnfranked.Amount
            [PTR85] = PTR:RP.JAUS:bafpr1.02.00:Income.DividendsFranked.Amount
            [PTR90] = PTR:RP.JAUS:bafpr1.02.04:Expense.AustralianInvestmentAllowableDeduction.Amount

            Technical Business Rule Format:
            ^PTR90 > 0 AND (^PTR82 = NULL AND ^PTR84 = NULL AND ^PTR85 = NULL)
    
            Data Elements:
    
            RP.JAUS:^PTR82 = bafpr2.02.02:Income.Interest.Gross.Amount
    
            RP.JAUS:^PTR84 = bafpr1.02.05:Income.DividendsUnfranked.Amount
    
            RP.JAUS:^PTR85 = bafpr1.02.00:Income.DividendsFranked.Amount
    
            RP.JAUS:^PTR90 = bafpr1.02.04:Expense.AustralianInvestmentAllowableDeduction.Amount
            */
            assertion = (report.PTR90.GetValueOrDefault() > 0 && (report.PTR82 == null && report.PTR84 == null && report.PTR85 == null));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.430106",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Gross interest or dividend income must be present",
                    LongDescription = @"If Deductions relating to Australian investment income is greater than zero, then Gross interest, Unfranked amount, or Franked amount must be present",
                    Location = "/xbrli:xbrl/bafpr2.02.02:Income.Interest.Gross.Amount",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.PTR.430106" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "PTR90", Value = report.PTR90.GetValueOrDefault().ToString() });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "PTR82", Value = report.PTR82.GetValueOrDefault().ToString() });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "PTR84", Value = report.PTR84.GetValueOrDefault().ToString() });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "PTR85", Value = report.PTR85.GetValueOrDefault().ToString() });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.PTR.430106

            #region VR.ATO.PTR.430107

            /*  VR.ATO.PTR.430107
            Monetary field must be unsigned, maximum of 11 numeric characters, no decimals
    
            Legacy Rule Format:
            IF [PTR90] <> NULL AND [PTR90] <> MONETARY(U,11,0)
               RETURN VALIDATION MESSAGE
            ENDIF
            
            [PTR90] = PTR:RP.JAUS:bafpr1.02.04:Expense.AustralianInvestmentAllowableDeduction.Amount

            Technical Business Rule Format:
            NotMonetary(^PTR90, 'U', 11, 0)
    
            Data Elements:
    
            RP.JAUS:^PTR90 = bafpr1.02.04:Expense.AustralianInvestmentAllowableDeduction.Amount
            */
            assertion = NotMonetary(report.PTR90, @"U", 11, 0);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.400011",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Field incorrect format. Exceeds maximum digits or decimals and/or contains illegal characters",
                    LongDescription = @"Field is in incorrect format for a monetary field. Field either exceeds maximum digits or decimals and/or contains illegal characters and/or for a signed field does not have the sign as the first character",
                    Location = "/xbrli:xbrl/bafpr1.02.04:Expense.AustralianInvestmentAllowableDeduction.Amount",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.PTR.430107" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "PTR90", Value = report.PTR90.GetValueOrDefault().ToString() });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.PTR.430107

            #region VR.ATO.PTR.430109

            /*  VR.ATO.PTR.430109
            Other deductions must not exceed 99 instances
    
            Legacy Rule Format:
            IF COUNT(ptr.0006.lodge.req.xx.xx:OtherDeductions) > 99
            RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            Count(^OtherDeductions) > 99
    
            Data Elements:
    
            ^OtherDeductions = ptr.0006.lodge.req.02.00:OtherDeductions
            */
            assertion = (Count(report.OtherDeductionsCollectionCount) > 99);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.430109",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Other deductions must not exceed 99 instances",
                    Location = "/xbrli:xbrl/ptr.0006.lodge.req.02.00:OtherDeductions" + OccurrenceIndex(report.OtherDeductionsCollection == null ? 0 : report.OtherDeductionsCollection.Last().OccurrenceIndex) + "/bafpr1.02.04:Expense.DeductibleOther.Text",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.PTR.430109" } },
                };
                response.Add(processMessage);
            }
            #endregion // VR.ATO.PTR.430109

            #region VR.ATO.PTR.430112

            /*  VR.ATO.PTR.430112
            Net Australian income must equal income less deductions
    
            Legacy Rule Format:
            IF [PTR97] <> NULL AND [PTR97] <> ([PTR61] + [PTR62] + [PTR65] + [PTR66] - [PTR68] + [PTR69] + [PTR70] - [PTR72] + [PTR322] - [PTR323] + [PTR77] - [PTR78] - [PTR79] - [PTR80] + [PTR81] + [PTR82] + [PTR84] + [PTR85] + [PTR86] + SUM([PTR89])) - ([PTR90] + [PTR91] + SUM([PTR96])) 
                RETURN VALIDATION MESSAGE
            ENDIF
            
            [PTR61] = PTR:RP.JAUS.Prim:bafpr1.02.23:Income.Net.Amount
            [PTR62] = PTR:RP.JAUS.NonPrim:bafpr1.02.23:Income.Net.Amount
            [PTR65] = PTR:RP.JAUS.Prim:bafpr1.02.02:Income.PartnershipDistributionGross.Amount
            [PTR66] = PTR:RP.JAUS.Prim:bafpr1.02.14:Income.TrustShareNet.Amount
            [PTR68] = PTR:RP.JAUS.Prim:bafpr1.02.04:Expense.DistributionDeductionsOther.Amount
            [PTR69] = PTR:RP.JAUS.NonPrim:bafpr1.02.04:Income.PartnershipDistributionLessForeignIncome.Amount
            [PTR70] = PTR:RP.JAUS.NonPrim:bafpr1.02.13:Income.TrustShareNetExcludeNetCapitalGainsAndForeignIncomeAndDistributionFranked.Amount
            [PTR72] = PTR:RP.JAUS.NonPrim:bafpr1.02.04:Expense.DistributionDeductionsOther.Amount
            [PTR77] = PTR:RP.JAUS:bafpr2.02.00:Income.Operating.RentalIncomeGross.Amount
            [PTR78] = PTR:RP.JAUS:bafpr1.02.05:Expense.Interest.Amount
            [PTR79] = PTR:RP.JAUS:bafpr1.02.02:Expense.CapitalWorksDeduction.Amount
            [PTR80] = PTR:RP.JAUS:rvctc2.02.00:IncomeTax.Deduction.RentalIncomeDeductionsOtherThanInterestAndCapitalWorks.Amount
            [PTR81] = PTR:RP.JAUS:bafpr1.02.23:Income.ForestryManagedInvestmentScheme.Amount
            [PTR82] = PTR:RP.JAUS:bafpr2.02.02:Income.Interest.Gross.Amount
            [PTR84] = PTR:RP.JAUS:bafpr1.02.05:Income.DividendsUnfranked.Amount
            [PTR85] = PTR:RP.JAUS:bafpr1.02.00:Income.DividendsFranked.Amount
            [PTR86] = PTR:RP.JAUS:rvctc1.02.03:IncomeTax.FrankingCredits.Amount
            [PTR89] = PTR:RP.JAUS:bafpr1.02.04:Income.Other.Amount IN TUPLE(ptr.0006.lodge.req.02.00:OtherAustralianIncome)
            [PTR90] = PTR:RP.JAUS:bafpr1.02.04:Expense.AustralianInvestmentAllowableDeduction.Amount
            [PTR91] = PTR:RP.JAUS:bafpr1.02.23:Expense.ForestryManagedInvestmentSchemeDeduction.Amount
            [PTR96] = PTR:RP.JAUS:bafpr1.02.04:Expense.DeductibleOther.Amount IN TUPLE(ptr.0006.lodge.req.02.00:OtherDeductions)
            [PTR97] = PTR:RP.JAUS:bafpr1.02.23:Income.Net.Amount
            [PTR322] = PTR:RP.JAUS.NonPrim:bafpr1.02.04:Income.TrustDistributionFranked.Amount
            [PTR323] = PTR:RP.JAUS.NonPrim:rvctc2.02.05:IncomeTax.Deduction.TrustDistributionFranked.Amount

            Technical Business Rule Format:
            ^PTR97 <> NULL AND ^PTR97 <> (^PTR61 + ^PTR62 + ^PTR65 + ^PTR66 - ^PTR68 + ^PTR69 + ^PTR70 - ^PTR72 + ^PTR322 - ^PTR323 + ^PTR77 - ^PTR78 - ^PTR79 - ^PTR80 + ^PTR81 + ^PTR82 + ^PTR84 + ^PTR85 + ^PTR86 + Sum(^PTR89)) - (^PTR90 + ^PTR91 + Sum(^PTR96))
    
            Data Elements:
    
            RP.JAUS:^PTR97 = bafpr1.02.23:Income.Net.Amount
    
            RP.JAUS.Prim:^PTR61 = bafpr1.02.23:Income.Net.Amount
    
            RP.JAUS.NonPrim:^PTR62 = bafpr1.02.23:Income.Net.Amount
    
            RP.JAUS.Prim:^PTR65 = bafpr1.02.02:Income.PartnershipDistributionGross.Amount
    
            RP.JAUS.Prim:^PTR66 = bafpr1.02.14:Income.TrustShareNet.Amount
    
            RP.JAUS.Prim:^PTR68 = bafpr1.02.04:Expense.DistributionDeductionsOther.Amount
    
            RP.JAUS.NonPrim:^PTR69 = bafpr1.02.04:Income.PartnershipDistributionLessForeignIncome.Amount
    
            RP.JAUS.NonPrim:^PTR70 = bafpr1.02.13:Income.TrustShareNetExcludeNetCapitalGainsAndForeignIncomeAndDistributionFranked.Amount
    
            RP.JAUS.NonPrim:^PTR72 = bafpr1.02.04:Expense.DistributionDeductionsOther.Amount
    
            RP.JAUS:^PTR77 = bafpr2.02.00:Income.Operating.RentalIncomeGross.Amount
    
            RP.JAUS:^PTR78 = bafpr1.02.05:Expense.Interest.Amount
    
            RP.JAUS:^PTR79 = bafpr1.02.02:Expense.CapitalWorksDeduction.Amount
    
            RP.JAUS:^PTR80 = rvctc2.02.00:IncomeTax.Deduction.RentalIncomeDeductionsOtherThanInterestAndCapitalWorks.Amount
    
            RP.JAUS:^PTR81 = bafpr1.02.23:Income.ForestryManagedInvestmentScheme.Amount
    
            RP.JAUS:^PTR82 = bafpr2.02.02:Income.Interest.Gross.Amount
    
            RP.JAUS:^PTR84 = bafpr1.02.05:Income.DividendsUnfranked.Amount
    
            RP.JAUS:^PTR85 = bafpr1.02.00:Income.DividendsFranked.Amount
    
            RP.JAUS:^PTR86 = rvctc1.02.03:IncomeTax.FrankingCredits.Amount
    
            RP.JAUS:^PTR89 = bafpr1.02.04:Income.Other.Amount
    
            RP.JAUS:^PTR90 = bafpr1.02.04:Expense.AustralianInvestmentAllowableDeduction.Amount
    
            RP.JAUS:^PTR91 = bafpr1.02.23:Expense.ForestryManagedInvestmentSchemeDeduction.Amount
    
            RP.JAUS:^PTR96 = bafpr1.02.04:Expense.DeductibleOther.Amount
    
            RP.JAUS.NonPrim:^PTR322 = bafpr1.02.04:Income.TrustDistributionFranked.Amount
    
            RP.JAUS.NonPrim:^PTR323 = rvctc2.02.05:IncomeTax.Deduction.TrustDistributionFranked.Amount
            */
            assertion = (report.PTR97 != null && report.PTR97.GetValueOrDefault() != report.PTR61.GetValueOrDefault() + report.PTR62.GetValueOrDefault() + report.PTR65.GetValueOrDefault() + report.PTR66.GetValueOrDefault() - report.PTR68.GetValueOrDefault() + report.PTR69.GetValueOrDefault() + report.PTR70.GetValueOrDefault() - report.PTR72.GetValueOrDefault() + report.PTR322.GetValueOrDefault() - report.PTR323.GetValueOrDefault() + report.PTR77.GetValueOrDefault() - report.PTR78.GetValueOrDefault() - report.PTR79.GetValueOrDefault() - report.PTR80.GetValueOrDefault() + report.PTR81.GetValueOrDefault() + report.PTR82.GetValueOrDefault() + report.PTR84.GetValueOrDefault() + report.PTR85.GetValueOrDefault() + report.PTR86.GetValueOrDefault() + (report.OtherAustralianIncomeCollection == null ? 0 : report.OtherAustralianIncomeCollection.Sum(f => f.PTR89.GetValueOrDefault())) - (report.PTR90.GetValueOrDefault() + report.PTR91.GetValueOrDefault() + (report.OtherDeductionsCollection == null ? 0 : report.OtherDeductionsCollection.Sum(f => f.PTR96.GetValueOrDefault()))));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.PTR.430112",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Net Australian income must equal income less deductions",
                    LongDescription = @"The Net Australian income or loss amount must equal the sum of all income less the sum of all deductions",
                    Location = "/xbrli:xbrl/bafpr1.02.23:Income.Net.Amount[@contextRef='" + report.RPJAUSId + "']",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.PTR.430112" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "PTR97", Value = report.PTR97.GetValueOrDefault().ToString() });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "PTR61", Value = report.PTR61.GetValueOrDefault().ToString() });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "PTR62", Value = report.PTR62.GetValueOrDefault().ToString() });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "PTR65", Value = report.PTR65.GetValueOrDefault().ToString() });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "PTR66", Value = report.PTR66.GetValueOrDefault().ToString() });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "PTR68", Value = report.PTR68.GetValueOrDefault().ToString() });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "PTR69", Value = report.PTR69.GetValueOrDefault().ToString() });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "PTR70", Value = report.PTR70.GetValueOrDefault().ToString() });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "PTR72", Value = report.PTR72.GetValueOrDefault().ToString() });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "PTR322", Value = report.PTR322.GetValueOrDefault().ToString() });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "PTR323", Value = report.PTR323.GetValueOrDefault().ToString() });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "PTR77", Value = report.PTR77.GetValueOrDefault().ToString() });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "PTR78", Value = report.PTR78.GetValueOrDefault().ToString() });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "PTR79", Value = report.PTR79.GetValueOrDefault().ToString() });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "PTR80", Value = report.PTR80.GetValueOrDefault().ToString() });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "PTR81", Value = report.PTR81.GetValueOrDefault().ToString() });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "PTR82", Value = report.PTR82.GetValueOrDefault().ToString() });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "PTR84", Value = report.PTR84.GetValueOrDefault().ToString() });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "PTR85", Value = report.PTR85.GetValueOrDefault().ToString() });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "PTR86", Value = report.PTR86.GetValueOrDefault().ToString() });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "PTR89", Value = "PTR89" });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "PTR90", Value = report.PTR90.GetValueOrDefault().ToString() });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "PTR91", Value = report.PTR91.GetValueOrDefault().ToString() });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "PTR96", Value = "PTR96" });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.PTR.430112

            #region VR.ATO.PTR.430113

            /*  VR.ATO.PTR.430113
            Monetary field may be signed, maximum of 11 chars, no decimals
    
            Legacy Rule Format:
            IF [PTR97] <> NULL AND [PTR97] <> MONETARY(S,11,0)
               RETURN VALIDATION MESSAGE
            ENDIF
            
            [PTR97] = PTR:RP.JAUS:bafpr1.02.23:Income.Net.Amount

            Technical Business Rule Format:
            NotMonetary(^PTR97, 'S', 11, 0)
    
            Data Elements:
    
            RP.JAUS:^PTR97 = bafpr1.02.23:Income.Net.Amount
            */
            assertion = NotMonetary(report.PTR97, @"S", 11, 0);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.400011",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Field incorrect format. Exceeds maximum digits or decimals and/or contains illegal characters",
                    LongDescription = @"Field is in incorrect format for a monetary field. Field either exceeds maximum digits or decimals and/or contains illegal characters and/or for a signed field does not have the sign as the first character",
                    Location = "/xbrli:xbrl/bafpr1.02.23:Income.Net.Amount[@contextRef='" + report.RPJAUSId + "']",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.PTR.430113" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "PTR97", Value = report.PTR97.GetValueOrDefault().ToString() });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.PTR.430113

            #region VR.ATO.PTR.430116

            /*  VR.ATO.PTR.430116
            Answer to Attributed foreign income question must be 'yes'
    
            Legacy Rule Format:
            IF ([PTR98] > 0 OR [PTR100] > 0) AND [PTR102] <> TRUE 
               RETURN VALIDATION MESSAGE
            ENDIF
            
            [PTR98] = PTR:RP.JFOREIGN.CFCLC:bafpr3.02.02:Income.InternationalDealings.Attributed.Amount
            [PTR100] = PTR:RP.JFOREIGN.CFCUC:bafpr3.02.02:Income.InternationalDealings.Attributed.Amount
            [PTR102] = PTR:RP.JFOREIGN:bafpr3.02.03:Income.InternationalDealings.AttributedForeignIncomeForeignTrustorControlledForeignCompanyorTransferorTrust.Indicator

            Technical Business Rule Format:
            (^PTR98 > 0 OR ^PTR100 > 0) AND ^PTR102 <> TRUE
    
            Data Elements:
    
            RP.JFOREIGN:^PTR102 = bafpr3.02.03:Income.InternationalDealings.AttributedForeignIncomeForeignTrustorControlledForeignCompanyorTransferorTrust.Indicator
    
            RP.JFOREIGN.CFCLC:^PTR98 = bafpr3.02.02:Income.InternationalDealings.Attributed.Amount
    
            RP.JFOREIGN.CFCUC:^PTR100 = bafpr3.02.02:Income.InternationalDealings.Attributed.Amount
            */
            assertion = ((report.PTR98.GetValueOrDefault() > 0 || report.PTR100.GetValueOrDefault() > 0) && report.PTR102 != true);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.430116",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Answer to Attributed foreign income question must be 'yes'",
                    LongDescription = @"If Attributed foreign income - Listed country, Section 404 country, or Unlisted country is greater than zero, then a yes (true) response to the question ""Did you have either a direct or indirect interest in a foreign trust, controlled foreign company or transferor trust?"" is required",
                    Location = "/xbrli:xbrl/bafpr3.02.03:Income.InternationalDealings.AttributedForeignIncomeForeignTrustorControlledForeignCompanyorTransferorTrust.Indicator",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.PTR.430116" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "PTR98", Value = report.PTR98.GetValueOrDefault().ToString() });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "PTR100", Value = report.PTR100.GetValueOrDefault().ToString() });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "PTR102", Value = GetValueOrEmpty(report.PTR102) });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.PTR.430116

            #region VR.ATO.PTR.430120

            /*  VR.ATO.PTR.430120
            Other assessable foreign source income 'Gross' amount must not be less than 'Net' amount
    
            Legacy Rule Format:
            IF [PTR104] < [PTR105] 
                RETURN VALIDATION MESSAGE
            ENDIF
            
            [PTR104] = PTR:RP.JFOREIGN:bafpr1.02.04:Income.Other.Amount
            [PTR105] = PTR:RP.JFOREIGN:bafpr1.02.23:Income.Net.Amount

            Technical Business Rule Format:
            ^PTR104 < ^PTR105
    
            Data Elements:
    
            RP.JFOREIGN:^PTR104 = bafpr1.02.04:Income.Other.Amount
    
            RP.JFOREIGN:^PTR105 = bafpr1.02.23:Income.Net.Amount
            */
            assertion = (report.PTR104.GetValueOrDefault() < report.PTR105.GetValueOrDefault());
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.430120",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Other assessable foreign source income 'Gross' amount must not be less than 'Net' amount",
                    Location = "/xbrli:xbrl/bafpr1.02.04:Income.Other.Amount[@contextRef='" + report.RPJFOREIGNId + "']",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.PTR.430120" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "PTR104", Value = report.PTR104.GetValueOrDefault().ToString() });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "PTR105", Value = report.PTR105.GetValueOrDefault().ToString() });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.PTR.430120

            #region VR.ATO.PTR.430127

            /*  VR.ATO.PTR.430127
            Monetary field must be unsigned, maximum of 11 numeric characters, no decimals
    
            Legacy Rule Format:
            IF [PTR114] <> NULL AND [PTR114] <> MONETARY(U,11,0)
               RETURN VALIDATION MESSAGE
            ENDIF
            
            [PTR114] = PTR:RP.Closing(Instant):bafpo1.02.13:Liabilities.Total.Amount

            Technical Business Rule Format:
            NotMonetary(^PTR114, 'U', 11, 0)
    
            Data Elements:
    
            RP.Closing(Instant):^PTR114 = bafpo1.02.13:Liabilities.Total.Amount
            */
            assertion = NotMonetary(report.PTR114, @"U", 11, 0);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.400011",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Field incorrect format. Exceeds maximum digits or decimals and/or contains illegal characters",
                    LongDescription = @"Field is in incorrect format for a monetary field. Field either exceeds maximum digits or decimals and/or contains illegal characters and/or for a signed field does not have the sign as the first character",
                    Location = "/xbrli:xbrl/bafpo1.02.13:Liabilities.Total.Amount",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.PTR.430127" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "PTR114", Value = report.PTR114.GetValueOrDefault().ToString() });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.PTR.430127

            #region VR.ATO.PTR.430129

            /*  VR.ATO.PTR.430129
            Business name and Business address of main business must be present
    
            Legacy Rule Format:
            IF ([PTR35] <> NULL OR [PTR36] <> NULL OR [PTR38] <> NULL OR [PTR39] <> NULL OR [PTR41] <> NULL OR [PTR43] <> NULL OR [PTR44] <> NULL) AND ([PTR116] = NULLORBLANK OR [PTR117] = NULLORBLANK) 
                RETURN VALIDATION MESSAGE
            ENDIF
            
            [PTR35] = PTR:RP.JAUS.Prim:lrla.02.00:Remuneration.ABNNotQuotedPaymentGross.Amount
            [PTR36] = PTR:RP.JAUS.NonPrim:lrla.02.00:Remuneration.ABNNotQuotedPaymentGross.Amount
            [PTR38] = PTR:RP.JAUS.NonPrim:lrla.02.00:Remuneration.PaymentToForeignResidentGross.Amount
            [PTR39] = PTR:RP.JAUS.Prim:gfagg.02.00:GovernmentFunding.GovernmentIndustryPaymentsAssessable.Amount
            [PTR41] = PTR:RP.JAUS.NonPrim:gfagg.02.00:GovernmentFunding.GovernmentIndustryPaymentsAssessable.Amount
            [PTR43] = PTR:RP.JAUS.Prim:bafpr1.02.04:Income.Other.Amount
            [PTR44] = PTR:RP.JAUS.NonPrim:bafpr1.02.04:Income.Other.Amount
            [PTR116] = PTR:RP:pyde.02.00:OrganisationNameDetails.OrganisationalName.Text WHERE (pyde.02.00:OrganisationNameDetails.OrganisationalNameType.Code = "MTR" AND pyde.02.00:OrganisationNameDetails.Currency.Code = "C") IN TUPLE(orgname2.02.00:OrganisationNameDetails)
            [PTR117] = PTR:RP:pyde.02.00:AddressDetails.Line1.Text WHERE (pyde.02.01:AddressDetails.Usage.Code = "BUS" AND pyde.02.00:AddressDetails.Currency.Code = "C") IN TUPLE(address2.02.02:AddressDetails)

            Technical Business Rule Format:
            (^PTR35 <> NULL OR ^PTR36 <> NULL OR ^PTR38 <> NULL OR ^PTR39 <> NULL OR ^PTR41 <> NULL OR ^PTR43 <> NULL OR ^PTR44 <> NULL) AND (^PTR116 = BLANK OR ^PTR117 = BLANK)
    
            Data Elements:
    
            RP:^PTR116 = pyde.02.00:OrganisationNameDetails.OrganisationalName.Text WHERE [OrganisationNameDetails.OrganisationalNameType.Code='MTR'].[OrganisationNameDetails.Currency.Code='C']
    
            RP.JAUS.Prim:^PTR35 = lrla.02.00:Remuneration.ABNNotQuotedPaymentGross.Amount
    
            RP.JAUS.NonPrim:^PTR36 = lrla.02.00:Remuneration.ABNNotQuotedPaymentGross.Amount
    
            RP.JAUS.NonPrim:^PTR38 = lrla.02.00:Remuneration.PaymentToForeignResidentGross.Amount
    
            RP.JAUS.Prim:^PTR39 = gfagg.02.00:GovernmentFunding.GovernmentIndustryPaymentsAssessable.Amount
    
            RP.JAUS.NonPrim:^PTR41 = gfagg.02.00:GovernmentFunding.GovernmentIndustryPaymentsAssessable.Amount
    
            RP.JAUS.Prim:^PTR43 = bafpr1.02.04:Income.Other.Amount
    
            RP.JAUS.NonPrim:^PTR44 = bafpr1.02.04:Income.Other.Amount
    
            RP:^PTR117 = pyde.02.00:AddressDetails.Line1.Text WHERE [AddressDetails.Usage.Code='BUS'].[AddressDetails.Currency.Code='C']
            */
            assertion = ((report.PTR35 != null || report.PTR36 != null || report.PTR38 != null || report.PTR39 != null || report.PTR41 != null || report.PTR43 != null || report.PTR44 != null) && (string.IsNullOrWhiteSpace(report.PTR116) == true || string.IsNullOrWhiteSpace(report.PTR117) == true));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.430129",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Business name and business address of main business must be present",
                    LongDescription = @"If Total business income (Primary production or Non-primary production) is present at business income and expenses, then both the Business name and Business address of main business must be present",
                    Location = "/xbrli:xbrl/orgname2.02.00:OrganisationNameDetails" + OccurrenceIndex(report.OrganisationNameDetailsCollection_MTR_C_RPOccurrenceIndex) + "/pyde.02.00:OrganisationNameDetails.OrganisationalName.Text",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.PTR.430129" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "PTR35", Value = report.PTR35.GetValueOrDefault().ToString() });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "PTR36", Value = report.PTR36.GetValueOrDefault().ToString() });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "PTR38", Value = report.PTR38.GetValueOrDefault().ToString() });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "PTR39", Value = report.PTR39.GetValueOrDefault().ToString() });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "PTR41", Value = report.PTR41.GetValueOrDefault().ToString() });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "PTR43", Value = report.PTR43.GetValueOrDefault().ToString() });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "PTR44", Value = report.PTR44.GetValueOrDefault().ToString() });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "PTR116", Value = report.PTR116 });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "PTR117", Value = report.PTR117 });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.PTR.430129

            #region VR.ATO.PTR.430130

            /*  VR.ATO.PTR.430130
            Business and professional amounts must be present
    
            Legacy Rule Format:
            IF ([PTR61] <> NULL OR [PTR62] <> NULL) 
            AND (([PTR111] = NULL) 
            AND ([PTR112] = NULL) 
            AND ([PTR113] = NULL)
            AND ([PTR114] = NULL)
            AND ([PTR121] = NULL)
            AND ([PTR122] = NULL)
            AND ([PTR123] = NULL)
            AND ([PTR125] = NULL)
            AND ([PTR126] = NULL)
            AND ([PTR127] = NULL)
            AND ([PTR129] = NULL)
            AND ([PTR136] = NULL)) 
                RETURN VALIDATION MESSAGE
            ENDIF
            
            
            [PTR61] = PTR:RP.JAUS.Prim:bafpr1.02.23:Income.Net.Amount
            [PTR62] = PTR:RP.JAUS.NonPrim:bafpr1.02.23:Income.Net.Amount
            [PTR111] = PTR:RP.Closing(Instant):bafpo1.02.00:Assets.CurrentTotal.Amount
            [PTR112] = PTR:RP.Closing(Instant):bafpo1.02.13:Assets.Total.Amount
            [PTR113] = PTR:RP.Closing(Instant):bafpo1.02.00:Liabilities.CurrentTotal.Amount
            [PTR114] = PTR:RP.Closing(Instant):bafpo1.02.13:Liabilities.Total.Amount
            [PTR121] = PTR:RP.Opening(Instant):bafpo7.02.04:Assets.Inventories.Total.Amount
            [PTR122] = PTR:RP:bafpr2.02.02:Expense.Purchases.AndOtherCosts.Amount
            [PTR123] = PTR:RP.Closing(Instant):bafpo7.02.04:Assets.Inventories.Total.Amount
            [PTR125] = PTR:RP.Closing(Instant):bafpo1.02.13:Assets.TradeandReceivablesOther.Amount
            [PTR126] = PTR:RP.Closing(Instant):bafpo1.02.13:Liabilities.TradeAndOtherPayablesTotal.Amount
            [PTR127] = PTR:RP:lrla.02.17:Remuneration.WagesAndSalaries.Total.Amount
            [PTR129] = PTR:RP:lrla.02.00:Remuneration.PaymentToRelatedPartiesGross.Amount
            [PTR136] = PTR:RP:lrla.02.00:Remuneration.FringeBenefits.EmployeeContribution.Amount

            Technical Business Rule Format:
            (^PTR61 <> NULL OR ^PTR62 <> NULL) AND ((^PTR111 = NULL) AND (^PTR112 = NULL) AND (^PTR113 = NULL) AND (^PTR114 = NULL) AND (^PTR121 = NULL) AND (^PTR122 = NULL) AND (^PTR123 = NULL) AND (^PTR125 = NULL) AND (^PTR126 = NULL) AND (^PTR127 = NULL) AND (^PTR129 = NULL) AND (^PTR136 = NULL))
    
            Data Elements:
    
            RP.Opening(Instant):^PTR121 = bafpo7.02.04:Assets.Inventories.Total.Amount
    
            RP.JAUS.Prim:^PTR61 = bafpr1.02.23:Income.Net.Amount
    
            RP.JAUS.NonPrim:^PTR62 = bafpr1.02.23:Income.Net.Amount
    
            RP.Closing(Instant):^PTR111 = bafpo1.02.00:Assets.CurrentTotal.Amount
    
            RP.Closing(Instant):^PTR112 = bafpo1.02.13:Assets.Total.Amount
    
            RP.Closing(Instant):^PTR113 = bafpo1.02.00:Liabilities.CurrentTotal.Amount
    
            RP.Closing(Instant):^PTR114 = bafpo1.02.13:Liabilities.Total.Amount
    
            RP:^PTR122 = bafpr2.02.02:Expense.Purchases.AndOtherCosts.Amount
    
            RP.Closing(Instant):^PTR123 = bafpo7.02.04:Assets.Inventories.Total.Amount
    
            RP.Closing(Instant):^PTR125 = bafpo1.02.13:Assets.TradeandReceivablesOther.Amount
    
            RP.Closing(Instant):^PTR126 = bafpo1.02.13:Liabilities.TradeAndOtherPayablesTotal.Amount
    
            RP:^PTR127 = lrla.02.17:Remuneration.WagesAndSalaries.Total.Amount
    
            RP:^PTR129 = lrla.02.00:Remuneration.PaymentToRelatedPartiesGross.Amount
    
            RP:^PTR136 = lrla.02.00:Remuneration.FringeBenefits.EmployeeContribution.Amount
            */
            assertion = ((report.PTR61 != null || report.PTR62 != null) && (report.PTR111 == null && report.PTR112 == null && report.PTR113 == null && report.PTR114 == null && report.PTR121 == null && report.PTR122 == null && report.PTR123 == null && report.PTR125 == null && report.PTR126 == null && report.PTR127 == null && report.PTR129 == null && report.PTR136 == null));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.PTR.439000",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Key financial information amounts and/or Business and professional amounts must be present",
                    LongDescription = @"If Net income or loss from business (either Primary production or Non-primary production) is present, then there must be an amount included under Assets, Liabilities or under Business and professional items",
                    Location = "/xbrli:xbrl/bafpo7.02.04:Assets.Inventories.Total.Amount[@contextRef='" + report.RPOpeningInstantId + "']",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.PTR.430130" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "PTR61", Value = report.PTR61.GetValueOrDefault().ToString() });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "PTR62", Value = report.PTR62.GetValueOrDefault().ToString() });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "PTR111", Value = report.PTR111.GetValueOrDefault().ToString() });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "PTR112", Value = report.PTR112.GetValueOrDefault().ToString() });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "PTR113", Value = report.PTR113.GetValueOrDefault().ToString() });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "PTR114", Value = report.PTR114.GetValueOrDefault().ToString() });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "PTR121", Value = report.PTR121.GetValueOrDefault().ToString() });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "PTR122", Value = report.PTR122.GetValueOrDefault().ToString() });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "PTR123", Value = report.PTR123.GetValueOrDefault().ToString() });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "PTR125", Value = report.PTR125.GetValueOrDefault().ToString() });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "PTR126", Value = report.PTR126.GetValueOrDefault().ToString() });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "PTR127", Value = report.PTR127.GetValueOrDefault().ToString() });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "PTR129", Value = report.PTR129.GetValueOrDefault().ToString() });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "PTR136", Value = report.PTR136.GetValueOrDefault().ToString() });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.PTR.430130

            #region VR.ATO.PTR.430131

            /*  VR.ATO.PTR.430131
            Total salary and wage expenses amount must be present if the action code is present
    
            Legacy Rule Format:
            IF [PTR128] <> NULL AND ([PTR127] = 0 OR [PTR127] = NULL) 
                RETURN VALIDATION MESSAGE
            ENDIF
            
            [PTR127] = PTR:RP:lrla.02.17:Remuneration.WagesAndSalaries.Total.Amount
            [PTR128] = PTR:RP:lrla.02.05:Remuneration.WagesAndSalariesAction.Code

            Technical Business Rule Format:
            ^PTR128 <> NULL AND (^PTR127 = 0 OR ^PTR127 = NULL)
    
            Data Elements:
    
            RP:^PTR127 = lrla.02.17:Remuneration.WagesAndSalaries.Total.Amount
    
            RP:^PTR128 = lrla.02.05:Remuneration.WagesAndSalariesAction.Code
            */
            assertion = (report.PTR128 != null && (report.PTR127 == 0 || report.PTR127 == null));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.430131",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Total salary and wage expenses amount must be present if the action code is present",
                    LongDescription = @"If a Total salary and wage expenses action code is present an amount must be present. Either enter an amount or delete the action code",
                    Location = "/xbrli:xbrl/lrla.02.17:Remuneration.WagesAndSalaries.Total.Amount",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.PTR.430131" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "PTR128", Value = report.PTR128 });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "PTR127", Value = report.PTR127.GetValueOrDefault().ToString() });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.PTR.430131

            #region VR.ATO.PTR.430132

            /*  VR.ATO.PTR.430132
            Total salary and wage expenses action code must be present
    
            Legacy Rule Format:
            IF [PTR127] > 0 AND [PTR128] = NULL
                RETURN VALIDATION MESSAGE
            ENDIF
            
            [PTR127] = PTR:RP:lrla.02.17:Remuneration.WagesAndSalaries.Total.Amount
            [PTR128] = PTR:RP:lrla.02.05:Remuneration.WagesAndSalariesAction.Code

            Technical Business Rule Format:
            ^PTR127 > 0 AND ^PTR128 = NULL
    
            Data Elements:
    
            RP:^PTR128 = lrla.02.05:Remuneration.WagesAndSalariesAction.Code
    
            RP:^PTR127 = lrla.02.17:Remuneration.WagesAndSalaries.Total.Amount
            */
            assertion = (report.PTR127.GetValueOrDefault() > 0 && report.PTR128 == null);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.430132",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Total salary and wage expenses action code must be present",
                    LongDescription = @"If a Total salary and wage expenses amount is present, then an action code must be present",
                    Location = "/xbrli:xbrl/lrla.02.05:Remuneration.WagesAndSalariesAction.Code",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.PTR.430132" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "PTR127", Value = report.PTR127.GetValueOrDefault().ToString() });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "PTR128", Value = report.PTR128 });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.PTR.430132

            #region VR.ATO.PTR.430134

            /*  VR.ATO.PTR.430134
            Monetary field must be unsigned, maximum of 11 numeric characters, no decimals
    
            Legacy Rule Format:
            IF [PTR138] <> NULL AND [PTR138] <> MONETARY(U,11,0)
               RETURN VALIDATION MESSAGE
            ENDIF
            
            [PTR138] = PTR:RP.JFOREIGN:bafpr1.02.27:Expense.Royalties.Amount

            Technical Business Rule Format:
            NotMonetary(^PTR138, 'U', 11, 0)
    
            Data Elements:
    
            RP.JFOREIGN:^PTR138 = bafpr1.02.27:Expense.Royalties.Amount
            */
            assertion = NotMonetary(report.PTR138, @"U", 11, 0);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.400011",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Field incorrect format. Exceeds maximum digits or decimals and/or contains illegal characters",
                    LongDescription = @"Field is in incorrect format for a monetary field. Field either exceeds maximum digits or decimals and/or contains illegal characters and/or for a signed field does not have the sign as the first character",
                    Location = "/xbrli:xbrl/bafpr1.02.27:Expense.Royalties.Amount[@contextRef='" + report.RPJFOREIGNId + "']",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.PTR.430134" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "PTR138", Value = report.PTR138.GetValueOrDefault().ToString() });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.PTR.430134

            #region VR.ATO.PTR.430142

            /*  VR.ATO.PTR.430142
            Tax file number must not be the TFN of the Trust or Partnership
    
            Legacy Rule Format:
            IF (ANY OCCURRENCE OF[PTR157]) = RP:entity.identfier.TFN
                RETURN VALIDATION MESSAGE
            ENDIF
            
            [PTR157] = PTR:RP:pyid.02.00:Identifiers.TaxFileNumber.Identifier IN TUPLE(ptr.0006.lodge.req.02.00:StatementOfDistribution)

            Technical Business Rule Format:
            (AnyOccurrence(^PTR157, ^PTR157 = ^TFN))
    
            Data Elements:
    
            RP:^PTR157 = pyid.02.00:Identifiers.TaxFileNumber.Identifier
    
            RP:^TFN = RP
            */
            assertion = (report.StatementOfDistributionCollection == null ? false : report.StatementOfDistributionCollection.Any(PTR157Repeat => PTR157Repeat.PTR157 == report.RPIdentifierTFN));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.430142",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Tax file number must not be the TFN of the Trust or Partnership",
                    LongDescription = @"The TFN provided for a Trustee or Partner in a Statement of distribution must not be the same as the TFN of the Trust or Partnership",
                    Location = "/xbrli:xbrl/ptr.0006.lodge.req.02.00:StatementOfDistribution/pyid.02.00:Identifiers.TaxFileNumber.Identifier",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.PTR.430142" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "PTR157", Value = "PTR157" });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.PTR.430142

            #region VR.ATO.PTR.430144

            /*  VR.ATO.PTR.430144
            Distribution of PP income cannot exceed net PP income
    
            Legacy Rule Format:
            IF SUM([PTR158]) > ([PTR61] + [PTR65] + [PTR66] - [PTR68])
                RETURN VALIDATION MESSAGE
            ENDIF
            
            [PTR61] = PTR:RP.JAUS.Prim:bafpr1.02.23:Income.Net.Amount
            [PTR65] = PTR:RP.JAUS.Prim:bafpr1.02.02:Income.PartnershipDistributionGross.Amount
            [PTR66] = PTR:RP.JAUS.Prim:bafpr1.02.14:Income.TrustShareNet.Amount
            [PTR68] = PTR:RP.JAUS.Prim:bafpr1.02.04:Expense.DistributionDeductionsOther.Amount
            [PTR158] = PTR:RP.Prim:bafpr1.02.04:Income.BeneficiaryShare.Amount IN TUPLE(ptr.0006.lodge.req.02.00:StatementOfDistribution)

            Technical Business Rule Format:
            Sum(^PTR158) > (^PTR61 + ^PTR65 + ^PTR66 - ^PTR68)
    
            Data Elements:
    
            RP.Prim:^PTR158 = bafpr1.02.04:Income.BeneficiaryShare.Amount
    
            RP.JAUS.Prim:^PTR61 = bafpr1.02.23:Income.Net.Amount
    
            RP.JAUS.Prim:^PTR65 = bafpr1.02.02:Income.PartnershipDistributionGross.Amount
    
            RP.JAUS.Prim:^PTR66 = bafpr1.02.14:Income.TrustShareNet.Amount
    
            RP.JAUS.Prim:^PTR68 = bafpr1.02.04:Expense.DistributionDeductionsOther.Amount
            */
            assertion = ((report.StatementOfDistributionCollection == null ? 0 : report.StatementOfDistributionCollection.Sum(f => f.PTR158.GetValueOrDefault())) > report.PTR61.GetValueOrDefault() + report.PTR65.GetValueOrDefault() + report.PTR66.GetValueOrDefault() - report.PTR68.GetValueOrDefault());
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.PTR.430144",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Distribution of PP income cannot exceed net PP income",
                    LongDescription = @"The amount of primary production income in the distribution statement (Item 65 Label A) is greater than the net primary production income received - net primary production income (Item 5 Label Q) plus primary production distribution from partnerships (Item 8 Label A) plus primary production distribution from trusts (Item 8 Label Z) less primary production deductions relating to distribution (Item 8 Label S).",
                    Location = "/xbrli:xbrl/ptr.0006.lodge.req.02.00:StatementOfDistribution/bafpr1.02.04:Income.BeneficiaryShare.Amount[@contextRef='" + report.RPPrimId + "']",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.PTR.430144" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "PTR158", Value = "PTR158" });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "PTR61", Value = report.PTR61.GetValueOrDefault().ToString() });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "PTR65", Value = report.PTR65.GetValueOrDefault().ToString() });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "PTR66", Value = report.PTR66.GetValueOrDefault().ToString() });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "PTR68", Value = report.PTR68.GetValueOrDefault().ToString() });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.PTR.430144

            #region VR.ATO.PTR.430145

            /*  VR.ATO.PTR.430145
            Net Australian income/loss must equal total Distributed PP and non-PP income
    
            Legacy Rule Format:
            IF [PTR97] <> NULL AND [PTR97] <> SUM([PTR158]) + SUM([PTR159]) + SUM([PTR324])
                RETURN VALIDATION MESSAGE
            ENDIF
            
            [PTR97] = PTR:RP.JAUS:bafpr1.02.23:Income.Net.Amount
            [PTR158] = PTR:RP.Prim:bafpr1.02.04:Income.BeneficiaryShare.Amount IN TUPLE(ptr.0006.lodge.req.02.00:StatementOfDistribution)
            [PTR159] = PTR:RP.NonPrim:bafpr1.02.04:Income.BeneficiaryShare.Amount IN TUPLE(ptr.0006.lodge.req.02.00:StatementOfDistribution)
            [PTR324] = PTR:RP:bafpr1.02.09:Income.DistributionFranked.Amount IN TUPLE(ptr.0006.lodge.req.02.00:StatementOfDistribution)

            Technical Business Rule Format:
            ^PTR97 <> NULL AND ^PTR97 <> Sum(^PTR158) + Sum(^PTR159) + Sum(^PTR324)
    
            Data Elements:
    
            RP.JAUS:^PTR97 = bafpr1.02.23:Income.Net.Amount
    
            RP.Prim:^PTR158 = bafpr1.02.04:Income.BeneficiaryShare.Amount
    
            RP.NonPrim:^PTR159 = bafpr1.02.04:Income.BeneficiaryShare.Amount
    
            RP:^PTR324 = bafpr1.02.09:Income.DistributionFranked.Amount
            */
            assertion = (report.PTR97 != null && report.PTR97.GetValueOrDefault() != (report.StatementOfDistributionCollection == null ? 0 : report.StatementOfDistributionCollection.Sum(f => f.PTR158.GetValueOrDefault())) + (report.StatementOfDistributionCollection == null ? 0 : report.StatementOfDistributionCollection.Sum(f => f.PTR159.GetValueOrDefault())) + (report.StatementOfDistributionCollection == null ? 0 : report.StatementOfDistributionCollection.Sum(f => f.PTR324.GetValueOrDefault())));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.PTR.430706",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Net Australian income/loss must equal total Distributed PP, non-PP income and Franked distributions",
                    LongDescription = @"The Net Australian income/loss amount must equal the sum of Share of income - Primary production plus Share of income - Non-primary production in all Statements of distribution plus Franked distributions",
                    Location = "/xbrli:xbrl/bafpr1.02.23:Income.Net.Amount[@contextRef='" + report.RPJAUSId + "']",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.PTR.430145" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "PTR97", Value = report.PTR97.GetValueOrDefault().ToString() });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "PTR158", Value = "PTR158" });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "PTR159", Value = "PTR159" });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "PTR324", Value = "PTR324" });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.PTR.430145

            #region VR.ATO.PTR.430151

            /*  VR.ATO.PTR.430151
            Tax withheld where ABN not quoted must equal distributed amounts
    
            Legacy Rule Format:
            IF SUM([PTR160]) <> [PTR63] + [PTR73] 
                RETURN VALIDATION MESSAGE
            ENDIF
            
            [PTR63] = PTR:RP.JAUS:rvctc2.02.00:IncomeTax.PayAsYouGoWithholding.CreditForTaxWithheldWhereABNNotQuoted.Amount
            [PTR73] = PTR:RP.JAUS:rvctc2.02.00:IncomeTax.PayAsYouGoWithholding.CreditForTaxWithheldWhereABNNotQuotedShareReceivedIndirectly.Amount
            [PTR160] = PTR:RP:rvctc2.02.00:IncomeTax.PayAsYouGoWithholding.CreditForTaxWithheldWhereABNNotQuoted.Amount IN TUPLE(ptr.0006.lodge.req.02.00:StatementOfDistribution)

            Technical Business Rule Format:
            Sum(^PTR160) <> ^PTR63 + ^PTR73
    
            Data Elements:
    
            RP.JAUS:^PTR63 = rvctc2.02.00:IncomeTax.PayAsYouGoWithholding.CreditForTaxWithheldWhereABNNotQuoted.Amount
    
            RP.JAUS:^PTR73 = rvctc2.02.00:IncomeTax.PayAsYouGoWithholding.CreditForTaxWithheldWhereABNNotQuotedShareReceivedIndirectly.Amount
    
            RP:^PTR160 = rvctc2.02.00:IncomeTax.PayAsYouGoWithholding.CreditForTaxWithheldWhereABNNotQuoted.Amount
            */
            assertion = ((report.StatementOfDistributionCollection == null ? 0 : report.StatementOfDistributionCollection.Sum(f => f.PTR160.GetValueOrDefault())) != report.PTR63.GetValueOrDefault() + report.PTR73.GetValueOrDefault());
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.PTR.430151",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Tax withheld where ABN not quoted must equal distributed amounts",
                    LongDescription = @"The total amount of Credit for tax withheld where ABN not quoted in the Statements of distribution, must equal the sum of Tax withheld where ABN not quoted, and Share of credit for tax withheld where ABN not quoted",
                    Location = "/xbrli:xbrl/rvctc2.02.00:IncomeTax.PayAsYouGoWithholding.CreditForTaxWithheldWhereABNNotQuoted.Amount[@contextRef='" + report.RPJAUSId + "']",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.PTR.430151" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "PTR160", Value = "PTR160" });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "PTR63", Value = report.PTR63.GetValueOrDefault().ToString() });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "PTR73", Value = report.PTR73.GetValueOrDefault().ToString() });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.PTR.430151

            #region VR.ATO.PTR.430153

            /*  VR.ATO.PTR.430153
            Franking credit amounts must equal distributed amounts
    
            Legacy Rule Format:
            IF SUM([PTR161]) <> [PTR74] + [PTR86] 
                RETURN VALIDATION MESSAGE
            ENDIF
            
            [PTR74] = PTR:RP.JAUS:rvctc2.02.00:IncomeTax.FrankingCredits.FrankingCreditShareReceivedIndirectly.Amount
            [PTR86] = PTR:RP.JAUS:rvctc1.02.03:IncomeTax.FrankingCredits.Amount
            [PTR161] = PTR:RP:rvctc2.02.00:IncomeTax.FrankingCredits.ReceivedFromAustralianCompanies.Amount IN TUPLE(ptr.0006.lodge.req.02.00:StatementOfDistribution)

            Technical Business Rule Format:
            Sum(^PTR161) <> ^PTR74 + ^PTR86
    
            Data Elements:
    
            RP.JAUS:^PTR74 = rvctc2.02.00:IncomeTax.FrankingCredits.FrankingCreditShareReceivedIndirectly.Amount
    
            RP.JAUS:^PTR86 = rvctc1.02.03:IncomeTax.FrankingCredits.Amount
    
            RP:^PTR161 = rvctc2.02.00:IncomeTax.FrankingCredits.ReceivedFromAustralianCompanies.Amount
            */
            assertion = ((report.StatementOfDistributionCollection == null ? 0 : report.StatementOfDistributionCollection.Sum(f => f.PTR161.GetValueOrDefault())) != report.PTR74.GetValueOrDefault() + report.PTR86.GetValueOrDefault());
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.PTR.431007",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Non-PP - Deductions relating to distribution amounts can not be more than zero",
                    LongDescription = @"If there is an amount present at Non-primary production- Deductions relating to amounts shown at Distribution from partnerships less foreign income or Non-primary production - Share of net income from trusts less capital gains, foreign income and franked distributions there must be an amount at either Non-primary production - Distribution from partnerships less foreign income or Non-primary production - Share of net income from trusts less capital gains, foreign income and franked distributions",
                    Location = "/xbrli:xbrl/rvctc2.02.00:IncomeTax.FrankingCredits.FrankingCreditShareReceivedIndirectly.Amount",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.PTR.430153" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "PTR161", Value = "PTR161" });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "PTR74", Value = report.PTR74.GetValueOrDefault().ToString() });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "PTR86", Value = report.PTR86.GetValueOrDefault().ToString() });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.PTR.430153

            #region VR.ATO.PTR.430155

            /*  VR.ATO.PTR.430155
            TFN amounts withheld must equal distributed amounts
    
            Legacy Rule Format:
            IF SUM([PTR162]) <> [PTR75] + [PTR83] + [PTR87] 
                RETURN VALIDATION MESSAGE
            ENDIF
            
            [PTR75] = PTR:RP.JAUS:rvctc2.02.00:IncomeTax.PayAsYouGoWithholding.CreditForAmountsWithheldFromInterestAndDividendsWhereTFNNotQuotedShareReceivedIndirectly.Amount
            [PTR83] = PTR:RP.JAUS:rvctc1.02.16:IncomeTax.TFNAmountsWithheldFromGrossInterest.Amount
            [PTR87] = PTR:RP.JAUS:rvctc1.02.00:IncomeTax.DividendsTFNAmountsWithheld.Amount
            [PTR162] = PTR:RP:rvctc2.02.04:IncomeTax.PayAsYouGoWithholding.CreditForAmountsWithheldWhereTFNNotQuoted.Amount IN TUPLE(ptr.0006.lodge.req.02.00:StatementOfDistribution)

            Technical Business Rule Format:
            Sum(^PTR162) <> ^PTR75 + ^PTR83 + ^PTR87
    
            Data Elements:
    
            RP.JAUS:^PTR75 = rvctc2.02.00:IncomeTax.PayAsYouGoWithholding.CreditForAmountsWithheldFromInterestAndDividendsWhereTFNNotQuotedShareReceivedIndirectly.Amount
    
            RP.JAUS:^PTR83 = rvctc1.02.16:IncomeTax.TFNAmountsWithheldFromGrossInterest.Amount
    
            RP.JAUS:^PTR87 = rvctc1.02.00:IncomeTax.DividendsTFNAmountsWithheld.Amount
    
            RP:^PTR162 = rvctc2.02.04:IncomeTax.PayAsYouGoWithholding.CreditForAmountsWithheldWhereTFNNotQuoted.Amount
            */
            assertion = ((report.StatementOfDistributionCollection == null ? 0 : report.StatementOfDistributionCollection.Sum(f => f.PTR162.GetValueOrDefault())) != report.PTR75.GetValueOrDefault() + report.PTR83.GetValueOrDefault() + report.PTR87.GetValueOrDefault());
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.PTR.430155",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"TFN amounts withheld must equal distributed amounts",
                    LongDescription = @"The sum of TFN amounts withheld in the Statements of distribution must equal the sum of Share of credit for TFN amounts withheld from interest, dividends and unit trust distributions, TFN amounts withheld from gross interest, and TFN amounts withheld from dividends",
                    Location = "/xbrli:xbrl/rvctc2.02.00:IncomeTax.PayAsYouGoWithholding.CreditForAmountsWithheldFromInterestAndDividendsWhereTFNNotQuotedShareReceivedIndirectly.Amount",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.PTR.430155" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "PTR162", Value = "PTR162" });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "PTR75", Value = report.PTR75.GetValueOrDefault().ToString() });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "PTR83", Value = report.PTR83.GetValueOrDefault().ToString() });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "PTR87", Value = report.PTR87.GetValueOrDefault().ToString() });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.PTR.430155

            #region VR.ATO.PTR.430158

            /*  VR.ATO.PTR.430158
            Credit for tax withheld - foreign resident withholding (excluding capital gains) amounts must equal distributed amounts
    
            Legacy Rule Format:
            IF SUM([PTR351]) <> [PTR350] + [PTR76] 
                RETURN VALIDATION MESSAGE
            ENDIF
            
            [PTR76] = PTR:RP.JAUS:rvctc4.02.00:InternationalDealings.CreditForTaxWithheldFromForeignResidentWithholdingShareReceivedIndirectly.Amount
            [PTR350] = PTR:RP.JAUS:rvctc2.02.13:IncomeTax.PayAsYouGoWithholding.CreditForAmountsWithheldFromForeignResidents.Amount
            [PTR351] = PTR:RP:rvctc2.02.13:IncomeTax.PayAsYouGoWithholding.CreditForAmountsWithheldFromForeignResidents.Amount IN TUPLE(ptr.0006.lodge.req.02.00:StatementOfDistribution)

            Technical Business Rule Format:
            Sum(^PTR351) <> ^PTR350 + ^PTR76
    
            Data Elements:
    
            RP.JAUS:^PTR350 = rvctc2.02.13:IncomeTax.PayAsYouGoWithholding.CreditForAmountsWithheldFromForeignResidents.Amount
    
            RP.JAUS:^PTR76 = rvctc4.02.00:InternationalDealings.CreditForTaxWithheldFromForeignResidentWithholdingShareReceivedIndirectly.Amount
    
            RP:^PTR351 = rvctc2.02.13:IncomeTax.PayAsYouGoWithholding.CreditForAmountsWithheldFromForeignResidents.Amount
            */
            assertion = ((report.StatementOfDistributionCollection == null ? 0 : report.StatementOfDistributionCollection.Sum(f => f.PTR351.GetValueOrDefault())) != report.PTR350.GetValueOrDefault() + report.PTR76.GetValueOrDefault());
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.PTR.430158",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Credit for tax withheld - foreign resident withholding (excluding capital gains) amounts must equal distributed amounts",
                    LongDescription = @"The total of Credit for tax withheld - foreign resident withholding (excluding capital gains) in the Statements of distribution, must equal the sum of Credit for tax withheld - foreign resident withholding (excluding capital gains), plus Share of credit for tax withheld from foreign resident withholding (excluding capital gains)",
                    Location = "/xbrli:xbrl/rvctc2.02.13:IncomeTax.PayAsYouGoWithholding.CreditForAmountsWithheldFromForeignResidents.Amount[@contextRef='" + report.RPJAUSId + "']",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.PTR.430158" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "PTR351", Value = "PTR351" });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "PTR350", Value = report.PTR350.GetValueOrDefault().ToString() });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "PTR76", Value = report.PTR76.GetValueOrDefault().ToString() });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.PTR.430158

            #region VR.ATO.PTR.430166

            /*  VR.ATO.PTR.430166
            National rental affordability scheme tax offset amounts
    
            Legacy Rule Format:
            IF [PTR248] <> SUM([PTR249]) 
                RETURN VALIDATION MESSAGE
            ENDIF
            
            [PTR248] = PTR:RP:gfati.02.01:TaxOffsetClaim.NationalRentalAffordabilitySchemeEntitlement.Amount
            [PTR249] = PTR:RP:gfati.02.01:TaxOffsetClaim.NationalRentalAffordabilitySchemeEntitlement.Amount IN TUPLE(ptr.0006.lodge.req.02.00:StatementOfDistribution)

            Technical Business Rule Format:
            ^PTR248 <> Sum(^PTR249)
    
            Data Elements:
    
            RP:^PTR248 = gfati.02.01:TaxOffsetClaim.NationalRentalAffordabilitySchemeEntitlement.Amount
    
            RP:^PTR249 = gfati.02.01:TaxOffsetClaim.NationalRentalAffordabilitySchemeEntitlement.Amount
            */
            assertion = (report.PTR248.GetValueOrDefault() != (report.StatementOfDistributionCollection == null ? 0 : report.StatementOfDistributionCollection.Sum(f => f.PTR249.GetValueOrDefault())));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.PTR.430166",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Total National rental affordability scheme tax offset amount must equal sum of distributed amounts",
                    LongDescription = @"The National rental affordability scheme tax offset entitlement amount must equal the sum of the Share of National rental affordability scheme tax offset amounts in the Statements of distribution",
                    Location = "/xbrli:xbrl/gfati.02.01:TaxOffsetClaim.NationalRentalAffordabilitySchemeEntitlement.Amount",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.PTR.430166" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "PTR248", Value = report.PTR248.GetValueOrDefault().ToString() });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "PTR249", Value = "PTR249" });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.PTR.430166

            #region VR.ATO.PTR.430171

            /*  VR.ATO.PTR.430171
            Net rental income must equal the sum of all partners' Share of net rental income
    
            Legacy Rule Format:
            IF [PTR255] <> SUM([PTR257])
                RETURN VALIDATION MESSAGE
            ENDIF
            
            [PTR255] = PTR:RP:bafpr1.02.04:Income.RentalIncomeOrLossNet.Amount
            [PTR257] = PTR:RP:bafpr1.02.04:Income.RentalIncomeOrLossNet.Amount IN TUPLE(ptr.0006.lodge.req.02.00:StatementOfDistribution)

            Technical Business Rule Format:
            ^PTR255 <> Sum(^PTR257)
    
            Data Elements:
    
            RP:^PTR255 = bafpr1.02.04:Income.RentalIncomeOrLossNet.Amount
    
            RP:^PTR257 = bafpr1.02.04:Income.RentalIncomeOrLossNet.Amount
            */
            assertion = (report.PTR255.GetValueOrDefault() != (report.StatementOfDistributionCollection == null ? 0 : report.StatementOfDistributionCollection.Sum(f => f.PTR257.GetValueOrDefault())));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.PTR.430171",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Net rental income must equal the sum of all partners' Share of net rental income",
                    LongDescription = @"The amount at Net rental property income or loss must equal the sum of all amounts recorded in the Statement of distribution - Share of net rental property income or loss",
                    Location = "/xbrli:xbrl/bafpr1.02.04:Income.RentalIncomeOrLossNet.Amount",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.PTR.430171" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "PTR255", Value = report.PTR255.GetValueOrDefault().ToString() });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "PTR257", Value = "PTR257" });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.PTR.430171

            #region VR.ATO.PTR.430172

            /*  VR.ATO.PTR.430172
            Net financial investment income must equal the sum of all partners' Share of net financial income
    
            Legacy Rule Format:
            IF ([PTR254] <> SUM([PTR256]))
                RETURN VALIDATION MESSAGE
            ENDIF
            
            [PTR254] = PTR:RP:bafpr2.02.04:Income.Investment.FinancialIncomeOrLossNet.Amount
            [PTR256] = PTR:RP:bafpr2.02.04:Income.Investment.FinancialIncomeOrLossNet.Amount IN TUPLE(ptr.0006.lodge.req.02.00:StatementOfDistribution)

            Technical Business Rule Format:
            (^PTR254 <> Sum(^PTR256))
    
            Data Elements:
    
            RP:^PTR254 = bafpr2.02.04:Income.Investment.FinancialIncomeOrLossNet.Amount
    
            RP:^PTR256 = bafpr2.02.04:Income.Investment.FinancialIncomeOrLossNet.Amount
            */
            assertion = (report.PTR254.GetValueOrDefault() != (report.StatementOfDistributionCollection == null ? 0 : report.StatementOfDistributionCollection.Sum(f => f.PTR256.GetValueOrDefault())));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.PTR.430172",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Net financial investment income must equal the sum of all partners' Share of net financial income",
                    LongDescription = @"The amount at Net financial investment income or loss must equal the sum of all amounts recorded in the Statement of distribution - Share of net financial investment income or loss",
                    Location = "/xbrli:xbrl/bafpr2.02.04:Income.Investment.FinancialIncomeOrLossNet.Amount",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.PTR.430172" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "PTR254", Value = report.PTR254.GetValueOrDefault().ToString() });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "PTR256", Value = "PTR256" });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.PTR.430172

            #region VR.ATO.PTR.430177

            /*  VR.ATO.PTR.430177
            Interposed Entity Election Status - Specified income year must be in range 1995 to the current income year
    
            Legacy Rule Format:
            IF [PTR26] <> NULL AND ([PTR26] < 1995 OR [PTR26] > [PTR1])
               RETURN VALIDATION MESSAGE
            ENDIF
            
            [PTR1] = PTR:RP:pyin.02.37:Report.TargetFinancial.Year
            [PTR26] = PTR:RP:rvctc3.02.00:Elections.InterposedEntityElectionStatus.Year

            Technical Business Rule Format:
            ^PTR26 <> NULL AND (^PTR26 < 1995 OR ^PTR26 > ^PTR1)
    
            Data Elements:
    
            RP:^PTR26 = rvctc3.02.00:Elections.InterposedEntityElectionStatus.Year
    
            RP:^PTR1 = pyin.02.37:Report.TargetFinancial.Year
            */
            assertion = (report.PTR26 != null && (report.PTR26.GetValueOrDefault() < 1995 || report.PTR26.GetValueOrDefault() > report.PTR1.GetValueOrDefault()));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.430702",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Interposed Entity Election Status - Specified income year must be in range 1995 to current year",
                    Location = "/xbrli:xbrl/rvctc3.02.00:Elections.InterposedEntityElectionStatus.Year",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.PTR.430177" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "PTR26", Value = report.PTR26.GetValueOrDefault().ToString() });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "PTR1", Value = report.PTR1.GetValueOrDefault().ToString() });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.PTR.430177

            #region VR.ATO.PTR.430194

            /*  VR.ATO.PTR.430194
            All current assets must not exceed Total assets
    
            Legacy Rule Format:
            IF  [PTR112] < [PTR111]
               RETURN VALIDATION MESSAGE
            ENDIF
            
            [PTR111] = PTR:RP.Closing(Instant):bafpo1.02.00:Assets.CurrentTotal.Amount
            [PTR112] = PTR:RP.Closing(Instant):bafpo1.02.13:Assets.Total.Amount

            Technical Business Rule Format:
            ^PTR112 < ^PTR111
    
            Data Elements:
    
            RP.Closing(Instant):^PTR111 = bafpo1.02.00:Assets.CurrentTotal.Amount
    
            RP.Closing(Instant):^PTR112 = bafpo1.02.13:Assets.Total.Amount
            */
            assertion = (report.PTR112.GetValueOrDefault() < report.PTR111.GetValueOrDefault());
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.430194",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"All current assets must not exceed Total assets",
                    Location = "/xbrli:xbrl/bafpo1.02.00:Assets.CurrentTotal.Amount",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.PTR.430194" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "PTR112", Value = report.PTR112.GetValueOrDefault().ToString() });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "PTR111", Value = report.PTR111.GetValueOrDefault().ToString() });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.PTR.430194

            #region VR.ATO.PTR.430195

            /*  VR.ATO.PTR.430195
            All current liabilities must not exceed Total liabilities
    
            Legacy Rule Format:
            IF [PTR114] < [PTR113]
               RETURN VALIDATION MESSAGE
            ENDIF
            
            [PTR113] = PTR:RP.Closing(Instant):bafpo1.02.00:Liabilities.CurrentTotal.Amount
            [PTR114] = PTR:RP.Closing(Instant):bafpo1.02.13:Liabilities.Total.Amount

            Technical Business Rule Format:
            ^PTR114 < ^PTR113
    
            Data Elements:
    
            RP.Closing(Instant):^PTR113 = bafpo1.02.00:Liabilities.CurrentTotal.Amount
    
            RP.Closing(Instant):^PTR114 = bafpo1.02.13:Liabilities.Total.Amount
            */
            assertion = (report.PTR114.GetValueOrDefault() < report.PTR113.GetValueOrDefault());
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.430195",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"All current liabilities must not exceed Total liabilities",
                    Location = "/xbrli:xbrl/bafpo1.02.00:Liabilities.CurrentTotal.Amount",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.PTR.430195" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "PTR114", Value = report.PTR114.GetValueOrDefault().ToString() });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "PTR113", Value = report.PTR113.GetValueOrDefault().ToString() });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.PTR.430195

            #region VR.ATO.PTR.430197

            /*  VR.ATO.PTR.430197
            Closing stock amount must be present if Code is present
    
            Legacy Rule Format:
            IF ([PTR124] <> NULL) AND ([PTR123] = NULL)
               RETURN VALIDATION MESSAGE
            ENDIF
            
            [PTR123] = PTR:RP.Closing(Instant):bafpo7.02.04:Assets.Inventories.Total.Amount
            [PTR124] = PTR:RP.Closing(Instant):bafpo7.02.18:Assets.Inventories.ValuationMethod.Code

            Technical Business Rule Format:
            (^PTR124 <> NULL) AND (^PTR123 = NULL)
    
            Data Elements:
    
            RP.Closing(Instant):^PTR123 = bafpo7.02.04:Assets.Inventories.Total.Amount
    
            RP.Closing(Instant):^PTR124 = bafpo7.02.18:Assets.Inventories.ValuationMethod.Code
            */
            assertion = (report.PTR124 != null && report.PTR123 == null);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.430197",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Closing stock amount must be present if Code is present",
                    Location = "/xbrli:xbrl/bafpo7.02.04:Assets.Inventories.Total.Amount[@contextRef='" + report.RPClosingInstantId + "']",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.PTR.430197" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "PTR124", Value = report.PTR124 });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "PTR123", Value = report.PTR123.GetValueOrDefault().ToString() });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.PTR.430197

            #region VR.ATO.PTR.430198

            /*  VR.ATO.PTR.430198
            Closing stock action code must be present if Closing stock amount is present
    
            Legacy Rule Format:
            IF ([PTR124] = NULL) AND ([PTR123] >0)
               RETURN VALIDATION MESSAGE
            ENDIF
            
            [PTR123] = PTR:RP.Closing(Instant):bafpo7.02.04:Assets.Inventories.Total.Amount
            [PTR124] = PTR:RP.Closing(Instant):bafpo7.02.18:Assets.Inventories.ValuationMethod.Code

            Technical Business Rule Format:
            (^PTR124 = NULL) AND (^PTR123 >0)
    
            Data Elements:
    
            RP.Closing(Instant):^PTR124 = bafpo7.02.18:Assets.Inventories.ValuationMethod.Code
    
            RP.Closing(Instant):^PTR123 = bafpo7.02.04:Assets.Inventories.Total.Amount
            */
            assertion = (report.PTR124 == null && report.PTR123.GetValueOrDefault() > 0);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.430198",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Closing stock action code must be present if Closing stock amount is present",
                    Location = "/xbrli:xbrl/bafpo7.02.18:Assets.Inventories.ValuationMethod.Code",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.PTR.430198" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "PTR124", Value = report.PTR124 });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "PTR123", Value = report.PTR123.GetValueOrDefault().ToString() });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.PTR.430198

            #region VR.ATO.PTR.430199

            /*  VR.ATO.PTR.430199
            Trade debtors amount must not exceed All current assets amount
    
            Legacy Rule Format:
            IF ([PTR111] < [PTR125])
                RETURN VALIDATION MESSAGE
            ENDIF
            
            [PTR111] = PTR:RP.Closing(Instant):bafpo1.02.00:Assets.CurrentTotal.Amount
            [PTR125] = PTR:RP.Closing(Instant):bafpo1.02.13:Assets.TradeandReceivablesOther.Amount

            Technical Business Rule Format:
            ^PTR111 < ^PTR125
    
            Data Elements:
    
            RP.Closing(Instant):^PTR125 = bafpo1.02.13:Assets.TradeandReceivablesOther.Amount
    
            RP.Closing(Instant):^PTR111 = bafpo1.02.00:Assets.CurrentTotal.Amount
            */
            assertion = (report.PTR111.GetValueOrDefault() < report.PTR125.GetValueOrDefault());
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.430199",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Trade debtors amount must not exceed All current assets amount",
                    Location = "/xbrli:xbrl/bafpo1.02.13:Assets.TradeandReceivablesOther.Amount",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.PTR.430199" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "PTR111", Value = report.PTR111.GetValueOrDefault().ToString() });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "PTR125", Value = report.PTR125.GetValueOrDefault().ToString() });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.PTR.430199

            #region VR.ATO.PTR.430200

            /*  VR.ATO.PTR.430200
            Trade creditors cannot exceed all current liabilities in Financial and other information
    
            Legacy Rule Format:
            IF [PTR113] < [PTR126]
               RETURN VALIDATION MESSAGE
            ENDIF
            
            [PTR113] = PTR:RP.Closing(Instant):bafpo1.02.00:Liabilities.CurrentTotal.Amount
            [PTR126] = PTR:RP.Closing(Instant):bafpo1.02.13:Liabilities.TradeAndOtherPayablesTotal.Amount

            Technical Business Rule Format:
            ^PTR113 < ^PTR126
    
            Data Elements:
    
            RP.Closing(Instant):^PTR126 = bafpo1.02.13:Liabilities.TradeAndOtherPayablesTotal.Amount
    
            RP.Closing(Instant):^PTR113 = bafpo1.02.00:Liabilities.CurrentTotal.Amount
            */
            assertion = (report.PTR113.GetValueOrDefault() < report.PTR126.GetValueOrDefault());
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.430200",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Trade creditors cannot exceed all current liabilities in Financial and other information",
                    Location = "/xbrli:xbrl/bafpo1.02.13:Liabilities.TradeAndOtherPayablesTotal.Amount",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.PTR.430200" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "PTR113", Value = report.PTR113.GetValueOrDefault().ToString() });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "PTR126", Value = report.PTR126.GetValueOrDefault().ToString() });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.PTR.430200

            #region VR.ATO.PTR.430207

            /*  VR.ATO.PTR.430207
            Industry code must be present
    
            Legacy Rule Format:
            IF ([PTR35] <> NULL OR [PTR36] <> NULL OR [PTR38] <> NULL OR [PTR39] <> NULL OR [PTR41] <> NULL OR [PTR43] <> NULL OR [PTR44] <> NULL) AND [PTR31] = NULLORBLANK 
                RETURN VALIDATION MESSAGE
            ENDIF
            
            [PTR31] = PTR:RP:pyde.02.30:OrganisationDetails.OrganisationIndustry2006Extended.Code
            [PTR35] = PTR:RP.JAUS.Prim:lrla.02.00:Remuneration.ABNNotQuotedPaymentGross.Amount
            [PTR36] = PTR:RP.JAUS.NonPrim:lrla.02.00:Remuneration.ABNNotQuotedPaymentGross.Amount
            [PTR38] = PTR:RP.JAUS.NonPrim:lrla.02.00:Remuneration.PaymentToForeignResidentGross.Amount
            [PTR39] = PTR:RP.JAUS.Prim:gfagg.02.00:GovernmentFunding.GovernmentIndustryPaymentsAssessable.Amount
            [PTR41] = PTR:RP.JAUS.NonPrim:gfagg.02.00:GovernmentFunding.GovernmentIndustryPaymentsAssessable.Amount
            [PTR43] = PTR:RP.JAUS.Prim:bafpr1.02.04:Income.Other.Amount
            [PTR44] = PTR:RP.JAUS.NonPrim:bafpr1.02.04:Income.Other.Amount

            Technical Business Rule Format:
            (^PTR35 <> NULL OR ^PTR36 <> NULL OR ^PTR38 <> NULL OR ^PTR39 <> NULL OR ^PTR41 <> NULL OR ^PTR43 <> NULL OR ^PTR44 <> NULL) AND ^PTR31 = BLANK
    
            Data Elements:
    
            RP:^PTR31 = pyde.02.30:OrganisationDetails.OrganisationIndustry2006Extended.Code
    
            RP.JAUS.Prim:^PTR35 = lrla.02.00:Remuneration.ABNNotQuotedPaymentGross.Amount
    
            RP.JAUS.NonPrim:^PTR36 = lrla.02.00:Remuneration.ABNNotQuotedPaymentGross.Amount
    
            RP.JAUS.NonPrim:^PTR38 = lrla.02.00:Remuneration.PaymentToForeignResidentGross.Amount
    
            RP.JAUS.Prim:^PTR39 = gfagg.02.00:GovernmentFunding.GovernmentIndustryPaymentsAssessable.Amount
    
            RP.JAUS.NonPrim:^PTR41 = gfagg.02.00:GovernmentFunding.GovernmentIndustryPaymentsAssessable.Amount
    
            RP.JAUS.Prim:^PTR43 = bafpr1.02.04:Income.Other.Amount
    
            RP.JAUS.NonPrim:^PTR44 = bafpr1.02.04:Income.Other.Amount
            */
            assertion = ((report.PTR35 != null || report.PTR36 != null || report.PTR38 != null || report.PTR39 != null || report.PTR41 != null || report.PTR43 != null || report.PTR44 != null) && string.IsNullOrWhiteSpace(report.PTR31) == true);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.430207",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"If  business income (PP or non-PP) is present, an Industry code (ANZSIC) must be present",
                    Location = "/xbrli:xbrl/pyde.02.30:OrganisationDetails.OrganisationIndustry2006Extended.Code",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.PTR.430207" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "PTR35", Value = report.PTR35.GetValueOrDefault().ToString() });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "PTR36", Value = report.PTR36.GetValueOrDefault().ToString() });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "PTR38", Value = report.PTR38.GetValueOrDefault().ToString() });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "PTR39", Value = report.PTR39.GetValueOrDefault().ToString() });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "PTR41", Value = report.PTR41.GetValueOrDefault().ToString() });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "PTR43", Value = report.PTR43.GetValueOrDefault().ToString() });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "PTR44", Value = report.PTR44.GetValueOrDefault().ToString() });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "PTR31", Value = report.PTR31 });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.PTR.430207

            #region VR.ATO.PTR.430212

            /*  VR.ATO.PTR.430212
            Net rental property income or loss, and shared distributions must be present
    
            Legacy Rule Format:
            IF ([PTR77] + [PTR78] + [PTR79] + [PTR80] > 0) AND ([PTR255] = NULL) AND ((ANY OCCURRENCE OF([PTR257])) = NULL)
                RETURN VALIDATION MESSAGE
            ENDIF
            
            [PTR77] = PTR:RP.JAUS:bafpr2.02.00:Income.Operating.RentalIncomeGross.Amount
            [PTR78] = PTR:RP.JAUS:bafpr1.02.05:Expense.Interest.Amount
            [PTR79] = PTR:RP.JAUS:bafpr1.02.02:Expense.CapitalWorksDeduction.Amount
            [PTR80] = PTR:RP.JAUS:rvctc2.02.00:IncomeTax.Deduction.RentalIncomeDeductionsOtherThanInterestAndCapitalWorks.Amount
            [PTR255] = PTR:RP:bafpr1.02.04:Income.RentalIncomeOrLossNet.Amount
            [PTR257] = PTR:RP:bafpr1.02.04:Income.RentalIncomeOrLossNet.Amount IN TUPLE(ptr.0006.lodge.req.02.00:StatementOfDistribution)

            Technical Business Rule Format:
            (^PTR77 + ^PTR78 + ^PTR79 + ^PTR80 > 0) AND (^PTR255 = NULL) AND (AnyOccurrence(^PTR257 , ^PTR257 = NULL))
    
            Data Elements:
    
            RP:^PTR255 = bafpr1.02.04:Income.RentalIncomeOrLossNet.Amount
    
            RP.JAUS:^PTR77 = bafpr2.02.00:Income.Operating.RentalIncomeGross.Amount
    
            RP.JAUS:^PTR78 = bafpr1.02.05:Expense.Interest.Amount
    
            RP.JAUS:^PTR79 = bafpr1.02.02:Expense.CapitalWorksDeduction.Amount
    
            RP.JAUS:^PTR80 = rvctc2.02.00:IncomeTax.Deduction.RentalIncomeDeductionsOtherThanInterestAndCapitalWorks.Amount
    
            RP:^PTR257 = bafpr1.02.04:Income.RentalIncomeOrLossNet.Amount
            */
            assertion = (report.PTR77.GetValueOrDefault() + report.PTR78.GetValueOrDefault() + report.PTR79.GetValueOrDefault() + report.PTR80.GetValueOrDefault() > 0 && report.PTR255 == null && (report.StatementOfDistributionCollection == null ? false : report.StatementOfDistributionCollection.Any(PTR257Repeat => PTR257Repeat.PTR257 == null)));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.PTR.430212",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Net rental property income or loss, and shared distributions must be present",
                    LongDescription = @"If Rent amounts greater than zero (including Gross rent and deductions) are present, then Net rental property income or loss must be present, and Share of net rental property income or loss must be present in each partner's Statement of distribution",
                    Location = "/xbrli:xbrl/bafpr1.02.04:Income.RentalIncomeOrLossNet.Amount",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.PTR.430212" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "PTR77", Value = report.PTR77.GetValueOrDefault().ToString() });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "PTR78", Value = report.PTR78.GetValueOrDefault().ToString() });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "PTR79", Value = report.PTR79.GetValueOrDefault().ToString() });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "PTR80", Value = report.PTR80.GetValueOrDefault().ToString() });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "PTR255", Value = report.PTR255.GetValueOrDefault().ToString() });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "PTR257", Value = "PTR257" });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.PTR.430212

            #region VR.ATO.PTR.430213

            /*  VR.ATO.PTR.430213
            Net financial investment income or loss, and shared distributions must be present
    
            Legacy Rule Format:
            IF (([PTR81] > 0 OR [PTR84] > 0 OR [PTR85] > 0 OR [PTR86] > 0 OR (([PTR82] = NULL OR [PTR82] = 0) AND [PTR90] > 0) OR [PTR91] > 0) AND [PTR254] = NULL AND ANY OCCURRENCE OF([PTR256]) = NULL)
                RETURN VALIDATION MESSAGE
            ENDIF
            
            [PTR81] = PTR:RP.JAUS:bafpr1.02.23:Income.ForestryManagedInvestmentScheme.Amount
            [PTR82] = PTR:RP.JAUS:bafpr2.02.02:Income.Interest.Gross.Amount
            [PTR84] = PTR:RP.JAUS:bafpr1.02.05:Income.DividendsUnfranked.Amount
            [PTR85] = PTR:RP.JAUS:bafpr1.02.00:Income.DividendsFranked.Amount
            [PTR86] = PTR:RP.JAUS:rvctc1.02.03:IncomeTax.FrankingCredits.Amount
            [PTR90] = PTR:RP.JAUS:bafpr1.02.04:Expense.AustralianInvestmentAllowableDeduction.Amount
            [PTR91] = PTR:RP.JAUS:bafpr1.02.23:Expense.ForestryManagedInvestmentSchemeDeduction.Amount
            [PTR254] = PTR:RP:bafpr2.02.04:Income.Investment.FinancialIncomeOrLossNet.Amount
            [PTR256] = PTR:RP:bafpr2.02.04:Income.Investment.FinancialIncomeOrLossNet.Amount IN TUPLE(ptr.0006.lodge.req.02.00:StatementOfDistribution)

            Technical Business Rule Format:
            ((^PTR81 > 0 OR ^PTR84 > 0 OR ^PTR85 > 0 OR ^PTR86 > 0 OR ((^PTR82 = NULL OR ^PTR82 = 0) AND ^PTR90 > 0) OR ^PTR91 > 0) AND ^PTR254 = NULL AND AnyOccurrence(^PTR256, ^PTR256 = NULL))
    
            Data Elements:
    
            RP:^PTR254 = bafpr2.02.04:Income.Investment.FinancialIncomeOrLossNet.Amount
    
            RP.JAUS:^PTR81 = bafpr1.02.23:Income.ForestryManagedInvestmentScheme.Amount
    
            RP.JAUS:^PTR82 = bafpr2.02.02:Income.Interest.Gross.Amount
    
            RP.JAUS:^PTR84 = bafpr1.02.05:Income.DividendsUnfranked.Amount
    
            RP.JAUS:^PTR85 = bafpr1.02.00:Income.DividendsFranked.Amount
    
            RP.JAUS:^PTR86 = rvctc1.02.03:IncomeTax.FrankingCredits.Amount
    
            RP.JAUS:^PTR90 = bafpr1.02.04:Expense.AustralianInvestmentAllowableDeduction.Amount
    
            RP.JAUS:^PTR91 = bafpr1.02.23:Expense.ForestryManagedInvestmentSchemeDeduction.Amount
    
            RP:^PTR256 = bafpr2.02.04:Income.Investment.FinancialIncomeOrLossNet.Amount
            */
            assertion = ((report.PTR81.GetValueOrDefault() > 0 || report.PTR84.GetValueOrDefault() > 0 || report.PTR85.GetValueOrDefault() > 0 || report.PTR86.GetValueOrDefault() > 0 || (report.PTR82 == null || report.PTR82 == 0) && report.PTR90.GetValueOrDefault() > 0 || report.PTR91.GetValueOrDefault() > 0) && report.PTR254 == null && (report.StatementOfDistributionCollection == null ? false : report.StatementOfDistributionCollection.Any(PTR256Repeat => PTR256Repeat.PTR256 == null)));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.PTR.430213",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Net financial investment income or loss, and shared distributions must be present",
                    LongDescription = @"If an amount greater than zero is present at FMIS income, or Dividend income, or Deductions relating to Australian investment income and Gross interest is not present, then Net financial investment income or loss must be present, and Share of net financial investment income or loss must be present in each partner's Statement of Distribution",
                    Location = "/xbrli:xbrl/bafpr2.02.04:Income.Investment.FinancialIncomeOrLossNet.Amount",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.PTR.430213" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "PTR81", Value = report.PTR81.GetValueOrDefault().ToString() });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "PTR84", Value = report.PTR84.GetValueOrDefault().ToString() });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "PTR85", Value = report.PTR85.GetValueOrDefault().ToString() });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "PTR86", Value = report.PTR86.GetValueOrDefault().ToString() });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "PTR82", Value = report.PTR82.GetValueOrDefault().ToString() });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "PTR90", Value = report.PTR90.GetValueOrDefault().ToString() });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "PTR91", Value = report.PTR91.GetValueOrDefault().ToString() });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "PTR254", Value = report.PTR254.GetValueOrDefault().ToString() });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "PTR256", Value = "PTR256" });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.PTR.430213

            #region VR.ATO.PTR.430216

            /*  VR.ATO.PTR.430216
            Mandatory field not supplied
    
            Legacy Rule Format:
            IF (PTR:RP:pyin.xx.xx:Report.TargetFinancial.Year = NULL)
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            ^PTR1 = NULL
    
            Data Elements:
    
            RP:^PTR1 = pyin.02.37:Report.TargetFinancial.Year
            */
            assertion = (report.PTR1 == null);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.001001",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Mandatory field not supplied",
                    Location = "/xbrli:xbrl/pyin.02.37:Report.TargetFinancial.Year",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.PTR.430216" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "PTR1", Value = report.PTR1.GetValueOrDefault().ToString() });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.PTR.430216

            #region VR.ATO.PTR.430217

            /*  VR.ATO.PTR.430217
            Mandatory field not supplied
    
            Legacy Rule Format:
            IF [PTR29] = NULL
            RETURN VALIDATION MESSAGE
            ENDIF
            
            [PTR29] = PTR:RP:pyin.02.28:Lodgment.FinalReturn.Indicator

            Technical Business Rule Format:
            ^PTR29 = NULL
    
            Data Elements:
    
            RP:^PTR29 = pyin.02.28:Lodgment.FinalReturn.Indicator
            */
            assertion = (report.PTR29 == null);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.001001",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Mandatory field not supplied",
                    Location = "/xbrli:xbrl/pyin.02.28:Lodgment.FinalReturn.Indicator",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.PTR.430217" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "PTR29", Value = GetValueOrEmpty(report.PTR29) });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.PTR.430217

            #region VR.ATO.PTR.430218

            /*  VR.ATO.PTR.430218
            There must be one incidence of an organisation name with a Type - 'MN' and Currency - 'C'
    
            Legacy Rule Format:
            IF COUNT TUPLE (xbrli/Organisationname2.xx.xx:OrganisationNameDetails WHERE TUPLE ELEMENT (pyde.xx.xx:OrganisationNameDetails.OrganisationalNameType.Code = "MN") AND (pyde.xx.xx:OrganisationNameDetails.Currency.Code = "C") AND (pyde.xx.xx:OrganisationNameDetails.OrganisationalName.Text  <> NULLORBLANK)) <> 1
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            ^PTR6 = BLANK
    
            Data Elements:
    
            RP:^PTR6 = pyde.02.00:OrganisationNameDetails.OrganisationalName.Text WHERE [OrganisationNameDetails.OrganisationalNameType.Code='MN'].[OrganisationNameDetails.Currency.Code='C']
            */
            assertion = (string.IsNullOrWhiteSpace(report.PTR6) == true);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.PTR.430218",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Current name of partnership must be present",
                    LongDescription = @"There must be one instance of the current name of the partnership. This name must be submitted with a currency code 'C' and a type of 'MN'",
                    Location = "/xbrli:xbrl/orgname2.02.00:OrganisationNameDetails" + OccurrenceIndex(report.OrganisationNameDetailsCollection_MN_C_RPOccurrenceIndex) + "/pyde.02.00:OrganisationNameDetails.OrganisationalName.Text",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.PTR.430218" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "PTR6", Value = report.PTR6 });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.PTR.430218

            #region VR.ATO.PTR.430219

            /*  VR.ATO.PTR.430219
            Mandatory field not supplied
    
            Legacy Rule Format:
            IF [PTR109] = NULL
               RETURN VALIDATION MESSAGE
            ENDIF
            
            [PTR109] = PTR:RP:rvctc4.02.11:InternationalDealings.TransactionsWithTaxHavenCountries.Indicator

            Technical Business Rule Format:
            ^PTR109 = NULL
    
            Data Elements:
    
            RP:^PTR109 = rvctc4.02.11:InternationalDealings.TransactionsWithTaxHavenCountries.Indicator
            */
            assertion = (report.PTR109 == null);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.001001",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Mandatory field not supplied",
                    Location = "/xbrli:xbrl/rvctc4.02.11:InternationalDealings.TransactionsWithTaxHavenCountries.Indicator",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.PTR.430219" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "PTR109", Value = GetValueOrEmpty(report.PTR109) });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.PTR.430219

            #region VR.ATO.PTR.430221

            /*  VR.ATO.PTR.430221
            Current postal address of the Partnership must be present
    
            Legacy Rule Format:
            IF COUNT TUPLE (xbrli/address2.xx.xx:AddressDetails WHERE TUPLE ELEMENT (pyde.xx.xx:AddressDetails.Usage.Code = "POS") AND (pyde.xx.xx:AddressDetails.Currency.Code = "C") AND (pyde.xx.xx:AddressDetails.Line1.Text <> NULLORBLANK)) <> 1
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            ^PTR9 = BLANK
    
            Data Elements:
    
            RP:^PTR9 = pyde.02.00:AddressDetails.Line1.Text WHERE [AddressDetails.Usage.Code='POS'].[AddressDetails.Currency.Code='C']
            */
            assertion = (string.IsNullOrWhiteSpace(report.PTR9) == true);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.PTR.430221",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Current postal address of the Partnership must be present",
                    LongDescription = @"There must be one instance of the current postal address of the Partnership. This address must be submitted with a Currency Code 'C' and a Usage of 'POS'. Line 1, Suburb/Town/City must be present",
                    Location = "/xbrli:xbrl/address2.02.02:AddressDetails" + OccurrenceIndex(report.AddressDetailsCollection_POS_C_RPOccurrenceIndex) + "/pyde.02.00:AddressDetails.Line1.Text",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.PTR.430221" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "PTR9", Value = report.PTR9 });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.PTR.430221

            #region VR.ATO.PTR.430222

            /*  VR.ATO.PTR.430222
            Mandatory field not supplied
    
            Legacy Rule Format:
            IF [PTR97] = NULL
               RETURN VALIDATION MESSAGE
            ENDIF
            
            [PTR97] = PTR:RP.JAUS:bafpr1.02.23:Income.Net.Amount

            Technical Business Rule Format:
            ^PTR97 = NULL
    
            Data Elements:
    
            RP.JAUS:^PTR97 = bafpr1.02.23:Income.Net.Amount
            */
            assertion = (report.PTR97 == null);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.001001",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Mandatory field not supplied",
                    Location = "/xbrli:xbrl/bafpr1.02.23:Income.Net.Amount[@contextRef='" + report.RPJAUSId + "']",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.PTR.430222" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "PTR97", Value = report.PTR97.GetValueOrDefault().ToString() });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.PTR.430222

            #region VR.ATO.PTR.430223

            /*  VR.ATO.PTR.430223
            Mandatory field not supplied
    
            Legacy Rule Format:
            IF [PTR102] = NULL
               RETURN VALIDATION MESSAGE
            ENDIF
            
            [PTR102] = PTR:RP.JFOREIGN:bafpr3.02.03:Income.InternationalDealings.AttributedForeignIncomeForeignTrustorControlledForeignCompanyorTransferorTrust.Indicator

            Technical Business Rule Format:
            ^PTR102 = NULL
    
            Data Elements:
    
            RP.JFOREIGN:^PTR102 = bafpr3.02.03:Income.InternationalDealings.AttributedForeignIncomeForeignTrustorControlledForeignCompanyorTransferorTrust.Indicator
            */
            assertion = (report.PTR102 == null);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.001001",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Mandatory field not supplied",
                    Location = "/xbrli:xbrl/bafpr3.02.03:Income.InternationalDealings.AttributedForeignIncomeForeignTrustorControlledForeignCompanyorTransferorTrust.Indicator",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.PTR.430223" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "PTR102", Value = GetValueOrEmpty(report.PTR102) });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.PTR.430223

            #region VR.ATO.PTR.430227

            /*  VR.ATO.PTR.430227
            Monetary field must be unsigned, maximum of 11 numeric characters, no decimals
    
            Legacy Rule Format:
            IF [PTR104] <> NULL AND [PTR104] <> MONETARY(U,11,0)
               RETURN VALIDATION MESSAGE
            ENDIF
            
            [PTR104] = PTR:RP.JFOREIGN:bafpr1.02.04:Income.Other.Amount

            Technical Business Rule Format:
            NotMonetary(^PTR104, 'U', 11, 0)
    
            Data Elements:
    
            RP.JFOREIGN:^PTR104 = bafpr1.02.04:Income.Other.Amount
            */
            assertion = NotMonetary(report.PTR104, @"U", 11, 0);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.400011",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Field incorrect format. Exceeds maximum digits or decimals and/or contains illegal characters",
                    LongDescription = @"Field is in incorrect format for a monetary field. Field either exceeds maximum digits or decimals and/or contains illegal characters and/or for a signed field does not have the sign as the first character",
                    Location = "/xbrli:xbrl/bafpr1.02.04:Income.Other.Amount[@contextRef='" + report.RPJFOREIGNId + "']",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.PTR.430227" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "PTR104", Value = report.PTR104.GetValueOrDefault().ToString() });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.PTR.430227

            #region VR.ATO.PTR.430228

            /*  VR.ATO.PTR.430228
            Monetary field must be unsigned, maximum of 11 numeric characters, no decimals
    
            Legacy Rule Format:
            IF [PTR122] <> NULL AND [PTR122] <> MONETARY(U,11,0)
               RETURN VALIDATION MESSAGE
            ENDIF
            
            [PTR122] = PTR:RP:bafpr2.02.02:Expense.Purchases.AndOtherCosts.Amount

            Technical Business Rule Format:
            NotMonetary(^PTR122, 'U', 11, 0)
    
            Data Elements:
    
            RP:^PTR122 = bafpr2.02.02:Expense.Purchases.AndOtherCosts.Amount
            */
            assertion = NotMonetary(report.PTR122, @"U", 11, 0);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.400011",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Field incorrect format. Exceeds maximum digits or decimals and/or contains illegal characters",
                    LongDescription = @"Field is in incorrect format for a monetary field. Field either exceeds maximum digits or decimals and/or contains illegal characters and/or for a signed field does not have the sign as the first character",
                    Location = "/xbrli:xbrl/bafpr2.02.02:Expense.Purchases.AndOtherCosts.Amount",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.PTR.430228" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "PTR122", Value = report.PTR122.GetValueOrDefault().ToString() });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.PTR.430228

            #region VR.ATO.PTR.430233

            /*  VR.ATO.PTR.430233
            Monetary field may be signed, maximum of 11 chars, no decimals
    
            Legacy Rule Format:
            IF [PTR254] <> NULL AND [PTR254] <> MONETARY(S,11,0)
               RETURN VALIDATION MESSAGE
            ENDIF
            
            [PTR254] = PTR:RP:bafpr2.02.04:Income.Investment.FinancialIncomeOrLossNet.Amount

            Technical Business Rule Format:
            NotMonetary(^PTR254, 'S', 11, 0)
    
            Data Elements:
    
            RP:^PTR254 = bafpr2.02.04:Income.Investment.FinancialIncomeOrLossNet.Amount
            */
            assertion = NotMonetary(report.PTR254, @"S", 11, 0);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.400011",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Field incorrect format. Exceeds maximum digits or decimals and/or contains illegal characters",
                    LongDescription = @"Field is in incorrect format for a monetary field. Field either exceeds maximum digits or decimals and/or contains illegal characters and/or for a signed field does not have the sign as the first character",
                    Location = "/xbrli:xbrl/bafpr2.02.04:Income.Investment.FinancialIncomeOrLossNet.Amount",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.PTR.430233" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "PTR254", Value = report.PTR254.GetValueOrDefault().ToString() });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.PTR.430233

            #region VR.ATO.PTR.430234

            /*  VR.ATO.PTR.430234
            Monetary field may be signed, maximum of 11 chars, no decimals
    
            Legacy Rule Format:
            IF [PTR255] <> NULL AND [PTR255] <> MONETARY(S,11,0)
               RETURN VALIDATION MESSAGE
            ENDIF
            
            [PTR255] = PTR:RP:bafpr1.02.04:Income.RentalIncomeOrLossNet.Amount

            Technical Business Rule Format:
            NotMonetary(^PTR255, 'S', 11, 0)
    
            Data Elements:
    
            RP:^PTR255 = bafpr1.02.04:Income.RentalIncomeOrLossNet.Amount
            */
            assertion = NotMonetary(report.PTR255, @"S", 11, 0);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.400011",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Field incorrect format. Exceeds maximum digits or decimals and/or contains illegal characters",
                    LongDescription = @"Field is in incorrect format for a monetary field. Field either exceeds maximum digits or decimals and/or contains illegal characters and/or for a signed field does not have the sign as the first character",
                    Location = "/xbrli:xbrl/bafpr1.02.04:Income.RentalIncomeOrLossNet.Amount",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.PTR.430234" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "PTR255", Value = report.PTR255.GetValueOrDefault().ToString() });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.PTR.430234

            #region VR.ATO.PTR.430240

            /*  VR.ATO.PTR.430240
            Mandatory field not supplied
    
            Legacy Rule Format:
            IF [PTR108] = NULL
            RETURN VALIDATION MESSAGE
            ENDIF
            
            [PTR108] = PTR:RP:rvctc4.02.10:InternationalDealings.RelatedPartiesTransactionsExcessAggregateValue.Indicator

            Technical Business Rule Format:
            ^PTR108 = NULL
    
            Data Elements:
    
            RP:^PTR108 = rvctc4.02.10:InternationalDealings.RelatedPartiesTransactionsExcessAggregateValue.Indicator
            */
            assertion = (report.PTR108 == null);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.001001",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Mandatory field not supplied",
                    Location = "/xbrli:xbrl/rvctc4.02.10:InternationalDealings.RelatedPartiesTransactionsExcessAggregateValue.Indicator",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.PTR.430240" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "PTR108", Value = GetValueOrEmpty(report.PTR108) });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.PTR.430240

            #region VR.ATO.PTR.430242

            /*  VR.ATO.PTR.430242
            Tax Agent contact name must be present
    
            Legacy Rule Format:
            IF CONTEXT(INT) <> NULL AND [PTR242] = NULLORBLANK
               RETURN VALIDATION MESSAGE
            ENDIF
            
            [PTR242] = PTR:INT:pyde.02.00:PersonUnstructuredName.FullName.Text WHERE (pyde.02.05:PersonUnstructuredName.Usage.Code = "Contact") IN TUPLE(prsnunstrcnm1.02.01:PersonUnstructuredName)

            Technical Business Rule Format:
            ^Context <> NULL AND ^PTR242 = BLANK
    
            Data Elements:
    
            INT:^PTR242 = pyde.02.00:PersonUnstructuredName.FullName.Text WHERE [PersonUnstructuredName.Usage.Code='Contact']
    
            INT:^Context = INT
            */
            assertion = (report.INTExists != false && string.IsNullOrWhiteSpace(report.PTR242) == true);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.PTR.430242",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Tax Agent contact name must be present",
                    Location = "/xbrli:xbrl/prsnunstrcnm1.02.01:PersonUnstructuredName" + OccurrenceIndex(report.PersonUnstructuredNameCollection_Contact_INTOccurrenceIndex) + "/pyde.02.00:PersonUnstructuredName.FullName.Text",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.PTR.430242" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "PTR242", Value = report.PTR242 });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.PTR.430242

            #region VR.ATO.PTR.430249

            /*  VR.ATO.PTR.430249
            The number of Other Australian income types and amounts must not exceed 99
    
            Legacy Rule Format:
            IF COUNT(ptr.0006.lodge.req.xx.xx:OtherAustralianIncome) > 99
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            Count(^OtherAustralianIncome) > 99
    
            Data Elements:
    
            ^OtherAustralianIncome = ptr.0006.lodge.req.02.00:OtherAustralianIncome
            */
            assertion = (Count(report.OtherAustralianIncomeCollectionCount) > 99);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.430248",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"The number of Other Australian income types and amounts must not exceed 99",
                    Location = "/xbrli:xbrl/ptr.0006.lodge.req.02.00:OtherAustralianIncome" + OccurrenceIndex(report.OtherAustralianIncomeCollection == null ? 0 : report.OtherAustralianIncomeCollection.Last().OccurrenceIndex) + "/bafpr1.02.00:Income.Other.Description",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.PTR.430249" } },
                };
                response.Add(processMessage);
            }
            #endregion // VR.ATO.PTR.430249

            #region VR.ATO.PTR.430293

            /*  VR.ATO.PTR.430293
            Monetary field must be unsigned, maximum of 11 numeric characters, no decimals
    
            Legacy Rule Format:
            IF [PTR252] <> NULL AND [PTR252] <> MONETARY(U,11,0)
               RETURN VALIDATION MESSAGE
            ENDIF
            
            [PTR252] = PTR:RP.TOFA:bafpr1.02.04:Expense.LossesTotal.Amount

            Technical Business Rule Format:
            NotMonetary(^PTR252, 'U', 11, 0)
    
            Data Elements:
    
            RP.TOFA:^PTR252 = bafpr1.02.04:Expense.LossesTotal.Amount
            */
            assertion = NotMonetary(report.PTR252, @"U", 11, 0);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.400011",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Field incorrect format. Exceeds maximum digits or decimals and/or contains illegal characters",
                    LongDescription = @"Field is in incorrect format for a monetary field. Field either exceeds maximum digits or decimals and/or contains illegal characters and/or for a signed field does not have the sign as the first character",
                    Location = "/xbrli:xbrl/bafpr1.02.04:Expense.LossesTotal.Amount",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.PTR.430293" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "PTR252", Value = report.PTR252.GetValueOrDefault().ToString() });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.PTR.430293

            #region VR.ATO.PTR.430297

            /*  VR.ATO.PTR.430297
            Monetary field must be unsigned, maximum of 11 numeric characters, no decimals
    
            Legacy Rule Format:
            IF [PTR251] <> NULL AND [PTR251] <> MONETARY(U,11,0)
               RETURN VALIDATION MESSAGE
            ENDIF
            
            [PTR251] = PTR:RP.TOFA:bafpr1.02.04:Income.GainsTotal.Amount

            Technical Business Rule Format:
            NotMonetary(^PTR251, 'U', 11, 0)
    
            Data Elements:
    
            RP.TOFA:^PTR251 = bafpr1.02.04:Income.GainsTotal.Amount
            */
            assertion = NotMonetary(report.PTR251, @"U", 11, 0);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.400011",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Field incorrect format. Exceeds maximum digits or decimals and/or contains illegal characters",
                    LongDescription = @"Field is in incorrect format for a monetary field. Field either exceeds maximum digits or decimals and/or contains illegal characters and/or for a signed field does not have the sign as the first character",
                    Location = "/xbrli:xbrl/bafpr1.02.04:Income.GainsTotal.Amount",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.PTR.430297" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "PTR251", Value = report.PTR251.GetValueOrDefault().ToString() });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.PTR.430297

            #region VR.ATO.PTR.430314

            /*  VR.ATO.PTR.430314
            The 'Share of credit for TFN amounts withheld from payments from closely held trusts' must equal the sum all 'Partner share - Share of credit for TFN amounts withheld from payments from closely held trusts'
    
            Legacy Rule Format:
            IF [PTR320] <> SUM([PTR321])
               RETURN VALIDATION MESSAGE
            ENDIF
            
            [PTR320] = PTR:RP.JAUS:rvctc2.02.03:IncomeTax.PayAsYouGoWithholding.CreditForTaxWithheldFromCloselyHeldTrustShareReceivedIndirectly.Amount
            [PTR321] = PTR:RP:rvctc2.02.03:IncomeTax.PayAsYouGoWithholding.CreditForTaxWithheldFromCloselyHeldTrust.Amount IN TUPLE(ptr.0006.lodge.req.02.00:StatementOfDistribution)

            Technical Business Rule Format:
            ^PTR320 <> Sum(^PTR321)
    
            Data Elements:
    
            RP.JAUS:^PTR320 = rvctc2.02.03:IncomeTax.PayAsYouGoWithholding.CreditForTaxWithheldFromCloselyHeldTrustShareReceivedIndirectly.Amount
    
            RP:^PTR321 = rvctc2.02.03:IncomeTax.PayAsYouGoWithholding.CreditForTaxWithheldFromCloselyHeldTrust.Amount
            */
            assertion = (report.PTR320.GetValueOrDefault() != (report.StatementOfDistributionCollection == null ? 0 : report.StatementOfDistributionCollection.Sum(f => f.PTR321.GetValueOrDefault())));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.PTR.430314",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Amount withheld from closely held trusts must equal sum of all partners' shares",
                    LongDescription = @"The 'Share of credit for TFN amounts withheld from payments from closely held trusts' must equal the sum all 'Partner share - Share of credit for TFN amounts withheld from payments from closely held trusts' in the Statements of Distribution",
                    Location = "/xbrli:xbrl/rvctc2.02.03:IncomeTax.PayAsYouGoWithholding.CreditForTaxWithheldFromCloselyHeldTrustShareReceivedIndirectly.Amount",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.PTR.430314" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "PTR320", Value = report.PTR320.GetValueOrDefault().ToString() });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "PTR321", Value = "PTR321" });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.PTR.430314

            #region VR.ATO.PTR.430315

            /*  VR.ATO.PTR.430315
            Monetary field - unsigned, maximum of 13 characters, may include a decimal point and up to 2 decimals
    
            Legacy Rule Format:
            IF [PTR320] <> NULL AND [PTR320] <> MONETARY(U,13,2)
               RETURN VALIDATION MESSAGE
            ENDIF
            
            [PTR320] = PTR:RP.JAUS:rvctc2.02.03:IncomeTax.PayAsYouGoWithholding.CreditForTaxWithheldFromCloselyHeldTrustShareReceivedIndirectly.Amount

            Technical Business Rule Format:
            NotMonetary(^PTR320, 'U', 13, 2)
    
            Data Elements:
    
            RP.JAUS:^PTR320 = rvctc2.02.03:IncomeTax.PayAsYouGoWithholding.CreditForTaxWithheldFromCloselyHeldTrustShareReceivedIndirectly.Amount
            */
            assertion = NotMonetary(report.PTR320, @"U", 13, 2);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.400011",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Field incorrect format. Exceeds maximum digits or decimals and/or contains illegal characters",
                    LongDescription = @"Field is in incorrect format for a monetary field. Field either exceeds maximum digits or decimals and/or contains illegal characters and/or for a signed field does not have the sign as the first character",
                    Location = "/xbrli:xbrl/rvctc2.02.03:IncomeTax.PayAsYouGoWithholding.CreditForTaxWithheldFromCloselyHeldTrustShareReceivedIndirectly.Amount",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.PTR.430315" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "PTR320", Value = report.PTR320.GetValueOrDefault().ToString() });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.PTR.430315

            #region VR.ATO.PTR.430704

            /*  VR.ATO.PTR.430704
            Monetary field must be unsigned, maximum of 11 numeric characters, no decimals
    
            Legacy Rule Format:
            IF [PTR322] <> NULL AND [PTR322] <> MONETARY(U,11,0)
               RETURN VALIDATION MESSAGE
            ENDIF
            
            [PTR322] = PTR:RP.JAUS.NonPrim:bafpr1.02.04:Income.TrustDistributionFranked.Amount

            Technical Business Rule Format:
            NotMonetary(^PTR322, 'U', 11, 0)
    
            Data Elements:
    
            RP.JAUS.NonPrim:^PTR322 = bafpr1.02.04:Income.TrustDistributionFranked.Amount
            */
            assertion = NotMonetary(report.PTR322, @"U", 11, 0);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.400011",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Field incorrect format. Exceeds maximum digits or decimals and/or contains illegal characters",
                    LongDescription = @"Field is in incorrect format for a monetary field. Field either exceeds maximum digits or decimals and/or contains illegal characters and/or for a signed field does not have the sign as the first character",
                    Location = "/xbrli:xbrl/bafpr1.02.04:Income.TrustDistributionFranked.Amount",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.PTR.430704" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "PTR322", Value = report.PTR322.GetValueOrDefault().ToString() });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.PTR.430704

            #region VR.ATO.PTR.430705

            /*  VR.ATO.PTR.430705
            When there is an amount present at non-primary production deductions relating to franked distributions from trusts, there must be an amount at non-primary production franked distributions from trusts.
    
            Legacy Rule Format:
            IF [PTR323] > 0 AND [PTR322] = NULL
               RETURN VALIDATION MESSAGE
            ENDIF
            
            [PTR322] = PTR:RP.JAUS.NonPrim:bafpr1.02.04:Income.TrustDistributionFranked.Amount
            [PTR323] = PTR:RP.JAUS.NonPrim:rvctc2.02.05:IncomeTax.Deduction.TrustDistributionFranked.Amount

            Technical Business Rule Format:
            ^PTR323 > 0 AND ^PTR322 = NULL
    
            Data Elements:
    
            RP.JAUS.NonPrim:^PTR323 = rvctc2.02.05:IncomeTax.Deduction.TrustDistributionFranked.Amount
    
            RP.JAUS.NonPrim:^PTR322 = bafpr1.02.04:Income.TrustDistributionFranked.Amount
            */
            assertion = (report.PTR323.GetValueOrDefault() > 0 && report.PTR322 == null);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.PTR.430705",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Non-PP franked distributions from trusts income must be present",
                    LongDescription = @"When there is an amount present at non-primary production deductions relating to franked distributions from trusts, there must be an amount at non-primary production franked distributions from trusts",
                    Location = "/xbrli:xbrl/rvctc2.02.05:IncomeTax.Deduction.TrustDistributionFranked.Amount",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.PTR.430705" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "PTR323", Value = report.PTR323.GetValueOrDefault().ToString() });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "PTR322", Value = report.PTR322.GetValueOrDefault().ToString() });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.PTR.430705

            #region VR.ATO.PTR.430706

            /*  VR.ATO.PTR.430706
            Monetary field must be unsigned, maximum of 11 numeric characters, no decimals
    
            Legacy Rule Format:
            IF [PTR323] <> NULL AND [PTR323] <> MONETARY(U,11,0)
               RETURN VALIDATION MESSAGE
            ENDIF
            
            [PTR323] = PTR:RP.JAUS.NonPrim:rvctc2.02.05:IncomeTax.Deduction.TrustDistributionFranked.Amount

            Technical Business Rule Format:
            NotMonetary(^PTR323, 'U', 11, 0)
    
            Data Elements:
    
            RP.JAUS.NonPrim:^PTR323 = rvctc2.02.05:IncomeTax.Deduction.TrustDistributionFranked.Amount
            */
            assertion = NotMonetary(report.PTR323, @"U", 11, 0);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.400011",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Field incorrect format. Exceeds maximum digits or decimals and/or contains illegal characters",
                    LongDescription = @"Field is in incorrect format for a monetary field. Field either exceeds maximum digits or decimals and/or contains illegal characters and/or for a signed field does not have the sign as the first character",
                    Location = "/xbrli:xbrl/rvctc2.02.05:IncomeTax.Deduction.TrustDistributionFranked.Amount",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.PTR.430706" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "PTR323", Value = report.PTR323.GetValueOrDefault().ToString() });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.PTR.430706

            #region VR.ATO.PTR.430851

            /*  VR.ATO.PTR.430851
            Monetary field must be unsigned, maximum of 11 numeric characters, no decimals
    
            Legacy Rule Format:
            IF [PTR111] <> NULL AND [PTR111] <> MONETARY(U,11,0)
               RETURN VALIDATION MESSAGE
            ENDIF
            
            [PTR111] = PTR:RP.Closing(Instant):bafpo1.02.00:Assets.CurrentTotal.Amount

            Technical Business Rule Format:
            NotMonetary(^PTR111, 'U', 11, 0)
    
            Data Elements:
    
            RP.Closing(Instant):^PTR111 = bafpo1.02.00:Assets.CurrentTotal.Amount
            */
            assertion = NotMonetary(report.PTR111, @"U", 11, 0);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.400011",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Field incorrect format. Exceeds maximum digits or decimals and/or contains illegal characters",
                    LongDescription = @"Field is in incorrect format for a monetary field. Field either exceeds maximum digits or decimals and/or contains illegal characters and/or for a signed field does not have the sign as the first character",
                    Location = "/xbrli:xbrl/bafpo1.02.00:Assets.CurrentTotal.Amount",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.PTR.430851" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "PTR111", Value = report.PTR111.GetValueOrDefault().ToString() });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.PTR.430851

            #region VR.ATO.PTR.430856

            /*  VR.ATO.PTR.430856
            Field incorrect format. Exceeds maximum digits or decimals and/or contains illegal characters
    
            Legacy Rule Format:
            IF PTR:RP:pyde.xx.xx:AddressDetails.LocalityName.Text <> NULLORBLANK AND LENGTH(PTR:RP:pyde.xx.xx:AddressDetails.LocalityName.Text) > 60
              RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            Length(^PTR10) > 60
    
            Data Elements:
    
            RP:^PTR10 = pyde.02.00:AddressDetails.LocalityName.Text WHERE [AddressDetails.Usage.Code='POS'].[AddressDetails.Currency.Code='C']
            */
            assertion = (Length(report.PTR10) > 60);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.001011",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Field value supplied exceeds maximum field length",
                    Location = "/xbrli:xbrl/address2.02.02:AddressDetails" + OccurrenceIndex(report.AddressDetailsCollection_POS_C_RPOccurrenceIndex) + "/pyde.02.00:AddressDetails.LocalityName.Text",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.PTR.430856" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "PTR10", Value = report.PTR10 });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.PTR.430856

            #region VR.ATO.PTR.430860

            /*  VR.ATO.PTR.430860
            Field incorrect format. Exceeds maximum digits or decimals and/or contains illegal characters
    
            Legacy Rule Format:
            IF [PTR30] <> NULLORBLANK AND LENGTH([PTR30]) > 100
              RETURN VALIDATION MESSAGE
            ENDIF
            
            [PTR30] = PTR:RP:pyde.02.05:OrganisationDetails.MainIncomeActivity.Description

            Technical Business Rule Format:
            Length(^PTR30) > 100
    
            Data Elements:
    
            RP:^PTR30 = pyde.02.05:OrganisationDetails.MainIncomeActivity.Description
            */
            assertion = (Length(report.PTR30) > 100);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.PTR.430860",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Description of main business activity must not exceed 100 characters",
                    Location = "/xbrli:xbrl/pyde.02.05:OrganisationDetails.MainIncomeActivity.Description",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.PTR.430860" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "PTR30", Value = report.PTR30 });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.PTR.430860

            #region VR.ATO.PTR.430866

            /*  VR.ATO.PTR.430866
            Monetary field must be unsigned, maximum of 11 numeric characters, no decimals
    
            Legacy Rule Format:
            IF [PTR81] <> NULL AND [PTR81] <> MONETARY(U,11,0)
               RETURN VALIDATION MESSAGE
            ENDIF
            
            [PTR81] = PTR:RP.JAUS:bafpr1.02.23:Income.ForestryManagedInvestmentScheme.Amount

            Technical Business Rule Format:
            NotMonetary(^PTR81, 'U', 11, 0)
    
            Data Elements:
    
            RP.JAUS:^PTR81 = bafpr1.02.23:Income.ForestryManagedInvestmentScheme.Amount
            */
            assertion = NotMonetary(report.PTR81, @"U", 11, 0);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.400011",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Field incorrect format. Exceeds maximum digits or decimals and/or contains illegal characters",
                    LongDescription = @"Field is in incorrect format for a monetary field. Field either exceeds maximum digits or decimals and/or contains illegal characters and/or for a signed field does not have the sign as the first character",
                    Location = "/xbrli:xbrl/bafpr1.02.23:Income.ForestryManagedInvestmentScheme.Amount",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.PTR.430866" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "PTR81", Value = report.PTR81.GetValueOrDefault().ToString() });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.PTR.430866

            #region VR.ATO.PTR.430867

            /*  VR.ATO.PTR.430867
            Monetary field must be unsigned, maximum of 11 numeric characters, no decimals
    
            Legacy Rule Format:
            IF [PTR82] <> NULL AND [PTR82] <> MONETARY(U,11,0)
               RETURN VALIDATION MESSAGE
            ENDIF
            
            [PTR82] = PTR:RP.JAUS:bafpr2.02.02:Income.Interest.Gross.Amount

            Technical Business Rule Format:
            NotMonetary(^PTR82, 'U', 11, 0)
    
            Data Elements:
    
            RP.JAUS:^PTR82 = bafpr2.02.02:Income.Interest.Gross.Amount
            */
            assertion = NotMonetary(report.PTR82, @"U", 11, 0);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.400011",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Field incorrect format. Exceeds maximum digits or decimals and/or contains illegal characters",
                    LongDescription = @"Field is in incorrect format for a monetary field. Field either exceeds maximum digits or decimals and/or contains illegal characters and/or for a signed field does not have the sign as the first character",
                    Location = "/xbrli:xbrl/bafpr2.02.02:Income.Interest.Gross.Amount",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.PTR.430867" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "PTR82", Value = report.PTR82.GetValueOrDefault().ToString() });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.PTR.430867

            #region VR.ATO.PTR.430870

            /*  VR.ATO.PTR.430870
            Monetary field must be unsigned, maximum of 11 numeric characters, no decimals
    
            Legacy Rule Format:
            IF [PTR136] <> NULL AND [PTR136] <> MONETARY(U,11,0)
               RETURN VALIDATION MESSAGE
            ENDIF
            
            [PTR136] = PTR:RP:lrla.02.00:Remuneration.FringeBenefits.EmployeeContribution.Amount

            Technical Business Rule Format:
            NotMonetary(^PTR136, 'U', 11, 0)
    
            Data Elements:
    
            RP:^PTR136 = lrla.02.00:Remuneration.FringeBenefits.EmployeeContribution.Amount
            */
            assertion = NotMonetary(report.PTR136, @"U", 11, 0);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.400011",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Field incorrect format. Exceeds maximum digits or decimals and/or contains illegal characters",
                    LongDescription = @"Field is in incorrect format for a monetary field. Field either exceeds maximum digits or decimals and/or contains illegal characters and/or for a signed field does not have the sign as the first character",
                    Location = "/xbrli:xbrl/lrla.02.00:Remuneration.FringeBenefits.EmployeeContribution.Amount",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.PTR.430870" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "PTR136", Value = report.PTR136.GetValueOrDefault().ToString() });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.PTR.430870

            #region VR.ATO.PTR.430876

            /*  VR.ATO.PTR.430876
            Monetary field must be unsigned, maximum of 11 numeric characters, no decimals
    
            Legacy Rule Format:
            IF [PTR46] <> NULL AND [PTR46] <> MONETARY(U,11,0)
               RETURN VALIDATION MESSAGE
            ENDIF
            
            [PTR46] = PTR:RP.JAUS:lrla.02.16:Remuneration.WagesAndSalaries.PaymentsContractorAndConsultant.Amount

            Technical Business Rule Format:
            NotMonetary(^PTR46, 'U', 11, 0)
    
            Data Elements:
    
            RP.JAUS:^PTR46 = lrla.02.16:Remuneration.WagesAndSalaries.PaymentsContractorAndConsultant.Amount
            */
            assertion = NotMonetary(report.PTR46, @"U", 11, 0);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.400011",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Field incorrect format. Exceeds maximum digits or decimals and/or contains illegal characters",
                    LongDescription = @"Field is in incorrect format for a monetary field. Field either exceeds maximum digits or decimals and/or contains illegal characters and/or for a signed field does not have the sign as the first character",
                    Location = "/xbrli:xbrl/lrla.02.16:Remuneration.WagesAndSalaries.PaymentsContractorAndConsultant.Amount",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.PTR.430876" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "PTR46", Value = report.PTR46.GetValueOrDefault().ToString() });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.PTR.430876

            #region VR.ATO.PTR.430883

            /*  VR.ATO.PTR.430883
            Monetary field must be unsigned, maximum of 11 numeric characters, no decimals
    
            Legacy Rule Format:
            IF [PTR137] <> NULL AND [PTR137] <> MONETARY(U,11,0)
               RETURN VALIDATION MESSAGE
            ENDIF
            
            [PTR137] = PTR:RP.JFOREIGN:bafpr1.02.05:Expense.Interest.Amount

            Technical Business Rule Format:
            NotMonetary(^PTR137, 'U', 11, 0)
    
            Data Elements:
    
            RP.JFOREIGN:^PTR137 = bafpr1.02.05:Expense.Interest.Amount
            */
            assertion = NotMonetary(report.PTR137, @"U", 11, 0);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.400011",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Field incorrect format. Exceeds maximum digits or decimals and/or contains illegal characters",
                    LongDescription = @"Field is in incorrect format for a monetary field. Field either exceeds maximum digits or decimals and/or contains illegal characters and/or for a signed field does not have the sign as the first character",
                    Location = "/xbrli:xbrl/bafpr1.02.05:Expense.Interest.Amount[@contextRef='" + report.RPJFOREIGNId + "']",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.PTR.430883" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "PTR137", Value = report.PTR137.GetValueOrDefault().ToString() });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.PTR.430883

            #region VR.ATO.PTR.430886

            /*  VR.ATO.PTR.430886
            Monetary field must be unsigned, maximum of 11 numeric characters, no decimals
    
            Legacy Rule Format:
            IF [PTR54] <> NULL AND [PTR54] <> MONETARY(U,11,0)
               RETURN VALIDATION MESSAGE
            ENDIF
            
            [PTR54] = PTR:RP.JAUS:bafpr1.02.04:Expense.DepreciationandAmortisation.Amount

            Technical Business Rule Format:
            NotMonetary(^PTR54, 'U', 11, 0)
    
            Data Elements:
    
            RP.JAUS:^PTR54 = bafpr1.02.04:Expense.DepreciationandAmortisation.Amount
            */
            assertion = NotMonetary(report.PTR54, @"U", 11, 0);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.400011",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Field incorrect format. Exceeds maximum digits or decimals and/or contains illegal characters",
                    LongDescription = @"Field is in incorrect format for a monetary field. Field either exceeds maximum digits or decimals and/or contains illegal characters and/or for a signed field does not have the sign as the first character",
                    Location = "/xbrli:xbrl/bafpr1.02.04:Expense.DepreciationandAmortisation.Amount",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.PTR.430886" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "PTR54", Value = report.PTR54.GetValueOrDefault().ToString() });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.PTR.430886

            #region VR.ATO.PTR.430890

            /*  VR.ATO.PTR.430890
            Monetary field must be unsigned, maximum of 11 numeric characters, no decimals
    
            Legacy Rule Format:
            IF [PTR58] <> NULL AND [PTR58] <> MONETARY(U,11,0)
               RETURN VALIDATION MESSAGE
            ENDIF
            
            [PTR58] = PTR:RP.JAUS:bafpr1.02.04:Expense.DeductibleOtherTotal.Amount

            Technical Business Rule Format:
            NotMonetary(^PTR58, 'U', 11, 0)
    
            Data Elements:
    
            RP.JAUS:^PTR58 = bafpr1.02.04:Expense.DeductibleOtherTotal.Amount
            */
            assertion = NotMonetary(report.PTR58, @"U", 11, 0);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.400011",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Field incorrect format. Exceeds maximum digits or decimals and/or contains illegal characters",
                    LongDescription = @"Field is in incorrect format for a monetary field. Field either exceeds maximum digits or decimals and/or contains illegal characters and/or for a signed field does not have the sign as the first character",
                    Location = "/xbrli:xbrl/bafpr1.02.04:Expense.DeductibleOtherTotal.Amount",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.PTR.430890" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "PTR58", Value = report.PTR58.GetValueOrDefault().ToString() });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.PTR.430890

            #region VR.ATO.PTR.430894

            /*  VR.ATO.PTR.430894
            Monetary field must be unsigned, maximum of 11 numeric characters, no decimals
    
            Legacy Rule Format:
            IF [PTR107] <> NULL AND [PTR107] <> MONETARY(U,11,0)
               RETURN VALIDATION MESSAGE
            ENDIF
            
            [PTR107] = PTR:RP.JFOREIGN:rvctc2.02.00:IncomeTax.FrankingCredits.ReceivedFromNewZealandCompanies.Amount

            Technical Business Rule Format:
            NotMonetary(^PTR107, 'U', 11, 0)
    
            Data Elements:
    
            RP.JFOREIGN:^PTR107 = rvctc2.02.00:IncomeTax.FrankingCredits.ReceivedFromNewZealandCompanies.Amount
            */
            assertion = NotMonetary(report.PTR107, @"U", 11, 0);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.400011",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Field incorrect format. Exceeds maximum digits or decimals and/or contains illegal characters",
                    LongDescription = @"Field is in incorrect format for a monetary field. Field either exceeds maximum digits or decimals and/or contains illegal characters and/or for a signed field does not have the sign as the first character",
                    Location = "/xbrli:xbrl/rvctc2.02.00:IncomeTax.FrankingCredits.ReceivedFromNewZealandCompanies.Amount[@contextRef='" + report.RPJFOREIGNId + "']",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.PTR.430894" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "PTR107", Value = report.PTR107.GetValueOrDefault().ToString() });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.PTR.430894

            #region VR.ATO.PTR.430899

            /*  VR.ATO.PTR.430899
            Monetary field must be unsigned, maximum of 11 numeric characters, no decimals
    
            Legacy Rule Format:
            IF [PTR91] <> NULL AND [PTR91] <> MONETARY(U,11,0)
               RETURN VALIDATION MESSAGE
            ENDIF
            
            [PTR91] = PTR:RP.JAUS:bafpr1.02.23:Expense.ForestryManagedInvestmentSchemeDeduction.Amount

            Technical Business Rule Format:
            NotMonetary(^PTR91, 'U', 11, 0)
    
            Data Elements:
    
            RP.JAUS:^PTR91 = bafpr1.02.23:Expense.ForestryManagedInvestmentSchemeDeduction.Amount
            */
            assertion = NotMonetary(report.PTR91, @"U", 11, 0);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.400011",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Field incorrect format. Exceeds maximum digits or decimals and/or contains illegal characters",
                    LongDescription = @"Field is in incorrect format for a monetary field. Field either exceeds maximum digits or decimals and/or contains illegal characters and/or for a signed field does not have the sign as the first character",
                    Location = "/xbrli:xbrl/bafpr1.02.23:Expense.ForestryManagedInvestmentSchemeDeduction.Amount",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.PTR.430899" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "PTR91", Value = report.PTR91.GetValueOrDefault().ToString() });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.PTR.430899

            #region VR.ATO.PTR.430901

            /*  VR.ATO.PTR.430901
            Monetary field must be unsigned, maximum of 11 numeric characters, no decimals
    
            Legacy Rule Format:
            IF [PTR134] <> NULL AND [PTR134] <> MONETARY(U,11,0)
               RETURN VALIDATION MESSAGE
            ENDIF
            
            [PTR134] = PTR:RP:bafpr1.02.02:Expense.ProjectPoolAllowableDeduction.Amount

            Technical Business Rule Format:
            NotMonetary(^PTR134, 'U', 11, 0)
    
            Data Elements:
    
            RP:^PTR134 = bafpr1.02.02:Expense.ProjectPoolAllowableDeduction.Amount
            */
            assertion = NotMonetary(report.PTR134, @"U", 11, 0);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.400011",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Field incorrect format. Exceeds maximum digits or decimals and/or contains illegal characters",
                    LongDescription = @"Field is in incorrect format for a monetary field. Field either exceeds maximum digits or decimals and/or contains illegal characters and/or for a signed field does not have the sign as the first character",
                    Location = "/xbrli:xbrl/bafpr1.02.02:Expense.ProjectPoolAllowableDeduction.Amount",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.PTR.430901" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "PTR134", Value = report.PTR134.GetValueOrDefault().ToString() });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.PTR.430901

            #region VR.ATO.PTR.430903

            /*  VR.ATO.PTR.430903
            Monetary field must be unsigned, maximum of 11 numeric characters, no decimals
    
            Legacy Rule Format:
            IF [PTR135] <> NULL AND [PTR135] <> MONETARY(U,11,0)
               RETURN VALIDATION MESSAGE
            ENDIF
            
            [PTR135] = PTR:RP:bafpr1.02.02:Expense.CapitalExpenditureSpecifiedAllowableDeduction.Amount

            Technical Business Rule Format:
            NotMonetary(^PTR135, 'U', 11, 0)
    
            Data Elements:
    
            RP:^PTR135 = bafpr1.02.02:Expense.CapitalExpenditureSpecifiedAllowableDeduction.Amount
            */
            assertion = NotMonetary(report.PTR135, @"U", 11, 0);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.400011",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Field incorrect format. Exceeds maximum digits or decimals and/or contains illegal characters",
                    LongDescription = @"Field is in incorrect format for a monetary field. Field either exceeds maximum digits or decimals and/or contains illegal characters and/or for a signed field does not have the sign as the first character",
                    Location = "/xbrli:xbrl/bafpr1.02.02:Expense.CapitalExpenditureSpecifiedAllowableDeduction.Amount",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.PTR.430903" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "PTR135", Value = report.PTR135.GetValueOrDefault().ToString() });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.PTR.430903

            #region VR.ATO.PTR.430919

            /*  VR.ATO.PTR.430919
            Monetary field must be unsigned, maximum of 11 numeric characters, no decimals
    
            Legacy Rule Format:
            IF [PTR121] <> NULL AND [PTR121] <> MONETARY(U,11,0)
               RETURN VALIDATION MESSAGE
            ENDIF
            
            [PTR121] = PTR:RP.Opening(Instant):bafpo7.02.04:Assets.Inventories.Total.Amount

            Technical Business Rule Format:
            NotMonetary(^PTR121, 'U', 11, 0)
    
            Data Elements:
    
            RP.Opening(Instant):^PTR121 = bafpo7.02.04:Assets.Inventories.Total.Amount
            */
            assertion = NotMonetary(report.PTR121, @"U", 11, 0);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.400011",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Field incorrect format. Exceeds maximum digits or decimals and/or contains illegal characters",
                    LongDescription = @"Field is in incorrect format for a monetary field. Field either exceeds maximum digits or decimals and/or contains illegal characters and/or for a signed field does not have the sign as the first character",
                    Location = "/xbrli:xbrl/bafpo7.02.04:Assets.Inventories.Total.Amount[@contextRef='" + report.RPOpeningInstantId + "']",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.PTR.430919" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "PTR121", Value = report.PTR121.GetValueOrDefault().ToString() });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.PTR.430919

            #region VR.ATO.PTR.430921

            /*  VR.ATO.PTR.430921
            Monetary field must be unsigned, maximum of 11 numeric characters, no decimals
    
            Legacy Rule Format:
            IF [PTR123] <> NULL AND [PTR123] <> MONETARY(U,11,0)
               RETURN VALIDATION MESSAGE
            ENDIF
            
            [PTR123] = PTR:RP.Closing(Instant):bafpo7.02.04:Assets.Inventories.Total.Amount

            Technical Business Rule Format:
            NotMonetary(^PTR123, 'U', 11, 0)
    
            Data Elements:
    
            RP.Closing(Instant):^PTR123 = bafpo7.02.04:Assets.Inventories.Total.Amount
            */
            assertion = NotMonetary(report.PTR123, @"U", 11, 0);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.400011",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Field incorrect format. Exceeds maximum digits or decimals and/or contains illegal characters",
                    LongDescription = @"Field is in incorrect format for a monetary field. Field either exceeds maximum digits or decimals and/or contains illegal characters and/or for a signed field does not have the sign as the first character",
                    Location = "/xbrli:xbrl/bafpo7.02.04:Assets.Inventories.Total.Amount[@contextRef='" + report.RPClosingInstantId + "']",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.PTR.430921" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "PTR123", Value = report.PTR123.GetValueOrDefault().ToString() });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.PTR.430921

            #region VR.ATO.PTR.430922

            /*  VR.ATO.PTR.430922
            Monetary field must be unsigned, maximum of 11 numeric characters, no decimals
    
            Legacy Rule Format:
            IF [PTR125] <> NULL AND [PTR125] <> MONETARY(U,11,0)
               RETURN VALIDATION MESSAGE
            ENDIF
            
            [PTR125] = PTR:RP.Closing(Instant):bafpo1.02.13:Assets.TradeandReceivablesOther.Amount

            Technical Business Rule Format:
            NotMonetary(^PTR125, 'U', 11, 0)
    
            Data Elements:
    
            RP.Closing(Instant):^PTR125 = bafpo1.02.13:Assets.TradeandReceivablesOther.Amount
            */
            assertion = NotMonetary(report.PTR125, @"U", 11, 0);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.400011",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Field incorrect format. Exceeds maximum digits or decimals and/or contains illegal characters",
                    LongDescription = @"Field is in incorrect format for a monetary field. Field either exceeds maximum digits or decimals and/or contains illegal characters and/or for a signed field does not have the sign as the first character",
                    Location = "/xbrli:xbrl/bafpo1.02.13:Assets.TradeandReceivablesOther.Amount",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.PTR.430922" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "PTR125", Value = report.PTR125.GetValueOrDefault().ToString() });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.PTR.430922

            #region VR.ATO.PTR.430923

            /*  VR.ATO.PTR.430923
            Monetary field must be unsigned, maximum of 11 numeric characters, no decimals
    
            Legacy Rule Format:
            IF [PTR112] <> NULL AND [PTR112] <> MONETARY(U,11,0)
               RETURN VALIDATION MESSAGE
            ENDIF
            
            [PTR112] = PTR:RP.Closing(Instant):bafpo1.02.13:Assets.Total.Amount

            Technical Business Rule Format:
            NotMonetary(^PTR112, 'U', 11, 0)
    
            Data Elements:
    
            RP.Closing(Instant):^PTR112 = bafpo1.02.13:Assets.Total.Amount
            */
            assertion = NotMonetary(report.PTR112, @"U", 11, 0);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.400011",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Field incorrect format. Exceeds maximum digits or decimals and/or contains illegal characters",
                    LongDescription = @"Field is in incorrect format for a monetary field. Field either exceeds maximum digits or decimals and/or contains illegal characters and/or for a signed field does not have the sign as the first character",
                    Location = "/xbrli:xbrl/bafpo1.02.13:Assets.Total.Amount",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.PTR.430923" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "PTR112", Value = report.PTR112.GetValueOrDefault().ToString() });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.PTR.430923

            #region VR.ATO.PTR.430924

            /*  VR.ATO.PTR.430924
            Monetary field must be unsigned, maximum of 11 numeric characters, no decimals
    
            Legacy Rule Format:
            IF [PTR126] <> NULL AND [PTR126] <> MONETARY(U,11,0)
               RETURN VALIDATION MESSAGE
            ENDIF
            
            [PTR126] = PTR:RP.Closing(Instant):bafpo1.02.13:Liabilities.TradeAndOtherPayablesTotal.Amount

            Technical Business Rule Format:
            NotMonetary(^PTR126, 'U', 11, 0)
    
            Data Elements:
    
            RP.Closing(Instant):^PTR126 = bafpo1.02.13:Liabilities.TradeAndOtherPayablesTotal.Amount
            */
            assertion = NotMonetary(report.PTR126, @"U", 11, 0);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.400011",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Field incorrect format. Exceeds maximum digits or decimals and/or contains illegal characters",
                    LongDescription = @"Field is in incorrect format for a monetary field. Field either exceeds maximum digits or decimals and/or contains illegal characters and/or for a signed field does not have the sign as the first character",
                    Location = "/xbrli:xbrl/bafpo1.02.13:Liabilities.TradeAndOtherPayablesTotal.Amount",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.PTR.430924" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "PTR126", Value = report.PTR126.GetValueOrDefault().ToString() });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.PTR.430924

            #region VR.ATO.PTR.430925

            /*  VR.ATO.PTR.430925
            Monetary field must be unsigned, maximum of 11 numeric characters, no decimals
    
            Legacy Rule Format:
            IF [PTR113] <> NULL AND [PTR113] <> MONETARY(U,11,0)
               RETURN VALIDATION MESSAGE
            ENDIF
            
            [PTR113] = PTR:RP.Closing(Instant):bafpo1.02.00:Liabilities.CurrentTotal.Amount

            Technical Business Rule Format:
            NotMonetary(^PTR113, 'U', 11, 0)
    
            Data Elements:
    
            RP.Closing(Instant):^PTR113 = bafpo1.02.00:Liabilities.CurrentTotal.Amount
            */
            assertion = NotMonetary(report.PTR113, @"U", 11, 0);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.400011",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Field incorrect format. Exceeds maximum digits or decimals and/or contains illegal characters",
                    LongDescription = @"Field is in incorrect format for a monetary field. Field either exceeds maximum digits or decimals and/or contains illegal characters and/or for a signed field does not have the sign as the first character",
                    Location = "/xbrli:xbrl/bafpo1.02.00:Liabilities.CurrentTotal.Amount",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.PTR.430925" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "PTR113", Value = report.PTR113.GetValueOrDefault().ToString() });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.PTR.430925

            #region VR.ATO.PTR.430938

            /*  VR.ATO.PTR.430938
            Monetary field must be unsigned, maximum of 11 numeric characters, no decimals
    
            Legacy Rule Format:
            IF [PTR130] <> NULL AND [PTR130] <> MONETARY(U,11,0)
               RETURN VALIDATION MESSAGE
            ENDIF
            
            [PTR130] = PTR:RP:bafpr3.02.20:Expense.DepreciatingAssets.IntangibleFirstDeducted.Amount

            Technical Business Rule Format:
            NotMonetary(^PTR130, 'U', 11, 0)
    
            Data Elements:
    
            RP:^PTR130 = bafpr3.02.20:Expense.DepreciatingAssets.IntangibleFirstDeducted.Amount
            */
            assertion = NotMonetary(report.PTR130, @"U", 11, 0);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.400011",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Field incorrect format. Exceeds maximum digits or decimals and/or contains illegal characters",
                    LongDescription = @"Field is in incorrect format for a monetary field. Field either exceeds maximum digits or decimals and/or contains illegal characters and/or for a signed field does not have the sign as the first character",
                    Location = "/xbrli:xbrl/bafpr3.02.20:Expense.DepreciatingAssets.IntangibleFirstDeducted.Amount",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.PTR.430938" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "PTR130", Value = report.PTR130.GetValueOrDefault().ToString() });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.PTR.430938

            #region VR.ATO.PTR.430939

            /*  VR.ATO.PTR.430939
            Monetary field must be unsigned, maximum of 11 numeric characters, no decimals
    
            Legacy Rule Format:
            IF [PTR131] <> NULL AND [PTR131] <> MONETARY(U,11,0)
               RETURN VALIDATION MESSAGE
            ENDIF
            
            [PTR131] = PTR:RP:bafpr3.02.20:Expense.DepreciatingAssets.OtherFirstDeducted.Amount

            Technical Business Rule Format:
            NotMonetary(^PTR131, 'U', 11, 0)
    
            Data Elements:
    
            RP:^PTR131 = bafpr3.02.20:Expense.DepreciatingAssets.OtherFirstDeducted.Amount
            */
            assertion = NotMonetary(report.PTR131, @"U", 11, 0);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.400011",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Field incorrect format. Exceeds maximum digits or decimals and/or contains illegal characters",
                    LongDescription = @"Field is in incorrect format for a monetary field. Field either exceeds maximum digits or decimals and/or contains illegal characters and/or for a signed field does not have the sign as the first character",
                    Location = "/xbrli:xbrl/bafpr3.02.20:Expense.DepreciatingAssets.OtherFirstDeducted.Amount",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.PTR.430939" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "PTR131", Value = report.PTR131.GetValueOrDefault().ToString() });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.PTR.430939

            #region VR.ATO.PTR.430940

            /*  VR.ATO.PTR.430940
            Monetary field must be unsigned, maximum of 11 numeric characters, no decimals
    
            Legacy Rule Format:
            IF [PTR132] <> NULL AND [PTR132] <> MONETARY(U,11,0)
               RETURN VALIDATION MESSAGE
            ENDIF
            
            [PTR132] = PTR:RP:bafpo7.02.02:Assets.DepreciatingAssets.IntangibleTerminationValue.Amount

            Technical Business Rule Format:
            NotMonetary(^PTR132, 'U', 11, 0)
    
            Data Elements:
    
            RP:^PTR132 = bafpo7.02.02:Assets.DepreciatingAssets.IntangibleTerminationValue.Amount
            */
            assertion = NotMonetary(report.PTR132, @"U", 11, 0);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.400011",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Field incorrect format. Exceeds maximum digits or decimals and/or contains illegal characters",
                    LongDescription = @"Field is in incorrect format for a monetary field. Field either exceeds maximum digits or decimals and/or contains illegal characters and/or for a signed field does not have the sign as the first character",
                    Location = "/xbrli:xbrl/bafpo7.02.02:Assets.DepreciatingAssets.IntangibleTerminationValue.Amount",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.PTR.430940" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "PTR132", Value = report.PTR132.GetValueOrDefault().ToString() });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.PTR.430940

            #region VR.ATO.PTR.430941

            /*  VR.ATO.PTR.430941
            Monetary field must be unsigned, maximum of 11 numeric characters, no decimals
    
            Legacy Rule Format:
            IF [PTR133] <> NULL AND [PTR133] <> MONETARY(U,11,0)
               RETURN VALIDATION MESSAGE
            ENDIF
            
            [PTR133] = PTR:RP:bafpo7.02.02:Assets.DepreciatingAssets.OtherTerminationValue.Amount

            Technical Business Rule Format:
            NotMonetary(^PTR133, 'U', 11, 0)
    
            Data Elements:
    
            RP:^PTR133 = bafpo7.02.02:Assets.DepreciatingAssets.OtherTerminationValue.Amount
            */
            assertion = NotMonetary(report.PTR133, @"U", 11, 0);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.400011",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Field incorrect format. Exceeds maximum digits or decimals and/or contains illegal characters",
                    LongDescription = @"Field is in incorrect format for a monetary field. Field either exceeds maximum digits or decimals and/or contains illegal characters and/or for a signed field does not have the sign as the first character",
                    Location = "/xbrli:xbrl/bafpo7.02.02:Assets.DepreciatingAssets.OtherTerminationValue.Amount",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.PTR.430941" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "PTR133", Value = report.PTR133.GetValueOrDefault().ToString() });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.PTR.430941

            #region VR.ATO.PTR.430942

            /*  VR.ATO.PTR.430942
            Monetary field must be unsigned, maximum of 11 numeric characters, no decimals
    
            Legacy Rule Format:
            IF [PTR127] <> NULL AND [PTR127] <> MONETARY(U,11,0)
               RETURN VALIDATION MESSAGE
            ENDIF
            
            [PTR127] = PTR:RP:lrla.02.17:Remuneration.WagesAndSalaries.Total.Amount

            Technical Business Rule Format:
            NotMonetary(^PTR127, 'U', 11, 0)
    
            Data Elements:
    
            RP:^PTR127 = lrla.02.17:Remuneration.WagesAndSalaries.Total.Amount
            */
            assertion = NotMonetary(report.PTR127, @"U", 11, 0);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.400011",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Field incorrect format. Exceeds maximum digits or decimals and/or contains illegal characters",
                    LongDescription = @"Field is in incorrect format for a monetary field. Field either exceeds maximum digits or decimals and/or contains illegal characters and/or for a signed field does not have the sign as the first character",
                    Location = "/xbrli:xbrl/lrla.02.17:Remuneration.WagesAndSalaries.Total.Amount",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.PTR.430942" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "PTR127", Value = report.PTR127.GetValueOrDefault().ToString() });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.PTR.430942

            #region VR.ATO.PTR.430943

            /*  VR.ATO.PTR.430943
            Monetary field must be unsigned, maximum of 11 numeric characters, no decimals
    
            Legacy Rule Format:
            IF [PTR129] <> NULL AND [PTR129] <> MONETARY(U,11,0)
               RETURN VALIDATION MESSAGE
            ENDIF
            
            [PTR129] = PTR:RP:lrla.02.00:Remuneration.PaymentToRelatedPartiesGross.Amount

            Technical Business Rule Format:
            NotMonetary(^PTR129, 'U', 11, 0)
    
            Data Elements:
    
            RP:^PTR129 = lrla.02.00:Remuneration.PaymentToRelatedPartiesGross.Amount
            */
            assertion = NotMonetary(report.PTR129, @"U", 11, 0);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.400011",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Field incorrect format. Exceeds maximum digits or decimals and/or contains illegal characters",
                    LongDescription = @"Field is in incorrect format for a monetary field. Field either exceeds maximum digits or decimals and/or contains illegal characters and/or for a signed field does not have the sign as the first character",
                    Location = "/xbrli:xbrl/lrla.02.00:Remuneration.PaymentToRelatedPartiesGross.Amount",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.PTR.430943" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "PTR129", Value = report.PTR129.GetValueOrDefault().ToString() });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.PTR.430943

            #region VR.ATO.PTR.430944

            /*  VR.ATO.PTR.430944
            Monetary field may be signed, maximum of 11 chars, no decimals
    
            Legacy Rule Format:
            IF [PTR105] <> NULL AND [PTR105] <> MONETARY(S,11,0)
               RETURN VALIDATION MESSAGE
            ENDIF
            
            [PTR105] = PTR:RP.JFOREIGN:bafpr1.02.23:Income.Net.Amount

            Technical Business Rule Format:
            NotMonetary(^PTR105, 'S', 11, 0)
    
            Data Elements:
    
            RP.JFOREIGN:^PTR105 = bafpr1.02.23:Income.Net.Amount
            */
            assertion = NotMonetary(report.PTR105, @"S", 11, 0);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.400011",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Field incorrect format. Exceeds maximum digits or decimals and/or contains illegal characters",
                    LongDescription = @"Field is in incorrect format for a monetary field. Field either exceeds maximum digits or decimals and/or contains illegal characters and/or for a signed field does not have the sign as the first character",
                    Location = "/xbrli:xbrl/bafpr1.02.23:Income.Net.Amount[@contextRef='" + report.RPJFOREIGNId + "']",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.PTR.430944" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "PTR105", Value = report.PTR105.GetValueOrDefault().ToString() });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.PTR.430944

            #region VR.ATO.PTR.430946

            /*  VR.ATO.PTR.430946
            Monetary field must be unsigned, maximum of 11 numeric characters, no decimals
    
            Legacy Rule Format:
            IF [PTR98] <> NULL AND [PTR98] <> MONETARY(U,11,0)
               RETURN VALIDATION MESSAGE
            ENDIF
            
            [PTR98] = PTR:RP.JFOREIGN.CFCLC:bafpr3.02.02:Income.InternationalDealings.Attributed.Amount

            Technical Business Rule Format:
            NotMonetary(^PTR98, 'U', 11, 0)
    
            Data Elements:
    
            RP.JFOREIGN.CFCLC:^PTR98 = bafpr3.02.02:Income.InternationalDealings.Attributed.Amount
            */
            assertion = NotMonetary(report.PTR98, @"U", 11, 0);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.400011",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Field incorrect format. Exceeds maximum digits or decimals and/or contains illegal characters",
                    LongDescription = @"Field is in incorrect format for a monetary field. Field either exceeds maximum digits or decimals and/or contains illegal characters and/or for a signed field does not have the sign as the first character",
                    Location = "/xbrli:xbrl/bafpr3.02.02:Income.InternationalDealings.Attributed.Amount[@contextRef='" + report.RPJFOREIGNCFCLCId + "']",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.PTR.430946" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "PTR98", Value = report.PTR98.GetValueOrDefault().ToString() });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.PTR.430946

            #region VR.ATO.PTR.430948

            /*  VR.ATO.PTR.430948
            Monetary field must be unsigned, maximum of 11 numeric characters, no decimals
    
            Legacy Rule Format:
            IF [PTR100] <> NULL AND [PTR100] <> MONETARY(U,11,0)
               RETURN VALIDATION MESSAGE
            ENDIF
            
            [PTR100] = PTR:RP.JFOREIGN.CFCUC:bafpr3.02.02:Income.InternationalDealings.Attributed.Amount

            Technical Business Rule Format:
            NotMonetary(^PTR100, 'U', 11, 0)
    
            Data Elements:
    
            RP.JFOREIGN.CFCUC:^PTR100 = bafpr3.02.02:Income.InternationalDealings.Attributed.Amount
            */
            assertion = NotMonetary(report.PTR100, @"U", 11, 0);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.400011",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Field incorrect format. Exceeds maximum digits or decimals and/or contains illegal characters",
                    LongDescription = @"Field is in incorrect format for a monetary field. Field either exceeds maximum digits or decimals and/or contains illegal characters and/or for a signed field does not have the sign as the first character",
                    Location = "/xbrli:xbrl/bafpr3.02.02:Income.InternationalDealings.Attributed.Amount[@contextRef='" + report.RPJFOREIGNCFCUCId + "']",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.PTR.430948" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "PTR100", Value = report.PTR100.GetValueOrDefault().ToString() });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.PTR.430948

            #region VR.ATO.PTR.430977

            /*  VR.ATO.PTR.430977
            Monetary field - unsigned, maximum of 13 characters, may include a decimal point and up to 2 decimals
    
            Legacy Rule Format:
            IF [PTR106] <> NULL AND [PTR106] <> MONETARY(U,13,2)
               RETURN VALIDATION MESSAGE
            ENDIF
            
            [PTR106] = PTR:RP.JFOREIGN:bafpr3.02.18:Income.InternationalDealings.TaxOffset.Amount

            Technical Business Rule Format:
            NotMonetary(^PTR106, 'U', 13, 2)
    
            Data Elements:
    
            RP.JFOREIGN:^PTR106 = bafpr3.02.18:Income.InternationalDealings.TaxOffset.Amount
            */
            assertion = NotMonetary(report.PTR106, @"U", 13, 2);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.400011",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Field incorrect format. Exceeds maximum digits or decimals and/or contains illegal characters",
                    LongDescription = @"Field is in incorrect format for a monetary field. Field either exceeds maximum digits or decimals and/or contains illegal characters and/or for a signed field does not have the sign as the first character",
                    Location = "/xbrli:xbrl/bafpr3.02.18:Income.InternationalDealings.TaxOffset.Amount",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.PTR.430977" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "PTR106", Value = report.PTR106.GetValueOrDefault().ToString() });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.PTR.430977

            #region VR.ATO.PTR.430983

            /*  VR.ATO.PTR.430983
            Monetary field must be unsigned, maximum of 11 numeric characters, no decimals
    
            Legacy Rule Format:
            IF [PTR63] <> NULL AND [PTR63] <> MONETARY(U,11,0)
               RETURN VALIDATION MESSAGE
            ENDIF
            
            [PTR63] = PTR:RP.JAUS:rvctc2.02.00:IncomeTax.PayAsYouGoWithholding.CreditForTaxWithheldWhereABNNotQuoted.Amount

            Technical Business Rule Format:
            NotMonetary(^PTR63, 'U', 11, 0)
    
            Data Elements:
    
            RP.JAUS:^PTR63 = rvctc2.02.00:IncomeTax.PayAsYouGoWithholding.CreditForTaxWithheldWhereABNNotQuoted.Amount
            */
            assertion = NotMonetary(report.PTR63, @"U", 11, 0);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.400011",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Field incorrect format. Exceeds maximum digits or decimals and/or contains illegal characters",
                    LongDescription = @"Field is in incorrect format for a monetary field. Field either exceeds maximum digits or decimals and/or contains illegal characters and/or for a signed field does not have the sign as the first character",
                    Location = "/xbrli:xbrl/rvctc2.02.00:IncomeTax.PayAsYouGoWithholding.CreditForTaxWithheldWhereABNNotQuoted.Amount[@contextRef='" + report.RPJAUSId + "']",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.PTR.430983" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "PTR63", Value = report.PTR63.GetValueOrDefault().ToString() });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.PTR.430983

            #region VR.ATO.PTR.430987

            /*  VR.ATO.PTR.430987
            Field incorrect format. Exceeds maximum digits or decimals and/or contains illegal characters
    
            Legacy Rule Format:
            IF LENGTH([PTR170]) > 12
                   RETURN VALIDATION MESSAGE
            ENDIF
            
            [PTR170] = PTR:INT:pyid.02.00:Identifiers.TaxAgentClientReference.Text

            Technical Business Rule Format:
            Length(^PTR170) > 12
    
            Data Elements:
    
            INT:^PTR170 = pyid.02.00:Identifiers.TaxAgentClientReference.Text
            */
            assertion = (Length(report.PTR170) > 12);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.430987",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Client's reference must not exceed 12 characters in length",
                    Location = "/xbrli:xbrl/pyid.02.00:Identifiers.TaxAgentClientReference.Text",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.PTR.430987" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "PTR170", Value = report.PTR170 });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.PTR.430987

            #region VR.ATO.PTR.430991

            /*  VR.ATO.PTR.430991
            Monetary field - unsigned, maximum of 13 characters, may include a decimal point and up to 2 decimals
    
            Legacy Rule Format:
            IF [PTR248] <> NULL AND [PTR248] <> MONETARY(U,13,2)
               RETURN VALIDATION MESSAGE
            ENDIF
            
            [PTR248] = PTR:RP:gfati.02.01:TaxOffsetClaim.NationalRentalAffordabilitySchemeEntitlement.Amount

            Technical Business Rule Format:
            NotMonetary(^PTR248, 'U', 13, 2)
    
            Data Elements:
    
            RP:^PTR248 = gfati.02.01:TaxOffsetClaim.NationalRentalAffordabilitySchemeEntitlement.Amount
            */
            assertion = NotMonetary(report.PTR248, @"U", 13, 2);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.400011",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Field incorrect format. Exceeds maximum digits or decimals and/or contains illegal characters",
                    LongDescription = @"Field is in incorrect format for a monetary field. Field either exceeds maximum digits or decimals and/or contains illegal characters and/or for a signed field does not have the sign as the first character",
                    Location = "/xbrli:xbrl/gfati.02.01:TaxOffsetClaim.NationalRentalAffordabilitySchemeEntitlement.Amount",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.PTR.430991" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "PTR248", Value = report.PTR248.GetValueOrDefault().ToString() });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.PTR.430991

            #region VR.ATO.PTR.431006

            /*  VR.ATO.PTR.431006
            Share of net income from trusts, less capital gains, foreign income and franked distributions action code must be present
    
            Legacy Rule Format:
            IF [PTR70] > 0 AND [PTR327] = NULL
                RETURN VALIDATION MESSAGE
            ENDIF
            
            [PTR70] = PTR:RP.JAUS.NonPrim:bafpr1.02.13:Income.TrustShareNetExcludeNetCapitalGainsAndForeignIncomeAndDistributionFranked.Amount
            [PTR327] = PTR:RP.JAUS.NonPrim:pyde.02.05:Party.TrustType.Code

            Technical Business Rule Format:
            ^PTR70 > 0 AND ^PTR327 = NULL
    
            Data Elements:
    
            RP.JAUS.NonPrim:^PTR327 = pyde.02.05:Party.TrustType.Code
    
            RP.JAUS.NonPrim:^PTR70 = bafpr1.02.13:Income.TrustShareNetExcludeNetCapitalGainsAndForeignIncomeAndDistributionFranked.Amount
            */
            assertion = (report.PTR70.GetValueOrDefault() > 0 && report.PTR327 == null);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.PTR.431006",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Share of net income from trusts, less capital gains, foreign income and franked distributions action code must be present",
                    LongDescription = @"If Non-primary production - Share of net income from trusts, less capital gains, foreign income and franked distributions is greater than zero, the Non-primary production - Share of net income from trusts, less capital gains, foreign income and franked distributions iaction code must be present",
                    Location = "/xbrli:xbrl/pyde.02.05:Party.TrustType.Code[@contextRef='" + report.RPJAUSNonPrimId + "']",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.PTR.431006" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "PTR70", Value = report.PTR70.GetValueOrDefault().ToString() });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "PTR327", Value = report.PTR327 });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.PTR.431006

            #region VR.ATO.PTR.431007

            /*  VR.ATO.PTR.431007
            Non-PP - Deductions relating to distribution amounts can not be more than zero
    
            Legacy Rule Format:
            IF [PTR72] > 0 AND ([PTR69] = NULL AND [PTR70] = NULL) 
                RETURN VALIDATION MESSAGE
            ENDIF
            
            [PTR69] = PTR:RP.JAUS.NonPrim:bafpr1.02.04:Income.PartnershipDistributionLessForeignIncome.Amount
            [PTR70] = PTR:RP.JAUS.NonPrim:bafpr1.02.13:Income.TrustShareNetExcludeNetCapitalGainsAndForeignIncomeAndDistributionFranked.Amount
            [PTR72] = PTR:RP.JAUS.NonPrim:bafpr1.02.04:Expense.DistributionDeductionsOther.Amount

            Technical Business Rule Format:
            ^PTR72 > 0 AND (^PTR69 = NULL AND ^PTR70 = NULL)
    
            Data Elements:
    
            RP.JAUS.NonPrim:^PTR72 = bafpr1.02.04:Expense.DistributionDeductionsOther.Amount
    
            RP.JAUS.NonPrim:^PTR69 = bafpr1.02.04:Income.PartnershipDistributionLessForeignIncome.Amount
    
            RP.JAUS.NonPrim:^PTR70 = bafpr1.02.13:Income.TrustShareNetExcludeNetCapitalGainsAndForeignIncomeAndDistributionFranked.Amount
            */
            assertion = (report.PTR72.GetValueOrDefault() > 0 && (report.PTR69 == null && report.PTR70 == null));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.PTR.431007",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Non-PP - Deductions relating to distribution amounts can not be more than zero",
                    LongDescription = @"If there is an amount present at Non-primary production- Deductions relating to amounts shown at Distribution from partnerships less foreign income or Non-primary production - Share of net income from trusts less capital gains, foreign income and franked distributions there must be an amount at either Non-primary production - Distribution from partnerships less foreign income or Non-primary production - Share of net income from trusts less capital gains, foreign income and franked distributions",
                    Location = "/xbrli:xbrl/bafpr1.02.04:Expense.DistributionDeductionsOther.Amount[@contextRef='" + report.RPJAUSNonPrimId + "']",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.PTR.431007" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "PTR72", Value = report.PTR72.GetValueOrDefault().ToString() });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "PTR69", Value = report.PTR69.GetValueOrDefault().ToString() });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "PTR70", Value = report.PTR70.GetValueOrDefault().ToString() });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.PTR.431007

            #region VR.ATO.PTR.431008

            /*  VR.ATO.PTR.431008
            Franking credits from a New Zealand franking company must not be less than distributed amounts
    
            Legacy Rule Format:
            IF SUM([PTR164]) > [PTR107]
                RETURN VALIDATION MESSAGE
            ENDIF
            
            [PTR107] = PTR:RP.JFOREIGN:rvctc2.02.00:IncomeTax.FrankingCredits.ReceivedFromNewZealandCompanies.Amount
            [PTR164] = PTR:RP:rvctc2.02.00:IncomeTax.FrankingCredits.ReceivedFromNewZealandCompanies.Amount IN TUPLE(ptr.0006.lodge.req.02.00:StatementOfDistribution)

            Technical Business Rule Format:
            Sum(^PTR164) > ^PTR107
    
            Data Elements:
    
            RP.JFOREIGN:^PTR107 = rvctc2.02.00:IncomeTax.FrankingCredits.ReceivedFromNewZealandCompanies.Amount
    
            RP:^PTR164 = rvctc2.02.00:IncomeTax.FrankingCredits.ReceivedFromNewZealandCompanies.Amount
            */
            assertion = ((report.StatementOfDistributionCollection == null ? 0 : report.StatementOfDistributionCollection.Sum(f => f.PTR164.GetValueOrDefault())) > report.PTR107.GetValueOrDefault());
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.PTR.431008",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Franking credits from a New Zealand franking company must not be less than distributed amounts",
                    LongDescription = @"The sum of Australian franking credits from a New Zealand franking company in the Statements of distribution must not be greater than Australian franking credits from a New Zealand franking company reported at Other assessable foreign income",
                    Location = "/xbrli:xbrl/rvctc2.02.00:IncomeTax.FrankingCredits.ReceivedFromNewZealandCompanies.Amount[@contextRef='" + report.RPJFOREIGNId + "']",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.PTR.431008" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "PTR164", Value = "PTR164" });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "PTR107", Value = report.PTR107.GetValueOrDefault().ToString() });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.PTR.431008

            #region VR.ATO.PTR.431009

            /*  VR.ATO.PTR.431009
            Share of net income from trusts action code must be present
    
            Legacy Rule Format:
            IF [PTR66] > 0 AND [PTR299] = NULL
                RETURN VALIDATION MESSAGE
            ENDIF
            
            [PTR66] = PTR:RP.JAUS.Prim:bafpr1.02.14:Income.TrustShareNet.Amount
            [PTR299] = PTR:RP.JAUS.Prim:pyde.02.05:Party.TrustType.Code

            Technical Business Rule Format:
            ^PTR66 > 0 AND ^PTR299 = NULL
    
            Data Elements:
    
            RP.JAUS.Prim:^PTR299 = pyde.02.05:Party.TrustType.Code
    
            RP.JAUS.Prim:^PTR66 = bafpr1.02.14:Income.TrustShareNet.Amount
            */
            assertion = (report.PTR66.GetValueOrDefault() > 0 && report.PTR299 == null);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.PTR.431009",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Share of net income from trusts action code must be present",
                    LongDescription = @"If Primary production - Share of net income  from trusts is greater than zero, the Primary production - Share of net income trusts action code must be present",
                    Location = "/xbrli:xbrl/pyde.02.05:Party.TrustType.Code[@contextRef='" + report.RPJAUSPrimId + "']",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.PTR.431009" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "PTR66", Value = report.PTR66.GetValueOrDefault().ToString() });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "PTR299", Value = report.PTR299 });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.PTR.431009

            #region VR.ATO.PTR.431010

            /*  VR.ATO.PTR.431010
            Monetary field must be unsigned, maximum of 11 numeric characters, no decimals
    
            Legacy Rule Format:
            IF (bafpr3.xx.xx:Expense.DepreciatingAssets.SmallBusinessEntity.Amount <> NULL) AND (bafpr3.xx.xx:Expense.DepreciatingAssets.SmallBusinessEntity.Amount <> MONETARY(U,11,0))
              RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            NotMonetary(^PTR141, 'U', 11, 0)
    
            Data Elements:
    
            RP.LowCost:^PTR141 = bafpr3.02.11:Expense.DepreciatingAssets.SmallBusinessEntity.Amount
            */
            assertion = NotMonetary(report.PTR141, @"U", 11, 0);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.400011",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Field incorrect format. Exceeds maximum digits or decimals and/or contains illegal characters",
                    LongDescription = @"Field is in incorrect format for a monetary field. Field either exceeds maximum digits or decimals and/or contains illegal characters and/or for a signed field does not have the sign as the first character",
                    Location = "/xbrli:xbrl/bafpr3.02.11:Expense.DepreciatingAssets.SmallBusinessEntity.Amount[@contextRef='" + report.RPLowCostId + "']",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.PTR.431010" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "PTR141", Value = report.PTR141.GetValueOrDefault().ToString() });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.PTR.431010

            #region VR.ATO.PTR.431011

            /*  VR.ATO.PTR.431011
            Monetary field, must be unsigned, maxuimum of 11 chars, no decimals
    
            Legacy Rule Format:
            IF (bafpr3.xx.xx:Expense.DepreciatingAssets.SmallBusinessEntity.Amount <> NULL) AND (bafpr3.xx.xx:Expense.DepreciatingAssets.SmallBusinessEntity.Amount <> MONETARY(U,11,0))
              RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            NotMonetary(^PTR142, 'U', 11, 0)
    
            Data Elements:
    
            RP.GeneralPool:^PTR142 = bafpr3.02.11:Expense.DepreciatingAssets.SmallBusinessEntity.Amount
            */
            assertion = NotMonetary(report.PTR142, @"U", 11, 0);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.400011",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Field incorrect format. Exceeds maximum digits or decimals and/or contains illegal characters",
                    LongDescription = @"Field is in incorrect format for a monetary field. Field either exceeds maximum digits or decimals and/or contains illegal characters and/or for a signed field does not have the sign as the first character",
                    Location = "/xbrli:xbrl/bafpr3.02.11:Expense.DepreciatingAssets.SmallBusinessEntity.Amount[@contextRef='" + report.RPGeneralPoolId + "']",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.PTR.431011" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "PTR142", Value = report.PTR142.GetValueOrDefault().ToString() });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.PTR.431011

            #region VR.ATO.PTR.431012

            /*  VR.ATO.PTR.431012
            Invalid context. The number of Reporting party:LowCost contexts must not exceed 1
    
            Legacy Rule Format:
            IF COUNT(CONTEXT(RP.LowCost)) > 1 
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            Count(^Context) > 1
    
            Data Elements:
    
            ^Context = RP:RP.LowCost
            */
            assertion = (Count(report.RPLowCostCount) > 1);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.PTR.431012",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Invalid context. The number of Reporting party:LowCost contexts must not exceed 1",
                    Location = "/xbrli:xbrl/xbrli:context" + OccurrenceIndex(report.RPLowCostLastOccurrenceIndex),
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.PTR.431012" } },
                };
                response.Add(processMessage);
            }
            #endregion // VR.ATO.PTR.431012

            #region VR.ATO.PTR.431013

            /*  VR.ATO.PTR.431013
            Invalid context. The number of Reporting party:GeneralPool contexts must not exceed 1
    
            Legacy Rule Format:
            IF COUNT(CONTEXT(RP.GeneralPool)) > 1 
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            Count(^Context) > 1
    
            Data Elements:
    
            ^Context = RP:RP.GeneralPool
            */
            assertion = (Count(report.RPGeneralPoolCount) > 1);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.PTR.431013",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Invalid context. The number of Reporting party:GeneralPool contexts must not exceed 1",
                    Location = "/xbrli:xbrl/xbrli:context" + OccurrenceIndex(report.RPGeneralPoolLastOccurrenceIndex),
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.PTR.431013" } },
                };
                response.Add(processMessage);
            }
            #endregion // VR.ATO.PTR.431013

            #region VR.ATO.PTR.431030

            /*  VR.ATO.PTR.431030
            PP - Distribution from trusts amount must be greater than zero if action code is present
    
            Legacy Rule Format:
            IF [PTR299] <> NULL AND ([PTR66] = NULL OR [PTR66] <= 0) 
                RETURN VALIDATION MESSAGE
            ENDIF
            
            [PTR66] = PTR:RP.JAUS.Prim:bafpr1.02.14:Income.TrustShareNet.Amount
            [PTR299] = PTR:RP.JAUS.Prim:pyde.02.05:Party.TrustType.Code

            Technical Business Rule Format:
            ^PTR299 <> NULL AND (^PTR66 = NULL OR ^PTR66 <= 0)
    
            Data Elements:
    
            RP.JAUS.Prim:^PTR66 = bafpr1.02.14:Income.TrustShareNet.Amount
    
            RP.JAUS.Prim:^PTR299 = pyde.02.05:Party.TrustType.Code
            */
            assertion = (report.PTR299 != null && (report.PTR66 == null || report.PTR66.GetValueOrDefault() <= 0));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.PTR.431030",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"PP - Share of net income from trusts amount must be greater than zero if action code is present",
                    LongDescription = @"If the Primary production - Share of net income  from trusts action code is present, then the Partnership and Trusts - Primary production - Share of net income  from trusts must be present and greater than zero. An action code is not required for negative or zero amounts",
                    Location = "/xbrli:xbrl/bafpr1.02.14:Income.TrustShareNet.Amount",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.PTR.431030" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "PTR299", Value = report.PTR299 });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "PTR66", Value = report.PTR66.GetValueOrDefault().ToString() });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.PTR.431030

            #region VR.ATO.PTR.431031

            /*  VR.ATO.PTR.431031
            Total distributions and income from a trust must not be less than zero
    
            Legacy Rule Format:
            IF ([PTR105] = NULL OR [PTR105] = 0) AND (([PTR66] + [PTR70] + [PTR322] + [PTR98]  + [PTR100] + [PTR106] + [PTR107]) < 0)
                RETURN VALIDATION MESSAGE
            ENDIF
            
            [PTR66] = PTR:RP.JAUS.Prim:bafpr1.02.14:Income.TrustShareNet.Amount
            [PTR70] = PTR:RP.JAUS.NonPrim:bafpr1.02.13:Income.TrustShareNetExcludeNetCapitalGainsAndForeignIncomeAndDistributionFranked.Amount
            [PTR98] = PTR:RP.JFOREIGN.CFCLC:bafpr3.02.02:Income.InternationalDealings.Attributed.Amount
            [PTR100] = PTR:RP.JFOREIGN.CFCUC:bafpr3.02.02:Income.InternationalDealings.Attributed.Amount
            [PTR105] = PTR:RP.JFOREIGN:bafpr1.02.23:Income.Net.Amount
            [PTR106] = PTR:RP.JFOREIGN:bafpr3.02.18:Income.InternationalDealings.TaxOffset.Amount
            [PTR107] = PTR:RP.JFOREIGN:rvctc2.02.00:IncomeTax.FrankingCredits.ReceivedFromNewZealandCompanies.Amount
            [PTR322] = PTR:RP.JAUS.NonPrim:bafpr1.02.04:Income.TrustDistributionFranked.Amount

            Technical Business Rule Format:
            (^PTR105 = NULL OR ^PTR105 = 0) AND ((^PTR66 + ^PTR70 + ^PTR322 + ^PTR98  + ^PTR100 + ^PTR106 + ^PTR107) < 0)
    
            Data Elements:
    
            RP.JAUS.Prim:^PTR66 = bafpr1.02.14:Income.TrustShareNet.Amount
    
            RP.JAUS.NonPrim:^PTR70 = bafpr1.02.13:Income.TrustShareNetExcludeNetCapitalGainsAndForeignIncomeAndDistributionFranked.Amount
    
            RP.JFOREIGN.CFCLC:^PTR98 = bafpr3.02.02:Income.InternationalDealings.Attributed.Amount
    
            RP.JFOREIGN.CFCUC:^PTR100 = bafpr3.02.02:Income.InternationalDealings.Attributed.Amount
    
            RP.JFOREIGN:^PTR105 = bafpr1.02.23:Income.Net.Amount
    
            RP.JFOREIGN:^PTR106 = bafpr3.02.18:Income.InternationalDealings.TaxOffset.Amount
    
            RP.JFOREIGN:^PTR107 = rvctc2.02.00:IncomeTax.FrankingCredits.ReceivedFromNewZealandCompanies.Amount
    
            RP.JAUS.NonPrim:^PTR322 = bafpr1.02.04:Income.TrustDistributionFranked.Amount
            */
            assertion = ((report.PTR105 == null || report.PTR105 == 0) && report.PTR66.GetValueOrDefault() + report.PTR70.GetValueOrDefault() + report.PTR322.GetValueOrDefault() + report.PTR98.GetValueOrDefault() + report.PTR100.GetValueOrDefault() + report.PTR106.GetValueOrDefault() + report.PTR107.GetValueOrDefault() < 0);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.PTR.431031",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Total distributions and income from a trust must not be less than zero",
                    LongDescription = @"The sum of Primary production - Share of net income from trusts, Non-primary production - Share of net income from trusts, less capital gains, foreign income and franked distributions, Franked distributions from trusts, Attributed foreign income, General net foreign source income, Australian franking credits from a New Zealand franking company can not be less than zero. This would indicate that an overall trust loss is being distributed from a trust to a partnership. Adjust distributed amounts until the overall negative amount no longer exists (i.e. make the sum equal to zero)",
                    Location = "/xbrli:xbrl/bafpr1.02.14:Income.TrustShareNet.Amount",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.PTR.431031" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "PTR105", Value = report.PTR105.GetValueOrDefault().ToString() });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "PTR66", Value = report.PTR66.GetValueOrDefault().ToString() });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "PTR70", Value = report.PTR70.GetValueOrDefault().ToString() });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "PTR322", Value = report.PTR322.GetValueOrDefault().ToString() });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "PTR98", Value = report.PTR98.GetValueOrDefault().ToString() });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "PTR100", Value = report.PTR100.GetValueOrDefault().ToString() });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "PTR106", Value = report.PTR106.GetValueOrDefault().ToString() });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "PTR107", Value = report.PTR107.GetValueOrDefault().ToString() });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.PTR.431031

            #region VR.ATO.PTR.431032

            /*  VR.ATO.PTR.431032
            Monetary field may be signed, maximum of 11 chars, no decimals
    
            Legacy Rule Format:
            IF [PTR66] <> NULL AND [PTR66] <> MONETARY(S,11,0)
               RETURN VALIDATION MESSAGE
            ENDIF
            
            [PTR66] = PTR:RP.JAUS.Prim:bafpr1.02.14:Income.TrustShareNet.Amount

            Technical Business Rule Format:
            NotMonetary(^PTR66, 'S', 11, 0)
    
            Data Elements:
    
            RP.JAUS.Prim:^PTR66 = bafpr1.02.14:Income.TrustShareNet.Amount
            */
            assertion = NotMonetary(report.PTR66, @"S", 11, 0);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.400011",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Field incorrect format. Exceeds maximum digits or decimals and/or contains illegal characters",
                    LongDescription = @"Field is in incorrect format for a monetary field. Field either exceeds maximum digits or decimals and/or contains illegal characters and/or for a signed field does not have the sign as the first character",
                    Location = "/xbrli:xbrl/bafpr1.02.14:Income.TrustShareNet.Amount",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.PTR.431032" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "PTR66", Value = report.PTR66.GetValueOrDefault().ToString() });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.PTR.431032

            #region VR.ATO.PTR.431033

            /*  VR.ATO.PTR.431033
            Mandatory field not supplied
    
            Legacy Rule Format:
            IF [PTR110] = NULL
               RETURN VALIDATION MESSAGE
            ENDIF
            
            [PTR110] = PTR:RP:bafpr1.02.13:Income.PersonalServicesIncomeIncluded.Indicator

            Technical Business Rule Format:
            ^PTR110 = NULL
    
            Data Elements:
    
            RP:^PTR110 = bafpr1.02.13:Income.PersonalServicesIncomeIncluded.Indicator
            */
            assertion = (report.PTR110 == null);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.001001",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Mandatory field not supplied",
                    Location = "/xbrli:xbrl/bafpr1.02.13:Income.PersonalServicesIncomeIncluded.Indicator",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.PTR.431033" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "PTR110", Value = GetValueOrEmpty(report.PTR110) });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.PTR.431033

            #region VR.ATO.PTR.431043

            /*  VR.ATO.PTR.431043
            Non-PP - Distribution from trusts amount must be present
    
            Legacy Rule Format:
            IF [PTR327] <> NULL AND ([PTR70] = NULL OR [PTR70] = 0) 
                RETURN VALIDATION MESSAGE
            ENDIF
            
            [PTR70] = PTR:RP.JAUS.NonPrim:bafpr1.02.13:Income.TrustShareNetExcludeNetCapitalGainsAndForeignIncomeAndDistributionFranked.Amount
            [PTR327] = PTR:RP.JAUS.NonPrim:pyde.02.05:Party.TrustType.Code

            Technical Business Rule Format:
            ^PTR327 <> NULL AND (^PTR70 = NULL OR ^PTR70 = 0)
    
            Data Elements:
    
            RP.JAUS.NonPrim:^PTR70 = bafpr1.02.13:Income.TrustShareNetExcludeNetCapitalGainsAndForeignIncomeAndDistributionFranked.Amount
    
            RP.JAUS.NonPrim:^PTR327 = pyde.02.05:Party.TrustType.Code
            */
            assertion = (report.PTR327 != null && (report.PTR70 == null || report.PTR70 == 0));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.PTR.431043",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Non-PP - Distribution from trusts amount must be present",
                    LongDescription = @"If Non-primary production - Distribution from trusts action code is present, then Non-primary production - Share of net income from trusts, less capital gains, foreign income and franked distributions must be greater than zero.  A trusts action code is not required for negative or zero amounts",
                    Location = "/xbrli:xbrl/bafpr1.02.13:Income.TrustShareNetExcludeNetCapitalGainsAndForeignIncomeAndDistributionFranked.Amount",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.PTR.431043" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "PTR327", Value = report.PTR327 });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "PTR70", Value = report.PTR70.GetValueOrDefault().ToString() });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.PTR.431043

            #region VR.ATO.PTR.431044

            /*  VR.ATO.PTR.431044
            Monetary field may be signed, maximum of 11 chars, no decimals
    
            Legacy Rule Format:
            IF [PTR70] <> NULL AND [PTR70] <> MONETARY(S,11,0)
               RETURN VALIDATION MESSAGE
            ENDIF
            
            [PTR70] = PTR:RP.JAUS.NonPrim:bafpr1.02.13:Income.TrustShareNetExcludeNetCapitalGainsAndForeignIncomeAndDistributionFranked.Amount

            Technical Business Rule Format:
            NotMonetary(^PTR70, 'S', 11, 0)
    
            Data Elements:
    
            RP.JAUS.NonPrim:^PTR70 = bafpr1.02.13:Income.TrustShareNetExcludeNetCapitalGainsAndForeignIncomeAndDistributionFranked.Amount
            */
            assertion = NotMonetary(report.PTR70, @"S", 11, 0);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.400011",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Field incorrect format. Exceeds maximum digits or decimals and/or contains illegal characters",
                    LongDescription = @"Field is in incorrect format for a monetary field. Field either exceeds maximum digits or decimals and/or contains illegal characters and/or for a signed field does not have the sign as the first character",
                    Location = "/xbrli:xbrl/bafpr1.02.13:Income.TrustShareNetExcludeNetCapitalGainsAndForeignIncomeAndDistributionFranked.Amount",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.PTR.431044" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "PTR70", Value = report.PTR70.GetValueOrDefault().ToString() });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.PTR.431044

            #region VR.ATO.PTR.432001

            /*  VR.ATO.PTR.432001
            The sum of Share of net small business income in the distribution statement cannot be greater than Net small business income
    
            Legacy Rule Format:
            IF SUM([PTR353]) > [PTR352]
               RETURN VALIDATION MESSAGE
            ENDIF
            
            [PTR352] = PTR:RP.JAUS:bafpr1.02.04:Income.SmallBusinessEntityNet.Amount
            [PTR353] = PTR:RP:bafpr1.02.04:Income.SmallBusinessEntityNet.Amount IN TUPLE(ptr.0006.lodge.req.02.00:StatementOfDistribution)

            Technical Business Rule Format:
            Sum(^PTR353) > ^PTR352
    
            Data Elements:
    
            RP:^PTR353 = bafpr1.02.04:Income.SmallBusinessEntityNet.Amount
    
            RP.JAUS:^PTR352 = bafpr1.02.04:Income.SmallBusinessEntityNet.Amount
            */
            assertion = ((report.StatementOfDistributionCollection == null ? 0 : report.StatementOfDistributionCollection.Sum(f => f.PTR353.GetValueOrDefault())) > report.PTR352.GetValueOrDefault());
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.PTR.432001",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Net small business income is incorrect",
                    LongDescription = @"The sum of Share of net small business income in the distribution statement cannot be greater than Net small business income",
                    Location = "/xbrli:xbrl/ptr.0006.lodge.req.02.00:StatementOfDistribution/bafpr1.02.04:Income.SmallBusinessEntityNet.Amount",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.PTR.432001" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "PTR353", Value = "PTR353" });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "PTR352", Value = report.PTR352.GetValueOrDefault().ToString() });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.PTR.432001

            #region VR.ATO.PTR.438052

            /*  VR.ATO.PTR.438052
            Monetary field may be signed, maximum of 11 chars, no decimals
    
            Legacy Rule Format:
            IF [PTR343] <> NULL AND [PTR343] <> MONETARY(S,11,0)
               RETURN VALIDATION MESSAGE
            ENDIF
            
            [PTR343] = PTR:RP.JAUS:bafpr2.02.16:Expense.Operating.CostOfSales.Amount

            Technical Business Rule Format:
            NotMonetary(^PTR343, 'S', 11, 0)
    
            Data Elements:
    
            RP.JAUS:^PTR343 = bafpr2.02.16:Expense.Operating.CostOfSales.Amount
            */
            assertion = NotMonetary(report.PTR343, @"S", 11, 0);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.400011",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Field incorrect format. Exceeds maximum digits or decimals and/or contains illegal characters",
                    LongDescription = @"Field is in incorrect format for a monetary field. Field either exceeds maximum digits or decimals and/or contains illegal characters and/or for a signed field does not have the sign as the first character",
                    Location = "/xbrli:xbrl/bafpr2.02.16:Expense.Operating.CostOfSales.Amount",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.PTR.438052" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "PTR343", Value = report.PTR343.GetValueOrDefault().ToString() });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.PTR.438052

            #region VR.ATO.PTR.438053

            /*  VR.ATO.PTR.438053
            Monetary field must be unsigned, maximum of 11 numeric characters, no decimals
    
            Legacy Rule Format:
            IF [PTR344] <> NULL AND [PTR344] <> MONETARY(U,11,0)
               RETURN VALIDATION MESSAGE
            ENDIF
            
            [PTR344] = PTR:RP.JAUS:bafpr2.02.16:Expense.Operating.BadDebts.Amount

            Technical Business Rule Format:
            NotMonetary(^PTR344, 'U', 11, 0)
    
            Data Elements:
    
            RP.JAUS:^PTR344 = bafpr2.02.16:Expense.Operating.BadDebts.Amount
            */
            assertion = NotMonetary(report.PTR344, @"U", 11, 0);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.400011",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Field incorrect format. Exceeds maximum digits or decimals and/or contains illegal characters",
                    LongDescription = @"Field is in incorrect format for a monetary field. Field either exceeds maximum digits or decimals and/or contains illegal characters and/or for a signed field does not have the sign as the first character",
                    Location = "/xbrli:xbrl/bafpr2.02.16:Expense.Operating.BadDebts.Amount",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.PTR.438053" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "PTR344", Value = report.PTR344.GetValueOrDefault().ToString() });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.PTR.438053

            #region VR.ATO.PTR.438054

            /*  VR.ATO.PTR.438054
            Monetary field must be unsigned, maximum of 11 numeric characters, no decimals
    
            Legacy Rule Format:
            IF [PTR345] <> NULL AND [PTR345] <> MONETARY(U,11,0)
               RETURN VALIDATION MESSAGE
            ENDIF
            
            [PTR345] = PTR:RP.JAUS:bafpr2.02.16:Expense.Operating.Rent.Amount

            Technical Business Rule Format:
            NotMonetary(^PTR345, 'U', 11, 0)
    
            Data Elements:
    
            RP.JAUS:^PTR345 = bafpr2.02.16:Expense.Operating.Rent.Amount
            */
            assertion = NotMonetary(report.PTR345, @"U", 11, 0);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.400011",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Field incorrect format. Exceeds maximum digits or decimals and/or contains illegal characters",
                    LongDescription = @"Field is in incorrect format for a monetary field. Field either exceeds maximum digits or decimals and/or contains illegal characters and/or for a signed field does not have the sign as the first character",
                    Location = "/xbrli:xbrl/bafpr2.02.16:Expense.Operating.Rent.Amount",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.PTR.438054" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "PTR345", Value = report.PTR345.GetValueOrDefault().ToString() });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.PTR.438054

            #region VR.ATO.PTR.438055

            /*  VR.ATO.PTR.438055
            Monetary field must be unsigned, maximum of 11 numeric characters, no decimals
    
            Legacy Rule Format:
            IF [PTR346] <> NULL AND [PTR346] <> MONETARY(U,11,0)
               RETURN VALIDATION MESSAGE
            ENDIF
            
            [PTR346] = PTR:RP.JAUS:bafpr2.02.17:Expense.Operating.Interest.Amount

            Technical Business Rule Format:
            NotMonetary(^PTR346, 'U', 11, 0)
    
            Data Elements:
    
            RP.JAUS:^PTR346 = bafpr2.02.17:Expense.Operating.Interest.Amount
            */
            assertion = NotMonetary(report.PTR346, @"U", 11, 0);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.400011",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Field incorrect format. Exceeds maximum digits or decimals and/or contains illegal characters",
                    LongDescription = @"Field is in incorrect format for a monetary field. Field either exceeds maximum digits or decimals and/or contains illegal characters and/or for a signed field does not have the sign as the first character",
                    Location = "/xbrli:xbrl/bafpr2.02.17:Expense.Operating.Interest.Amount",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.PTR.438055" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "PTR346", Value = report.PTR346.GetValueOrDefault().ToString() });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.PTR.438055

            #region VR.ATO.PTR.438056

            /*  VR.ATO.PTR.438056
            Monetary field must be unsigned, maximum of 11 numeric characters, no decimals
    
            Legacy Rule Format:
            IF [PTR347] <> NULL AND [PTR347] <> MONETARY(U,11,0)
               RETURN VALIDATION MESSAGE
            ENDIF
            
            [PTR347] = PTR:RP.JAUS:bafpr1.02.27:Expense.Royalties.Amount

            Technical Business Rule Format:
            NotMonetary(^PTR347, 'U', 11, 0)
    
            Data Elements:
    
            RP.JAUS:^PTR347 = bafpr1.02.27:Expense.Royalties.Amount
            */
            assertion = NotMonetary(report.PTR347, @"U", 11, 0);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.400011",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Field incorrect format. Exceeds maximum digits or decimals and/or contains illegal characters",
                    LongDescription = @"Field is in incorrect format for a monetary field. Field either exceeds maximum digits or decimals and/or contains illegal characters and/or for a signed field does not have the sign as the first character",
                    Location = "/xbrli:xbrl/bafpr1.02.27:Expense.Royalties.Amount[@contextRef='" + report.RPJAUSId + "']",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.PTR.438056" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "PTR347", Value = report.PTR347.GetValueOrDefault().ToString() });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.PTR.438056

            #region VR.ATO.PTR.438057

            /*  VR.ATO.PTR.438057
            Monetary field must be unsigned, maximum of 11 numeric characters, no decimals
    
            Legacy Rule Format:
            IF [PTR348] <> NULL AND [PTR348] <> MONETARY(U,11,0)
               RETURN VALIDATION MESSAGE
            ENDIF
            
            [PTR348] = PTR:RP.JAUS:bafpr2.02.16:Expense.Operating.MotorVehicle.Amount

            Technical Business Rule Format:
            NotMonetary(^PTR348, 'U', 11, 0)
    
            Data Elements:
    
            RP.JAUS:^PTR348 = bafpr2.02.16:Expense.Operating.MotorVehicle.Amount
            */
            assertion = NotMonetary(report.PTR348, @"U", 11, 0);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.400011",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Field incorrect format. Exceeds maximum digits or decimals and/or contains illegal characters",
                    LongDescription = @"Field is in incorrect format for a monetary field. Field either exceeds maximum digits or decimals and/or contains illegal characters and/or for a signed field does not have the sign as the first character",
                    Location = "/xbrli:xbrl/bafpr2.02.16:Expense.Operating.MotorVehicle.Amount",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.PTR.438057" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "PTR348", Value = report.PTR348.GetValueOrDefault().ToString() });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.PTR.438057

            #region VR.ATO.PTR.438058

            /*  VR.ATO.PTR.438058
            Monetary field must be unsigned, maximum of 11 numeric characters, no decimals
    
            Legacy Rule Format:
            IF [PTR349] <> NULL AND [PTR349] <> MONETARY(U,11,0)
               RETURN VALIDATION MESSAGE
            ENDIF
            
            [PTR349] = PTR:RP.JAUS:bafpr2.02.16:Expense.Operating.RepairsAndMaintenance.Amount

            Technical Business Rule Format:
            NotMonetary(^PTR349, 'U', 11, 0)
    
            Data Elements:
    
            RP.JAUS:^PTR349 = bafpr2.02.16:Expense.Operating.RepairsAndMaintenance.Amount
            */
            assertion = NotMonetary(report.PTR349, @"U", 11, 0);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.400011",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Field incorrect format. Exceeds maximum digits or decimals and/or contains illegal characters",
                    LongDescription = @"Field is in incorrect format for a monetary field. Field either exceeds maximum digits or decimals and/or contains illegal characters and/or for a signed field does not have the sign as the first character",
                    Location = "/xbrli:xbrl/bafpr2.02.16:Expense.Operating.RepairsAndMaintenance.Amount",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.PTR.438058" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "PTR349", Value = report.PTR349.GetValueOrDefault().ToString() });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.PTR.438058

            #region VR.ATO.PTR.438059

            /*  VR.ATO.PTR.438059
            Monetary field must be unsigned, maximum of 11 numeric characters, no decimals
    
            Legacy Rule Format:
            IF [PTR350] <> NULL AND [PTR350] <> MONETARY(U,11,0)
               RETURN VALIDATION MESSAGE
            ENDIF
            
            [PTR350] = PTR:RP.JAUS:rvctc2.02.13:IncomeTax.PayAsYouGoWithholding.CreditForAmountsWithheldFromForeignResidents.Amount

            Technical Business Rule Format:
            NotMonetary(^PTR350, 'U', 11, 0)
    
            Data Elements:
    
            RP.JAUS:^PTR350 = rvctc2.02.13:IncomeTax.PayAsYouGoWithholding.CreditForAmountsWithheldFromForeignResidents.Amount
            */
            assertion = NotMonetary(report.PTR350, @"U", 11, 0);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.400011",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Field incorrect format. Exceeds maximum digits or decimals and/or contains illegal characters",
                    LongDescription = @"Field is in incorrect format for a monetary field. Field either exceeds maximum digits or decimals and/or contains illegal characters and/or for a signed field does not have the sign as the first character",
                    Location = "/xbrli:xbrl/rvctc2.02.13:IncomeTax.PayAsYouGoWithholding.CreditForAmountsWithheldFromForeignResidents.Amount[@contextRef='" + report.RPJAUSId + "']",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.PTR.438059" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "PTR350", Value = report.PTR350.GetValueOrDefault().ToString() });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.PTR.438059

            #region VR.ATO.PTR.438070

            /*  VR.ATO.PTR.438070
            Monetary field must be unsigned, maximum of 11 numeric characters, no decimals
    
            Legacy Rule Format:
            IF [PTR352] <> NULL AND [PTR352] <> MONETARY(U,11,0)
               RETURN VALIDATION  MESSAGE
            ENDIF
            
            [PTR352] = PTR:RP.JAUS:bafpr1.02.04:Income.SmallBusinessEntityNet.Amount

            Technical Business Rule Format:
            NotMonetary(^PTR352, 'U', 11, 0)
    
            Data Elements:
    
            RP.JAUS:^PTR352 = bafpr1.02.04:Income.SmallBusinessEntityNet.Amount
            */
            assertion = NotMonetary(report.PTR352, @"U", 11, 0);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.400011",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Field incorrect format. Exceeds maximum digits or decimals and/or contains illegal characters",
                    LongDescription = @"Field is in incorrect format for a monetary field. Field either exceeds maximum digits or decimals and/or contains illegal characters and/or for a signed field does not have the sign as the first character",
                    Location = "/xbrli:xbrl/bafpr1.02.04:Income.SmallBusinessEntityNet.Amount[@contextRef='" + report.RPJAUSId + "']",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.PTR.438070" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "PTR352", Value = report.PTR352.GetValueOrDefault().ToString() });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.PTR.438070

            #region VR.ATO.PTR.438075

            /*  VR.ATO.PTR.438075
            Invalid value. Incorrect action code has been provided.
    
            Legacy Rule Format:
            IF [PTR56] <> NULL AND ([PTR56] = "T" OR [PTR56] = "O")
               RETURN VALIDATION MESSAGE
            ENDIF
            
            [PTR56] = PTR:RP.JAUS:bafpr2.02.04:Expense.Operating.MotorVehicleTypeOrClaimMethod.Code

            Technical Business Rule Format:
            ^PTR56 <> NULL AND (^PTR56 = 'T' OR ^PTR56 = 'O')
    
            Data Elements:
    
            RP.JAUS:^PTR56 = bafpr2.02.04:Expense.Operating.MotorVehicleTypeOrClaimMethod.Code
            */
            assertion = (report.PTR56 != null && (report.PTR56 == @"T" || report.PTR56 == @"O"));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.PTR.438075",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Field contains incorrect value. Motor vehicle expenses action code cannot contain values T or O",
                    Location = "/xbrli:xbrl/bafpr2.02.04:Expense.Operating.MotorVehicleTypeOrClaimMethod.Code",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.PTR.438075" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "PTR56", Value = report.PTR56 });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.PTR.438075

            #region VR.ATO.PTR.438080

            /*  VR.ATO.PTR.438080
            Year of return must be either the current or next (future) lodgment year associated with this service. If a future year lodgment is required, then it must be lodged prior to the 1st of July of that future year, unless it is an amendment. After the 1st July of that future year, a return for that lodgment year must be submitted using the new service associated to that specific lodgment year.
    
            Legacy Rule Format:
            IF ([PTR1] <> SET(2017, 2018))
            OR ([PTR1] = 2018 AND DATE(Today) > 2018-06-30 
            AND [PTR316] <> TRUE)
             RETURN VALIDATION MESSAGE
            ENDIF
            
            [PTR1] = PTR:RP:pyin.02.37:Report.TargetFinancial.Year
            [PTR316] = PTR:RP:pyin.02.28:Report.Amendment.Indicator

            Technical Business Rule Format:
            (NotInSet(^PTR1, '"2017", "2018"')) OR (^PTR1 = 2018 AND Today() > AsDate('2018-06-30') AND ^PTR316 <> TRUE)
    
            Data Elements:
    
            RP:^PTR1 = pyin.02.37:Report.TargetFinancial.Year
    
            RP:^PTR316 = pyin.02.28:Report.Amendment.Indicator
            */
            assertion = (!(IsMatch(report.PTR1.GetValueOrDefault().ToString(), @"^(2017|2018)$", RegexOptions.IgnoreCase)) || report.PTR1.GetValueOrDefault() == 2018 && DateTime.Now.Date > AsDate(@"2018-06-30") && report.PTR316 != true);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.438055",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Year of return is invalid",
                    LongDescription = @"Year of return must be either the current or next (future) lodgment year associated with this service. If a future year lodgment is required, then it must be lodged prior to the 1st of July of that future year, unless it is an amendment. After the 1st July of that future year, a return for that lodgment year must be submitted using the new service associated to that specific lodgment year.",
                    Location = "/xbrli:xbrl/pyin.02.37:Report.TargetFinancial.Year",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.PTR.438080" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "PTR1", Value = report.PTR1.GetValueOrDefault().ToString() });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "PTR316", Value = GetValueOrEmpty(report.PTR316) });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.PTR.438080

            #region VR.ATO.PTR.438081

            /*  VR.ATO.PTR.438081
            The Statement of distribution must be present and must not exceed 160
    
            Legacy Rule Format:
            IF COUNT(ptr.0006.lodge.req.xx.xx:StatementOfDistribution) < 1 OR COUNT(ptr.0006.lodge.req.xx.xx:StatementOfDistribution) > 160 
                RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            Count(^StatementOfDistribution) < 1 OR Count(^StatementOfDistribution) > 160
    
            Data Elements:
    
            ^StatementOfDistribution = ptr.0006.lodge.req.02.00:StatementOfDistribution
            */
            assertion = (Count(report.StatementOfDistributionCollectionCount) < 1 || Count(report.StatementOfDistributionCollectionCount) > 160);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.PTR.438081",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"The Statement of distribution must be present and must not exceed 160",
                    Location = "/xbrli:xbrl/ptr.0006.lodge.req.02.00:StatementOfDistribution/lrla.02.00:Remuneration.PartnerWithNoControlAndShareDisposalInPartnershipIncome.Indicator",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.PTR.438081" } },
                };
                response.Add(processMessage);
            }
            #endregion // VR.ATO.PTR.438081

            #region Repeating report.StatementOfDistributionCollection
            if (report.StatementOfDistributionCollection != null)
            {
                int itemIndex1 = 0;
                foreach (PTR2017.StatementOfDistribution statementOfDistribution in report.StatementOfDistributionCollection)
                {
                    itemIndex1++;

                    #region VR.ATO.GEN.000406

                    /*  VR.ATO.GEN.000406
                    Address line 1 contains incorrect 'care of' reference.  The only acceptable 'care of' reference is C/-
    
                    Legacy Rule Format:
                    IF (pyde.xx.xx:AddressDetails.Line1.Text <> NULLORBLANK) AND (pyde.xx.xx:AddressDetails.Line1.Text STARTSWITH SET("C/O ","C/ ","Care Of ","CO ")) OR (pyde.xx.xx:AddressDetails.Line1.Text CONTAINS SET(" C/O "," C/ "," Care Of "))
                       RETURN VALIDATION MESSAGE
                    ENDIF
        
                    Technical Business Rule Format:
                    (^PTR151 <> BLANK) AND (StartsWithSet(^PTR151,'"C/O ","C/ ","Care Of ","CO "')) OR (ContainsSet(^PTR151, '" C/O "," C/ "," Care Of "'))
            
                    Data Elements:
            
                    RP:^PTR151 = pyde.02.00:AddressDetails.Line1.Text WHERE [AddressDetails.Usage.Code='BUS'].[AddressDetails.Currency.Code='C']
                    */
                    assertion = (string.IsNullOrWhiteSpace(statementOfDistribution.PTR151) != true && IsMatch(statementOfDistribution.PTR151, @"^(C/O |C/ |Care Of |CO )", RegexOptions.IgnoreCase) || IsMatch(statementOfDistribution.PTR151, @" C/O | C/ | Care Of ", RegexOptions.IgnoreCase));
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.GEN.000406",
                            Severity = ProcessMessageSeverity.Error,
                            Description = @"Address line 1 contains incorrect 'care of' reference.  The only acceptable 'care of' reference is C/-",
                            Location = "/xbrli:xbrl/ptr.0006.lodge.req.02.00:StatementOfDistribution" + OccurrenceIndex(statementOfDistribution.OccurrenceIndex) + "/address2.02.02:AddressDetails" + OccurrenceIndex(statementOfDistribution.StatementOfDistribution_AddressDetailsCollection_BUS_C_RPOccurrenceIndex) + "/pyde.02.00:AddressDetails.Line1.Text",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.000406" } },
                        };
                        processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "PTR151", Value = statementOfDistribution.PTR151 });

                        response.Add(processMessage);
                    }
                    #endregion // VR.ATO.GEN.000406

                    #region VR.ATO.GEN.000406

                    /*  VR.ATO.GEN.000406
                    Address line 1 contains incorrect 'care of' reference.  The only acceptable 'care of' reference is C/-
    
                    Legacy Rule Format:
                    IF (pyde.xx.xx:AddressDetails.Line1.Text <> NULLORBLANK) AND (pyde.xx.xx:AddressDetails.Line1.Text STARTSWITH SET("C/O ","C/ ","Care Of ","CO ")) OR (pyde.xx.xx:AddressDetails.Line1.Text CONTAINS SET(" C/O "," C/ "," Care Of "))
                       RETURN VALIDATION MESSAGE
                    ENDIF
        
                    Technical Business Rule Format:
                    (^PTR332 <> BLANK) AND (StartsWithSet(^PTR332,'"C/O ","C/ ","Care Of ","CO "')) OR (ContainsSet(^PTR332, '" C/O "," C/ "," Care Of "'))
            
                    Data Elements:
            
                    RP:^PTR332 = pyde.02.00:AddressDetails.Line1.Text WHERE [AddressDetails.Usage.Code='RES'].[AddressDetails.Currency.Code='C']
                    */
                    assertion = (string.IsNullOrWhiteSpace(statementOfDistribution.PTR332) != true && IsMatch(statementOfDistribution.PTR332, @"^(C/O |C/ |Care Of |CO )", RegexOptions.IgnoreCase) || IsMatch(statementOfDistribution.PTR332, @" C/O | C/ | Care Of ", RegexOptions.IgnoreCase));
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.GEN.000406",
                            Severity = ProcessMessageSeverity.Error,
                            Description = @"Address line 1 contains incorrect 'care of' reference.  The only acceptable 'care of' reference is C/-",
                            Location = "/xbrli:xbrl/ptr.0006.lodge.req.02.00:StatementOfDistribution" + OccurrenceIndex(statementOfDistribution.OccurrenceIndex) + "/address2.02.02:AddressDetails" + OccurrenceIndex(statementOfDistribution.StatementOfDistribution_AddressDetailsCollection_RES_C_RPOccurrenceIndex) + "/pyde.02.00:AddressDetails.Line1.Text",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.000406" } },
                        };
                        processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "PTR332", Value = statementOfDistribution.PTR332 });

                        response.Add(processMessage);
                    }
                    #endregion // VR.ATO.GEN.000406

                    #region VR.ATO.GEN.000408

                    /*  VR.ATO.GEN.000408
                    Care of' is not valid for address line 2
    
                    Legacy Rule Format:
                    IF (pyde.xx.xx:AddressDetails.Line2.Text <> NULLORBLANK) AND (pyde.xx.xx:AddressDetails.Line1.Text STARTSWITH SET("C/O ","C/ ","Care Of ","CO ")) OR (pyde.xx.xx:AddressDetails.Line1.Text CONTAINS SET(" C/O "," C/ "," Care Of "))
                       RETURN VALIDATION MESSAGE
                    ENDIF
        
                    Technical Business Rule Format:
                    (^PTR152 <> BLANK) AND (StartsWithSet(^PTR151,'"C/O ","C/ ","Care Of ","CO "')) OR (ContainsSet(^PTR151, '" C/O "," C/ "," Care Of "'))
            
                    Data Elements:
            
                    RP:^PTR152 = pyde.02.00:AddressDetails.Line2.Text WHERE [AddressDetails.Usage.Code='BUS'].[AddressDetails.Currency.Code='C']
            
                    RP:^PTR151 = pyde.02.00:AddressDetails.Line1.Text WHERE [AddressDetails.Usage.Code='BUS'].[AddressDetails.Currency.Code='C']
                    */
                    assertion = (string.IsNullOrWhiteSpace(statementOfDistribution.PTR152) != true && IsMatch(statementOfDistribution.PTR151, @"^(C/O |C/ |Care Of |CO )", RegexOptions.IgnoreCase) || IsMatch(statementOfDistribution.PTR151, @" C/O | C/ | Care Of ", RegexOptions.IgnoreCase));
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.GEN.000408",
                            Severity = ProcessMessageSeverity.Error,
                            Description = @"Care of' is not valid for address line 2",
                            Location = "/xbrli:xbrl/ptr.0006.lodge.req.02.00:StatementOfDistribution" + OccurrenceIndex(statementOfDistribution.OccurrenceIndex) + "/address2.02.02:AddressDetails" + OccurrenceIndex(statementOfDistribution.StatementOfDistribution_AddressDetailsCollection_BUS_C_RPOccurrenceIndex) + "/pyde.02.00:AddressDetails.Line2.Text",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.000408" } },
                        };
                        processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "PTR152", Value = statementOfDistribution.PTR152 });

                        processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "PTR151", Value = statementOfDistribution.PTR151 });

                        response.Add(processMessage);
                    }
                    #endregion // VR.ATO.GEN.000408

                    #region VR.ATO.GEN.000408

                    /*  VR.ATO.GEN.000408
                    Care of' is not valid for address line 2
    
                    Legacy Rule Format:
                    IF (pyde.xx.xx:AddressDetails.Line2.Text <> NULLORBLANK) AND (pyde.xx.xx:AddressDetails.Line1.Text STARTSWITH SET("C/O ","C/ ","Care Of ","CO ")) OR (pyde.xx.xx:AddressDetails.Line1.Text CONTAINS SET(" C/O "," C/ "," Care Of "))
                       RETURN VALIDATION MESSAGE
                    ENDIF
        
                    Technical Business Rule Format:
                    (^PTR333 <> BLANK) AND (StartsWithSet(^PTR332,'"C/O ","C/ ","Care Of ","CO "')) OR (ContainsSet(^PTR332, '" C/O "," C/ "," Care Of "'))
            
                    Data Elements:
            
                    RP:^PTR333 = pyde.02.00:AddressDetails.Line2.Text WHERE [AddressDetails.Usage.Code='RES'].[AddressDetails.Currency.Code='C']
            
                    RP:^PTR332 = pyde.02.00:AddressDetails.Line1.Text WHERE [AddressDetails.Usage.Code='RES'].[AddressDetails.Currency.Code='C']
                    */
                    assertion = (string.IsNullOrWhiteSpace(statementOfDistribution.PTR333) != true && IsMatch(statementOfDistribution.PTR332, @"^(C/O |C/ |Care Of |CO )", RegexOptions.IgnoreCase) || IsMatch(statementOfDistribution.PTR332, @" C/O | C/ | Care Of ", RegexOptions.IgnoreCase));
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.GEN.000408",
                            Severity = ProcessMessageSeverity.Error,
                            Description = @"Care of' is not valid for address line 2",
                            Location = "/xbrli:xbrl/ptr.0006.lodge.req.02.00:StatementOfDistribution" + OccurrenceIndex(statementOfDistribution.OccurrenceIndex) + "/address2.02.02:AddressDetails" + OccurrenceIndex(statementOfDistribution.StatementOfDistribution_AddressDetailsCollection_RES_C_RPOccurrenceIndex) + "/pyde.02.00:AddressDetails.Line2.Text",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.000408" } },
                        };
                        processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "PTR333", Value = statementOfDistribution.PTR333 });

                        processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "PTR332", Value = statementOfDistribution.PTR332 });

                        response.Add(processMessage);
                    }
                    #endregion // VR.ATO.GEN.000408

                    #region VR.ATO.GEN.000458

                    /*  VR.ATO.GEN.000458
                    Suffix is invalid
    
                    Legacy Rule Format:
                    IF (pyde.xx.xx:PersonNameDetails.NameSuffix.Text <> NULLORBLANK) AND (pyde.xx.xx:PersonNameDetails.NameSuffix.Text <> SET(DOMAIN(SUFFIX CODES)))
                       RETURN VALIDATION MESSAGE
                    ENDIF
        
                    Technical Business Rule Format:
                    (^PTR312 <> BLANK) AND (NotInDomain(^PTR312, 'SUFFIX CODES'))
            
                    Data Elements:
            
                    RP:^PTR312 = pyde.02.00:PersonNameDetails.NameSuffix.Text WHERE [PersonNameDetails.PersonNameType.Code='LGL'].[PersonNameDetails.Usage.Code='Contact']
                    */
                    assertion = (string.IsNullOrWhiteSpace(statementOfDistribution.PTR312) != true && !(IsMatch(statementOfDistribution.PTR312, @"^(ESQ|II|III|IV|JNR|JP|MHA|MHR|MLA|MLC|MP|QC|SNR)$", RegexOptions.IgnoreCase)));
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.GEN.000458",
                            Severity = ProcessMessageSeverity.Error,
                            Description = @"Suffix is invalid",
                            Location = "/xbrli:xbrl/ptr.0006.lodge.req.02.00:StatementOfDistribution" + OccurrenceIndex(statementOfDistribution.OccurrenceIndex) + "/prsnstrcnm3.02.01:PersonNameDetails" + OccurrenceIndex(statementOfDistribution.StatementOfDistribution_PersonNameDetailsCollection_LGL_Contact_RPOccurrenceIndex) + "/pyde.02.00:PersonNameDetails.NameSuffix.Text",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.000458" } },
                        };
                        processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "PTR312", Value = statementOfDistribution.PTR312 });

                        response.Add(processMessage);
                    }
                    #endregion // VR.ATO.GEN.000458

                    #region VR.ATO.GEN.000459

                    /*  VR.ATO.GEN.000459
                    Title is invalid
    
                    Legacy Rule Format:
                    IF (pyde.xx.xx:PersonNameDetails.Title.Text <> NULLORBLANK) AND (pyde.xx.xx:PersonNameDetails.Title.Text <> SET(DOMAIN(TITLE CODES)))
                       RETURN VALIDATION MESSAGE
                    ENDIF
        
                    Technical Business Rule Format:
                    (^PTR311 <> BLANK) AND (NotInDomain(^PTR311, 'TITLE CODES'))
            
                    Data Elements:
            
                    RP:^PTR311 = pyde.02.00:PersonNameDetails.Title.Text WHERE [PersonNameDetails.PersonNameType.Code='LGL'].[PersonNameDetails.Usage.Code='Contact']
                    */
                    assertion = (string.IsNullOrWhiteSpace(statementOfDistribution.PTR311) != true && !(IsMatch(statementOfDistribution.PTR311, @"^(2LT|AB|ABBOT|AC|ACM|ADM|AIR CDRE|ALDERMAN|AM|ARCHBISHOP|ARCHDEACON|ASSOC PROF|AVM|BARON|BARONESS|BISHOP|BR|BRIG|CANON|CAPT|CARDINAL|CDRE|CDT|CHAP|CMDR|CMM|COL|CONST|COUNT|COUNTESS|CPL|CPO|DAME|DEACON|DEACONESS|DEAN|DEPUTY SUPT|DR|DUCHESS|DUKE|EARL|EF|FLGOFF|FLT LT|FR|FSGT|GEN|GNR|GP CAPT|HON JUDGE|HON JUST|HON|HRH|INSP|JUDGE|JUST|LAC|LACW|LADY|LBDR|LCPL|LORD|LS|LT CMDR|LT COL|LT GEN|LT|MADAM|MAJ GEN|MAJ|MASTER|MATRON|MAYOR|MAYORESS|MIDN|MISS|MON|MOST REV|MR|MRS|MS|PASTOR|PATRIARCH|PLT OFF|PO|PRIOR|PROF|PTE|RABBI|RADM|RECTOR|REV|RF|RT HON|RT REV BISHOP|RT REV MON|RT REV|SBLT|SEN|SGT|SIR|SMN|SNR CONST|SQN LDR|SR|SSGT|SUPR|SWAMI|TF|VADM|VERY REV|VICAR|VISCOUNT|WG CDR|WO|WO1|WO2)$", RegexOptions.IgnoreCase)));
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.GEN.000459",
                            Severity = ProcessMessageSeverity.Error,
                            Description = @"Title is invalid",
                            Location = "/xbrli:xbrl/ptr.0006.lodge.req.02.00:StatementOfDistribution" + OccurrenceIndex(statementOfDistribution.OccurrenceIndex) + "/prsnstrcnm3.02.01:PersonNameDetails" + OccurrenceIndex(statementOfDistribution.StatementOfDistribution_PersonNameDetailsCollection_LGL_Contact_RPOccurrenceIndex) + "/pyde.02.00:PersonNameDetails.Title.Text",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.000459" } },
                        };
                        processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "PTR311", Value = statementOfDistribution.PTR311 });

                        response.Add(processMessage);
                    }
                    #endregion // VR.ATO.GEN.000459

                    #region VR.ATO.GEN.300003

                    /*  VR.ATO.GEN.300003
                    For an Australian address, state or territory must be one of: ACT, NSW, NT, QLD, SA, VIC, WA, TAS, AAT
    
                    Legacy Rule Format:
                    IF (pyde.xx.xx:AddressDetails.StateOrTerritory.Code <> NULLORBLANK) AND (pyde.xx.xx:AddressDetails.StateOrTerritory.Code <> SET ("ACT","NSW","NT","QLD","SA","VIC","WA","TAS","AAT"))
                       RETURN VALIDATION MESSAGE
                    ENDIF
        
                    Technical Business Rule Format:
                    (^PTR154 <> BLANK) AND (NotInSet(^PTR154, '"ACT","NSW","NT","QLD","SA","VIC","WA","TAS","AAT"'))
            
                    Data Elements:
            
                    RP:^PTR154 = pyde.02.00:AddressDetails.StateOrTerritory.Code WHERE [AddressDetails.Usage.Code='BUS'].[AddressDetails.Currency.Code='C']
                    */
                    assertion = (string.IsNullOrWhiteSpace(statementOfDistribution.PTR154) != true && !(IsMatch(statementOfDistribution.PTR154, @"^(ACT|NSW|NT|QLD|SA|VIC|WA|TAS|AAT)$", RegexOptions.IgnoreCase)));
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.GEN.300003",
                            Severity = ProcessMessageSeverity.Error,
                            Description = @"For an Australian address, state or territory must be one of: ACT, NSW, NT, QLD, SA, VIC, WA, TAS, AAT",
                            Location = "/xbrli:xbrl/ptr.0006.lodge.req.02.00:StatementOfDistribution" + OccurrenceIndex(statementOfDistribution.OccurrenceIndex) + "/address2.02.02:AddressDetails" + OccurrenceIndex(statementOfDistribution.StatementOfDistribution_AddressDetailsCollection_BUS_C_RPOccurrenceIndex) + "/pyde.02.00:AddressDetails.StateOrTerritory.Code",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.300003" } },
                        };
                        processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "PTR154", Value = statementOfDistribution.PTR154 });

                        response.Add(processMessage);
                    }
                    #endregion // VR.ATO.GEN.300003

                    #region VR.ATO.GEN.300003

                    /*  VR.ATO.GEN.300003
                    For an Australian address, state or territory must be one of: ACT, NSW, NT, QLD, SA, VIC, WA, TAS, AAT
    
                    Legacy Rule Format:
                    IF (pyde.xx.xx:AddressDetails.StateOrTerritory.Code <> NULLORBLANK) AND (pyde.xx.xx:AddressDetails.StateOrTerritory.Code <> SET ("ACT","NSW","NT","QLD","SA","VIC","WA","TAS","AAT"))
                       RETURN VALIDATION MESSAGE
                    ENDIF
        
                    Technical Business Rule Format:
                    (^PTR336 <> BLANK) AND (NotInSet(^PTR336, '"ACT","NSW","NT","QLD","SA","VIC","WA","TAS","AAT"'))
            
                    Data Elements:
            
                    RP:^PTR336 = pyde.02.00:AddressDetails.StateOrTerritory.Code WHERE [AddressDetails.Usage.Code='RES'].[AddressDetails.Currency.Code='C']
                    */
                    assertion = (string.IsNullOrWhiteSpace(statementOfDistribution.PTR336) != true && !(IsMatch(statementOfDistribution.PTR336, @"^(ACT|NSW|NT|QLD|SA|VIC|WA|TAS|AAT)$", RegexOptions.IgnoreCase)));
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.GEN.300003",
                            Severity = ProcessMessageSeverity.Error,
                            Description = @"For an Australian address, state or territory must be one of: ACT, NSW, NT, QLD, SA, VIC, WA, TAS, AAT",
                            Location = "/xbrli:xbrl/ptr.0006.lodge.req.02.00:StatementOfDistribution" + OccurrenceIndex(statementOfDistribution.OccurrenceIndex) + "/address2.02.02:AddressDetails" + OccurrenceIndex(statementOfDistribution.StatementOfDistribution_AddressDetailsCollection_RES_C_RPOccurrenceIndex) + "/pyde.02.00:AddressDetails.StateOrTerritory.Code",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.300003" } },
                        };
                        processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "PTR336", Value = statementOfDistribution.PTR336 });

                        response.Add(processMessage);
                    }
                    #endregion // VR.ATO.GEN.300003

                    #region VR.ATO.GEN.410002

                    /*  VR.ATO.GEN.410002
                    Address Line 2 must be present if Address Line 1 contains 'C/-'
    
                    Legacy Rule Format:
                    WHERE IN TUPLE(addressdetails2.xx.xx:AddressDetails)
                    IF ((pyde.xx.xx:AddressDetails.Line1.Text CONTAINS "C/-") AND (pyde.xx.xx:AddressDetails.Line2.Text = NULLORBLANK))  
                       RETURN VALIDATION MESSAGE
                    ENDIF
        
                    Technical Business Rule Format:
                    ((Contains(^PTR151, 'C/-')) AND (^PTR152 = BLANK))
            
                    Data Elements:
            
                    RP:^PTR151 = pyde.02.00:AddressDetails.Line1.Text WHERE [AddressDetails.Usage.Code='BUS'].[AddressDetails.Currency.Code='C']
            
                    RP:^PTR152 = pyde.02.00:AddressDetails.Line2.Text WHERE [AddressDetails.Usage.Code='BUS'].[AddressDetails.Currency.Code='C']
                    */
                    assertion = ((statementOfDistribution.PTR151 == null ? false : statementOfDistribution.PTR151.ToUpperInvariant().Contains(@"C/-")) && string.IsNullOrWhiteSpace(statementOfDistribution.PTR152) == true);
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.GEN.410002",
                            Severity = ProcessMessageSeverity.Error,
                            Description = @"Address Line 2 must be present if Address Line 1 contains 'C/-'",
                            Location = "/xbrli:xbrl/ptr.0006.lodge.req.02.00:StatementOfDistribution" + OccurrenceIndex(statementOfDistribution.OccurrenceIndex) + "/address2.02.02:AddressDetails" + OccurrenceIndex(statementOfDistribution.StatementOfDistribution_AddressDetailsCollection_BUS_C_RPOccurrenceIndex) + "/pyde.02.00:AddressDetails.Line1.Text",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.410002" } },
                        };
                        processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "PTR151", Value = statementOfDistribution.PTR151 });

                        processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "PTR152", Value = statementOfDistribution.PTR152 });

                        response.Add(processMessage);
                    }
                    #endregion // VR.ATO.GEN.410002

                    #region VR.ATO.GEN.410002

                    /*  VR.ATO.GEN.410002
                    Address Line 2 must be present if Address Line 1 contains 'C/-'
    
                    Legacy Rule Format:
                    WHERE IN TUPLE(addressdetails2.xx.xx:AddressDetails)
                    IF ((pyde.xx.xx:AddressDetails.Line1.Text CONTAINS "C/-") AND (pyde.xx.xx:AddressDetails.Line2.Text = NULLORBLANK))  
                       RETURN VALIDATION MESSAGE
                    ENDIF
        
                    Technical Business Rule Format:
                    ((Contains(^PTR332, 'C/-')) AND (^PTR333 = BLANK))
            
                    Data Elements:
            
                    RP:^PTR332 = pyde.02.00:AddressDetails.Line1.Text WHERE [AddressDetails.Usage.Code='RES'].[AddressDetails.Currency.Code='C']
            
                    RP:^PTR333 = pyde.02.00:AddressDetails.Line2.Text WHERE [AddressDetails.Usage.Code='RES'].[AddressDetails.Currency.Code='C']
                    */
                    assertion = ((statementOfDistribution.PTR332 == null ? false : statementOfDistribution.PTR332.ToUpperInvariant().Contains(@"C/-")) && string.IsNullOrWhiteSpace(statementOfDistribution.PTR333) == true);
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.GEN.410002",
                            Severity = ProcessMessageSeverity.Error,
                            Description = @"Address Line 2 must be present if Address Line 1 contains 'C/-'",
                            Location = "/xbrli:xbrl/ptr.0006.lodge.req.02.00:StatementOfDistribution" + OccurrenceIndex(statementOfDistribution.OccurrenceIndex) + "/address2.02.02:AddressDetails" + OccurrenceIndex(statementOfDistribution.StatementOfDistribution_AddressDetailsCollection_RES_C_RPOccurrenceIndex) + "/pyde.02.00:AddressDetails.Line1.Text",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.410002" } },
                        };
                        processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "PTR332", Value = statementOfDistribution.PTR332 });

                        processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "PTR333", Value = statementOfDistribution.PTR333 });

                        response.Add(processMessage);
                    }
                    #endregion // VR.ATO.GEN.410002

                    #region VR.ATO.GEN.410008

                    /*  VR.ATO.GEN.410008
                    Address Line 3 must be left blank
    
                    Legacy Rule Format:
                    IF (pyde.xx.xx:AddressDetails.Line3.Text <> NULLORBLANK) 
                      RETURN VALIDATION MESSAGE
                    ENDIF
        
                    Technical Business Rule Format:
                    ^PTR238 <> BLANK
            
                    Data Elements:
            
                    RP:^PTR238 = pyde.02.00:AddressDetails.Line3.Text WHERE [AddressDetails.Usage.Code='BUS'].[AddressDetails.Currency.Code='C']
                    */
                    assertion = (string.IsNullOrWhiteSpace(statementOfDistribution.PTR238) != true);
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.GEN.410008",
                            Severity = ProcessMessageSeverity.Error,
                            Description = @"Address Line 3 must be left blank",
                            LongDescription = @"Lines 3 and 4 of the address are not used on this form",
                            Location = "/xbrli:xbrl/ptr.0006.lodge.req.02.00:StatementOfDistribution" + OccurrenceIndex(statementOfDistribution.OccurrenceIndex) + "/address2.02.02:AddressDetails" + OccurrenceIndex(statementOfDistribution.StatementOfDistribution_AddressDetailsCollection_BUS_C_RPOccurrenceIndex) + "/pyde.02.00:AddressDetails.Line3.Text",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.410008" } },
                        };
                        processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "PTR238", Value = statementOfDistribution.PTR238 });

                        response.Add(processMessage);
                    }
                    #endregion // VR.ATO.GEN.410008

                    #region VR.ATO.GEN.410008

                    /*  VR.ATO.GEN.410008
                    Address Line 3 must be left blank
    
                    Legacy Rule Format:
                    IF (pyde.xx.xx:AddressDetails.Line3.Text <> NULLORBLANK) 
                      RETURN VALIDATION MESSAGE
                    ENDIF
        
                    Technical Business Rule Format:
                    ^PTR339 <> BLANK
            
                    Data Elements:
            
                    RP:^PTR339 = pyde.02.00:AddressDetails.Line3.Text WHERE [AddressDetails.Usage.Code='RES'].[AddressDetails.Currency.Code='C']
                    */
                    assertion = (string.IsNullOrWhiteSpace(statementOfDistribution.PTR339) != true);
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.GEN.410008",
                            Severity = ProcessMessageSeverity.Error,
                            Description = @"Address Line 3 must be left blank",
                            LongDescription = @"Lines 3 and 4 of the address are not used on this form",
                            Location = "/xbrli:xbrl/ptr.0006.lodge.req.02.00:StatementOfDistribution" + OccurrenceIndex(statementOfDistribution.OccurrenceIndex) + "/address2.02.02:AddressDetails" + OccurrenceIndex(statementOfDistribution.StatementOfDistribution_AddressDetailsCollection_RES_C_RPOccurrenceIndex) + "/pyde.02.00:AddressDetails.Line3.Text",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.410008" } },
                        };
                        processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "PTR339", Value = statementOfDistribution.PTR339 });

                        response.Add(processMessage);
                    }
                    #endregion // VR.ATO.GEN.410008

                    #region VR.ATO.GEN.410013

                    /*  VR.ATO.GEN.410013
                    Address Line 4 must be left blank
    
                    Legacy Rule Format:
                    IF (pyde.xx.xx:AddressDetails.Line4.Text <> NULLORBLANK)
                       RETURN VALIDATION MESSAGE
                    ENDIF
        
                    Technical Business Rule Format:
                    ^PTR239 <> BLANK
            
                    Data Elements:
            
                    RP:^PTR239 = pyde.02.00:AddressDetails.Line4.Text WHERE [AddressDetails.Usage.Code='BUS'].[AddressDetails.Currency.Code='C']
                    */
                    assertion = (string.IsNullOrWhiteSpace(statementOfDistribution.PTR239) != true);
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.GEN.410013",
                            Severity = ProcessMessageSeverity.Error,
                            Description = @"Address Line 4 must be left blank",
                            LongDescription = @"Lines 3 and 4 of the address are not used on this form",
                            Location = "/xbrli:xbrl/ptr.0006.lodge.req.02.00:StatementOfDistribution" + OccurrenceIndex(statementOfDistribution.OccurrenceIndex) + "/address2.02.02:AddressDetails" + OccurrenceIndex(statementOfDistribution.StatementOfDistribution_AddressDetailsCollection_BUS_C_RPOccurrenceIndex) + "/pyde.02.00:AddressDetails.Line4.Text",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.410013" } },
                        };
                        processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "PTR239", Value = statementOfDistribution.PTR239 });

                        response.Add(processMessage);
                    }
                    #endregion // VR.ATO.GEN.410013

                    #region VR.ATO.GEN.410013

                    /*  VR.ATO.GEN.410013
                    Address Line 4 must be left blank
    
                    Legacy Rule Format:
                    IF (pyde.xx.xx:AddressDetails.Line4.Text <> NULLORBLANK)
                       RETURN VALIDATION MESSAGE
                    ENDIF
        
                    Technical Business Rule Format:
                    ^PTR340 <> BLANK
            
                    Data Elements:
            
                    RP:^PTR340 = pyde.02.00:AddressDetails.Line4.Text WHERE [AddressDetails.Usage.Code='RES'].[AddressDetails.Currency.Code='C']
                    */
                    assertion = (string.IsNullOrWhiteSpace(statementOfDistribution.PTR340) != true);
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.GEN.410013",
                            Severity = ProcessMessageSeverity.Error,
                            Description = @"Address Line 4 must be left blank",
                            LongDescription = @"Lines 3 and 4 of the address are not used on this form",
                            Location = "/xbrli:xbrl/ptr.0006.lodge.req.02.00:StatementOfDistribution" + OccurrenceIndex(statementOfDistribution.OccurrenceIndex) + "/address2.02.02:AddressDetails" + OccurrenceIndex(statementOfDistribution.StatementOfDistribution_AddressDetailsCollection_RES_C_RPOccurrenceIndex) + "/pyde.02.00:AddressDetails.Line4.Text",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.410013" } },
                        };
                        processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "PTR340", Value = statementOfDistribution.PTR340 });

                        response.Add(processMessage);
                    }
                    #endregion // VR.ATO.GEN.410013

                    #region VR.ATO.GEN.410031

                    /*  VR.ATO.GEN.410031
                    The Tax File Number has failed the Tax File Number algorithm check.
    
                    Legacy Rule Format:
                    IF (pyid.xx.xx:Identifiers.TaxFileNumber.Identifier <> NULLORBLANK) AND (TFNALGORITHM (pyid.xx.xx:Identifiers.TaxFileNumber.Identifier) = FALSE)
                      RETURN VALIDATION MESSAGE
                    ENDIF
        
                    Technical Business Rule Format:
                    (^PTR157 <> BLANK) AND (FailsTFNAlgorithm(^PTR157))
            
                    Data Elements:
            
                    RP:^PTR157 = pyid.02.00:Identifiers.TaxFileNumber.Identifier
                    */
                    assertion = (string.IsNullOrWhiteSpace(statementOfDistribution.PTR157) != true && FailsTFNAlgorithm(statementOfDistribution.PTR157));
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.GEN.410031",
                            Severity = ProcessMessageSeverity.Error,
                            Description = @"TFN must be a valid TFN",
                            Location = "/xbrli:xbrl/ptr.0006.lodge.req.02.00:StatementOfDistribution" + OccurrenceIndex(statementOfDistribution.OccurrenceIndex) + "/pyid.02.00:Identifiers.TaxFileNumber.Identifier",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.410031" } },
                        };
                        processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "PTR157", Value = statementOfDistribution.PTR157 });

                        response.Add(processMessage);
                    }
                    #endregion // VR.ATO.GEN.410031

                    #region VR.ATO.GEN.410038

                    /*  VR.ATO.GEN.410038
                    Non-individual name field cannot contain 'Pship', 'P'ship or 'P/Ship'
    
                    Legacy Rule Format:
                    IF (pyde.xx.xx:OrganisationNameDetails.OrganisationalName.Text STARTSWITH "T/A ") AND (pyde.xx.xx:OrganisationNameDetails.OrganisationalName.Text ENDSWITH SET(" Pship"," P'ship"," P/ship"))
                      RETURN VALIDATION MESSAGE
                    ENDIF
        
                    Technical Business Rule Format:
                    (StartsWith(^PTR149, 'T/A ')) AND (EndsWithSet(^PTR149, '" Pship"," P'ship"," P/ship"'))
            
                    Data Elements:
            
                    RP:^PTR149 = pyde.02.00:OrganisationNameDetails.OrganisationalName.Text WHERE [OrganisationNameDetails.OrganisationalNameType.Code='MN'].[OrganisationNameDetails.Currency.Code='C']
                    */
                    assertion = (IsMatch(statementOfDistribution.PTR149, @"^(T/A )", RegexOptions.IgnoreCase) && IsMatch(statementOfDistribution.PTR149, @"( Pship| P'ship| P/ship)$", RegexOptions.IgnoreCase));
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.GEN.410038",
                            Severity = ProcessMessageSeverity.Error,
                            Description = @"Non-individual name field cannot start with 'T/A' and end with 'Pship', 'P'ship or 'P/Ship'",
                            Location = "/xbrli:xbrl/ptr.0006.lodge.req.02.00:StatementOfDistribution" + OccurrenceIndex(statementOfDistribution.OccurrenceIndex) + "/orgname2.02.00:OrganisationNameDetails" + OccurrenceIndex(statementOfDistribution.StatementOfDistribution_OrganisationNameDetailsCollection_MN_C_RPOccurrenceIndex) + "/pyde.02.00:OrganisationNameDetails.OrganisationalName.Text",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.410038" } },
                        };
                        processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "PTR149", Value = statementOfDistribution.PTR149 });

                        response.Add(processMessage);
                    }
                    #endregion // VR.ATO.GEN.410038

                    #region VR.ATO.GEN.410039

                    /*  VR.ATO.GEN.410039
                    Organisation name must contain at least one alpha or numeric character
    
                    Legacy Rule Format:
                    IF (pyde.xx.xx:OrganisationNameDetails.OrganisationalName.Text  <> NULLORBLANK) AND (pyde.xx.xx:OrganisationNameDetails.OrganisationalName.Text  DOES NOT CONTAIN SET("a-z", "A-Z", "0-9"))
                       RETURN VALIDATION MESSAGE
                    ENDIF
        
                    Technical Business Rule Format:
                    (^PTR149  <> BLANK) AND (NotContainsSet(^PTR149, '"a-z", "A-Z", "0-9"'))
            
                    Data Elements:
            
                    RP:^PTR149 = pyde.02.00:OrganisationNameDetails.OrganisationalName.Text WHERE [OrganisationNameDetails.OrganisationalNameType.Code='MN'].[OrganisationNameDetails.Currency.Code='C']
                    */
                    assertion = (string.IsNullOrWhiteSpace(statementOfDistribution.PTR149) != true && !(IsMatch(statementOfDistribution.PTR149, @"[a-z]|[A-Z]|[0-9]", RegexOptions.IgnoreCase)));
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.GEN.410039",
                            Severity = ProcessMessageSeverity.Error,
                            Description = @"Organisation name must contain at least one alpha or numeric character",
                            Location = "/xbrli:xbrl/ptr.0006.lodge.req.02.00:StatementOfDistribution" + OccurrenceIndex(statementOfDistribution.OccurrenceIndex) + "/orgname2.02.00:OrganisationNameDetails" + OccurrenceIndex(statementOfDistribution.StatementOfDistribution_OrganisationNameDetailsCollection_MN_C_RPOccurrenceIndex) + "/pyde.02.00:OrganisationNameDetails.OrganisationalName.Text",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.410039" } },
                        };
                        processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "PTR149", Value = statementOfDistribution.PTR149 });

                        response.Add(processMessage);
                    }
                    #endregion // VR.ATO.GEN.410039

                    #region VR.ATO.GEN.410040

                    /*  VR.ATO.GEN.410040
                    Family name must contain at least one alphabetical character
    
                    Legacy Rule Format:
                    IF (pyde.xx.xx:PersonNameDetails.FamilyName.Text <> NULLORBLANK) AND (pyde.xx.xx:PersonNameDetails.FamilyName.Text DOES NOT CONTAIN SET("A-Z","a-z"))
                       RETURN VALIDATION MESSAGE
                    ENDIF
        
                    Technical Business Rule Format:
                    (^PTR146 <> BLANK) AND (NotContainsSet(^PTR146, '"A-Z","a-z"'))
            
                    Data Elements:
            
                    RP:^PTR146 = pyde.02.00:PersonNameDetails.FamilyName.Text WHERE [PersonNameDetails.PersonNameType.Code='LGL'].[PersonNameDetails.Usage.Code='Contact']
                    */
                    assertion = (string.IsNullOrWhiteSpace(statementOfDistribution.PTR146) != true && !(IsMatch(statementOfDistribution.PTR146, @"[A-Z]|[a-z]", RegexOptions.IgnoreCase)));
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.GEN.410040",
                            Severity = ProcessMessageSeverity.Error,
                            Description = @"Family name must contain at least one alphabetical character",
                            Location = "/xbrli:xbrl/ptr.0006.lodge.req.02.00:StatementOfDistribution" + OccurrenceIndex(statementOfDistribution.OccurrenceIndex) + "/prsnstrcnm3.02.01:PersonNameDetails" + OccurrenceIndex(statementOfDistribution.StatementOfDistribution_PersonNameDetailsCollection_LGL_Contact_RPOccurrenceIndex) + "/pyde.02.00:PersonNameDetails.FamilyName.Text",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.410040" } },
                        };
                        processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "PTR146", Value = statementOfDistribution.PTR146 });

                        response.Add(processMessage);
                    }
                    #endregion // VR.ATO.GEN.410040

                    #region VR.ATO.GEN.410063

                    /*  VR.ATO.GEN.410063
                    Given Name must contain at least one alphabetical character
    
                    Legacy Rule Format:
                    IF (pyde.xx.xx:PersonNameDetails.GivenName.Text <> NULLORBLANK) AND (pyde.xx.xx:PersonNameDetails.GivenName.Text DOES NOT CONTAIN SET("A-Z","a-z"))
                       RETURN VALIDATION MESSAGE
                    ENDIF
        
                    Technical Business Rule Format:
                    (^PTR147 <> BLANK) AND (NotContainsSet(^PTR147, '"A-Z","a-z"'))
            
                    Data Elements:
            
                    RP:^PTR147 = pyde.02.00:PersonNameDetails.GivenName.Text WHERE [PersonNameDetails.PersonNameType.Code='LGL'].[PersonNameDetails.Usage.Code='Contact']
                    */
                    assertion = (string.IsNullOrWhiteSpace(statementOfDistribution.PTR147) != true && !(IsMatch(statementOfDistribution.PTR147, @"[A-Z]|[a-z]", RegexOptions.IgnoreCase)));
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.GEN.410063",
                            Severity = ProcessMessageSeverity.Error,
                            Description = @"Given Name must contain at least one alphabetical character",
                            Location = "/xbrli:xbrl/ptr.0006.lodge.req.02.00:StatementOfDistribution" + OccurrenceIndex(statementOfDistribution.OccurrenceIndex) + "/prsnstrcnm3.02.01:PersonNameDetails" + OccurrenceIndex(statementOfDistribution.StatementOfDistribution_PersonNameDetailsCollection_LGL_Contact_RPOccurrenceIndex) + "/pyde.02.00:PersonNameDetails.GivenName.Text",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.410063" } },
                        };
                        processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "PTR147", Value = statementOfDistribution.PTR147 });

                        response.Add(processMessage);
                    }
                    #endregion // VR.ATO.GEN.410063

                    #region VR.ATO.GEN.410131

                    /*  VR.ATO.GEN.410131
                    Other Given Name must contain at least one alphabetical character
    
                    Legacy Rule Format:
                    IF (pyde.xx.xx:PersonNameDetails.OtherGivenName.Text <> NULLORBLANK) AND (pyde.xx.xx:PersonNameDetails.OtherGivenName.Text DOES NOT CONTAIN SET("A-Z","a-z"))
                         RETURN VALIDATION MESSAGE 
                    ENDIF
        
                    Technical Business Rule Format:
                    (^PTR148 <> BLANK) AND (NotContainsSet(^PTR148, '"A-Z","a-z"'))
            
                    Data Elements:
            
                    RP:^PTR148 = pyde.02.00:PersonNameDetails.OtherGivenName.Text WHERE [PersonNameDetails.PersonNameType.Code='LGL'].[PersonNameDetails.Usage.Code='Contact']
                    */
                    assertion = (string.IsNullOrWhiteSpace(statementOfDistribution.PTR148) != true && !(IsMatch(statementOfDistribution.PTR148, @"[A-Z]|[a-z]", RegexOptions.IgnoreCase)));
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.GEN.410131",
                            Severity = ProcessMessageSeverity.Error,
                            Description = @"Other Given Name must contain at least one alphabetical character",
                            Location = "/xbrli:xbrl/ptr.0006.lodge.req.02.00:StatementOfDistribution" + OccurrenceIndex(statementOfDistribution.OccurrenceIndex) + "/prsnstrcnm3.02.01:PersonNameDetails" + OccurrenceIndex(statementOfDistribution.StatementOfDistribution_PersonNameDetailsCollection_LGL_Contact_RPOccurrenceIndex) + "/pyde.02.00:PersonNameDetails.OtherGivenName.Text",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.410131" } },
                        };
                        processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "PTR148", Value = statementOfDistribution.PTR148 });

                        response.Add(processMessage);
                    }
                    #endregion // VR.ATO.GEN.410131

                    #region VR.ATO.GEN.410167

                    /*  VR.ATO.GEN.410167
                    Overseas address indicator must be present
    
                    Legacy Rule Format:
                    WHERE IN TUPLE(addressdetails2.xx.xx:AddressDetails)
                    IF (pyde.xx.xx:AddressDetails.OverseasAddress.Indicator = NULLORBLANK)
                      RETURN VALIDATION MESSAGE
                    ENDIF
        
                    Technical Business Rule Format:
                    (^AddressDetails <> NULL) AND ((^PTR240 = BLANK))
            
                    Data Elements:
            
                    RP:^PTR240 = pyde.02.00:AddressDetails.OverseasAddress.Indicator WHERE [AddressDetails.Usage.Code='BUS'].[AddressDetails.Currency.Code='C']
            
                    ^AddressDetails = address2.02.02:AddressDetails
                    */
                    assertion = (statementOfDistribution.StatementOfDistribution_AddressDetailsCollection_BUS_C_RPExists != false && statementOfDistribution.PTR240 == null);
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.GEN.410167",
                            Severity = ProcessMessageSeverity.Error,
                            Description = @"Overseas address indicator must be present",
                            Location = "/xbrli:xbrl/ptr.0006.lodge.req.02.00:StatementOfDistribution" + OccurrenceIndex(statementOfDistribution.OccurrenceIndex) + "/address2.02.02:AddressDetails" + OccurrenceIndex(statementOfDistribution.StatementOfDistribution_AddressDetailsCollection_BUS_C_RPOccurrenceIndex) + "/pyde.02.00:AddressDetails.OverseasAddress.Indicator",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.410167" } },
                        };
                        processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "PTR240", Value = GetValueOrEmpty(statementOfDistribution.PTR240) });

                        response.Add(processMessage);
                    }
                    #endregion // VR.ATO.GEN.410167

                    #region VR.ATO.GEN.410167

                    /*  VR.ATO.GEN.410167
                    Overseas address indicator must be present
    
                    Legacy Rule Format:
                    WHERE IN TUPLE(addressdetails2.xx.xx:AddressDetails)
                    IF (pyde.xx.xx:AddressDetails.OverseasAddress.Indicator = NULLORBLANK)
                      RETURN VALIDATION MESSAGE
                    ENDIF
        
                    Technical Business Rule Format:
                    (^AddressDetails <> NULL) AND ((^PTR329 = BLANK))
            
                    Data Elements:
            
                    RP:^PTR329 = pyde.02.00:AddressDetails.OverseasAddress.Indicator WHERE [AddressDetails.Usage.Code='RES'].[AddressDetails.Currency.Code='C']
            
                    ^AddressDetails = address2.02.02:AddressDetails
                    */
                    assertion = (statementOfDistribution.StatementOfDistribution_AddressDetailsCollection_RES_C_RPExists != false && statementOfDistribution.PTR329 == null);
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.GEN.410167",
                            Severity = ProcessMessageSeverity.Error,
                            Description = @"Overseas address indicator must be present",
                            Location = "/xbrli:xbrl/ptr.0006.lodge.req.02.00:StatementOfDistribution" + OccurrenceIndex(statementOfDistribution.OccurrenceIndex) + "/address2.02.02:AddressDetails" + OccurrenceIndex(statementOfDistribution.StatementOfDistribution_AddressDetailsCollection_RES_C_RPOccurrenceIndex) + "/pyde.02.00:AddressDetails.OverseasAddress.Indicator",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.410167" } },
                        };
                        processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "PTR329", Value = GetValueOrEmpty(statementOfDistribution.PTR329) });

                        response.Add(processMessage);
                    }
                    #endregion // VR.ATO.GEN.410167

                    #region VR.ATO.GEN.410183

                    /*  VR.ATO.GEN.410183
                    Person Name Currency Code must be present
    
                    Legacy Rule Format:
                    WHERE IN TUPLE(personstructuredname3.xx.xx:PersonNameDetails)
                    IF (pyde.xx.xx:PersonNameDetails.Currency.Code = NULLORBLANK)
                       RETURN VALIDATION MESSAGE
                    ENDIF
        
                    Technical Business Rule Format:
                    (^PNAM <> NULL) AND ((^PTR267 = BLANK))
            
                    Data Elements:
            
                    RP:^PTR267 = pyde.02.00:PersonNameDetails.Currency.Code WHERE [PersonNameDetails.PersonNameType.Code='LGL'].[PersonNameDetails.Usage.Code='Contact']
            
                    ^PNAM = prsnstrcnm3.02.01:PersonNameDetails
                    */
                    assertion = (statementOfDistribution.StatementOfDistribution_PersonNameDetailsCollection_LGL_Contact_RPExists != false && string.IsNullOrWhiteSpace(statementOfDistribution.PTR267) == true);
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.GEN.410183",
                            Severity = ProcessMessageSeverity.Error,
                            Description = @"Person Name Currency Code must be present",
                            Location = "/xbrli:xbrl/ptr.0006.lodge.req.02.00:StatementOfDistribution" + OccurrenceIndex(statementOfDistribution.OccurrenceIndex) + "/prsnstrcnm3.02.01:PersonNameDetails" + OccurrenceIndex(statementOfDistribution.StatementOfDistribution_PersonNameDetailsCollection_LGL_Contact_RPOccurrenceIndex) + "/pyde.02.00:PersonNameDetails.Currency.Code",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.410183" } },
                        };
                        processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "PTR267", Value = statementOfDistribution.PTR267 });

                        response.Add(processMessage);
                    }
                    #endregion // VR.ATO.GEN.410183

                    #region VR.ATO.GEN.410191

                    /*  VR.ATO.GEN.410191
                    Country code must be present for overseas address
    
                    Legacy Rule Format:
                    WHERE IN TUPLE(addressdetails2.xx.xx:AddressDetails)
                    IF (pyde.xx.xx:AddressDetails.OverseasAddress.Indicator = TRUE) AND (pyde.xx.xx:AddressDetails.Country.Code = NULLORBLANK)
                      RETURN VALIDATION MESSAGE
                    ENDIF
        
                    Technical Business Rule Format:
                    (^PTR240 = TRUE) AND (^PTR308 = BLANK)
            
                    Data Elements:
            
                    RP:^PTR308 = pyde.02.08:AddressDetails.Country.Code WHERE [AddressDetails.Usage.Code='BUS'].[AddressDetails.Currency.Code='C']
            
                    RP:^PTR240 = pyde.02.00:AddressDetails.OverseasAddress.Indicator WHERE [AddressDetails.Usage.Code='BUS'].[AddressDetails.Currency.Code='C']
                    */
                    assertion = (statementOfDistribution.PTR240 == true && string.IsNullOrWhiteSpace(statementOfDistribution.PTR308) == true);
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.GEN.410191",
                            Severity = ProcessMessageSeverity.Error,
                            Description = @"Country code must be present for overseas address",
                            LongDescription = @"When the overseas address indicator is 'TRUE', the corresponding Country Code must be supplied",
                            Location = "/xbrli:xbrl/ptr.0006.lodge.req.02.00:StatementOfDistribution" + OccurrenceIndex(statementOfDistribution.OccurrenceIndex) + "/address2.02.02:AddressDetails" + OccurrenceIndex(statementOfDistribution.StatementOfDistribution_AddressDetailsCollection_BUS_C_RPOccurrenceIndex) + "/pyde.02.08:AddressDetails.Country.Code",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.410191" } },
                        };
                        processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "PTR240", Value = GetValueOrEmpty(statementOfDistribution.PTR240) });

                        processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "PTR308", Value = statementOfDistribution.PTR308 });

                        response.Add(processMessage);
                    }
                    #endregion // VR.ATO.GEN.410191

                    #region VR.ATO.GEN.410191

                    /*  VR.ATO.GEN.410191
                    Country code must be present for overseas address
    
                    Legacy Rule Format:
                    WHERE IN TUPLE(addressdetails2.xx.xx:AddressDetails)
                    IF (pyde.xx.xx:AddressDetails.OverseasAddress.Indicator = TRUE) AND (pyde.xx.xx:AddressDetails.Country.Code = NULLORBLANK)
                      RETURN VALIDATION MESSAGE
                    ENDIF
        
                    Technical Business Rule Format:
                    (^PTR329 = TRUE) AND (^PTR337 = BLANK)
            
                    Data Elements:
            
                    RP:^PTR337 = pyde.02.08:AddressDetails.Country.Code WHERE [AddressDetails.Usage.Code='RES'].[AddressDetails.Currency.Code='C']
            
                    RP:^PTR329 = pyde.02.00:AddressDetails.OverseasAddress.Indicator WHERE [AddressDetails.Usage.Code='RES'].[AddressDetails.Currency.Code='C']
                    */
                    assertion = (statementOfDistribution.PTR329 == true && string.IsNullOrWhiteSpace(statementOfDistribution.PTR337) == true);
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.GEN.410191",
                            Severity = ProcessMessageSeverity.Error,
                            Description = @"Country code must be present for overseas address",
                            LongDescription = @"When the overseas address indicator is 'TRUE', the corresponding Country Code must be supplied",
                            Location = "/xbrli:xbrl/ptr.0006.lodge.req.02.00:StatementOfDistribution" + OccurrenceIndex(statementOfDistribution.OccurrenceIndex) + "/address2.02.02:AddressDetails" + OccurrenceIndex(statementOfDistribution.StatementOfDistribution_AddressDetailsCollection_RES_C_RPOccurrenceIndex) + "/pyde.02.08:AddressDetails.Country.Code",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.410191" } },
                        };
                        processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "PTR329", Value = GetValueOrEmpty(statementOfDistribution.PTR329) });

                        processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "PTR337", Value = statementOfDistribution.PTR337 });

                        response.Add(processMessage);
                    }
                    #endregion // VR.ATO.GEN.410191

                    #region VR.ATO.GEN.410192

                    /*  VR.ATO.GEN.410192
                    Country code must be a valid code
    
                    Legacy Rule Format:
                    WHERE IN TUPLE(addressdetails2.xx.xx:AddressDetails)
                    IF (pyde.xx.xx:AddressDetails.Country.Code <> NULLORBLANK) AND (pyde.xx.xx:AddressDetails.Country.Code <> SET(DOMAIN(COUNTRY CODES)))
                       RETURN VALIDATION MESSAGE
                    ENDIF
        
                    Technical Business Rule Format:
                    (^PTR308 <> BLANK) AND (NotInDomain(^PTR308, 'COUNTRY CODES'))
            
                    Data Elements:
            
                    RP:^PTR308 = pyde.02.08:AddressDetails.Country.Code WHERE [AddressDetails.Usage.Code='BUS'].[AddressDetails.Currency.Code='C']
                    */
                    assertion = (string.IsNullOrWhiteSpace(statementOfDistribution.PTR308) != true && !(IsMatch(statementOfDistribution.PTR308, @"^(ad|ae|af|ag|ai|al|am|an|ao|aq|ar|as|at|au|aw|ax|az|ba|bb|bd|be|bf|bg|bh|bi|bj|bl|bm|bn|bo|bq|br|bs|bt|bv|bw|by|bz|ca|cc|cd|cf|cg|ch|ci|ck|cl|cm|cn|co|cr|cu|cv|cw|cx|cy|cz|de|dj|dk|dm|do|dz|ec|ee|eg|eh|er|es|et|fi|fj|fk|fm|fo|fr|ga|gb|gd|ge|gf|gg|gh|gi|gl|gm|gn|gp|gq|gr|gs|gt|gu|gw|gy|hk|hm|hn|hr|ht|hu|id|ie|il|im|in|io|iq|ir|is|it|je|jm|jo|jp|ke|kg|kh|ki|km|kn|kp|kr|kw|ky|kz|la|lb|lc|li|lk|lr|ls|lt|lu|lv|ly|ma|mc|md|me|mf|mg|mh|mk|ml|mm|mn|mo|mp|mq|mr|ms|mt|mu|mv|mw|mx|my|mz|na|nc|ne|nf|ng|ni|nl|no|np|nr|nu|nz|om|pa|pe|pf|pg|ph|pk|pl|pm|pn|pr|ps|pt|pw|py|qa|re|ro|rs|ru|rw|sa|sb|sc|sd|se|sg|sh|si|sj|sk|sl|sm|sn|so|sr|ss|st|sv|sx|sy|sz|tc|td|tf|tg|th|tj|tk|tl|tm|tn|to|tr|tt|tv|tw|tz|ua|ug|um|us|uy|uz|va|vc|ve|vg|vi|vn|vu|wf|ws|ye|yt|za|zm|zw)$", RegexOptions.IgnoreCase)));
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.GEN.410192",
                            Severity = ProcessMessageSeverity.Error,
                            Description = @"Country code must be a valid code",
                            LongDescription = @"Country Code must be one of the two character lower-case codes from Australian Standard 4590, Section 5, Sub-section 5.15",
                            Location = "/xbrli:xbrl/ptr.0006.lodge.req.02.00:StatementOfDistribution" + OccurrenceIndex(statementOfDistribution.OccurrenceIndex) + "/address2.02.02:AddressDetails" + OccurrenceIndex(statementOfDistribution.StatementOfDistribution_AddressDetailsCollection_BUS_C_RPOccurrenceIndex) + "/pyde.02.08:AddressDetails.Country.Code",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.410192" } },
                        };
                        processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "PTR308", Value = statementOfDistribution.PTR308 });

                        response.Add(processMessage);
                    }
                    #endregion // VR.ATO.GEN.410192

                    #region VR.ATO.GEN.410192

                    /*  VR.ATO.GEN.410192
                    Country code must be a valid code
    
                    Legacy Rule Format:
                    WHERE IN TUPLE(addressdetails2.xx.xx:AddressDetails)
                    IF (pyde.xx.xx:AddressDetails.Country.Code <> NULLORBLANK) AND (pyde.xx.xx:AddressDetails.Country.Code <> SET(DOMAIN(COUNTRY CODES)))
                       RETURN VALIDATION MESSAGE
                    ENDIF
        
                    Technical Business Rule Format:
                    (^PTR337 <> BLANK) AND (NotInDomain(^PTR337, 'COUNTRY CODES'))
            
                    Data Elements:
            
                    RP:^PTR337 = pyde.02.08:AddressDetails.Country.Code WHERE [AddressDetails.Usage.Code='RES'].[AddressDetails.Currency.Code='C']
                    */
                    assertion = (string.IsNullOrWhiteSpace(statementOfDistribution.PTR337) != true && !(IsMatch(statementOfDistribution.PTR337, @"^(ad|ae|af|ag|ai|al|am|an|ao|aq|ar|as|at|au|aw|ax|az|ba|bb|bd|be|bf|bg|bh|bi|bj|bl|bm|bn|bo|bq|br|bs|bt|bv|bw|by|bz|ca|cc|cd|cf|cg|ch|ci|ck|cl|cm|cn|co|cr|cu|cv|cw|cx|cy|cz|de|dj|dk|dm|do|dz|ec|ee|eg|eh|er|es|et|fi|fj|fk|fm|fo|fr|ga|gb|gd|ge|gf|gg|gh|gi|gl|gm|gn|gp|gq|gr|gs|gt|gu|gw|gy|hk|hm|hn|hr|ht|hu|id|ie|il|im|in|io|iq|ir|is|it|je|jm|jo|jp|ke|kg|kh|ki|km|kn|kp|kr|kw|ky|kz|la|lb|lc|li|lk|lr|ls|lt|lu|lv|ly|ma|mc|md|me|mf|mg|mh|mk|ml|mm|mn|mo|mp|mq|mr|ms|mt|mu|mv|mw|mx|my|mz|na|nc|ne|nf|ng|ni|nl|no|np|nr|nu|nz|om|pa|pe|pf|pg|ph|pk|pl|pm|pn|pr|ps|pt|pw|py|qa|re|ro|rs|ru|rw|sa|sb|sc|sd|se|sg|sh|si|sj|sk|sl|sm|sn|so|sr|ss|st|sv|sx|sy|sz|tc|td|tf|tg|th|tj|tk|tl|tm|tn|to|tr|tt|tv|tw|tz|ua|ug|um|us|uy|uz|va|vc|ve|vg|vi|vn|vu|wf|ws|ye|yt|za|zm|zw)$", RegexOptions.IgnoreCase)));
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.GEN.410192",
                            Severity = ProcessMessageSeverity.Error,
                            Description = @"Country code must be a valid code",
                            LongDescription = @"Country Code must be one of the two character lower-case codes from Australian Standard 4590, Section 5, Sub-section 5.15",
                            Location = "/xbrli:xbrl/ptr.0006.lodge.req.02.00:StatementOfDistribution" + OccurrenceIndex(statementOfDistribution.OccurrenceIndex) + "/address2.02.02:AddressDetails" + OccurrenceIndex(statementOfDistribution.StatementOfDistribution_AddressDetailsCollection_RES_C_RPOccurrenceIndex) + "/pyde.02.08:AddressDetails.Country.Code",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.410192" } },
                        };
                        processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "PTR337", Value = statementOfDistribution.PTR337 });

                        response.Add(processMessage);
                    }
                    #endregion // VR.ATO.GEN.410192

                    #region VR.ATO.GEN.410194

                    /*  VR.ATO.GEN.410194
                    Address Line 1 must not exceed 38 characters
    
                    Legacy Rule Format:
                    IF LENGTH(pyde.xx.xx:AddressDetails.Line1.Text) > 38 
                        RETURN VALIDATION MESSAGE
                    ENDIF
        
                    Technical Business Rule Format:
                    Length(^PTR151) > 38
            
                    Data Elements:
            
                    RP:^PTR151 = pyde.02.00:AddressDetails.Line1.Text WHERE [AddressDetails.Usage.Code='BUS'].[AddressDetails.Currency.Code='C']
                    */
                    assertion = (Length(statementOfDistribution.PTR151) > 38);
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.GEN.410194",
                            Severity = ProcessMessageSeverity.Error,
                            Description = @"Address Line 1 must not exceed 38 characters",
                            LongDescription = @"Although SBR sets a limit of 50 characters for the Line 1 of an address, the maximum allowable for this form is 38",
                            Location = "/xbrli:xbrl/ptr.0006.lodge.req.02.00:StatementOfDistribution" + OccurrenceIndex(statementOfDistribution.OccurrenceIndex) + "/address2.02.02:AddressDetails" + OccurrenceIndex(statementOfDistribution.StatementOfDistribution_AddressDetailsCollection_BUS_C_RPOccurrenceIndex) + "/pyde.02.00:AddressDetails.Line1.Text",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.410194" } },
                        };
                        processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "PTR151", Value = statementOfDistribution.PTR151 });

                        response.Add(processMessage);
                    }
                    #endregion // VR.ATO.GEN.410194

                    #region VR.ATO.GEN.410194

                    /*  VR.ATO.GEN.410194
                    Address Line 1 must not exceed 38 characters
    
                    Legacy Rule Format:
                    IF LENGTH(pyde.xx.xx:AddressDetails.Line1.Text) > 38 
                        RETURN VALIDATION MESSAGE
                    ENDIF
        
                    Technical Business Rule Format:
                    Length(^PTR332) > 38
            
                    Data Elements:
            
                    RP:^PTR332 = pyde.02.00:AddressDetails.Line1.Text WHERE [AddressDetails.Usage.Code='RES'].[AddressDetails.Currency.Code='C']
                    */
                    assertion = (Length(statementOfDistribution.PTR332) > 38);
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.GEN.410194",
                            Severity = ProcessMessageSeverity.Error,
                            Description = @"Address Line 1 must not exceed 38 characters",
                            LongDescription = @"Although SBR sets a limit of 50 characters for the Line 1 of an address, the maximum allowable for this form is 38",
                            Location = "/xbrli:xbrl/ptr.0006.lodge.req.02.00:StatementOfDistribution" + OccurrenceIndex(statementOfDistribution.OccurrenceIndex) + "/address2.02.02:AddressDetails" + OccurrenceIndex(statementOfDistribution.StatementOfDistribution_AddressDetailsCollection_RES_C_RPOccurrenceIndex) + "/pyde.02.00:AddressDetails.Line1.Text",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.410194" } },
                        };
                        processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "PTR332", Value = statementOfDistribution.PTR332 });

                        response.Add(processMessage);
                    }
                    #endregion // VR.ATO.GEN.410194

                    #region VR.ATO.GEN.410195

                    /*  VR.ATO.GEN.410195
                    Address Line 2 must not exceed 38 characters
    
                    Legacy Rule Format:
                    IF LENGTH(pyde.xx.xx:AddressDetails.Line2.Text) > 38
                         RETURN VALIDATION MESSAGE
                    ENDIF
        
                    Technical Business Rule Format:
                    Length(^PTR152) > 38
            
                    Data Elements:
            
                    RP:^PTR152 = pyde.02.00:AddressDetails.Line2.Text WHERE [AddressDetails.Usage.Code='BUS'].[AddressDetails.Currency.Code='C']
                    */
                    assertion = (Length(statementOfDistribution.PTR152) > 38);
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.GEN.410195",
                            Severity = ProcessMessageSeverity.Error,
                            Description = @"Address Line 2 must not exceed 38 characters",
                            LongDescription = @"Although SBR sets a limit of 50 characters for the Line 2 of an address, the maximum allowable for this form is 38",
                            Location = "/xbrli:xbrl/ptr.0006.lodge.req.02.00:StatementOfDistribution" + OccurrenceIndex(statementOfDistribution.OccurrenceIndex) + "/address2.02.02:AddressDetails" + OccurrenceIndex(statementOfDistribution.StatementOfDistribution_AddressDetailsCollection_BUS_C_RPOccurrenceIndex) + "/pyde.02.00:AddressDetails.Line2.Text",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.410195" } },
                        };
                        processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "PTR152", Value = statementOfDistribution.PTR152 });

                        response.Add(processMessage);
                    }
                    #endregion // VR.ATO.GEN.410195

                    #region VR.ATO.GEN.410195

                    /*  VR.ATO.GEN.410195
                    Address Line 2 must not exceed 38 characters
    
                    Legacy Rule Format:
                    IF LENGTH(pyde.xx.xx:AddressDetails.Line2.Text) > 38
                         RETURN VALIDATION MESSAGE
                    ENDIF
        
                    Technical Business Rule Format:
                    Length(^PTR333) > 38
            
                    Data Elements:
            
                    RP:^PTR333 = pyde.02.00:AddressDetails.Line2.Text WHERE [AddressDetails.Usage.Code='RES'].[AddressDetails.Currency.Code='C']
                    */
                    assertion = (Length(statementOfDistribution.PTR333) > 38);
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.GEN.410195",
                            Severity = ProcessMessageSeverity.Error,
                            Description = @"Address Line 2 must not exceed 38 characters",
                            LongDescription = @"Although SBR sets a limit of 50 characters for the Line 2 of an address, the maximum allowable for this form is 38",
                            Location = "/xbrli:xbrl/ptr.0006.lodge.req.02.00:StatementOfDistribution" + OccurrenceIndex(statementOfDistribution.OccurrenceIndex) + "/address2.02.02:AddressDetails" + OccurrenceIndex(statementOfDistribution.StatementOfDistribution_AddressDetailsCollection_RES_C_RPOccurrenceIndex) + "/pyde.02.00:AddressDetails.Line2.Text",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.410195" } },
                        };
                        processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "PTR333", Value = statementOfDistribution.PTR333 });

                        response.Add(processMessage);
                    }
                    #endregion // VR.ATO.GEN.410195

                    #region VR.ATO.GEN.410200

                    /*  VR.ATO.GEN.410200
                    Family name cannot contain "Exec for", "Rep for" or "Trustee for"
    
                    Legacy Rule Format:
                    IF (pyde.xx.xx:PersonNameDetails.FamilyName.Text = FOUND("Exec for","Rep for","Trustee for"))
                       RETURN VALIDATION MESSAGE
                    ENDIF
        
                    Technical Business Rule Format:
                    (FoundSet(^PTR146, '"Exec for","Rep for","Trustee for"'))
            
                    Data Elements:
            
                    RP:^PTR146 = pyde.02.00:PersonNameDetails.FamilyName.Text WHERE [PersonNameDetails.PersonNameType.Code='LGL'].[PersonNameDetails.Usage.Code='Contact']
                    */
                    assertion = IsMatch(statementOfDistribution.PTR146, @"(^(Exec for|Rep for|Trustee for)$)|(^(.*)( Exec for | Rep for | Trustee for )(.*)$)|(^(Exec for |Rep for |Trustee for )(.*)$)|(^(.*)( Exec for| Rep for| Trustee for)$)", RegexOptions.IgnoreCase);
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.GEN.000424",
                            Severity = ProcessMessageSeverity.Error,
                            Description = @"Family name cannot contain ""Exec for"", ""Rep for"" or ""Trustee for""",
                            Location = "/xbrli:xbrl/ptr.0006.lodge.req.02.00:StatementOfDistribution" + OccurrenceIndex(statementOfDistribution.OccurrenceIndex) + "/prsnstrcnm3.02.01:PersonNameDetails" + OccurrenceIndex(statementOfDistribution.StatementOfDistribution_PersonNameDetailsCollection_LGL_Contact_RPOccurrenceIndex) + "/pyde.02.00:PersonNameDetails.FamilyName.Text",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.410200" } },
                        };
                        processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "PTR146", Value = statementOfDistribution.PTR146 });

                        response.Add(processMessage);
                    }
                    #endregion // VR.ATO.GEN.410200

                    #region VR.ATO.GEN.410201

                    /*  VR.ATO.GEN.410201
                    Family name cannot contain "MR", "MRS", "MISS" or "MS"
    
                    Legacy Rule Format:
                    IF (pyde.xx.xx:PersonNameDetails.FamilyName.Text = FOUND("MR","MRS","MISS","MS"))
                       RETURN VALIDATION MESSAGE
                    ENDIF
        
                    Technical Business Rule Format:
                    (FoundSet(^PTR146, '"MR","MRS","MISS","MS"'))
            
                    Data Elements:
            
                    RP:^PTR146 = pyde.02.00:PersonNameDetails.FamilyName.Text WHERE [PersonNameDetails.PersonNameType.Code='LGL'].[PersonNameDetails.Usage.Code='Contact']
                    */
                    assertion = IsMatch(statementOfDistribution.PTR146, @"(^(MR|MRS|MISS|MS)$)|(^(.*)( MR | MRS | MISS | MS )(.*)$)|(^(MR |MRS |MISS |MS )(.*)$)|(^(.*)( MR| MRS| MISS| MS)$)", RegexOptions.IgnoreCase);
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.GEN.000426",
                            Severity = ProcessMessageSeverity.Error,
                            Description = @"Family name cannot contain ""MR"", ""MRS"", ""MISS"" or ""MS""",
                            Location = "/xbrli:xbrl/ptr.0006.lodge.req.02.00:StatementOfDistribution" + OccurrenceIndex(statementOfDistribution.OccurrenceIndex) + "/prsnstrcnm3.02.01:PersonNameDetails" + OccurrenceIndex(statementOfDistribution.StatementOfDistribution_PersonNameDetailsCollection_LGL_Contact_RPOccurrenceIndex) + "/pyde.02.00:PersonNameDetails.FamilyName.Text",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.410201" } },
                        };
                        processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "PTR146", Value = statementOfDistribution.PTR146 });

                        response.Add(processMessage);
                    }
                    #endregion // VR.ATO.GEN.410201

                    #region VR.ATO.GEN.410202

                    /*  VR.ATO.GEN.410202
                    First name cannot contain suffix ESQ, II, III, IV, JNR, JP, MHA, MHR, MLA, MLC, MP, QC or SNR
    
                    Legacy Rule Format:
                    IF (pyde.xx.xx:PersonNameDetails.GivenName.Text = FOUND("ESQ","II","III","IV","JNR","JP","MHA","MHR","MLA","MLC","MP","QC","SNR"))
                       RETURN VALIDATION MESSAGE
                    ENDIF
        
                    Technical Business Rule Format:
                    (FoundSet(^PTR147, '"ESQ","II","III","IV","JNR","JP","MHA","MHR","MLA","MLC","MP","QC","SNR"'))
            
                    Data Elements:
            
                    RP:^PTR147 = pyde.02.00:PersonNameDetails.GivenName.Text WHERE [PersonNameDetails.PersonNameType.Code='LGL'].[PersonNameDetails.Usage.Code='Contact']
                    */
                    assertion = IsMatch(statementOfDistribution.PTR147, @"(^(ESQ|II|III|IV|JNR|JP|MHA|MHR|MLA|MLC|MP|QC|SNR)$)|(^(.*)( ESQ | II | III | IV | JNR | JP | MHA | MHR | MLA | MLC | MP | QC | SNR )(.*)$)|(^(ESQ |II |III |IV |JNR |JP |MHA |MHR |MLA |MLC |MP |QC |SNR )(.*)$)|(^(.*)( ESQ| II| III| IV| JNR| JP| MHA| MHR| MLA| MLC| MP| QC| SNR)$)", RegexOptions.IgnoreCase);
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.GEN.000436",
                            Severity = ProcessMessageSeverity.Error,
                            Description = @"First name cannot contain suffix ESQ, II, III, IV, JNR, JP, MHA, MHR, MLA, MLC, MP, QC or SNR",
                            Location = "/xbrli:xbrl/ptr.0006.lodge.req.02.00:StatementOfDistribution" + OccurrenceIndex(statementOfDistribution.OccurrenceIndex) + "/prsnstrcnm3.02.01:PersonNameDetails" + OccurrenceIndex(statementOfDistribution.StatementOfDistribution_PersonNameDetailsCollection_LGL_Contact_RPOccurrenceIndex) + "/pyde.02.00:PersonNameDetails.GivenName.Text",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.410202" } },
                        };
                        processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "PTR147", Value = statementOfDistribution.PTR147 });

                        response.Add(processMessage);
                    }
                    #endregion // VR.ATO.GEN.410202

                    #region VR.ATO.GEN.410203

                    /*  VR.ATO.GEN.410203
                    First name cannot contain "Exec for", "Rep for" or "Trustee for"
    
                    Legacy Rule Format:
                    IF (pyde.xx.xx:PersonNameDetails.GivenName.Text = FOUND("Exec for","Rep for","Trustee for"))
                       RETURN VALIDATION MESSAGE
                    ENDIF
        
                    Technical Business Rule Format:
                    (FoundSet(^PTR147, '"Exec for","Rep for","Trustee for"'))
            
                    Data Elements:
            
                    RP:^PTR147 = pyde.02.00:PersonNameDetails.GivenName.Text WHERE [PersonNameDetails.PersonNameType.Code='LGL'].[PersonNameDetails.Usage.Code='Contact']
                    */
                    assertion = IsMatch(statementOfDistribution.PTR147, @"(^(Exec for|Rep for|Trustee for)$)|(^(.*)( Exec for | Rep for | Trustee for )(.*)$)|(^(Exec for |Rep for |Trustee for )(.*)$)|(^(.*)( Exec for| Rep for| Trustee for)$)", RegexOptions.IgnoreCase);
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.GEN.000437",
                            Severity = ProcessMessageSeverity.Error,
                            Description = @"First name cannot contain ""Exec for"", ""Rep for"" or ""Trustee for""",
                            Location = "/xbrli:xbrl/ptr.0006.lodge.req.02.00:StatementOfDistribution" + OccurrenceIndex(statementOfDistribution.OccurrenceIndex) + "/prsnstrcnm3.02.01:PersonNameDetails" + OccurrenceIndex(statementOfDistribution.StatementOfDistribution_PersonNameDetailsCollection_LGL_Contact_RPOccurrenceIndex) + "/pyde.02.00:PersonNameDetails.GivenName.Text",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.410203" } },
                        };
                        processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "PTR147", Value = statementOfDistribution.PTR147 });

                        response.Add(processMessage);
                    }
                    #endregion // VR.ATO.GEN.410203

                    #region VR.ATO.GEN.410204

                    /*  VR.ATO.GEN.410204
                    First name cannot contain "MR", "MRS", "MISS" or "MS"
    
                    Legacy Rule Format:
                    IF (pyde.xx.xx:PersonNameDetails.GivenName.Text = FOUND("MR","MRS","MISS","MS"))  
                       RETURN VALIDATION MESSAGE 
                    ENDIF
        
                    Technical Business Rule Format:
                    (FoundSet(^PTR147, '"MR","MRS","MISS","MS"'))
            
                    Data Elements:
            
                    RP:^PTR147 = pyde.02.00:PersonNameDetails.GivenName.Text WHERE [PersonNameDetails.PersonNameType.Code='LGL'].[PersonNameDetails.Usage.Code='Contact']
                    */
                    assertion = IsMatch(statementOfDistribution.PTR147, @"(^(MR|MRS|MISS|MS)$)|(^(.*)( MR | MRS | MISS | MS )(.*)$)|(^(MR |MRS |MISS |MS )(.*)$)|(^(.*)( MR| MRS| MISS| MS)$)", RegexOptions.IgnoreCase);
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.GEN.000438",
                            Severity = ProcessMessageSeverity.Error,
                            Description = @"First name cannot contain ""MR"", ""MRS"", ""MISS"" or ""MS""",
                            Location = "/xbrli:xbrl/ptr.0006.lodge.req.02.00:StatementOfDistribution" + OccurrenceIndex(statementOfDistribution.OccurrenceIndex) + "/prsnstrcnm3.02.01:PersonNameDetails" + OccurrenceIndex(statementOfDistribution.StatementOfDistribution_PersonNameDetailsCollection_LGL_Contact_RPOccurrenceIndex) + "/pyde.02.00:PersonNameDetails.GivenName.Text",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.410204" } },
                        };
                        processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "PTR147", Value = statementOfDistribution.PTR147 });

                        response.Add(processMessage);
                    }
                    #endregion // VR.ATO.GEN.410204

                    #region VR.ATO.GEN.410205

                    /*  VR.ATO.GEN.410205
                    Address Line 1 must not contain 'AS ABOVE'
    
                    Legacy Rule Format:
                    IF (pyde.xx.xx:AddressDetails.Line1.Text = FOUND("AS ABOVE"))
                       RETURN VALIDATION MESSAGE
                    ENDIF
        
                    Technical Business Rule Format:
                    (FoundSet(^PTR151, '"AS ABOVE"'))
            
                    Data Elements:
            
                    RP:^PTR151 = pyde.02.00:AddressDetails.Line1.Text WHERE [AddressDetails.Usage.Code='BUS'].[AddressDetails.Currency.Code='C']
                    */
                    assertion = IsMatch(statementOfDistribution.PTR151, @"(^(AS ABOVE)$)|(^(.*)( AS ABOVE )(.*)$)|(^(AS ABOVE )(.*)$)|(^(.*)( AS ABOVE)$)", RegexOptions.IgnoreCase);
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.GEN.410001",
                            Severity = ProcessMessageSeverity.Error,
                            Description = @"Address Line 1 must not contain 'AS ABOVE'",
                            Location = "/xbrli:xbrl/ptr.0006.lodge.req.02.00:StatementOfDistribution" + OccurrenceIndex(statementOfDistribution.OccurrenceIndex) + "/address2.02.02:AddressDetails" + OccurrenceIndex(statementOfDistribution.StatementOfDistribution_AddressDetailsCollection_BUS_C_RPOccurrenceIndex) + "/pyde.02.00:AddressDetails.Line1.Text",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.410205" } },
                        };
                        processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "PTR151", Value = statementOfDistribution.PTR151 });

                        response.Add(processMessage);
                    }
                    #endregion // VR.ATO.GEN.410205

                    #region VR.ATO.GEN.410205

                    /*  VR.ATO.GEN.410205
                    Address Line 1 must not contain 'AS ABOVE'
    
                    Legacy Rule Format:
                    IF (pyde.xx.xx:AddressDetails.Line1.Text = FOUND("AS ABOVE"))
                       RETURN VALIDATION MESSAGE
                    ENDIF
        
                    Technical Business Rule Format:
                    (FoundSet(^PTR332, '"AS ABOVE"'))
            
                    Data Elements:
            
                    RP:^PTR332 = pyde.02.00:AddressDetails.Line1.Text WHERE [AddressDetails.Usage.Code='RES'].[AddressDetails.Currency.Code='C']
                    */
                    assertion = IsMatch(statementOfDistribution.PTR332, @"(^(AS ABOVE)$)|(^(.*)( AS ABOVE )(.*)$)|(^(AS ABOVE )(.*)$)|(^(.*)( AS ABOVE)$)", RegexOptions.IgnoreCase);
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.GEN.410001",
                            Severity = ProcessMessageSeverity.Error,
                            Description = @"Address Line 1 must not contain 'AS ABOVE'",
                            Location = "/xbrli:xbrl/ptr.0006.lodge.req.02.00:StatementOfDistribution" + OccurrenceIndex(statementOfDistribution.OccurrenceIndex) + "/address2.02.02:AddressDetails" + OccurrenceIndex(statementOfDistribution.StatementOfDistribution_AddressDetailsCollection_RES_C_RPOccurrenceIndex) + "/pyde.02.00:AddressDetails.Line1.Text",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.410205" } },
                        };
                        processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "PTR332", Value = statementOfDistribution.PTR332 });

                        response.Add(processMessage);
                    }
                    #endregion // VR.ATO.GEN.410205

                    #region VR.ATO.GEN.410206

                    /*  VR.ATO.GEN.410206
                    Non Individual Name cannot contain "The trustee", "The Exec", "exec" or "the TTE"
    
                    Legacy Rule Format:
                    IF (pyde.xx.xx:OrganisationNameDetails.OrganisationalName.Text = FOUND("The trustee","The Exec","exec","The TTE"))
                       RETURN VALIDATION MESSAGE
                    ENDIF
        
                    Technical Business Rule Format:
                    (FoundSet(^PTR149, '"The trustee","The Exec","exec","The TTE"'))
            
                    Data Elements:
            
                    RP:^PTR149 = pyde.02.00:OrganisationNameDetails.OrganisationalName.Text WHERE [OrganisationNameDetails.OrganisationalNameType.Code='MN'].[OrganisationNameDetails.Currency.Code='C']
                    */
                    assertion = IsMatch(statementOfDistribution.PTR149, @"(^(The trustee|The Exec|exec|The TTE)$)|(^(.*)( The trustee | The Exec | exec | The TTE )(.*)$)|(^(The trustee |The Exec |exec |The TTE )(.*)$)|(^(.*)( The trustee| The Exec| exec| The TTE)$)", RegexOptions.IgnoreCase);
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.GEN.428042",
                            Severity = ProcessMessageSeverity.Error,
                            Description = @"Non Individual Name cannot contain ""The trustee"", ""The Exec"", ""exec"" or ""the TTE""",
                            Location = "/xbrli:xbrl/ptr.0006.lodge.req.02.00:StatementOfDistribution" + OccurrenceIndex(statementOfDistribution.OccurrenceIndex) + "/orgname2.02.00:OrganisationNameDetails" + OccurrenceIndex(statementOfDistribution.StatementOfDistribution_OrganisationNameDetailsCollection_MN_C_RPOccurrenceIndex) + "/pyde.02.00:OrganisationNameDetails.OrganisationalName.Text",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.410206" } },
                        };
                        processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "PTR149", Value = statementOfDistribution.PTR149 });

                        response.Add(processMessage);
                    }
                    #endregion // VR.ATO.GEN.410206

                    #region VR.ATO.GEN.410207

                    /*  VR.ATO.GEN.410207
                    Other given name cannot contain suffix ESQ, II, III, IV, JNR, JP, MHA, MHR, MLA, MLC, MP, QC or SNR
    
                    Legacy Rule Format:
                    IF (pyde.xx.xx:PersonNameDetails.OtherGivenName.Text = FOUND("ESQ","II","III","IV","JNR","JP","MHA","MHR","MLA","MLC","MP","QC","SNR"))
                       RETURN VALIDATION MESSAGE
                    ENDIF
        
                    Technical Business Rule Format:
                    (FoundSet(^PTR148, '"ESQ","II","III","IV","JNR","JP","MHA","MHR","MLA","MLC","MP","QC","SNR"'))
            
                    Data Elements:
            
                    RP:^PTR148 = pyde.02.00:PersonNameDetails.OtherGivenName.Text WHERE [PersonNameDetails.PersonNameType.Code='LGL'].[PersonNameDetails.Usage.Code='Contact']
                    */
                    assertion = IsMatch(statementOfDistribution.PTR148, @"(^(ESQ|II|III|IV|JNR|JP|MHA|MHR|MLA|MLC|MP|QC|SNR)$)|(^(.*)( ESQ | II | III | IV | JNR | JP | MHA | MHR | MLA | MLC | MP | QC | SNR )(.*)$)|(^(ESQ |II |III |IV |JNR |JP |MHA |MHR |MLA |MLC |MP |QC |SNR )(.*)$)|(^(.*)( ESQ| II| III| IV| JNR| JP| MHA| MHR| MLA| MLC| MP| QC| SNR)$)", RegexOptions.IgnoreCase);
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.GEN.000448",
                            Severity = ProcessMessageSeverity.Error,
                            Description = @"Other given name cannot contain suffix ESQ, II, III, IV, JNR, JP, MHA, MHR, MLA, MLC, MP, QC or SNR",
                            Location = "/xbrli:xbrl/ptr.0006.lodge.req.02.00:StatementOfDistribution" + OccurrenceIndex(statementOfDistribution.OccurrenceIndex) + "/prsnstrcnm3.02.01:PersonNameDetails" + OccurrenceIndex(statementOfDistribution.StatementOfDistribution_PersonNameDetailsCollection_LGL_Contact_RPOccurrenceIndex) + "/pyde.02.00:PersonNameDetails.OtherGivenName.Text",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.410207" } },
                        };
                        processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "PTR148", Value = statementOfDistribution.PTR148 });

                        response.Add(processMessage);
                    }
                    #endregion // VR.ATO.GEN.410207

                    #region VR.ATO.GEN.410208

                    /*  VR.ATO.GEN.410208
                    Other given name cannot contain "Exec for", "Rep for" or "Trustee for"
    
                    Legacy Rule Format:
                    IF (pyde.xx.xx:PersonNameDetails.OtherGivenName.Text = FOUND("Exec for","Rep for","Trustee for"))
                       RETURN VALIDATION MESSAGE
                    ENDIF
        
                    Technical Business Rule Format:
                    (FoundSet(^PTR148, '"Exec for","Rep for","Trustee for"'))
            
                    Data Elements:
            
                    RP:^PTR148 = pyde.02.00:PersonNameDetails.OtherGivenName.Text WHERE [PersonNameDetails.PersonNameType.Code='LGL'].[PersonNameDetails.Usage.Code='Contact']
                    */
                    assertion = IsMatch(statementOfDistribution.PTR148, @"(^(Exec for|Rep for|Trustee for)$)|(^(.*)( Exec for | Rep for | Trustee for )(.*)$)|(^(Exec for |Rep for |Trustee for )(.*)$)|(^(.*)( Exec for| Rep for| Trustee for)$)", RegexOptions.IgnoreCase);
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.GEN.000449",
                            Severity = ProcessMessageSeverity.Error,
                            Description = @"Other given name cannot contain ""Exec for"", ""Rep for"" or ""Trustee for""",
                            Location = "/xbrli:xbrl/ptr.0006.lodge.req.02.00:StatementOfDistribution" + OccurrenceIndex(statementOfDistribution.OccurrenceIndex) + "/prsnstrcnm3.02.01:PersonNameDetails" + OccurrenceIndex(statementOfDistribution.StatementOfDistribution_PersonNameDetailsCollection_LGL_Contact_RPOccurrenceIndex) + "/pyde.02.00:PersonNameDetails.OtherGivenName.Text",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.410208" } },
                        };
                        processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "PTR148", Value = statementOfDistribution.PTR148 });

                        response.Add(processMessage);
                    }
                    #endregion // VR.ATO.GEN.410208

                    #region VR.ATO.GEN.410209

                    /*  VR.ATO.GEN.410209
                    Other given name cannot contain "MR", "MRS", "MISS" or "MS"
    
                    Legacy Rule Format:
                    IF (pyde.xx.xx:PersonNameDetails.OtherGivenName.Text = FOUND("MR","MRS","MISS","MS"))
                       RETURN VALIDATION MESSAGE
                    ENDIF
        
                    Technical Business Rule Format:
                    (FoundSet(^PTR148, '"MR","MRS","MISS","MS"'))
            
                    Data Elements:
            
                    RP:^PTR148 = pyde.02.00:PersonNameDetails.OtherGivenName.Text WHERE [PersonNameDetails.PersonNameType.Code='LGL'].[PersonNameDetails.Usage.Code='Contact']
                    */
                    assertion = IsMatch(statementOfDistribution.PTR148, @"(^(MR|MRS|MISS|MS)$)|(^(.*)( MR | MRS | MISS | MS )(.*)$)|(^(MR |MRS |MISS |MS )(.*)$)|(^(.*)( MR| MRS| MISS| MS)$)", RegexOptions.IgnoreCase);
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.GEN.000450",
                            Severity = ProcessMessageSeverity.Error,
                            Description = @"Other given name cannot contain ""MR"", ""MRS"", ""MISS"" or ""MS""",
                            Location = "/xbrli:xbrl/ptr.0006.lodge.req.02.00:StatementOfDistribution" + OccurrenceIndex(statementOfDistribution.OccurrenceIndex) + "/prsnstrcnm3.02.01:PersonNameDetails" + OccurrenceIndex(statementOfDistribution.StatementOfDistribution_PersonNameDetailsCollection_LGL_Contact_RPOccurrenceIndex) + "/pyde.02.00:PersonNameDetails.OtherGivenName.Text",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.410209" } },
                        };
                        processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "PTR148", Value = statementOfDistribution.PTR148 });

                        response.Add(processMessage);
                    }
                    #endregion // VR.ATO.GEN.410209

                    #region VR.ATO.GEN.410211

                    /*  VR.ATO.GEN.410211
                    Overseas address indicator must not be 'TRUE' if Country Code is 'au'
    
                    Legacy Rule Format:
                    WHERE IN TUPLE(addressdetails2.xx.xx:AddressDetails)
                    IF (pyde.xx.xx:AddressDetails.Country.Code = "au") AND  (pyde.xx.xx:AddressDetails.OverseasAddress.Indicator = TRUE)   
                       RETURN VALIDATION MESSAGE
                    ENDIF
        
                    Technical Business Rule Format:
                    (^PTR308 = 'au') AND  (^PTR240 = TRUE)
            
                    Data Elements:
            
                    RP:^PTR308 = pyde.02.08:AddressDetails.Country.Code WHERE [AddressDetails.Usage.Code='BUS'].[AddressDetails.Currency.Code='C']
            
                    RP:^PTR240 = pyde.02.00:AddressDetails.OverseasAddress.Indicator WHERE [AddressDetails.Usage.Code='BUS'].[AddressDetails.Currency.Code='C']
                    */
                    assertion = (statementOfDistribution.PTR308 == @"au" && statementOfDistribution.PTR240 == true);
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.GEN.410211",
                            Severity = ProcessMessageSeverity.Error,
                            Description = @"Overseas address indicator must not be 'TRUE' if Country Code is 'au'",
                            LongDescription = @"The overseas address indicator must be 'FALSE' if the Country Code is 'au' - representing Australia. If the address is an overseas address please supply the correct Country Code",
                            Location = "/xbrli:xbrl/ptr.0006.lodge.req.02.00:StatementOfDistribution" + OccurrenceIndex(statementOfDistribution.OccurrenceIndex) + "/address2.02.02:AddressDetails" + OccurrenceIndex(statementOfDistribution.StatementOfDistribution_AddressDetailsCollection_BUS_C_RPOccurrenceIndex) + "/pyde.02.08:AddressDetails.Country.Code",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.410211" } },
                        };
                        processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "PTR308", Value = statementOfDistribution.PTR308 });

                        processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "PTR240", Value = GetValueOrEmpty(statementOfDistribution.PTR240) });

                        response.Add(processMessage);
                    }
                    #endregion // VR.ATO.GEN.410211

                    #region VR.ATO.GEN.410211

                    /*  VR.ATO.GEN.410211
                    Overseas address indicator must not be 'TRUE' if Country Code is 'au'
    
                    Legacy Rule Format:
                    WHERE IN TUPLE(addressdetails2.xx.xx:AddressDetails)
                    IF (pyde.xx.xx:AddressDetails.Country.Code = "au") AND  (pyde.xx.xx:AddressDetails.OverseasAddress.Indicator = TRUE)   
                       RETURN VALIDATION MESSAGE
                    ENDIF
        
                    Technical Business Rule Format:
                    (^PTR337 = 'au') AND  (^PTR329 = TRUE)
            
                    Data Elements:
            
                    RP:^PTR337 = pyde.02.08:AddressDetails.Country.Code WHERE [AddressDetails.Usage.Code='RES'].[AddressDetails.Currency.Code='C']
            
                    RP:^PTR329 = pyde.02.00:AddressDetails.OverseasAddress.Indicator WHERE [AddressDetails.Usage.Code='RES'].[AddressDetails.Currency.Code='C']
                    */
                    assertion = (statementOfDistribution.PTR337 == @"au" && statementOfDistribution.PTR329 == true);
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.GEN.410211",
                            Severity = ProcessMessageSeverity.Error,
                            Description = @"Overseas address indicator must not be 'TRUE' if Country Code is 'au'",
                            LongDescription = @"The overseas address indicator must be 'FALSE' if the Country Code is 'au' - representing Australia. If the address is an overseas address please supply the correct Country Code",
                            Location = "/xbrli:xbrl/ptr.0006.lodge.req.02.00:StatementOfDistribution" + OccurrenceIndex(statementOfDistribution.OccurrenceIndex) + "/address2.02.02:AddressDetails" + OccurrenceIndex(statementOfDistribution.StatementOfDistribution_AddressDetailsCollection_RES_C_RPOccurrenceIndex) + "/pyde.02.08:AddressDetails.Country.Code",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.410211" } },
                        };
                        processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "PTR337", Value = statementOfDistribution.PTR337 });

                        processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "PTR329", Value = GetValueOrEmpty(statementOfDistribution.PTR329) });

                        response.Add(processMessage);
                    }
                    #endregion // VR.ATO.GEN.410211

                    #region VR.ATO.GEN.410212

                    /*  VR.ATO.GEN.410212
                    If an overseas Country Code is used, the Overseas Address indicator must be 'TRUE'
    
                    Legacy Rule Format:
                    WHERE IN TUPLE(addressdetails2.xx.xx:AddressDetails)IF (pyde.xx.xx:AddressDetails.Country.Code <> NULLORBLANK) AND (pyde.xx.xx:AddressDetails.Country.Code <> "au") AND  (pyde.xx.xx:AddressDetails.OverseasAddress.Indicator = FALSE)
                       RETURN VALIDATION MESSAGE
                    ENDIF
        
                    Technical Business Rule Format:
                    (^PTR308 <> BLANK) AND (^PTR308 <> 'au') AND  (^PTR240 = FALSE)
            
                    Data Elements:
            
                    RP:^PTR308 = pyde.02.08:AddressDetails.Country.Code WHERE [AddressDetails.Usage.Code='BUS'].[AddressDetails.Currency.Code='C']
            
                    RP:^PTR240 = pyde.02.00:AddressDetails.OverseasAddress.Indicator WHERE [AddressDetails.Usage.Code='BUS'].[AddressDetails.Currency.Code='C']
                    */
                    assertion = (string.IsNullOrWhiteSpace(statementOfDistribution.PTR308) != true && statementOfDistribution.PTR308 != @"au" && statementOfDistribution.PTR240 == false);
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.GEN.410212",
                            Severity = ProcessMessageSeverity.Error,
                            Description = @"If an overseas Country Code is used, the Overseas Address indicator must be 'TRUE'",
                            LongDescription = @"If the address is an Australian address, the overseas address indicator must be set to 'FALSE' and you do not need to supply a Country Code. If the address is an overseas address, the overseas address indicator must be set to 'TRUE' and you must supply a Country Code other than 'au'",
                            Location = "/xbrli:xbrl/ptr.0006.lodge.req.02.00:StatementOfDistribution" + OccurrenceIndex(statementOfDistribution.OccurrenceIndex) + "/address2.02.02:AddressDetails" + OccurrenceIndex(statementOfDistribution.StatementOfDistribution_AddressDetailsCollection_BUS_C_RPOccurrenceIndex) + "/pyde.02.08:AddressDetails.Country.Code",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.410212" } },
                        };
                        processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "PTR308", Value = statementOfDistribution.PTR308 });

                        processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "PTR240", Value = GetValueOrEmpty(statementOfDistribution.PTR240) });

                        response.Add(processMessage);
                    }
                    #endregion // VR.ATO.GEN.410212

                    #region VR.ATO.GEN.410212

                    /*  VR.ATO.GEN.410212
                    If an overseas Country Code is used, the Overseas Address indicator must be 'TRUE'
    
                    Legacy Rule Format:
                    WHERE IN TUPLE(addressdetails2.xx.xx:AddressDetails)IF (pyde.xx.xx:AddressDetails.Country.Code <> NULLORBLANK) AND (pyde.xx.xx:AddressDetails.Country.Code <> "au") AND  (pyde.xx.xx:AddressDetails.OverseasAddress.Indicator = FALSE)
                       RETURN VALIDATION MESSAGE
                    ENDIF
        
                    Technical Business Rule Format:
                    (^PTR337 <> BLANK) AND (^PTR337 <> 'au') AND  (^PTR329 = FALSE)
            
                    Data Elements:
            
                    RP:^PTR337 = pyde.02.08:AddressDetails.Country.Code WHERE [AddressDetails.Usage.Code='RES'].[AddressDetails.Currency.Code='C']
            
                    RP:^PTR329 = pyde.02.00:AddressDetails.OverseasAddress.Indicator WHERE [AddressDetails.Usage.Code='RES'].[AddressDetails.Currency.Code='C']
                    */
                    assertion = (string.IsNullOrWhiteSpace(statementOfDistribution.PTR337) != true && statementOfDistribution.PTR337 != @"au" && statementOfDistribution.PTR329 == false);
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.GEN.410212",
                            Severity = ProcessMessageSeverity.Error,
                            Description = @"If an overseas Country Code is used, the Overseas Address indicator must be 'TRUE'",
                            LongDescription = @"If the address is an Australian address, the overseas address indicator must be set to 'FALSE' and you do not need to supply a Country Code. If the address is an overseas address, the overseas address indicator must be set to 'TRUE' and you must supply a Country Code other than 'au'",
                            Location = "/xbrli:xbrl/ptr.0006.lodge.req.02.00:StatementOfDistribution" + OccurrenceIndex(statementOfDistribution.OccurrenceIndex) + "/address2.02.02:AddressDetails" + OccurrenceIndex(statementOfDistribution.StatementOfDistribution_AddressDetailsCollection_RES_C_RPOccurrenceIndex) + "/pyde.02.08:AddressDetails.Country.Code",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.410212" } },
                        };
                        processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "PTR337", Value = statementOfDistribution.PTR337 });

                        processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "PTR329", Value = GetValueOrEmpty(statementOfDistribution.PTR329) });

                        response.Add(processMessage);
                    }
                    #endregion // VR.ATO.GEN.410212

                    #region VR.ATO.GEN.410213

                    /*  VR.ATO.GEN.410213
                    Country Name must be left blank
    
                    Legacy Rule Format:
                    IF (pyde.xx.xx:AddressDetails.CountryName.Text <> NULLORBLANK)
                        RETURN VALIDATION MESSAGE
                    ENDIF
        
                    Technical Business Rule Format:
                    ^PTR156 <> BLANK
            
                    Data Elements:
            
                    RP:^PTR156 = pyde.02.08:AddressDetails.CountryName.Text WHERE [AddressDetails.Usage.Code='BUS'].[AddressDetails.Currency.Code='C']
                    */
                    assertion = (string.IsNullOrWhiteSpace(statementOfDistribution.PTR156) != true);
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.GEN.410213",
                            Severity = ProcessMessageSeverity.Error,
                            Description = @"Country Name must be left blank",
                            LongDescription = @"For SBR submissions to the ATO, the Country Name must be left blank. To supply the country for an overseas address please supply a Country Code",
                            Location = "/xbrli:xbrl/ptr.0006.lodge.req.02.00:StatementOfDistribution" + OccurrenceIndex(statementOfDistribution.OccurrenceIndex) + "/address2.02.02:AddressDetails" + OccurrenceIndex(statementOfDistribution.StatementOfDistribution_AddressDetailsCollection_BUS_C_RPOccurrenceIndex) + "/pyde.02.08:AddressDetails.CountryName.Text",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.410213" } },
                        };
                        processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "PTR156", Value = statementOfDistribution.PTR156 });

                        response.Add(processMessage);
                    }
                    #endregion // VR.ATO.GEN.410213

                    #region VR.ATO.GEN.410213

                    /*  VR.ATO.GEN.410213
                    Country Name must be left blank
    
                    Legacy Rule Format:
                    IF (pyde.xx.xx:AddressDetails.CountryName.Text <> NULLORBLANK)
                        RETURN VALIDATION MESSAGE
                    ENDIF
        
                    Technical Business Rule Format:
                    ^PTR341 <> BLANK
            
                    Data Elements:
            
                    RP:^PTR341 = pyde.02.08:AddressDetails.CountryName.Text WHERE [AddressDetails.Usage.Code='RES'].[AddressDetails.Currency.Code='C']
                    */
                    assertion = (string.IsNullOrWhiteSpace(statementOfDistribution.PTR341) != true);
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.GEN.410213",
                            Severity = ProcessMessageSeverity.Error,
                            Description = @"Country Name must be left blank",
                            LongDescription = @"For SBR submissions to the ATO, the Country Name must be left blank. To supply the country for an overseas address please supply a Country Code",
                            Location = "/xbrli:xbrl/ptr.0006.lodge.req.02.00:StatementOfDistribution" + OccurrenceIndex(statementOfDistribution.OccurrenceIndex) + "/address2.02.02:AddressDetails" + OccurrenceIndex(statementOfDistribution.StatementOfDistribution_AddressDetailsCollection_RES_C_RPOccurrenceIndex) + "/pyde.02.08:AddressDetails.CountryName.Text",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.410213" } },
                        };
                        processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "PTR341", Value = statementOfDistribution.PTR341 });

                        response.Add(processMessage);
                    }
                    #endregion // VR.ATO.GEN.410213

                    #region VR.ATO.GEN.410214

                    /*  VR.ATO.GEN.410214
                    Address Line 1 must be present
    
                    Legacy Rule Format:
                    WHERE IN TUPLE(addressdetails2.xx.xx:AddressDetails)
                    IF (pyde.xx.xx:AddressDetails.Line1.Text = NULLORBLANK)
                      RETURN VALIDATION MESSAGE
                    ENDIF
        
                    Technical Business Rule Format:
                    (^AddressDetails <> NULL) AND ((^PTR151 = BLANK))
            
                    Data Elements:
            
                    RP:^PTR151 = pyde.02.00:AddressDetails.Line1.Text WHERE [AddressDetails.Usage.Code='BUS'].[AddressDetails.Currency.Code='C']
            
                    ^AddressDetails = address2.02.02:AddressDetails
                    */
                    assertion = (statementOfDistribution.StatementOfDistribution_AddressDetailsCollection_BUS_C_RPExists != false && string.IsNullOrWhiteSpace(statementOfDistribution.PTR151) == true);
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.GEN.410214",
                            Severity = ProcessMessageSeverity.Error,
                            Description = @"Address Line 1 must be present",
                            Location = "/xbrli:xbrl/ptr.0006.lodge.req.02.00:StatementOfDistribution" + OccurrenceIndex(statementOfDistribution.OccurrenceIndex) + "/address2.02.02:AddressDetails" + OccurrenceIndex(statementOfDistribution.StatementOfDistribution_AddressDetailsCollection_BUS_C_RPOccurrenceIndex) + "/pyde.02.00:AddressDetails.Line1.Text",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.410214" } },
                        };
                        processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "PTR151", Value = statementOfDistribution.PTR151 });

                        response.Add(processMessage);
                    }
                    #endregion // VR.ATO.GEN.410214

                    #region VR.ATO.GEN.410214

                    /*  VR.ATO.GEN.410214
                    Address Line 1 must be present
    
                    Legacy Rule Format:
                    WHERE IN TUPLE(addressdetails2.xx.xx:AddressDetails)
                    IF (pyde.xx.xx:AddressDetails.Line1.Text = NULLORBLANK)
                      RETURN VALIDATION MESSAGE
                    ENDIF
        
                    Technical Business Rule Format:
                    (^AddressDetails <> NULL) AND ((^PTR332 = BLANK))
            
                    Data Elements:
            
                    RP:^PTR332 = pyde.02.00:AddressDetails.Line1.Text WHERE [AddressDetails.Usage.Code='RES'].[AddressDetails.Currency.Code='C']
            
                    ^AddressDetails = address2.02.02:AddressDetails
                    */
                    assertion = (statementOfDistribution.StatementOfDistribution_AddressDetailsCollection_RES_C_RPExists != false && string.IsNullOrWhiteSpace(statementOfDistribution.PTR332) == true);
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.GEN.410214",
                            Severity = ProcessMessageSeverity.Error,
                            Description = @"Address Line 1 must be present",
                            Location = "/xbrli:xbrl/ptr.0006.lodge.req.02.00:StatementOfDistribution" + OccurrenceIndex(statementOfDistribution.OccurrenceIndex) + "/address2.02.02:AddressDetails" + OccurrenceIndex(statementOfDistribution.StatementOfDistribution_AddressDetailsCollection_RES_C_RPOccurrenceIndex) + "/pyde.02.00:AddressDetails.Line1.Text",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.410214" } },
                        };
                        processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "PTR332", Value = statementOfDistribution.PTR332 });

                        response.Add(processMessage);
                    }
                    #endregion // VR.ATO.GEN.410214

                    #region VR.ATO.GEN.410215

                    /*  VR.ATO.GEN.410215
                    Organisation name must be present
    
                    Legacy Rule Format:
                    WHERE IN TUPLE(organisationname2.xx.xx:OrganisationNameDetails)
                    IF (pyde.xx.xx:OrganisationNameDetails.OrganisationalName.Text = NULLORBLANK)
                       RETURN VALIDATION MESSAGE
                    ENDIF
        
                    Technical Business Rule Format:
                    (^ORG <> NULL) AND ((^PTR149 = BLANK))
            
                    Data Elements:
            
                    RP:^PTR149 = pyde.02.00:OrganisationNameDetails.OrganisationalName.Text WHERE [OrganisationNameDetails.OrganisationalNameType.Code='MN'].[OrganisationNameDetails.Currency.Code='C']
            
                    ^ORG = orgname2.02.00:OrganisationNameDetails
                    */
                    assertion = (statementOfDistribution.StatementOfDistribution_OrganisationNameDetailsCollection_MN_C_RPExists != false && string.IsNullOrWhiteSpace(statementOfDistribution.PTR149) == true);
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.GEN.410215",
                            Severity = ProcessMessageSeverity.Error,
                            Description = @"Organisation name must be present",
                            Location = "/xbrli:xbrl/ptr.0006.lodge.req.02.00:StatementOfDistribution" + OccurrenceIndex(statementOfDistribution.OccurrenceIndex) + "/orgname2.02.00:OrganisationNameDetails" + OccurrenceIndex(statementOfDistribution.StatementOfDistribution_OrganisationNameDetailsCollection_MN_C_RPOccurrenceIndex) + "/pyde.02.00:OrganisationNameDetails.OrganisationalName.Text",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.410215" } },
                        };
                        processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "PTR149", Value = statementOfDistribution.PTR149 });

                        response.Add(processMessage);
                    }
                    #endregion // VR.ATO.GEN.410215

                    #region VR.ATO.GEN.410217

                    /*  VR.ATO.GEN.410217
                    Family Name must be present
    
                    Legacy Rule Format:
                    WHERE IN TUPLE(personstructuredname3.xx.xx:PersonNameDetails)
                    IF (pyde.xx.xx:PersonNameDetails.FamilyName.Text = NULLORBLANK)
                        RETURN VALIDATION MESSAGE
                    ENDIF
        
                    Technical Business Rule Format:
                    (^PNAM <> NULL) AND ((^PTR146 = BLANK))
            
                    Data Elements:
            
                    RP:^PTR146 = pyde.02.00:PersonNameDetails.FamilyName.Text WHERE [PersonNameDetails.PersonNameType.Code='LGL'].[PersonNameDetails.Usage.Code='Contact']
            
                    ^PNAM = prsnstrcnm3.02.01:PersonNameDetails
                    */
                    assertion = (statementOfDistribution.StatementOfDistribution_PersonNameDetailsCollection_LGL_Contact_RPExists != false && string.IsNullOrWhiteSpace(statementOfDistribution.PTR146) == true);
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.GEN.410217",
                            Severity = ProcessMessageSeverity.Error,
                            Description = @"Family Name must be present",
                            Location = "/xbrli:xbrl/ptr.0006.lodge.req.02.00:StatementOfDistribution" + OccurrenceIndex(statementOfDistribution.OccurrenceIndex) + "/prsnstrcnm3.02.01:PersonNameDetails" + OccurrenceIndex(statementOfDistribution.StatementOfDistribution_PersonNameDetailsCollection_LGL_Contact_RPOccurrenceIndex) + "/pyde.02.00:PersonNameDetails.FamilyName.Text",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.410217" } },
                        };
                        processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "PTR146", Value = statementOfDistribution.PTR146 });

                        response.Add(processMessage);
                    }
                    #endregion // VR.ATO.GEN.410217

                    #region VR.ATO.GEN.410218

                    /*  VR.ATO.GEN.410218
                    FirstName must be entered if OtherGivenNames is entered
    
                    Legacy Rule Format:
                    WHERE IN TUPLE(personstructuredname3.xx.xx:PersonNameDetails) 
                    IF (pyde.xx.xx:PersonNameDetails.OtherGivenName.Text <> NULLORBLANK) AND (pyde.xx.xx:PersonNameDetails.GivenName.Text = NULLORBLANK) 
                        RETURN VALIDATION MESSAGE 
                     ENDIF
        
                    Technical Business Rule Format:
                    (^PNAM <> NULL) AND ((^PTR148 <> BLANK) AND (^PTR147 = BLANK))
            
                    Data Elements:
            
                    RP:^PTR147 = pyde.02.00:PersonNameDetails.GivenName.Text WHERE [PersonNameDetails.PersonNameType.Code='LGL'].[PersonNameDetails.Usage.Code='Contact']
            
                    RP:^PTR148 = pyde.02.00:PersonNameDetails.OtherGivenName.Text WHERE [PersonNameDetails.PersonNameType.Code='LGL'].[PersonNameDetails.Usage.Code='Contact']
            
                    ^PNAM = prsnstrcnm3.02.01:PersonNameDetails
                    */
                    assertion = (statementOfDistribution.StatementOfDistribution_PersonNameDetailsCollection_LGL_Contact_RPExists != false && (string.IsNullOrWhiteSpace(statementOfDistribution.PTR148) != true && string.IsNullOrWhiteSpace(statementOfDistribution.PTR147) == true));
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.GEN.410218",
                            Severity = ProcessMessageSeverity.Error,
                            Description = @"FirstName must be entered if OtherGivenNames is entered",
                            Location = "/xbrli:xbrl/ptr.0006.lodge.req.02.00:StatementOfDistribution" + OccurrenceIndex(statementOfDistribution.OccurrenceIndex) + "/prsnstrcnm3.02.01:PersonNameDetails" + OccurrenceIndex(statementOfDistribution.StatementOfDistribution_PersonNameDetailsCollection_LGL_Contact_RPOccurrenceIndex) + "/pyde.02.00:PersonNameDetails.GivenName.Text",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.410218" } },
                        };
                        processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "PTR148", Value = statementOfDistribution.PTR148 });

                        processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "PTR147", Value = statementOfDistribution.PTR147 });

                        response.Add(processMessage);
                    }
                    #endregion // VR.ATO.GEN.410218

                    #region VR.ATO.GEN.428230

                    /*  VR.ATO.GEN.428230
                    Suburb/Town/City cannot contain state code
    
                    Legacy Rule Format:
                    IF (pyde.xx.xx:AddressDetails.LocalityName.Text = FOUND("QLD","NSW","VIC","SA","WA","NT","ACT","TAS"))
                       RETURN VALIDATION MESSAGE
                    ENDIF
        
                    Technical Business Rule Format:
                    (FoundSet(^PTR153, '"QLD","NSW","VIC","SA","WA","NT","ACT","TAS"'))
            
                    Data Elements:
            
                    RP:^PTR153 = pyde.02.00:AddressDetails.LocalityName.Text WHERE [AddressDetails.Usage.Code='BUS'].[AddressDetails.Currency.Code='C']
                    */
                    assertion = IsMatch(statementOfDistribution.PTR153, @"(^(QLD|NSW|VIC|SA|WA|NT|ACT|TAS)$)|(^(.*)( QLD | NSW | VIC | SA | WA | NT | ACT | TAS )(.*)$)|(^(QLD |NSW |VIC |SA |WA |NT |ACT |TAS )(.*)$)|(^(.*)( QLD| NSW| VIC| SA| WA| NT| ACT| TAS)$)", RegexOptions.IgnoreCase);
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.GEN.000411",
                            Severity = ProcessMessageSeverity.Error,
                            Description = @"Suburb/Town/City cannot contain state code",
                            Location = "/xbrli:xbrl/ptr.0006.lodge.req.02.00:StatementOfDistribution" + OccurrenceIndex(statementOfDistribution.OccurrenceIndex) + "/address2.02.02:AddressDetails" + OccurrenceIndex(statementOfDistribution.StatementOfDistribution_AddressDetailsCollection_BUS_C_RPOccurrenceIndex) + "/pyde.02.00:AddressDetails.LocalityName.Text",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.428230" } },
                        };
                        processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "PTR153", Value = statementOfDistribution.PTR153 });

                        response.Add(processMessage);
                    }
                    #endregion // VR.ATO.GEN.428230

                    #region VR.ATO.GEN.428230

                    /*  VR.ATO.GEN.428230
                    Suburb/Town/City cannot contain state code
    
                    Legacy Rule Format:
                    IF (pyde.xx.xx:AddressDetails.LocalityName.Text = FOUND("QLD","NSW","VIC","SA","WA","NT","ACT","TAS"))
                       RETURN VALIDATION MESSAGE
                    ENDIF
        
                    Technical Business Rule Format:
                    (FoundSet(^PTR334, '"QLD","NSW","VIC","SA","WA","NT","ACT","TAS"'))
            
                    Data Elements:
            
                    RP:^PTR334 = pyde.02.00:AddressDetails.LocalityName.Text WHERE [AddressDetails.Usage.Code='RES'].[AddressDetails.Currency.Code='C']
                    */
                    assertion = IsMatch(statementOfDistribution.PTR334, @"(^(QLD|NSW|VIC|SA|WA|NT|ACT|TAS)$)|(^(.*)( QLD | NSW | VIC | SA | WA | NT | ACT | TAS )(.*)$)|(^(QLD |NSW |VIC |SA |WA |NT |ACT |TAS )(.*)$)|(^(.*)( QLD| NSW| VIC| SA| WA| NT| ACT| TAS)$)", RegexOptions.IgnoreCase);
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.GEN.000411",
                            Severity = ProcessMessageSeverity.Error,
                            Description = @"Suburb/Town/City cannot contain state code",
                            Location = "/xbrli:xbrl/ptr.0006.lodge.req.02.00:StatementOfDistribution" + OccurrenceIndex(statementOfDistribution.OccurrenceIndex) + "/address2.02.02:AddressDetails" + OccurrenceIndex(statementOfDistribution.StatementOfDistribution_AddressDetailsCollection_RES_C_RPOccurrenceIndex) + "/pyde.02.00:AddressDetails.LocalityName.Text",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.428230" } },
                        };
                        processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "PTR334", Value = statementOfDistribution.PTR334 });

                        response.Add(processMessage);
                    }
                    #endregion // VR.ATO.GEN.428230

                    #region VR.ATO.GEN.428231

                    /*  VR.ATO.GEN.428231
                    Family name cannot contain suffix ESQ, II, III, IV, JNR, JP, MHA, MHR, MLA, MLC, MP, QC or SNR
    
                    Legacy Rule Format:
                    IF (pyde.xx.xx:PersonNameDetails.FamilyName.Text = FOUND("ESQ","II","III","IV","JNR","JP","MHA","MHR","MLA","MLC","MP","QC","SNR"))
                       RETURN VALIDATION MESSAGE
                    ENDIF
        
                    Technical Business Rule Format:
                    (FoundSet(^PTR146, '"ESQ","II","III","IV","JNR","JP","MHA","MHR","MLA","MLC","MP","QC","SNR"'))
            
                    Data Elements:
            
                    RP:^PTR146 = pyde.02.00:PersonNameDetails.FamilyName.Text WHERE [PersonNameDetails.PersonNameType.Code='LGL'].[PersonNameDetails.Usage.Code='Contact']
                    */
                    assertion = IsMatch(statementOfDistribution.PTR146, @"(^(ESQ|II|III|IV|JNR|JP|MHA|MHR|MLA|MLC|MP|QC|SNR)$)|(^(.*)( ESQ | II | III | IV | JNR | JP | MHA | MHR | MLA | MLC | MP | QC | SNR )(.*)$)|(^(ESQ |II |III |IV |JNR |JP |MHA |MHR |MLA |MLC |MP |QC |SNR )(.*)$)|(^(.*)( ESQ| II| III| IV| JNR| JP| MHA| MHR| MLA| MLC| MP| QC| SNR)$)", RegexOptions.IgnoreCase);
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.GEN.000422",
                            Severity = ProcessMessageSeverity.Error,
                            Description = @"Family name cannot contain suffix ESQ, II, III, IV, JNR, JP, MHA, MHR, MLA, MLC, MP, QC or SNR",
                            Location = "/xbrli:xbrl/ptr.0006.lodge.req.02.00:StatementOfDistribution" + OccurrenceIndex(statementOfDistribution.OccurrenceIndex) + "/prsnstrcnm3.02.01:PersonNameDetails" + OccurrenceIndex(statementOfDistribution.StatementOfDistribution_PersonNameDetailsCollection_LGL_Contact_RPOccurrenceIndex) + "/pyde.02.00:PersonNameDetails.FamilyName.Text",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.428231" } },
                        };
                        processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "PTR146", Value = statementOfDistribution.PTR146 });

                        response.Add(processMessage);
                    }
                    #endregion // VR.ATO.GEN.428231

                    #region VR.ATO.GEN.428254

                    /*  VR.ATO.GEN.428254
                    Address line 1 cannot contain "UNKNOWN"
    
                    Legacy Rule Format:
                    IF (pyde.xx.xx:AddressDetails.Line1.Text CONTAINS "UNKNOWN")
                       RETURN VALIDATION MESSAGE
                    ENDIF
        
                    Technical Business Rule Format:
                    Contains(^PTR151, 'UNKNOWN')
            
                    Data Elements:
            
                    RP:^PTR151 = pyde.02.00:AddressDetails.Line1.Text WHERE [AddressDetails.Usage.Code='BUS'].[AddressDetails.Currency.Code='C']
                    */
                    assertion = (statementOfDistribution.PTR151 == null ? false : statementOfDistribution.PTR151.ToUpperInvariant().Contains(@"UNKNOWN"));
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.GEN.000405",
                            Severity = ProcessMessageSeverity.Error,
                            Description = @"Address line 1 cannot contain ""UNKNOWN""",
                            Location = "/xbrli:xbrl/ptr.0006.lodge.req.02.00:StatementOfDistribution" + OccurrenceIndex(statementOfDistribution.OccurrenceIndex) + "/address2.02.02:AddressDetails" + OccurrenceIndex(statementOfDistribution.StatementOfDistribution_AddressDetailsCollection_BUS_C_RPOccurrenceIndex) + "/pyde.02.00:AddressDetails.Line1.Text",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.428254" } },
                        };
                        processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "PTR151", Value = statementOfDistribution.PTR151 });

                        response.Add(processMessage);
                    }
                    #endregion // VR.ATO.GEN.428254

                    #region VR.ATO.GEN.428254

                    /*  VR.ATO.GEN.428254
                    Address line 1 cannot contain "UNKNOWN"
    
                    Legacy Rule Format:
                    IF (pyde.xx.xx:AddressDetails.Line1.Text CONTAINS "UNKNOWN")
                       RETURN VALIDATION MESSAGE
                    ENDIF
        
                    Technical Business Rule Format:
                    Contains(^PTR332, 'UNKNOWN')
            
                    Data Elements:
            
                    RP:^PTR332 = pyde.02.00:AddressDetails.Line1.Text WHERE [AddressDetails.Usage.Code='RES'].[AddressDetails.Currency.Code='C']
                    */
                    assertion = (statementOfDistribution.PTR332 == null ? false : statementOfDistribution.PTR332.ToUpperInvariant().Contains(@"UNKNOWN"));
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.GEN.000405",
                            Severity = ProcessMessageSeverity.Error,
                            Description = @"Address line 1 cannot contain ""UNKNOWN""",
                            Location = "/xbrli:xbrl/ptr.0006.lodge.req.02.00:StatementOfDistribution" + OccurrenceIndex(statementOfDistribution.OccurrenceIndex) + "/address2.02.02:AddressDetails" + OccurrenceIndex(statementOfDistribution.StatementOfDistribution_AddressDetailsCollection_RES_C_RPOccurrenceIndex) + "/pyde.02.00:AddressDetails.Line1.Text",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.428254" } },
                        };
                        processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "PTR332", Value = statementOfDistribution.PTR332 });

                        response.Add(processMessage);
                    }
                    #endregion // VR.ATO.GEN.428254

                    #region VR.ATO.GEN.428255

                    /*  VR.ATO.GEN.428255
                    Address line 2 cannot contain "UNKNOWN"
    
                    Legacy Rule Format:
                    IF (pyde.xx.xx:AddressDetails.Line2.Text CONTAINS "UNKNOWN")
                       RETURN VALIDATION MESSAGE
                    ENDIF
        
                    Technical Business Rule Format:
                    Contains(^PTR152, 'UNKNOWN')
            
                    Data Elements:
            
                    RP:^PTR152 = pyde.02.00:AddressDetails.Line2.Text WHERE [AddressDetails.Usage.Code='BUS'].[AddressDetails.Currency.Code='C']
                    */
                    assertion = (statementOfDistribution.PTR152 == null ? false : statementOfDistribution.PTR152.ToUpperInvariant().Contains(@"UNKNOWN"));
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.GEN.000407",
                            Severity = ProcessMessageSeverity.Error,
                            Description = @"Address line 2 cannot contain ""UNKNOWN""",
                            Location = "/xbrli:xbrl/ptr.0006.lodge.req.02.00:StatementOfDistribution" + OccurrenceIndex(statementOfDistribution.OccurrenceIndex) + "/address2.02.02:AddressDetails" + OccurrenceIndex(statementOfDistribution.StatementOfDistribution_AddressDetailsCollection_BUS_C_RPOccurrenceIndex) + "/pyde.02.00:AddressDetails.Line2.Text",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.428255" } },
                        };
                        processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "PTR152", Value = statementOfDistribution.PTR152 });

                        response.Add(processMessage);
                    }
                    #endregion // VR.ATO.GEN.428255

                    #region VR.ATO.GEN.428255

                    /*  VR.ATO.GEN.428255
                    Address line 2 cannot contain "UNKNOWN"
    
                    Legacy Rule Format:
                    IF (pyde.xx.xx:AddressDetails.Line2.Text CONTAINS "UNKNOWN")
                       RETURN VALIDATION MESSAGE
                    ENDIF
        
                    Technical Business Rule Format:
                    Contains(^PTR333, 'UNKNOWN')
            
                    Data Elements:
            
                    RP:^PTR333 = pyde.02.00:AddressDetails.Line2.Text WHERE [AddressDetails.Usage.Code='RES'].[AddressDetails.Currency.Code='C']
                    */
                    assertion = (statementOfDistribution.PTR333 == null ? false : statementOfDistribution.PTR333.ToUpperInvariant().Contains(@"UNKNOWN"));
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.GEN.000407",
                            Severity = ProcessMessageSeverity.Error,
                            Description = @"Address line 2 cannot contain ""UNKNOWN""",
                            Location = "/xbrli:xbrl/ptr.0006.lodge.req.02.00:StatementOfDistribution" + OccurrenceIndex(statementOfDistribution.OccurrenceIndex) + "/address2.02.02:AddressDetails" + OccurrenceIndex(statementOfDistribution.StatementOfDistribution_AddressDetailsCollection_RES_C_RPOccurrenceIndex) + "/pyde.02.00:AddressDetails.Line2.Text",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.428255" } },
                        };
                        processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "PTR333", Value = statementOfDistribution.PTR333 });

                        response.Add(processMessage);
                    }
                    #endregion // VR.ATO.GEN.428255

                    #region VR.ATO.GEN.428256

                    /*  VR.ATO.GEN.428256
                    Suburb/Town/City cannot contain "UNKNOWN"
    
                    Legacy Rule Format:
                    IF (pyde.xx.xx:AddressDetails.LocalityName.Text CONTAINS "UNKNOWN")
                       RETURN VALIDATION MESSAGE
                    ENDIF
        
                    Technical Business Rule Format:
                    Contains(^PTR153, 'UNKNOWN')
            
                    Data Elements:
            
                    RP:^PTR153 = pyde.02.00:AddressDetails.LocalityName.Text WHERE [AddressDetails.Usage.Code='BUS'].[AddressDetails.Currency.Code='C']
                    */
                    assertion = (statementOfDistribution.PTR153 == null ? false : statementOfDistribution.PTR153.ToUpperInvariant().Contains(@"UNKNOWN"));
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.GEN.000409",
                            Severity = ProcessMessageSeverity.Error,
                            Description = @"Suburb/Town/City cannot contain ""UNKNOWN""",
                            Location = "/xbrli:xbrl/ptr.0006.lodge.req.02.00:StatementOfDistribution" + OccurrenceIndex(statementOfDistribution.OccurrenceIndex) + "/address2.02.02:AddressDetails" + OccurrenceIndex(statementOfDistribution.StatementOfDistribution_AddressDetailsCollection_BUS_C_RPOccurrenceIndex) + "/pyde.02.00:AddressDetails.LocalityName.Text",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.428256" } },
                        };
                        processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "PTR153", Value = statementOfDistribution.PTR153 });

                        response.Add(processMessage);
                    }
                    #endregion // VR.ATO.GEN.428256

                    #region VR.ATO.GEN.428256

                    /*  VR.ATO.GEN.428256
                    Suburb/Town/City cannot contain "UNKNOWN"
    
                    Legacy Rule Format:
                    IF (pyde.xx.xx:AddressDetails.LocalityName.Text CONTAINS "UNKNOWN")
                       RETURN VALIDATION MESSAGE
                    ENDIF
        
                    Technical Business Rule Format:
                    Contains(^PTR334, 'UNKNOWN')
            
                    Data Elements:
            
                    RP:^PTR334 = pyde.02.00:AddressDetails.LocalityName.Text WHERE [AddressDetails.Usage.Code='RES'].[AddressDetails.Currency.Code='C']
                    */
                    assertion = (statementOfDistribution.PTR334 == null ? false : statementOfDistribution.PTR334.ToUpperInvariant().Contains(@"UNKNOWN"));
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.GEN.000409",
                            Severity = ProcessMessageSeverity.Error,
                            Description = @"Suburb/Town/City cannot contain ""UNKNOWN""",
                            Location = "/xbrli:xbrl/ptr.0006.lodge.req.02.00:StatementOfDistribution" + OccurrenceIndex(statementOfDistribution.OccurrenceIndex) + "/address2.02.02:AddressDetails" + OccurrenceIndex(statementOfDistribution.StatementOfDistribution_AddressDetailsCollection_RES_C_RPOccurrenceIndex) + "/pyde.02.00:AddressDetails.LocalityName.Text",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.428256" } },
                        };
                        processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "PTR334", Value = statementOfDistribution.PTR334 });

                        response.Add(processMessage);
                    }
                    #endregion // VR.ATO.GEN.428256

                    #region VR.ATO.GEN.428258

                    /*  VR.ATO.GEN.428258
                    Non Individual Name cannot contain space hyphen space combinations
    
                    Legacy Rule Format:
                    IF (pyde.xx.xx:OrganisationNameDetails.OrganisationalName.Text CONTAINS  " - " )
                       RETURN VALIDATION MESSAGE
                    ENDIF
        
                    Technical Business Rule Format:
                    Contains(^PTR149, ' - ')
            
                    Data Elements:
            
                    RP:^PTR149 = pyde.02.00:OrganisationNameDetails.OrganisationalName.Text WHERE [OrganisationNameDetails.OrganisationalNameType.Code='MN'].[OrganisationNameDetails.Currency.Code='C']
                    */
                    assertion = (statementOfDistribution.PTR149 == null ? false : statementOfDistribution.PTR149.ToUpperInvariant().Contains(@" - "));
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.GEN.000413",
                            Severity = ProcessMessageSeverity.Error,
                            Description = @"Non Individual Name cannot contain space hyphen space combinations",
                            Location = "/xbrli:xbrl/ptr.0006.lodge.req.02.00:StatementOfDistribution" + OccurrenceIndex(statementOfDistribution.OccurrenceIndex) + "/orgname2.02.00:OrganisationNameDetails" + OccurrenceIndex(statementOfDistribution.StatementOfDistribution_OrganisationNameDetailsCollection_MN_C_RPOccurrenceIndex) + "/pyde.02.00:OrganisationNameDetails.OrganisationalName.Text",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.428258" } },
                        };
                        processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "PTR149", Value = statementOfDistribution.PTR149 });

                        response.Add(processMessage);
                    }
                    #endregion // VR.ATO.GEN.428258

                    #region VR.ATO.GEN.428259

                    /*  VR.ATO.GEN.428259
                    Non Individual Name cannot contain "P/L"
    
                    Legacy Rule Format:
                    IF (pyde.xx.xx:OrganisationNameDetails.OrganisationalName.Text CONTAINS "P/L")
                       RETURN VALIDATION MESSAGE
                    ENDIF
        
                    Technical Business Rule Format:
                    Contains(^PTR149, 'P/L')
            
                    Data Elements:
            
                    RP:^PTR149 = pyde.02.00:OrganisationNameDetails.OrganisationalName.Text WHERE [OrganisationNameDetails.OrganisationalNameType.Code='MN'].[OrganisationNameDetails.Currency.Code='C']
                    */
                    assertion = (statementOfDistribution.PTR149 == null ? false : statementOfDistribution.PTR149.ToUpperInvariant().Contains(@"P/L"));
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.GEN.000414",
                            Severity = ProcessMessageSeverity.Error,
                            Description = @"Non Individual Name cannot contain ""P/L""",
                            Location = "/xbrli:xbrl/ptr.0006.lodge.req.02.00:StatementOfDistribution" + OccurrenceIndex(statementOfDistribution.OccurrenceIndex) + "/orgname2.02.00:OrganisationNameDetails" + OccurrenceIndex(statementOfDistribution.StatementOfDistribution_OrganisationNameDetailsCollection_MN_C_RPOccurrenceIndex) + "/pyde.02.00:OrganisationNameDetails.OrganisationalName.Text",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.428259" } },
                        };
                        processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "PTR149", Value = statementOfDistribution.PTR149 });

                        response.Add(processMessage);
                    }
                    #endregion // VR.ATO.GEN.428259

                    #region VR.ATO.GEN.428260

                    /*  VR.ATO.GEN.428260
                    Non Individual Name cannot contain repeated hyphen, apostrophe, or space characters
    
                    Legacy Rule Format:
                    IF (pyde.xx.xx:OrganisationNameDetails.OrganisationalName.Text CONTAINS SET("--","''","  ") )
                       RETURN VALIDATION MESSAGE
                    ENDIF
        
                    Technical Business Rule Format:
                    (ContainsSet(^PTR149, '"--","''","  "'))
            
                    Data Elements:
            
                    RP:^PTR149 = pyde.02.00:OrganisationNameDetails.OrganisationalName.Text WHERE [OrganisationNameDetails.OrganisationalNameType.Code='MN'].[OrganisationNameDetails.Currency.Code='C']
                    */
                    assertion = IsMatch(statementOfDistribution.PTR149, @"--|''|  ", RegexOptions.IgnoreCase);
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.GEN.000416",
                            Severity = ProcessMessageSeverity.Error,
                            Description = @"Non Individual Name cannot contain repeated hyphen, apostrophe, or space characters",
                            Location = "/xbrli:xbrl/ptr.0006.lodge.req.02.00:StatementOfDistribution" + OccurrenceIndex(statementOfDistribution.OccurrenceIndex) + "/orgname2.02.00:OrganisationNameDetails" + OccurrenceIndex(statementOfDistribution.StatementOfDistribution_OrganisationNameDetailsCollection_MN_C_RPOccurrenceIndex) + "/pyde.02.00:OrganisationNameDetails.OrganisationalName.Text",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.428260" } },
                        };
                        processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "PTR149", Value = statementOfDistribution.PTR149 });

                        response.Add(processMessage);
                    }
                    #endregion // VR.ATO.GEN.428260

                    #region VR.ATO.GEN.428261

                    /*  VR.ATO.GEN.428261
                    Non Individual Name cannot end with "T/A", "T/A P'ship", "T/A Pship", "T/A P/Ship" or "T/A Partnership"
    
                    Legacy Rule Format:
                    IF (pyde.xx.xx:OrganisationNameDetails.OrganisationalName.Text ENDSWITH SET(" T/A"," T/A P'ship"," T/A Pship"," T/A P/Ship"," T/A Partnership"))
                      RETURN VALIDATION MESSAGE
                    ENDIF
        
                    Technical Business Rule Format:
                    (EndsWithSet(^PTR149, '" T/A"," T/A P'ship"," T/A Pship"," T/A P/Ship"," T/A Partnership"'))
            
                    Data Elements:
            
                    RP:^PTR149 = pyde.02.00:OrganisationNameDetails.OrganisationalName.Text WHERE [OrganisationNameDetails.OrganisationalNameType.Code='MN'].[OrganisationNameDetails.Currency.Code='C']
                    */
                    assertion = IsMatch(statementOfDistribution.PTR149, @"( T/A| T/A P'ship| T/A Pship| T/A P/Ship| T/A Partnership)$", RegexOptions.IgnoreCase);
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.GEN.000417",
                            Severity = ProcessMessageSeverity.Error,
                            Description = @"Non Individual Name cannot end with ""T/A"", ""T/A P'ship"", ""T/A Pship"", ""T/A P/Ship"" or ""T/A Partnership""",
                            Location = "/xbrli:xbrl/ptr.0006.lodge.req.02.00:StatementOfDistribution" + OccurrenceIndex(statementOfDistribution.OccurrenceIndex) + "/orgname2.02.00:OrganisationNameDetails" + OccurrenceIndex(statementOfDistribution.StatementOfDistribution_OrganisationNameDetailsCollection_MN_C_RPOccurrenceIndex) + "/pyde.02.00:OrganisationNameDetails.OrganisationalName.Text",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.428261" } },
                        };
                        processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "PTR149", Value = statementOfDistribution.PTR149 });

                        response.Add(processMessage);
                    }
                    #endregion // VR.ATO.GEN.428261

                    #region VR.ATO.GEN.428262

                    /*  VR.ATO.GEN.428262
                    Family Name cannot contain "space hyphen space"
    
                    Legacy Rule Format:
                    IF (pyde.xx.xx:PersonNameDetails.FamilyName.Text CONTAINS " - ")
                       RETURN VALIDATION MESSAGE
                    ENDIF
        
                    Technical Business Rule Format:
                    Contains(^PTR146, ' - ')
            
                    Data Elements:
            
                    RP:^PTR146 = pyde.02.00:PersonNameDetails.FamilyName.Text WHERE [PersonNameDetails.PersonNameType.Code='LGL'].[PersonNameDetails.Usage.Code='Contact']
                    */
                    assertion = (statementOfDistribution.PTR146 == null ? false : statementOfDistribution.PTR146.ToUpperInvariant().Contains(@" - "));
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.GEN.000423",
                            Severity = ProcessMessageSeverity.Error,
                            Description = @"Family Name cannot contain ""space hyphen space""",
                            Location = "/xbrli:xbrl/ptr.0006.lodge.req.02.00:StatementOfDistribution" + OccurrenceIndex(statementOfDistribution.OccurrenceIndex) + "/prsnstrcnm3.02.01:PersonNameDetails" + OccurrenceIndex(statementOfDistribution.StatementOfDistribution_PersonNameDetailsCollection_LGL_Contact_RPOccurrenceIndex) + "/pyde.02.00:PersonNameDetails.FamilyName.Text",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.428262" } },
                        };
                        processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "PTR146", Value = statementOfDistribution.PTR146 });

                        response.Add(processMessage);
                    }
                    #endregion // VR.ATO.GEN.428262

                    #region VR.ATO.GEN.428263

                    /*  VR.ATO.GEN.428263
                    Family name cannot contain repeated hyphen, apostrophe, or space characters
    
                    Legacy Rule Format:
                    IF (pyde.xx.xx:PersonNameDetails.FamilyName.Text CONTAINS SET("--","''","  ") )
                       RETURN VALIDATION MESSAGE
                    ENDIF
        
                    Technical Business Rule Format:
                    (ContainsSet(^PTR146, '"--","''","  "'))
            
                    Data Elements:
            
                    RP:^PTR146 = pyde.02.00:PersonNameDetails.FamilyName.Text WHERE [PersonNameDetails.PersonNameType.Code='LGL'].[PersonNameDetails.Usage.Code='Contact']
                    */
                    assertion = IsMatch(statementOfDistribution.PTR146, @"--|''|  ", RegexOptions.IgnoreCase);
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.GEN.000427",
                            Severity = ProcessMessageSeverity.Error,
                            Description = @"Family name cannot contain repeated hyphen, apostrophe, or space characters",
                            Location = "/xbrli:xbrl/ptr.0006.lodge.req.02.00:StatementOfDistribution" + OccurrenceIndex(statementOfDistribution.OccurrenceIndex) + "/prsnstrcnm3.02.01:PersonNameDetails" + OccurrenceIndex(statementOfDistribution.StatementOfDistribution_PersonNameDetailsCollection_LGL_Contact_RPOccurrenceIndex) + "/pyde.02.00:PersonNameDetails.FamilyName.Text",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.428263" } },
                        };
                        processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "PTR146", Value = statementOfDistribution.PTR146 });

                        response.Add(processMessage);
                    }
                    #endregion // VR.ATO.GEN.428263

                    #region VR.ATO.GEN.428264

                    /*  VR.ATO.GEN.428264
                    First name cannot contain space hyphen space
    
                    Legacy Rule Format:
                    IF (pyde.xx.xx:PersonNameDetails.GivenName.Text CONTAINS  " - ")
                       RETURN VALIDATION MESSAGE
                    ENDIF
        
                    Technical Business Rule Format:
                    Contains(^PTR147, ' - ')
            
                    Data Elements:
            
                    RP:^PTR147 = pyde.02.00:PersonNameDetails.GivenName.Text WHERE [PersonNameDetails.PersonNameType.Code='LGL'].[PersonNameDetails.Usage.Code='Contact']
                    */
                    assertion = (statementOfDistribution.PTR147 == null ? false : statementOfDistribution.PTR147.ToUpperInvariant().Contains(@" - "));
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.GEN.000434",
                            Severity = ProcessMessageSeverity.Error,
                            Description = @"First name cannot contain space hyphen space",
                            Location = "/xbrli:xbrl/ptr.0006.lodge.req.02.00:StatementOfDistribution" + OccurrenceIndex(statementOfDistribution.OccurrenceIndex) + "/prsnstrcnm3.02.01:PersonNameDetails" + OccurrenceIndex(statementOfDistribution.StatementOfDistribution_PersonNameDetailsCollection_LGL_Contact_RPOccurrenceIndex) + "/pyde.02.00:PersonNameDetails.GivenName.Text",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.428264" } },
                        };
                        processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "PTR147", Value = statementOfDistribution.PTR147 });

                        response.Add(processMessage);
                    }
                    #endregion // VR.ATO.GEN.428264

                    #region VR.ATO.GEN.428265

                    /*  VR.ATO.GEN.428265
                    First name cannot contain repeated hyphen, apostrophe, or space characters
    
                    Legacy Rule Format:
                    IF (pyde.xx.xx:PersonNameDetails.GivenName.Text CONTAINS SET("--","''","  ") )
                       RETURN VALIDATION MESSAGE
                    ENDIF
        
                    Technical Business Rule Format:
                    (ContainsSet(^PTR147, '"--","''","  "'))
            
                    Data Elements:
            
                    RP:^PTR147 = pyde.02.00:PersonNameDetails.GivenName.Text WHERE [PersonNameDetails.PersonNameType.Code='LGL'].[PersonNameDetails.Usage.Code='Contact']
                    */
                    assertion = IsMatch(statementOfDistribution.PTR147, @"--|''|  ", RegexOptions.IgnoreCase);
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.GEN.000439",
                            Severity = ProcessMessageSeverity.Error,
                            Description = @"First name cannot contain repeated hyphen, apostrophe, or space characters",
                            Location = "/xbrli:xbrl/ptr.0006.lodge.req.02.00:StatementOfDistribution" + OccurrenceIndex(statementOfDistribution.OccurrenceIndex) + "/prsnstrcnm3.02.01:PersonNameDetails" + OccurrenceIndex(statementOfDistribution.StatementOfDistribution_PersonNameDetailsCollection_LGL_Contact_RPOccurrenceIndex) + "/pyde.02.00:PersonNameDetails.GivenName.Text",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.428265" } },
                        };
                        processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "PTR147", Value = statementOfDistribution.PTR147 });

                        response.Add(processMessage);
                    }
                    #endregion // VR.ATO.GEN.428265

                    #region VR.ATO.GEN.428266

                    /*  VR.ATO.GEN.428266
                    Other given name cannot contain space hyphen space
    
                    Legacy Rule Format:
                    IF (pyde.xx.xx:PersonNameDetails.OtherGivenName.Text CONTAINS  " - ")
                       RETURN VALIDATION MESSAGE
                    ENDIF
        
                    Technical Business Rule Format:
                    Contains(^PTR148, ' - ')
            
                    Data Elements:
            
                    RP:^PTR148 = pyde.02.00:PersonNameDetails.OtherGivenName.Text WHERE [PersonNameDetails.PersonNameType.Code='LGL'].[PersonNameDetails.Usage.Code='Contact']
                    */
                    assertion = (statementOfDistribution.PTR148 == null ? false : statementOfDistribution.PTR148.ToUpperInvariant().Contains(@" - "));
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.GEN.000446",
                            Severity = ProcessMessageSeverity.Error,
                            Description = @"Other given name cannot contain space hyphen space",
                            Location = "/xbrli:xbrl/ptr.0006.lodge.req.02.00:StatementOfDistribution" + OccurrenceIndex(statementOfDistribution.OccurrenceIndex) + "/prsnstrcnm3.02.01:PersonNameDetails" + OccurrenceIndex(statementOfDistribution.StatementOfDistribution_PersonNameDetailsCollection_LGL_Contact_RPOccurrenceIndex) + "/pyde.02.00:PersonNameDetails.OtherGivenName.Text",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.428266" } },
                        };
                        processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "PTR148", Value = statementOfDistribution.PTR148 });

                        response.Add(processMessage);
                    }
                    #endregion // VR.ATO.GEN.428266

                    #region VR.ATO.GEN.428267

                    /*  VR.ATO.GEN.428267
                    Other given name cannot contain repeated hyphen, apostrophe, or space characters
    
                    Legacy Rule Format:
                    IF (pyde.xx.xx:PersonNameDetails.OtherGivenName.Text CONTAINS SET("--","''","  ") )
                       RETURN VALIDATION MESSAGE
                    ENDIF
        
                    Technical Business Rule Format:
                    (ContainsSet(^PTR148, '"--","''","  "'))
            
                    Data Elements:
            
                    RP:^PTR148 = pyde.02.00:PersonNameDetails.OtherGivenName.Text WHERE [PersonNameDetails.PersonNameType.Code='LGL'].[PersonNameDetails.Usage.Code='Contact']
                    */
                    assertion = IsMatch(statementOfDistribution.PTR148, @"--|''|  ", RegexOptions.IgnoreCase);
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.GEN.000451",
                            Severity = ProcessMessageSeverity.Error,
                            Description = @"Other given name cannot contain repeated hyphen, apostrophe, or space characters",
                            Location = "/xbrli:xbrl/ptr.0006.lodge.req.02.00:StatementOfDistribution" + OccurrenceIndex(statementOfDistribution.OccurrenceIndex) + "/prsnstrcnm3.02.01:PersonNameDetails" + OccurrenceIndex(statementOfDistribution.StatementOfDistribution_PersonNameDetailsCollection_LGL_Contact_RPOccurrenceIndex) + "/pyde.02.00:PersonNameDetails.OtherGivenName.Text",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.428267" } },
                        };
                        processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "PTR148", Value = statementOfDistribution.PTR148 });

                        response.Add(processMessage);
                    }
                    #endregion // VR.ATO.GEN.428267

                    #region VR.ATO.GEN.430245

                    /*  VR.ATO.GEN.430245
                    Suburb/Town/City must be present in an address
    
                    Legacy Rule Format:
                    WHERE IN TUPLE(addressdetails2.xx.xx:AddressDetails)
                    IF (pyde.xx.xx:AddressDetails.LocalityName.Text = NULLORBLANK)
                      RETURN VALIDATION MESSAGE
                    ENDIF
        
                    Technical Business Rule Format:
                    (^AddressDetails <> NULL) AND ((^PTR153 = BLANK))
            
                    Data Elements:
            
                    RP:^PTR153 = pyde.02.00:AddressDetails.LocalityName.Text WHERE [AddressDetails.Usage.Code='BUS'].[AddressDetails.Currency.Code='C']
            
                    ^AddressDetails = address2.02.02:AddressDetails
                    */
                    assertion = (statementOfDistribution.StatementOfDistribution_AddressDetailsCollection_BUS_C_RPExists != false && string.IsNullOrWhiteSpace(statementOfDistribution.PTR153) == true);
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.GEN.430245",
                            Severity = ProcessMessageSeverity.Error,
                            Description = @"Suburb/Town/City must be present in an address",
                            Location = "/xbrli:xbrl/ptr.0006.lodge.req.02.00:StatementOfDistribution" + OccurrenceIndex(statementOfDistribution.OccurrenceIndex) + "/address2.02.02:AddressDetails" + OccurrenceIndex(statementOfDistribution.StatementOfDistribution_AddressDetailsCollection_BUS_C_RPOccurrenceIndex) + "/pyde.02.00:AddressDetails.LocalityName.Text",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.430245" } },
                        };
                        processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "PTR153", Value = statementOfDistribution.PTR153 });

                        response.Add(processMessage);
                    }
                    #endregion // VR.ATO.GEN.430245

                    #region VR.ATO.GEN.430245

                    /*  VR.ATO.GEN.430245
                    Suburb/Town/City must be present in an address
    
                    Legacy Rule Format:
                    WHERE IN TUPLE(addressdetails2.xx.xx:AddressDetails)
                    IF (pyde.xx.xx:AddressDetails.LocalityName.Text = NULLORBLANK)
                      RETURN VALIDATION MESSAGE
                    ENDIF
        
                    Technical Business Rule Format:
                    (^AddressDetails <> NULL) AND ((^PTR334 = BLANK))
            
                    Data Elements:
            
                    RP:^PTR334 = pyde.02.00:AddressDetails.LocalityName.Text WHERE [AddressDetails.Usage.Code='RES'].[AddressDetails.Currency.Code='C']
            
                    ^AddressDetails = address2.02.02:AddressDetails
                    */
                    assertion = (statementOfDistribution.StatementOfDistribution_AddressDetailsCollection_RES_C_RPExists != false && string.IsNullOrWhiteSpace(statementOfDistribution.PTR334) == true);
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.GEN.430245",
                            Severity = ProcessMessageSeverity.Error,
                            Description = @"Suburb/Town/City must be present in an address",
                            Location = "/xbrli:xbrl/ptr.0006.lodge.req.02.00:StatementOfDistribution" + OccurrenceIndex(statementOfDistribution.OccurrenceIndex) + "/address2.02.02:AddressDetails" + OccurrenceIndex(statementOfDistribution.StatementOfDistribution_AddressDetailsCollection_RES_C_RPOccurrenceIndex) + "/pyde.02.00:AddressDetails.LocalityName.Text",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.430245" } },
                        };
                        processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "PTR334", Value = statementOfDistribution.PTR334 });

                        response.Add(processMessage);
                    }
                    #endregion // VR.ATO.GEN.430245

                    #region VR.ATO.GEN.430246

                    /*  VR.ATO.GEN.430246
                    Suburb/Town/City cannot contain numeric characters
    
                    Legacy Rule Format:
                    WHERE IN TUPLE(addressdetails2.xx.xx:AddressDetails)
                    IF (pyde.xx.xx:AddressDetails.OverseasAddress.Indicator = FALSE) AND (pyde.xx.xx:AddressDetails.LocalityName.Text CONTAINS NUMERICS SET(0-9))
                       RETURN VALIDATION MESSAGE
                    ENDIF
        
                    Technical Business Rule Format:
                    (^PTR240 = FALSE) AND (ContainsSet(^PTR153, '0-9'))
            
                    Data Elements:
            
                    RP:^PTR153 = pyde.02.00:AddressDetails.LocalityName.Text WHERE [AddressDetails.Usage.Code='BUS'].[AddressDetails.Currency.Code='C']
            
                    RP:^PTR240 = pyde.02.00:AddressDetails.OverseasAddress.Indicator WHERE [AddressDetails.Usage.Code='BUS'].[AddressDetails.Currency.Code='C']
                    */
                    assertion = (statementOfDistribution.PTR240 == false && IsMatch(statementOfDistribution.PTR153, @"[0-9]", RegexOptions.IgnoreCase));
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.GEN.000410",
                            Severity = ProcessMessageSeverity.Error,
                            Description = @"Suburb/Town/City cannot contain numeric characters",
                            Location = "/xbrli:xbrl/ptr.0006.lodge.req.02.00:StatementOfDistribution" + OccurrenceIndex(statementOfDistribution.OccurrenceIndex) + "/address2.02.02:AddressDetails" + OccurrenceIndex(statementOfDistribution.StatementOfDistribution_AddressDetailsCollection_BUS_C_RPOccurrenceIndex) + "/pyde.02.00:AddressDetails.LocalityName.Text",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.430246" } },
                        };
                        processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "PTR240", Value = GetValueOrEmpty(statementOfDistribution.PTR240) });

                        processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "PTR153", Value = statementOfDistribution.PTR153 });

                        response.Add(processMessage);
                    }
                    #endregion // VR.ATO.GEN.430246

                    #region VR.ATO.GEN.430246

                    /*  VR.ATO.GEN.430246
                    Suburb/Town/City cannot contain numeric characters
    
                    Legacy Rule Format:
                    WHERE IN TUPLE(addressdetails2.xx.xx:AddressDetails)
                    IF (pyde.xx.xx:AddressDetails.OverseasAddress.Indicator = FALSE) AND (pyde.xx.xx:AddressDetails.LocalityName.Text CONTAINS NUMERICS SET(0-9))
                       RETURN VALIDATION MESSAGE
                    ENDIF
        
                    Technical Business Rule Format:
                    (^PTR329 = FALSE) AND (ContainsSet(^PTR334, '0-9'))
            
                    Data Elements:
            
                    RP:^PTR334 = pyde.02.00:AddressDetails.LocalityName.Text WHERE [AddressDetails.Usage.Code='RES'].[AddressDetails.Currency.Code='C']
            
                    RP:^PTR329 = pyde.02.00:AddressDetails.OverseasAddress.Indicator WHERE [AddressDetails.Usage.Code='RES'].[AddressDetails.Currency.Code='C']
                    */
                    assertion = (statementOfDistribution.PTR329 == false && IsMatch(statementOfDistribution.PTR334, @"[0-9]", RegexOptions.IgnoreCase));
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.GEN.000410",
                            Severity = ProcessMessageSeverity.Error,
                            Description = @"Suburb/Town/City cannot contain numeric characters",
                            Location = "/xbrli:xbrl/ptr.0006.lodge.req.02.00:StatementOfDistribution" + OccurrenceIndex(statementOfDistribution.OccurrenceIndex) + "/address2.02.02:AddressDetails" + OccurrenceIndex(statementOfDistribution.StatementOfDistribution_AddressDetailsCollection_RES_C_RPOccurrenceIndex) + "/pyde.02.00:AddressDetails.LocalityName.Text",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.430246" } },
                        };
                        processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "PTR329", Value = GetValueOrEmpty(statementOfDistribution.PTR329) });

                        processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "PTR334", Value = statementOfDistribution.PTR334 });

                        response.Add(processMessage);
                    }
                    #endregion // VR.ATO.GEN.430246

                    #region VR.ATO.GEN.434147

                    /*  VR.ATO.GEN.434147
                    State and post code must be present for an Australian address
    
                    Legacy Rule Format:
                    WHERE IN TUPLE(addressdetails2.xx.xx:AddressDetails)
                    IF pyde.xx.xx:AddressDetails.OverseasAddressIndicator = FALSE AND (pyde.xx.xx:AddressDetails.StateOrTerritory.Code = NULLORBLANK OR pyde.xx.xx:AddressDetails.Postcode.Text = NULLORBLANK)
                       RETURN VALIDATION MESSAGE
                    END IF
        
                    Technical Business Rule Format:
                    (^AddressDetails <> NULL) AND (^PTR240 = FALSE AND (^PTR154 = BLANK OR ^PTR155 = BLANK))
            
                    Data Elements:
            
                    RP:^PTR240 = pyde.02.00:AddressDetails.OverseasAddress.Indicator WHERE [AddressDetails.Usage.Code='BUS'].[AddressDetails.Currency.Code='C']
            
                    RP:^PTR154 = pyde.02.00:AddressDetails.StateOrTerritory.Code WHERE [AddressDetails.Usage.Code='BUS'].[AddressDetails.Currency.Code='C']
            
                    RP:^PTR155 = pyde.02.00:AddressDetails.Postcode.Text WHERE [AddressDetails.Usage.Code='BUS'].[AddressDetails.Currency.Code='C']
            
                    ^AddressDetails = address2.02.02:AddressDetails
                    */
                    assertion = (statementOfDistribution.StatementOfDistribution_AddressDetailsCollection_BUS_C_RPExists != false && (statementOfDistribution.PTR240 == false && (string.IsNullOrWhiteSpace(statementOfDistribution.PTR154) == true || string.IsNullOrWhiteSpace(statementOfDistribution.PTR155) == true)));
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.GEN.434147",
                            Severity = ProcessMessageSeverity.Error,
                            Description = @"State and post code must be present for an Australian address",
                            LongDescription = @"If the overseas indicator is no (false), then an Australian state code and post code must be present",
                            Location = "/xbrli:xbrl/ptr.0006.lodge.req.02.00:StatementOfDistribution" + OccurrenceIndex(statementOfDistribution.OccurrenceIndex) + "/address2.02.02:AddressDetails" + OccurrenceIndex(statementOfDistribution.StatementOfDistribution_AddressDetailsCollection_BUS_C_RPOccurrenceIndex) + "/pyde.02.00:AddressDetails.OverseasAddress.Indicator",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.434147" } },
                        };
                        processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "PTR240", Value = GetValueOrEmpty(statementOfDistribution.PTR240) });

                        processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "PTR154", Value = statementOfDistribution.PTR154 });

                        processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "PTR155", Value = statementOfDistribution.PTR155 });

                        response.Add(processMessage);
                    }
                    #endregion // VR.ATO.GEN.434147

                    #region VR.ATO.GEN.434147

                    /*  VR.ATO.GEN.434147
                    State and post code must be present for an Australian address
    
                    Legacy Rule Format:
                    WHERE IN TUPLE(addressdetails2.xx.xx:AddressDetails)
                    IF pyde.xx.xx:AddressDetails.OverseasAddressIndicator = FALSE AND (pyde.xx.xx:AddressDetails.StateOrTerritory.Code = NULLORBLANK OR pyde.xx.xx:AddressDetails.Postcode.Text = NULLORBLANK)
                       RETURN VALIDATION MESSAGE
                    END IF
        
                    Technical Business Rule Format:
                    (^AddressDetails <> NULL) AND (^PTR329 = FALSE AND (^PTR336 = BLANK OR ^PTR335 = BLANK))
            
                    Data Elements:
            
                    RP:^PTR329 = pyde.02.00:AddressDetails.OverseasAddress.Indicator WHERE [AddressDetails.Usage.Code='RES'].[AddressDetails.Currency.Code='C']
            
                    RP:^PTR336 = pyde.02.00:AddressDetails.StateOrTerritory.Code WHERE [AddressDetails.Usage.Code='RES'].[AddressDetails.Currency.Code='C']
            
                    RP:^PTR335 = pyde.02.00:AddressDetails.Postcode.Text WHERE [AddressDetails.Usage.Code='RES'].[AddressDetails.Currency.Code='C']
            
                    ^AddressDetails = address2.02.02:AddressDetails
                    */
                    assertion = (statementOfDistribution.StatementOfDistribution_AddressDetailsCollection_RES_C_RPExists != false && (statementOfDistribution.PTR329 == false && (string.IsNullOrWhiteSpace(statementOfDistribution.PTR336) == true || string.IsNullOrWhiteSpace(statementOfDistribution.PTR335) == true)));
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.GEN.434147",
                            Severity = ProcessMessageSeverity.Error,
                            Description = @"State and post code must be present for an Australian address",
                            LongDescription = @"If the overseas indicator is no (false), then an Australian state code and post code must be present",
                            Location = "/xbrli:xbrl/ptr.0006.lodge.req.02.00:StatementOfDistribution" + OccurrenceIndex(statementOfDistribution.OccurrenceIndex) + "/address2.02.02:AddressDetails" + OccurrenceIndex(statementOfDistribution.StatementOfDistribution_AddressDetailsCollection_RES_C_RPOccurrenceIndex) + "/pyde.02.00:AddressDetails.OverseasAddress.Indicator",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.434147" } },
                        };
                        processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "PTR329", Value = GetValueOrEmpty(statementOfDistribution.PTR329) });

                        processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "PTR336", Value = statementOfDistribution.PTR336 });

                        processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "PTR335", Value = statementOfDistribution.PTR335 });

                        response.Add(processMessage);
                    }
                    #endregion // VR.ATO.GEN.434147

                    #region VR.ATO.GEN.434223

                    /*  VR.ATO.GEN.434223
                    ABN must pass the ABN algorithm
    
                    Legacy Rule Format:
                    IF (pyid.xx.xx:Identifiers.AustralianBusinessNumber.Identifer <> NULLORBLANK) and (ABNALGORITHM(pyid.xx.xx:Identifiers.AustralianBusinessNumber.Identifier) = FALSE)
                    RETURN VALIDATION MESSAGE
                    ENDIF
        
                    Technical Business Rule Format:
                    (^PTR285 <> BLANK) and (FailsABNAlgorithm(^PTR285))
            
                    Data Elements:
            
                    RP:^PTR285 = pyid.02.00:Identifiers.AustralianBusinessNumber.Identifier
                    */
                    assertion = (string.IsNullOrWhiteSpace(statementOfDistribution.PTR285) != true && FailsABNAlgorithm(statementOfDistribution.PTR285));
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.GEN.000477",
                            Severity = ProcessMessageSeverity.Error,
                            Description = @"ABN is invalid",
                            Location = "/xbrli:xbrl/ptr.0006.lodge.req.02.00:StatementOfDistribution" + OccurrenceIndex(statementOfDistribution.OccurrenceIndex) + "/pyid.02.00:Identifiers.AustralianBusinessNumber.Identifier",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.434223" } },
                        };
                        processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "PTR285", Value = statementOfDistribution.PTR285 });

                        response.Add(processMessage);
                    }
                    #endregion // VR.ATO.GEN.434223

                    #region VR.ATO.PTR.430136

                    /*  VR.ATO.PTR.430136
                    Statement of distribution - name must be present
    
                    Legacy Rule Format:
                    WHERE IN TUPLE(ptr.0006.lodge.req.xx.xx:StatementOfDistribution)
                    IF [PTR146] = NULLORBLANK AND [PTR149] = NULLORBLANK 
                        RETURN VALIDATION MESSAGE
                    ENDIF
                    
                    [PTR146] = PTR:RP:pyde.02.00:PersonNameDetails.FamilyName.Text WHERE (pyde.02.00:PersonNameDetails.PersonNameType.Code = "LGL" AND pyde.02.05:PersonNameDetails.Usage.Code = "Contact") IN TUPLE(prsnstrcnm3.02.01:PersonNameDetails) IN TUPLE(ptr.0006.lodge.req.02.00:StatementOfDistribution)
                    [PTR149] = PTR:RP:pyde.02.00:OrganisationNameDetails.OrganisationalName.Text WHERE (pyde.02.00:OrganisationNameDetails.OrganisationalNameType.Code = "MN" AND pyde.02.00:OrganisationNameDetails.Currency.Code = "C") IN TUPLE(orgname2.02.00:OrganisationNameDetails) IN TUPLE(ptr.0006.lodge.req.02.00:StatementOfDistribution)
        
                    Technical Business Rule Format:
                    ^PTR146 = BLANK AND ^PTR149 = BLANK
            
                    Data Elements:
            
                    RP:^PTR146 = pyde.02.00:PersonNameDetails.FamilyName.Text WHERE [PersonNameDetails.PersonNameType.Code='LGL'].[PersonNameDetails.Usage.Code='Contact']
            
                    RP:^PTR149 = pyde.02.00:OrganisationNameDetails.OrganisationalName.Text WHERE [OrganisationNameDetails.OrganisationalNameType.Code='MN'].[OrganisationNameDetails.Currency.Code='C']
                    */
                    assertion = (string.IsNullOrWhiteSpace(statementOfDistribution.PTR146) == true && string.IsNullOrWhiteSpace(statementOfDistribution.PTR149) == true);
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.GEN.430136",
                            Severity = ProcessMessageSeverity.Error,
                            Description = @"Statement of distribution - name must be present",
                            LongDescription = @"If the partner/beneficiary tax file number or current postal address is present in the Statement of distribution, then an individual  name or non-individual name must be present",
                            Location = "/xbrli:xbrl/ptr.0006.lodge.req.02.00:StatementOfDistribution" + OccurrenceIndex(statementOfDistribution.OccurrenceIndex) + "/prsnstrcnm3.02.01:PersonNameDetails" + OccurrenceIndex(statementOfDistribution.StatementOfDistribution_PersonNameDetailsCollection_LGL_Contact_RPOccurrenceIndex) + "/pyde.02.00:PersonNameDetails.FamilyName.Text",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.PTR.430136" } },
                        };
                        processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "PTR146", Value = statementOfDistribution.PTR146 });

                        processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "PTR149", Value = statementOfDistribution.PTR149 });

                        response.Add(processMessage);
                    }
                    #endregion // VR.ATO.PTR.430136

                    #region VR.ATO.PTR.430139

                    /*  VR.ATO.PTR.430139
                    Statement of distribution must not contain both an individual name and a non-individual name
    
                    Legacy Rule Format:
                    WHERE IN TUPLE(ptr.0006.lodge.req.xx.xx:StatementOfDistribution)
                    IF [PTR146] <> NULLORBLANK AND [PTR149] <> NULLORBLANK
                        RETURN VALIDATION MESSAGE
                    ENDIF
                    
                    [PTR146] = PTR:RP:pyde.02.00:PersonNameDetails.FamilyName.Text WHERE (pyde.02.00:PersonNameDetails.PersonNameType.Code = "LGL" AND pyde.02.05:PersonNameDetails.Usage.Code = "Contact") IN TUPLE(prsnstrcnm3.02.01:PersonNameDetails) IN TUPLE(ptr.0006.lodge.req.02.00:StatementOfDistribution)
                    [PTR149] = PTR:RP:pyde.02.00:OrganisationNameDetails.OrganisationalName.Text WHERE (pyde.02.00:OrganisationNameDetails.OrganisationalNameType.Code = "MN" AND pyde.02.00:OrganisationNameDetails.Currency.Code = "C") IN TUPLE(orgname2.02.00:OrganisationNameDetails) IN TUPLE(ptr.0006.lodge.req.02.00:StatementOfDistribution)
        
                    Technical Business Rule Format:
                    ^PTR146 <> BLANK AND ^PTR149 <> BLANK
            
                    Data Elements:
            
                    RP:^PTR149 = pyde.02.00:OrganisationNameDetails.OrganisationalName.Text WHERE [OrganisationNameDetails.OrganisationalNameType.Code='MN'].[OrganisationNameDetails.Currency.Code='C']
            
                    RP:^PTR146 = pyde.02.00:PersonNameDetails.FamilyName.Text WHERE [PersonNameDetails.PersonNameType.Code='LGL'].[PersonNameDetails.Usage.Code='Contact']
                    */
                    assertion = (string.IsNullOrWhiteSpace(statementOfDistribution.PTR146) != true && string.IsNullOrWhiteSpace(statementOfDistribution.PTR149) != true);
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.GEN.430139",
                            Severity = ProcessMessageSeverity.Error,
                            Description = @"Both family name and company name present",
                            LongDescription = @"Family name and non-individual name line 1 are both present in the distribution statement (Item 65). Please delete the incorrect name as only one individual or non-individual name may be present.",
                            Location = "/xbrli:xbrl/ptr.0006.lodge.req.02.00:StatementOfDistribution" + OccurrenceIndex(statementOfDistribution.OccurrenceIndex) + "/orgname2.02.00:OrganisationNameDetails" + OccurrenceIndex(statementOfDistribution.StatementOfDistribution_OrganisationNameDetailsCollection_MN_C_RPOccurrenceIndex) + "/pyde.02.00:OrganisationNameDetails.OrganisationalName.Text",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.PTR.430139" } },
                        };
                        processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "PTR146", Value = statementOfDistribution.PTR146 });

                        processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "PTR149", Value = statementOfDistribution.PTR149 });

                        response.Add(processMessage);
                    }
                    #endregion // VR.ATO.PTR.430139

                    #region VR.ATO.PTR.430141

                    /*  VR.ATO.PTR.430141
                    Statement of distribution -  Individual's  residential address  or Non-Individual's business address or Tax File Number or Australian Business Number must be present
    
                    Legacy Rule Format:
                    WHERE IN TUPLE(ptr.0006.lodge.req.xx.xx:StatementOfDistribution)
                    IF [PTR332] = NULLORBLANK AND [PTR157] = NULL AND [PTR151] = NULLORBLANK AND [PTR285] = NULL
                        RETURN VALIDATION MESSAGE
                    ENDIF
                    
                    [PTR151] = PTR:RP:pyde.02.00:AddressDetails.Line1.Text WHERE (pyde.02.01:AddressDetails.Usage.Code = "BUS" AND pyde.02.00:AddressDetails.Currency.Code = "C") IN TUPLE(address2.02.02:AddressDetails) IN TUPLE(ptr.0006.lodge.req.02.00:StatementOfDistribution)
                    [PTR157] = PTR:RP:pyid.02.00:Identifiers.TaxFileNumber.Identifier IN TUPLE(ptr.0006.lodge.req.02.00:StatementOfDistribution)
                    [PTR285] = PTR:RP:pyid.02.00:Identifiers.AustralianBusinessNumber.Identifier IN TUPLE(ptr.0006.lodge.req.02.00:StatementOfDistribution)
                    [PTR332] = PTR:RP:pyde.02.00:AddressDetails.Line1.Text WHERE (pyde.02.01:AddressDetails.Usage.Code = "RES" AND pyde.02.00:AddressDetails.Currency.Code = "C") IN TUPLE(address2.02.02:AddressDetails) IN TUPLE(ptr.0006.lodge.req.02.00:StatementOfDistribution)
        
                    Technical Business Rule Format:
                    ^PTR332 = BLANK AND ^PTR157 = NULL AND ^PTR151 = BLANK AND ^PTR285 = NULL
            
                    Data Elements:
            
                    RP:^PTR157 = pyid.02.00:Identifiers.TaxFileNumber.Identifier
            
                    RP:^PTR151 = pyde.02.00:AddressDetails.Line1.Text WHERE [AddressDetails.Usage.Code='BUS'].[AddressDetails.Currency.Code='C']
            
                    RP:^PTR285 = pyid.02.00:Identifiers.AustralianBusinessNumber.Identifier
            
                    RP:^PTR332 = pyde.02.00:AddressDetails.Line1.Text WHERE [AddressDetails.Usage.Code='RES'].[AddressDetails.Currency.Code='C']
                    */
                    assertion = (string.IsNullOrWhiteSpace(statementOfDistribution.PTR332) == true && statementOfDistribution.PTR157 == null && string.IsNullOrWhiteSpace(statementOfDistribution.PTR151) == true && statementOfDistribution.PTR285 == null);
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.PTR.431053",
                            Severity = ProcessMessageSeverity.Error,
                            Description = @"Individual's  residential address  or Non-Individual's business address or Tax File Number or Australian Business Number  must be present in each Statement of Distribution",
                            Location = "/xbrli:xbrl/ptr.0006.lodge.req.02.00:StatementOfDistribution" + OccurrenceIndex(statementOfDistribution.OccurrenceIndex) + "/pyid.02.00:Identifiers.TaxFileNumber.Identifier",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.PTR.430141" } },
                        };
                        processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "PTR332", Value = statementOfDistribution.PTR332 });

                        processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "PTR157", Value = statementOfDistribution.PTR157 });

                        processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "PTR151", Value = statementOfDistribution.PTR151 });

                        processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "PTR285", Value = statementOfDistribution.PTR285 });

                        response.Add(processMessage);
                    }
                    #endregion // VR.ATO.PTR.430141

                    #region VR.ATO.PTR.430143

                    /*  VR.ATO.PTR.430143
                    PP, Non-PP or Franked distribution required.
    
                    Legacy Rule Format:
                    WHERE IN TUPLE(ptr.0006.lodge.req.xx.xx:StatementOfDistribution)
                    IF [PTR160] > 0 AND [PTR158] = NULL AND [PTR159] = NULL AND [PTR324] = NULL 
                        RETURN VALIDATION MESSAGE
                    ENDIF
                    
                    [PTR158] = PTR:RP.Prim:bafpr1.02.04:Income.BeneficiaryShare.Amount IN TUPLE(ptr.0006.lodge.req.02.00:StatementOfDistribution)
                    [PTR159] = PTR:RP.NonPrim:bafpr1.02.04:Income.BeneficiaryShare.Amount IN TUPLE(ptr.0006.lodge.req.02.00:StatementOfDistribution)
                    [PTR160] = PTR:RP:rvctc2.02.00:IncomeTax.PayAsYouGoWithholding.CreditForTaxWithheldWhereABNNotQuoted.Amount IN TUPLE(ptr.0006.lodge.req.02.00:StatementOfDistribution)
                    [PTR324] = PTR:RP:bafpr1.02.09:Income.DistributionFranked.Amount IN TUPLE(ptr.0006.lodge.req.02.00:StatementOfDistribution)
        
                    Technical Business Rule Format:
                    ^PTR160 > 0 AND ^PTR158 = NULL AND ^PTR159 = NULL AND ^PTR324 = NULL
            
                    Data Elements:
            
                    RP.Prim:^PTR158 = bafpr1.02.04:Income.BeneficiaryShare.Amount
            
                    RP.NonPrim:^PTR159 = bafpr1.02.04:Income.BeneficiaryShare.Amount
            
                    RP:^PTR160 = rvctc2.02.00:IncomeTax.PayAsYouGoWithholding.CreditForTaxWithheldWhereABNNotQuoted.Amount
            
                    RP:^PTR324 = bafpr1.02.09:Income.DistributionFranked.Amount
                    */
                    assertion = (statementOfDistribution.PTR160.GetValueOrDefault() > 0 && statementOfDistribution.PTR158 == null && statementOfDistribution.PTR159 == null && statementOfDistribution.PTR324 == null);
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.PTR.430707",
                            Severity = ProcessMessageSeverity.Error,
                            Description = @"PP, Non-PP or Franked distributions required",
                            LongDescription = @"Credit for tax withheld where ABN not quoted is greater than zero and there is no Primary production or Non-primary production income distribution, or Franked Distributions",
                            Location = "/xbrli:xbrl/ptr.0006.lodge.req.02.00:StatementOfDistribution" + OccurrenceIndex(statementOfDistribution.OccurrenceIndex) + "/bafpr1.02.04:Income.BeneficiaryShare.Amount[@contextRef='" + report.RPPrimId + "']",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.PTR.430143" } },
                        };
                        processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "PTR160", Value = statementOfDistribution.PTR160.GetValueOrDefault().ToString() });

                        processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "PTR158", Value = statementOfDistribution.PTR158.GetValueOrDefault().ToString() });

                        processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "PTR159", Value = statementOfDistribution.PTR159.GetValueOrDefault().ToString() });

                        processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "PTR324", Value = statementOfDistribution.PTR324.GetValueOrDefault().ToString() });

                        response.Add(processMessage);
                    }
                    #endregion // VR.ATO.PTR.430143

                    #region VR.ATO.PTR.430146

                    /*  VR.ATO.PTR.430146
                    Partner distribution amount(s) missing
    
                    Legacy Rule Format:
                    WHERE IN TUPLE(ptr.0006.lodge.req.xx.xx:StatementOfDistribution)
                    IF(
                    ([PTR146] <> NULLORBLANK) OR 
                    ([PTR149] <> NULLORBLANK) OR 
                    ([PTR151] <> NULLORBLANK) OR 
                    ([PTR157] <> NULLORBLANK) OR 
                    ([PTR332] <> NULLORBLANK)
                    ) AND
                    ([PTR158] = NULL OR [PTR158] = 0) AND 
                    ([PTR159] = NULL OR [PTR159] = 0) AND 
                    ([PTR324] = NULL OR [PTR324] = 0) AND 
                    ([PTR161] = NULL OR [PTR161] = 0) AND 
                    ([PTR162] = NULL OR [PTR162] = 0) AND 
                    ([PTR164] = NULL OR [PTR164] = 0) 
                        RETURN VALIDATION MESSAGE
                    ENDIF
                    
                    
                    [PTR146] = PTR:RP:pyde.02.00:PersonNameDetails.FamilyName.Text WHERE (pyde.02.00:PersonNameDetails.PersonNameType.Code = "LGL" AND pyde.02.05:PersonNameDetails.Usage.Code = "Contact") IN TUPLE(prsnstrcnm3.02.01:PersonNameDetails) IN TUPLE(ptr.0006.lodge.req.02.00:StatementOfDistribution)
                    [PTR149] = PTR:RP:pyde.02.00:OrganisationNameDetails.OrganisationalName.Text WHERE (pyde.02.00:OrganisationNameDetails.OrganisationalNameType.Code = "MN" AND pyde.02.00:OrganisationNameDetails.Currency.Code = "C") IN TUPLE(orgname2.02.00:OrganisationNameDetails) IN TUPLE(ptr.0006.lodge.req.02.00:StatementOfDistribution)
                    [PTR151] = PTR:RP:pyde.02.00:AddressDetails.Line1.Text WHERE (pyde.02.01:AddressDetails.Usage.Code = "BUS" AND pyde.02.00:AddressDetails.Currency.Code = "C") IN TUPLE(address2.02.02:AddressDetails) IN TUPLE(ptr.0006.lodge.req.02.00:StatementOfDistribution)
                    [PTR157] = PTR:RP:pyid.02.00:Identifiers.TaxFileNumber.Identifier IN TUPLE(ptr.0006.lodge.req.02.00:StatementOfDistribution)
                    [PTR158] = PTR:RP.Prim:bafpr1.02.04:Income.BeneficiaryShare.Amount IN TUPLE(ptr.0006.lodge.req.02.00:StatementOfDistribution)
                    [PTR159] = PTR:RP.NonPrim:bafpr1.02.04:Income.BeneficiaryShare.Amount IN TUPLE(ptr.0006.lodge.req.02.00:StatementOfDistribution)
                    [PTR161] = PTR:RP:rvctc2.02.00:IncomeTax.FrankingCredits.ReceivedFromAustralianCompanies.Amount IN TUPLE(ptr.0006.lodge.req.02.00:StatementOfDistribution)
                    [PTR162] = PTR:RP:rvctc2.02.04:IncomeTax.PayAsYouGoWithholding.CreditForAmountsWithheldWhereTFNNotQuoted.Amount IN TUPLE(ptr.0006.lodge.req.02.00:StatementOfDistribution)
                    [PTR164] = PTR:RP:rvctc2.02.00:IncomeTax.FrankingCredits.ReceivedFromNewZealandCompanies.Amount IN TUPLE(ptr.0006.lodge.req.02.00:StatementOfDistribution)
                    [PTR324] = PTR:RP:bafpr1.02.09:Income.DistributionFranked.Amount IN TUPLE(ptr.0006.lodge.req.02.00:StatementOfDistribution)
                    [PTR332] = PTR:RP:pyde.02.00:AddressDetails.Line1.Text WHERE (pyde.02.01:AddressDetails.Usage.Code = "RES" AND pyde.02.00:AddressDetails.Currency.Code = "C") IN TUPLE(address2.02.02:AddressDetails) IN TUPLE(ptr.0006.lodge.req.02.00:StatementOfDistribution)
        
                    Technical Business Rule Format:
                    ((^PTR146 <> BLANK) OR (^PTR149 <> BLANK) OR (^PTR151 <> BLANK) OR (^PTR157 <> BLANK) OR (^PTR332 <> BLANK)) AND (^PTR158 = NULL OR ^PTR158 = 0) AND (^PTR159 = NULL OR ^PTR159 = 0) AND (^PTR324 = NULL OR ^PTR324 = 0) AND (^PTR161 = NULL OR ^PTR161 = 0) AND (^PTR162 = NULL OR ^PTR162 = 0) AND (^PTR164 = NULL OR ^PTR164 = 0)
            
                    Data Elements:
            
                    RP.Prim:^PTR158 = bafpr1.02.04:Income.BeneficiaryShare.Amount
            
                    RP:^PTR146 = pyde.02.00:PersonNameDetails.FamilyName.Text WHERE [PersonNameDetails.PersonNameType.Code='LGL'].[PersonNameDetails.Usage.Code='Contact']
            
                    RP:^PTR149 = pyde.02.00:OrganisationNameDetails.OrganisationalName.Text WHERE [OrganisationNameDetails.OrganisationalNameType.Code='MN'].[OrganisationNameDetails.Currency.Code='C']
            
                    RP:^PTR151 = pyde.02.00:AddressDetails.Line1.Text WHERE [AddressDetails.Usage.Code='BUS'].[AddressDetails.Currency.Code='C']
            
                    RP:^PTR157 = pyid.02.00:Identifiers.TaxFileNumber.Identifier
            
                    RP.NonPrim:^PTR159 = bafpr1.02.04:Income.BeneficiaryShare.Amount
            
                    RP:^PTR161 = rvctc2.02.00:IncomeTax.FrankingCredits.ReceivedFromAustralianCompanies.Amount
            
                    RP:^PTR162 = rvctc2.02.04:IncomeTax.PayAsYouGoWithholding.CreditForAmountsWithheldWhereTFNNotQuoted.Amount
            
                    RP:^PTR164 = rvctc2.02.00:IncomeTax.FrankingCredits.ReceivedFromNewZealandCompanies.Amount
            
                    RP:^PTR324 = bafpr1.02.09:Income.DistributionFranked.Amount
            
                    RP:^PTR332 = pyde.02.00:AddressDetails.Line1.Text WHERE [AddressDetails.Usage.Code='RES'].[AddressDetails.Currency.Code='C']
                    */
                    assertion = ((string.IsNullOrWhiteSpace(statementOfDistribution.PTR146) != true || string.IsNullOrWhiteSpace(statementOfDistribution.PTR149) != true || string.IsNullOrWhiteSpace(statementOfDistribution.PTR151) != true || string.IsNullOrWhiteSpace(statementOfDistribution.PTR157) != true || string.IsNullOrWhiteSpace(statementOfDistribution.PTR332) != true) && (statementOfDistribution.PTR158 == null || statementOfDistribution.PTR158 == 0) && (statementOfDistribution.PTR159 == null || statementOfDistribution.PTR159 == 0) && (statementOfDistribution.PTR324 == null || statementOfDistribution.PTR324 == 0) && (statementOfDistribution.PTR161 == null || statementOfDistribution.PTR161 == 0) && (statementOfDistribution.PTR162 == null || statementOfDistribution.PTR162 == 0) && (statementOfDistribution.PTR164 == null || statementOfDistribution.PTR164 == 0));
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.PTR.431047",
                            Severity = ProcessMessageSeverity.Error,
                            Description = @"For Partner distribution amount(s) missing or have a value of zero (0)",
                            LongDescription = @"There must be an amount value greater than zero (0)  for the fields - Share of income Primary Production, Share of income Non-primary production, Franked Distributions, Franking credit, TFN amounts withheld or Australian franking credits from a New Zealand company, for each partner listed in the statement of distribution. For remaining fields where the amount is nil, zero (0) is to be inserted at the relevant label.",
                            Location = "/xbrli:xbrl/ptr.0006.lodge.req.02.00:StatementOfDistribution" + OccurrenceIndex(statementOfDistribution.OccurrenceIndex) + "/bafpr1.02.04:Income.BeneficiaryShare.Amount[@contextRef='" + report.RPPrimId + "']",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.PTR.430146" } },
                        };
                        processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "PTR146", Value = statementOfDistribution.PTR146 });

                        processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "PTR149", Value = statementOfDistribution.PTR149 });

                        processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "PTR151", Value = statementOfDistribution.PTR151 });

                        processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "PTR157", Value = statementOfDistribution.PTR157 });

                        processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "PTR332", Value = statementOfDistribution.PTR332 });

                        processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "PTR158", Value = statementOfDistribution.PTR158.GetValueOrDefault().ToString() });

                        processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "PTR159", Value = statementOfDistribution.PTR159.GetValueOrDefault().ToString() });

                        processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "PTR324", Value = statementOfDistribution.PTR324.GetValueOrDefault().ToString() });

                        processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "PTR161", Value = statementOfDistribution.PTR161.GetValueOrDefault().ToString() });

                        processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "PTR162", Value = statementOfDistribution.PTR162.GetValueOrDefault().ToString() });

                        processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "PTR164", Value = statementOfDistribution.PTR164.GetValueOrDefault().ToString() });

                        response.Add(processMessage);
                    }
                    #endregion // VR.ATO.PTR.430146

                    #region VR.ATO.PTR.430147

                    /*  VR.ATO.PTR.430147
                    Monetary field may be signed, maximum of 11 chars, no decimals
    
                    Legacy Rule Format:
                    IF [PTR158] <> NULL AND [PTR158] <> MONETARY(S,11,0)
                       RETURN VALIDATION MESSAGE
                    ENDIF
                    
                    [PTR158] = PTR:RP.Prim:bafpr1.02.04:Income.BeneficiaryShare.Amount IN TUPLE(ptr.0006.lodge.req.02.00:StatementOfDistribution)
        
                    Technical Business Rule Format:
                    NotMonetary(^PTR158, 'S', 11, 0)
            
                    Data Elements:
            
                    RP.Prim:^PTR158 = bafpr1.02.04:Income.BeneficiaryShare.Amount
                    */
                    assertion = NotMonetary(statementOfDistribution.PTR158, @"S", 11, 0);
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.GEN.400011",
                            Severity = ProcessMessageSeverity.Error,
                            Description = @"Field incorrect format. Exceeds maximum digits or decimals and/or contains illegal characters",
                            LongDescription = @"Field is in incorrect format for a monetary field. Field either exceeds maximum digits or decimals and/or contains illegal characters and/or for a signed field does not have the sign as the first character",
                            Location = "/xbrli:xbrl/ptr.0006.lodge.req.02.00:StatementOfDistribution" + OccurrenceIndex(statementOfDistribution.OccurrenceIndex) + "/bafpr1.02.04:Income.BeneficiaryShare.Amount[@contextRef='" + report.RPPrimId + "']",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.PTR.430147" } },
                        };
                        processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "PTR158", Value = statementOfDistribution.PTR158.GetValueOrDefault().ToString() });

                        response.Add(processMessage);
                    }
                    #endregion // VR.ATO.PTR.430147

                    #region VR.ATO.PTR.430148

                    /*  VR.ATO.PTR.430148
                    Share of income - Non-PP or Franked distributions must be present
    
                    Legacy Rule Format:
                    WHERE IN TUPLE(ptr.0006.lodge.req.xx.xx:StatementOfDistribution)
                    IF ([PTR160] <> NULL OR [PTR161] <> NULL OR [PTR162] <> NULL OR [PTR351] <> NULL) AND [PTR159] = NULL AND [PTR324] = NULL
                       RETURN VALIDATION MESSAGE
                    ENDIF
                    
                    [PTR159] = PTR:RP.NonPrim:bafpr1.02.04:Income.BeneficiaryShare.Amount IN TUPLE(ptr.0006.lodge.req.02.00:StatementOfDistribution)
                    [PTR160] = PTR:RP:rvctc2.02.00:IncomeTax.PayAsYouGoWithholding.CreditForTaxWithheldWhereABNNotQuoted.Amount IN TUPLE(ptr.0006.lodge.req.02.00:StatementOfDistribution)
                    [PTR161] = PTR:RP:rvctc2.02.00:IncomeTax.FrankingCredits.ReceivedFromAustralianCompanies.Amount IN TUPLE(ptr.0006.lodge.req.02.00:StatementOfDistribution)
                    [PTR162] = PTR:RP:rvctc2.02.04:IncomeTax.PayAsYouGoWithholding.CreditForAmountsWithheldWhereTFNNotQuoted.Amount IN TUPLE(ptr.0006.lodge.req.02.00:StatementOfDistribution)
                    [PTR324] = PTR:RP:bafpr1.02.09:Income.DistributionFranked.Amount IN TUPLE(ptr.0006.lodge.req.02.00:StatementOfDistribution)
                    [PTR351] = PTR:RP:rvctc2.02.13:IncomeTax.PayAsYouGoWithholding.CreditForAmountsWithheldFromForeignResidents.Amount IN TUPLE(ptr.0006.lodge.req.02.00:StatementOfDistribution)
        
                    Technical Business Rule Format:
                    (^PTR160 <> NULL OR ^PTR161 <> NULL OR ^PTR162 <> NULL OR ^PTR351 <> NULL) AND ^PTR159 = NULL AND ^PTR324 = NULL
            
                    Data Elements:
            
                    RP.NonPrim:^PTR159 = bafpr1.02.04:Income.BeneficiaryShare.Amount
            
                    RP:^PTR160 = rvctc2.02.00:IncomeTax.PayAsYouGoWithholding.CreditForTaxWithheldWhereABNNotQuoted.Amount
            
                    RP:^PTR161 = rvctc2.02.00:IncomeTax.FrankingCredits.ReceivedFromAustralianCompanies.Amount
            
                    RP:^PTR162 = rvctc2.02.04:IncomeTax.PayAsYouGoWithholding.CreditForAmountsWithheldWhereTFNNotQuoted.Amount
            
                    RP:^PTR324 = bafpr1.02.09:Income.DistributionFranked.Amount
            
                    RP:^PTR351 = rvctc2.02.13:IncomeTax.PayAsYouGoWithholding.CreditForAmountsWithheldFromForeignResidents.Amount
                    */
                    assertion = ((statementOfDistribution.PTR160 != null || statementOfDistribution.PTR161 != null || statementOfDistribution.PTR162 != null || statementOfDistribution.PTR351 != null) && statementOfDistribution.PTR159 == null && statementOfDistribution.PTR324 == null);
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.PTR.430710",
                            Severity = ProcessMessageSeverity.Error,
                            Description = @"Share of income - Non-PP income distribution or Franked distributions must be present",
                            LongDescription = @"In the statement of distribution, if there is an amount at either Credit for tax withheld where ABN not quoted, Franking Credit, TFN amounts withheld, or Credit for tax withheld - foreign resident withholding, then there must be an amount at either Share of income - Non-primary production or Franked distributions",
                            Location = "/xbrli:xbrl/ptr.0006.lodge.req.02.00:StatementOfDistribution" + OccurrenceIndex(statementOfDistribution.OccurrenceIndex) + "/bafpr1.02.04:Income.BeneficiaryShare.Amount[@contextRef='" + report.RPNonPrimId + "']",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.PTR.430148" } },
                        };
                        processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "PTR160", Value = statementOfDistribution.PTR160.GetValueOrDefault().ToString() });

                        processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "PTR161", Value = statementOfDistribution.PTR161.GetValueOrDefault().ToString() });

                        processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "PTR162", Value = statementOfDistribution.PTR162.GetValueOrDefault().ToString() });

                        processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "PTR351", Value = statementOfDistribution.PTR351.GetValueOrDefault().ToString() });

                        processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "PTR159", Value = statementOfDistribution.PTR159.GetValueOrDefault().ToString() });

                        processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "PTR324", Value = statementOfDistribution.PTR324.GetValueOrDefault().ToString() });

                        response.Add(processMessage);
                    }
                    #endregion // VR.ATO.PTR.430148

                    #region VR.ATO.PTR.430149

                    /*  VR.ATO.PTR.430149
                    Share of income must be present
    
                    Legacy Rule Format:
                    WHERE IN TUPLE(ptr.0006.lodge.req.xx.xx:StatementOfDistribution)
                    IF [PTR351] > 0 AND [PTR158] = NULL AND [PTR159] = NULL AND [PTR324] = NULL
                       RETURN VALIDATION MESSAGE
                    ENDIF
                    
                    [PTR158] = PTR:RP.Prim:bafpr1.02.04:Income.BeneficiaryShare.Amount IN TUPLE(ptr.0006.lodge.req.02.00:StatementOfDistribution)
                    [PTR159] = PTR:RP.NonPrim:bafpr1.02.04:Income.BeneficiaryShare.Amount IN TUPLE(ptr.0006.lodge.req.02.00:StatementOfDistribution)
                    [PTR324] = PTR:RP:bafpr1.02.09:Income.DistributionFranked.Amount IN TUPLE(ptr.0006.lodge.req.02.00:StatementOfDistribution)
                    [PTR351] = PTR:RP:rvctc2.02.13:IncomeTax.PayAsYouGoWithholding.CreditForAmountsWithheldFromForeignResidents.Amount IN TUPLE(ptr.0006.lodge.req.02.00:StatementOfDistribution)
        
                    Technical Business Rule Format:
                    ^PTR351 > 0 AND ^PTR158 = NULL AND ^PTR159 = NULL AND ^PTR324 = NULL
            
                    Data Elements:
            
                    RP.Prim:^PTR158 = bafpr1.02.04:Income.BeneficiaryShare.Amount
            
                    RP.NonPrim:^PTR159 = bafpr1.02.04:Income.BeneficiaryShare.Amount
            
                    RP:^PTR324 = bafpr1.02.09:Income.DistributionFranked.Amount
            
                    RP:^PTR351 = rvctc2.02.13:IncomeTax.PayAsYouGoWithholding.CreditForAmountsWithheldFromForeignResidents.Amount
                    */
                    assertion = (statementOfDistribution.PTR351.GetValueOrDefault() > 0 && statementOfDistribution.PTR158 == null && statementOfDistribution.PTR159 == null && statementOfDistribution.PTR324 == null);
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.PTR.430709",
                            Severity = ProcessMessageSeverity.Error,
                            Description = @"Share of income must be present",
                            LongDescription = @"In a Statement of distribution, if Credit for tax withheld - foreign resident withholding is greater than zero, then there must be an amount at Share of income - primary production or Share of income - non-primary production or Franked distibutions",
                            Location = "/xbrli:xbrl/ptr.0006.lodge.req.02.00:StatementOfDistribution" + OccurrenceIndex(statementOfDistribution.OccurrenceIndex) + "/bafpr1.02.04:Income.BeneficiaryShare.Amount[@contextRef='" + report.RPPrimId + "']",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.PTR.430149" } },
                        };
                        processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "PTR351", Value = statementOfDistribution.PTR351.GetValueOrDefault().ToString() });

                        processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "PTR158", Value = statementOfDistribution.PTR158.GetValueOrDefault().ToString() });

                        processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "PTR159", Value = statementOfDistribution.PTR159.GetValueOrDefault().ToString() });

                        processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "PTR324", Value = statementOfDistribution.PTR324.GetValueOrDefault().ToString() });

                        response.Add(processMessage);
                    }
                    #endregion // VR.ATO.PTR.430149

                    #region VR.ATO.PTR.430150

                    /*  VR.ATO.PTR.430150
                    Monetary field may be signed, maximum of 11 chars, no decimals
    
                    Legacy Rule Format:
                    IF [PTR159] <> NULL AND [PTR159] <> MONETARY(S,11,0)
                       RETURN VALIDATION MESSAGE
                    ENDIF
                    
                    [PTR159] = PTR:RP.NonPrim:bafpr1.02.04:Income.BeneficiaryShare.Amount IN TUPLE(ptr.0006.lodge.req.02.00:StatementOfDistribution)
        
                    Technical Business Rule Format:
                    NotMonetary(^PTR159, 'S', 11, 0)
            
                    Data Elements:
            
                    RP.NonPrim:^PTR159 = bafpr1.02.04:Income.BeneficiaryShare.Amount
                    */
                    assertion = NotMonetary(statementOfDistribution.PTR159, @"S", 11, 0);
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.GEN.400011",
                            Severity = ProcessMessageSeverity.Error,
                            Description = @"Field incorrect format. Exceeds maximum digits or decimals and/or contains illegal characters",
                            LongDescription = @"Field is in incorrect format for a monetary field. Field either exceeds maximum digits or decimals and/or contains illegal characters and/or for a signed field does not have the sign as the first character",
                            Location = "/xbrli:xbrl/ptr.0006.lodge.req.02.00:StatementOfDistribution" + OccurrenceIndex(statementOfDistribution.OccurrenceIndex) + "/bafpr1.02.04:Income.BeneficiaryShare.Amount[@contextRef='" + report.RPNonPrimId + "']",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.PTR.430150" } },
                        };
                        processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "PTR159", Value = statementOfDistribution.PTR159.GetValueOrDefault().ToString() });

                        response.Add(processMessage);
                    }
                    #endregion // VR.ATO.PTR.430150

                    #region VR.ATO.PTR.430152

                    /*  VR.ATO.PTR.430152
                    Monetary field must be unsigned, maximum of 11 numeric characters, no decimals
    
                    Legacy Rule Format:
                    IF [PTR160] <> NULL AND [PTR160] <> MONETARY(U,11,0)
                       RETURN VALIDATION MESSAGE
                    ENDIF
                    
                    [PTR160] = PTR:RP:rvctc2.02.00:IncomeTax.PayAsYouGoWithholding.CreditForTaxWithheldWhereABNNotQuoted.Amount IN TUPLE(ptr.0006.lodge.req.02.00:StatementOfDistribution)
        
                    Technical Business Rule Format:
                    NotMonetary(^PTR160, 'U', 11, 0)
            
                    Data Elements:
            
                    RP:^PTR160 = rvctc2.02.00:IncomeTax.PayAsYouGoWithholding.CreditForTaxWithheldWhereABNNotQuoted.Amount
                    */
                    assertion = NotMonetary(statementOfDistribution.PTR160, @"U", 11, 0);
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.GEN.400011",
                            Severity = ProcessMessageSeverity.Error,
                            Description = @"Field incorrect format. Exceeds maximum digits or decimals and/or contains illegal characters",
                            LongDescription = @"Field is in incorrect format for a monetary field. Field either exceeds maximum digits or decimals and/or contains illegal characters and/or for a signed field does not have the sign as the first character",
                            Location = "/xbrli:xbrl/ptr.0006.lodge.req.02.00:StatementOfDistribution" + OccurrenceIndex(statementOfDistribution.OccurrenceIndex) + "/rvctc2.02.00:IncomeTax.PayAsYouGoWithholding.CreditForTaxWithheldWhereABNNotQuoted.Amount",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.PTR.430152" } },
                        };
                        processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "PTR160", Value = statementOfDistribution.PTR160.GetValueOrDefault().ToString() });

                        response.Add(processMessage);
                    }
                    #endregion // VR.ATO.PTR.430152

                    #region VR.ATO.PTR.430154

                    /*  VR.ATO.PTR.430154
                    Monetary field - unsigned, maximum of 13 characters, may include a decimal point and up to 2 decimals
    
                    Legacy Rule Format:
                    IF [PTR161] <> NULL AND [PTR161] <> MONETARY(U,13,2)
                       RETURN VALIDATION MESSAGE
                    ENDIF
                    
                    [PTR161] = PTR:RP:rvctc2.02.00:IncomeTax.FrankingCredits.ReceivedFromAustralianCompanies.Amount IN TUPLE(ptr.0006.lodge.req.02.00:StatementOfDistribution)
        
                    Technical Business Rule Format:
                    NotMonetary(^PTR161, 'U', 13, 2)
            
                    Data Elements:
            
                    RP:^PTR161 = rvctc2.02.00:IncomeTax.FrankingCredits.ReceivedFromAustralianCompanies.Amount
                    */
                    assertion = NotMonetary(statementOfDistribution.PTR161, @"U", 13, 2);
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.GEN.400011",
                            Severity = ProcessMessageSeverity.Error,
                            Description = @"Field incorrect format. Exceeds maximum digits or decimals and/or contains illegal characters",
                            LongDescription = @"Field is in incorrect format for a monetary field. Field either exceeds maximum digits or decimals and/or contains illegal characters and/or for a signed field does not have the sign as the first character",
                            Location = "/xbrli:xbrl/ptr.0006.lodge.req.02.00:StatementOfDistribution" + OccurrenceIndex(statementOfDistribution.OccurrenceIndex) + "/rvctc2.02.00:IncomeTax.FrankingCredits.ReceivedFromAustralianCompanies.Amount",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.PTR.430154" } },
                        };
                        processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "PTR161", Value = statementOfDistribution.PTR161.GetValueOrDefault().ToString() });

                        response.Add(processMessage);
                    }
                    #endregion // VR.ATO.PTR.430154

                    #region VR.ATO.PTR.430157

                    /*  VR.ATO.PTR.430157
                    Monetary field - unsigned, maximum of 13 characters, may include a decimal point and up to 2 decimals
    
                    Legacy Rule Format:
                    IF [PTR162] <> NULL AND [PTR162] <> MONETARY(U,13,2)
                       RETURN VALIDATION MESSAGE
                    ENDIF
                    
                    [PTR162] = PTR:RP:rvctc2.02.04:IncomeTax.PayAsYouGoWithholding.CreditForAmountsWithheldWhereTFNNotQuoted.Amount IN TUPLE(ptr.0006.lodge.req.02.00:StatementOfDistribution)
        
                    Technical Business Rule Format:
                    NotMonetary(^PTR162, 'U', 13, 2)
            
                    Data Elements:
            
                    RP:^PTR162 = rvctc2.02.04:IncomeTax.PayAsYouGoWithholding.CreditForAmountsWithheldWhereTFNNotQuoted.Amount
                    */
                    assertion = NotMonetary(statementOfDistribution.PTR162, @"U", 13, 2);
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.GEN.400011",
                            Severity = ProcessMessageSeverity.Error,
                            Description = @"Field incorrect format. Exceeds maximum digits or decimals and/or contains illegal characters",
                            LongDescription = @"Field is in incorrect format for a monetary field. Field either exceeds maximum digits or decimals and/or contains illegal characters and/or for a signed field does not have the sign as the first character",
                            Location = "/xbrli:xbrl/ptr.0006.lodge.req.02.00:StatementOfDistribution" + OccurrenceIndex(statementOfDistribution.OccurrenceIndex) + "/rvctc2.02.04:IncomeTax.PayAsYouGoWithholding.CreditForAmountsWithheldWhereTFNNotQuoted.Amount",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.PTR.430157" } },
                        };
                        processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "PTR162", Value = statementOfDistribution.PTR162.GetValueOrDefault().ToString() });

                        response.Add(processMessage);
                    }
                    #endregion // VR.ATO.PTR.430157

                    #region VR.ATO.PTR.430161

                    /*  VR.ATO.PTR.430161
                    Monetary field must be unsigned, maximum of 11 numeric characters, no decimals
    
                    Legacy Rule Format:
                    IF [PTR164] <> NULL AND [PTR164] <> MONETARY(U,11,0)
                       RETURN VALIDATION MESSAGE
                    ENDIF
                    
                    [PTR164] = PTR:RP:rvctc2.02.00:IncomeTax.FrankingCredits.ReceivedFromNewZealandCompanies.Amount IN TUPLE(ptr.0006.lodge.req.02.00:StatementOfDistribution)
        
                    Technical Business Rule Format:
                    NotMonetary(^PTR164, 'U', 11, 0)
            
                    Data Elements:
            
                    RP:^PTR164 = rvctc2.02.00:IncomeTax.FrankingCredits.ReceivedFromNewZealandCompanies.Amount
                    */
                    assertion = NotMonetary(statementOfDistribution.PTR164, @"U", 11, 0);
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.GEN.400011",
                            Severity = ProcessMessageSeverity.Error,
                            Description = @"Field incorrect format. Exceeds maximum digits or decimals and/or contains illegal characters",
                            LongDescription = @"Field is in incorrect format for a monetary field. Field either exceeds maximum digits or decimals and/or contains illegal characters and/or for a signed field does not have the sign as the first character",
                            Location = "/xbrli:xbrl/ptr.0006.lodge.req.02.00:StatementOfDistribution" + OccurrenceIndex(statementOfDistribution.OccurrenceIndex) + "/rvctc2.02.00:IncomeTax.FrankingCredits.ReceivedFromNewZealandCompanies.Amount",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.PTR.430161" } },
                        };
                        processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "PTR164", Value = statementOfDistribution.PTR164.GetValueOrDefault().ToString() });

                        response.Add(processMessage);
                    }
                    #endregion // VR.ATO.PTR.430161

                    #region VR.ATO.PTR.430167

                    /*  VR.ATO.PTR.430167
                    Monetary field - unsigned, maximum of 13 characters, may include a decimal point and up to 2 decimals
    
                    Legacy Rule Format:
                    IF [PTR249] <> NULL AND [PTR249] <> MONETARY(U,13,2)
                       RETURN VALIDATION MESSAGE
                    ENDIF
                    
                    [PTR249] = PTR:RP:gfati.02.01:TaxOffsetClaim.NationalRentalAffordabilitySchemeEntitlement.Amount IN TUPLE(ptr.0006.lodge.req.02.00:StatementOfDistribution)
        
                    Technical Business Rule Format:
                    NotMonetary(^PTR249, 'U', 13, 2)
            
                    Data Elements:
            
                    RP:^PTR249 = gfati.02.01:TaxOffsetClaim.NationalRentalAffordabilitySchemeEntitlement.Amount
                    */
                    assertion = NotMonetary(statementOfDistribution.PTR249, @"U", 13, 2);
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.GEN.400011",
                            Severity = ProcessMessageSeverity.Error,
                            Description = @"Field incorrect format. Exceeds maximum digits or decimals and/or contains illegal characters",
                            LongDescription = @"Field is in incorrect format for a monetary field. Field either exceeds maximum digits or decimals and/or contains illegal characters and/or for a signed field does not have the sign as the first character",
                            Location = "/xbrli:xbrl/ptr.0006.lodge.req.02.00:StatementOfDistribution" + OccurrenceIndex(statementOfDistribution.OccurrenceIndex) + "/gfati.02.01:TaxOffsetClaim.NationalRentalAffordabilitySchemeEntitlement.Amount",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.PTR.430167" } },
                        };
                        processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "PTR249", Value = statementOfDistribution.PTR249.GetValueOrDefault().ToString() });

                        response.Add(processMessage);
                    }
                    #endregion // VR.ATO.PTR.430167

                    #region VR.ATO.PTR.430235

                    /*  VR.ATO.PTR.430235
                    Monetary field may be signed, maximum of 11 chars, no decimals
    
                    Legacy Rule Format:
                    IF [PTR256] <> NULL AND [PTR256] <> MONETARY(S,11,0)
                       RETURN VALIDATION MESSAGE
                    ENDIF
                    
                    [PTR256] = PTR:RP:bafpr2.02.04:Income.Investment.FinancialIncomeOrLossNet.Amount IN TUPLE(ptr.0006.lodge.req.02.00:StatementOfDistribution)
        
                    Technical Business Rule Format:
                    NotMonetary(^PTR256, 'S', 11, 0)
            
                    Data Elements:
            
                    RP:^PTR256 = bafpr2.02.04:Income.Investment.FinancialIncomeOrLossNet.Amount
                    */
                    assertion = NotMonetary(statementOfDistribution.PTR256, @"S", 11, 0);
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.GEN.400011",
                            Severity = ProcessMessageSeverity.Error,
                            Description = @"Field incorrect format. Exceeds maximum digits or decimals and/or contains illegal characters",
                            LongDescription = @"Field is in incorrect format for a monetary field. Field either exceeds maximum digits or decimals and/or contains illegal characters and/or for a signed field does not have the sign as the first character",
                            Location = "/xbrli:xbrl/ptr.0006.lodge.req.02.00:StatementOfDistribution" + OccurrenceIndex(statementOfDistribution.OccurrenceIndex) + "/bafpr2.02.04:Income.Investment.FinancialIncomeOrLossNet.Amount",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.PTR.430235" } },
                        };
                        processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "PTR256", Value = statementOfDistribution.PTR256.GetValueOrDefault().ToString() });

                        response.Add(processMessage);
                    }
                    #endregion // VR.ATO.PTR.430235

                    #region VR.ATO.PTR.430236

                    /*  VR.ATO.PTR.430236
                    Monetary field may be signed, maximum of 11 chars, no decimals
    
                    Legacy Rule Format:
                    IF [PTR257] <> NULL AND [PTR257] <> MONETARY(S,11,0)
                       RETURN VALIDATION MESSAGE
                    ENDIF
                    
                    [PTR257] = PTR:RP:bafpr1.02.04:Income.RentalIncomeOrLossNet.Amount IN TUPLE(ptr.0006.lodge.req.02.00:StatementOfDistribution)
        
                    Technical Business Rule Format:
                    NotMonetary(^PTR257, 'S', 11, 0)
            
                    Data Elements:
            
                    RP:^PTR257 = bafpr1.02.04:Income.RentalIncomeOrLossNet.Amount
                    */
                    assertion = NotMonetary(statementOfDistribution.PTR257, @"S", 11, 0);
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.GEN.400011",
                            Severity = ProcessMessageSeverity.Error,
                            Description = @"Field incorrect format. Exceeds maximum digits or decimals and/or contains illegal characters",
                            LongDescription = @"Field is in incorrect format for a monetary field. Field either exceeds maximum digits or decimals and/or contains illegal characters and/or for a signed field does not have the sign as the first character",
                            Location = "/xbrli:xbrl/ptr.0006.lodge.req.02.00:StatementOfDistribution" + OccurrenceIndex(statementOfDistribution.OccurrenceIndex) + "/bafpr1.02.04:Income.RentalIncomeOrLossNet.Amount",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.PTR.430236" } },
                        };
                        processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "PTR257", Value = statementOfDistribution.PTR257.GetValueOrDefault().ToString() });

                        response.Add(processMessage);
                    }
                    #endregion // VR.ATO.PTR.430236

                    #region VR.ATO.PTR.430280

                    /*  VR.ATO.PTR.430280
                    If the Partner Aged 18 or Over without real and  effective control of income share indicator is yes then person name details must be present
    
                    Legacy Rule Format:
                    WHERE IN TUPLE(ptr.0006.lodge.req.xx.xx:StatementOfDistribution)
                    IF (lrla.xx.xx:Remuneration.PartnerWithNoControlAndShareDisposalInPartnershipIncome.Indicator = TRUE) AND COUNT TUPLE (prsnstrcnm3.xx.xx:PersonNameDetails) <> 1
                       RETURN VALIDATION MESSAGE
                    ENDIF
        
                    Technical Business Rule Format:
                    (^PTR246 = TRUE) AND Count(^PersonNameDetails) <> 1
            
                    Data Elements:
            
                    RP:^PTR246 = lrla.02.00:Remuneration.PartnerWithNoControlAndShareDisposalInPartnershipIncome.Indicator
            
                    ^PersonNameDetails = prsnstrcnm3.02.01:PersonNameDetails
                    */
                    assertion = (statementOfDistribution.PTR246 == true && Count(statementOfDistribution.StatementOfDistribution_PersonNameDetailsCollection_LGL_Contact_RPCount) != 1);
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.PTR.430280",
                            Severity = ProcessMessageSeverity.Error,
                            Description = @"Individual name must be present",
                            LongDescription = @"If the Partner Aged 18 or Over without real and  effective control of income share indicator is set to true then person name details must be present",
                            Location = "/xbrli:xbrl/ptr.0006.lodge.req.02.00:StatementOfDistribution" + OccurrenceIndex(statementOfDistribution.OccurrenceIndex) + "/lrla.02.00:Remuneration.PartnerWithNoControlAndShareDisposalInPartnershipIncome.Indicator",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.PTR.430280" } },
                        };
                        processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "PTR246", Value = GetValueOrEmpty(statementOfDistribution.PTR246) });

                        response.Add(processMessage);
                    }
                    #endregion // VR.ATO.PTR.430280

                    #region VR.ATO.PTR.430313

                    /*  VR.ATO.PTR.430313
                    Monetary field - unsigned, maximum of 13 characters, may include a decimal point and up to 2 decimals
    
                    Legacy Rule Format:
                    IF [PTR321] <> NULL AND [PTR321] <> MONETARY(U,13,2)
                       RETURN VALIDATION MESSAGE
                    ENDIF
                    
                    [PTR321] = PTR:RP:rvctc2.02.03:IncomeTax.PayAsYouGoWithholding.CreditForTaxWithheldFromCloselyHeldTrust.Amount IN TUPLE(ptr.0006.lodge.req.02.00:StatementOfDistribution)
        
                    Technical Business Rule Format:
                    NotMonetary(^PTR321, 'U', 13, 2)
            
                    Data Elements:
            
                    RP:^PTR321 = rvctc2.02.03:IncomeTax.PayAsYouGoWithholding.CreditForTaxWithheldFromCloselyHeldTrust.Amount
                    */
                    assertion = NotMonetary(statementOfDistribution.PTR321, @"U", 13, 2);
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.GEN.400011",
                            Severity = ProcessMessageSeverity.Error,
                            Description = @"Field incorrect format. Exceeds maximum digits or decimals and/or contains illegal characters",
                            LongDescription = @"Field is in incorrect format for a monetary field. Field either exceeds maximum digits or decimals and/or contains illegal characters and/or for a signed field does not have the sign as the first character",
                            Location = "/xbrli:xbrl/ptr.0006.lodge.req.02.00:StatementOfDistribution" + OccurrenceIndex(statementOfDistribution.OccurrenceIndex) + "/rvctc2.02.03:IncomeTax.PayAsYouGoWithholding.CreditForTaxWithheldFromCloselyHeldTrust.Amount",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.PTR.430313" } },
                        };
                        processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "PTR321", Value = statementOfDistribution.PTR321.GetValueOrDefault().ToString() });

                        response.Add(processMessage);
                    }
                    #endregion // VR.ATO.PTR.430313

                    #region VR.ATO.PTR.430711

                    /*  VR.ATO.PTR.430711
                    Monetary field must be unsigned, maximum of 11 numeric characters, no decimals
    
                    Legacy Rule Format:
                    IF [PTR324] <> NULL AND [PTR324] <> MONETARY(U,11,0)
                       RETURN VALIDATION MESSAGE
                    ENDIF
                    
                    [PTR324] = PTR:RP:bafpr1.02.09:Income.DistributionFranked.Amount IN TUPLE(ptr.0006.lodge.req.02.00:StatementOfDistribution)
        
                    Technical Business Rule Format:
                    NotMonetary(^PTR324, 'U', 11, 0)
            
                    Data Elements:
            
                    RP:^PTR324 = bafpr1.02.09:Income.DistributionFranked.Amount
                    */
                    assertion = NotMonetary(statementOfDistribution.PTR324, @"U", 11, 0);
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.GEN.400011",
                            Severity = ProcessMessageSeverity.Error,
                            Description = @"Field incorrect format. Exceeds maximum digits or decimals and/or contains illegal characters",
                            LongDescription = @"Field is in incorrect format for a monetary field. Field either exceeds maximum digits or decimals and/or contains illegal characters and/or for a signed field does not have the sign as the first character",
                            Location = "/xbrli:xbrl/ptr.0006.lodge.req.02.00:StatementOfDistribution" + OccurrenceIndex(statementOfDistribution.OccurrenceIndex) + "/bafpr1.02.09:Income.DistributionFranked.Amount",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.PTR.430711" } },
                        };
                        processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "PTR324", Value = statementOfDistribution.PTR324.GetValueOrDefault().ToString() });

                        response.Add(processMessage);
                    }
                    #endregion // VR.ATO.PTR.430711

                    #region VR.ATO.PTR.431049

                    /*  VR.ATO.PTR.431049
                    Values for at least one of the Identifying fields must be provided - Date of Birth, TFN, Individual name, Address (Business and Residential), ABN, Non-Individual Name
    
                    Legacy Rule Format:
                    WHERE IN TUPLE(ptr.0006.lodge.req.xx.xx:StatementOfDistribution)
                    IF ([PTR146] = NULL AND
                    [PTR157] = NULL AND
                    [PTR284] = NULL AND
                    [PTR332] = NULL AND
                    [PTR149] = NULL AND
                    [PTR157] = NULL AND
                    [PTR285] = NULL AND
                    [PTR151] = NULL)
                       RETURN VALIDATION MESSAGE
                    ENDIF
                    
                    [PTR146] = PTR:RP:pyde.02.00:PersonNameDetails.FamilyName.Text WHERE (pyde.02.00:PersonNameDetails.PersonNameType.Code = "LGL" AND pyde.02.05:PersonNameDetails.Usage.Code = "Contact") IN TUPLE(prsnstrcnm3.02.01:PersonNameDetails) IN TUPLE(ptr.0006.lodge.req.02.00:StatementOfDistribution)
                    [PTR149] = PTR:RP:pyde.02.00:OrganisationNameDetails.OrganisationalName.Text WHERE (pyde.02.00:OrganisationNameDetails.OrganisationalNameType.Code = "MN" AND pyde.02.00:OrganisationNameDetails.Currency.Code = "C") IN TUPLE(orgname2.02.00:OrganisationNameDetails) IN TUPLE(ptr.0006.lodge.req.02.00:StatementOfDistribution)
                    [PTR151] = PTR:RP:pyde.02.00:AddressDetails.Line1.Text WHERE (pyde.02.01:AddressDetails.Usage.Code = "BUS" AND pyde.02.00:AddressDetails.Currency.Code = "C") IN TUPLE(address2.02.02:AddressDetails) IN TUPLE(ptr.0006.lodge.req.02.00:StatementOfDistribution)
                    [PTR157] = PTR:RP:pyid.02.00:Identifiers.TaxFileNumber.Identifier IN TUPLE(ptr.0006.lodge.req.02.00:StatementOfDistribution)
                    [PTR284] = PTR:RP:pyde.02.00:PersonDemographicDetails.Birth.Date IN TUPLE(ptr.0006.lodge.req.02.00:StatementOfDistribution)
                    [PTR285] = PTR:RP:pyid.02.00:Identifiers.AustralianBusinessNumber.Identifier IN TUPLE(ptr.0006.lodge.req.02.00:StatementOfDistribution)
                    [PTR332] = PTR:RP:pyde.02.00:AddressDetails.Line1.Text WHERE (pyde.02.01:AddressDetails.Usage.Code = "RES" AND pyde.02.00:AddressDetails.Currency.Code = "C") IN TUPLE(address2.02.02:AddressDetails) IN TUPLE(ptr.0006.lodge.req.02.00:StatementOfDistribution)
        
                    Technical Business Rule Format:
                    ^PTR146 = NULL AND ^PTR157 = NULL AND ^PTR284 = NULL AND ^PTR332 = NULL AND ^PTR149 = NULL AND ^PTR157 = NULL AND ^PTR285 = NULL AND ^PTR151 = NULL
            
                    Data Elements:
            
                    RP:^PTR157 = pyid.02.00:Identifiers.TaxFileNumber.Identifier
            
                    RP:^PTR146 = pyde.02.00:PersonNameDetails.FamilyName.Text WHERE [PersonNameDetails.PersonNameType.Code='LGL'].[PersonNameDetails.Usage.Code='Contact']
            
                    RP:^PTR149 = pyde.02.00:OrganisationNameDetails.OrganisationalName.Text WHERE [OrganisationNameDetails.OrganisationalNameType.Code='MN'].[OrganisationNameDetails.Currency.Code='C']
            
                    RP:^PTR151 = pyde.02.00:AddressDetails.Line1.Text WHERE [AddressDetails.Usage.Code='BUS'].[AddressDetails.Currency.Code='C']
            
                    RP:^PTR284 = pyde.02.00:PersonDemographicDetails.Birth.Date
            
                    RP:^PTR285 = pyid.02.00:Identifiers.AustralianBusinessNumber.Identifier
            
                    RP:^PTR332 = pyde.02.00:AddressDetails.Line1.Text WHERE [AddressDetails.Usage.Code='RES'].[AddressDetails.Currency.Code='C']
                    */
                    assertion = (statementOfDistribution.PTR146 == null && statementOfDistribution.PTR157 == null && statementOfDistribution.PTR284 == null && statementOfDistribution.PTR332 == null && statementOfDistribution.PTR149 == null && statementOfDistribution.PTR157 == null && statementOfDistribution.PTR285 == null && statementOfDistribution.PTR151 == null);
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.PTR.431049",
                            Severity = ProcessMessageSeverity.Error,
                            Description = @"Identifying fields missing in the statement of distribution",
                            LongDescription = @"Values for at least one of the Identifying fields must be provided - Date of Birth, TFN, Individual name, Address (Business and Residential), ABN, Non-Individual Name.",
                            Location = "/xbrli:xbrl/ptr.0006.lodge.req.02.00:StatementOfDistribution" + OccurrenceIndex(statementOfDistribution.OccurrenceIndex) + "/pyid.02.00:Identifiers.TaxFileNumber.Identifier",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.PTR.431049" } },
                        };
                        processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "PTR146", Value = statementOfDistribution.PTR146 });

                        processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "PTR157", Value = statementOfDistribution.PTR157 });

                        processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "PTR284", Value = GetValueOrEmpty(statementOfDistribution.PTR284) });

                        processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "PTR332", Value = statementOfDistribution.PTR332 });

                        processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "PTR149", Value = statementOfDistribution.PTR149 });

                        processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "PTR285", Value = statementOfDistribution.PTR285 });

                        processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "PTR151", Value = statementOfDistribution.PTR151 });

                        response.Add(processMessage);
                    }
                    #endregion // VR.ATO.PTR.431049

                    #region VR.ATO.PTR.431050

                    /*  VR.ATO.PTR.431050
                    Australian Business Number and
                    Individual's Surname/family name must not be accepted together
    
                    Legacy Rule Format:
                    WHERE IN TUPLE(ptr.0006.lodge.req.xx.xx:StatementOfDistribution)
                    IF([PTR285] <> NULL AND
                    [PTR146] <> NULL)  
                       RETURN VALIDATION MESSAGE
                    ENDIF
                    
                    [PTR146] = PTR:RP:pyde.02.00:PersonNameDetails.FamilyName.Text WHERE (pyde.02.00:PersonNameDetails.PersonNameType.Code = "LGL" AND pyde.02.05:PersonNameDetails.Usage.Code = "Contact") IN TUPLE(prsnstrcnm3.02.01:PersonNameDetails) IN TUPLE(ptr.0006.lodge.req.02.00:StatementOfDistribution)
                    [PTR285] = PTR:RP:pyid.02.00:Identifiers.AustralianBusinessNumber.Identifier IN TUPLE(ptr.0006.lodge.req.02.00:StatementOfDistribution)
        
                    Technical Business Rule Format:
                    ^PTR285 <> NULL AND ^PTR146 <> NULL
            
                    Data Elements:
            
                    RP:^PTR146 = pyde.02.00:PersonNameDetails.FamilyName.Text WHERE [PersonNameDetails.PersonNameType.Code='LGL'].[PersonNameDetails.Usage.Code='Contact']
            
                    RP:^PTR285 = pyid.02.00:Identifiers.AustralianBusinessNumber.Identifier
                    */
                    assertion = (statementOfDistribution.PTR285 != null && statementOfDistribution.PTR146 != null);
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.PTR.431050",
                            Severity = ProcessMessageSeverity.Error,
                            Description = @"Either Australian Business Number or
Individual's Surname/family name must not be supplied together",
                            Location = "/xbrli:xbrl/ptr.0006.lodge.req.02.00:StatementOfDistribution" + OccurrenceIndex(statementOfDistribution.OccurrenceIndex) + "/pyid.02.00:Identifiers.TaxFileNumber.Identifier",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.PTR.431050" } },
                        };
                        processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "PTR285", Value = statementOfDistribution.PTR285 });

                        processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "PTR146", Value = statementOfDistribution.PTR146 });

                        response.Add(processMessage);
                    }
                    #endregion // VR.ATO.PTR.431050

                    #region VR.ATO.PTR.431051

                    /*  VR.ATO.PTR.431051
                    Australian Business Number OR Non-Individual's company name 
                    AND Individual's Date of Birth must not be accepted  together
    
                    Legacy Rule Format:
                    WHERE IN TUPLE(ptr.0006.lodge.req.xx.xx:StatementOfDistribution)
                    IF([PTR285] <> NULL OR [PTR149] <> NULL) AND
                    ([PTR284] <> NULL)  
                       RETURN VALIDATION MESSAGE
                    ENDIF
                    
                    [PTR149] = PTR:RP:pyde.02.00:OrganisationNameDetails.OrganisationalName.Text WHERE (pyde.02.00:OrganisationNameDetails.OrganisationalNameType.Code = "MN" AND pyde.02.00:OrganisationNameDetails.Currency.Code = "C") IN TUPLE(orgname2.02.00:OrganisationNameDetails) IN TUPLE(ptr.0006.lodge.req.02.00:StatementOfDistribution)
                    [PTR284] = PTR:RP:pyde.02.00:PersonDemographicDetails.Birth.Date IN TUPLE(ptr.0006.lodge.req.02.00:StatementOfDistribution)
                    [PTR285] = PTR:RP:pyid.02.00:Identifiers.AustralianBusinessNumber.Identifier IN TUPLE(ptr.0006.lodge.req.02.00:StatementOfDistribution)
        
                    Technical Business Rule Format:
                    (^PTR285 <> NULL OR ^PTR149 <> NULL) AND (^PTR284 <> NULL)
            
                    Data Elements:
            
                    RP:^PTR149 = pyde.02.00:OrganisationNameDetails.OrganisationalName.Text WHERE [OrganisationNameDetails.OrganisationalNameType.Code='MN'].[OrganisationNameDetails.Currency.Code='C']
            
                    RP:^PTR284 = pyde.02.00:PersonDemographicDetails.Birth.Date
            
                    RP:^PTR285 = pyid.02.00:Identifiers.AustralianBusinessNumber.Identifier
                    */
                    assertion = ((statementOfDistribution.PTR285 != null || statementOfDistribution.PTR149 != null) && statementOfDistribution.PTR284 != null);
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.PTR.431051",
                            Severity = ProcessMessageSeverity.Error,
                            Description = @"Australian Business Number OR Non-Individual's company name 
AND Individual's Date of Birth must not be supplied together",
                            Location = "/xbrli:xbrl/ptr.0006.lodge.req.02.00:StatementOfDistribution" + OccurrenceIndex(statementOfDistribution.OccurrenceIndex) + "/pyid.02.00:Identifiers.TaxFileNumber.Identifier",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.PTR.431051" } },
                        };
                        processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "PTR285", Value = statementOfDistribution.PTR285 });

                        processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "PTR149", Value = statementOfDistribution.PTR149 });

                        processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "PTR284", Value = GetValueOrEmpty(statementOfDistribution.PTR284) });

                        response.Add(processMessage);
                    }
                    #endregion // VR.ATO.PTR.431051

                    #region VR.ATO.PTR.438036

                    /*  VR.ATO.PTR.438036
                    Date of Birth must not be later than today
    
                    Legacy Rule Format:
                    IF [PTR284] <> NULL AND [PTR284] > DATE(TODAY)
                       RETURN VALIDATION MESSAGE
                    ENDIF
                    
                    [PTR284] = PTR:RP:pyde.02.00:PersonDemographicDetails.Birth.Date IN TUPLE(ptr.0006.lodge.req.02.00:StatementOfDistribution)
        
                    Technical Business Rule Format:
                    ^PTR284 <> NULL AND ^PTR284 > Today()
            
                    Data Elements:
            
                    RP:^PTR284 = pyde.02.00:PersonDemographicDetails.Birth.Date
                    */
                    assertion = (statementOfDistribution.PTR284 != null && statementOfDistribution.PTR284.GetValueOrDefault() > DateTime.Now.Date);
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.PTR.438036",
                            Severity = ProcessMessageSeverity.Error,
                            Description = @"Date of Birth must not be later than today",
                            Location = "/xbrli:xbrl/ptr.0006.lodge.req.02.00:StatementOfDistribution" + OccurrenceIndex(statementOfDistribution.OccurrenceIndex) + "/pyde.02.00:PersonDemographicDetails.Birth.Date",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.PTR.438036" } },
                        };
                        processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "PTR284", Value = GetValueOrEmpty(statementOfDistribution.PTR284) });

                        response.Add(processMessage);
                    }
                    #endregion // VR.ATO.PTR.438036

                    #region VR.ATO.PTR.438050

                    /*  VR.ATO.PTR.438050
                    Monetary field must be unsigned, maximum of 11 numeric characters, no decimals
    
                    Legacy Rule Format:
                    IF [PTR351] <> NULL AND [PTR351] <> MONETARY(U,11,0)
                       RETURN VALIDATION MESSAGE
                    ENDIF
                    
                    [PTR351] = PTR:RP:rvctc2.02.13:IncomeTax.PayAsYouGoWithholding.CreditForAmountsWithheldFromForeignResidents.Amount IN TUPLE(ptr.0006.lodge.req.02.00:StatementOfDistribution)
        
                    Technical Business Rule Format:
                    NotMonetary(^PTR351, 'U', 11, 0)
            
                    Data Elements:
            
                    RP:^PTR351 = rvctc2.02.13:IncomeTax.PayAsYouGoWithholding.CreditForAmountsWithheldFromForeignResidents.Amount
                    */
                    assertion = NotMonetary(statementOfDistribution.PTR351, @"U", 11, 0);
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.GEN.400011",
                            Severity = ProcessMessageSeverity.Error,
                            Description = @"Field incorrect format. Exceeds maximum digits or decimals and/or contains illegal characters",
                            LongDescription = @"Field is in incorrect format for a monetary field. Field either exceeds maximum digits or decimals and/or contains illegal characters and/or for a signed field does not have the sign as the first character",
                            Location = "/xbrli:xbrl/ptr.0006.lodge.req.02.00:StatementOfDistribution" + OccurrenceIndex(statementOfDistribution.OccurrenceIndex) + "/rvctc2.02.13:IncomeTax.PayAsYouGoWithholding.CreditForAmountsWithheldFromForeignResidents.Amount",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.PTR.438050" } },
                        };
                        processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "PTR351", Value = statementOfDistribution.PTR351.GetValueOrDefault().ToString() });

                        response.Add(processMessage);
                    }
                    #endregion // VR.ATO.PTR.438050

                    #region VR.ATO.PTR.438051

                    /*  VR.ATO.PTR.438051
                    Monetary field must be unsigned, maximum of 11 numeric characters, no decimals
    
                    Legacy Rule Format:
                    IF [PTR342] <> NULL AND [PTR342] <> MONETARY(U,11,0)
                       RETURN VALIDATION MESSAGE
                    ENDIF
                    
                    [PTR342] = PTR:RP:rvctc1.02.18:IncomeTax.ExplorationDevelopmentIncentiveCreditDistributed.Amount IN TUPLE(ptr.0006.lodge.req.02.00:StatementOfDistribution)
        
                    Technical Business Rule Format:
                    NotMonetary(^PTR342, 'U', 11, 0)
            
                    Data Elements:
            
                    RP:^PTR342 = rvctc1.02.18:IncomeTax.ExplorationDevelopmentIncentiveCreditDistributed.Amount
                    */
                    assertion = NotMonetary(statementOfDistribution.PTR342, @"U", 11, 0);
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.GEN.400011",
                            Severity = ProcessMessageSeverity.Error,
                            Description = @"Field incorrect format. Exceeds maximum digits or decimals and/or contains illegal characters",
                            LongDescription = @"Field is in incorrect format for a monetary field. Field either exceeds maximum digits or decimals and/or contains illegal characters and/or for a signed field does not have the sign as the first character",
                            Location = "/xbrli:xbrl/ptr.0006.lodge.req.02.00:StatementOfDistribution" + OccurrenceIndex(statementOfDistribution.OccurrenceIndex) + "/rvctc1.02.18:IncomeTax.ExplorationDevelopmentIncentiveCreditDistributed.Amount",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.PTR.438051" } },
                        };
                        processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "PTR342", Value = statementOfDistribution.PTR342.GetValueOrDefault().ToString() });

                        response.Add(processMessage);
                    }
                    #endregion // VR.ATO.PTR.438051

                    #region VR.ATO.PTR.438071

                    /*  VR.ATO.PTR.438071
                    Monetary field must be unsigned, maximum of 11 numeric characters, no decimals
    
                    Legacy Rule Format:
                    IF [PTR353] <> NULL AND [PTR353] <> MONETARY(U,11,0)
                       RETURN VALIDATION MESSAGE
                    ENDIF
                    
                    [PTR353] = PTR:RP:bafpr1.02.04:Income.SmallBusinessEntityNet.Amount IN TUPLE(ptr.0006.lodge.req.02.00:StatementOfDistribution)
        
                    Technical Business Rule Format:
                    NotMonetary(^PTR353, 'U', 11, 0)
            
                    Data Elements:
            
                    RP:^PTR353 = bafpr1.02.04:Income.SmallBusinessEntityNet.Amount
                    */
                    assertion = NotMonetary(statementOfDistribution.PTR353, @"U", 11, 0);
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.GEN.400011",
                            Severity = ProcessMessageSeverity.Error,
                            Description = @"Field incorrect format. Exceeds maximum digits or decimals and/or contains illegal characters",
                            LongDescription = @"Field is in incorrect format for a monetary field. Field either exceeds maximum digits or decimals and/or contains illegal characters and/or for a signed field does not have the sign as the first character",
                            Location = "/xbrli:xbrl/ptr.0006.lodge.req.02.00:StatementOfDistribution" + OccurrenceIndex(statementOfDistribution.OccurrenceIndex) + "/bafpr1.02.04:Income.SmallBusinessEntityNet.Amount",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.PTR.438071" } },
                        };
                        processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "PTR353", Value = statementOfDistribution.PTR353.GetValueOrDefault().ToString() });

                        response.Add(processMessage);
                    }
                    #endregion // VR.ATO.PTR.438071
                }
            }

            #endregion // Foreach loop

            #region Repeating report.OtherAustralianIncomeCollection
            if (report.OtherAustralianIncomeCollection != null)
            {
                int itemIndex1 = 0;
                foreach (PTR2017.OtherAustralianIncome otherAustralianIncome in report.OtherAustralianIncomeCollection)
                {
                    itemIndex1++;

                    #region VR.ATO.PTR.430104

                    /*  VR.ATO.PTR.430104
                    Other Australian income - type and amount must both be present
    
                    Legacy Rule Format:
                    WHERE IN TUPLE(ptr.0006.lodge.req.xx.xx:OtherAustralianIncome)
                    IF (bafpr1.xx.xx:Income.Other.Description <> NULLORBLANK OR bafpr1.xx.xx:Income.Other.Amount <> NULL) AND (bafpr1.xx.xx:Income.Other.Description = NULLORBLANK OR bafpr1.xx.xx:Income.Other.Amount  = NULL)
                        RETURN VALIDATION MESSAGE
                    ENDIF
        
                    Technical Business Rule Format:
                    (^PTR88 <> BLANK OR ^PTR89 <> NULL) AND (^PTR88 = BLANK OR ^PTR89  = NULL)
            
                    Data Elements:
            
                    RP.JAUS:^PTR89 = bafpr1.02.04:Income.Other.Amount
            
                    RP.JAUS:^PTR88 = bafpr1.02.00:Income.Other.Description
                    */
                    assertion = ((string.IsNullOrWhiteSpace(otherAustralianIncome.PTR88) != true || otherAustralianIncome.PTR89 != null) && (string.IsNullOrWhiteSpace(otherAustralianIncome.PTR88) == true || otherAustralianIncome.PTR89 == null));
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.PTR.430104",
                            Severity = ProcessMessageSeverity.Error,
                            Description = @"Other Australian income - type and amount must both be present",
                            LongDescription = @"If Other Australian income - Type of income is present, then the income amount must be present. If Other Australian income - amount is present, then the Type of income must be present.",
                            Location = "/xbrli:xbrl/ptr.0006.lodge.req.02.00:OtherAustralianIncome" + OccurrenceIndex(otherAustralianIncome.OccurrenceIndex) + "/bafpr1.02.04:Income.Other.Amount",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.PTR.430104" } },
                        };
                        processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "PTR88", Value = otherAustralianIncome.PTR88 });

                        processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "PTR89", Value = otherAustralianIncome.PTR89.GetValueOrDefault().ToString() });

                        response.Add(processMessage);
                    }
                    #endregion // VR.ATO.PTR.430104

                    #region VR.ATO.PTR.430105

                    /*  VR.ATO.PTR.430105
                    Monetary field may be signed, maximum of 11 chars, no decimals
    
                    Legacy Rule Format:
                    IF [PTR89] <> NULL AND [PTR89] <> MONETARY(S,11,0)
                       RETURN VALIDATION MESSAGE
                    ENDIF
                    
                    [PTR89] = PTR:RP.JAUS:bafpr1.02.04:Income.Other.Amount IN TUPLE(ptr.0006.lodge.req.02.00:OtherAustralianIncome)
        
                    Technical Business Rule Format:
                    NotMonetary(^PTR89, 'S', 11, 0)
            
                    Data Elements:
            
                    RP.JAUS:^PTR89 = bafpr1.02.04:Income.Other.Amount
                    */
                    assertion = NotMonetary(otherAustralianIncome.PTR89, @"S", 11, 0);
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.GEN.400011",
                            Severity = ProcessMessageSeverity.Error,
                            Description = @"Field incorrect format. Exceeds maximum digits or decimals and/or contains illegal characters",
                            LongDescription = @"Field is in incorrect format for a monetary field. Field either exceeds maximum digits or decimals and/or contains illegal characters and/or for a signed field does not have the sign as the first character",
                            Location = "/xbrli:xbrl/ptr.0006.lodge.req.02.00:OtherAustralianIncome" + OccurrenceIndex(otherAustralianIncome.OccurrenceIndex) + "/bafpr1.02.04:Income.Other.Amount",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.PTR.430105" } },
                        };
                        processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "PTR89", Value = otherAustralianIncome.PTR89.GetValueOrDefault().ToString() });

                        response.Add(processMessage);
                    }
                    #endregion // VR.ATO.PTR.430105
                }
            }

            #endregion // Foreach loop

            #region Repeating report.OtherDeductionsCollection
            if (report.OtherDeductionsCollection != null)
            {
                int itemIndex1 = 0;
                foreach (PTR2017.OtherDeductions otherDeductions in report.OtherDeductionsCollection)
                {
                    itemIndex1++;

                    #region VR.ATO.PTR.430108

                    /*  VR.ATO.PTR.430108
                    Other deductions - Name of each item of deduction must be present
    
                    Legacy Rule Format:
                    WHERE IN TUPLE(ptr.0006.lodge.req.xx.xx:OtherDeductions)
                    IF (bafpr1.xx.xx:Expense.DeductibleOther.Amount > 0) AND (bafpr1.xx.xx:Expense.DeductibleOther.Text = NULLORBLANK)
                        RETURN VALIDATION MESSAGE
                    ENDIF
        
                    Technical Business Rule Format:
                    (^PTR96 > 0) AND (^PTR95 = BLANK)
            
                    Data Elements:
            
                    RP.JAUS:^PTR95 = bafpr1.02.04:Expense.DeductibleOther.Text
            
                    RP.JAUS:^PTR96 = bafpr1.02.04:Expense.DeductibleOther.Amount
                    */
                    assertion = (otherDeductions.PTR96.GetValueOrDefault() > 0 && string.IsNullOrWhiteSpace(otherDeductions.PTR95) == true);
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.GEN.430108",
                            Severity = ProcessMessageSeverity.Error,
                            Description = @"Other deductions - Name of each item of deduction must be present",
                            LongDescription = @"If Other deductions - Amount is present, the Name of each item of deduction must be present",
                            Location = "/xbrli:xbrl/ptr.0006.lodge.req.02.00:OtherDeductions" + OccurrenceIndex(otherDeductions.OccurrenceIndex) + "/bafpr1.02.04:Expense.DeductibleOther.Text",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.PTR.430108" } },
                        };
                        processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "PTR96", Value = otherDeductions.PTR96.GetValueOrDefault().ToString() });

                        processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "PTR95", Value = otherDeductions.PTR95 });

                        response.Add(processMessage);
                    }
                    #endregion // VR.ATO.PTR.430108

                    #region VR.ATO.PTR.430110

                    /*  VR.ATO.PTR.430110
                    Other deductions amount must be present
    
                    Legacy Rule Format:
                    WHERE IN TUPLE(ptr.0006.lodge.req.xx.xx:OtherDeductions)
                    IF [PTR95] <> NULLORBLANK AND ([PTR96] = 0 OR [PTR96] = NULL) 
                        RETURN VALIDATION MESSAGE
                    ENDIF
                    
                    [PTR95] = PTR:RP.JAUS:bafpr1.02.04:Expense.DeductibleOther.Text IN TUPLE(ptr.0006.lodge.req.02.00:OtherDeductions)
                    [PTR96] = PTR:RP.JAUS:bafpr1.02.04:Expense.DeductibleOther.Amount IN TUPLE(ptr.0006.lodge.req.02.00:OtherDeductions)
        
                    Technical Business Rule Format:
                    ^PTR95 <> BLANK AND (^PTR96 = 0 OR ^PTR96 = NULL)
            
                    Data Elements:
            
                    RP.JAUS:^PTR96 = bafpr1.02.04:Expense.DeductibleOther.Amount
            
                    RP.JAUS:^PTR95 = bafpr1.02.04:Expense.DeductibleOther.Text
                    */
                    assertion = (string.IsNullOrWhiteSpace(otherDeductions.PTR95) != true && (otherDeductions.PTR96 == 0 || otherDeductions.PTR96 == null));
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.GEN.430110",
                            Severity = ProcessMessageSeverity.Error,
                            Description = @"Other deductions amount must be present",
                            LongDescription = @"If Name of each item of deduction is present, then the Other deductions - Amount must be present and greater than zero",
                            Location = "/xbrli:xbrl/ptr.0006.lodge.req.02.00:OtherDeductions" + OccurrenceIndex(otherDeductions.OccurrenceIndex) + "/bafpr1.02.04:Expense.DeductibleOther.Amount",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.PTR.430110" } },
                        };
                        processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "PTR95", Value = otherDeductions.PTR95 });

                        processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "PTR96", Value = otherDeductions.PTR96.GetValueOrDefault().ToString() });

                        response.Add(processMessage);
                    }
                    #endregion // VR.ATO.PTR.430110

                    #region VR.ATO.PTR.430196

                    /*  VR.ATO.PTR.430196
                    Monetary field must be unsigned, maximum of 11 numeric characters, no decimals
    
                    Legacy Rule Format:
                    IF [PTR96] <> NULL AND [PTR96] <> MONETARY(U,11,0)
                       RETURN VALIDATION MESSAGE
                    ENDIF
                    
                    [PTR96] = PTR:RP.JAUS:bafpr1.02.04:Expense.DeductibleOther.Amount IN TUPLE(ptr.0006.lodge.req.02.00:OtherDeductions)
        
                    Technical Business Rule Format:
                    NotMonetary(^PTR96, 'U', 11, 0)
            
                    Data Elements:
            
                    RP.JAUS:^PTR96 = bafpr1.02.04:Expense.DeductibleOther.Amount
                    */
                    assertion = NotMonetary(otherDeductions.PTR96, @"U", 11, 0);
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.GEN.400011",
                            Severity = ProcessMessageSeverity.Error,
                            Description = @"Field incorrect format. Exceeds maximum digits or decimals and/or contains illegal characters",
                            LongDescription = @"Field is in incorrect format for a monetary field. Field either exceeds maximum digits or decimals and/or contains illegal characters and/or for a signed field does not have the sign as the first character",
                            Location = "/xbrli:xbrl/ptr.0006.lodge.req.02.00:OtherDeductions" + OccurrenceIndex(otherDeductions.OccurrenceIndex) + "/bafpr1.02.04:Expense.DeductibleOther.Amount",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.PTR.430196" } },
                        };
                        processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "PTR96", Value = otherDeductions.PTR96.GetValueOrDefault().ToString() });

                        response.Add(processMessage);
                    }
                    #endregion // VR.ATO.PTR.430196
                }
            }

            #endregion // Foreach loop

            foreach (ProcessMessageDocument currentProcessMessage in response)
            {
                if (currentProcessMessage.Parameters != null)
                {
                    foreach (ProcessMessageParameter currentParameter in currentProcessMessage.Parameters)
                    {
                        if (string.IsNullOrEmpty(currentParameter.Name))
                        {
                            currentParameter.Name = _emptyParameterValue;
                        }

                        if (currentParameter.Name.Length > _maxParameterNameLength)
                        {
                            currentParameter.Name = currentParameter.Name.Substring(0, _maxParameterNameLength - 1);
                        }

                        if (string.IsNullOrEmpty(currentParameter.Value))
                        {
                            currentParameter.Value = _emptyParameterValue;
                        }

                        if (currentParameter.Value.Length > _maxParameterValueLength)
                        {
                            currentParameter.Value = currentParameter.Value.Substring(0, _maxParameterValueLength - 1);
                        }
                    }
                }
            }

            return response;
        }


    }
}
